﻿using System;
using System.Drawing;
using System.Windows;
using System.Windows.Input;
using Application = System.Windows.Application;
using MessageBox = System.Windows.MessageBox;

namespace ClinSchd.Infrastructure
{
   public static class CLaunchWindow
    {
       private static Action EmptyDelegate = delegate() { };

        public static void LaunchWindow(object content, string title, int height, int width, string name = "", bool resize = false, bool isModal = true, bool launchAtCursor = false, bool titleBar = true, System.Windows.Media.Brush background = null)
        {
            try
            {
                var newWin = new Window
                {
                    Owner = Keys.MainWindow,
                    Content = content,
                    Title = title,
                    Name = name,
                    ResizeMode = !resize ? ResizeMode.NoResize : ResizeMode.CanResizeWithGrip
                };

                Keys.SetRenderingMode(newWin);

                if (background != null)
                    newWin.Background = background;
                else
                    newWin.Background = System.Windows.SystemColors.WindowBrush;

                if (!launchAtCursor)
                    newWin.WindowStartupLocation = ((Keys.MainWindow == null) ? WindowStartupLocation.CenterScreen : WindowStartupLocation.CenterOwner);
                else
                {
                    var point = Mouse.GetPosition(Application.Current.MainWindow);
                    newWin.Left = point.X;
                    newWin.Top = point.Y;
                }
                if (height != 0)
                    newWin.Height = height;
                if (width != 0)
                    newWin.Width = width;

                if (!titleBar)
                    newWin.WindowStyle = WindowStyle.None;

                newWin.UpdateLayout();
                System.Windows.Forms.Application.DoEvents();

                if (isModal)
                    newWin.ShowDialog();
                else
                {
                    newWin.Show();
                    newWin.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Send, EmptyDelegate);
                    System.Windows.Forms.Application.DoEvents();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Unable to Load Dialog " + name + ": " + ex.Message);
            }
        }

        public static void LaunchWindow(object content, string title, int height, int width, double top, double left, string name = "", bool resize = false, bool isModal = true, bool titleBar=true, System.Windows.Media.Brush background = null)
       {
           try
           {
               var newWin = new Window
               {
                   Owner = Keys.MainWindow,
                   Content = content,
                   Title = title,
                   Name = name,
                   ResizeMode = !resize ? ResizeMode.NoResize : ResizeMode.CanResizeWithGrip
               };

               Keys.SetRenderingMode(newWin);

               if (background != null)
                   newWin.Background = background;
               else
                   newWin.Background = System.Windows.SystemColors.WindowBrush;

                newWin.Left = left;
                newWin.Top = top;

               if (height != 0)
                   newWin.Height = height;
               if (width != 0)
                   newWin.Width = width;

               if (!titleBar)
                   newWin.WindowStyle = WindowStyle.None;

                newWin.UpdateLayout();
                System.Windows.Forms.Application.DoEvents();

                if (isModal)
                    newWin.ShowDialog();
                else
                {
                    newWin.Show();
                    newWin.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Send, EmptyDelegate);
                    System.Windows.Forms.Application.DoEvents();
                }
            }
           catch (Exception ex)
           {
               MessageBox.Show("Unable to Load Dialog " + name + ": " + ex.Message);
           }
       }
    }
}
