﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.ComponentModel;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

using ClinSchd.Infrastructure;

namespace ClinSchd.Infrastructure.Controls
{
    public class AsyncWrapper : UserControl
    {
        private DispatcherTimer _delayTimer;
        private int _mainWindowDisableCounter = 0;

        protected override void OnInitialized( EventArgs e )
        {
            base.OnInitialized( e );
            if ( IsInDesignMode )
                return;
            _delayTimer = new DispatcherTimer();
            _delayTimer.Tick += _delayTimer_Tick;
            LoadBusyIndicator();
            Loaded += AsyncWrapper_Loaded;
        }

        private void LoadBusyIndicator()
        {
            Grid grid = new Grid();

            //loadingImage = new GifImage (new System.Uri("pack://application:,,,/ClinSchd.Infrastructure;component/Controls/Resources/ajax-loader.gif"));
            //loadingImage.Width = 66;
            //loadingImage.Height = 66;
            //loadingImage.Visibility = BusyOnLoad ? System.Windows.Visibility.Visible : System.Windows.Visibility.Hidden;
            //loadingImage.VerticalAlignment = System.Windows.VerticalAlignment.Center;
            //loadingImage.HorizontalAlignment = System.Windows.HorizontalAlignment.Center;

            _busyIndicator = new BusyIndicatorPanel
            {
                Name = "BusyIndicatorPanel",
                Width = 600,
                HorizontalAlignment = HorizontalAlignment.Center,
                VerticalAlignment = VerticalAlignment.Center,
                IsCloseButtonVisible = false,
                CanCloseWindow = false,
                IsMessageVisible = true,
                IsSubMessageVisible = true
            };

            //_defaultBusyIndicatorWidth = _busyIndicator.Width;
            _contentPresenter = new ContentPresenter
            {
                Content = Content
            };

            grid.Children.Add( _contentPresenter );
            grid.Children.Add( _busyIndicator );
            //grid.Children.Add (loadingImage);
            Content = grid;
        }

        private void _delayTimer_Tick( object sender, EventArgs e )
        {
            _delayTimer.Stop();

            if (DisableMainWindow && (Keys.MainWindow != null))
            {
                _contentPresenterOpacity = Keys.MainWindow.Opacity;
                Keys.MainWindow.Opacity = .5;
            }
            else
            {
                _contentPresenterOpacity = _contentPresenter.Opacity;
                _contentPresenter.Opacity = .5;
            }

            _busyIndicator.IsLoading = true;

            if (DisableMainWindow && (Keys.MainWindow != null))
            {
                Keys.MainWindow.IsEnabled = false;
                _mainWindowDisableCounter++;
            }
            else
                IsEnabled = false;
        }

        private void AsyncWrapper_Loaded( object sender, RoutedEventArgs e )
        {
            if ( !_isRegistered && !IsInDesignMode)
            {
                RegisterWithModel();
                _isRegistered = true;
            }
        }

        private bool _isRegistered;

        //Image loadingImage;
        private BusyIndicatorPanel _busyIndicator;

        private ContentPresenter _contentPresenter;
        private double _contentPresenterOpacity;
        private Cursor _oldCursor;
        //private double _defaultBusyIndicatorWidth;

        public void EnableScreen()
        {
            _delayTimer.Stop();

            if (DisableMainWindow && (Keys.MainWindow != null))
            {
                Keys.MainWindow.Opacity = 1.0;
                Keys.MainWindow.IsEnabled = true;
            }
            else
                _contentPresenter.Opacity = 1.0; // contentPresenterOpacity;

            _busyIndicator.IsLoading = false;
            _busyIndicator.Message = string.Empty;
            _busyIndicator.SubMessage = string.Empty;

            Mouse.OverrideCursor = _oldCursor ?? Cursors.Arrow;

            if (DisableMainWindow && (Keys.MainWindow != null))
            {
                _mainWindowDisableCounter--;

                if (_mainWindowDisableCounter <= 0)
                {
                    _mainWindowDisableCounter = 0;
                    Keys.MainWindow.IsEnabled = true;
                }
            }
            else
                IsEnabled = true;
        }

        public void StopDelayTimer()
        {
            _delayTimer.Stop();
        }

        public void DisableScreen( string message = "", string subMessage = "", TimeSpan? delayDisplay = null )
        {
            _oldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.AppStarting;
            _busyIndicator.Message = message;
            _busyIndicator.IsMessageVisible = !string.IsNullOrEmpty( message );
            _busyIndicator.SubMessage = subMessage;
            _busyIndicator.IsSubMessageVisible = !string.IsNullOrEmpty( subMessage );

            if ( !_busyIndicator.IsMessageVisible && !_busyIndicator.IsSubMessageVisible )
                _busyIndicator.Width = 81;

            if ( delayDisplay.HasValue )
            {
                _delayTimer.Interval = delayDisplay.Value;
                _delayTimer.Start();
            }
            else
                _delayTimer_Tick( this, null );
        }

        public static readonly DependencyProperty DisableMainWindowProperty =
            DependencyProperty.Register("DisableMainScreen", typeof(bool), typeof(AsyncWrapper));

        public bool DisableMainWindow
        {
            get { return (bool) GetValue(DisableMainWindowProperty); }
            set { SetValue(DisableMainWindowProperty, value); }
        }

        public static readonly DependencyProperty BackingModelProperty =
            DependencyProperty.Register ("BackingModel", typeof (AsyncValidatableModel), typeof (AsyncWrapper));

        public AsyncValidatableModel BackingModel
        {
            get { return (AsyncValidatableModel)GetValue( BackingModelProperty ); }
            set { SetValue( BackingModelProperty, value ); }
        }

        public static readonly DependencyProperty BusyOnLoadProperty =
            DependencyProperty.Register ("BusyOnLoad", typeof (bool), typeof (AsyncWrapper));

        public bool BusyOnLoad
        {
            get { return (bool)GetValue( BusyOnLoadProperty ); }
            set { SetValue( BusyOnLoadProperty, value ); }
        }

        private void RegisterWithModel()
        {
            if ( BackingModel == null )
            {
                throw new InvalidCastException( "AsyncWrapper must have its DataContext bound to an instance of AsyncValidatableModel." );
            }
            BackingModel.AsyncWrapper = this;
        }

        private class GifImage : Image
        {
            public int FrameIndex
            {
                get { return (int)GetValue( FrameIndexProperty ); }
                set { SetValue( FrameIndexProperty, value ); }
            }

            public static readonly DependencyProperty FrameIndexProperty =
                DependencyProperty.Register ("FrameIndex", typeof (int), typeof (GifImage), new UIPropertyMetadata (0, ChangingFrameIndex));

            private static void ChangingFrameIndex( DependencyObject obj, DependencyPropertyChangedEventArgs ev )
            {
                GifImage ob = obj as GifImage;
                if ( ob != null )
                {
                    ob.Source = ob._gf.Frames[(int)ev.NewValue];
                    ob.InvalidateVisual();
                }
            }

            private readonly GifBitmapDecoder _gf;
            private readonly Int32Animation _anim;

            public GifImage( Uri uri )
            {
                _gf = new GifBitmapDecoder( uri, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default );
                _anim = new Int32Animation( 0, _gf.Frames.Count - 1, new Duration( new TimeSpan( 0, 0, 0, _gf.Frames.Count/12, (int)( ( _gf.Frames.Count/12.0 - _gf.Frames.Count/12.0 )*1000 ) ) ) )
                {
                    RepeatBehavior = RepeatBehavior.Forever
                };
                Source = _gf.Frames[0];
            }

            private bool _animationIsWorking;

            protected override void OnRender( DrawingContext dc )
            {
                base.OnRender( dc );
                if ( !_animationIsWorking )
                {
                    BeginAnimation( FrameIndexProperty, _anim );
                    _animationIsWorking = true;
                }
            }
        }

        public void ShowBusyIndicator()
        {
            if ( _busyIndicator == null )
                LoadBusyIndicator();

            DisableScreen();
        }

        public void ShowBusyIndicator( string message, string subMessage = "", bool isCloseButtonVisible = false, TimeSpan? displayAfter = null )
        {
            if ( _busyIndicator == null )
                LoadBusyIndicator();

            if ( _busyIndicator != null )
                _busyIndicator.IsCloseButtonVisible = isCloseButtonVisible;
            DisableScreen( message, subMessage, displayAfter );
        }

        public void HideBusyIndicator()
        {
            EnableScreen();
        }

        private bool IsInDesignMode
        {
            get { return DesignerProperties.GetIsInDesignMode(new DependencyObject()); }
        }
    }
}