﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Converters
{
    public static class ShortDateTimeConverter
    {
        public static string ConvertMonthStringDateToShortDateTime(string dateStr)
        {
            if (string.IsNullOrEmpty(dateStr))
                return "";

            var datepc = dateStr.Split(' ');

            if (datepc.Length != 3)
                return "";

            DateTime dt;
            var culture = new CultureInfo("en-us");
            var dtStyles = DateTimeStyles.AssumeLocal;
            bool success = DateTime.TryParse(dateStr, out dt);

            if (!success)
                success = DateTime.TryParseExact(dateStr, "MMM dd, yyyyy", culture, dtStyles, out dt);
                
            if (!success)
                success = DateTime.TryParseExact(dateStr, "MMM d, yyyyy", culture, dtStyles, out dt);

            if (success)
                return dt.ToString("M/d/yyyy");
            else
                return "";
        }

        public static string ConvertMonthStringDateToShortDateTime2(string dateStr)
        {
            if (string.IsNullOrEmpty(dateStr))
                return "";

            var datepc = dateStr.Split(' ');

            if (datepc.Length != 2)
                return "";

            DateTime dt;
            var culture = new CultureInfo("en-us");
            var dtStyles = DateTimeStyles.AssumeLocal;
            bool success = DateTime.TryParse(dateStr, out dt);

            if (!success)
                success = DateTime.TryParseExact(dateStr, "MMM dd,yyyyy", culture, dtStyles, out dt);
                
            if (!success)
                success = DateTime.TryParseExact(dateStr, "MMM d,yyyyy", culture, dtStyles, out dt);

            if (success)
                return dt.ToString("M/d/yyyy");
            else
                return "";
        }


        private static string ConvertStringToShortMonth(string month)
        {
            switch (month.ToUpper())
            {
                case "JAN":
                    return "1";
                case "FEB":
                    return "2";
                case "MAR":
                    return "3";
                case "APR":
                    return "4";
                case "MAY":
                    return "5";
                case "JUN":
                    return "6";
                case "JUL":
                    return "7";
                case "AUG":
                    return "8";
                case "SEP":
                    return "9";
                case "OCT":
                    return "10";
                case "NOV":
                    return "11";
                case "DEC":
                    return "12";
                default:
                    return "";

            }
        }


        private static string ConvertStringToMonth(string month)
        {
            switch (month.ToUpper())
            {
                case "01":
                case "1":
                    return "JAN";
                case "02":
                case "2":
                    return "FEB";
                case "03":
                case "3":
                    return "MAR";
                case "04":
                case "4":
                    return "APR";
                case "05":
                case "5":
                    return "MAY";
                case "06":
                case "6":
                    return "JUN";
                case "07":
                case "7":
                    return "JUL";
                case "08":
                case "8":
                    return "AUG";
                case "09":
                case "9":
                    return "SEP";
                case "10":
                    return "OCT";
                case "11":
                    return "NOV";
                case "12":
                    return "DEC";
                default:
                    return "";

            }
        }


        public static string ConvertMumpsDate(string origDate)
        {
            if (string.IsNullOrEmpty(origDate))
                return null;
            int millenium = Convert.ToInt32(origDate.Substring(0, 1));

            var year = origDate.Substring(1, 2);
            var month = ConvertStringToMonth(origDate.Substring(3, 2));
            var day = origDate.Substring(5, 2);
            origDate = month + " " + day + ", " + (millenium + 17).ToString() + year;

            return origDate;
        }

        public static string ConvertShortDateTimeWithLeadingZeros(string dateStr)
        {
            if (string.IsNullOrEmpty(dateStr))
                return "";

            var datepc = dateStr.Split(' ');

            switch (datepc.Length)
            {
                case 2:
                    return ConvertMonthStringDateToShortDateTime2(dateStr);
                case 3:
                    return ConvertMonthStringDateToShortDateTime(dateStr);
                default:
                    DateTime dt;
                    var culture = new CultureInfo("en-us");
                    bool success = DateTime.TryParse(dateStr, out dt);

                    if (success)
                        return dt.ToString("M/d/yyyy");
                    else
                        return "";
            }
        }
    }
}
