﻿using ClinSchd.Infrastructure.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Factories
{
    public class ContactAttemptFactory
    {
        public static List<ContactAttempt> CreateContactAttempts(PatientContact patientContact, string sdecMultiDisplayRpc)
        {
            var rpcRecords = getRpcRecords(sdecMultiDisplayRpc);

            if (rpcRecords.Length == 0)
                return new List<ContactAttempt>();

            return createContactAttemptsList(patientContact, rpcRecords);
        }        

        #region HELPERS
        private static string[] getRpcRecords(string rpcString)
        {
            if (string.IsNullOrWhiteSpace(rpcString))
                return new string[0];

            return rpcString.Split(';');
        }
        private static List<ContactAttempt> createContactAttemptsList(PatientContact patientContact, string[] rpcRecords)
        {
            var contactAttemptsList = new List<ContactAttempt>();

            foreach (var record in rpcRecords)
                contactAttemptsList.Add(createContactAttempt(patientContact, record));

            contactAttemptsList.RemoveAll(item => item == null);

            return contactAttemptsList;
        }
                private static ContactAttempt createContactAttempt(PatientContact patientContact, string record)
                {
                    var rpcPieces = getRpcStringPieces(record);

                    if (rpcPieces.Length != 9)
                        return null;

                    return mapContactAttempt(patientContact, rpcPieces);          
                }


                        private static string[] getRpcStringPieces(string rpcRecord)
                        {
                            if (string.IsNullOrWhiteSpace(rpcRecord))
                                return new string[0];

                            return rpcRecord.Split('^');
                        }
                        private static ContactAttempt mapContactAttempt(PatientContact patientContact, string[] rpcPieces)
                        {
                            var contactAttempt = new ContactAttempt();
                            contactAttempt.ContactAttemptType = 
                                (rpcPieces[1] == "C") ? ContactAttemptType.Call : (rpcPieces[1] == "L") ? ContactAttemptType.Letter : ContactAttemptType.NA;
                            contactAttempt.Comments = rpcPieces[2];
                            contactAttempt.SequenceNumber = FactoryUtilities.IntParser(rpcPieces[4]);
                            contactAttempt.EnteredByUserIen = FactoryUtilities.IntParser(rpcPieces[5]);
                            contactAttempt.EnteredDateTime = ExtensionMethods.ConvertFromFilemanDateTime(rpcPieces[6]);
                            contactAttempt.EnteredByUserName = rpcPieces[7];
                            contactAttempt.ContactAttemptDateTime = ExtensionMethods.ConvertFromFilemanDateTime(rpcPieces[8]);
                            contactAttempt.ValidForCurrentRequest = (contactAttempt.SequenceNumber >= patientContact.MainSequence) ? true : false;
                            return contactAttempt;
                        }
        #endregion

    }
}
