﻿using ClinSchd.Infrastructure.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Factories
{
    public class PatientContactRequestFactory
    {
        public static List<string> CreateReadRequestParams(AppointmentRequest appointmentRequest)
        {
            return new List<string>
            {
                appointmentRequest.Patient.IEN,
                appointmentRequest.ClinicIen,
                ExtensionMethods.ConvertToFilemanDateTime(Convert.ToDateTime(appointmentRequest.DesiredApptDate)),
                Keys.AppointmentRequestTypeCode(appointmentRequest.AppointmentRequestType),
                (appointmentRequest.ServiceSpecialty != null) ? appointmentRequest.ServiceSpecialty.Ien : ""
            };
        }
        public static List<string> CreateReadRequestParams(SchdAppointment appointment)
        {
            return new List<string>
            {
                appointment.Patient.IEN,
                appointment.ClinicIen,
                ExtensionMethods.ConvertToFilemanDateTime(Convert.ToDateTime(appointment.DesiredAppointmentDate)),
                appointment.APPTREQTYPE_REQTYPE,
                ""
            };
        }

        public static List<string> CreateReadMulipleRequestParams(PatientContact patientContact)
        {
            return new List<string>
            {
                patientContact.PatientContactFileIen.ToString()
            };
        }

        public static List<string> CreateStopCodeRequestParams(AppointmentRequest appointmentRequest)
        {
            return new List<string>
            {
                appointmentRequest.ClinicIen,
            };
        }

        public static List<string> CreateInsertRequestParams(AppointmentRequest appointmentRequest, ContactAttempt newcontactAttempt)
        {
            return new List<string>
            {
                appointmentRequest.Patient.IEN,
                appointmentRequest.ClinicIen,
                ExtensionMethods.ConvertToFilemanDateTime(Convert.ToDateTime(appointmentRequest.DesiredApptDate)),
                Keys.AppointmentRequestTypeCode(appointmentRequest.AppointmentRequestType),
                (appointmentRequest.ServiceSpecialty != null) ? appointmentRequest.ServiceSpecialty.Ien : "",
                ExtensionMethods.ConvertToFilemanDateTime(newcontactAttempt.ContactAttemptDateTime),
                (newcontactAttempt.ContactAttemptType == ContactAttemptType.Letter) ? "L" : "C",
                newcontactAttempt.Comments,
                ExtensionMethods.ConvertToFilemanDateTime(DateTime.Now),                
            };
        }

        public static List<string> CreateUpdateRequestParams(AppointmentRequest appointmentRequest, ContactAttempt newcontactAttempt)
        {
            return new List<string>
            {
                appointmentRequest.PatientContact.PatientContactFileIen.ToString(),
                ExtensionMethods.ConvertToFilemanDateTime(newcontactAttempt.ContactAttemptDateTime),
                (newcontactAttempt.ContactAttemptType == ContactAttemptType.Letter) ? "L" : "C",
                newcontactAttempt.Comments,
                ExtensionMethods.ConvertToFilemanDateTime(DateTime.Now),
            };
        }
    }
}
