using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using ClinSchd.Infrastructure.Models;
using System.Data;
using System.Threading.Tasks;

namespace ClinSchd.Infrastructure.Interfaces
{
    public interface IDataAccessService
    {
        //Clinical Scheduling Methods
        //int GetClinicStopCode(string hospitalLocationId); - This has been replaced with SchdResource.Clinic.STOP_CODE
        //int GetClinicApptLengthByResource(string resourceName); - This has been replaced with SchdResource.Clinic.Duration
        bool IsCompAndPenAppt(string apptId);
        // int GetClinicTimeScaleInMinutes(string clinic); - This has been replaced with SchdResource.TIMESCALE
        //int GetClinicTimeScale(string clinic); - This has been replaced with SchdResource.Clinic.DISPLAY_INCREMENTS
        IList<Patient> GetPatients(string searchString, int maxRows, ref bool allRecordsDisplayed, string lastSub = "");
        IList<Patient> GetPatients(string searchString, int maxRows);


        IList<SchdResource> GetResources (string searchString, bool includeInactive);
//		SchdResource GetResource (string resourceID);
        PatientInformation GetPatientInfo (string patientIEN);
        bool LoadGlobalRecordsets ();
        IList<SchdAppointment> GetAppointments (SchdResource resource, string startDate, string endDate);
        IList<SchdAvailability> GetAvailabilities(SchdResource resource, string startDate, string endDate);
        bool CanAddNewAppointmentPreCheck(string command, SchdAppointment appointment, out bool overbookAppointment, out bool isTelehealth, string conflictExcludeAppointmentID = "", bool allowConflicts = true);
        List<string> AddNewAppointment(AppointmentData a, PatientInformation p, bool bMakeChartRequest, bool isMoveAppt = false);

        
        
        string CheckSlotsAvailable (string startTime, string endTime, string resourceName);
        string CheckSlotsAvailable(SchdResource resource, string startTime, string endTime);
        
        string CreateNewAppointment (AppointmentData a, PatientInformation p, bool bMakeChartRequest);
        string EditAppointment (AppointmentData a);
        string AddNewAvailability (SchdAvailability a);
        IList<Provider> GetProviders();
        IList<Provider> GetProviders(string searchString);
        IList<Provider> GetPtCSchProviders(string searchString);
        IList<NameValue> GetSummaryReport();
        IList<NameValue> GetCancellationReasons (string type);
        IList<NameValue> GetAccessGroups ();
        IList<NameValue> GetScheduleUsers ();
        IList<NameValue> GetResourceGroupList();
        IList<SchdAccessType> GetAccessTypes ();
        IList<ResourceUser> GetResourceUsers ();
        IList<ResourceUser> GetResourceUsersByResourceID (string resourceID);
        IList<ResourceUser> RemoveResourceUserByID (string userID, string resourceID);
        string CheckInPatient (CheckInPatient checkInData);
        string UndoCheckIn (string apptID);
        string CheckOutPatient (CheckOutPatient checkOutData);
        string UndoCheckOut (string apptID);
        string CancelAppointment(CancelAppointment cancelAppointmentData, SchdAppointment schdAppt = null);
        string UndoCancelAppointment (string apptID);
        string AppointmentNoShow (string apptID, bool bNoShow);
        string AutoRebook (AutoRebookData autoRebookData);
        List<Clinic> GetClinicsWithMatchingStopCode(string clinicID, string searchString = "");
        IList<SchdResource> GetClinicsByProvider (string providerIEN);
        IList<Provider> GetProvidersByClinic (string clinicIEN);
        RPMSClinic GetClinicByID (string clinicIEN);
        void RefreshClinic(RPMSClinic clinic);
        string AddEditClinic (SchdResource schdResource);
        string DeleteClinic (string clinicID);
        string AddEditResourceUser(ResourceUser resourceUser, bool isInitialLoadOfPorhibitedClinics = false);
        DataSet GlobalDataSet { get; set; }
        bool SchedManager { get; set; }
        Divisions CurrentDivision { get; set; }
        string CurrentDivisionIEN { get; }
        IList<SchdResourceGroup> GetResourceGroups();
//		IList<SchdGroupedResources> GetGroupedResources ();
        IList<SchdGroupedResources> GetResourcesByGroupID (string groupId);
        IList<SchdResource> GetSchdResourcesByGroupID(string groupId);
        IList<SchdGroupedResources> RemoveGroupedClinicByID (string groupID, string groupedClinicID);
        IList<NameValue> RemoveResourceGroup (string groupName);
        IList<SchdGroupedResources> AddClinicToGroupByID (string groupID, string groupedClinicID);
        string AddEditClinicGroup (string groupID, string groupName);
        IList<SchdGroupedAccessTypes> GetGroupedAccessTypes ();
        IList<SchdGroupedAccessTypes> GetAccessTypeByGroupID (string groupID);
        IList<FindGroupedAccessTypes> GetFindAccessTypeByGroupID (string groupID);
        IList<SchdGroupedAccessTypes> RemoveGroupedAccessTypeByID (string groupID, string groupedAccessTypeID);
        IList<NameValue> RemoveAccessTypeGroup (string groupID);
        IList<SchdGroupedAccessTypes> AddAccessTypeToGroupByID (string groupID, string groupedAccessTypeID);
        string AddEditAccessTypeGroup (string groupID, string groupName);
        string AddEditAccessType (SchdAccessType accessType);        
        SchdAccessType GetAccessTypeByID (string accessTypeID);
        IList<FindAppointmentResult> FindAppointments (FindAppointmentData searchData);
        bool GetVariableAppointmentsFlag(string HOSP_LOC_ID);
        string DeleteAvailability (string apptID);
        string DeleteAvailability (SchdAvailability a);
        void CancelAccessBlocks (string sResourceID, string sResourceName, string sStart, string sEnd);
        string IsHoliday (DateTime startDate);
        bool OverBookAuthority(SchdAppointment a, SchdResource res = null, bool silent = false, bool isMRTC = false);
        bool ClinicOverbook (string sClinicIen, DateTime dStartTime, string sResource, bool silent = false, bool isMRTC = false);
        Keys.OverbookingStatus ClinicOverbook(SchdAppointment appointment);
        //void RefreshAvailabilitySchedule (string m_sResourcesName, DateTime m_dStartDate, DateTime m_dEndDate);
        void RefreshAvailabilitySchedule(string m_sResourcesName, DateTime m_dStartDate, DateTime m_dEndDate, string src = "0");
        void RefreshAvailabilitySchedule(SchdResource resource, DateTime m_dStartDate, DateTime m_dEndDate, string src = "0");
        //string NoShow (string sPatientIen, string sClinicIEN, string sAppointmentType);
        string NoShow (string sPatientIen, string sResourceName, string sResourceID, string sAppointmentType);
        string GetReportText (string reportRPC, List<List<string>> parms);
        IList<NameValue> GetServerPrinters ();
        IList<ServerPrinter> GetDynamicServerPrinters(string searchString, string lastSub = "");
        void ServerPrinterPrint(string appointmenrID, string letterType, string deviceIEN);
        DataTable GetWaitingList (string ResourceIEN);
        bool CheckConflictAppointment(
            string command,
            string sResourceName,
            string sPatientIEN,
            string sPatientName,
            string sStartDate,
            string sEndDate,
            out bool isTelehealth,
            string excludeAppointmentID = "",
            bool silent = false,
            bool exactMatchOnly = false
            );
        IList<ConflictAppointments> GetConflictAppointment (string sResourceName, string sPatientIEN, string sStartDate, string sEndDate, bool isNewAppointment, string excludeAppointmentID = "");
        string CurrentUser { get; }
        bool IsEnableUpdateAppointment (string sResourceId);
        bool IsEnableUpdateAvailability (string sResourceID);
        IList<Divisions> GetDivisions ();
        bool SetDivision (string divisionId);
        Divisions GetCurrentDivision ();
        Divisions GetDefaultDivision ();
        void ChangeUserServer ();

        bool LoginUser(string accessCode, string verifyCode, out string szErrorMsg);
        bool LoadUserInfo(out string errorMessage);
        void StartRefreshTimer();
        DateTime? ConvertOrigDate(string origDate);
        List<AppointmentType> GetAppointmentTypes(string patientIen);
        IList<ServiceSpecialty> GetServiceSpecialties();
        IList<Clinic> GetPtCSchClinics(string clinicID, string clinicName);
        IList<Clinic> SearchPtCSchClinics(string searchString, string sdNoLet = "");
        IList<NameValue> GetEwlClinics(string searchString);
        List<string> GetPatientPreferences(string dfn);
        AppointmentRequest GetConsultApptRequestByIen(string recIen);
        List<AppointmentRequest> GetConsultAppointmentRequests(bool isInThread = false, DataTable dt = null);
        AppointmentRequest GetPtCSchApptRequest( string requestIen );
        IList<AppointmentRequest> GetPtCSchAppointmentRequests(bool isInThread = false, List<string> messages = null);
        IList<AppointmentRequest> GetPtCSchAppointmentRequests(DataTable dt, bool isInThread = false, List<string> messages = null);
        ObservableCollection<FuturePatientAppointment> GetFutureAppointments(string patientIen);
        ObservableCollection<FuturePatientAppointment> GetFutureAppointments( string iEN, DateTime dateTime );
        //        List<AppointmentRequest> GetPtCSchAppointmentRequests();
        DataTable GetPatientSpecialNeedsPreferences(string patientIen);
        void GetPatientSpecialNeedsPreferences(string patientIEN, ref SpecialNeedsPreferences prefs);
        bool SetPatientSpecialNeedsPreferences(string patientIEN, SpecialNeedsPreferences prefs);
//	    string GetPriorityGroup(string patientName);
        void Logout();
        string GetHospitalLocationIdByResourceIen(string resourceIen);
        List<PtCSchAppointmentType> GetPtCSchAppointmentTypes();
        List<DataTable> GetAuditReportData(DateTime? startDate, DateTime? endDate, string scheduler = "", string lastSub = "");
        DataTable GetClinicData(int pageSize, ref string lastSub, ref string searchStr);
        List<ScheduleUser> GetSchedulers();
        bool? GetClinicStopInfo(string patientIen, string hospitalLocationId);
        bool? GetServiceStopInfo(string patientIen, string serviceInfo);
        AppointmentType GetDefaultClinicAppointmentType(string clinicIEN, string clinicName, string sdNoLet = "");
        Patient GetPatient(ref Patient newPatient);
        List<Clinic> GetAssociatedClinicsForServiceSpeciatly(string svcSpecialtyName);
//	    IList<Clinic> GetHospitalLocationsAkaClincs();
        string GetResourceIdByClinicIdAndName(string clinicId, string clinName, bool includeInactive, bool silent = true);
        bool IsSensitive(string patientIen);
        List<Ethnicity> GetEthnicities();
        Ethnicity GetEthnicity(string patientIen);

        bool SetPatientDemographics(Patient patient);

        void GetPatientPhoneNumbers(ref Patient patient);
        IList<State> GetStates();
        IList<Country> GetCountries();

        ObservableCollection<AppointmentRequest> GetSensitiveRecordInfo(ObservableCollection<AppointmentRequest> appointmentRequests);
        IList<Race> GetRaces();
        Race GetRace(string patientIen);
//        IList<SchdResource> GetAllResources(bool includeInactive);
//        IList<SchdResource> GetResourcesByType(string type, bool includeInactive = false);
//        IList<SchdResource> GetResourcesByType(string type, string searchValue, bool includeInactive = false);
        //string GetClinicDefaultTimeScale(string clinName);  9_22_2015_Obsolete_bj
        DataTable GetClinicSetupParameters(string hospital_location_id, string hospital_location, string sdNoLet = "");
        DataTable SearchClinicSetupParameters(string searchString);
//	    DataTable SearchClinicSetupParams(string hospId, string hospLoc);
        DataTable GetClinicStop( string searchString );
//	    DataTable GetResources();
        DataTable GetHospitalLocations(string hospital_localtion_id);
//	    DataTable GetHospitalResources();
        State GetPatientState(string patientIen);
        string GetPatientCityState(string patientIen);
        string GetPatientWard(string patientIen);
//	    int GetClinicApptLength(string clinicIen); - This method has been replaces, use SchdResource.Clinic.Duration
        void LoadWaitList(AsyncValidatableModel.WorkCompletedMethod workCompletedMethod);
        List<AppointmentRequest> GetEwlAppointmentRequests(bool isInThread = false);
        List<AppointmentRequest> GetEwlAppointmentRequests(DataTable dt, bool isInThread = false);
        AppointmentRequest GetEwlApptRequest( string requestIen );

        /* Jasper Reports RPC Calls */
        void GetEncounterDataFilterData(AsyncValidatableModel.WorkCompletedMethod workCompletedMethod);
        string GetEncounterDataFilterData();
        void GetEncounterDataReportData(string reportType, string dateAgregate, string dateRange, AsyncValidatableModel.WorkCompletedMethod workCompletedMethod);
        string GetEncounterDataReportData(string reportType, string dateAgregate, string dateRange);
        /***************************/
        void RefreshAllAppointmentRequests();
        object LoadAllWaitListRecords();
        string ReopenWaitListRecordAfterCancelAppointment(string reqTypeAbbrev, string reqIen, string appointmentId, string name, string patIen, string desiredDate, bool desiredDateUpdated = false, bool addToWaitList = true);
        string GetPatientSelectionPrompts(string patientIEN, ref Patient patient, bool checkForSimilarPatients = true, bool checkForSensitiveRecord = true);
        bool IsDeceased(string patientIen);
        string IsOwnRecord(string patientIen);
        //////////////////////////
        bool GetWaitListRecords(string lastRec, List<string> messages = null);

        IList<NonDisplayPatientRequest> GetNonDisplayPatientRequests(string patientIen = "", string clinicIen = "");
        IList<DisabledClinic> GetDisabledClinics();
        List<AppointmentRequest> GetApptAppointmentRequests();
        List<AppointmentRequest> GetApptAppointmentRequests(DataTable dt);
//        IList<ProhibitedClinic> GetProhibitedClincs();
        //IList<SchdResource> GetProhibitedResourcesByType();
        //IList<ResourceUser> GetProhibitedResourceUsersByResourceId(string resourceId);
        bool IsProhibitedClinic(string clinicName, string clinicIen, bool bSilent = false);
        string ConvertToExternalMumpsFormat(DateTime dt);
        //bool GetWaitListRecordsForSelectedPatient(string patientIen, string lastRec = "", bool isNewQuery = false, bool byDescendingIen = false, string requestType = "");
        bool GetWaitListRecordsForSelectedPatient(string patientIen, string lastRec = "", bool isNewQuery = false, bool byDescendingIen = false, string requestType = "", string newlyOpenedRequestIen = "", bool isInThread = false, List<string> messages = null);
        bool GetFilteredWaitListRecords(string filters, string lastRec = "", bool isNewQuery = false, string sort = null, List<string> messages = null);
        bool UpdatePrivilegedUsers(List<string> parms);
//        string GetResourceIdByClinicIdAndNameForSpecialtyService(string clinicId, string clinName, bool includeInactive);
        List<ConsultUrgency> GetConsultUrgencyValues();
        int ClosestToZero(List<double> haystack);
        AppointmentRequest GetAppointmentRequestByAppointmentIen(string appointmentId, ref string reqType);
        AppointmentRequest GetAppointmentRequestByRequestIenAndType(string requestIen, string requestType);
        string WLClose(string requestManagmentIen, string selectedCause);
        string ARClose(string apptRequestIen, string selectedCause);
        void RemoveJasperReportDependenciesFromUsersFolder();
        int CheckDeferredRpcStatus(string rpcHandle);
        //string GetDeferredRpcData(string rpcHandle);
        string[][] GetDeferredRpcData(string rpcHandle);
        void ClearDeferredRpcData(string rpcHandle);
        bool UserHasAccessToProhibitedClinic(string hospLocId);
        SchdResource GetSdecResource(string resourceId);
        IList<SchdResource> GetSdecHResources(string resourceName);
        IList<SchdResource> SearchSdecHResources(string searchString, string lastSub = null);
        IList<Provider> GetSdecProviders(string providerName);
        IList<Provider> SearchSdecProviders(string searchStr, string lastSub = null);
        SchdResource GetSdecResourceByName(string resourceName);
        IList<ProhibitedClinic> GetSdecProhibitedClinics(string clinicName, string clinicIEN);
        IList<ProhibitedClinic> SearchSdecProhibitedClinics(string searchString);
        bool UnlockAppointmentRequest(string request);
        PatientDisabilitiesModel GetPatientDisabilities(string PatientDFN);
        string GetEwlClinicIen(string clinicName);
        SchdAppointment GetAppointment(SchdResource resource, string startDate, string endDate, string appointmentId);
        bool ClinicCanBookOnHoliday(string clincIen, string clinic);
//	    string GetAppointmentSlotsLeft(DateTime startTime, DateTime endTime);
        bool ClinicExists(string sClinicName);
        bool ServiceExists(string serviceName);
        bool CheckIfAvailabilityEverDefined(SchdResource resource, bool isSilent = false);
        bool SetMRTCChildRequest(AppointmentRequest request, string requestIEN = "", string appointmentIEN = "");
        bool UpdateMRTCChildPreferredDates(int requestIEN, List<DateTime> mrtcPrefDates);
        // Expects mrtcLinks to be a list of strings formatted as "AppointmentID^ApptRequestIEN"
        bool UpdateMRTCChildLinks(int requestIEN, List<string> mrtcLinks);
        AppointmentRequest GetApptRequestByRequestIEN(string requestIEN);
        SchdAppointment GetAppointmentByAppointmentID(string appointmentID);
        int GetMRTCParentBookedAppointmentsCount(string requestIEN);
        AppointmentRequest ReopenAppointmentRequestByRequestIen(string requestId, ref string reqType);
        int GetMRTCChildPreferredDates(int requestIEN, List<string> prefDates);
        IList<MaritalStatus> GetMaritalStatusList();
        IList<Religion> GetReligionList();

        List<string> GetUserPreferences( string userIen );
        bool SetUserPreference( string userIEN, List<string> prefs );

        void GetFilteredData( string waitListQueryLastRec, bool isNewQuery, bool isInThread = false, List<string> messages = null );

        List<ZipLinkInfo> GetLinkedCityState( string zipcode );

        NameValue GetCCOWCurrentPatient();

        void SetCurrentPatient( Patient selectedPatient );

        List<string> AddNewAppointment(AppointmentData a, PatientInformation p, bool bMakeChartRequest, AppointmentRequest apptRequest);

        string GetVistAPatientAppointmentStatus(string patientIen, DateTime appointmentDate);

        #region EXPANDED ENTRY CALLS
        string[] GetExpandedEntryInfomation(string patientIen, DateTime appointmentDateTime);
        string GetExpandedEntryDemographics(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedEntryEventLog(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedEntryWaitTime(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedCheckoutProviders(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedCheckoutDiagnosis(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedCheckoutStopCodes(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedCheckoutClassifications(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedCheckoutProcedures(string patientIen, string _filemanAppointmentDateTime);
        string GetExpandedPatientInfo(string patientIen, string _filemanAppointmentDateTime);

        #endregion

        #region CONTACT ATTEMPT CALLS
        string GetPatientContact(List<string> patientContactRequestParams);
        string GetContactAttempts(List<string> patientContactRequestParams);
        void UpdateContactAttempt(List<string> updatePatientContactRequestParams);
        void CreateNewSdecContact(List<string> createPatientContactRequestParams);
        string GetClinicStopCodes(List<string> clinicIen);
        void ResetPatientContactMainSequence(List<string> patientContactRequestParams);
        #endregion

        #region BENEFIT/ELIGIBILITY CALLS
        string GetPatientEligibilities(string patientIen);

        string GetEligibility(string dfn, string clinicIen, string appointmentDateTime);
        #endregion

        #region RTC CALLS
        string GetPrerequisites(string requestIen);
        #endregion
    }
}
