﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace ClinSchd.Infrastructure.Models
{
    internal class ADTDataRepository
    {
        // Constants variables
        private static int HEADER_PARSE_RANGE_0 = 0;

        private static int HEADER_PARSE_RANGE_1 = 1;
        private static int HEADER_PARSE_RANGE_3 = 3;
        private static int HEADER_PARSE_RANGE_4 = 4;
        private static int HEADER_PARSE_RANGE_5 = 5;
        private static int HEADER_PARSE_RANGE_6 = 6;
        private static char ROW_SEPARATOR_CHR = (char)30; // ASCII CHAR 30
        private static char UNIT_SEPARATOR_CHR = (char)31; // ASCII CHAR 31
        private static string EMPTY_STRING = "";
        private static char FIELD_SEPARATOR = '^';
        private static string CARROT = "^";
        private static string BMX_BMX_RPC_HEADER = "BMX";
        private static string BMX_BSDX_RPC_HEADER = "BSDX";
        private static string BMX_SDEC_RPC_HEADER = "SDEC";
        private static string BMX_SELECT_RPC_HEADER = "SELECT";
        private static string BMX_ORWU_RPC_HEADER = "ORWU";

        //public static double AVOGADROS_NUMBER   = 6.02214199e23;

        // Variables
        //private static nOVID.CIABrokerConnection ciaConn = null;
        //private Func<string, string, RPCConnection> connection;
        private Func<string, string, RPCBrokerConnection> connection;

        private static string _accessCode;
        private static string _verifyCode;
        private bool _adtDataExtType = true;
        private bool _genericRPCCall;
        private RPCResponse.ResponseType _rpcResponseType;

        private static RPCBrokerConnection _rpcBrokerConnection;

        public static bool IsLoggedIn { get; private set; }

        /* MSC/DKA 9/26/14 Added Logout() method to facilitate clean exit from RPC Broker connection */
        public void Logout()
        {
            if(_rpcBrokerConnection != null)
                _rpcBrokerConnection.close();
        }

        public ADTDataRepository()
        {
            _genericRPCCall = false;
            _rpcResponseType = RPCResponse.ResponseType.ARRAY;

            var host   = System.Configuration.ConfigurationManager.AppSettings["host"];
            var port   = System.Configuration.ConfigurationManager.AppSettings["port"];
            var nspace = System.Configuration.ConfigurationManager.AppSettings["nspace"];
            var style  = System.Configuration.ConfigurationManager.AppSettings["style"];
        
            connection = (a, v)=>GetConnection(host, port, a, v, nspace,style);
        }

        public void setADTDataExtType(bool b)
        {
            _adtDataExtType = b;
        }

        public bool isADTDataExtType()
        {
            return _adtDataExtType;
        }

        public string TestType(string[] columnNames, string column)
        {
            var count = 1;
            var currentColumn = column;
            IList<string> columnNamesList = columnNames;

            while (true)
            {
                if (columnNamesList.Contains(currentColumn))
                {
                    currentColumn = string.Format("{0}{1}", column, count);
                    count++;
                }
                break;
            }
            return currentColumn;
        }

        public string removeChar(string s, char c)
        {
            return s.Replace(c.ToString(), "");
        }

        public void ModifiyDuplicateColumnName(string column, string[] columnNames)
        {
            var count = 1;
            var index = 1;
            var arrayIndex = 0;
            var columnNamesLength = columnNames.Length;

            for (var i = 0; i < columnNamesLength; i++)
            {
                if (columnNames[arrayIndex].Equals(column))
                {
                    if (count > 1)
                    {
                        columnNames[arrayIndex] = string.Format("{0}{1}", column, index);
                        arrayIndex++;
                        index++;
                        count++;
                    }
                    else
                    {
                        count++;
                        arrayIndex++;
                    }
                }
                else
                {
                    arrayIndex++;
                }
            }
        }

        private string[][] ParseLinesAndFields(string[] items)
        {
            var lineNumber = 0;

            var list = new string[items.Length + 2][];
            
            foreach (var item in items)
            {
                /* ***********************************************************************************
                   START BLOCK: JTS -This block of code is a hack. A more elegant solution should be found given time.
                   ********************************************************************************* */

                // JTS - String item1 = removeChar(item, '');
                // JTS - String item2 = removeChar(item1, '');
                var item1 = removeChar(item, ROW_SEPARATOR_CHR);
                var item2 = removeChar(item1, UNIT_SEPARATOR_CHR);

                // don't bother with empty/blank lines
                // JTS - if (item.Trim().Equals("") == true) {
                if (item.Trim().Equals(EMPTY_STRING))
                {
                    //Must have header information
                    if (lineNumber == 0)
                    {
                        throw new RPCException("ADTDataRepository.ParseLinesAndFields (RPCException): Results does not contain header information.");
                    }

                    // skip further processing on this line and go to the next line
                    continue;
                }

                // JTS - String parts[] = item2.Split("\\^", -1);
                var parts = item2.Split(FIELD_SEPARATOR);
                /* ***********************************************************************************
                   END BLOCK:
                   *********************************************************************************** */

                if (lineNumber == 0)
                {
                    // throw exception on error
                    if (parts[0].Trim().Equals("-1"))
                    {
                        throw new RPCException("ADTDataRepository.ParseLinesAndFields (RPCException): CODE=" + parts[0].Trim() + "\nMessage=" + parts[1].Trim());
                    }

                    var currentField = 0;
                    var numberOfFields = parts.Length;
                    var fieldNames = new string[numberOfFields];
                    var fieldTypes = new string[numberOfFields];
                    var fieldSizes = new string[numberOfFields];

                    foreach (var fields in parts)
                    {
                        // JTS - field_names[current_field] = fields.Substring(6);
                        // JTS - field_types[current_field] = fields.Substring(1, 6);
                        // JTS - field_sizes[current_field] = fields.Substring(0, 1);
                        fieldNames[currentField] = fields.Substring(HEADER_PARSE_RANGE_6);
                        fieldSizes[currentField] = fields.Substring(HEADER_PARSE_RANGE_1, HEADER_PARSE_RANGE_5);
                        fieldTypes[currentField] = fields.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_1);

                        currentField++;
                    }

                    foreach (var column in fieldNames)
                    {
                        ModifiyDuplicateColumnName(column, fieldNames);
                    }

                    list[lineNumber] = fieldNames;
                    lineNumber++;

                    list[lineNumber] = fieldTypes;
                    lineNumber++;

                    list[lineNumber] = fieldSizes;
                    lineNumber++;
                }
                else
                {
                    // JTS - String[] row = item2.Split("\\^", -1);
                    var row = item2.Split(FIELD_SEPARATOR);

                    list[lineNumber] = row;
                    lineNumber++;
                }
            }
            return list;
        }

        private List<string> ParseFields(string[] items)
        {
            var row = new List<string>();

            foreach (var item in items)
            {
                // JTS - String parts[] = item.Split("\\^", -1);
                var parts = item.Split(FIELD_SEPARATOR);

                for (var i = 1; i < parts.Length; i++)
                {
                    var data = parts[i];
                    row.Add(data);
                }
            }
            return row;
        }

        private List<List<string>> ParseLines(string[] items)
        {
            var list = new List<List<string>>();

            foreach (var line in items)
            {
                // JTS - String lines[] = line.Split("", -1);
                var lines = line.Split(ROW_SEPARATOR_CHR);

                list.Add(ParseFields(lines));
            }
            return list;
        }

        private string[][] ParseHeadersAndRows(string[] items)
        {
            var lineNumber = 0;

            var list = new string[items.Length + 2][];

            //String[] items = line_items[0].Split("", -1);

            foreach (var item in items)
            {
                // don't bother with empty/blank lines
                // JTS - if (item.Trim().Equals("") == true) {
                if (item.Trim().Equals(EMPTY_STRING))
                {
                    //Must have header information
                    if (lineNumber == 0)
                    {
                        throw new RPCException("ADTDataRepository.ParseHeadersAndRows (RPCException): Results does not contain header information.");
                    }

                    // skip further processing on this line and go to the next line
                    continue;
                }

                // JTS - String parts[] = item.Split("\\^", -1);
                var parts = item.Split(FIELD_SEPARATOR);

                if (lineNumber == 0)
                {
                    var currentField = 0;
                    var numberOfFields = parts.Length;
                    var fieldNames = new string[numberOfFields];
                    var fieldTypes = new string[numberOfFields];
                    var fieldSizes = new string[numberOfFields];

                    foreach (var fields in parts)
                    {
                        // JTS - field_names[current_field] = fields.Substring(6);
                        // JTS - field_types[current_field] = fields.Substring(1, 6);
                        // JTS - field_sizes[current_field] = fields.Substring(0, 1);

                        if (fields == "T")
                            continue;

                        fieldNames[currentField] = fields.Substring(HEADER_PARSE_RANGE_6);
                        fieldSizes[currentField] = fields.Substring(HEADER_PARSE_RANGE_1, HEADER_PARSE_RANGE_5);
                        fieldTypes[currentField] = fields.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_1);

                        currentField++;
                    }

                    foreach (var column in fieldNames)
                    {
                        ModifiyDuplicateColumnName(column, fieldNames);
                    }

                    list[lineNumber] = fieldNames;
                    lineNumber++;

                    list[lineNumber] = fieldTypes;
                    lineNumber++;

                    list[lineNumber] = fieldSizes;
                    lineNumber++;
                }
                else
                {
                    list[lineNumber] = parts;
                    lineNumber++;
                }
            }
            return list;
        }

        private void ScrubItems(string[] items, char charToScrub)
        {
            var index = 0;

            foreach (var item in items)
            {
                // house keeping scrub data by removing ASCII control char 31 (Unit Separator)
                items[index] = removeChar(item, charToScrub);
                index++;
            }
        }

        private void ScrubItems(string[] items, string charsToScrub, string replaceWith)
        {
            var index = 0;

            var r = new Regex(charsToScrub);

            foreach (var item in items)
            {
                // house keeping scrub data by removing ASCII control char 31 (Unit Separator)
                items[index] = r.Replace(item, replaceWith);  //this was ReplaceAll -- dunno what's intended.
                index++;
            }
        }

        private string[][] parse2(string[] items)
        {
            if (items.Length != 1)
            {
                // JTS - StringBuilder result = new StringBuilder("");
                // JTS - StringBuilder lines = new StringBuilder("");
                var result = new StringBuilder(EMPTY_STRING);
                var lines = new StringBuilder(EMPTY_STRING);

                if (items.Length < 1)
                {
                    result.Append("few");
                }

                //else if(items.Length > 1) {
                else
                {
                    result.Append("many");
                    var count = 0;
                    foreach (var item in items)
                    {
                        lines.Append("" + count + ": " + item + ",");
                        count++;
                    }
                }

                throw new RPCException("ADTDataRepository.parse2 (RPCException): Too " + result + " lines to parse. You can only parse one line at a time: " + lines);
            }

            // house keeping: scrub data by removing ASCII control char 31 (Unit Separator)
            // JTS - ScrubItems(items, '');
            ScrubItems(items, UNIT_SEPARATOR_CHR);

            // house keeping: scrub data by removing the ASCII control
            // chars 10 (Line Feed), 13 (Carriage Return), 31 (Unit Separator)
            // JTS - ScrubItems(items, "\\r\\n|\\r|\\n|", "");
            // MSC/DKA 2014-09-15 Leave CR/LF for multi-line fields.
            //ScrubItems(items, GROUP_SEPARATOR, EMPTY_STRING);

            // We are concerned with the 1 and only line at this point to create our
            // list of unparsed rows by parsing ASCII control char 30 (Row Separator).
            // JTS - String line_items[] = items[0].Split("", -1);
            var lineItems = items[0].Split(ROW_SEPARATOR_CHR);

            return ParseHeadersAndRows(lineItems);
        }

        private List<List<string>> parse(string[] items)
        {
            // UNCOMMENT ONLY IF NECESSARY:
            // This is for standard parsing of lines and fields
            // List<List<String>> list = ParseLinesAndFields(items);

            // This is for special parsing of lines then fields
            var list = ParseLines(items);

            return list;
        }

        private void validateGenericRPCCall(string bsdxCall)
        {
            // JTS - bool bmx    = bsdx_call.Substring(0, 3).Equals("BMX");
            // JTS - bool bsdx   = bsdx_call.Substring(0, 4).Equals("BSDX");
            // JTS - bool select = bsdx_call.Substring(0, 6).Equals("SELECT");

            var bmx    = bsdxCall.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_3).Equals(BMX_BMX_RPC_HEADER);
            var bsdx   = bsdxCall.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_4).Equals(BMX_BSDX_RPC_HEADER);
            var sdec   = bsdxCall.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_4).Equals(BMX_SDEC_RPC_HEADER);
            var vse    = bsdxCall.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_6).Equals("SD VSE");
            var select = bsdxCall.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_6).Equals(BMX_SELECT_RPC_HEADER);
            var orwu   = bsdxCall.Substring(HEADER_PARSE_RANGE_0, HEADER_PARSE_RANGE_4).Equals(BMX_ORWU_RPC_HEADER);

            //if (bmx || bsdx || select) {
            if (select)
            {
                throw new Exception("ADTDataRepository.validateGenericRPCCall(Exception): Illegal operation: " + bsdxCall);
            }
            if (bmx || bsdx || sdec || vse || orwu)
            {
                _genericRPCCall = true;

                //rpc_response_type = ResponseType.SINGLE_VALUE;
                setResponseType(RPCResponse.ResponseType.SINGLE_VALUE);
            }
            else
            {
                _genericRPCCall = false;
                var diedOn = bsdxCall == Keys.PatientPromptRpcs.DiedOn;
                var idInfo = bsdxCall == Keys.PatientPromptRpcs.DiedOnInfo;
                var sensitiveBulleting = bsdxCall == Keys.PatientPromptRpcs.SensitiveBulletin;
                //rpc_response_type = ResponseType.ARRAY;
                if(diedOn || idInfo || sensitiveBulleting)
                    setResponseType(RPCResponse.ResponseType.SINGLE_VALUE);
                else  
                    setResponseType(RPCResponse.ResponseType.ARRAY);
            }
        }

        private bool isGenericRPCCall()
        {
            return _genericRPCCall;
        }

        public bool IsSingleValueReturn { get; set; }
        private RPCResponse.ResponseType getResponseType()
        {
            return _rpcResponseType;
        }

        private void setResponseType(RPCResponse.ResponseType arg)
        {
            _rpcResponseType = arg;
        }

        /*
         * NOTE: FOR DEBUG ONLY - this code is only for debugging purposes and can
         * be removed if necessary
         * <br>Method Name: <b>DebugInfo</b>
         * @param rpc_name
         * @param rpc_params
         * @return void
         */
        private void DebugInfo(string rpcName, string[] rpcParams)
        {
            var paramNumber = 0;
            var @params = new StringBuilder();

            if (rpcParams != null)
            {
                foreach (var param in rpcParams)
                {
                    paramNumber++;

                    // JTS - params.Append(param_number > 1 ? "^" : "");
                    @params.Append(paramNumber > 1 ? CARROT : EMPTY_STRING);
                    @params.Append(param.Trim());
                }
            }
        }

        public string[][] getADTDataExt(string remoteProcCallName, object[] rpcParams, bool isOvidCompliant = true)
        {

            //RPCConnection rpcConnection;
            var rpcName = remoteProcCallName.Trim();

            try
            {
                // determine if this is a BSDX call and handle accordingly
                validateGenericRPCCall(rpcName);
                //rpcConnection = connection(_accessCode, _verifyCode);
                _rpcBrokerConnection = connection(_accessCode, _verifyCode);
                var rpc = new VistaRPC(rpcName, getResponseType());

                // ***************************************************************
                // NOTE: FOR DEBUG ONLY
                // this code is only for debugging purposes and can be
                // removed if necessary
                // ***************************************************************
                //DebugInfo(rpcName, rpcParams);

                // ***************************************************************
                // END DEBUG ONLY
                // ***************************************************************

                if (rpcParams != null)
                {
                    //logger.Debug("ADTDataRepository.getADTDataExt - RPC call has parameters");
                    var paramNumber = 0;

                    foreach (var param in rpcParams)
                    {
                        paramNumber++;

                        // set up input arguments and remove leading/trailing spaces
                        if (param is string)
                            rpc.setParam(paramNumber, ((string)param).Trim().ToUpper());
                        else
                            rpc.setParam(paramNumber, param);
                    }   
                }

                // ***************************************************************
                // The rpcConnection returned null before putting in this condition. Why? I have no idea.
                // ***************************************************************
                // MSC/DKA 2014-09-03 Commented out.
                //if (_rpcBrokerConnection == null)
                //{
                //    //System.out.println("JTS - 1a (ADTDataRepository.getADTDataExt) CONNECTION IS NULL.");
                //    return null;
                //}

                //connection.setContext("MSC FM RESOURCE USER");
                //RPCResponse response = rpcConnection.execute(rpc);
                var response = _rpcBrokerConnection.execute(rpc);

                if (response != null)
                {
                    if (response.ToString().StartsWith("ERROR:") || response.ToString().Contains("M  ERROR=<"))
                        throw new RPCException(string.Format("The {0} RPC call returned an error:\r\n{1}", rpcName, response));

                    if (isGenericRPCCall() || IsSingleValueReturn)
                    {
                        string[] items = { response.getString() };
                        return parse2(items);
                    }
                    else
                    {
                        var items = response.getArray();
                        if (!isOvidCompliant)
                        {
                            var rslt = new[] { items };
                            return rslt;
                        }
                        return ParseLinesAndFields(items);
                    }
                }
            }
            catch (RPCException e)
            {
                throw new RPCException("RPCException (ADTDataRepository.getADTDataExt): ", e);
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "getADTDataExt", e);
                //MessageBox.Show(e.Message, "Get Data Ext Error");
            }
            return null;
        }

        public string[][] getADTDataExt(string remoteProcCallName, string[] rpcParams, bool isOvidCompliant = true)
        {

            //RPCConnection rpcConnection;
            var rpcName = remoteProcCallName.Trim();

            try
            {
                // determine if this is a BSDX call and handle accordingly
                validateGenericRPCCall(rpcName);
                //rpcConnection = connection(_accessCode, _verifyCode);
                _rpcBrokerConnection = connection(_accessCode, _verifyCode);
                var rpc = new VistaRPC(rpcName, getResponseType());

                // ***************************************************************
                // NOTE: FOR DEBUG ONLY
                // this code is only for debugging purposes and can be
                // removed if necessary
                // ***************************************************************
                DebugInfo(rpcName, rpcParams);

                // ***************************************************************
                // END DEBUG ONLY
                // ***************************************************************

                if (rpcParams != null)
                {
                    //logger.Debug("ADTDataRepository.getADTDataExt - RPC call has parameters");
                    var paramNumber = 0;

                    foreach (var param in rpcParams)
                    {
                        paramNumber++;

                        // set up input arguments and remove leading/trailing spaces
                        rpc.setParam(paramNumber, param.Trim().ToUpper());
                    }
                }

                // ***************************************************************
                // The rpcConnection returned null before putting in this condition. Why? I have no idea.
                // ***************************************************************
                // MSC/DKA 2014-09-03 Commented out.
                //if (_rpcBrokerConnection == null)
                //{
                //    //System.out.println("JTS - 1a (ADTDataRepository.getADTDataExt) CONNECTION IS NULL.");
                //    return null;
                //}

                //connection.setContext("MSC FM RESOURCE USER");
                //RPCResponse response = rpcConnection.execute(rpc);
                var response = _rpcBrokerConnection.execute(rpc);

                if (response != null)
                {
                    if (response.ToString().StartsWith("ERROR:") || response.ToString().Contains("M  ERROR=<"))
                       throw new RPCException(string.Format("The {0} RPC call returned an error:\r\n{1}", rpcName, response));

                    if (isGenericRPCCall() || IsSingleValueReturn)
                    {
                        string[] items = { response.getString() };
                        return parse2(items);
                    }
                    else
                    {
                        var items = response.getArray();
                        if (!isOvidCompliant)
                        {
                            var rslt = new[] {items};
                            return rslt;
                        }
                        return ParseLinesAndFields(items);
                    }
                }
            }
            catch (RPCException e)
            {
                throw new RPCException("RPCException (ADTDataRepository.getADTDataExt): ", e);
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "getADTDataExt", e);
                //MessageBox.Show(e.Message, "Get Data Ext Error");
            }
            return null;
        }

        public string GetAdtDataExtString(string remoteProcCallName, string[] rpcParams, bool isOvidCompliant = true)
        {
            //RPCConnection rpcConnection;
            var rpcName = remoteProcCallName.Trim();

            try
            {
                // determine if this is a BSDX call and handle accordingly
                validateGenericRPCCall(rpcName);
                //rpcConnection = connection(_accessCode, _verifyCode);
                _rpcBrokerConnection = connection(_accessCode, _verifyCode);
                var rpc = new VistaRPC(rpcName, getResponseType());

                // ***************************************************************
                // NOTE: FOR DEBUG ONLY
                // this code is only for debugging purposes and can be
                // removed if necessary
                // ***************************************************************
                DebugInfo(rpcName, rpcParams);

                // ***************************************************************
                // END DEBUG ONLY
                // ***************************************************************

                if (rpcParams != null)
                {
                    //logger.Debug("ADTDataRepository.getADTDataExt - RPC call has parameters");
                    var paramNumber = 0;

                    foreach (var param in rpcParams)
                    {
                        paramNumber++;

                        // set up input arguments and remove leading/trailing spaces
                        rpc.setParam(paramNumber, param.Trim().ToUpper());
                    }
                }

                // ***************************************************************
                // The rpcConnection returned null before putting in this condition. Why? I have no idea.
                // ***************************************************************
                // MSC/DKA 2014-09-03 Commented out.
                //if (_rpcBrokerConnection == null)
                //{
                //    //System.out.println("JTS - 1a (ADTDataRepository.getADTDataExt) CONNECTION IS NULL.");
                //    return null;
                //}

                //connection.setContext("MSC FM RESOURCE USER");
                //RPCResponse response = rpcConnection.execute(rpc);
                var response = _rpcBrokerConnection.execute(rpc);

                if (response != null)
                    return response.ToString();
            }
            catch (RPCException e)
            {
                throw new RPCException("RPCException (ADTDataRepository.GetADTDataExtString): ", e);
            }
            catch (Exception e)
            {
                throw new Exception("RPCException (ADTDataRepository.GetADTDataExtString): ", e);
            }
            return null;
        }

        public string GetAdtDataExtString(string remoteProcCallName, object[] rpcParams, bool isOvidCompliant = true)
        {
            //RPCConnection rpcConnection;
            var rpcName = remoteProcCallName.Trim();

            try
            {
                // determine if this is a BSDX call and handle accordingly
                validateGenericRPCCall(rpcName);
                //rpcConnection = connection(_accessCode, _verifyCode);
                _rpcBrokerConnection = connection(_accessCode, _verifyCode);
                var rpc = new VistaRPC(rpcName, getResponseType());

                // ***************************************************************
                // NOTE: FOR DEBUG ONLY
                // this code is only for debugging purposes and can be
                // removed if necessary
                // ***************************************************************
                //DebugInfo(rpcName, rpcParams);

                // ***************************************************************
                // END DEBUG ONLY
                // ***************************************************************

                if (rpcParams != null)
                {
                    //logger.Debug("ADTDataRepository.getADTDataExt - RPC call has parameters");
                    var paramNumber = 0;

                    foreach (var param in rpcParams)
                    {
                        paramNumber++;

                        // set up input arguments and remove leading/trailing spaces
                        if (param is string)
                            rpc.setParam(paramNumber, ((string)param).Trim().ToUpper());
                        else
                            rpc.setParam(paramNumber, param);
                    }
                }

                // ***************************************************************
                // The rpcConnection returned null before putting in this condition. Why? I have no idea.
                // ***************************************************************
                // MSC/DKA 2014-09-03 Commented out.
                //if (_rpcBrokerConnection == null)
                //{
                //    //System.out.println("JTS - 1a (ADTDataRepository.getADTDataExt) CONNECTION IS NULL.");
                //    return null;
                //}

                //connection.setContext("MSC FM RESOURCE USER");
                //RPCResponse response = rpcConnection.execute(rpc);
                var response = _rpcBrokerConnection.execute(rpc);

                if (response != null)
                    return response.ToString();
            }
            catch (RPCException e)
            {
                throw new RPCException("RPCException (ADTDataRepository.GetADTDataExtString): ", e);
            }
            catch (Exception e)
            {
                throw new Exception("RPCException (ADTDataRepository.GetADTDataExtString): ", e);
            }
            return null;
        }

        //public static IRpcConnection GetConnection(
        public static RPCBrokerConnection GetConnection(
            string server,
            string port,
            string accessCode,
            string verifyCode,
            string uci,
            string style)
        {
            {
                if (_rpcBrokerConnection == null)
                {
                    _rpcBrokerConnection = new RPCBrokerConnection(server, int.Parse(port), accessCode, verifyCode);
                    _rpcBrokerConnection.setContext("SDECRPC");
                }
                return _rpcBrokerConnection;
            }
        }

        internal string LoginUser(string accessCode, string verifyCode, out string szErrorMsg)
        {
            string duz = null;
            string szMsgTmp = null;
            try
            {
                var conn = connection(accessCode, verifyCode); //DKA XWB
                duz = conn.getDUZ();

                _accessCode = accessCode;
                _verifyCode = verifyCode;
                IsLoggedIn = true;
            }
            catch (Exception e)
            {
                Console.WriteLine(string.Format("Exception: {0}",e.Message));
                szMsgTmp = e.Message;
                IsLoggedIn = false;
            }
            szErrorMsg = szMsgTmp;
            return duz;
        }
    }
}
