﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public class AppointmentData : MonitoredValuesBaseClass, INotifyPropertyChanged
    {
        private IDataAccessService _dataAccessService;

        private AppointmentRequest apRequest;
        public AppointmentData()
        {
            apRequest = new AppointmentRequest(_dataAccessService);
            IsOverbook = false;
        }
        public AppointmentData(IDataAccessService dataAccessService, string patientId)
        {
            _dataAccessService = dataAccessService;
            PatientId = patientId;
            IsOverbook = false;
        }

        public string PatientId { get; set; }
        public string ResourceName { get; set; }

        public DateTime StartTimeValue
        {
            get { return Convert.ToDateTime(StartTime); }
            set
            {
                StartTime = value.ToString("M/d/yyyy hh:mm tt");
                OnPropertyChanged("StartTimeValue");

                if (LastSaveAttemptStartTime.HasValue)
                    OKButtonIsEnabled = (LastSaveAttemptStartTime.Value.CompareTo(value) != 0);
            }
        }

        public string StartDateFormatted {  get { return StartTimeValue.ToString("MM/dd/yyyy"); } }

        public DateTime? LastSaveAttemptStartTime { get; set; } = null;

        private bool _okButtonIsEnabled = true;
        public bool OKButtonIsEnabled
        {
            get { return _okButtonIsEnabled; }
            set
            {
                _okButtonIsEnabled = value;
                OnPropertyChanged("OKButtonIsEnabled");
            }
        }

        public string Notes { get; set; }

        public string StartTime
        {
            get { return GetMonitoredValue("StartTime"); }
            set
            {
                if ( Equals( value, StartTime ) )
                    return;

                SetMonitoredValue("StartTime", value);
                OnPropertyChanged( "StartTime" );

                if (!String.IsNullOrEmpty(StartTime) && !String.IsNullOrEmpty(Duration))
                    EndTime = StartTimeValue.AddMinutes(Convert.ToInt32(Duration)).ToString("M/d/yyyy hh:mm tt");
            }
        }

        private string _endTime;
        public string EndTime
        {
            get
            {
                //string _startTime = StartTime;
                ////To account for availability start time Offsets in VistA... i.e., 1:15 start time for a 30 minute appt len clinic...
                //if (!string.IsNullOrEmpty(_startTime) && !string.IsNullOrEmpty(_endTime) && Keys.Methods.IsDate(_startTime) && Keys.Methods.IsDate(_endTime))
                //{
                //    var t1 = Convert.ToDateTime(_startTime) - Convert.ToDateTime(_endTime);
                //    var t2 = new TimeSpan(0, 0, Convert.ToInt32(Duration), 0, 0);
                //    if (t1 < t2)
                //        _endTime = Convert.ToDateTime(_startTime).AddMinutes(Convert.ToInt32(Duration)).ToString();
                //}
                return _endTime;
            }
            set { _endTime = value; }
        }

        public string Duration
        {
            get { return GetMonitoredValue("Duration"); }
            set
            {
                SetMonitoredValue("Duration", value);
                OnPropertyChanged("Duration");

                if (!String.IsNullOrEmpty(StartTime) && !String.IsNullOrEmpty(Duration))
                    EndTime = StartTimeValue.AddMinutes(Convert.ToInt32(Duration)).ToString("M/d/yyyy hh:mm tt");
            }
        }
        public string AppointmentCommand { get; set; }
        public string AppointmentID { get; set; }
        public string AccessTypeID { get; set; }
        public int Slots { get; set; }
        private int Column { get; set; }
        public string AccessTypeName { get; set; }
        public string AppointmentRequestIen { get; set; }
        public string RequestType { get; set; }
        public SchdAppointment Appointment { get; set; }
        public string AppointmentTypeIen { get; set; }
        private IList<AppointmentType> _apptTypes; 
        public IList<AppointmentType> AppointmentTypes
        {
            get
            {
                if(_apptTypes == null)
                    _apptTypes = _dataAccessService.GetAppointmentTypes(PatientId);
                return _apptTypes;
            }
            set { _apptTypes = value; }
        }
        public bool IsServiceConnected { get; set; }
        public string ServicePercentage { get; set; }
        public bool IsOverbook { get; set; }


        public string EligibilityIen { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
