﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using ClinSchd.Infrastructure.Converters;
using ClinSchd.Infrastructure.Interfaces;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace ClinSchd.Infrastructure.Models
{
    public class AppointmentRequest : INotifyPropertyChanged
    {
        private readonly IDataAccessService _dataAccessService;

        public List<ScheduledAppointment> ScheduledAppointments;

        #region BJ'S PATIENT CONTACT REPLACEMENT
        public PatientContact PatientContact { get; set; }
        #endregion

        #region HPE OLD PATIENT CONTACT PROPERTY
        //public IList<PatientContact> PatientContacts { get; set; }
        #endregion


        public AppointmentRequest( IDataAccessService dataAccessService )
        {
            _dataAccessService = dataAccessService;
            // _dataAccessService.LoadGlobalRecordsets();
        }

        public IList<int> StopCodes { get; set; }

        public bool IsProhibitedClinic { get; set; }

        public string ClinicResourceId { get; set; }

        public int MrtcAppointmentCount { get; set; }

        public string ClinicStopName { get; set; }

        public string ClinicStopIen { get; set; }

        public PtCSchAppointmentType SelectedPtCSchAppointmentType { get; set; }

        public List<PtCSchAppointmentType> PtCSchApptTypes
        {
            get
            {
                if ( _ptCSchApptTypes == null )
                    _ptCSchApptTypes = _dataAccessService.GetPtCSchAppointmentTypes();
                return _ptCSchApptTypes;
            }
            set
            {
                _ptCSchApptTypes = value; 
                NotifyPropertyChanged( "PtCSchApptTypes" );
            }
        }

        //        public int MrtcNumberOfAppointmentsBooked { get; set; }
        public string RequestManagmentIen { get; set; }

        public string OriginatingUser { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged( string info )
        {
            if ( PropertyChanged != null )
                PropertyChanged( this, new PropertyChangedEventArgs( info ) );
        }

        private Patient _patient;

        public Patient Patient
        {
            get {
                return _patient;
            }
            set
            {
                if ( Equals( value, _patient ) )
                    return;
                _patient = value;
                NotifyPropertyChanged( "Patient" );
            }
        }

        public void PatientUpdate( object T )
        {
            // Patient Updated so I need to reaquire the patient data
            bool allRecordsDisplayed = false;
            var patients = _dataAccessService.GetPatients( Patient.Name, 1, ref allRecordsDisplayed );
           
            // Get SpecialPreferences
            var pt = patients.FirstOrDefault();
            if ( pt != null )
            {
                var sp = new SpecialNeedsPreferences();
                _dataAccessService.GetPatientSpecialNeedsPreferences( pt.IEN, ref sp );
                pt.SpecialNeedsPreferences = sp;
            }
            Patient = pt;
        }

        public List<PendingAppointment> PendingAppointments { get; set; }

        public string OriginationUser { get; set; }

        public string Institution { get; set; }

        private bool _multipleAppointmentsRequired;

        public bool MultipleAppointmentsRequired
        {
            get { return _multipleAppointmentsRequired; }
            set
            {
                _multipleAppointmentsRequired = value;
                NotifyPropertyChanged( "MultipleAppointmentsRequired" );
            }
        }

        public string MultipleAppointmentImage
        {
            get
            {
                if ( !MultipleAppointmentsRequired )
                    return null;

                if ( IsMRTCParent )
                    return "pack://application:,,,/VistaSchedulingGUI;component/Resources/parent_check.png";
                return "pack://application:,,,/VistaSchedulingGUI;component/Resources/check.png";
            }
        }

        public int NumberOfAppointmentsRequired { get; set; }

        public int AppointmentIntervalInDays { get; set; }

        public static int Errors { get; set; }

        private bool _chkSvcRelatedEnabled = true;

        public bool ChkSvcRelatedEnabled
        {
            get { return _chkSvcRelatedEnabled; }
            set
            {
                _chkSvcRelatedEnabled = value;
                NotifyPropertyChanged("ChkSvcRelatedEnabled");
            }
        }

        public IList<AppointmentType> AppointmentTypes
        {
            get { return _dataAccessService.GetAppointmentTypes( Patient.IEN ); }
        }

        public string AppointmentType
        {
            get { return _appointmentType; }
            set
            {
                _appointmentType = value;
                NotifyPropertyChanged( "AppointmentType" );
            }
        }

        public string Specialty { get; set; }

        public List<string> Specialties { get; set; }

        public string SpecialtyIen { get; set; }

        public ServiceSpecialty ServiceSpecialty { get; set; }

        public IList<ServiceSpecialty> ServiceSpecialties
        {
            get { return _dataAccessService.GetServiceSpecialties(); }
        }

        private IList<ServiceSpecialty> _apptServiceSpecialties;

        public IList<ServiceSpecialty> GetApptServiceSpecialties( string searchString )
        {
            //if ( _apptServiceSpecialties == null ) // Can't do this since the user might change the searchString
            //{
                _apptServiceSpecialties = new List<ServiceSpecialty>();
                var clinStop = _dataAccessService.GetClinicStop( searchString );
                foreach ( DataRow clin in clinStop.Rows )
                {
                    var newSs = new ServiceSpecialty
                    {
                        Name = clin["NAME"].ToString(),
                        Ien = clin["CLINIC_STOP_IEN"].ToString()
                    };
                    _apptServiceSpecialties.Add( newSs );
                }
            //}
            return _apptServiceSpecialties;
        }


        private readonly IList<Clinic> _ptCSchClinics = new List<Clinic>();

        public IList<Clinic> PtCSchClinics
        {
            get
            {
                if (_ptCSchClinics.Count == 0 && !string.IsNullOrEmpty(_clinName))
                {
                    var list = _dataAccessService.SearchPtCSchClinics(_clinName, "1");
                    foreach (var item in list)
                        _ptCSchClinics.Add(item);
                }
                return _ptCSchClinics;
            }
        }

        public Clinic Clinic { get; set; }

        private IList<Clinic> _ewlClinics;

        private IList<State> _states;

        public IList<State> States
        {
            get { return _states ?? ( _states = _dataAccessService.GetStates() ); }
        }

        private string _clinServiceSpecialtyMerge;

        public string ClinicServiceSpecialtyMerge
        {
            get
            {

                if ( !string.IsNullOrEmpty( _clinServiceSpecialtyMerge ) )
                    return _clinServiceSpecialtyMerge;
                if ( Keys.Methods.IsApptOrVet( AppointmentRequestType ) && !string.IsNullOrEmpty( ClinicStopName ) )
                {
                    _clinServiceSpecialtyMerge = ClinicStopName;
                    return _clinServiceSpecialtyMerge;
                }
                return string.IsNullOrEmpty( Specialty ) ? ClinName : Specialty;
            }
            set
            {
                _clinServiceSpecialtyMerge = value;
                NotifyPropertyChanged( "ClinicServiceSpecialtyMerge" );
            }
        }

        public string ClinName
        {
            get { return _clinName; }
            set
            {
                if (_clinName != value)
                {
                    _clinName = value;
                    _ptCSchClinics.Clear();
                    NotifyPropertyChanged("ClinName");
                    NotifyPropertyChanged("PtCSchClinics");
                }
            }
        }

        //public List<string> Clinics { get; set; }
        public string ClinicIen { get; set; }

        public string AddedToRequestManagmentQueueBy { get; set; }

        public string RequestManagmentComment { get; set; }

        public bool HasRequestManagmentComment
        {
            get
            {
                var prerequisities = _dataAccessService.GetPrerequisites(ApptRequestIen.ToString());
                return !string.IsNullOrEmpty( RequestManagmentComment ) 
                    || (!string.IsNullOrEmpty(prerequisities) && !prerequisities.Contains("-1"));
            }
        }

        public string RequestManagementCommentTrunc
        {
            get
            {
                string result = string.Empty;

                if ( MultipleAppointmentsRequired )
                {
                    if ( IsMRTCParent )
                        result = "{Appts: " + NumberOfAppointmentsRequired + "; Interval: " + AppointmentIntervalInDays + "} ";
                    else
                        result = "{Appt: " + AppointmentIntervalInDays + " of " + NumberOfAppointmentsRequired + "} ";
                }

                result += string.IsNullOrEmpty( RequestManagmentComment ) ? string.Empty : RequestManagmentComment;

                if ( !string.IsNullOrEmpty( result ) )
                {
                    return result.Length < 100 ? result : result.Substring( 0, 97 ) + "...";
                }
                return !string.IsNullOrEmpty( PtCSchComment ) ? PtCSchComment : "";
            }
        }

        //private string _desiredApptDatebyType;

        public string DesiredApptDateByType
        {
            get
            {
                if ( string.IsNullOrEmpty( DesiredApptDateShortString ) )
                    return null;
                if ( _desiredApptDates != null && _desiredApptDates.Count > 1 )
                    return Convert.ToDateTime( _desiredApptDates[0] ).ToShortDateString();
                return Convert.ToDateTime( DesiredApptDateShortString ).ToShortDateString();
            }
           // set { _desiredApptDatebyType = value; }
        }

        public string DesiredApptDateShortString
        {
            get { return _desiredApptDate == null ? "" : _desiredApptDate.Value.Date.ToString("MM/dd/yyyy"); }
        }

        public string DesiredApptDateShortStringSortable
        {
            get { return _desiredApptDate == null ? "" : _desiredApptDate.Value.Date.ToString( "yyyy/MM/dd" ); }
        }

        private DateTime? _initialDesiredApptDate;
        private DateTime? _desiredApptDate;

        public DateTime? DesiredApptDate
        {
            get { return _desiredApptDate == null ? null : (DateTime?)_desiredApptDate.Value.Date; }
            set
            {
                if ( _desiredApptDate == null && _initialDesiredApptDate == null )
                    _initialDesiredApptDate = value;
                _desiredApptDate = value;
            }
        }

        private IList<DateTime?> _desiredApptDates;

        public IList<DateTime?> DesiredApptDates
        {
            get
            {
                _desiredApptDates = new List<DateTime?>();
                if ( DesiredApptDate == null )
                    return null;

                if ( _initialDesiredApptDate == null )
                    _desiredApptDates.Add( DesiredApptDate );
                else
                    _desiredApptDates.Add( _initialDesiredApptDate );

                for ( int i = 1; i < NumberOfAppointmentsRequired; i++ )
                {
                    if ( _initialDesiredApptDate == null )
                        _desiredApptDates.Add( DesiredApptDate.Value.AddDays( AppointmentIntervalInDays*i ) );
                    else
                        _desiredApptDates.Add( _initialDesiredApptDate.Value.AddDays( AppointmentIntervalInDays*i ) );
                }
                return _desiredApptDates;
            }
        }

        public int? WaitTime
        {
            get
            {
                if ( AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch )
                {
                    var ptCSchDiff = DateTime.Today - DesiredApptDate;
                    if ( ptCSchDiff != null )
                    {
                        var ptCSchWaitTime = ptCSchDiff.Value.Days;
                        return ptCSchWaitTime;
                    }
                }
                if ( AppointmentRequestType == Keys.AppointmentRequestTypes.Consult && Keys.Methods.IsDate( ConsultUrgency ) )
                {
                    var conDiff = DateTime.Now - Convert.ToDateTime( ConsultUrgency );
                    var waitTime2 = conDiff.Days;
                    return waitTime2;
                }
                if ( OriginatingDate == null )
                    return null;
                var diff = DateTime.Today - OriginatingDate;
                var waitTime = diff.Value.Days;
                return waitTime;
            }
        }

        //private readonly List<string> _waitListTypes = new List<string>(new[] { "PCMM TEAM ASSIGNMENT", "PCMM POSITION ASSIGNMENT", "SERVICE/SPECIALITY", "SPECIFIC CLINIC" });

        public List<string> WaitListTypes { get; } = new List<string>( new[] {Keys.RequestManagementKeys.ServiceSpecialty, Keys.RequestManagementKeys.SpecificClinic} );

        public string WaitListType { get; set; }

        private DateTime? _originatingDate;

        public DateTime? OriginatingDate
        {
            get
            {
                if ( !_originatingDate.HasValue )
                    _originatingDate = DateTime.Now.Date;

                return Convert.ToDateTime( _originatingDate.Value.Month + "/" + _originatingDate.Value.Day + "/" + _originatingDate.Value.Year );
            }
            set { _originatingDate = value; }
        }

        //private string _originatingDateString;
        public string OriginatingDateString
        {
            get { return ( OriginatingDate ?? DateTime.Now ).ToString("MM/dd/yyyy"); }
            set { OriginatingDate = StringToDateTime( value ); }
        }

        public string OriginatingDateStringSortable
        {
            get { return ( OriginatingDate.HasValue ? OriginatingDate.Value : DateTime.Now ).ToString( "yyyy/MM/dd" ); }
        }

        public DateTime? DateEntered { get; set; }

        public string DateEnteredString
        {
            get { return DateEntered.HasValue ? DateEntered.Value.ToString() : ""; }
            set { DateEntered = StringToDateTime( value ); }
        }

        /*public string OriginatingUser { get; set; }
        public IList<Provider> OriginatingUsers
        {
            get { return _dataAccessService.GetProviders(); }
        }
        public string OriginatingUserName { get; set; }
        public IList<Provider> Providers { get; set; }
        */
        private IList<Provider> _providers = new List<Provider>();

        public Provider Provider
        {
            get { return _provider; }
            set
            {
                _provider = value;
                NotifyPropertyChanged( "Provider" );
            }
        }

        public bool HasMultipleProviders
        {
            get { return _hasMultipleProviders; }
            set
            {
                _hasMultipleProviders = value; 
                NotifyPropertyChanged( "HasMultipleProviders" );
            }
        }

        public IList<Provider> Providers
        {
            get { return _providers; }
            set
            {
                _providers = value;
                NotifyPropertyChanged("Providers");
            }
        }

        private IList<NameValue> _requestedByValues = null;

        public IList<NameValue> RequestedByValues
        {
            get
            {
                if (_requestedByValues == null)
                    _requestedByValues = Keys.RequestedByOptions;

                return _requestedByValues;
            }
        }

        private NameValue _requestedBy = null;

        public NameValue RequestedBy
        {
            get { return _requestedBy; }
            set
            {
                _requestedBy = value;
                NotifyPropertyChanged("RequestedBy");
                NotifyPropertyChanged("RequestedByText");
            }
        }

        public string RequestedByText
        {
            get
            {
                if (RequestedBy == null)
                    return string.Empty;

                return RequestedBy.Name;
            }
            set
            {
                RequestedBy = RequestedByValues.FirstOrDefault(rb => (rb.Name == value));
            }
        }

        private string _requestByProviderMerge;

        public string RequestedByProviderMerge
        {
            get
            {
                if ( AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl || Keys.Methods.IsApptOrVet( AppointmentRequestType ) )
                    return RequestedByText.Contains( "PATIENT" ) ? "PATIENT" : Provider.Name;
                return _requestByProviderMerge;
            }
            set
            {
                _requestByProviderMerge = value;
                NotifyPropertyChanged( "RequestedByProviderMerge" );
            }
        }

        private readonly List<string> _prioritys = new List<string>( new[] {"ASAP", "FUTURE"} );

        public List<string> Prioritys
        {
            get { return _prioritys; }
        }

        private string _priority;

        public string Priority
        {
            get
            {
                if ( !string.IsNullOrEmpty( _priority ) )
                    return _priority;
                if ( !Keys.Methods.IsDate( DesiredApptDateShortString ) )
                    return "ASAP";
                if ( Convert.ToDateTime( DesiredApptDateShortString ) <= DateTime.Now.AddDays( 30 ) )
                    return "ASAP";
                return "FUTURE";
            }
            set { _priority = value; }
        }

        public string PriorityText
        {
            get { return Priority == "A" ? _prioritys[0] : _prioritys[1]; }
        }

        public List<string> AppointmentRequestTypes { get; set; }

        public string AppointmentRequestType
        {
            get { return _appointmentRequestType; }
            set
            {
                _appointmentRequestType = value;
                NotifyPropertyChanged( "AppointmentRequestType" );
            }
        }

        public string AppointmentScheduledDate { get; set; }

        //Request Consultation type Request..
        public int ConsultationRequestIen { get; set; }

        public string ConsultUrgency { get; set; }


        //APPT - AppointmentRequest type Requests...
        public int ApptRequestIen { get; set; }

        //PtCSch..
        public bool IsPtCSch
        {
            get { return AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch; }
        }

        public bool IsEwl
        {
            get { return AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl; }
        }

        public bool IsAppt
        {
            get { return AppointmentRequestType == Keys.AppointmentRequestTypes.Appt 
                    || AppointmentRequestType == Keys.AppointmentRequestTypes.ReturnToClinic 
                    || IsVeteran; }
        }

        public bool IsW2VA
        {
            get { return AppointmentRequestType == Keys.AppointmentRequestTypes.WelcomeToVa; }
        }

        public bool IsApptOrEwl
        {
            get
            {
                return AppointmentRequestType == Keys.AppointmentRequestTypes.Appt
                       ||
                       AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl
                       ||
                       AppointmentRequestType == Keys.AppointmentRequestTypes.Veteran
                       ||
                       AppointmentRequestType == Keys.AppointmentRequestTypes.WelcomeToVa
                       ||
                       AppointmentRequestType == Keys.AppointmentRequestTypes.ReturnToClinic;
            }
        }

        public bool IsMRTCParent
        {
            get { return MultipleAppointmentsRequired && ( string.IsNullOrEmpty( MRTCParentRequestIEN.Trim() ) || MRTCParentRequestIEN == ApptRequestIen.ToString() ); }
        }

        public bool IsMRTCChild
        {
            get { return MultipleAppointmentsRequired && !IsMRTCParent; }
        }

        public bool IsMultipleAppointmentsRequiredVisible
        {
            get { return IsApptOrEwl && !IsVeteran; }
        }

        private bool _isVeteran;

        public bool IsVeteran
        {
            get { return _isVeteran; }
            set
            {
                _isVeteran = value;
                NotifyPropertyChanged( "IsVeteran" );
            }
        }

        public int PtCSchRequestIen { get; set; }

        public string Accession { get; set; }

        public string AppointmentLength { get; set; }

        public DateTime? PtCSchDate { get; set; }

        public DateTime? PtCSchDatePerPatient { get; set; }

        private string _ptCSchDateReminder;

        public string PtCSchDateReminder
        {
            get { return ShortDateTimeConverter.ConvertMonthStringDateToShortDateTime( _ptCSchDateReminder ); }
            set { _ptCSchDateReminder = value; }
        }

        public string PtCSchSecondPrintDate { get; set; }

        public string PtCSchComment { get; set; }

        private string _fasting = "NON-FASTING";

        public string Fasting
        {
            get
            {
                return _fasting.ToUpper() == "FASTING" ? _fasting : "NON-FASTING";
            }
            set { _fasting = value; }
        }

        public bool IsFasting
        {
            get { return _fasting.ToUpper() == "FASTING" || _fasting.ToUpper() == "F"; }
            set { Fasting = value ? "FASTING" : "NON-FASTING"; }
        }

        public string DesiredAppointmentDateByType
        {
            get { return DesiredApptDateShortString; }
        }

        public string DesiredAppointmentDateByTypeSortable
        {
            get { return DesiredApptDateShortStringSortable; }
        }

        //PtCSch.................................
        public List<string> PriorityGroups
        {
            get { return new List<string>( new[] {"", "GROUP 1", "GROUP 2", "GROUP 3", "GROUP 4", "GROUP 5", "GROUP 6", "GROUP 7", "GROUP 8"} ); }
        }

        public ObservableCollection<Provider> PtCSchProviders
        {
            get { return _ptCSchProviders; }
            set
            {
                _ptCSchProviders = value;
                NotifyPropertyChanged( "PtCSchProviders" );
            }
        }

        private readonly List<string> _enrolleeStatuses = new List<string>( new[] {"NEW", "ESTABLISHED"} );

        public List<string> EnrolleeStatuses
        {
            get { return _enrolleeStatuses; }
        }

        private string _enrolleeStatus;

        public string EnrolleeStatus
        {
            get
            {
                if ( string.IsNullOrEmpty( _enrolleeStatus ) )
                    return "";
                return _enrolleeStatus.Substring( 0, 1 ) == "E" ? "ESTABLISHED" : "NEW";
            }
            set { _enrolleeStatus = value; }
        }

        public string EnrollmentPriority { get; set; }

        public IList<Race> Races
        {
            get { return _dataAccessService.GetRaces(); }
        }

        private DateTime? StringToDateTime( string source )
        {
            DateTime? result = null;

            if ( !string.IsNullOrEmpty( source ) )
                result = Convert.ToDateTime( source.Replace( '@', ' ' ) );

            return result;
        }

        public string FormattedPhoneResidence
        {
            get { return FormattedPhoneNumber( Patient.PrimaryContactInfo.PhoneResidence ); }
            set { Patient.PrimaryContactInfo.PhoneResidence = value; }
        }

        public string FormattedPhoneWork
        {
            get { return FormattedPhoneNumber( Patient.PrimaryContactInfo.PhoneWork ); }
            set { Patient.PrimaryContactInfo.PhoneWork = value; }
        }

        public string FormattedPhoneNumber( string phoneNumber )
        {
            if ( phoneNumber == null )
                return string.Empty;

            return Keys.Methods.StandardPhoneFormat(phoneNumber);
        }

        public string FormattedSsnRmGrid
        {
            get { return Patient.SSNFormatted; }
        }

        public string MRTCParentRequestIEN { get; set; }

        public IList<string> MRTCAppointmentIENs { get; } = new List<string>();

        private List<PtCSchAppointmentType> _ptCSchApptTypes;
        private ObservableCollection<Provider> _ptCSchProviders = new ObservableCollection<Provider>();
        private string _clinName;
        private string _appointmentType;
        private string _appointmentRequestType;
        private ObservableCollection<Provider> _providerList = new ObservableCollection<Provider>();
        private Provider _provider;
        private bool _hasMultipleProviders;

        public IList<string> MRTCAppointmentRequestIENs { get; } = new List<string>();

        public string DispositionDate { get; set; }

        public string DispositionUserID { get; set; }

        public string DispositionUserName { get; set; }

        public string UniqueID
        {
            get { return AppointmentRequestType + "^" + RequestManagmentIen; }
        }

        private ContactAttempt lastContactAttempt;
        public ContactAttempt LastContactAttempt {
            get { return lastContactAttempt;  }
            set
            {
                if (value == lastContactAttempt)
                    return;

                lastContactAttempt = value;
            }
        }

        public bool NoLaterThanRequest { get; set; }

        public string PreRequisites { get; set; }

        public int RtcOrderIen { get; set; }


    }
}