﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Windows.Input;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public abstract class BaseViewModel : IBaseViewModel
    {
        #region public properties

        public bool ShowProgressAnimation
        {
            get { return _showProgressAnimation; }

            set
            {
                if ( _showProgressAnimation != value )
                {
                    _showProgressAnimation = value;
                    NotifyPropertyChanged( "ShowProgressAnimation" );
                }
            }
        }

        public string Title
        {
            get { return _title; }
            set
            {
                if ( _title != value )
                {
                    _title = value;
                    NotifyPropertyChanged( "Title" );
                }
            }
        }

        public object Parent
        {
            get { return _parent; }
            set
            {
                if ( _parent != value )
                {
                    _parent = value;
                    NotifyPropertyChanged( "Parent" );
                }
            }
        }

        #endregion public properties

        #region public methods

        /// <summary>
        /// In debug mode, notifies properties not listed
        /// by looking at the class in reflection.
        /// </summary>
        [Conditional( "DEBUG" )]
        [DebuggerStepThrough]
        private void VerifyPropertyName( string propertyName )
        {
            Debug.Assert(
                TypeDescriptor.GetProperties( this )[propertyName] != null,
                string.Format( "Invalid property name: {0}", propertyName ) );
        }

        #endregion public methods

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        public void NotifyPropertyChanged( string propertyName )
        {
            VerifyPropertyName( propertyName );

            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }

        #endregion INotifyPropertyChanged Members

        //protected static void ShowMessage( string text, string title )
        //{
        //    var mvm = new MessageViewModel
        //    {
        //        Message = text,
        //        Title = title
        //    };
        //    var dlg = new MessageView
        //    {
        //        Context = mvm
        //    };
        //    dlg.ShowDialog();
        //}

        //protected static bool ConfirmMessage( string title, string desc, string message, out string retComment, bool getComment )
        //{
        //    var vm = new ConfirmDialogViewModel
        //    {
        //        Title = title,
        //        ItemDesc = message,
        //        GetComment = getComment,
        //        Message = desc
        //    };
        //    var dlg = new ConfirmDialogView
        //    {
        //        Context = vm
        //    };
        //    var result = false;
        //    var comment = "";
        //    vm.CloseCallback = cd =>
        //    {
        //        dlg.Close();
        //        if ( cd )
        //        {
        //            comment = vm.Comment;
        //            result = true;
        //        }
        //    };
        //    dlg.ShowDialog();
        //    retComment = comment;
        //    return result;
        //}

        #region protected methods

        protected ICommand GetCommand( string name, Action<object> execute, Predicate<object> canExecute )
        {
            ICommand command;

            if ( _commands.ContainsKey( name ) )
            {
                command = _commands[name];
            }
            else
            {
                command = new CommandRelay( execute, canExecute );
                _commands.Add( name, command );
            }

            return command;
        }

        #endregion protected methods

        #region private fields

        private string _title;
        private bool _showProgressAnimation;
        private object _parent;
        private readonly Dictionary<string, ICommand> _commands = new Dictionary<string, ICommand>();

        #endregion private fields
    }
}