﻿using System.Windows;

namespace ClinSchd.Infrastructure.Models
{
    public class BindingProxy : Freezable
    {
        /*
         * The DataContext property in WPF is extremely handy, because it is automatically inherited by all children
         * of the element where you assign it; therefore you don’t need to set it again on each element you want to 
         * bind. However, in some cases the DataContext is not accessible: it happens for elements that are not part 
         * of the visual or logical tree. It can be very difficult then to bind a property on those elements…
         * */
        // This section is Critical for defining the DataContext of DataGridColumns and DataGridHeaders.
        // There is a 'fault' in the Column and header not inheriting the DataContext of the Grid.
        #region Overrides of Freezable     

        protected override Freezable CreateInstanceCore()
        {
            return new BindingProxy();
        }

        #endregion

        public object Data
        {
            get { return GetValue( DataProperty ); }
            set { SetValue( DataProperty, value ); }
        }

        // Using a DependencyProperty as the backing store for Data.  This enables animation, styling, binding, etc...    
        public static readonly DependencyProperty DataProperty =
            DependencyProperty.Register( "Data", typeof( object ), typeof( BindingProxy ), new UIPropertyMetadata( null ) );
    }
}
