﻿using System;
using System.Windows;
using CONTEXTMANAGERLib;
using VERGENCECONTEXTORLib;

namespace ClinSchd.Infrastructure.Models
{
    public class CCOWBase : IDisposable
    {
        /* Constants included in CCOW_const.pas */
        //The VistA Domain
        const string CCOW_LOGON_ID = "user.id.logon.vistalogon";
        //The VistA Token
        const string CCOW_LOGON_TOKEN = "user.id.logon.vistatoken";
        //The VistA user Name
        const string CCOW_LOGON_NAME = "user.id.logon.vistaname";
        // The VistA Vpid
        const string CCOW_LOGON_VPID = "user.id.logon.vpid";
        // The generic name
        const string CCOW_USER_NAME = "user.co.name";

        const string CCOW_PATIENT_ID_MRN_DFN = "patient.id.mrn.dfn_";
        const string CCOW_PATIENT_ID_MRN_NATIONAL_ID_NUMBER = "patient.id.mrn.nationalidnumber";
        const string CCOW_PATIENT_CO_NAME = "patient.co.patientname";

        private string _passCode1;
        private string _passCode2;

        public ContextorControl Contextor;
        public IContextManager Manager;
        public ContextorParticipant Participant;

        public int MostRecentContextCoupon
        {
            get { throw new NotImplementedException(); }
        }

        public CCOWPatientInfo CCOWPatient { get; set; }

        public CCOWBase(string applicationName)
        {
            Manager     = new ContextManager();
            Participant = new ContextorParticipant();
            Contextor   = new ContextorControl();

            Contextor.Committed += ContextorCommittedEvent;
            Contextor.Pending += ContextorPendingEvent;
            Contextor.Canceled += ContextorCanceledEvent;

            Keys.CCOWEnabled = StartCCOW(applicationName);
        }
        ~CCOWBase()
        {
            Manager.LeaveCommonContext(Manager.MostRecentContextCoupon);
        }

        public bool StartCCOW(string appName)
        {
            try
            {
                Keys.CCOWToken = GetCCOWHandle();
                if (Keys.CCOWToken.Length > 0)
                {
                    //CCOWContextor = (ContextorControl)Activator.CreateInstance(Type.GetTypeFromCLSID(Guid.Parse("8778ACF7-5CA9-11D3-8727-0060B0B5E137")));
                    Contextor.Run(appName + "#", "", true, "*");
                    //CCOWContextor.Run(appName, PassCode1 + PassCode2, true, "*");
                    try
                    {
                        // var result = Manager.JoinCommonContext(Participant, appName, true, true);
                    }
                    catch (Exception ex)
                    {
                        if (!ex.Message.Contains("0x8000222"))
                            MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace, "Join Failed");
                    }
                    CCOWSetUser(Keys.LoggedInUser, Keys.CCOWToken, Keys.CCOWDomainName + "^" + Keys.CCOWDomainIEN, null, Contextor);
                    return true;
                }
            }
            catch (Exception ex)
            {
                //CCOWManager.SuspendParticipation(CCOWManager.MostRecentContextCoupon);
                MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace, "Run Failure");
            }
            return false;
        }

        public bool HaveCCOWPatientData { get; set; }

        public bool SendCCOWPatientData { get; set; }

        private void ContextorCanceledEvent()
        {
            //Do nothing;
            Console.WriteLine("CCOW Patient canceled");
        }

        private void ContextorCommittedEvent()
        {
            HaveCCOWPatientData = true;

            // Flag that we have data for a patient;
            Console.WriteLine("CCOW Patient committed");
        }

        private void ContextorPendingEvent(object contextItemCollection)
        {
            HaveCCOWPatientData = false;

            if (contextItemCollection == null)
                return;

            // Get patient information
            CCOWPatient = new CCOWPatientInfo();
            var cic = (ContextItemCollection)contextItemCollection;
            foreach (ContextItem contextItem in cic)
            {
                if (contextItem.Name == CCOW_PATIENT_CO_NAME)
                    CCOWPatient.Name = contextItem.Value.Piece("^", 1) + "," + contextItem.Value.Piece("^", 2);
                if (contextItem.Name.StartsWith(CCOW_PATIENT_ID_MRN_DFN))
                    CCOWPatient.DFN = contextItem.Value;
                if (contextItem.Name.StartsWith(CCOW_PATIENT_ID_MRN_NATIONAL_ID_NUMBER))
                    CCOWPatient.NationalID = contextItem.Value;
            }
            Contextor.SetSurveyResponse("accept_conditional");

            HaveCCOWPatientData = true;
        }

        public void CCOWSetPatient(string ptName, string ptSite, string ptID, string ptICN, ContextorControl contextor)
        {
            if (contextor == null)
                return;

            try
            {
                //Part 1
                contextor.StartContextChange();
                //Part 2 Set the new proposed context data
                var contextItems = new ContextItemCollection();

                var dataItem1 = new ContextItem
                {
                    Name = CCOW_PATIENT_ID_MRN_DFN + ptSite,
                    Value = ptID
                };
                contextItems.Add(dataItem1);

                var dataItem2 = new ContextItem
                {
                    Name = CCOW_PATIENT_ID_MRN_NATIONAL_ID_NUMBER,
                    Value = ptICN
                };
                contextItems.Add(dataItem2);

                var dataItem3 = new ContextItem
                {
                    Name = CCOW_PATIENT_CO_NAME,
                    Value = ptName
                };
                contextItems.Add(dataItem3);

                //Part 3 Make change
                try
                {
                    var userResponse = contextor.EndContextChange(true, contextItems);
                }
                catch { }
                var result = contextor.CurrentContext;
                //We don't need to check CCOWresponce
            }
            catch (Exception e)
            {
                string exceptStr = e.Message;
                MessageBox.Show(exceptStr, "CCOWSetPatient");

                Console.WriteLine("CCOWSetPatient error is " + exceptStr, "CCOWSetPatient");
            }
        }

        public void CCOWSetUser(string uname, string token, string domain, string vpid, ContextorControl contextor)
        {
            if ( contextor == null )
                return;
            try
            {
                //Part 1
                contextor.StartContextChange();
                //Part 2 Set the new proposed context data
                var contextItems = new ContextItemCollection();

                var dataItem1 = new ContextItem
                {
                    Name = CCOW_LOGON_ID,
                    Value = domain
                };
                contextItems.Add(dataItem1);

                var dataItem2 = new ContextItem
                {
                    Name = CCOW_LOGON_TOKEN,
                    Value = token
                };
                contextItems.Add(dataItem2);

                var dataItem3 = new ContextItem
                {
                    Name = CCOW_LOGON_NAME,
                    Value = uname
                };
                contextItems.Add(dataItem3);

                var dataItem4 = new ContextItem
                {
                    Name = CCOW_LOGON_VPID,
                    Value = vpid
                };
                contextItems.Add(dataItem4);

                var dataItem5 = new ContextItem
                {
                    Name = CCOW_USER_NAME,
                    Value = uname
                };
                contextItems.Add(dataItem5);

                //Part 3 Make change
                var userResponse = contextor.EndContextChange(true, contextItems);
                //We don't need to check CCOWresponce
            }
            catch (Exception e)
            {
                string exceptStr = e.Message;

                Console.WriteLine("CCOWSetUser error is " + exceptStr, "CCCOWSetUser");
            }
        }

        public string GetCCOWHandle(/*CCOWRPCBrokerConnection ConnectedBroker*/)
        {
            var result = "";
            try
            {
                var adt = OVIDDataTableRepository.Instance;

                var results = adt.CallVistaRpc(Keys.RpcCalls.CCOWGetToken, null, false);
                if (results != null)
                {
                    result = results[0][0];
                    Keys.CCOWDomainName = results[0][1].Piece("^", 1);
                    Keys.CCOWDomainIEN  = results[0][1].Piece("^", 2);
                }

                results = adt.CallVistaRpc(Keys.RpcCalls.CCOWVaultParam, null, false);
                if (results != null)
                {
                    _passCode1 = results[0][0];
                    _passCode2 = results[0][1];
                }
            }
            catch
            {
                result = "";
            }
            return result;
        }

        public void Dispose()
        {
            Manager.LeaveCommonContext(Manager.MostRecentContextCoupon);
        }
    }
}
