﻿using System;
using VERGENCECONTEXTORLib;
using System.Windows;

namespace ClinSchd.Infrastructure.Models
{
    class CCOWRPCBrokerConnection : RPCBrokerConnection
    {
        /*======================================================================
         * 2016-12-05 Initial conversion from CCOWRPCBroker.pas
         * These are the methods that will be added to this class:
            procedure	SetConnected
            function	GetCCOWHandle
            procedure	CCOWsetUser
            function	GetCCOWduz
            function	GetCCOWtoken
            function	IsUserCleared
            function	WasUserDefined
            function	IsUserContextPending
            procedure	AuthenticateUser
         * 
         * ====================================================================*/
        /* Constants included in CCOW_const.pas */
        //The VistA Domain
        const string CCOW_LOGON_ID = "user.id.logon.vistalogon";
        //The VistA Token
        const string CCOW_LOGON_TOKEN = "user.id.logon.vistatoken";
        //The VistA user Name
        const string CCOW_LOGON_NAME = "user.id.logon.vistaname";
        // The VistA Vpid
        const string CCOW_LOGON_VPID = "user.id.logon.vpid";
        // The generic name
        const string CCOW_USER_NAME = "user.co.name";

        // These are declared as "var" in the Delphi implementation section:
        private string _token;
        private string _domain;
        private string _passCode1;
        private string _passCode2;

        /* Declarations missing from RPCBrokerConnection that are in trpcb.pas */
        private bool _connected;

        private bool _wasUserDefined;
        private bool _connecting;
        private int _socket;
        /* End of missing declarations */

        private ContextorControl Contextor { get; set; }

        private string CCOWLogonIDName { get; set; }

        private string CCOWLogonIDValue { get; set; }

        private string CCOWLogonName { get; set; }

        private string CCOWLogonNameValue { get; set; }

        private string CCOWLogonVpid { get; set; }

        private string CCOWLogonVpidValue { get; set; }

        public bool Connected { get { return _connected; } set { SetConnected(true); } }

        public CCOWRPCBrokerConnection(string host, int port, string access, string verify) : base(host, port, access, verify)
        {
#if DEBUG
            Console.WriteLine("Instantiating new RPCBrokerConnection to " + host + ", " + port + " as " + access);
#endif
            if ( host == null )
                throw new RPCException( "No host specified" );

            if ( access == null )
                throw new RPCException( "No access code specified" );

            if ( verify == null )
                throw new RPCException( "No verify code specified" );

            Connect(host, port, access, verify, null);
        }

        public CCOWRPCBrokerConnection(string host, int port, string token) : base(host, port, token)
        {
#if DEBUG
            Console.WriteLine("Instantiating new RPCBrokerConnection to " + host + ", " + port + " with " + token);
#endif
            if ( host == null )
                throw new RPCException( "No host specified" );

            if ( token == null )
                throw new RPCException( "No token specified" );

            Connect(host, port, null, null, token);
        }

        public void Connect(string host, int port, string access, string verify, string token)
        {
        }

        public bool WasUserDefined()
        {
            return _wasUserDefined;
        }

        public bool IsUserCleared()
        {
            if ( Contextor == null )
                return false;

            try
            {
                var contextItem = Contextor.CurrentContext;
                var dataItem1 = contextItem.Present( CCOW_LOGON_ID );
                if ( dataItem1 == null || dataItem1.Value == "" )
                    return true;
                
                _wasUserDefined = true;
            }
            finally
            {
            }
            return false;
        }

        public string GetCCOWHandle(CCOWRPCBrokerConnection connectedBroker)
        {
            var result = "";
            try
            {
                var adt = OVIDDataTableRepository.Instance;
                var results = adt.CallVistaRpc(Keys.RpcCalls.CCOWGetToken, null, false);
                if (results != null)
                {
                    result = results[0][0];
                    _domain = results[0][1].Piece( "^", 1 );
                }

                results = adt.CallVistaRpc(Keys.RpcCalls.CCOWVaultParam, null, false);
                if (results != null)
                {
                    _passCode1 = results[0][0];
                    _passCode2 = results[0][1];
                }
            }
            catch
            {
                result = "";
            }
            return result;
        }

        // Get name from CCOW context
        public string GetCCOWduz(ContextorControl contextor)
        {
            string result = "";

            if (contextor != null)
            {
                try
                {
                    var contextItem = contextor.CurrentContext;
                    // See if context contains the ID item
                    var dataItem1 = contextItem.Present( CCOW_LOGON_ID );
                    if (dataItem1 != null)
                    {
                        result = dataItem1.Value;
                        if (result != "")
                            _wasUserDefined = true;
                    }
                }
                finally { }
            }
            return result;
        }

        public bool IsUserContextPending(IContextItemCollection contextItemCollection)
        {
            if ( !WasUserDefined() )
                return false;

            var value = "";

            var dataItem1 = contextItemCollection.Present(CCOW_LOGON_ID);
            if ( dataItem1 != null ) //1
                value = dataItem1.Value;

            dataItem1 = contextItemCollection.Present(CCOW_LOGON_ID);
            if ( dataItem1 != null )
                value += "^" + dataItem1.Value;

            dataItem1 = contextItemCollection.Present(CCOW_LOGON_NAME);
            if ( dataItem1 != null )
                value += "^" + dataItem1.Value;

            dataItem1 = contextItemCollection.Present(CCOW_LOGON_VPID);
            if ( dataItem1 != null )
                value += "^" + dataItem1.Value;
            
            dataItem1 = contextItemCollection.Present(CCOW_USER_NAME);
            if ( dataItem1 != null )
                value += "^" + dataItem1.Value;

            return value == "";
        }

        //CCOW start
        public void CCOWSetUser(string uname, string token, string domain, string vpid, ContextorControl contextor)
        {
            if ( contextor == null )
                return;

            try
            {
                //Part 1
                contextor.StartContextChange();
                //Part 2 Set the new proposed context data
                var data = new ContextItemCollection();
                var dataItem1 = new ContextItem
                {
                    Name = CCOW_LOGON_ID,
                    Value = domain
                };
                data.Add(dataItem1);

                var dataItem2 = new ContextItem
                {
                    Name = CCOW_LOGON_TOKEN,
                    Value = token
                };
                data.Add(dataItem2);

                var dataItem3 = new ContextItem
                {
                    Name = CCOW_LOGON_NAME,
                    Value = uname
                };
                data.Add(dataItem3);

                var dataItem4 = new ContextItem
                {
                    Name = CCOW_LOGON_VPID,
                    Value = vpid
                };
                data.Add(dataItem4);

                var dataItem5 = new ContextItem
                {
                    Name = CCOW_USER_NAME,
                    Value = uname
                };
                data.Add(dataItem5);

                //Part 3 Make change
                contextor.EndContextChange(true, data);
                //We don't need to check CCOWresponce
            }
            finally { }
        }

        //Get Token from CCOW context
        public string GetCCOWToken(ContextorControl contextor)
        {
            var result = "";
            var name = CCOW_LOGON_TOKEN;

            if ( contextor == null )
                return "";
            try
            {
                var contextItem = contextor.CurrentContext;

                //See if context contains the ID item
                var dataItem = contextItem.Present(name);
                if (dataItem != null)
                {
                    result = dataItem.Value;
                    if (result != "")
                        _wasUserDefined = true;
                }
                CCOWLogonIDName = CCOW_LOGON_ID;
                CCOWLogonName   = CCOW_LOGON_NAME;
                CCOWLogonVpid   = CCOW_LOGON_VPID;

                dataItem = contextItem.Present(CCOW_LOGON_ID);
                if (dataItem != null)
                    CCOWLogonIDValue = dataItem.Value;

                dataItem = contextItem.Present(CCOW_LOGON_NAME);
                if (dataItem != null)
                    CCOWLogonNameValue = dataItem.Value;

                dataItem = contextItem.Present(CCOW_LOGON_VPID);
                if (dataItem != null)
                    CCOWLogonVpidValue = dataItem.Value;
            }
            finally { }
            return result;
        }

        //public void AuthenticateUser(CCOWRPCBrokerConnection ConnectingBroker)
        //{
        //    bool SaveClearParmeters, SaveClearResults;
        //    TParams SaveParam;
        //    string SaveRemoteProcedure, SaveRpcVersion;
        //    TStrings SaveResults;
        //    bool blnSignedOn;
        //    bool SaveKernelLogin;
        //    TVistaLogin SaveVistaLogin;
        //    TExceptionEvent OldExceptionHandler;
        //    THandle OldHandle;

        //    SaveParam = TParams.Create(null);
        //    SaveParam.Assign(ConnectingBroker.Param);                  //save off settings
        //    SaveRemoteProcedure = ConnectingBroker.RemoteProcedure;
        //    SaveRpcVersion = ConnectingBroker.RpcVersion;
        //    SaveResults = ConnectingBroker.Results;
        //    SaveClearParmeters = ConnectingBroker.ClearParameters;
        //    SaveClearResults = ConnectingBroker.ClearResults;
        //    ConnectingBroker.ClearParameters = true;                  //set'em as I need'em
        //    ConnectingBroker.ClearResults = true;
        //    SaveKernelLogin = ConnectingBroker.KernelLogin;     //  p13
        //    SaveVistaLogin = ConnectingBroker.Login;            //  p13

        //    blnSignedOn = false;                       //initialize to bad sign-on

        //    if (ConnectingBroker.AccessVerifyCodes != "")   // p13 handle as AVCode single signon
        //    {
        //        ConnectingBroker.Login.AccessCode = Piece(ConnectingBroker.AccessVerifyCodes, ";", 1);
        //        ConnectingBroker.Login.VerifyCode = Piece(ConnectingBroker.AccessVerifyCodes, ";", 2);
        //        ConnectingBroker.Login.Mode = lmAVCodes;
        //        ConnectingBroker.KernelLogIn = false;
        //    }

        //    //CCOW start
        //    if (ConnectingBroker.KernelLogIn && (ConnectingBroker.Contextor != null))
        //    {
        //        CCOWToken = ConnectingBroker.GetCCOWtoken(ConnectingBroker.Contextor);
        //        if (CCOWtoken.Length > 0)
        //        {
        //            ConnectingBroker.FKernelLogIn = false;
        //            ConnectingBroker.Login.Mode = lmAppHandle;
        //            ConnectingBroker.Login.LogInHandle = CCOWtoken;
        //        }
        //    }
        //    //CCOW end
        //    //CCOW Start                                // p13  following section for silent signon
        //    if (!ConnectingBroker.FKernelLogIn)
        //    {
        //        if (ConnectingBroker.FLogin != null)     //the user.  vistalogin contains login info
        //        {
        //            blnsignedon = SilentLogin(ConnectingBroker);    // RpcSLogin unit
        //            if (!blnSignedOn)
        //            {     //Switch back to Kernel Login
        //                ConnectingBroker.FKernelLogIn = true;
        //                ConnectingBroker.Login.Mode = lmAVCodes;
        //            }
        //        }
        //        //CCOW end

        //        if (ConnectingBroker.FKernelLogIn)
        //        {   //p13
        //            if (Assigned(Application.OnException))
        //            {
        //                OldExceptionHandler = Application.OnException;
        //            }
        //            else
        //                OldExceptionHandler = null;
        //            Application.OnException = TfrmErrMsg.RPCBShowException;
        //            frmSignon = TfrmSignon.Create(Application);
        //            try
        //            {
        //                //    ShowApplicationAndFocusOK(Application);
        //                OldHandle = GetForegroundWindow;
        //                SetForegroundWindow(frmSignon.Handle);
        //                PrepareSignonForm(ConnectingBroker);
        //                if (SetUpSignOn)                       //SetUpSignOn in loginfrm unit.
        //                {                                     //True if signon needed
        //                    if (frmSignOn.lblServer.Caption != "")
        //                    {
        //                        frmSignOn.ShowModal;                    //do interactive logon   // p13
        //                        if (frmSignOn.Tag == 1)               //Tag=1 for good logon
        //                            blnSignedOn = true;                   //Successfull logon
        //                    }
        //                }
        //                else                                      //False when no logon needed
        //                    blnSignedOn = NoSignOnNeeded;          //Returns True always (for now!)
        //                if (blnSignedOn)                       //P6 If logged on, retrieve user info.
        //                {
        //                    GetBrokerInfo(ConnectingBroker);
        //                    if (!SelDiv.ChooseDiv("", ConnectingBroker))
        //                    {
        //                        blnSignedOn = false;//P8
        //                        /*Select division if multi - division user.First parameter is 'userid'
        //                         (DUZ or username) for future use. (P8)*/
        //                        ConnectingBroker.Login.ErrorText = "Failed to select Division";  // p13 set some text indicating problem
        //                    }
        //                }
        //                SetForegroundWindow(OldHandle);
        //            }
        //            finally
        //            {
        //                frmSignon.Free();
        //                //      frmSignon.Release;                        //get rid of signon form

        //                //      if (ConnectingBroker.Owner is TForm) {
        //                //        SetForegroundWindow(TForm(ConnectingBroker.Owner).Handle)
        //                //      else
        //                //        SetForegroundWindow(ActiveWindow);
        //                ShowApplicationAndFocusOK(Application);
        //            } //try
        //            if (Assigned(OldExceptionHandler))
        //                Application.OnException = OldExceptionHandler;
        //        }   //if kernellogin
        //            // p13  following section for silent signon
        //        if ((!ConnectingBroker.KernelLogIn) && (!blnsignedon))     // was doing the signon twice if already true
        //            if (ConnectingBroker.Login != null)     //the user.  vistalogin contains login info
        //                blnsignedon = SilentLogin(ConnectingBroker);    // RpcSLogin unit
        //        if (!blnsignedon)
        //        {
        //            //    ConnectingBroker.Login.FailedLogin(ConnectingBroker.Login);
        //            TXWBWinsock(ConnectingBroker.XWBWinsock).NetworkDisconnect(ConnectingBroker.Socket);
        //        }
        //        else
        //            GetBrokerInfo(ConnectingBroker);

        //        //reset the Broker
        //        ConnectingBroker.ClearParameters = SaveClearParmeters;
        //        ConnectingBroker.ClearResults = SaveClearResults;
        //        ConnectingBroker.Param.Assign(SaveParam);                  //restore settings
        //        SaveParam.Free();
        //        ConnectingBroker.RemoteProcedure = SaveRemoteProcedure;
        //        ConnectingBroker.RpcVersion = SaveRpcVersion;
        //        ConnectingBroker.Results = SaveResults;
        //        ConnectingBroker.FKernelLogin = SaveKernelLogin;         // p13
        //        ConnectingBroker.FLogin = SaveVistaLogin;                // p13

        //        if (!blnSignedOn)                     //Flag for unsuccessful signon.
        //            TXWBWinsock(ConnectingBroker.XWBWinsock).NetError("", XWB_BadSignOn);               //Will raise error.
        //    }
        //}        // **DKA** Placeholder stubs to allow clean compile


        public void SetConnected(bool value)
        {
            //string BrokerDir, Str1, Str2, Str3 ;
            MessageBox.Show("CCOWRPCBroker.SetConnected - Inside SetConnected");
            //Login.ErrorText = "";
            if (Connected != value)
            {        // && !(csReading in ComponentState)) {
                MessageBox.Show("CCOWRPCBroker.SetConnected - Connected != Value");
                if (value && !_connecting )
                {
                    MessageBox.Show("CCOWRPCBroker.SetConnected - Checking Existing Socket");
                    _socket = ExistingSocket(this);
                    MessageBox.Show(string.Format("CCOWRPCBroker.SetConnected - FSocket = {0}", _socket));
                    _connecting = true; // FConnected := true;
                    try
                    {
                        if (true)  //if (FSocket == 0) **DKA** Always do this for now
                        {
                            /*Execute Client Agent from directory in Registry.*/
                            //BrokerDir = ReadRegData(HKLM, REG_BROKER, "BrokerDr");
                            //if (BrokerDir != "")
                            //    ProcessExecute(BrokerDir + "\ClAgent.Exe", sw_ShowNoActivate)
                            //else
                            //    ProcessExecute("ClAgent.Exe", sw_ShowNoActivate);
                            //              if (DebugMode && (!OldConnectionOnly))
                            //              {
                            //                  Str1 = "Control of debugging FOR UCX OR NON-CALLBACK CONNECTIONS has been moved from the client to the server. To start a Debug session, do the following:" +#13#10#13#10;
                            //Str2 = "1. On the server, set initial breakpoints where desired." +#13#10+"2. DO DEBUG^XWBTCPM."+#13#10+"3. Enter a unique Listener port number (i.e., a port number not in general use)."+#13#10;
                            //Str3 = "4. Connect the client application using the port number entered in Step #3.";
                            //                  ShowMessage(Str1 + Str2 + Str3);
                            //              }
                            //TXWBWinsock(XWBWinsock).IsBackwardsCompatible = IsBackwardCompatibleConnection;
                            //TXWBWinsock(XWBWinsock).OldConnectionOnly = OldConnectionOnly;
                            //FSocket = TXWBWinsock(XWBWinsock).NetworkConnect(DebugMode, FServer,
                            //ListenerPort, FRPCTimeLimit);
                            //AuthenticateUser(Self);
                            //StoreConnection(Self); //MUST store connection before CreateContext()
                            //CCOW start
                            MessageBox.Show(string.Format("CCOWRPCBroker.SetConnected\nContextor is {0}, CCOWToken.Length={1}",Contextor == null ? "null" : "not null",_token == null ? "<No CCOWToken>" : _token.Length.ToString()));
                            if (true) // if ((Contextor != null) && (CCOWToken.Length == 0)) **DKA** Force the creation for now.
                            {
                                MessageBox.Show("CCOWRPCBroker.SetConnected\nContextor != null. About to get CCOWToken.");
                                //Get new CCOW token
                                _token = GetCCOWHandle(this);
                                MessageBox.Show(string.Format("Got CCOWToken. Length={0}", _token.Length));
                                if (_token.Length > 0)
                                {
                                    try
                                    {
                                        MessageBox.Show("CCOWRPCBroker.SetConnected\nAbout to create ContextorControl");
                                        //RPCBContextor = ContextorControl..Create(Application);
                                        var rpcbContextor = new ContextorControl();
                                        MessageBox.Show("CCOWRPCBroker.SetConnected\nCalling RCBContextor.Run");
                                        rpcbContextor.Run("BrokerLoginModule#", _passCode1 + _passCode2, true, "*");
                                        //CCOWsetUser(user.name, CCOWToken, Domain, user.Vpid, RPCBContextor); //Clear token
                                        // TODO: Call XUS GET USER INFO to get the VPID
                                        MessageBox.Show("CCOWRPCBroker.SetConnected\nCalled Contextor.Run. About to call CCOWsetUser.");
                                        CCOWSetUser(Keys.LoggedInUser, _token, _domain, "", rpcbContextor); //Clear token
                                        MessageBox.Show("CCOWRPCBroker.SetConnected\nCalled CCOWsetUser. Saving values.");
                                        CCOWLogonIDName = Keys.LoggedInUserIen; // CCOW_LOGON_ID;
                                        CCOWLogonIDValue = _domain;
                                        CCOWLogonName = CCOW_LOGON_NAME;
                                        CCOWLogonNameValue = Keys.LoggedInUser;
                                        if (Keys.LoggedInUser != "")
                                            _wasUserDefined = true;
                                        CCOWLogonVpid = CCOW_LOGON_VPID;
                                        CCOWLogonVpidValue = ""; // user.Vpid; **DKA**
                                        //RPCBContextor.Free();
                                        MessageBox.Show("CCOWRPCBroker.SetConnected\nSaved Values. Nulling Contextor.");
                                        rpcbContextor = null;
                                        MessageBox.Show("CCOWRPCBroker.SetConnected\nNulled Contextor.");
                                    }
                                    catch(Exception err)
                                    {
                                        MessageBox.Show(string.Format("CCOWRPCBroker.SetConnected\nProblem with Contextor.Run\n{0}", err.Message)); // TODO: Replace with Scheduler Alert **DKA**
                                        //FreeAndNil(RPCBContextor); // TODO: Figure out replacement for this **DKA**
                                    }
                                } // if Length(CCOWToken) > 0
                            } //if
                              //CCOW end
                              //FPulse.Enabled = true; //P6 Start heartbeat. **DKA**
                              //CreateContext(""); //Closes XUS SIGNON context.
                        }
                        else
                        { //p13
                            //StoreConnection(Self); **DKA**
                            //FPulse.Enabled = true; //p13 **DKA**
                        } //p13
                        _connected = true; // jli mod 12/17/01
                        _connecting = false;
                    }
                    //catch
                    //{
                    /* **DKA** Commented out entire catch block
                     * //on E: EBrokerError do
                    {
                        if (err.Code == XWB_BadSignOn)
                            TXWBWinsock(XWBWinsock).NetworkDisconnect(FSocket);
                        FSocket = 0;
                        FConnected = false;
                        FConnecting = false;
                        FRPCBError = E.Message; // p13  handle errors as specified
                        if (Login.ErrorText != "")
                            FRPCBError = E.Message + chr(10) + Login.ErrorText;
                        if (Assigned(FOnRPCBFailure)) // p13
                            FOnRPCBFailure(Self) // p13

                        if (ShowErrorMsgs == semRaise)
                            Raise; // p13
                                   //          raise;    /*this is where I would do OnNetError*/
                    //} /*on*/
                    //}
                    finally { }
                } /*try*/
            } /*if*/
              //if (!Value)
              //{ //p13
              //    FConnected = false; //p13
              //    FPulse.Enabled = false; //p13
              //    if (RemoveConnection(Self) == NoMore)
              //    {
              //        /*FPulse.Enabled = false; ///P6;p13 */
              //        TXWBWinsock(XWBWinsock).NetworkDisconnect(Socket); /*actually disconnect from server*/
              //        FSocket = 0; /*store internal*/
              //                        //FConnected := false;      //p13
              //    } /*if*/ 
              //} /*else*/
              //} /*if*/ 
        }

        private int ExistingSocket( CCOWRPCBrokerConnection ccowrpcBrokerConnection )
        {
            throw new NotImplementedException();
        }
    }
}
