﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Models
{
    public class CSlot
    {
        public const string AvailabilityCodes = "0123456789jklmnopqrstuvwxyz";
        public const string OverbookCodes = "0ABCDEFGHIJKLMNOPQRSTUVW";
        public const string UnavailableOverbookCodes = "*$!@#?"; 
        private const string CanceledCode = "X";

        public DateTime? Date { get; set; }

        public DateTime StartTime { get; set; }

        public DateTime EndTime { get; set; }

        public string SlotCount { get; set; }

        public int DurationIndex
        {
            get { return Math.Max(GetDurationIndex(StartTime), GetDurationIndex(EndTime));  }
        }

        public int Duration { get { return 60 / DurationIndex; } }

        private int GetDurationIndex(DateTime source)
        {
            switch(source.Minute)
            {
                case 0:
                    return 1;
                case 30:
                    return 2;
                case 20:
                case 40:
                    return 3;
                case 15:
                case 45:
                    return 4;
                case 10:
                case 50:
                    return 6;
                case 5:
                case 25:
                case 35:
                case 55:
                    return 12;
                default:
                    return 2;
            }
        }

        public bool IsInSlot(DateTime startTime, DateTime endTime)
        {
            return Keys.Methods.TimesOverlap(startTime, endTime, StartTime, EndTime);
        }

        public int AvailabilityCount
        {
            get
            {
                int slotCount = AvailabilityCodes.IndexOf(SlotCount);
                return ((AccessType == Keys.AccessTypes.Available) ? ((slotCount < 0) ? 0 : slotCount) : 0);
            }
        }

        public int OverbookedCount { get { return OverbookCodes.IndexOf(SlotCount); } }

        public string AccessType
        {
            get
            {
                if (!String.IsNullOrEmpty(SlotCount) && ((AvailabilityCodes.Contains(SlotCount) || OverbookCodes.Contains(SlotCount))))
                    return Keys.AccessTypes.Available;
                if (!String.IsNullOrEmpty(SlotCount) && (SlotCount == CanceledCode))
                    return Keys.AccessTypes.Cancelled;
                else
                    return Keys.AccessTypes.Unavailable;
            }
        }

        public bool IsOverbookOutOfHours { get { return ((AccessType == Keys.AccessTypes.Unavailable) && (SlotCount.Trim().Length > 0)); } }

        public bool HasSlotsAvailable { get { return (AvailabilityCount > 0); } }

        public bool IsAMSlot { get { return (StartTime.ToString("tt") == "AM"); } }

        public bool IsPMSlot { get { return (StartTime.ToString("tt") == "PM"); } }

        public string DayOfWeekCode { get { return Keys.Methods.DayOfWeekCode(StartTime.DayOfWeek); } }
    }
}
