﻿using System;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public class Clinic
    {
        protected readonly IDataAccessService DataAccessService;
        private string _code = string.Empty;
        private string _inactivatedDate = string.Empty;
        private string _reactivatedDate = string.Empty;
        private SchdResource _parentResource;
        private bool _hospLocValuesHaveBeenRead;
        private bool _parentResourceHasBeenRead;

        public Clinic(IDataAccessService dataAccessService)
        {
            DataAccessService = dataAccessService;
        }

        public string ClinicIen { get; set; }
        public string ClinicName { get; set; }

        public string HospitalLocationId
        {
            get { return ClinicIen; }
            set { ClinicIen = value; }
        }

        public string HospitalLocation
        {
            get { return ClinicName; }
            set { ClinicName = value; }
        }
        public string CreateVisit { get; set; }
        public string VisitServiceCategory { get; set; }
        public bool MultipleClinicCodes { get; set; }
        public bool VisitProviderRequired { get; set; }
        public bool GeneratePccPlusForms { get; set; }
        public bool IsSelected { get; set; }
        public bool IsCompleteRecord { get; set; } = true;

        public int MaxOverBooks
        {
            get { return Keys.Methods.IsNumeric( MaxOverBooksRaw ) ? Convert.ToInt32( MaxOverBooksRaw ) : 0; }
        }

        public string MaxOverBooksRaw { get; set; }
        public string DefaultAppointmentTypeId { get; set; }
        public string DefaultAppointmentTypeName { get; set; }

        public int AppointmentLength
        {
            get { return Duration; }
            set { Duration = value; }
        }
        public bool VariableAppointmentLength { get; set; }
        public string InactivatedDate
        {
            get
            {
                if (string.IsNullOrEmpty(_inactivatedDate))
                    GetHospLocValues();

                return _inactivatedDate;
            }
            set { _inactivatedDate = value; }
        }
        public string ReactivatedDate
        {
            get
            {
                if (string.IsNullOrEmpty(_reactivatedDate))
                    GetHospLocValues();

                return _reactivatedDate;
            }
            set { _reactivatedDate = value; }
        }
        public string SpecialInstructions { get; set; }
        public int Duration { get; set; }

        public bool ClinicCanBookOnHolidays
        {
            get { return HOLIDAYS == "Y";  }
        }

        public string SLOTS { get; set; }
        public string PROVUSERPRESENT_BOOL { get; set; }
        public string PROTECTED { get; set; }
        public string HOUR_DISPLAY_BEGIN { get; set; }
        public int DISPLAY_INCREMENTS { get; set; }
        public string HOLIDAYS { get; set; }
        public string CLINIC_STOP { get; set; }
        public string ABBR { get; set; }
        public bool MoreRecords { get; set; } = false;

        public string CODE
        {
            get
            {
                if (string.IsNullOrEmpty(_code))
                    GetHospLocValues();

                return _code;
            }
            set { _code = value; }
        }

        public SchdResource ParentResource
        {
            get
            {
                if (_parentResource == null && DataAccessService != null && !_parentResourceHasBeenRead)
                {
                    if (!string.IsNullOrEmpty(HospitalLocation))
                    {
                        // Look up Resource based on Location name
                        var schdResList = DataAccessService.GetSdecHResources(HospitalLocation);

                        if (schdResList != null && schdResList.Count > 0)
                            _parentResource = schdResList[0];
                    }
                    _parentResourceHasBeenRead = true;
                }
                return _parentResource;
            }
            set { _parentResource = value; }
        }

        private void GetHospLocValues()
        {
            if (!_hospLocValuesHaveBeenRead && (ParentResource != null))
            {
                if (string.IsNullOrEmpty(_code))
                    _code = ParentResource.StopCodeNumber;

                if (string.IsNullOrEmpty(_inactivatedDate))
                    _inactivatedDate = ParentResource.InactiveDate;

                if (string.IsNullOrEmpty(_reactivatedDate))
                    _reactivatedDate = ParentResource.ReactivatedDate;

                _hospLocValuesHaveBeenRead = true;
            }
        }
    }
}
