﻿using System;
using System.Diagnostics;
using System.Windows.Input;

namespace ClinSchd.Infrastructure.Models
{
    /// <summary>
    /// Allows a view model to implement the interface indirectly so 
    /// the command can be exposed as a property.
    /// </summary>
    public class CommandRelay : ICommand
    {
        private readonly Action<object> _executeMethod;
        private readonly Predicate<object> _canExecuteMethod;

        public CommandRelay( Action<object> executeMethod, Predicate<object> canExecuteMethod )
        {
            _executeMethod = executeMethod;
            _canExecuteMethod = canExecuteMethod;
        }

        #region ICommand Members

        [DebuggerStepThrough]
        public bool CanExecute( object parameter )
        {
            if ( _canExecuteMethod != null )
                return _canExecuteMethod( parameter );

            return true;
        }

        public event EventHandler CanExecuteChanged
        {
            add
            {
                CommandManager.RequerySuggested += value;
            }
            remove
            {
                CommandManager.RequerySuggested -= value;
            }
        }

        public void Execute( object parameter )
        {
            _executeMethod( parameter );
        }

        #endregion

    }
}
