﻿using System;
using System.Windows.Media;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ClinSchd.Infrastructure.Models
{
    public class CustomSlot : Slot
    {
        #region private fields

        private SchdAvailability _schdAvailability = null;
        private int _slots;
        private string _slotCount;

        #endregion private fields

        #region constructor

        public CustomSlot() : base() { }

        public CustomSlot(DateTime start, DateTime end) : base(start, end) { }

        public CustomSlot(DateTime start, DateTime end, Category slotCategory ) : this(start, end) { this.SlotCategory = slotCategory; }

        public CustomSlot(SchdAvailability schdAvailability) : this() { SchdAvailability = schdAvailability; }

        public CustomSlot(SchdAvailability schdAvailability, Category category) : this(schdAvailability) { SlotCategory = category; }

        #endregion constructor

        #region public members

        public SchdAvailability SchdAvailability
        {
            get { return _schdAvailability; }
            set
            {
                _schdAvailability = value;

                if (value.START_TIME.HasValue)
                    base.Start = value.START_TIME.Value;

                if (value.END_TIME.HasValue)
                    base.End = value.END_TIME.Value;

                UniqueID = value.RESOURCENAME + "^" + value.APPOINTMENTID;

                CustomSlot.UpdateResources(this, _schdAvailability.APPOINTMENTID, "SlotID");
                OnPropertyChanged("SchdAvailability");
            }
        }

        public Category SlotCategory
        {
            get
            {
                Resource resource = CustomSlot.FindResourceByResourceType(this, "Category");

                if (resource == null)
                    return null;

                return Keys.ColorCodes.SlotCategories[resource.ResourceName];
            }
            set
            {
                CustomSlot.UpdateResources(this, value.CategoryName, "Category");

                //IsReadOnly = Keys.ColorCodes.NonMenuedSlotCategories.Contains(value.CategoryName);
            }
        }

        public string SlotCount
        {
            get
            {
                if (_schdAvailability == null)
                    return _slotCount;

                return _schdAvailability.SlotCount;
            }
            set
            {
                if (_schdAvailability == null)
                    _slotCount = value;
                else
                    _schdAvailability.SlotCount = value;
            }
        }

        public System.Windows.VerticalAlignment SlotCountVerticalAlignment { get; set; } = System.Windows.VerticalAlignment.Top;

        public bool HasAvailability {  get { return !String.IsNullOrEmpty(SlotCount); } }

        public Brush SlotCountBackground {  get { return Keys.ColorCodes.AvailableSlotsAreaColor;  } } 

        public Brush AvailableSlotBorderColor { get { return Keys.ColorCodes.AvailableSlotBorderColor; } }

        public string UniqueID { get; set; }

        public int SLOTS
        {
            get
            {
                if (_schdAvailability == null)
                    return _slots;

                return _schdAvailability.SLOTS;
            }
            set
            {
                if (_schdAvailability == null)
                    _slots = value;
                else
                    _schdAvailability.SLOTS = value;
            }
        }

        #endregion public members

        #region overidden members

        public override Slot Copy()
        {
            Slot slot = new CustomSlot(this.Start, this.End);
            slot.CopyFrom(this);
            return slot;
        }

        public override void CopyFrom(Slot other)
        {
            var otherSlot = other as CustomSlot;

            if (otherSlot != null)
            {
                this.SchdAvailability = otherSlot.SchdAvailability;
                this.SlotCategory = otherSlot.SlotCategory;
                this.SLOTS = otherSlot.SLOTS;
                this.SlotCount = otherSlot.SlotCount;
                base.CopyFrom(otherSlot);
            }
        }

        public new DateTime End
        {
            get { return base.End; }
            set
            {
                if (_schdAvailability != null)
                    _schdAvailability.END_TIME = value;

                base.End = value;
            }
        }

        public new DateTime Start
        {
            get { return base.Start; }
            set
            {
                if (_schdAvailability != null)
                    _schdAvailability.START_TIME = value;

                base.Start = value;
            }
        }

        #endregion overidden members

        #region static members

        public static Resource FindResourceByResourceType(Slot slot, string resourceType)
        {
            foreach (Resource r in slot.Resources)
                if (r.ResourceType == resourceType)
                    return r;

            return null;
        }

        public static void UpdateResources(Slot slot, string resourceValue, string resourceType)
        {
            Resource resource = CustomSlot.FindResourceByResourceType(slot, resourceType);

            if (resource == null)
                slot.Resources.Add(new Resource(resourceValue, resourceType));
            else
                resource.ResourceName = resourceValue;
        }

        #endregion static members
    }
}
