﻿using System.Collections.Generic;

namespace ClinSchd.Infrastructure.Models
{
    public class MonitoredValuesBaseClass
    {
        #region private classes

        private class MonitoredValue
        {
            public MonitoredValue(string valueName, string value)
            {
                ValueName = valueName;
                InitialValue = value;
                Value = value;
            }
            public string ValueName { get; set; }
            public string InitialValue { get; set; }
            public string Value { get; set; }

            public bool HasChanged
            {
                get { return InitialValue != Value; }
            }

            public bool IsDeleted
            {
                get { return HasChanged && !string.IsNullOrEmpty(InitialValue) && string.IsNullOrEmpty(Value); }
            }

            public string RPCValue
            {
                get { return IsDeleted ? "@" : Value; }
            }
        }

        #endregion private classes

        #region private fields

        private readonly Dictionary<string, MonitoredValue> _monitoredValues = new Dictionary<string, MonitoredValue>();

        #endregion private fields

        #region protected members

        protected string GetMonitoredValue(string valueName)
        {
            if (_monitoredValues.ContainsKey(valueName))
                return _monitoredValues[valueName].Value;
            return string.Empty;
        }

        protected void SetMonitoredValue(string valueName, string value)
        {
            if (_monitoredValues.ContainsKey(valueName))
                _monitoredValues[valueName].Value = value;
            else
                _monitoredValues.Add(valueName, new MonitoredValue(valueName, value));
        }

        #endregion protected members

        #region  public members

        public string GetRPCValue(string propertyName)
        {
            if (_monitoredValues.ContainsKey(propertyName))
            {
                var mv = _monitoredValues[propertyName];

                return mv.IsDeleted ? "@" : mv.Value;
            }
            return string.Empty;
        }

        public bool ValueHasChanged(string propertyName)
        {
            if (_monitoredValues.ContainsKey(propertyName))
                return _monitoredValues[propertyName].HasChanged;

            return false;
        }

        #endregion  public members
    }
}
