﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Models
{
    public class PatientContact
    {
        #region HPE ORIGINAL CODE (COMMENTED OUT)

        /*
         1. DATE ENTERED external date/time
         2. PC ENTERED BY USER IEN Pointer to NEW PERSON file
         3. PC ENTERED BY USER NAME NAME from NEW PERSION file
         4. ACTION C=Called; M=Message Left; L=LETTER
         5. PATIENT PHONE Free-Text 4-20 characters
       */

        //
        //THESE LINES WERE NOT COMMENTED OUT
        //
        //public string DateEntered { get; set; }
        //public string PCEnteredByUserIen { get; set; }
        //public string PCEnteredByUserName { get; set; }
        //public string Action { get; set; }
        //public string PatientPhone { get; set; }


        /*
        public string PatientName { get; set; }
        public bool PatientContacted1 { get; set; }
        public DateTime PatientContact1DateTime { get; set; }
        public bool PatientContacted2 { get; set; }
        public DateTime PatientContact2DateTime { get; set; }
        public bool PatientContacted3 { get; set; }
        public DateTime PatientContact3DateTime { get; set; }
        public bool PatientLetterSent { get; set; }
        public DateTime PatientLetterSentDateTime { get; set; }
        public string AppointmentRequestIen { get; set; }
        */

        #endregion
    
        public int PatientIen { get; set; }
        public int ClinicIen { get; set; }
        public DateTime CidPreferredDate { get; set; }
        public string RequestType { get; set; }
        public int MainSequence { get; set; }
        public int PatientContactFileIen { get; internal set; }
        public bool IsNonMentalHealthContactsSatisfied { get; set; }
        public bool IsMentalHealthContactsSatisfied { get; set; }


        private List<ContactAttempt> contactAttempts;
        public List<ContactAttempt> ContactAttempts
        {
            get { return contactAttempts; }
            set
            {
                if (value == contactAttempts)
                    return;

                contactAttempts = value;
                if (value == null)
                    return; 
                determineMentalHealthContactsSatisfiedStatus();
                determineNonMentalHealthContactsSatisfiedStatus();
            }
        }


        public List<string> ValidationErrors { get; set; }

        #region HELPERS
        private void determineMentalHealthContactsSatisfiedStatus()
        {
            if (countValidContactCallAttempts() >= 3 && hasValidContactLetter())
                IsMentalHealthContactsSatisfied = true;
        }
        private void determineNonMentalHealthContactsSatisfiedStatus()
        {
            if (getCallAttemptsWithValidSequenceNumbers().Count() >= 1 && hasValidContactLetter())
                IsNonMentalHealthContactsSatisfied = true;
        }
        private bool hasValidContactLetter()
        {
            foreach (var contactAttempt in ContactAttempts)
                if (contactAttempt.ContactAttemptType == ContactAttemptType.Letter 
                    && contactAttempt.SequenceNumber >= MainSequence 
                    && contactAttempt.ContactAttemptDateTime < DateTime.Now.AddDays(-14))
                    return true;

            return false;
        }
        private int countValidContactCallAttempts()
        {
            return countCallAttemptsWithValidDates(getCallAttemptsWithValidSequenceNumbers());
        }
        private int countCallAttemptsWithValidDates(IEnumerable<ContactAttempt> callAttemptsWithValidSequenceNumbers)
        {
            if (!callAttemptsWithValidSequenceNumbers.Any())
                return 0;

            callAttemptsWithValidSequenceNumbers = callAttemptsWithValidSequenceNumbers.OrderBy(c => c.ContactAttemptDateTime);

            var lastValidDate = callAttemptsWithValidSequenceNumbers.First().ContactAttemptDateTime.AddDays(-1);

            int validContactCallAttempts = 0;

            foreach (var contactAttempt in callAttemptsWithValidSequenceNumbers)
                if (isValidContactDate(contactAttempt, lastValidDate))
                {
                    validContactCallAttempts++;
                    lastValidDate = contactAttempt.ContactAttemptDateTime;
                    { }
                }

            return validContactCallAttempts;
        }
        private bool isValidContactDate(ContactAttempt contactAttempt, DateTime lastValidDate)
        {
            return contactAttempt.ContactAttemptDateTime.Date > lastValidDate.Date;
        }
        private IEnumerable<ContactAttempt> getCallAttemptsWithValidSequenceNumbers()
        {
            return ContactAttempts.Where(c => c.SequenceNumber >= MainSequence && c.ContactAttemptType == ContactAttemptType.Call);
        }
        #endregion
    }

    public class ContactAttempt
    {
        public DateTime ContactAttemptDateTime { get; set; }
        public int SequenceNumber { get; set; }
        public ContactAttemptType ContactAttemptType { get; set; }
        public int EnteredByUserIen { get; set; }
        public string EnteredByUserName { get; set; }
        public DateTime EnteredDateTime { get; set; }
        public string Comments { get; set; }
        public string LeftMessage { get; set; }
        public List<string> ValidationErrors { get; set; }
        public bool ValidForCurrentRequest { get; internal set; }
    }

    public enum ContactAttemptType
    {
       NA,
       Call,
       Letter
    }

    
}
