﻿using System;
using System.Collections.Generic;
using System.Data;

namespace ClinSchd.Infrastructure.Models
{
    public class PatientDisabilitiesModel
    {
        public static IList<NameValue> ExtremetyList = new List<NameValue>
        {
            new NameValue("BOTH LOWER", "BL"),
            new NameValue("BOTH UPPER", "BU"),
            new NameValue("RIGHT LOWER", "RL"),
            new NameValue("RIGHT UPPER", "RU"),
            new NameValue("LEFT LOWER", "LL"),
            new NameValue("LEFT UPPER", "LU")
        };

        public IList<NameValue> ExtremitiesList
        {
            get { return ExtremetyList; }
        }

        public string PatientIEN { get; private set; }

        public IList<PatientDisabilityModel> Disabilities { get; } = new List<PatientDisabilityModel>();

        public bool HasDisabilities
        {
            get
            {
                return Disabilities.Count > 0
                       ||
                       Secondaries.Count > 0
                       ||
                       !string.IsNullOrEmpty(ServiceConnectedPercentage)
                       ||
                       !string.IsNullOrEmpty(PrimaryName)
                       ||
                       !string.IsNullOrEmpty(PrimaryIEN);
            }
        }

        public bool ServiceConnected { get; private set; }

        public string ServiceConnectedPercentage { get; }

        public string PrimaryIEN { get; }

        public string PrimaryName { get; }

        public IList<NameValue> Secondaries { get; } = new List<NameValue>();

        public PatientDisabilitiesModel(DataRow row)
        {
            PatientIEN = row["DFN"].ToString();

            string disab = row["DISABILITIES"].ToString();

            if (!string.IsNullOrEmpty(disab))
            {
                string[] disabParts = disab.Split('|');

                foreach(var item in disabParts)
                    if (!string.IsNullOrEmpty(item))
                        Disabilities.Add(new PatientDisabilityModel(item));
            }

            ServiceConnected = row["SVCCONN"].ToString().ToUpper() == "Y";

            string p = row["SVCCONNP"].ToString();
            ServiceConnectedPercentage = string.IsNullOrEmpty(p) ? string.Empty : string.Format("{0}%", p);

            PrimaryIEN = row["PRIMARY"].ToString();

            PrimaryName = row["PRIMARY_NAME"].ToString();

            string seconds = row["SECONDARY"].ToString();

            if (!string.IsNullOrEmpty(seconds))
            {
                string[] separator = { ";;" };
                string[] secondsList = seconds.Split('|');

                foreach (var second in secondsList)
                    if (!string.IsNullOrEmpty(second))
                    {
                        string[] secondParts = second.Split(separator, 2, StringSplitOptions.None);

                        if (secondParts[0] != PrimaryIEN)
                            Secondaries.Add(new NameValue(secondParts[1], secondParts[0]));
                    }
            }
        }
    }
}
