﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Infrastructure.Models
{
    public enum PatientFlagType { UNKNOWN, National, Local, FugitiveFelon, CatastrophicDisability }
    public class PatientFlag
    {
        #region private fields

        private IList<string> _flagDetails = null;

        #endregion private fields

        #region constructors

        public PatientFlag() { }

        public PatientFlag(PatientFlagType flagType, string source)
        {
            Initialize(flagType, source);
        }

        #endregion constructors

        #region public members

        public void Initialize(PatientFlagType flagType, string source)
        {
            FlagType = flagType;
            string[] separator = { ";;" };
            string[] parts = source.Split(separator, 5, System.StringSplitOptions.None);
            AssignmentID = parts[0];
            AssignmentStatus = ((parts[1] == "1") || (parts[1].ToUpper() == "YES"));
            FlagID = parts[2];
            FlagName = parts[3];
            FlagStatus = ((parts[4] == "1") || (parts[4].ToUpper() == "YES"));
        }

        public PatientFlagType FlagType { get; set; }

        public string FlagTypeName
        {
            get
            {
                switch (FlagType)
                {
                    case PatientFlagType.CatastrophicDisability:
                        return "DISABILITY";
                    case PatientFlagType.FugitiveFelon:
                        return "FUGITIVE FELON";
                    case PatientFlagType.Local:
                        return "II (LOCAL)";
                    case PatientFlagType.National:
                        return "I (NATIONAL)";
                    default:
                        return "UNKNOWN";
                }
            }
        }

        public string AssignmentID { get; set; }

        public bool AssignmentStatus { get; set; }

        public string FlagID { get; set; }

        public string FlagName { get; set; }

        public bool FlagStatus { get; set; }

        public IList<string> FlagDetails
        {
            get
            {
                if (_flagDetails == null)
                {
                    _flagDetails = new List<string>()
                    {
                        String.Format("Flag Name:               {0}", FlagName),
                        String.Empty,
                        "Assignment Narrative:   ",
                        String.Empty,
                        String.Empty,
                        "Flag Type:               UNKNOWN",
                        String.Format("Flag Category:           {0}", FlagTypeName),
                        String.Format("Assignment Status:       {0}", (AssignmentStatus ? "Active" : "Inactive")),
                        "Initial Assigned Date:   UNKNOWN",
                        "Approved by:             UNKNOWN",
                        "Next Review Date:        UNKNOWN",
                        "Owner Site:              UNKNOWN",
                        "Originating Site:        UNKNOWN"
                    };
                }

                return _flagDetails;
            }
        }

        public void SetFlagType(string source)
        {
            switch (source)
            {
                case "0":
                    FlagType = PatientFlagType.Local;
                    break;
                case "1":
                    FlagType = PatientFlagType.National;
                    break;
                default:
                    FlagType = PatientFlagType.UNKNOWN;
                    break;
            }
        }

        #endregion public members
    }
}
