﻿using System;
using System.ComponentModel.DataAnnotations;
using System.Globalization;
using System.Text.RegularExpressions;
using ClinSchd.Infrastructure.Converters;

namespace ClinSchd.Infrastructure.Models
{
    public class PatientInformation
    {
        public PatientInformation()
        {
            GAF = new PatientGAF();
        }
        public string IEN { get; set; }
        public string Name { get; set; }
        public string HRN { get; set; }
        public bool IsSensitiveRecord { get; set; }

        private string _ssn;
        public string SSN
        {
            get { return SSNFormatted; }
            set { _ssn = value; }
        }

        public string SSNRaw
        {
            get { return _ssn; }
        }

        public string SSNFormatted { get { return Patient.FormattedSSN(_ssn, IEN, IsSensitiveRecord); } }

        public string SSN4
        {
            get
            {
                if (!string.IsNullOrEmpty(_ssn) && _ssn.Length > 8)
                    return _ssn.Substring(5, 4);

                return string.Empty;
            }
        }

        public DateTime? DOBDate { get { return Patient.DOBToDateTime(_dob); } }

        public string DOBFormatted { get { return Patient.FormattedDOB(_dob, IEN, IsSensitiveRecord); } }

        public string DOBRaw { get { return _dob; } }

        private string _dob;
        [Required(ErrorMessage = @"Please provide date of birth")]
        public string DOB
        {
            get { return DOBFormatted; }
            set { _dob = value; }
        }

        public string Age { get { return Patient.CalculateAge(_dob); } }

        public string Street { get; set; }
        public string City { get; set; }
        public string State { get; set; }
        public string Zip { get; set; }
        public string OfficePhone { get; set; }
        public string HomePhone { get; set; }
        public string ServiceConnection { get; set; } //YES/NO
        public string ServiceConnectionPercentage { get; set; }
        public PatientGAF GAF { get; set; } //Global Assessment of Functioning...
    }
}
