﻿using System.ComponentModel;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    /// <summary>
    /// RPMS Clinic information
    /// </summary>
    public class RPMSClinic : Clinic, INotifyPropertyChanged
    {
        private string _provider;

        public RPMSClinic(IDataAccessService dataAccessService) : base (dataAccessService) { }

        public string PROVIDER
        {
            get
            {
                if (_provider == null)
                {
                    var providers = DataAccessService.GetProvidersByClinic(ClinicIen);

                    foreach (Provider p in providers)
                        if (p.IsDefaultProvider)
                        {
                            _provider = p.Name;
                            break;
                        }
                }

                return _provider;
            }
            set
            {
                if (Equals(value, _provider)) return;
                _provider = value;
                OnPropertyChanged("PROVIDER");
            }
        }

        public string HOSPITAL_LOCATION
        {
            get { return ClinicName;  }
            set { ClinicName = value; }
        }
        public string HOSPITAL_LOCATION_ID
        {
            get { return ClinicIen; }
            set { ClinicIen = value; }
        }
        public string OVERBOOK
        {
            get { return MaxOverBooksRaw; }
            set { MaxOverBooksRaw = value; }
        }
        public string INACTIVE
        {
            get { return InactivatedDate; }
            set { InactivatedDate = value; }
        }
        public string REACTIVE
        {
            get { return ReactivatedDate; }
            set { ReactivatedDate = value; }
        }
        public string CREATEVISIT
        {
            get { return CreateVisit; }
            set { CreateVisit = value; }
        }
        public string VISITSERVICE
        {
            get { return VisitServiceCategory; }
            set { VisitServiceCategory = value; }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
