﻿using System.ComponentModel;
using System.Diagnostics;

namespace ClinSchd.Infrastructure.Models
{
    public class Race : INotifyPropertyChanged
    {
        public string Ien { get; set; } = "";
        private string _raceName;
        private bool _isSelected = false;
        public string Name
        {
            get { return _raceName; }
            set
            {
                if ( value == null )
                    return;

                if( value.Contains("|"))
                {
                    var multRace = value.Split('|');
                    _raceName = multRace[0];
                }
                else  
                    _raceName = value;
            }
        }
        public string Abbreviation { get; set; }
        // IsSelected prooperty for use in list select options
        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                _isSelected = value;
                NotifyPropertyChanged("IsSelected");
            }
        }
        public bool LastIsSelected { get; set; } = false;
        public bool IsDefault { get; set; } = false;
        public void Reset()
        {
            IsSelected = false;
            LastIsSelected = false;
            IsDefault = false;
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        public void NotifyPropertyChanged(string propertyName)
        {
            VerifyPropertyName(propertyName);

            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// In debug mode, notifies properties not listed
        /// by looking at the class in reflection.
        /// </summary>
        [Conditional("DEBUG")]
        [DebuggerStepThrough]
        private void VerifyPropertyName(string propertyName)
        {
            Debug.Assert(
                TypeDescriptor.GetProperties(this)[propertyName] != null,
                string.Format("Invalid property name: {0}", propertyName));
        }

        #endregion INotifyPropertyChanged Members

    }
}
