﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using Microsoft.Practices.Unity;

namespace ClinSchd.Infrastructure.Models
{
    public enum AppointmentStatuses
    {
        New,
        WalkIn_NotCheckedIn,
        WalkIn_CheckdIn,
        NoShow,
        CheckedIn,
        CheckedOut,
        Cancelled,
        Unknown
    }

	public class SchdAppointment
	{
        public bool IsNewAppointment { get; set; } = false;
        public bool IsDead { get; set; }
        public bool IsFirstApptBookedInMrtcRequest { get; set; }
        public AppointmentRequest AppointmentRequest { get; set; }
        public DateTime? DesiredAppointmentDate { get; set; }
		public string APPOINTMENTID { get; set; }
		public string START_TIME { get; set; }
		public string END_TIME { get; set; }
		public string CHECKIN { get; set; }
		public string AUXTIME { get; set; }		
		public string RESOURCENAME { get; set; }
        public string ResourceId { get; set; }
        public string ResourceScheduleID { get; set; }
		public string NOSHOW { get; set; }
		public string HRN { get; set; }
		public string ACCESSTYPEID { get; set; }
		public string WALKIN { get; set; }
		public string CHECKOUT { get; set; }
		public string VPROVIDER { get; set; }
		public string CANCELLED { get; set; }
		public string NOTE { get; set; }
		public string Command { get; set; }
        public string MRTCParentRequestIEN { get; set; }
        public string APPTREQTYPE_REQTYPE { get; set; }
        public string APPTREQTYPE_REQIEN { get; set; }
        public string APPTTYPE_IEN { get; set; }
        public string APPTTYPE_NAME { get; set; }

        public string PATIENTNAME { get; set; }
		
		private string patientId;
		public string PATIENTID
		{
			get
			{
				return patientId;
			}
			set
			{
				this.patientId = value;
				//this.Patient = PatientFactory.Create (); // MSC/DKA 2014-08-28 This was causing a null reference.
                if (this.Patient != null)
                {
                    this.Patient.IEN = value;
                    this.Patient.LoadPatientInfoByIEN();
                }
			}
		}

		public Patient Patient { get; set; }

        public AppointmentStatuses AppointmentStatus
        {
            get
            {
                if (NOSHOW == "1")
                    return AppointmentStatuses.NoShow;
                else if (CANCELLED == "1")
                    return AppointmentStatuses.Cancelled;
                else if (!String.IsNullOrEmpty(CHECKOUT))
                    return AppointmentStatuses.CheckedOut;
                else if (WALKIN == "1")
                {
                    if (String.IsNullOrEmpty(CHECKIN))
                        return AppointmentStatuses.WalkIn_NotCheckedIn;
                    else
                        return AppointmentStatuses.WalkIn_CheckdIn;
                }
                else if (!String.IsNullOrEmpty(CHECKIN))
                    return AppointmentStatuses.CheckedIn;
                else
                    return AppointmentStatuses.New;
            }
        }

        public string AppointmentStatusText
        {
            get
            {
                switch(AppointmentStatus)
                {
                    case AppointmentStatuses.New:
                        return "New";
                    case AppointmentStatuses.WalkIn_NotCheckedIn:
                        return "Walk-In Not Checked In";
                    case AppointmentStatuses.WalkIn_CheckdIn:
                        return "Walk-In Checked In";
                    case AppointmentStatuses.NoShow:
                        return "No Show";
                    case AppointmentStatuses.CheckedIn:
                        return "Checked In";
                    case AppointmentStatuses.CheckedOut:
                        return "Checked Out";
                    case AppointmentStatuses.Cancelled:
                        return "Cancelled";
                }

                return String.Empty;
            }
        }

	    public string CategoryColorName
	    {
	        get
	        {
	            string catColor = Keys.Categories.New_Appointment;

	            switch (AppointmentStatus)
	            {
	                case AppointmentStatuses.New:
	                    catColor = Keys.Categories.New_Appointment;
	                    break;
	                case AppointmentStatuses.WalkIn_NotCheckedIn:
	                case AppointmentStatuses.WalkIn_CheckdIn:
	                    catColor = Keys.Categories.WalkIn_Appointment;
	                    break;
	                case AppointmentStatuses.NoShow:
	                    catColor = Keys.Categories.NoShow_Appointment;
	                    break;
	                case AppointmentStatuses.CheckedIn:
	                    catColor = Keys.Categories.CheckedIn_Appointment;
	                    break;
	                case AppointmentStatuses.CheckedOut:
	                    catColor = Keys.Categories.CheckedOut_Appointment;
	                    break;
	                case AppointmentStatuses.Cancelled:
	                    catColor = Keys.Categories.Cancelled_Appointment;
	                    break;
	                default:
	                    catColor = Keys.Categories.New_Appointment;
	                    break;
	            }

	            return catColor;
;	        }
	    }

	    public string CategoryName
	    {
	        get
	        {
	            switch (AppointmentStatus)
	            {
	                case AppointmentStatuses.New:
	                    return Keys.Categories.New_Appointment;
	                case AppointmentStatuses.WalkIn_NotCheckedIn:
	                case AppointmentStatuses.WalkIn_CheckdIn:
	                    return Keys.Categories.WalkIn_Appointment;
	                case AppointmentStatuses.NoShow:
	                    return Keys.Categories.NoShow_Appointment;
	                case AppointmentStatuses.CheckedIn:
	                    return Keys.Categories.CheckedIn_Appointment;
	                case AppointmentStatuses.CheckedOut:
	                    return Keys.Categories.CheckedOut_Appointment;
	                case AppointmentStatuses.Cancelled:
	                    return Keys.Categories.Cancelled_Appointment;
	                default:
	                    return Keys.Categories.New_Appointment;
	            }
	        }
	    }

        public string AppointmentResourceName
        {
            get
            {
                switch (AppointmentStatus)
                {
                    case AppointmentStatuses.New:
                        return "EXIST";
                    case AppointmentStatuses.WalkIn_NotCheckedIn:
                    case AppointmentStatuses.WalkIn_CheckdIn:
                        return "WALKIN";
                    case AppointmentStatuses.NoShow:
                        return "NOSHOW";
                    case AppointmentStatuses.CheckedIn:
                        return "CHECKIN";
                    case AppointmentStatuses.CheckedOut:
                        return "CHECKOUT";
                    case AppointmentStatuses.Cancelled:
                        return "CANCELLED";
                    default:
                        return "EXIST";
                }
            }
        }

        public bool CanEdit { get { return !IsDead; } }

        public bool CanView { get { return true; } }

	    public bool CanCancel
	    {
	        get
	        {
	            return (
	                !IsDead
	                &&
	                (
	                    (AppointmentStatus == AppointmentStatuses.New)
	                    ||
	                    (AppointmentStatus == AppointmentStatuses.NoShow)
	                    ||
	                    (AppointmentStatus == AppointmentStatuses.WalkIn_CheckdIn)
	                    ||
	                    (AppointmentStatus == AppointmentStatuses.WalkIn_NotCheckedIn)
	                    )
	                );
	        }
	    }

        public bool CanUndoCancel { get { return (AppointmentStatus == AppointmentStatuses.Cancelled); } }

	    public bool CanCheckIn
	    {
	        get
	        {
	            return (
                    !IsDead
                &&
	                (
	                    (AppointmentStatus == AppointmentStatuses.New)
	                ||
	                    ((AppointmentStatus == AppointmentStatuses.WalkIn_NotCheckedIn))
	                )
                &&
	                (Convert.ToDateTime(START_TIME).Date <= DateTime.Now.Date)
	            );
	        }
	    }

        public bool CanUndoCheckIn
        {
            get
            {
                return (
                    !IsDead
                &&
                    (
                        (AppointmentStatus == AppointmentStatuses.CheckedIn)
                    ||
                        (AppointmentStatus == AppointmentStatuses.WalkIn_CheckdIn)
                    )
                );
            }
        }

	    public bool CanCheckOut
	    {
	        get
	        {
	            return (
                    !IsDead
                &&
                    (
                        (AppointmentStatus == AppointmentStatuses.CheckedIn)
                    ||
                        (AppointmentStatus == AppointmentStatuses.WalkIn_CheckdIn)
                    )
                );
	        }
	    }

        public bool CanUndoCheckOut { get { return (!IsDead && (AppointmentStatus == AppointmentStatuses.CheckedOut)); } }

        public bool CanMarkAsNoShow
        {
            get
            {
                return (
                    !IsDead
                &&
                    (
                        (AppointmentStatus == AppointmentStatuses.New)
                   &&
                        (Convert.ToDateTime(START_TIME).Date <= DateTime.Now.Date)
                   )
                );
            }
        }

        public bool CanUndoMarkAsNoShow { get { return (!IsDead && (AppointmentStatus == AppointmentStatuses.NoShow)); } }

        public SchdResource SelectedResource { get; set; }

		[Dependency]
		public Factory<Patient> PatientFactory { get; set; }

	    public SchdAppointment Copy(bool isNewAppointment = false)
	    {
	        return new SchdAppointment()
	        {
	            IsDead = this.IsDead,
	            IsFirstApptBookedInMrtcRequest = this.IsFirstApptBookedInMrtcRequest,
	            AppointmentRequest = this.AppointmentRequest,
	            DesiredAppointmentDate = this.DesiredAppointmentDate,
	            APPOINTMENTID = (isNewAppointment ? string.Empty : this.APPOINTMENTID),
	            START_TIME = this.START_TIME,
	            END_TIME = this.END_TIME,
	            CHECKIN = this.CHECKIN,
	            AUXTIME = this.AUXTIME,
	            RESOURCENAME = this.RESOURCENAME,
	            ResourceId = this.ResourceId,
	            ResourceScheduleID = this.ResourceScheduleID,
	            NOSHOW = this.NOSHOW,
	            HRN = this.HRN,
	            ACCESSTYPEID = this.ACCESSTYPEID,
	            WALKIN = this.WALKIN,
	            CHECKOUT = this.CHECKOUT,
	            VPROVIDER = this.VPROVIDER,
	            CANCELLED = this.CANCELLED,
	            NOTE = this.NOTE,
	            Command = this.Command,
	            MRTCParentRequestIEN = this.MRTCParentRequestIEN,
	            APPTREQTYPE_REQTYPE = this.APPTREQTYPE_REQTYPE,
	            APPTREQTYPE_REQIEN = this.APPTREQTYPE_REQIEN,
	            PATIENTNAME = this.PATIENTNAME,
	            PATIENTID = this.PATIENTID,
	            Patient = this.Patient,
	            SelectedResource = this.SelectedResource,
	            PatientFactory = this.PatientFactory
	        };
	    }
        public string ClinicIen { get; set; }
    }
}
