﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.Unity;

namespace ClinSchd.Infrastructure.Models
{
	[Serializable]
	public class SchdAvailability
	{
		public string APPOINTMENTID { get; set; }
        public DateTime? START_TIME { get; set; }
        public string StartTime
        {
            get
            {
                if (START_TIME.HasValue)
                    return START_TIME.ToString();

                return String.Empty;
            }
            set
            {
                if (String.IsNullOrEmpty(value))
                {
                    START_TIME = null;
                    return;
                }

                DateTime dateTime;

                if (Keys.Methods.TryParseDateTime(value, out dateTime))
                    START_TIME = dateTime;
                else
                    throw new Exception(String.Format("'{0}' is an invalid Date\\Time string.", value));
            }
        }
        public DateTime? END_TIME { get; set; }
		public string EndTime
        {
            get
            {
                if (END_TIME.HasValue)
                    return END_TIME.ToString();

                return String.Empty;
            }
            set
            {
                if (String.IsNullOrEmpty(value))
                {
                    END_TIME = null;
                    return;
                }

                DateTime dateTime;

                if (Keys.Methods.TryParseDateTime(value, out dateTime))
                    END_TIME = dateTime;
                else
                    throw new Exception(String.Format("'{0}' is an invalid Date\\Time string.", value));
            }
        }
        public SchdResource Resource { get; set; }
        private string _RESOURCENAME = String.Empty;
        public string RESOURCENAME
        {
            get
            {
                if (Resource != null)
                    return Resource.RESOURCE_NAME;

                return _RESOURCENAME;
            }
            set
            {
                _RESOURCENAME = value;
            }
        }
        public string ResourceScheduleID { get; set; }
        public string ACCESSTYPEID { get; set; }
		public string ACCESSTYPENAME { get; set; }
		public string Note { get; set; }
		public string Command { get; set; }
		public bool IsPreventAccess { get; set; }
		public int SLOTS { get; set; }
        public string SlotCount { get; set; }
        public bool HasSlotsAvailable { get { return SLOTS > 0; } }
        public int OverbookedCount { get; set; }
        //public int ScheduleSLOTS { get; set; }
		public int COLUMNS { get; set; }
		public int GridColumn { get; set; }
		public string AccessRuleList { get; set; }
		public string DisplayColor { get; set; }
		public int Red { get; set; }
		public int Green { get; set; }
		public int Blue { get; set; }

		public double? Duration
		{
			get
			{
				try {
					return (DateTime.Parse (EndTime).Subtract (DateTime.Parse (StartTime))).TotalMinutes;
				} catch (Exception ex) {
					if (ex is ArgumentException || ex is ArgumentNullException) {
						return null;
					} else {
						throw;
					}
				}
			}
		}

		#region Methods

		public void Create (string startTime, string endTime, string nAvailabilityType, int nSlots, string sResourceList, string sAccessRuleList)
		{
			StartTime = startTime;
			EndTime = endTime;
			ACCESSTYPEID = nAvailabilityType;
			SLOTS = nSlots;
			RESOURCENAME = sResourceList;
			AccessRuleList = sAccessRuleList;
		}

		public void Create (string startTime, string endTime, string nAvailabilityType, int nSlots, string sResourceList)
		{
			StartTime = startTime;
			EndTime = endTime;
			ACCESSTYPEID = nAvailabilityType;
			SLOTS = nSlots;
			RESOURCENAME = sResourceList;
			AccessRuleList = "";
		}

        public void Create(DateTime startTime, DateTime endTime, string nAvailabilityType, int nSlots, string slotCount, SchdResource resource)
        {
            START_TIME = startTime;
            END_TIME = endTime;
            ACCESSTYPEID = nAvailabilityType;
            SLOTS = nSlots;
            SlotCount = slotCount;
            Resource = resource;
            AccessRuleList = "";
        }

        public void Create (string startTime, string endTime, string nAvailabilityType, int nSlots)
		{
			StartTime = startTime;
			EndTime = endTime;
			ACCESSTYPEID = nAvailabilityType;
			SLOTS = nSlots;
			RESOURCENAME = "";
			AccessRuleList = "";
		}

		public void Create (string startTime, string endTime, int nSlots)
		{
			StartTime = startTime;
			EndTime = endTime;
			ACCESSTYPEID = "0";
			SLOTS = nSlots;
			RESOURCENAME = "";
			AccessRuleList = "";
		}

        public List<CSlot> GetTimeSlots(TimeSpan duration)
        {
            List<CSlot> results = new List<CSlot>();

            if (duration.TotalMinutes > 0)
            {
                DateTime startTime = Convert.ToDateTime(StartTime);
                DateTime endTime = Convert.ToDateTime(EndTime);

                for (DateTime slotStart = startTime; slotStart < endTime; slotStart = slotStart.AddMinutes(duration.TotalMinutes))
                {
                    CSlot slot = new CSlot { Date = slotStart.Date, StartTime = slotStart, EndTime = slotStart.AddMinutes(duration.TotalMinutes) };
                    results.Add(slot);
                }
            }

            return results;
        }

        public bool ContainsTimeSlot(DateTime startTime, DateTime endTime)
        {
            return (GetTimeSlots(endTime - startTime).FirstOrDefault(bs => (bs.IsInSlot(startTime, endTime))) != null);
        }

		#endregion //Methods

	}
}
