﻿using System;
using System.Collections.Generic;
using ClinSchd.Infrastructure.Interfaces;

namespace ClinSchd.Infrastructure.Models
{
    public class SchdResourceGroup : AsyncValidatableModel
    {
        private readonly IDataAccessService _dataAccessService;
        private IList<SchdResource> _resources;
        private string _name;

        public string Name
        {
            get { return _name; }
            set
            {
                _name = value;
                NotifyPropertyChanged( "Name" );
            }
        }

        public string IEN { get; set; }
        public Guid LoadingID { get; set; }
        public IList<SchdResource> Resources
        {
            get
            {
                if (_resources == null)
                {
                    if (string.IsNullOrEmpty(IEN))
                        _resources = new List<SchdResource>();
                    else
                        _resources = _dataAccessService.GetSchdResourcesByGroupID(IEN);
                }

                return _resources;
            }
            set
            {
                _resources = value;
                NotifyPropertyChanged("Resources");
            }
        }

        public SchdResourceGroup (IDataAccessService dataAccessService)
        {
            _dataAccessService = dataAccessService;
        }

        public void GetAllResourceGroups(WorkCompletedMethod workCompletedMethod)
        {
            DoWorkAsync ((s, args) => {
                args.Result = _dataAccessService.GetResourceGroups();
            }, workCompletedMethod);
        }
    }
}
