﻿using System;
using System.ComponentModel;

namespace ClinSchd.Infrastructure.Models
{
    internal struct Preference
    {
        public bool Monday { get; set; }
        public bool Tuesday { get; set; }
        public bool Wednesday { get; set; }
        public bool Thursday { get; set; }
        public bool Friday { get; set; }
        public bool Saturday { get; set; }
        public bool Am { get; set; }
        public bool Pm { get; set; }
        public bool BothAmPm { get { return !Am && !Pm; } }
        public bool BothChecked { get; set; }
        public bool GenderSpecificProvider { get; set; }
        public bool HearingImpaired { get; set; }
        public bool LanguagePreference { get; set; }
        public bool NeedsEscort { get; set; }
        public bool OnStretcher { get; set; }
        public bool SpecialModeOfTransportation { get; set; }

        public string Remarks { get; set; }
        public string DateAdded { get; set; }
        public string AddedByUserIen { get; set; }
        public string AddedByUserName { get; set; }
        public string DateInactivated { get; set; }
        public string InactivatedByUserIen { get; set; }
        public string InactivatedByUserName { get; set; }
    }

    public class SpecialNeedsPreferences
    {
        private Preference _oldValues;

        public SpecialNeedsPreferences() { Initialize(); }

        private void Initialize()
        {
            Monday                      = false;
            Tuesday                     = false;
            Wednesday                   = false;
            Thursday                    = false;
            Friday                      = false;
            Saturday                    = false;
            Am                          = false;
            Pm                          = false;
            BothChecked                 = true;
            GenderSpecificProvider      = false;
            HearingImpaired             = false;
            LanguagePreference          = false;
            NeedsEscort                 = false;
            OnStretcher                 = false;
            SpecialModeOfTransportation = false;

            Remarks               = string.Empty;
            DateAdded             = string.Empty;
            AddedByUserIen        = string.Empty;
            AddedByUserName       = string.Empty;
            DateInactivated       = string.Empty;
            InactivatedByUserIen  = string.Empty;
            InactivatedByUserName = string.Empty;

            _oldValues.Monday                      = false;
            _oldValues.Tuesday                     = false;
            _oldValues.Wednesday                   = false;
            _oldValues.Thursday                    = false;
            _oldValues.Friday                      = false;
            _oldValues.Saturday                    = false;
            _oldValues.Am                          = false;
            _oldValues.Pm                          = false;
            _oldValues.BothChecked                 = true;
            _oldValues.GenderSpecificProvider      = false;
            _oldValues.HearingImpaired             = false;
            _oldValues.LanguagePreference          = false;
            _oldValues.NeedsEscort                 = false;
            _oldValues.OnStretcher                 = false;
            _oldValues.SpecialModeOfTransportation = false;

            _oldValues.Remarks               = string.Empty;
            _oldValues.DateAdded             = string.Empty;
            _oldValues.AddedByUserIen        = string.Empty;
            _oldValues.AddedByUserName       = string.Empty;
            _oldValues.DateInactivated       = string.Empty;
            _oldValues.InactivatedByUserIen  = string.Empty;
            _oldValues.InactivatedByUserName = string.Empty;
        }

        public void Reset()
        {
            Monday                      = _oldValues.Monday;
            Tuesday                     = _oldValues.Tuesday;
            Wednesday                   = _oldValues.Wednesday;
            Thursday                    = _oldValues.Thursday;
            Friday                      = _oldValues.Friday;
            Saturday                    = _oldValues.Saturday;
            Am                          = _oldValues.Am;
            Pm                          = _oldValues.Pm;
            BothChecked                 = _oldValues.BothChecked;
            GenderSpecificProvider      = _oldValues.GenderSpecificProvider;
            HearingImpaired             = _oldValues.HearingImpaired;
            LanguagePreference          = _oldValues.LanguagePreference;
            NeedsEscort                 = _oldValues.NeedsEscort;
            OnStretcher                 = _oldValues.OnStretcher;
            SpecialModeOfTransportation = _oldValues.SpecialModeOfTransportation;

            Remarks               = _oldValues.Remarks;
            DateAdded             = _oldValues.DateAdded;
            AddedByUserIen        = _oldValues.AddedByUserIen;
            AddedByUserName       = _oldValues.AddedByUserName;
            DateInactivated       = _oldValues.DateInactivated;
            InactivatedByUserIen  = _oldValues.InactivatedByUserIen;
            InactivatedByUserName = _oldValues.InactivatedByUserName;
        }

        public string PatientIEN { get; set; }

        public void SetPreference(string pref, string remarks)
        {
            switch (pref)
            {
                case "MONDAY":
                    Monday = true;
                    _oldValues.Monday = true;
                    break;
                case "TUESDAY":
                    Tuesday = true;
                    _oldValues.Tuesday = true;
                    break;
                case "WEDNESDAY":
                    Wednesday = true;
                    _oldValues.Wednesday = true;
                    break;
                case "THURSDAY":
                    Thursday = true;
                    _oldValues.Thursday = true;
                    break;
                case "FRIDAY":
                    Friday = true;
                    _oldValues.Friday = true;
                    break;
                case "SATURDAY":
                    Saturday = true;
                    _oldValues.Saturday = true;
                    break;
                case "MORNING":
                    Am = true;
                    _oldValues.Am = true;

                    if (Am && Pm)
                    {
                        BothChecked = true;
                        _oldValues.BothChecked = true;
                        Am = false;
                        _oldValues.Am = false;
                        Pm = false;
                        _oldValues.Pm = false;
                    }
                    break;
                case "AFTERNOON":
                    Pm = true;
                    _oldValues.Pm = true;

                    if (Am && Pm)
                    {
                        BothChecked = true;
                        _oldValues.BothChecked = true;
                        Am = false;
                        _oldValues.Am = false;
                        Pm = false;
                        _oldValues.Pm = false;
                    }
                    break;
                case "GENDER SPECIFIC PROVIDER":
                    GenderSpecificProvider = true;
                    _oldValues.GenderSpecificProvider = true;
                    break;
                case "HEARING IMPAIRED":
                    HearingImpaired = true;
                    _oldValues.HearingImpaired = true;
                    break;
                case "LANGUAGE PREFERENCE":
                    LanguagePreference = true;
                    _oldValues.LanguagePreference = true;
                    break;
                case "NEEDS ESCORT":
                    NeedsEscort = true;
                    _oldValues.NeedsEscort = true;
                    break;
                case "ON STRETCHER":
                    OnStretcher = true;
                    _oldValues.OnStretcher = true;
                    break;
                case "SPECIAL MODE OF TRANSPORTATION":                    
                    SpecialModeOfTransportation = true;
                    _oldValues.SpecialModeOfTransportation = true;
                    break;
            }

            if (string.IsNullOrEmpty(Remarks))
            {
                Remarks = remarks;
                _oldValues.Remarks = remarks;
            }
        }

        public string ToPREF_TEXT()
        {
            var spNeedsPrefStr = "";
            //var remarks = TxtSpecialNeedsRemarks.Text;
            // spNeedsPrefStr += "Preference Text|DateTime Added|User Ien|Date Time inactivates|Inactivated User IEN|remarks( obsolete )|active flag^";
            if (_oldValues.GenderSpecificProvider && !GenderSpecificProvider)
                spNeedsPrefStr += "GENDER SPECIFIC PROVIDER|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (GenderSpecificProvider)
                spNeedsPrefStr += "GENDER SPECIFIC PROVIDER|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.HearingImpaired && !HearingImpaired)
                spNeedsPrefStr += "HEARING IMPAIRED|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (HearingImpaired)
                spNeedsPrefStr += "HEARING IMPAIRED|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.LanguagePreference && !LanguagePreference)
                spNeedsPrefStr += "LANGUAGE PREFERENCE|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (LanguagePreference)
                spNeedsPrefStr += "LANGUAGE PREFERENCE|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.NeedsEscort && !NeedsEscort)
                spNeedsPrefStr += "NEEDS ESCORT|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (NeedsEscort)
                spNeedsPrefStr += "NEEDS ESCORT|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.OnStretcher && !OnStretcher)
                spNeedsPrefStr += "ON STRETCHER|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (OnStretcher)
                spNeedsPrefStr += "ON STRETCHER|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.SpecialModeOfTransportation && !SpecialModeOfTransportation)
                spNeedsPrefStr += "SPECIAL MODE OF TRANSPORTATION|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (SpecialModeOfTransportation)
                spNeedsPrefStr += "SPECIAL MODE OF TRANSPORTATION|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if ((_oldValues.Am || _oldValues.BothChecked) && !Am && !BothChecked)
                spNeedsPrefStr += "MORNING|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Am || BothChecked)
                spNeedsPrefStr += "MORNING|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if ((_oldValues.Pm || _oldValues.BothChecked) && !Pm && !BothChecked)
                spNeedsPrefStr += "AFTERNOON|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Pm || BothChecked)
                spNeedsPrefStr += "AFTERNOON|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.Monday && !Monday)
                spNeedsPrefStr += "MONDAY|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Monday)
                spNeedsPrefStr += "MONDAY|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.Tuesday && !Tuesday)
                spNeedsPrefStr += "TUESDAY|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Tuesday)
                spNeedsPrefStr += "TUESDAY|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.Wednesday && !Wednesday)
                spNeedsPrefStr += "WEDNESDAY|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Wednesday)
                spNeedsPrefStr += "WEDNESDAY|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.Thursday && !Thursday)
                spNeedsPrefStr += "THURSDAY|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Thursday)
                spNeedsPrefStr += "THURSDAY|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.Friday && !Friday)
                spNeedsPrefStr += "FRIDAY|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Friday)
                spNeedsPrefStr += "FRIDAY|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            if (_oldValues.Saturday && !Saturday)
                spNeedsPrefStr += "SATURDAY|||" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||0^";
            else if (Saturday)
                spNeedsPrefStr += "SATURDAY|" + DateTime.Now.ToShortDateString() + "|" + Keys.LoggedInUserIen + "||||1^";

            return spNeedsPrefStr;
        }

        public bool HasChanged()
        {
            return _oldValues.Monday != Monday
                   ||
                   _oldValues.Tuesday != Tuesday
                   ||
                   _oldValues.Wednesday != Wednesday
                   ||
                   _oldValues.Thursday != Thursday
                   ||
                   _oldValues.Friday != Friday
                   ||
                   _oldValues.Saturday != Saturday
                   ||
                   _oldValues.Am != Am
                   ||
                   _oldValues.Pm != Pm
                   ||
                   _oldValues.BothChecked != BothChecked
                   ||
                   _oldValues.GenderSpecificProvider != GenderSpecificProvider
                   ||
                   _oldValues.HearingImpaired != HearingImpaired
                   ||
                   _oldValues.LanguagePreference != LanguagePreference
                   ||
                   _oldValues.NeedsEscort != NeedsEscort
                   ||
                   _oldValues.OnStretcher != OnStretcher
                   ||
                   _oldValues.SpecialModeOfTransportation != SpecialModeOfTransportation
                   ||
                   _oldValues.Remarks != Remarks
                   ||
                   _oldValues.DateAdded != DateAdded
                   ||
                   _oldValues.AddedByUserIen != AddedByUserIen
                   ||
                   _oldValues.AddedByUserName != AddedByUserName
                   ||
                   _oldValues.DateInactivated != DateInactivated
                   ||
                   _oldValues.InactivatedByUserIen != InactivatedByUserIen
                   ||
                   _oldValues.InactivatedByUserName != InactivatedByUserName;
        }
        
        //Special needs..
        public bool Monday{ get; set; }
        public bool Tuesday { get; set; }
        public bool Wednesday { get; set; }
        public bool Thursday { get; set; }
        public bool Friday { get; set; }
        public bool Saturday { get; set; }
        public bool Am { get; set; }
        public bool Pm { get; set; }
        public bool BothAmPm { get { return !Am && !Pm; } }
        public bool BothChecked { get; set; } = true;
        public bool GenderSpecificProvider { get; set; }
        public bool HearingImpaired { get; set; }
        public bool LanguagePreference { get; set; }
        public bool NeedsEscort { get; set; }
        public bool OnStretcher { get; set; }
        public bool SpecialModeOfTransportation { get; set; }

        public string Remarks { get; set; }

        public string RemarksParam
        {
            get
            {
                if (string.IsNullOrEmpty(Remarks) && !string.IsNullOrEmpty(_oldValues.Remarks))
                    return "@";

                return Remarks;
            }
        }
        public string DateAdded { get; set; }
        public string AddedByUserIen { get; set; }
        public string AddedByUserName { get; set; }
        public string DateInactivated { get; set; }
        public string InactivatedByUserIen { get; set; }
        public string InactivatedByUserName { get; set; }
    }
}
