﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Windows;

//using CIA_CSS;
// using OVIDDomain;

//using CSS_User;

namespace ClinSchd.Infrastructure.Models
{
    public class OVIDDataTableRepository : IDisposable
    {
        #region private Members

        //private ICSS_Session _session;
        private ADTDataRepository _dataRepo;

        //private IRpcClient _rpcClient;
        //private IRpcConnection _rpcConnection;

        private static OVIDDataTableRepository _instance;
        private static object _lock = new object();

        #endregion private Members

        #region public Members

        public static bool LoginDialogShow { get { return !ADTDataRepository.IsLoggedIn; } }

        public static OVIDDataTableRepository Instance
        {
            get
            {
                if (_instance == null) {
                    lock (_lock) {
                        if (_instance == null)
                            _instance = new OVIDDataTableRepository();
                    }
                }
                return _instance;
            }
        }

        //public String UserID { get { return _user.Handle.ToString(); } }

        public string UserID { get; set; }

        /*
         * MSCDataTableRepository
         *
         * Constructor. initialize the service reference.
         *
         * Params:
         *    None
         *
         * Returns:
         *    MSCDataTableRepository object
         *
         */

        public OVIDDataTableRepository()
        {
            //_session = new CSS_Server().Session;

            //_user = (ICSS_User)_session.FindServiceByProgID("CSS_User.User");

            //_rpcClient = new RpcClient(_session);
            //_rpcConnection = new RpcConnection(_rpcClient);

            _dataRepo = new ADTDataRepository();
        }

        //public static System.Windows.Controls.Panel Container { get; set; }
        public static System.Windows.Controls.Grid Container { get; set; }

        /* MSC/DKA 9/26/14 Added Logout() method to facilitate clean exit from RPC Broker connection */
        public void UserInfoLogout()
        {
            _dataRepo.Logout();
        }

        public bool LoginUser(string accessCode, string verifyCode, out string szErrorMsg)
        {
            szErrorMsg = "";
            UserID = _dataRepo.LoginUser(accessCode, verifyCode, out szErrorMsg);
            if(UserID != null)
            {
                Keys.LoggedInUserAccessCode = accessCode;
                Keys.LoggedInUserVerifyCode = verifyCode;
            }
            return UserID != null;
        }

        /*
         * PopulateDataTable
         *
         * This method logout the authenticated user and remove the connection to the backend services.
         *
         * Params: none
         *
         * Returns: void
         *
         * Throws: AuthenticationException, RPCException.
         *
         */

        private static DataTable PopulateDataTable(string[][] data_res)
        {
            if (data_res == null)
                return null;
            DataTable dt = new DataTable();

            try {

                // used to get the header information returned from the RPC command.
                int count = 0;

                // loop through the RPC command results array and add data to the DataTable
                foreach (var p in data_res) {
                    if (p == null)
                        continue;

                    // Check for DataTable header information
                    if (count == 0) {

                        // loop through each header item
                        foreach (string st in p) {

                            // add each header item to the DataTable object
                            dt.Columns.Add(st, typeof(string));
                        }

                        // increment the count to avoid adding header info again
                        count++;
                    } else if (count > 2) {

                        // DataTable row
                        DataRow dr = dt.NewRow();
                        for (int i = 0; i < p.Length; i++) {
                            dr[i] = p[i];
                        }

                        // add the DataRow to the DataTable
                        dt.Rows.Add(dr);
                    } else {

                        // increment the count to avoid rows 1 and 2
                        count++;
                    }
                }

                ////Console.WriteLine("Call Successful");
                //WriteToConsole("Call Successful");
            } catch (Exception e)
            {
                MessageBox.Show(e.Message, "Populate Data Table Error");
                throw;
            }

            // return the DataTable
            return dt;
        }

        /*
         * ScrubDataArray
         *
         * This method scrubs the array data so there are no null values.
         *
         * Params:
         *    string arr - an array of strings data
         *
         * Returns:
         *    string[]
         *
         */

        public object[] ScrubDataArray(object[] arr)
        {
            // set all null values in the array to empty strings to avoid problems
            for (int i = 0; i < arr.Length; i++)
            {
                if (arr[i] is string)
                    if (arr[i] == null)
                        arr[i] = string.Empty;
            }

            return arr;
        }

        public string[] ScrubDataArray(string[] arr)
        {
            // set all null values in the array to empty strings to avoid problems
            for (int i = 0; i < arr.Length; i++) {
                if (arr[i] == null)
                    arr[i] = string.Empty;
            }

            return arr;
        }

        /*
         * callOVIDRPC
         *
         * This method loads the results of the RPC command into a DataTable object. The DataTable oject
         * is then added to a DataSet object and assigned the a table name. If an attempt is made to add
         * a DataTable to the DataSet using a table name that currently exist in the DataSet then first,
         * the currently existing DataTable object and table name are deleted from the DataSet object
         * and second, the new DataTable object and table name are added to the DataSet object.
         * The m_dsGlobal DataSet also contains a copy of the DataTable returned by this method.
         *
         * Params:
         *    rpc_name - The name of the RPC command to invoke
         *    rpc_params - A list of string containing the RPC parameters
         *    table_name - The name of the DataTable to be added to the DataSet
         *    m_dsGlobal - The name of the DataSet that contains the DataTables
         *
         * Returns:
         *    DataTable object containing the results of the call to the RPC command.
         *
         * NOTE: This method is primarily called by BMX users.
         *
         */

        public DataTable callOVIDRPC(string rpc_name, List<string> rpc_params, string table_name, DataSet m_dsGlobal)
        {
            // data table to add to data set
            DataTable dt = null;

            try {

                // validate data set contains requested data table
                if (m_dsGlobal.Tables.Contains(table_name)) {

                    // updating or refreshing so remove existing data table from data set
                    m_dsGlobal.Tables.Remove(table_name);
                }

                // request new data table, name it, and add to data set
                dt = callOVIDRPC(rpc_name, rpc_params);
                dt.TableName = table_name;
                m_dsGlobal.Tables.Add(dt);
            } catch (InvalidOperationException) {
                throw;
            } catch (Exception e) {
                throw;
            }

            // return the DataTable
            return dt;
        }

        public DataTable callOVIDRPC(string rpc_name, List<object> rpc_params)
        {
#if DEBUG
            Console.WriteLine("callOVIDRPC: rpc_name={0}", rpc_name);
#endif
            DataTable dt = null;

            try
            {

                //object args = null;

                //if(rpc_params != null && rpc_params.Count > 0)
                //    args = new object[] { rpc_params.ToArray() };

                //string response = _session.CallRPCStr(rpc_name, args);

                // Must be done before attempting to call any WS methods
                //Login("USER123", "USER321$");
                //Login("SUPER,1", "ITSUP,1!");

                // create MSC Domain request object used to store data sent to the RPC
                //GetPIMSMSCDataExtRequest data_req = null;
                string[][] data_res;

                // check for a valid parameter object
                if (rpc_params != null)
                {

                    // convert the list to an array
                    object[] rpcParms = ScrubDataArray(rpc_params.ToArray());

                    data_res = _dataRepo.getADTDataExt(rpc_name, rpcParms);

                    // setup the RPC command request object with the parameters
                    //data_req = new GetPIMSMSCDataExtRequest(rpc_name, rpcParms);
                    //data_req = new GetPIMSMSCDataExtRequest(rpc_name, rpc_params.ToArray());
                }
                else
                {

                    // this RPC takes no parameters so setup the RPC command request object with the null array
                    //String[] null_param = null;
                    data_res = _dataRepo.getADTDataExt(rpc_name, null);

                    //data_req = new GetPIMSMSCDataExtRequest(rpc_name, null);
                }

                // authentication succeeded so call the RPC with the request object and
                // create MSC Domain response object used to store data received from the RPC
                //GetPIMSMSCDataExtResponse data_res = service.GetPIMSMSCDataExt(data_req);

                dt = PopulateDataTable(data_res);
            }
            catch (InvalidOperationException e)
            {
                throw;
            }
            catch (Exception e)
            {
                throw;
            }

            if (dt == null)
                dt = new DataTable();

            // return the DataTable
            return dt;
        }


        /*
         * callOVIDRPC
         *
         * This method loads the results of the RPC command into a DataTable object.
         *
         * Params:
         *    rpc_name - The name of the RPC command to invoke
         *    rpc_params - A list of string containing the RPC parameters
         *
         * Returns:
         *    DataTable object containing the results of the call to the RPC command.
         *
         */

        public DataTable callOVIDRPC(string rpc_name, List<string> rpc_params)
        {
#if DEBUG
            Console.WriteLine("callOVIDRPC: rpc_name={0}", rpc_name);
#endif
            DataTable dt = null;

            try {

                //object args = null;

                //if(rpc_params != null && rpc_params.Count > 0)
                //    args = new object[] { rpc_params.ToArray() };

                //string response = _session.CallRPCStr(rpc_name, args);

                // Must be done before attempting to call any WS methods
                //Login("USER123", "USER321$");
                //Login("SUPER,1", "ITSUP,1!");

                // create MSC Domain request object used to store data sent to the RPC
                //GetPIMSMSCDataExtRequest data_req = null;
                string[][] data_res;

                // check for a valid parameter object
                if (rpc_params != null) {

                    // convert the list to an array
                    string[] rpcParms = ScrubDataArray(rpc_params.ToArray());
                    
                    data_res = _dataRepo.getADTDataExt(rpc_name, rpcParms);

                    // setup the RPC command request object with the parameters
                    //data_req = new GetPIMSMSCDataExtRequest(rpc_name, rpcParms);
                    //data_req = new GetPIMSMSCDataExtRequest(rpc_name, rpc_params.ToArray());
                } else {

                    // this RPC takes no parameters so setup the RPC command request object with the null array
                    //String[] null_param = null;
                    data_res = _dataRepo.getADTDataExt(rpc_name, null);

                    //data_req = new GetPIMSMSCDataExtRequest(rpc_name, null);
                }

                // authentication succeeded so call the RPC with the request object and
                // create MSC Domain response object used to store data received from the RPC
                //GetPIMSMSCDataExtResponse data_res = service.GetPIMSMSCDataExt(data_req);

                dt = PopulateDataTable(data_res);
            } catch (InvalidOperationException e) {
                throw;
            } catch (Exception e) {
                throw;
            }

            if (dt == null)
                dt = new DataTable();

            // return the DataTable
            return dt;
        }

        /*
         * callOVIDRPC
         *
         * This method loads the results of the RPC command into a DataTable object.
         *
         * Params:
         *    rpc_name - The name of the RPC command to invoke
         *    rpc_params - A list containing a list of string where each string is an single RPC parameter
         *                 and each list of string can be an array of parameters to the RPC. This allow
         *                 for combinations of single string parameter and arrays of parameters to be
         *                 to the RPC at once.
         *
         * Returns:
         *    DataTable object containing the results of the call to the RPC command.
         *
         * NOTE: This method is primarily called by BMX users.
         *
         */

        public DataTable callOVIDRPC(string rpc_name, List<List<string>> rpc_params)
        {
            DataTable dt = new DataTable();

            try {

                // Must be done before attempting to call any WS methods
                //Login("USER123", "USER321$");
                //Login("SUPER,1", "ITSUP,1!");

                // create MSC Domain request objects used to login and retrieve data from the RPC
                //GetPIMSMSCDataObjExtRequest data_req = null;

                //// create the list of objects. each object contains an array of strings
                //List<rpcParamData> rpd = new List<rpcParamData>();

                //if (rpc_params != null)
                //{
                //    // iterate over each list of strings in the list object
                //    foreach (List<String> p in rpc_params)
                //    {
                //        // create the object that holds the array
                //        rpcParamData rpdObj = new rpcParamData();

                //        // convert the list of string to an array and add it to the object
                //        rpdObj.RPCParamData = ScrubDataArray(p.ToArray());

                //        // add the object containing the array of string of string to the list of abjects
                //        rpd.Add(rpdObj);
                //    }

                //    // setup the RPC command request object with the parameters
                //    data_req = new GetPIMSMSCDataObjExtRequest(rpc_name, rpd.ToArray());
                //}
                //else
                //{
                //    // this RPC takes no parameters so setup the RPC command request object with the null array
                //    data_req = new GetPIMSMSCDataObjExtRequest(rpc_name, null);
                //}

                //// authentication succeeded so call the RPC with the request object
                //GetPIMSMSCDataObjExtResponse data_res = service.GetPIMSMSCDataObjExt(data_req);

                //// NOTE: the follwing two lines are considered a hack. The GetPIMSMSCDataExtResponse and
                ////       GetPIMSMSCDataObjExtResponse objects are essentially the same and the PopulateDataTable
                ////       method process GetPIMSMSCDataExtResponse object. Assigning the stringArray values of the
                ////       GetPIMSMSCDataObjExtResponse to GetPIMSMSCDataExtResponse allows the data to be processed
                ////       without additional code or the need to copy the individual values of the stringArray
                ////       to a string[]. Thus avoiding aditional code. Not great but it works for now.
                //GetPIMSMSCDataExtResponse data_store = new GetPIMSMSCDataExtResponse(data_res.@return);
                //dt = PopulateDataTable(data_store);
            } catch (InvalidOperationException e) {
                throw;
            } catch (Exception e) {
                throw;
            }

            // return the DataTable
            return dt;
        }

        public string CallVistaRpcString(string rpc_name, List<string> rpc_params)
        {
            string dataRes;
            _dataRepo.IsSingleValueReturn = true;
            if (rpc_params != null)
            {
                var rpcParms = ScrubDataArray(rpc_params.ToArray());
                dataRes = _dataRepo.GetAdtDataExtString(rpc_name, rpcParms, false);
            }
            else
                dataRes = _dataRepo.GetAdtDataExtString(rpc_name, null, false);
            return dataRes;
        }

        public string CallVistaRpcString(string rpc_name, List<object> rpc_params)
        {
            string dataRes;
            _dataRepo.IsSingleValueReturn = true;
            if (rpc_params != null)
            {
                var rpcParms = ScrubDataArray(rpc_params.ToArray());
                dataRes = _dataRepo.GetAdtDataExtString(rpc_name, rpcParms, false);
            }
            else
                dataRes = _dataRepo.GetAdtDataExtString(rpc_name, null, false);
            return dataRes;
        }

        public string[][] CallVistaRpc(string rpc_name, List<string> rpc_params, bool isSingleReturn = false)
        {
            string[][] dataRes;
            _dataRepo.IsSingleValueReturn = isSingleReturn;
            if (rpc_params != null)
            {
                var rpcParms = ScrubDataArray(rpc_params.ToArray());
                dataRes = _dataRepo.getADTDataExt(rpc_name, rpcParms, false);
            }
            else
                dataRes = _dataRepo.getADTDataExt(rpc_name, null, false);
            return dataRes;
        }


        #endregion public Members

        #region IDisposable Members

        public void Dispose()
        {
            //if (_session != null)
            //{
            //    Marshal.ReleaseComObject(_session);
            //    _session = null;
            //}

            //if (_user != null)
            //{
            //    Marshal.ReleaseComObject(_user);
            //    _user = null;
            //}
        }

        #endregion IDisposable Members
    }
}
