using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;

namespace ClinSchd.Modules.CancelAppt.CancelAppt
{
    /// <summary>
    /// Interaction logic for CancelApptView.xaml
    /// </summary>
    public partial class CancelApptView : ICancelApptView
    {
        public CancelApptView ()
        {
            InitializeComponent();
            Owner = Keys.MainWindow;
            AddToWaitList = true;
        }

        public bool AddToWaitList { get; set; }

        public CancelApptPresentationModel Model
        {
            get
            {
                return DataContext as CancelApptPresentationModel;
            }

            set
            {
                DataContext = value;
            }
        }

        protected override void OnClosing (System.ComponentModel.CancelEventArgs e)
        {
        } 

        private void Button_OK_Click(object sender, RoutedEventArgs e)
        {
            if (LbCancelReason.SelectedItems.Count == 0)
            {
                MessageBox.Show("Please provide a reason for cancelling this appointment.");
                LbCancelReason.Focus();
                return;
            }
            Model.CancelAppointment.CancelledByClinic = CancelledByClinic.IsChecked.Value;
            Model.ExecuteCancelApptAppointmentCommand ("Cancel Appointment");

            if (Model.ValidationMessage.IsValid) {
                DialogResult = true;

                if (ConfirmUser("Print the Patient Letter?", "Print Letter?"))
                    Model.PrintLetterCommand.Execute("");

                Model.EventAggregator.GetEvent<AppointmentCancelledEvent>().Publish(
                        Model.CancelAppointment.ApptID + "^" + 
                        (Model.CancelAppointment.CancelledByClinic ? "1" : "0") + "^" +
                        Model.CancelAppointment.ReasonIEN + "^" +
                        Model.CancelAppointment.Notes
                    );
                var name = Model.SelectedAppointment.PATIENTNAME; //?????
                var patIen = Model.SelectedAppointment.PATIENTID;
                string waitListIen = string.Empty;

                bool desiredDateUpdated = Model.CancelAppointment.OriginalDesiredDate != Model.CancelAppointment.PatientDesiredDate;
                waitListIen = Model.DataAccessService.ReopenWaitListRecordAfterCancelAppointment(
                    Model.SelectedAppointment.APPTREQTYPE_REQTYPE,
                    Model.SelectedAppointment.APPTREQTYPE_REQIEN,
                    Model.CancelAppointment.ApptID,
                    name,
                    patIen,
                    Model.CancelAppointment.PatientDesiredDate.ToShortDateString(),
                    desiredDateUpdated,
                    AddToWaitList
                );

                if (AddToWaitList)
                    Model.EventAggregator.GetEvent<RefreshWaitlistEvent>().Publish(waitListIen);

                Close ();
            } else if (!Model.ValidationMessage.IsValid && Model.ValidationMessage.Title == "AutoRebook Appointment") {
                AlertUser (Model.ValidationMessage.Message, Model.ValidationMessage.Title);
                Close ();
            } else {
                AlertUser (Model.ValidationMessage.Message, Model.ValidationMessage.Title);
            }
        }

        private void Button_Cancel_Click(object sender, RoutedEventArgs e)
        {
            string errorText = Model.CloseTheAppointmentRequest();
            Model.EventAggregator.GetEvent<AppointmentCancelledEvent>().Publish(null);

            if (!string.IsNullOrEmpty(errorText))
                MessageBox.Show(errorText);

            Close();
        }

        public void Refresh ()
        {
        }

        private bool _bDialogResult;
        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }   

        public void AlertUser (string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        private void AnyType_Checked (object sender, RoutedEventArgs e)
        {
            Model.AutoRebook.IsAccessTypeChecked = false;
        }

        private void CancelledByClinic_Checked(object sender, RoutedEventArgs e)
        {
            if (Model == null)
            {
                return;
            }
            Model.CancelAppointment.CancelledByClinic = true;
        }

        private void CancelledByPatient_Checked(object sender, RoutedEventArgs e)
        {
            if (Model == null)
            {
                return;
            }
            Model.CancelAppointment.CancelledByClinic = false;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            LbCancelReason.Focus();
            Keyboard.Focus(LbCancelReason);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }
    }
}
