using System;
using System.Collections.Generic;
using ClinSchd.Infrastructure.Interfaces;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.CancelAppt.Services;
using ClinSchd.Modules.CancelAppt.CancelAppt;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Unity;

namespace ClinSchd.Modules.CancelAppt.Controllers
{
    public class CancelApptController : ICancelApptController
    {
        private readonly IUnityContainer _container;
        private readonly ICancelApptService _cancelApptService;
        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;

        public CancelApptController (IUnityContainer container,
            ICancelApptService cancelApptService,
            IEventAggregator eventAggregator, IDataAccessService dataAccessService)
        {
            _container = container;
            _cancelApptService = cancelApptService;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
        }

        public void Run()
        {			
            _eventAggregator.GetEvent<LaunchCancelApptDialogEvent> ().Subscribe (LaunchCancelApptDialog, ThreadOption.UIThread, true);
        }

        public void LaunchCancelApptDialog (CancelApptDialogEventPayload payload)
        {
            SchdAppointment appointment = payload.ApptToCancel;
            var requestType = "";
            var apptRequest = _dataAccessService.GetAppointmentRequestByAppointmentIen(appointment.APPOINTMENTID, ref requestType);
            appointment.AppointmentRequest = apptRequest;

            if (apptRequest != null)
                appointment.DesiredAppointmentDate = apptRequest.DesiredApptDate;
            
            ICancelApptPresentationModel model = _container.Resolve<ICancelApptPresentationModel> ();
            model.View.AddToWaitList = payload.AddToWaitList;
            model.SelectedAppointment = appointment;
            model.SelectedRequest = apptRequest;
            model.CancelApptAppointment (appointment);
            if (model.ValidationMessage.IsValid) {
                _cancelApptService.ShowDialog (model.View, model, payload.Callback);
            } else {
                model.View.AlertUser (model.ValidationMessage.Message, model.ValidationMessage.Title);
                model.ValidationMessage.IsValid = true;
                model.ValidationMessage.Title = string.Empty;
                model.ValidationMessage.Message = string.Empty;
            }
        }
    }
}
