
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using ClinSchd.Infrastructure.Events;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.CheckIn.Services;
using Microsoft.Practices.Unity;

namespace ClinSchd.Modules.CheckIn.CheckIn
{
    public class CheckInPresentationModel : ICheckInPresentationModel, INotifyPropertyChanged
    {
        private readonly ICheckInService _checkInService;
        private readonly IDataAccessService _dataAccessService;
        public readonly IEventAggregator EventAggregator;
        private ValidationMessage _validationMessage;
        private bool _isSummaryReportChecked;
        private string _patientName;
        private CheckInPatient _checkInData;
        private string _healthSummaryLetterIEN;
        private bool _isNewWalkIn = false;

        public CheckInPatient CheckInData
        {
            get { return _checkInData; }
            set
            {
                _checkInData = value;
                OnPropertyChanged( "CheckIn" );
            }
        }

        public CheckInPresentationModel (
            ICheckInView view,
            ICheckInService checkInService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View               = view;
            View.Model         = this;
            EventAggregator    = eventAggregator;
            _checkInService    = checkInService;
            _dataAccessService = dataAccessService;

            ValidationMessage = new ValidationMessage();
            InitialErrorMessage();
            CheckInAppointmentCommand = new DelegateCommand<string> (ExecuteCheckInAppointmentCommand, CanExecuteCheckInAppointmentCommand);
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public void PromptToRemoveRequest(string patientName)
        {
            EventAggregator.GetEvent<RemoveRequestEvent>().Publish(patientName);
        }

        public void GetCheckInAppointment (SchdAppointment newAppointment)
        {
            _isNewWalkIn = (newAppointment.IsNewAppointment && (newAppointment.AppointmentStatus == AppointmentStatuses.WalkIn_NotCheckedIn) || (newAppointment.AppointmentStatus == AppointmentStatuses.WalkIn_CheckdIn));
            CheckInPatient checkInData = new CheckInPatient ();
            if ( newAppointment == null || newAppointment.APPOINTMENTID == null )
                return;

            checkInData.ApptID = newAppointment.APPOINTMENTID;
            checkInData.PatientIEN = newAppointment.PATIENTID;

            IsSummaryReportChecked = false;

            if (Convert.ToDateTime (newAppointment.START_TIME).Date > DateTime.Today.Date) {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check In Patient";
                _validationMessage.Message = "It is too early to check in " + newAppointment.PATIENTNAME;

                return;
            }
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            PatientName = newAppointment.PATIENTNAME;

            //Find the default provider for the resource & load into combo box
            //DataView rv = new DataView (this.dataAccessService.GetResources());
            
            
            /*DataView rv = new DataView(Keys.Tables.AllResourceTable);


            rv.Sort = "RESOURCE_NAME ASC";
            int nFind = rv.Find ((string)newAppointment.RESOURCENAME);
            if (nFind < 0)
                return;
            DataRowView drv = rv[nFind];*/
            var schdRes = _dataAccessService.GetSdecResourceByName(newAppointment.RESOURCENAME);

            string sHospLoc = schdRes.HOSPITAL_LOCATION_ID; // drv["HOSPITAL_LOCATION_ID"].ToString ();
            sHospLoc = sHospLoc == string.Empty ? "0" : sHospLoc;
            int nHospLoc = 0;
            try
            {
                nHospLoc = Convert.ToInt32 (sHospLoc);
            }
            catch (Exception ex)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check In Patient";
                _validationMessage.Message = ex.Message;
            }

            string sProv;
            //bool bProvReqd = false;
            //bool bPCC = false;
            //bool bMultCodes = false;

            if (nHospLoc > 0) {
                //Get hospital locations
                
                /*DataView hv = new DataView(this.dataAccessService.GetHospitalLocations());
                //DataView hv = new DataView(Keys.Tables.HospitalResourceTable);
                hv.Sort = "HOSPITAL_LOCATION_ID";
                int hFind = hv.Find (nHospLoc);
                DataRowView dhv = hv[hFind];*/
                
                sProv = schdRes.DefaultProvider; // dhv["DEFAULT_PROVIDER"].ToString ();
                var sStopCode = schdRes.StopCodeNumber;

                if ( !string.IsNullOrEmpty( sStopCode ) )
                {
                    var sv = new DataView( _dataAccessService.GetClinicStop( sStopCode ) )
                    {
                        Sort = "NAME ASC"
                    };
                    int sFind = sv.Find( sStopCode );
                    if ( sFind > -1 )
                    {
                        checkInData.ClinicStopIEN = sv[sFind].Row["CLINIC_STOP_IEN"].ToString();
                    }
                }

                DataTable dt = _dataAccessService.GetClinicSetupParameters(sHospLoc, null);

                if (dt.Rows.Count > 0)
                {
                    //DataRow dr = dt.Rows[0];
                    //var sProvReqd = dr["VISIT_PROVIDER_REQUIRED"].ToString();
                    //var sPCC = dr["GENERATE_PCCPLUS_FORMS?"].ToString();
                    //var sMultCodes = dr["MULTIPLE_CLINIC_CODES_USED?"].ToString();

                    //bProvReqd = sProvReqd == "YES";
                    //bPCC = sPCC == "YES";
                    //bMultCodes = sMultCodes == "YES";
                }
            } else {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check In Patient";
                _validationMessage.Message = "You need to select a clinic for this resource in Scheduling Management.";
                return;
            }

            //Provider processing
            if (!string.IsNullOrEmpty(sProv))
            {
                var provList = _dataAccessService.GetSdecProviders(sProv);
                if (provList.Count > 0)
                    ProviderIEN = provList[0].IEN;
            }
/*
            Providers = this.dataAccessService.GetProviders ();

            bool isNone = false;
            foreach (Provider nameValue in Providers) {
                if (nameValue.Name == "<None>")
                    isNone = true;
            }

            if (!isNone) {
                Provider newProv = ProviderFactory.Create ();
                newProv.Name = "<None>";
                newProv.IEN = "0";
                Providers.Insert (0, newProv);
            }
            OnPropertyChanged ("Providers");

            if (sProv != string.Empty) {
                foreach (Provider n in Providers) {
                    if (n.Name == sProv)
                        this.ProviderIEN = n.IEN;
                }
            } else {
                this.ProviderIEN = Providers[0].IEN;
            }
 */
            checkInData.ProviderIEN = ProviderIEN;

            //PCC Plus processing
            //...

            //Health Summary Report processing
            /* - Don't need - per Fay S 10/30/2014,,,,,,,,,,
            SummaryReport = this.dataAccessService.GetSummaryReport ();
            OnPropertyChanged ("SummaryReport");
            HealthSummaryLetterIEN = string.Empty;
            OnPropertyChanged ("HealthSummaryLetterIEN");
            */
            //default the check in time
            checkInData.CheckInDateTime = DateTime.Now.ToString ();

            CheckInData = checkInData;
        }

        public void ExecuteCheckInAppointmentCommand (string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            string errorMessage = _dataAccessService.CheckInPatient (CheckIn);

            if (errorMessage == string.Empty)
            {
                if (!_isNewWalkIn)
                    EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(String.Empty);

                EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);
            }
            else
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Check In Patient";
                _validationMessage.Message = errorMessage;
            }
        }

        public bool CanExecuteCheckInAppointmentCommand (string command)
        {
            return true;
        }

        public void OnClose()
        {
            View.Close();
        }

        [Dependency]
        public Factory<Provider> ProviderFactory { get; set; }

        #region ICheckInPresentationModel Members

        public ICheckInView View { get; }

        public string PatientName
        {
            get { return _patientName; }
            set
            {
                _patientName = value;
                OnPropertyChanged( "PatientName" );
            }
        }

        public string ProviderIEN { get; set; }
        public string PCCClinicIEN { get; set; }
        public string PCCFormIEN { get; set; }

        public string HealthSummaryLetterIEN
        {
            get { return _healthSummaryLetterIEN; }
            set
            {
                _healthSummaryLetterIEN = value;
                OnPropertyChanged( "HealthSummaryLetterIEN" );
            }
        }

        public bool IsSummaryReportChecked
        {
            get { return _isSummaryReportChecked; }
            set
            {
                _isSummaryReportChecked = value;
                OnPropertyChanged( "IsSummaryReportChecked" );
            }
        }

        public IList<Provider> Providers { get; set; }
        public SchdAppointment SelectedAppointment { get; set; }
        public IList<NameValue> SummaryReport { get; set; }
        public DelegateCommand<string> CheckInAppointmentCommand { get; private set; }

        public CheckInPatient CheckIn
        {
            get { return CheckInData; }
            set { CheckInData = value; }
        }

        public string IsSummaryReportVisible { get; set; } = "Hidden";

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set { _validationMessage = value; }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
