using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;

namespace ClinSchd.Modules.CheckOut.CheckOut
{
    /// <summary>
    /// Interaction logic for CheckOutView.xaml
    /// </summary>
    public partial class CheckOutView : ICheckOutView
    {
        public CheckOutView ()
        {
            InitializeComponent();
            Owner = Keys.MainWindow;
        }

        public CheckOutPresentationModel Model
        {
            get { return DataContext as CheckOutPresentationModel; }
            set { DataContext = value; }
        }

        protected override void OnClosing (System.ComponentModel.CancelEventArgs e)
        {
        } 

        private void Button_OK_Click(object sender, RoutedEventArgs e)
        {
            Model.ExecuteCheckOutAppointmentCommand ("CheckOut Patient");

            if (Model.ValidationMessage.IsValid)
            {
                Close ();
            } else {
                Model.View.AlertUser (Model.ValidationMessage.Message, Model.ValidationMessage.Title);
            }
        }

        private void Button_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        public void Refresh ()
        {
        }

        private bool _bDialogResult;
        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }   

        public void AlertUser (string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            OkButton.Focus();
            Keyboard.Focus(OkButton);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }
    }
}
