﻿using System;
using ClinSchd.Modules.ContactAttempt.ContactAttempt;
using VistADataAccessService.Models;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.ContactAttempt.Services;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure;
using Microsoft.Practices.Composite.Presentation.Commands;
using System.Collections.Generic;
using ClinSchd.Infrastructure.Factories;

namespace ClinSchd.Modules.ContactAttempt
{
    public class ContactAttemptPresentationModel : IContactAttemptPresentationModel, INotifyPropertyChanged
    {
        private readonly IContactAttemptService _ContactAttemptService;
        public readonly IDataAccessService _DataAccessService;
        public readonly IEventAggregator _EventAggregator;

        public ContactAttemptPresentationModel(
            IContactAttemptView view,
            IContactAttemptService ContactAttemptService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            _EventAggregator = eventAggregator;
            _ContactAttemptService = ContactAttemptService;
            _DataAccessService = dataAccessService;

            View = view;
            View.Model = this;
            ValidationMessage = new Infrastructure.Models.ValidationMessage();
            InitialErrorMessage();

            SubmitContactAttemptCommand = new DelegateCommand<string>(ExecuteSubmitContactAttemptCommand);
        }

        public void LoadAppointmentRequest(AppointmentRequest apptRequest)
        {
            AppointmentRequest = apptRequest;
            if (AppointmentRequest.PatientContact == null)
                loadPatientContact();
        }

        #region HELPERS
        private void loadPatientContact()
        {
            AppointmentRequest.PatientContact = PatientContactFactory.CreatePatientContact(
                _DataAccessService.GetPatientContact(PatientContactRequestFactory.CreateReadRequestParams(
                    appointmentRequest)));

            if (AppointmentRequest.PatientContact != null)
                loadContactAttempts(); 
        }
        private void loadContactAttempts()
        {
            AppointmentRequest.PatientContact.ContactAttempts = ContactAttemptFactory.CreateContactAttempts(
                AppointmentRequest.PatientContact,_DataAccessService.GetContactAttempts(
                    PatientContactRequestFactory.CreateReadMulipleRequestParams(AppointmentRequest.PatientContact)));

            OnPropertyChanged("AppointmentRequest");
        }

        #endregion

        #region COMMANDS
        public DelegateCommand<string> SubmitContactAttemptCommand { get; private set; }
        private void ExecuteSubmitContactAttemptCommand(string command)
        {
            if (AppointmentRequest.PatientContact == null)
                insertNewPatientContact();
            else
                updatePatientContacts();                       
        }
        #endregion

        #region COMMAND HELPERS 
        private void insertNewPatientContact()
        {
            _DataAccessService.CreateNewSdecContact(PatientContactRequestFactory.CreateInsertRequestParams(
                AppointmentRequest, NewContactAttempt)); 

            clearContactAttemptFields();
            loadPatientContact();
        }
        private void updatePatientContacts()
        {
            _DataAccessService.UpdateContactAttempt(PatientContactRequestFactory.CreateUpdateRequestParams(
                AppointmentRequest, NewContactAttempt));

            clearContactAttemptFields();
            loadPatientContact();
        }

        private void clearContactAttemptFields()
        {
            NewContactAttempt =
                new Infrastructure.Models.ContactAttempt { ContactAttemptDateTime = DateTime.Now };

            ContactTypeCall = true; 
        }
        #endregion
        
        #region PRESENTATION PROPERTIES


        private AppointmentRequest appointmentRequest;
        public AppointmentRequest AppointmentRequest
        {
            get { return appointmentRequest; }
            set
            {
                if (value == appointmentRequest)
                    return;
                appointmentRequest = value;
                OnPropertyChanged("AppointmentRequest");
            }
        }


        private Infrastructure.Models.ContactAttempt newContactAttempt = 
            new Infrastructure.Models.ContactAttempt { ContactAttemptDateTime = DateTime.Now };
        public Infrastructure.Models.ContactAttempt NewContactAttempt
        {
            get { return newContactAttempt; }
            set
            {
                if (value == newContactAttempt)
                    return;

                newContactAttempt = value;
                OnPropertyChanged("NewContactAttempt");
            }
        }


        private bool contactTypeLetter;
        public bool ContactTypeLetter
        {
            get { return contactTypeLetter; }
            set
            {
                if (value == contactTypeLetter)
                    return;

                contactTypeLetter = value;
                if (value == true)
                    NewContactAttempt.ContactAttemptType = ContactAttemptType.Letter;
                OnPropertyChanged("ContactTypeLetter");
            }
        }


        private bool contactTypeCall = true;
        public bool ContactTypeCall
        {
            get { return contactTypeCall; }
            set
            {
                if (value == contactTypeCall)
                    return;

                contactTypeCall = value;
                if (value == true)
                    NewContactAttempt.ContactAttemptType = ContactAttemptType.Call;
                OnPropertyChanged("ContactTypeCall");
            }
        }


        #endregion

        #region VSE MODULE PATTERN PROPERTIES
        public IContactAttemptView View { get; }

        public Infrastructure.Models.ValidationMessage ValidationMessage { get; set; }

        public void InitialErrorMessage()
        {
            ValidationMessage.IsValid = true;
            ValidationMessage.Title = string.Empty;
            ValidationMessage.Message = string.Empty;
        }

        public void OnClose()
        {
            View.Close();
        }
        #endregion

        #region PROPERTY CHANGED
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}