﻿using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.ContactAttempt.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Unity;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ClinSchd.Modules.ContactAttempt.Controllers
{
    public class ContactAttemptController : IContactAttemptController
    {
        private readonly IContactAttemptService _contactAttemptService;
        private readonly IEventAggregator _eventAggregator;
        private readonly IUnityContainer _container;

        public ContactAttemptController(IUnityContainer container,
            IContactAttemptService contactAttemptService,
            IEventAggregator eventAggregator)
        {
            _contactAttemptService = contactAttemptService;
            _eventAggregator = eventAggregator;
            _container = container;
        }

        public void Run()
        {
            _eventAggregator.GetEvent<LaunchContactAttemptDialogEvent>().Subscribe(LaunchContactAttemptDialog, ThreadOption.UIThread, true);
        }

        public void LaunchContactAttemptDialog(AppointmentRequest appointmentRequest)
        {
            var model = _container.Resolve<IContactAttemptPresentationModel>();

            if (appointmentRequest == null)
            {
                model.View.AlertUser("You haven't selected an appointment request.", "Oops!");
                return;
            }

            model.LoadAppointmentRequest(appointmentRequest);
            _contactAttemptService.ShowDialog(model.View, model, () => model.OnClose());
        }

    }
}
