﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.DataAccess.Services;

namespace ClinSchd.Modules.DataAccess.Views
{
    /// <summary>
    /// Interaction logic for SimilarPatientsPrompt.xaml
    /// </summary>
    public partial class SimilarPatientsPrompt : UserControl
    {
        private IDataAccessService das;
        public List<SimilarPatient> SimilarPatientList { get; set; }

        public Patient SelectedPatient { get; set; }
        public bool BDialogResult { get; set; }

        public SimilarPatientsPrompt(IDataAccessService dataAccessService)
        {
            das = dataAccessService;
            InitializeComponent();
            DataContext = this;
        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            BDialogResult = true;
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            BDialogResult = false;
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void SimilarPatientsSelection_Changed(object sender, SelectionChangedEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Wait;
            var similarPatient = e.AddedItems[0] as SimilarPatient;
            if (similarPatient == null)
            {
                Mouse.OverrideCursor = Cursors.Arrow;
                MessageBox.Show("No Patient Selected");
                return;
            }
            var changePatient = new Patient(das) {IEN = similarPatient.Ien};
            var rslt = UpdatePatientData(ref changePatient);
//            changePatient.Providers = das.GetProviders();
            if (!rslt)
            {
                das.GetPatient(ref changePatient);
//                changePatient.Providers = das.GetProviders();
            }
            SelectedPatient = changePatient;
            Mouse.OverrideCursor = Cursors.Arrow;
        }

        private bool UpdatePatientData(ref Patient patient)
        {
            try
            {
                
                IList<Patient> patObj = das.GetPatients("#" + patient.IEN, 1);
                patient = patObj[0];
                return patient != null;
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo("Similar Patients Check", "", ex.Source + ": " + ex.Message);
                return false;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }
    }
}
