﻿using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.ExpandedEntry.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Modules.ExpandedEntry.Mappings
{
    public static class ExpandedEntryMapping
    {
        public static ExpandedAppointmentProfile MapExpandedEntryFromRPC(IDataAccessService dataAccessService, string dfn, string filemanAppointmentDateTime)
        {
            var expandedAppointmentProfile = new ExpandedAppointmentProfile();

            expandedAppointmentProfile.AppointmentDemographics = 
                mapAppointmentDemographicsFromRPC(dataAccessService.GetExpandedEntryDemographics(dfn, filemanAppointmentDateTime));

            expandedAppointmentProfile.AppointmentEventLog =
                mapAppointmentEventLogFromRPC(dataAccessService.GetExpandedEntryEventLog(dfn, filemanAppointmentDateTime));

            expandedAppointmentProfile.ClinicWaitTimeInfo =
                mapClinicWaitTimeInfoFromRPC(dataAccessService.GetExpandedEntryWaitTime(dfn, filemanAppointmentDateTime));

            expandedAppointmentProfile.PatientInfo =
                mapPatientInfoFromRPC(dataAccessService.GetExpandedPatientInfo(dfn, filemanAppointmentDateTime));

            if (!String.IsNullOrEmpty(expandedAppointmentProfile.AppointmentEventLog.CheckOutDate))
            {
                expandedAppointmentProfile.CheckOutInfo = addCheckoutInfo(dataAccessService, dfn, filemanAppointmentDateTime);
                expandedAppointmentProfile.CheckOutVisibility = System.Windows.Visibility.Visible;
            }
            else
            {
                expandedAppointmentProfile.CheckOutInfo = new CheckOutInformation();
                expandedAppointmentProfile.CheckOutVisibility = System.Windows.Visibility.Collapsed;
            }

            return expandedAppointmentProfile;
        }

        #region HELPERS
        private static AppointmentDemographics mapAppointmentDemographicsFromRPC(string demoRpcString)
        {
            var rpcPieces = demoRpcString.Split('^'); 
            return new AppointmentDemographics
            {
                AppointmentType = rpcPieces[7],
                Clinic = rpcPieces[1],
                CollateralAppointment = rpcPieces[13],
                DateTime = rpcPieces[3],
                EKG = rpcPieces[12],
                EligibilityOfAppointment = rpcPieces[9],
                EnrolledInThisClinic = rpcPieces[15],
                Id = rpcPieces[2],
                Lab = rpcPieces[8],
                LengthOfAppointment = rpcPieces[6],
                Name = rpcPieces[0],
                Other = rpcPieces[14],
                OverBook = rpcPieces[11],
                PurposeOfVisit = rpcPieces[5],
                Status = rpcPieces[4],
                XRay = rpcPieces[10]
            }; 
        }
        private static AppointmentEventLog mapAppointmentEventLogFromRPC(string eventLogRpcString)
        {
            var rpcPieces = eventLogRpcString.Split('^');
            return new AppointmentEventLog
            {
                AppointmentMadeDate = rpcPieces[0],
                AppointmentMadeUser = rpcPieces[1],
                CancelReason = rpcPieces[9],
                CancelRemark = rpcPieces[10],
                CheckInDate = rpcPieces[2],
                CheckInUser = rpcPieces[3],
                CheckOutDate = rpcPieces[4],
                CheckOutEnteredDate = rpcPieces[6],
                CheckOutEnteredUser = "",
                CheckOutUser = rpcPieces[5],
                NoShowCancelDate = rpcPieces[7],
                NoShowCancelUser = rpcPieces[8],
                RebookDate = rpcPieces[11],
                CheckedOut = (string.IsNullOrWhiteSpace(rpcPieces[4])) ? "" : "Yes",
            };
        }
        private static ClinicWaitTimeInformation mapClinicWaitTimeInfoFromRPC(string waitTimeRpcString)
        {
            var rpcPieces = waitTimeRpcString.Split('^');
            return new ClinicWaitTimeInformation
            {
                CIDPreferredDate = rpcPieces[2],
                ClinicWaitTime1 = rpcPieces[4],
                ClinicWaitTime2 = rpcPieces[5],
                FollowUpVisit = rpcPieces[3],
                NextAvailableType = rpcPieces[1],
                RequestType = rpcPieces[0],
            };
        }
        private static PatientInformation mapPatientInfoFromRPC(string patientInfoRpcString)
        {
            var rpcPieces = patientInfoRpcString.Split('^');
            return new PatientInformation
            {
                Address = rpcPieces[7],
                AOExpLoc = rpcPieces[16],
                CellPhoneNumber = rpcPieces[9],
                CombatVeteran = rpcPieces[18],
                CombatVeteranEndDate = rpcPieces[19],
                DateOfBirth = rpcPieces[0],
                EmailAddress = rpcPieces[11],
                Id = rpcPieces[1],
                LastAdmitLodgerDate = rpcPieces[15],
                LastDischargeLodgerDate = rpcPieces[17],
                MaritalStatus = rpcPieces[3],
                PagerNumber = rpcPieces[10],
                PhoneNumber = rpcPieces[8],
                POS = rpcPieces[6],
                PrimaryEligibility = rpcPieces[5],
                PrisonerOfWar = rpcPieces[14],
                Proj112Shad = rpcPieces[20],
                RadiattionExposure = rpcPieces[12],
                ReligiousPreference = rpcPieces[4],
                Sex = rpcPieces[2],
                Status = (string.IsNullOrEmpty(rpcPieces[13])) ? "No Inpatient Status" : rpcPieces[13],
                SWAsiaConditions = rpcPieces[21],
            };            
        }
        private static CheckOutInformation addCheckoutInfo(IDataAccessService dataAccessService, string dfn, string filemanAppointmentDateTime)
        {
            var checkoutInfo = new CheckOutInformation(); 
            checkoutInfo.Diagnosis = mapCheckoutDiagnosisFromRPC(dataAccessService.GetExpandedCheckoutDiagnosis(dfn, filemanAppointmentDateTime));
            checkoutInfo.StopCodes = mapCheckoutStopCodesFromRPC(dataAccessService.GetExpandedCheckoutStopCodes(dfn, filemanAppointmentDateTime));
            checkoutInfo.Procedures = mapCheckoutProceduresFromRPC(dataAccessService.GetExpandedCheckoutProcedures(dfn, filemanAppointmentDateTime));
            mapCheckoutProvidersFromRPC(checkoutInfo, dataAccessService.GetExpandedCheckoutProviders(dfn, filemanAppointmentDateTime));
            mapCheckoutClassificationFromRPC(checkoutInfo, dataAccessService.GetExpandedCheckoutClassifications(dfn, filemanAppointmentDateTime));
            return checkoutInfo;
        }
                private static CheckOutInformation mapCheckoutProvidersFromRPC(CheckOutInformation checkoutInfo, string checkoutProviders)
                {
                    var providers = checkoutProviders.Split('^').ToList();
                    if(providers.Count() > 0)
                        checkoutInfo.Provider1Name = providers[0];
                    if (providers.Count() > 1)
                        checkoutInfo.Provider2Name = providers[1];
                    return checkoutInfo;
                }
                private static List<string> mapCheckoutDiagnosisFromRPC(string checkoutDiagnosisRpcString)
                {
                    var rpcPieces = checkoutDiagnosisRpcString.Split('^');
                    List<string> diagnosisList = new List<string>();

                    foreach (var p in rpcPieces)
                        if (p != "")
                           diagnosisList.Add(p);

                    return diagnosisList;
                }
                private static List<string> mapCheckoutStopCodesFromRPC(string checkoutStopCodeRpcString)
                {
                    var rpcPieces = checkoutStopCodeRpcString.Split('^');
                    List<string> stopCodeList = new List<string>();

                    foreach (var p in rpcPieces)
                        if (p != "")
                            stopCodeList.Add(p);

                    return stopCodeList;
                }
                private static List<string> mapCheckoutProceduresFromRPC(string checkoutProceduresRpcString)
                {
                    var rpcPieces = checkoutProceduresRpcString.Split('^');
                    List<string> proceduresList = new List<string>();

                    foreach (var p in rpcPieces)
                        if (p != "")
                            proceduresList.Add(p);

                    return proceduresList;
                }
                private static CheckOutInformation mapCheckoutClassificationFromRPC(CheckOutInformation checkoutInfo, string patientInfoRpcString)
                {
                    var rpcPieces = patientInfoRpcString.Split('^');
                    checkoutInfo.AgentOrangeExposure = rpcPieces[0];
                    checkoutInfo.CombatVetCombatRelated = rpcPieces[6];
                    checkoutInfo.HeadAndOrNeckCancer = rpcPieces[5];
                    checkoutInfo.IonizingRadiationExposure = rpcPieces[1];
                    checkoutInfo.MilitarySexualTrauma = rpcPieces[4];
                    checkoutInfo.Proj112Shad = rpcPieces[7];
                    checkoutInfo.SWAsiaConditions = rpcPieces[3];
                    checkoutInfo.TreatmentForServiceConnectedCondition = rpcPieces[2];
                    return checkoutInfo;
                }
        #endregion
    }
}
