using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ClinSchd.Modules.FindAppt.FindAppt;
using ClinSchd.Modules.FindAppt.Controllers;
using ClinSchd.Modules.FindAppt.Services;
using ClinSchd.Modules.FindAppt.Tests.Mocks;

namespace ClinSchd.Modules.FindAppt.Tests
{
    /// <summary>
    /// Summary description for FindAppt
    /// </summary>
    [TestClass]
    public class FindApptModuleFixture
    {
        private MockUnityResolver _container;
        private MockFindApptController _controller;

        [TestMethod]
        public void FindApptModuleRegistersFindApptViewAndFindApptService ()
        {
            var findApptModule = CreateTestableFindApptModule ();

            findApptModule.InvokeRegisterViewsAndServices ();

            Assert.AreEqual (typeof (FindApptView), _container.Types[typeof (IFindApptView)]);
            Assert.AreEqual (typeof (FindApptController), _container.Types[typeof (IFindApptController)]);
            Assert.AreEqual (typeof (FindApptPresentationModel), _container.Types[typeof (IFindApptPresentationModel)]);
            Assert.AreEqual (typeof (FindApptService), _container.Types[typeof (IFindApptService)]);
#if !SILVERLIGHT
#endif
        }

        [TestMethod]
        public void InitCallsRunOnChangeDivisionController ()
        {
            var changeDivisionModule = CreateTestableFindApptModule ();

            changeDivisionModule.Initialize ();

            Assert.IsTrue(_controller.RunCalled);
        }

        private TestableFindApptModule CreateTestableFindApptModule ()
        {
            _container = new MockUnityResolver();
            _controller = new MockFindApptController ();

            _container.Bag.Add (typeof (IFindApptController), _controller);
            _container.Bag.Add (typeof (IFindApptView), null);

            return new TestableFindApptModule (_container);
        }

        internal class TestableFindApptModule : FindApptModule
        {
            public TestableFindApptModule (IUnityContainer container)
                : base(container, null, null)
            {
            }

            public void InvokeRegisterViewsAndServices()
            {
                RegisterViewsAndServices();
            }
        }
    }
}
