﻿using System;
using System.ComponentModel;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.FindAppt.FindAppt
{
    public class BookApptPresentationModel : INotifyPropertyChanged
    {
        #region fields

        private SchdAppointment _appointment;
        private DateTime? _desiredDate;
        private AppointmentRequest _request;
        private string _cancelStatus = string.Empty;

        #endregion fields

        #region constructors

        public BookApptPresentationModel(int index, DateTime desiredDate)
        {
            Index = index;
            DesiredDate = desiredDate;
            RequestIsOpen = false;
            RequestIsLocked = false;
            CancelTheAppointment = false;
            CloseTheRequest = false;
        }

        #endregion constructors

        #region public properties

        public SchdAppointment Appointment
        {
            get { return _appointment; }
            set
            {
                _appointment = value;
                OnPropertyChanged("Appointment");
            }
        }

        public bool AppointmentBooked { get { return Appointment != null; } }

        public string CancelStatus
        {
            get { return _cancelStatus;  }
            set
            {
                _cancelStatus = value;
                OnPropertyChanged("CancelStatus");
            }
        }

        public bool CancelTheAppointment { get; set; }

        public bool CanCancelAppointment
        {
            get
            {
                if (Appointment == null)
                    return false;
                return string.IsNullOrEmpty(Appointment.CHECKOUT) && Appointment.NOSHOW == "0" && Appointment.CANCELLED == "0";
            }
        }

        public bool CanCloseRequest
        {
            get
            {
                if (Request == null)
                    return false;
                if (Appointment == null)
                    return true;
                return !string.IsNullOrEmpty(Appointment.CANCELLED);
            }
        }

        public bool CloseTheRequest { get; set; }

        public DateTime? DesiredDate
        {
            get { return _desiredDate; }
            private set
            {
                _desiredDate = value;
                OnPropertyChanged("DesiredDate");
            }
        }

        public string DesiredDateStr { get { return DesiredDate.HasValue ? DesiredDate.Value.ToShortDateString() : string.Empty; } }

        public int Index { get; private set; }

        public AppointmentRequest Request
        {
            get { return _request; }
            set
            {
                _request = value;
                OnPropertyChanged("Request");
            }
        }

        public bool RequestIsLocked { get; set; }

        public bool RequestIsOpen { get; set; }

        #endregion public properties

        #region INotifyPropertyChanged members

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion INotifyPropertyChanged members


    }
}
