using System;
using System.Collections.Generic;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using Telerik.Windows.Controls;

namespace ClinSchd.Modules.FindAppt.FindAppt
{
    /// <summary>
    /// Interaction logic for FindApptView.xaml
    /// </summary>
    public partial class FindApptView : UserControl, IFindApptView
    {
        public bool IsMrtc { get; set; }
        //private bool _selectedClinicIsProhibited = false;
        private const string SEARCH = "Search";
        public DateTime? OriginalDesiredDate { get; set; }

        public FindApptView(IFindApptPresentationModel model)
        {
            Model = (FindApptPresentationModel)model;
            Model.View = this;
            InitializeComponent();
            _cmbClinicsSearchTimer = new Timer(Keys.SearchTimerInterval);
            _cmbClinicsSearchTimer.Elapsed += _cmbClinicsSearchTimer_Elapsed;
        }

        public FindApptPresentationModel Model
        {
            get { return DataContext as FindApptPresentationModel; }
            set { DataContext = value; }
        }

        public bool? DialogResult { get; set; }
        public event EventHandler Closed;
        public bool? ShowDialog()
        {
            throw new NotImplementedException();
        }

        protected void OnClosing (System.ComponentModel.CancelEventArgs e)
        {
        } 

        public void Refresh ()
        {
            SearchResultGrid.Rebind ();
        }

        private bool _bDialogResult;

        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }   

        public void AlertUser (string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        private void FindApptView_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
                Model.BookingStatusView_Show();
        }

        private void Providers_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (BtnSearchOrBook != null)
                BtnSearchOrBook.Content = SEARCH;
        }

        private void ClinicGroups_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (BtnSearchOrBook != null)
                BtnSearchOrBook.Content = SEARCH;
        }

        private void IntervalText_Changed(object sender, TextChangedEventArgs e)
        {
            if (!Keys.Methods.IsNumeric(TxtIntervalBetweenAppointmentsInDays.Text))
            {
                MessageBox.Show("Interval between Appointments is not valid.");
                return;
            }
            if (BtnSearchOrBook != null)
                BtnSearchOrBook.Content = SEARCH;
            Model.AppointmentRequest.AppointmentIntervalInDays = Convert.ToInt32(TxtIntervalBetweenAppointmentsInDays.Text);
        }

        private void NumAppointments_Changed(object sender, TextChangedEventArgs e)
        {
            if(!Keys.Methods.IsNumeric(TxtNumberOfAppointments.Text))
            {
                MessageBox.Show("Number of Appointments provided is not valid.");
                return;
            }
            if (BtnSearchOrBook != null)
                BtnSearchOrBook.Content = SEARCH;
            Model.AppointmentRequest.NumberOfAppointmentsRequired = Convert.ToInt32(TxtNumberOfAppointments.Text);

        }

        private void SpecialNeeds_Click(object sender, RoutedEventArgs e)
        {
            if (BtnSearchOrBook != null)
                BtnSearchOrBook.Content = SEARCH;
        }

        public bool UpdateDesiredDateComboBox(List<DateTime?> desiredDates)
        {
            return true;
        }

        private void BookCheckBox_Click(object sender, RoutedEventArgs e)
        {
            var selectedApptSlot = sender as CheckBox;
        }

        #region CmbClinics members

        private readonly Timer _cmbClinicsSearchTimer;
        public string CmbClinicsSearchString { get; set; }

        void _cmbClinicsSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _cmbClinicsSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => Model.ClinicsStringChanged(CmbClinicsSearchString)));
        }

        private void CmbClinics_KeyUp(object sender, KeyEventArgs e)
        {
            _cmbClinicsSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _cmbClinicsSearchTimer.Start();
        }

        private void CmbClinics_OnLostFocus(object sender, RoutedEventArgs e)
        {
            SetCmbClinicsComboBoxOpen(false);
        }

        private bool _selectionChanging = false;
        private void CmbClinics_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (!_selectionChanging)
                CmbClinicsSearchString = ((RadComboBox) sender).Text;
        }

        public void SetCmbClinicsComboBoxOpen(bool openDropDown)
        {
            CmbClinics.IsDropDownOpen = openDropDown;
        }

        #endregion CmbClinics members

        private void SearchResultGrid_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.None && (e.Key == Key.Space || e.Key == Key.Enter))
            {
                if (SearchResultGrid.HasItems)
                {
                    foreach (var rowItem in Model.FindAppointmentResult)
                    {
                        if (rowItem == SearchResultGrid.SelectedItem)
                        {
                            rowItem.Book = !rowItem.Book;
                        }
                    }
                }
            }
            if (Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Tab)
            {
                e.Handled = true;
                Keyboard.Focus(BtnSearchOrBook);
            }
            if (Keyboard.Modifiers == ModifierKeys.Shift && e.Key == Key.Tab)
            {
                e.Handled = true;
                Keyboard.Focus(CmbDesiredDates);
            }

        }

        private void FocusFirstRowInResults()
        {
            if (SearchResultGrid.HasItems)
            {
                SearchResultGrid.SelectedItem = SearchResultGrid.Items[0];
                SearchResultGrid.ScrollIntoView(SearchResultGrid.SelectedItem);
                SearchResultGrid.Focus();
            }
        }

        private void HotKeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                switch (k)
                {
                    case Key.B:
                        Keyboard.Focus(BtnBook);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnClose);
                        break;
                    case Key.N:
                        Keyboard.Focus(BtnNext);
                        break;
                }
            }
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            Key key = e.Key == Key.System ? e.SystemKey : e.Key;
            HotKeyHandler(key);
        }

        private void BtnBookOrNext_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Shift && e.Key == Key.Tab)
            {
                e.Handled = true;
                FocusFirstRowInResults();
            }
        }

        private void CmbDesiredDates_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Tab)
            {
                e.Handled = true;
                FocusFirstRowInResults();
            }
        }

        private void SpecialInstructions_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (Model.SpecialInstructionsCommand.CanExecute(string.Empty))
                Model.SpecialInstructionsCommand.Execute(string.Empty);
        }

    }
}
