using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Windows;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.FindAppt.CancelMRTC;
using ClinSchd.Modules.DataAccess.Services;
using ClinSchd.Modules.FindAppt.FindAppt;
using Microsoft.Practices.Composite.Events;

namespace ClinSchd.Modules.FindAppt.Services
{
    public class FindApptService : IFindApptService
    {
        #region IFindAppt Members

        public DateTime? OriginalDesiredDate { get; set; }
        private IEventAggregator _eventAggregator;

        public void ShowDialog<FindApptPresentationModel>
            ( IFindApptView view, IEventAggregator eventAggrator, FindApptPresentationModel viewModel, Action onDialogClose )
        {
            _eventAggregator = eventAggrator;
            
            view.DataContext = viewModel;
            if ( onDialogClose != null )
            {
                view.Closed += ( sender, e ) => onDialogClose();
            }

            try
            {
                var findApptDialog = view; // new FindApptView();

                if ( Keys.SelectedAppointmentRequest == null )
                {
                    MessageBox.Show( "Please select a request from the Appointment Request Queue." );
                    return;
                }
                AppointmentRequest request = Keys.SelectedAppointmentRequest;
                OriginalDesiredDate = request.DesiredApptDate;

                if ( request.Clinic == null )
                {
                    IList<Clinic> clinics = view.Model.DataAccessService.GetPtCSchClinics( request.ClinicIen, request.ClinName );

                    if ( clinics.Count > 0 )
                        request.Clinic = clinics[0];
                }

                if ( request.Clinic != null )
                    if (view.Model.DataAccessService.IsProhibitedClinic( request.Clinic.ClinicName, request.Clinic.HospitalLocationId, true ) )
                        return;

                request.Patient.SpecialNeedsPreferences = new SpecialNeedsPreferences();
                //Get Patients SpecialNeeds/Preferences....................................................
                var dasResult = view.Model.DataAccessService.GetPatientSpecialNeedsPreferences( request.Patient.IEN );
                request.Patient.SpecialNeedsPreferences = new SpecialNeedsPreferences();
                foreach ( DataRow dr in dasResult.Rows )
                {
                    if ( dasResult.Columns.Contains( "RETURNCODE" ) )
                        continue;

                    request.Patient.SpecialNeedsPreferences.Remarks               = dr["REMARKS"].ToString();
                    request.Patient.SpecialNeedsPreferences.DateAdded             = dr["DATE_ADDED"].ToString();
                    request.Patient.SpecialNeedsPreferences.AddedByUserIen        = dr["ADDED_BY_USER_IEN"].ToString();
                    request.Patient.SpecialNeedsPreferences.AddedByUserName       = dr["ADDED_BY_USER_NAME"].ToString();
                    request.Patient.SpecialNeedsPreferences.DateInactivated       = dr["DATE_INACTIVATED"].ToString();
                    request.Patient.SpecialNeedsPreferences.InactivatedByUserIen  = dr["INACTIVATED_BY_USER_IEN"].ToString();
                    request.Patient.SpecialNeedsPreferences.InactivatedByUserName = dr["INACTIVATED_BY_USER_NAME"].ToString();

                    request.Patient.Preferences.Add( dr["PREF_TEXT"].ToString() );
                    var prefText = dr["PREF_TEXT"].ToString();
                    switch ( prefText )
                    {
                        case "ON STRETCHER" :
                            request.Patient.SpecialNeedsPreferences.OnStretcher = true;
                            break;
                        case "NEEDS ESCORT" :
                            request.Patient.SpecialNeedsPreferences.NeedsEscort = true;
                            break;
                        case "GENDER SPECIFIC PROVIDER" :
                            request.Patient.SpecialNeedsPreferences.GenderSpecificProvider = true;
                            break;
                        case "HEARING IMPAIRED" :
                            request.Patient.SpecialNeedsPreferences.HearingImpaired = true;
                            break;
                        case "SPECIAL MODE OF TRANSPORTATION" :
                            request.Patient.SpecialNeedsPreferences.SpecialModeOfTransportation = true;
                            break;
                        case "LANGUAGE PREFERENCE" :
                            request.Patient.SpecialNeedsPreferences.LanguagePreference = true;
                            break;
                        case "MORNING" :
                            request.Patient.SpecialNeedsPreferences.Am = true;
                            break;
                        case "AFTERNOON" :
                            request.Patient.SpecialNeedsPreferences.Pm = true;
                            break;
                        case "MONDAY" :
                            request.Patient.SpecialNeedsPreferences.Monday = true;
                            break;
                        case "TUESDAY" :
                            request.Patient.SpecialNeedsPreferences.Tuesday = true;
                            break;
                        case "WEDNESDAY" :
                            request.Patient.SpecialNeedsPreferences.Wednesday = true;
                            break;
                        case "THURSDAY" :
                            request.Patient.SpecialNeedsPreferences.Thursday = true;
                            break;
                        case "FRIDAY" :
                            request.Patient.SpecialNeedsPreferences.Friday = true;
                            break;
                        case "SATURDAY" :
                            request.Patient.SpecialNeedsPreferences.Saturday = true;
                            break;
                    }
                }

                findApptDialog.Model.IsAMChecked   = request.Patient.SpecialNeedsPreferences.Am;
                findApptDialog.Model.IsPMChecked   = request.Patient.SpecialNeedsPreferences.Pm;
                findApptDialog.Model.IsBothChecked = request.Patient.SpecialNeedsPreferences.BothAmPm;
                findApptDialog.Model.IsMonChecked  = request.Patient.SpecialNeedsPreferences.Monday;
                findApptDialog.Model.IsTueChecked  = request.Patient.SpecialNeedsPreferences.Tuesday;
                findApptDialog.Model.IsWedChecked  = request.Patient.SpecialNeedsPreferences.Wednesday;
                findApptDialog.Model.IsThuChecked  = request.Patient.SpecialNeedsPreferences.Thursday;
                findApptDialog.Model.IsFriChecked  = request.Patient.SpecialNeedsPreferences.Friday;
                findApptDialog.Model.IsSatChecked  = request.Patient.SpecialNeedsPreferences.Saturday;

                findApptDialog.Model.AppointmentRequest = request;

                ( (FindApptView)findApptDialog ).OriginalDesiredDate = OriginalDesiredDate;
                CLaunchWindow.LaunchWindow( findApptDialog, "Find Appointment for " + request.Patient.Name, 640, 760, "FindApptDialog", true, true, false, true, ( (FindApptView)findApptDialog ).Background );
                request.DesiredApptDate = OriginalDesiredDate;
            }
            catch ( Exception ex )
            {
                TraceLog.LogException( GetType().Name, "ShowDialog", ex );
                view.AlertUser( "There was a problem opening the Find Appointment dialog.\n" + ex.Message, "Find Appointment" );
                //throw;
            }
        }

        public bool ShowMRTCCancelDialog(IConfirmCloseMRTCRequestPresentationModel model)
        {
            try
            {
                model.View.ShowDialog();
                return model.DialogResult;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "Show MRTC Cancel Dialog", ex);
                Keys.AlertUserError(this, ex, "MRTC Cancel");
                return false;
            }
        }

        public static ObservableCollection<BookApptPresentationModel> LoadMRTCChildRequests(IDataAccessService dataAccessService, AppointmentRequest parentRequest)
        {
            ObservableCollection<BookApptPresentationModel> bookedAppointments = new ObservableCollection<BookApptPresentationModel>();
            // Load the preferred dates, if any
            List<string> prefDates = new List<string>();
            int prefDatesCount = dataAccessService.GetMRTCChildPreferredDates(parentRequest.ApptRequestIen, prefDates);

            if (prefDatesCount > 1)
            {
                foreach (string prefDate in prefDates)
                {
                    DateTime pDate = Convert.ToDateTime(prefDate);
                    BookApptPresentationModel booking = CreateBookingRecord(bookedAppointments, pDate);
                }
            }

            // Load any previously created Child MRTC Requests
            for (int reqIndex = 0; reqIndex < parentRequest.MRTCAppointmentRequestIENs.Count; reqIndex++)
            {
                string reqIEN = parentRequest.MRTCAppointmentRequestIENs[reqIndex];

                if (reqIEN == parentRequest.ApptRequestIen.ToString())
                    continue;

                AppointmentRequest req = dataAccessService.GetApptRequestByRequestIEN(reqIEN);

                if (req != null)
                {
                    BookApptPresentationModel booking = bookedAppointments.FirstOrDefault(br => br.DesiredDate.Value.CompareTo(req.DesiredApptDate.Value) == 0);

                    if (booking == null)
                        booking = CreateBookingRecord(bookedAppointments, req.DesiredApptDate.Value);

                    booking.Request = req;
                }
            }

            // Load any previously create child MRTC Appointments and match them up with their child requests
            List<BookApptPresentationModel> withRequests = bookedAppointments.Where(br => br.Request != null).ToList();

            for (int apptIndex = 0; apptIndex < parentRequest.MRTCAppointmentIENs.Count; apptIndex++)
            {
                string apptIEN = parentRequest.MRTCAppointmentIENs[apptIndex];

                if (!string.IsNullOrEmpty(apptIEN))
                {
                    SchdAppointment appt = dataAccessService.GetAppointmentByAppointmentID(apptIEN);

                    if (appt != null)
                    {

                        BookApptPresentationModel bRecord = withRequests.FirstOrDefault(br => (br.Request.ApptRequestIen.ToString() == appt.APPTREQTYPE_REQIEN));

                        if (bRecord != null)
                        {
                            appt.MRTCParentRequestIEN = bRecord.Request.ApptRequestIen.ToString();
                            appt.AppointmentRequest = bRecord.Request;
                            bRecord.Appointment = appt;
                        }
                    }
                }
            }
            return bookedAppointments;
        }

        public static BookApptPresentationModel CreateBookingRecord(ObservableCollection<BookApptPresentationModel> bookedAppointments, DateTime desiredDate)
        {
            int nextIndex = bookedAppointments.Count + 1;
            BookApptPresentationModel bookingRecord = new BookApptPresentationModel(nextIndex, desiredDate);
            bookedAppointments.Add(bookingRecord);
            return bookingRecord;
        }

        #endregion
    }
}
