using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ClinSchd.Modules.Management.AccessGroups.Controllers;
using ClinSchd.Modules.Management.AccessGroups.Services;
using ClinSchd.Modules.Management.AccessGroups.Tests.Mocks;

namespace ClinSchd.Modules.Management.AccessGroups.Tests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class ManagementAccessGroupsModuleFixture
    {
        private MockUnityResolver _container;
        private MockManagementAccessGroupsController _controller;

        [TestMethod]
        public void ManagementAccessGroupsModuleRegistersManagementAccessGroupsViewAndManagementAccessGroupsService()
        {
            var managementAccessGroupsModule = CreateTestableManagementAccessGroupsModule();

            managementAccessGroupsModule.InvokeRegisterViewsAndServices();

            Assert.AreEqual(typeof(AccessGroupsView), _container.Types[typeof(IAccessGroupsView)]);
            Assert.AreEqual(typeof(ManagementAccessGroupsController), _container.Types[typeof(IManagementAccessGroupsController)]);
            Assert.AreEqual(typeof(AccessGroupsPresentationModel), _container.Types[typeof(IAccessGroupsPresentationModel)]);
            Assert.AreEqual(typeof(ManagementAccessGroupsService), _container.Types[typeof(IManagementAccessGroupsService)]);
#if !SILVERLIGHT
#endif
        }

        [TestMethod]
        public void InitCallsRunOnManagementAccessGroupsController()
        {
            var managementAccessGroupsModule = CreateTestableManagementAccessGroupsModule();

            managementAccessGroupsModule.Initialize();

            Assert.IsTrue(_controller.RunCalled);
        }

        private TestableManagementAccessGroupsModule CreateTestableManagementAccessGroupsModule()
        {
            _container = new MockUnityResolver();
            _controller = new MockManagementAccessGroupsController();

            _container.Bag.Add(typeof(IManagementAccessGroupsController), _controller);
            _container.Bag.Add(typeof(IAccessGroupsView), null);

            return new TestableManagementAccessGroupsModule(_container);
        }

        internal class TestableManagementAccessGroupsModule : ManagementAccessGroupsModule
        {
            public TestableManagementAccessGroupsModule(IUnityContainer container)
                : base(container)
            {
            }

            public void InvokeRegisterViewsAndServices()
            {
                RegisterViewsAndServices();
            }
        }
    }
}
