using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AddAccessGroup.Services;

namespace ClinSchd.Modules.Management.AddAccessGroup
{
    public class AddAccessGroupPresentationModel : IAddAccessGroupPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementAddAccessGroupService _managementAddAccessGroupService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";
        private string _accessTypeGroupName;

        public AddAccessGroupPresentationModel(IAddAccessGroupView view,
            IManagementAddAccessGroupService managementAddAccessGroupService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementAddAccessGroupService = managementAddAccessGroupService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();
        }

        public void AddEditAccessTypeGroup ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            string errorMessage = _dataAccessService.AddEditAccessTypeGroup (AccessTypeGroupID, AccessTypeGroupName);

            if (errorMessage != string.Empty) {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Add/Edit Access Type Group";
                _validationMessage.Message = errorMessage;
            } else {
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("AccessTypeGroups");
            }
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IAddAccessGroupView View { get; }

        public string AccessTypeGroupName
        {
            get { return _accessTypeGroupName; }
            set
            {
                _accessTypeGroupName = value;
                OnPropertyChanged( "AccessTypeGroupName" );
            }
        }

        public string AccessTypeGroupID { get; set; }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value; 
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
