using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Management.AddAccessGroup.Controllers;
using ClinSchd.Modules.Management.AddAccessGroup.Services;

namespace ClinSchd.Modules.Management.AddAccessGroup
{
    public class ManagementAddAccessGroupModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;
        private IManagementAddAccessGroupController _controller;

        public ManagementAddAccessGroupModule (IUnityContainer container, IDataAccessService dataAccessService, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();
            _eventAggregator.GetEvent<LaunchAddAccessGroupDialogEvent> ().Subscribe (LaunchAddAccessGroupDialog, ThreadOption.UIThread, true);
        }

        public void LaunchAddAccessGroupDialog (string title)
        {
            _controller = _container.Resolve<IManagementAddAccessGroupController> ();
            if (title == "Add Access Group")
            {
                _controller.Model.PaneTitle = title;
                _controller.Model.AccessTypeGroupID = string.Empty;
                _controller.Model.AccessTypeGroupName = string.Empty;
            } else {
                _controller.Model.PaneTitle = "Edit Access Group";
                _controller.Model.AccessTypeGroupID = title;
                _controller.Model.AccessTypeGroupName = GetGroupName (title);
            }

            if (_controller.Model.ValidationMessage.IsValid)
            {
                _controller.Run ();
            } else {
                _controller.Model.View.AlertUser (_controller.Model.ValidationMessage.Message, _controller.Model.ValidationMessage.Title);
            }
        }

        private string GetGroupName (string groupID)
        {
            var groupName = string.Empty;
            var g = _dataAccessService.GetAccessGroups ();
            foreach (var group in g)
            {
                if (groupID == group.Value)
                {
                    groupName = group.Name;
                    break;
                }
            }
            return groupName;
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IManagementAddAccessGroupController, ManagementAddAccessGroupController>();
            _container.RegisterType<IAddAccessGroupView, AddAccessGroupView>();
            _container.RegisterType<IAddAccessGroupPresentationModel, AddAccessGroupPresentationModel>();
            _container.RegisterType<IManagementAddAccessGroupService, ManagementAddAccessGroupService>();
        }
    }
}
