using System;
using System.ComponentModel;
using System.Windows.Media;

using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AddAccessType.Services;

namespace ClinSchd.Modules.Management.AddAccessType
{
    public class AddAccessTypePresentationModel : IAddAccessTypePresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementAddAccessTypeService _managementAddAccessTypeService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";
        private bool _isInactiveChecked;
        private bool _isPreventChecked;
        private string _accessTypeColor = "Gray";
        private string _accessTypeName;

        public AddAccessTypePresentationModel(IAddAccessTypeView view,
            IManagementAddAccessTypeService managementAddAccessTypeService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementAddAccessTypeService = managementAddAccessTypeService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            AccessType = new SchdAccessType ();
            InitialErrorMessage ();
        }

        public void LoadEditAccessType ()
        {
            AccessType = _dataAccessService.GetAccessTypeByID (EditAccessTypeID);
            if (AccessType != null)
            {
                AccessTypeName = AccessType.NAME;
                View.ATName.Foreground = new SolidColorBrush (Color.FromRgb (Convert.ToByte (AccessType.RED), Convert.ToByte (AccessType.GREEN), Convert.ToByte (AccessType.BLUE)));
                View.AccessTypeColorPicker.SelectedColor = Color.FromRgb (Convert.ToByte (AccessType.RED), Convert.ToByte (AccessType.GREEN), Convert.ToByte (AccessType.BLUE));
                IsInactiveChecked = AccessType.INACTIVE == "YES";
                IsPreventChecked = AccessType.PREVENT_ACCESS == "YES";
            }

        }

        public void AddEditAccessType ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            AccessType.BSDX_ACCESS_TYPE_IEN = EditAccessTypeID;
            AccessType.NAME                 = AccessTypeName;
            AccessType.INACTIVE             = _isInactiveChecked ? "YES" : "NO";
            AccessType.PREVENT_ACCESS       = _isPreventChecked ? "YES" : "NO";
            AccessType.DISPLAY_COLOR        = AccessTypeColor;
            AccessType.RED                  = View.AccessTypeColorPicker.SelectedColor.R.ToString ();
            AccessType.GREEN                = View.AccessTypeColorPicker.SelectedColor.G.ToString ();
            AccessType.BLUE                 = View.AccessTypeColorPicker.SelectedColor.B.ToString ();

            string errorMessage = _dataAccessService.AddEditAccessType (AccessType);

            if (errorMessage != string.Empty)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Add/Edit Access Type";
                _validationMessage.Message = errorMessage;
            } else {
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("AccessType");
            }
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IAddAccessTypeView View { get; }
        public string EditAccessTypeID { get; set; }

        public string AccessTypeName
        {
            get { return _accessTypeName; }
            set
            {
                _accessTypeName = value;
                OnPropertyChanged( "AccessTypeName" );
            }
        }

        public string AccessTypeColor
        {
            get { return _accessTypeColor; }
            set
            {
                if ( _accessTypeColor != value )
                {
                    _accessTypeColor = value;
                    OnPropertyChanged( "AccessTypeColor" );
                }
            }
        }

        public bool IsInactiveChecked
        {
            get { return _isInactiveChecked; }
            set
            {
                _isInactiveChecked = value;
                OnPropertyChanged( "IsInactiveChecked" );
            }
        }

        public bool IsPreventChecked
        {
            get { return _isPreventChecked; }
            set
            {
                _isPreventChecked = value;
                OnPropertyChanged( "IsPreventChecked" );
            }
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public SchdAccessType AccessType { get; set; }

        public void OnClose ()
        {
            View.Close ();
        }

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
