using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Modules.Management.AddAccessType.Controllers;
using ClinSchd.Modules.Management.AddAccessType.Services;

namespace ClinSchd.Modules.Management.AddAccessType
{
    public class ManagementAddAccessTypeModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private IManagementAddAccessTypeController _controller;

        public ManagementAddAccessTypeModule (IUnityContainer container, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();
            _eventAggregator.GetEvent<LaunchAddAccessTypeDialogEvent> ().Subscribe (LaunchAddAccessTypeDialog, ThreadOption.UIThread, true);
        }

        public void LaunchAddAccessTypeDialog (string title)
        {
            _controller = _container.Resolve<IManagementAddAccessTypeController> ();
            if (title == "Add Access Type")
            {
                _controller.Model.PaneTitle = title;
                _controller.Model.EditAccessTypeID = string.Empty;
            } else {
                _controller.Model.PaneTitle = "Edit Access Type";
                _controller.Model.EditAccessTypeID = title;
                _controller.Model.LoadEditAccessType ();
            }

            if (_controller.Model.ValidationMessage.IsValid) {
                _controller.Run ();
            } else {
                _controller.Model.View.AlertUser (_controller.Model.ValidationMessage.Message, _controller.Model.ValidationMessage.Title);
            }
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IManagementAddAccessTypeController, ManagementAddAccessTypeController>();
            _container.RegisterType<IAddAccessTypeView, AddAccessTypeView>();
            _container.RegisterType<IAddAccessTypePresentationModel, AddAccessTypePresentationModel>();
            _container.RegisterType<IManagementAddAccessTypeService, ManagementAddAccessTypeService>();
        }
    }
}
