using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AddResource.Services;

namespace ClinSchd.Modules.Management.AddResource
{
    public class AddResourcePresentationModel : IAddResourcePresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementAddResourceService _managementAddResourceService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private IList<RPMSClinic> _clinicList;
        private RPMSClinic _selectedClinic;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";
        private bool _isClinicEnabled = true;
        private IList<TimeScaleModel> _timeScale;
        private string _timeScaleValue;
        private string _hospitalLocationID;
        private bool _isInactive;
        private string _editClinicID;

        public AddResourcePresentationModel(IAddResourceView view,
            IManagementAddResourceService managementAddResourceService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementAddResourceService = managementAddResourceService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            SchdResource = new SchdResource ();
            InitialErrorMessage ();
        }

        public void LoadEditClinic ()
        {
            //this.SchdResource = this.dataAccessService.GetResource (this.EditClinicID);
            SchdResource = _dataAccessService.GetSdecResource(EditClinicID);
            SelectedClinic = SchdResource.Clinic;
            HospitalLocationID = SelectedClinic.HOSPITAL_LOCATION_ID;
            TimeScaleValue = SchdResource.Clinic.Duration.ToString();
            //this.TimeScaleValue = this.SchdResource.TIMESCALE;
            IsInactive = SchdResource.INACTIVE == "1";
        }

        public void LoadClinicSetupParemters ()
        {
            SelectedClinic = _dataAccessService.GetClinicByID (HospitalLocationID);
            TimeScaleValue = SelectedClinic.Duration.ToString();
        }

        public void LoadTimeScale ()
        {
            TimeScale = Keys.TimeScaleList;
            TimeScaleValue = "30";
/*
            List<NameValue> timeScale = new List<NameValue>();
            timeScale.Insert (0, new NameValue ("30", "30"));
            timeScale.Insert (0, new NameValue ("20", "20"));
            timeScale.Insert (0, new NameValue ("15", "15"));
            timeScale.Insert (0, new NameValue ("10", "10"));
            this.TimeScale = timeScale;

            IList<NameValue> TimeScaleList = new List<NameValue>()
            {
                { new NameValue("10", "0") },
                { new NameValue("15", "1") },
                { new NameValue("20", "2") },
                { new NameValue("30", "3") }
            };
*/
        }

        public void AddEditClinic ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            SchdResource.Clinic = SelectedClinic;
            SchdResource.RESOURCEID = EditClinicID;
            SchdResource.TIMESCALE = TimeScaleValue;
            if (IsInactive)
            {
                SchdResource.INACTIVE = "YES";
                //var waitListModel = new ResourceWaitlistPresentationModel (new ResourceWaitlistView (), _dataAccessService, SchdResource.RESOURCEID);
                _eventAggregator.GetEvent<CloseSchedulesEvent> ().Publish (SelectedClinic.HOSPITAL_LOCATION_ID);
            } else {
                SchdResource.INACTIVE = "NO";
            }

            string errorMessage = _dataAccessService.AddEditClinic (SchdResource);

            if (errorMessage != string.Empty)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Add/Edit Clinic";
                _validationMessage.Message = errorMessage;
            } else {
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("Resources");
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("ResourceInGroups");
            }
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IAddResourceView View { get; }
        public Patient Patient { get; set; }

        public string HospitalLocationID
        {
            get { return _hospitalLocationID; }
            set
            {
                _hospitalLocationID = value; 
                OnPropertyChanged( "HospitalLocationID" );
            }
        }

        public string EditClinicID
        {
            get { return _editClinicID; }
            set
            {
                _editClinicID = value; 
                OnPropertyChanged( "EditClinicID" );
            }
        }

        public IList<TimeScaleModel> TimeScale
        {
            get { return _timeScale; }
            set
            {
                _timeScale = value; 
                OnPropertyChanged( "TimeScale" );
            }
        }

        public string TimeScaleValue
        {
            get { return _timeScaleValue; }
            set
            {
                _timeScaleValue = value;
                OnPropertyChanged( "TimeScaleValue" );
            }
        }

        public SchdResource SchdResource { get; set; }

        public bool IsInactive
        {
            get { return _isInactive; }
            set
            {
                _isInactive = value;
                OnPropertyChanged( "IsInactive" );
            }
        }

        public string ReminderLetter { get; set; }
        public string RebookLetter { get; set; }
        public string CancellationLetter { get; set; }

        public bool IsClinicEnabled
        {
            get { return _isClinicEnabled; }
            set
            {
                _isClinicEnabled = value;
                OnPropertyChanged( "IsClinicEnabled" );
            }
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public IList<RPMSClinic> ClinicList
        {
            get { return _clinicList; }
            set
            {
                if ( _clinicList != value )
                {
                    _clinicList = value;
                    OnPropertyChanged( "ClinicList" );
                }
            }
        }

        public RPMSClinic SelectedClinic
        {
            get { return _selectedClinic; }
            private set
            {
                if ( _selectedClinic != value )
                {
                    _selectedClinic = value;
                    OnPropertyChanged( "SelectedClinic" );
                }
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
