using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace ClinSchd.Modules.Management.AddResource
{
    /// <summary>
	/// Interaction logic for AddResourceView.xaml
    /// </summary>
	public partial class AddResourceView : Window, IAddResourceView
    {
		public AddResourceView()
        {
            InitializeComponent();
            Owner = Infrastructure.Keys.MainWindow;
        }

		public AddResourcePresentationModel Model
        {
            get
            {
				return this.DataContext as AddResourcePresentationModel;
            }

            set
            {
                this.DataContext = value;
            }
        }

		protected override void OnClosing (System.ComponentModel.CancelEventArgs e)
		{
			this.Visibility = Visibility.Hidden;
			e.Cancel = true;
		}

		private void CancelButton_Click (object sender, RoutedEventArgs e)
		{
			Close ();
		}

		private void OkButton_Click (object sender, RoutedEventArgs e)
		{
			this.Model.AddEditClinic ();
			if (this.Model.ValidationMessage.IsValid) {
				Close ();
			} else {
				this.Model.View.AlertUser (this.Model.ValidationMessage.Message, this.Model.ValidationMessage.Title);
			}
		}

		private bool bDialogResult = false;
		public bool ConfirmUser (string message, string caption)
		{
            bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return bDialogResult;
        }   

		public void AlertUser (string message, string caption)
		{
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

		private void RPMSClinicName_SelectionChanged (object sender, SelectionChangedEventArgs e)
		{
			if (this.Model.HospitalLocationID != null) {
				this.Model.LoadClinicSetupParemters ();
			}
		}

        private void paneManagementAddResource_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }
    }
}
