using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Modules.Management.AddResource.Controllers;
using ClinSchd.Modules.Management.AddResource.Services;

namespace ClinSchd.Modules.Management.AddResource
{
    public class ManagementAddResourceModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private IManagementAddResourceController _controller;

        public ManagementAddResourceModule (IUnityContainer container, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();

            _eventAggregator.GetEvent<LaunchAddResourceDialogEvent> ().Subscribe (LaunchAddResourceDialog, ThreadOption.UIThread, true);
        }

        public void LaunchAddResourceDialog (string title)
        {
            _controller = _container.Resolve<IManagementAddResourceController> ();

            if (title == "Add Clinic")
            {
                _controller.Model.PaneTitle = title;
                _controller.Model.IsClinicEnabled = true;
                _controller.Model.EditClinicID = string.Empty;
            } else {
                _controller.Model.PaneTitle = "Edit Clinic";
                _controller.Model.IsClinicEnabled = false;
                _controller.Model.EditClinicID = title;
                _controller.Model.LoadEditClinic ();
            }

            if (_controller.Model.ValidationMessage.IsValid)
            {
                _controller.Run ();
                _controller.Model.ValidationMessage.IsValid = true;
                _controller.Model.ValidationMessage.Title = string.Empty;
                _controller.Model.ValidationMessage.Message = string.Empty;

            } else {
                _controller.Model.View.AlertUser (_controller.Model.ValidationMessage.Message, _controller.Model.ValidationMessage.Title);
            }
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IManagementAddResourceController, ManagementAddResourceController>();
            _container.RegisterType<IAddResourceView, AddResourceView>();
            _container.RegisterType<IAddResourcePresentationModel, AddResourcePresentationModel>();
            _container.RegisterType<IManagementAddResourceService, ManagementAddResourceService>();
        }
    }
}
