using System.Collections.Generic;
using System.ComponentModel;
using System.Data;

using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Management.AddResource
{
    public class ResourceWaitlistPresentationModel : INotifyPropertyChanged
    {
        //private readonly IManagementAddResourceService ManagementAddResourceService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private IList<RPMSClinic> _clinicList;
        private RPMSClinic _selectedClinic;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";
        private bool _isClinicEnabled = true;
        private string _hospitalLocationID;
        private string _timeScaleValue;
        private bool _isInactive;
        private IList<NameValue> _timeScale;

        public DataRowCollection WaitingList { get; set; }

        public ResourceWaitlistPresentationModel (IResourceWaitlistView view, IDataAccessService dataAccessService, EventAggregator eventAggregator, string resourceIEN)
        {
            View = view;
            View.Model = this;
            //this.ManagementAddResourceService = ManagementAddResourceService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            SchdResource = new SchdResource ();
            InitialErrorMessage ();

            WaitingList = _dataAccessService.GetWaitingList (resourceIEN).Rows;
            LoadClinics ();
            if (WaitingList.Count > 0) {
                View.ShowDialog ();
            }
        }

        public void LoadClinics ()
        {
        }

        public void LoadEditClinic()
        {
            SchdResource = _dataAccessService.GetSdecResource( EditClinicID );

            SelectedClinic     = SchdResource.Clinic;
            HospitalLocationID = SelectedClinic.HOSPITAL_LOCATION_ID;
            TimeScaleValue     = SchdResource.TIMESCALE;
            IsInactive         = SchdResource.INACTIVE == "1";
        }

        public void LoadClinicSetupParemters ()
        {
            _selectedClinic = _dataAccessService.GetClinicByID (HospitalLocationID);
            SelectedClinic.ParentResource = SchdResource;
        }

        public void LoadTimeScale ()
        {
            List<NameValue> timeScale = new List<NameValue>();
            timeScale.Insert (0, new NameValue ("60", "60"));
            timeScale.Insert (0, new NameValue ("30", "30"));
            timeScale.Insert (0, new NameValue ("20", "20"));
            timeScale.Insert (0, new NameValue ("15", "15"));
            timeScale.Insert (0, new NameValue ("10", "10"));
            timeScale.Insert (0, new NameValue ("5", "5"));
            TimeScaleValue = timeScale[2].Value;
            TimeScale = timeScale;
        }

        public void AddEditClinic ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            SchdResource.Clinic = SelectedClinic;
            SchdResource.RESOURCEID = EditClinicID;
            SchdResource.TIMESCALE = TimeScaleValue;
            if (IsInactive)
            {
                SchdResource.INACTIVE = "YES";

                _eventAggregator.GetEvent<CloseSchedulesEvent> ().Publish (SelectedClinic.HOSPITAL_LOCATION_ID);
            } else {
                SchdResource.INACTIVE = "NO";
            }

            string errorMessage = _dataAccessService.AddEditClinic (SchdResource);

            if (errorMessage != string.Empty)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Add/Edit Clinic";
                _validationMessage.Message = errorMessage;
            } else {
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("Resources");
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("ResourceInGroups");
            }
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IResourceWaitlistView View { get; }
        public Patient Patient { get; set; }

        public string HospitalLocationID
        {
            get { return _hospitalLocationID; }
            set
            {
                _hospitalLocationID = value;
                OnPropertyChanged( "HospitalLocationID" );
            }
        }

        public string EditClinicID { get; set; }

        public IList<NameValue> TimeScale
        {
            get { return _timeScale; }
            set
            {
                _timeScale = value;
                OnPropertyChanged( "TimeScale" );
            }
        }

        public string TimeScaleValue
        {
            get { return _timeScaleValue; }
            set
            {
                _timeScaleValue = value;
                OnPropertyChanged( "TimeScaleValue" );
            }
        }

        public SchdResource SchdResource { get; set; }

        public bool IsInactive
        {
            get { return _isInactive; }
            set
            {
                _isInactive = value;
                OnPropertyChanged( "IsInactive" );
            }
        }

        public string ReminderLetter { get; set; }
        public string RebookLetter { get; set; }
        public string CancellationLetter { get; set; }

        public bool IsClinicEnabled
        {
            get { return _isClinicEnabled; }
            set
            {
                _isClinicEnabled = value;
                OnPropertyChanged( "IsClinicEnabled" );
            }
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public IList<RPMSClinic> ClinicList
        {
            get { return _clinicList; }
            set
            {
                if (_clinicList != value)
                {
                    _clinicList = value;
                    OnPropertyChanged ("ClinicList");
                }
            }
        }

        public RPMSClinic SelectedClinic
        {
            get { return _selectedClinic; }
            private set
            {
                if ( _selectedClinic != value )
                {
                    _selectedClinic = value;
                    OnPropertyChanged( "SelectedClinic" );
                }
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
