using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AddResourceGroup.Services;

namespace ClinSchd.Modules.Management.AddResourceGroup
{
    public class AddResourceGroupPresentationModel : IAddResourceGroupPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementAddResourceGroupService _managementAddResourceGroupService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";
        private string _clinicGroupName;

        public AddResourceGroupPresentationModel(IAddResourceGroupView view,
            IManagementAddResourceGroupService managementAddResourceGroupService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementAddResourceGroupService = managementAddResourceGroupService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();
        }

        public void AddEditClinicGroup ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            string errorMessage = _dataAccessService.AddEditClinicGroup (ClinicGroupID, ClinicGroupName);

            if (errorMessage != string.Empty)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Add/Edit Clinic Group";
                _validationMessage.Message = errorMessage;
            } else {
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("ResourceGroups");
            }
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IAddResourceGroupView View { get; }

        public string ClinicGroupName
        {
            get { return _clinicGroupName; }
            set
            {
                _clinicGroupName = value;
                OnPropertyChanged( "ClinicGroupName" );
            }
        }

        public string ClinicGroupID { get; set; }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
