using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Management.AddResourceGroup.Controllers;
using ClinSchd.Modules.Management.AddResourceGroup.Services;

namespace ClinSchd.Modules.Management.AddResourceGroup
{
    public class ManagementAddResourceGroupModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;
        private IManagementAddResourceGroupController _controller;

        public ManagementAddResourceGroupModule (IUnityContainer container, IDataAccessService dataAccessService, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();
            _eventAggregator.GetEvent<LaunchAddResourceGroupDialogEvent> ().Subscribe (LaunchAddResourceGroupDialog, ThreadOption.UIThread, true);
        }

        public void LaunchAddResourceGroupDialog (string title)
        {
            _controller = _container.Resolve<IManagementAddResourceGroupController> ();

            if (title == "Add Clinic Group")
            {
                _controller.Model.PaneTitle = title;
                _controller.Model.ClinicGroupID = string.Empty;
                _controller.Model.ClinicGroupName = string.Empty;
            } else {
                _controller.Model.PaneTitle = "Edit Clinic Group";
                _controller.Model.ClinicGroupID = title;
                _controller.Model.ClinicGroupName = GetGroupName (title);
            }

            if (_controller.Model.ValidationMessage.IsValid)
            {
                _controller.Run ();
            } else {
                _controller.Model.View.AlertUser (_controller.Model.ValidationMessage.Message, _controller.Model.ValidationMessage.Title);
            }
        }

        private string GetGroupName (string groupID)
        {
            var groupName = string.Empty;
            var g = _dataAccessService.GetResourceGroupList ();
            foreach (var group in g)
            {
                if (groupID == group.Value)
                {
                    groupName = group.Name;
                    break;
                }
            }
            return groupName;
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IManagementAddResourceGroupController, ManagementAddResourceGroupController>();
            _container.RegisterType<IAddResourceGroupView, AddResourceGroupView>();
            _container.RegisterType<IAddResourceGroupPresentationModel, AddResourceGroupPresentationModel>();
            _container.RegisterType<IManagementAddResourceGroupService, ManagementAddResourceGroupService>();
        }
    }
}
