using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AddResourceUser.Services;

namespace ClinSchd.Modules.Management.AddResourceUser
{
    public class AddResourceUserPresentationModel : IAddResourceUserPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementAddResourceUserService _managementAddResourceUserService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private ResourceUser _resourceUser;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";
        private bool _isUpdateChecked;
        private bool _isModifyChecked;
        private string _overbookValue;
        private IList<NameValue> _overBookAuthority;
        private string _resourceUserName;

        public AddResourceUserPresentationModel(IAddResourceUserView view,
            IManagementAddResourceUserService managementAddResourceUserService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementAddResourceUserService = managementAddResourceUserService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            ResourceUser = new ResourceUser ();
            InitialErrorMessage ();

            LoadOverBookAuthority ();
        }

        public void AddEditResourceUser ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            if (OverbookValue == "0")
            {
                ResourceUser.OVERBOOK = "NO";
                ResourceUser.MASTEROVERBOOK = "NO";
            } else if (OverbookValue == "1") {
                ResourceUser.OVERBOOK = "YES";
                ResourceUser.MASTEROVERBOOK = "NO";
            } else {
                ResourceUser.OVERBOOK = "NO";
                ResourceUser.MASTEROVERBOOK = "YES";
            }
            ResourceUser.MODIFY_APPTS = IsUpdateChecked ? "YES" : "NO";
            ResourceUser.MODIFY_SCHEDULE = IsModifyChecked ? "YES" : "NO";

            var errorMessage = _dataAccessService.AddEditResourceUser (ResourceUser);

            if (errorMessage != string.Empty)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Add/Edit Resource User";
                _validationMessage.Message = errorMessage;
            } else {
                _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Publish ("ResourceUser");
            }
        }

        public void LoadOverBookAuthority ()
        {
            var overBookAuthority = new List<NameValue> ();
            overBookAuthority.Insert (0, new NameValue ("Master", "2"));
            overBookAuthority.Insert (0, new NameValue ("Regular", "1"));
            overBookAuthority.Insert (0, new NameValue ("None", "0"));
            OverbookValue = overBookAuthority[0].Value;
            OverBookAuthority = overBookAuthority;
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IAddResourceUserView View { get; }

        public string ResourceUserName
        {
            get { return _resourceUserName; }
            set
            {
                _resourceUserName = value;
                OnPropertyChanged( "ResourceUserName" );
            }
        }

        public string OverbookValue
        {
            get { return _overbookValue; }
            set
            {
                _overbookValue = value;
                OnPropertyChanged( "OverbookValue" );
            }
        }

        public IList<NameValue> OverBookAuthority
        {
            get { return _overBookAuthority; }
            set
            {
                _overBookAuthority = value;
                OnPropertyChanged( "OverBookAuthority" );
            }
        }

        public bool IsUpdateChecked
        {
            get { return _isUpdateChecked; }
            set
            {
                _isUpdateChecked = value;
                OnPropertyChanged( "IsUpdateChecked" );
            }
        }

        public bool IsModifyChecked
        {
            get { return _isModifyChecked; }
            set
            {
                _isModifyChecked = value;
                OnPropertyChanged( "IsModifyChecked" );
            }
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public ResourceUser ResourceUser
        {
            get { return _resourceUser; }
            set
            {
                if ( _resourceUser != value )
                {
                    _resourceUser = value;
                    OnPropertyChanged( "ResourceUser" );
                }
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
