using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.ComponentModel;
using ClinSchd.Infrastructure;

namespace ClinSchd.Modules.Management.AddResourceUser
{
    /// <summary>
    /// Interaction logic for AddResourceUserView.xaml
    /// </summary>
    public partial class AddResourceUserView : IAddResourceUserView
    {
        public AddResourceUserView()
        {
            InitializeComponent();
            Owner = Keys.MainWindow;
        }

        public AddResourceUserPresentationModel Model
        {
            get { return DataContext as AddResourceUserPresentationModel; }
            set { DataContext = value; }
        }

        protected override void OnClosing (CancelEventArgs e)
        {
            Visibility = Visibility.Hidden;
            e.Cancel = true;
        }

        private void CancelButton_Click (object sender, RoutedEventArgs e)
        {
            Close ();
        }

        private void OkButton_Click (object sender, RoutedEventArgs e)
        {
            Model.AddEditResourceUser ();
            if (Model.ValidationMessage.IsValid)
            {
                Close ();
            } else {
                Model.View.AlertUser (Model.ValidationMessage.Message, Model.ValidationMessage.Title);
            }
        }

        private bool _bDialogResult;
        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }   

        public void AlertUser (string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        private void Overbook_SelectionChanged (object sender, SelectionChangedEventArgs e)
        {
            switch (Model.OverbookValue) {
            case "0":
                Model.IsUpdateChecked = false;
                Model.IsModifyChecked = false;
                break;
            case "1":
            case "2":
                Model.IsUpdateChecked = true;
                break;
            }
        }

        private void UpdateAppointment_Checked (object sender, RoutedEventArgs e)
        {
            if (Model.IsUpdateChecked)
            {
                if (Model.OverbookValue == "0")
                {
                    Model.OverbookValue = "1";
                }
                Model.IsModifyChecked = true;
            } 
        }

        private void ModifyScheduler_Checked (object sender, RoutedEventArgs e)
        {
            if (Model.IsModifyChecked)
            {
                if (Model.OverbookValue == "0")
                {
                    Model.OverbookValue = "1";
                }
            }
            Model.IsUpdateChecked = true;
        }

        private void paneManagementAddResourceUser_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            OkButton.Focus();
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }
    }
}
