using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.AddResourceUser.Controllers;
using ClinSchd.Modules.Management.AddResourceUser.Services;

namespace ClinSchd.Modules.Management.AddResourceUser
{
    public class ManagementAddResourceUserModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private IManagementAddResourceUserController _controller;

        public ManagementAddResourceUserModule (IUnityContainer container, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();

            _eventAggregator.GetEvent<LaunchAddResourceUserDialogEvent> ().Subscribe (LaunchAddResourceUserDialog, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<AddEditResourceUserEvent> ().Subscribe (LoadResourceUser, ThreadOption.UIThread, true);
        }

        public void LaunchAddResourceUserDialog( string title )
        {
            _controller = _container.Resolve<IManagementAddResourceUserController>();

            if ( title == "Add Clinic User" )
                _controller.Model.PaneTitle = title;
            else
                _controller.Model.PaneTitle = "Edit Clinic User";

            if ( _controller.Model.ValidationMessage.IsValid )
                _controller.Run();
            else
                _controller.Model.View.AlertUser( _controller.Model.ValidationMessage.Message, _controller.Model.ValidationMessage.Title );
        }

        public void LoadResourceUser (ResourceUser r)
        {
            _controller.Model.ResourceUserName = r.USERNAME;
            _controller.Model.ResourceUser.RESOURCE_ID = r.RESOURCE_ID;
            _controller.Model.ResourceUser.USER_ID = r.USER_ID;
            if (_controller.Model.PaneTitle != "Add Clinic User")
            {
                _controller.Model.ResourceUser = r;

                if ( r.OVERBOOK == "YES" )
                    _controller.Model.OverbookValue = "1";
                else if ( r.MASTEROVERBOOK == "YES" )
                    _controller.Model.OverbookValue = "2";
                else
                    _controller.Model.OverbookValue = "0";
            }
            _controller.Model.IsUpdateChecked = r.MODIFY_APPTS== "YES";
            _controller.Model.IsModifyChecked = r.MODIFY_SCHEDULE == "YES";
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IManagementAddResourceUserController, ManagementAddResourceUserController>();
            _container.RegisterType<IAddResourceUserView, AddResourceUserView>();
            _container.RegisterType<IAddResourceUserPresentationModel, AddResourceUserPresentationModel>();
            _container.RegisterType<IManagementAddResourceUserService, ManagementAddResourceUserService>();
        }
    }
}
