using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Modules.Management.ReactivateResource.Controllers;
using ClinSchd.Modules.Management.ReactivateResource.Services;

namespace ClinSchd.Modules.Management.ReactivateResource
{
    public class ManagementReactivateResourceModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private IManagementReactivateResourceController _controller;

        public ManagementReactivateResourceModule (IUnityContainer container, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();

            _eventAggregator.GetEvent<LaunchReactivateResourceDialogEvent> ().Subscribe (LaunchReactivateResourceDialog, ThreadOption.UIThread, true);
        }

        public void LaunchReactivateResourceDialog (string title)
        {
            _controller = _container.Resolve<IManagementReactivateResourceController>();
            _controller.Model.PaneTitle = "Activate Clinic(s)";
            _controller.Model.EditClinicID = title;

            if (_controller.Model.ValidationMessage.IsValid)
            {
                _controller.Run();
                _controller.Model.ValidationMessage.IsValid = true;
                _controller.Model.ValidationMessage.Title = string.Empty;
                _controller.Model.ValidationMessage.Message = string.Empty;
            }
            else
            {
                _controller.Model.View.AlertUser(_controller.Model.ValidationMessage.Message, _controller.Model.ValidationMessage.Title);
            }
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IManagementReactivateResourceController, ManagementReactivateResourceController>();
            _container.RegisterType<IReactivateResourceView, ReactivateResourceView>();
            _container.RegisterType<IReactivateResourcePresentationModel, ReactivateResourcePresentationModel>();
            _container.RegisterType<IManagementReactivateResourceService, ManagementReactivateResourceService>();
        }
    }
}
