using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;

using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.ReactivateResource.Services;

namespace ClinSchd.Modules.Management.ReactivateResource
{
    public class ReactivateResourcePresentationModel : IReactivateResourcePresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementReactivateResourceService _managementReactivateResourceService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private IList<SchdResource> _clinicList;
        private IList<SchdResource> _selectedClinicList;
        private ValidationMessage _validationMessage;
        private string _paneTitle = "";

        public ReactivateResourcePresentationModel(IReactivateResourceView view,
            IManagementReactivateResourceService managementReactivateResourceService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _managementReactivateResourceService = managementReactivateResourceService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            InitialErrorMessage ();

            LoadClinics ();
        }

        public void LoadClinics ()
        {
//            this.ClinicList = this.dataAccessService.GetResourcesByType("H", true).Where(c => (c.INACTIVE == "YES")).ToList();
            SelectedClinicList = new List<SchdResource>();
        }

        public void ActivateClinics()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
            List<string> errorMessages = new List<string>();

//            foreach (RPMSClinic clinic in this.SelectedClinicList)
            foreach (SchdResource schdResource in SelectedClinicList)
            {
//                SchdResource schdResource = this.dataAccessService.GetResource (this.EditClinicID);

//			    schdResource.Clinic = clinic;
//			    schdResource.RESOURCEID = this.EditClinicID;
                schdResource.INACTIVE = "NO";
                string errorMessage = _dataAccessService.AddEditClinic(schdResource);

                if (errorMessage != string.Empty)
                    errorMessages.Add("Error activating " + schdResource.Clinic.HOSPITAL_LOCATION + ": " + errorMessage);
            }

            if (errorMessages.Count > 0)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Activate Clinic";
                _validationMessage.Message = errorMessages.Aggregate((i, j) => i + "\r\n" + j);
            }

            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Publish("Resources");
            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Publish("ResourceInGroups");
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public IReactivateResourceView View { get; }
        public string EditClinicID { get; set; }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );

            }
        }

        public ValidationMessage ValidationMessage
        {
            get
            {
                return _validationMessage;
            }
            set
            {
                _validationMessage = value;
            }
        }
        public IList<SchdResource> ClinicList
        {
            get
            {
                return _clinicList;
            }
            private set
            {
                if (_clinicList != value) {
                    _clinicList = value;
                    OnPropertyChanged ("ClinicList");
                }
            }
        }
        public IList<SchdResource> SelectedClinicList
        {
            get
            {
                return _selectedClinicList;
            }
            private set
            {
                if (_selectedClinicList != value) {
                    _selectedClinicList = value;
                    OnPropertyChanged ("SelectedClinicList");
                }
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
