using System.Windows;
using System.Windows.Input;

using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Management.ReactivateResource
{
    /// <summary>
	/// Interaction logic for ReactivateResourceView.xaml
    /// </summary>
	public partial class ReactivateResourceView : Window, IReactivateResourceView
    {
		public ReactivateResourceView()
        {
            InitializeComponent();
            Owner = Infrastructure.Keys.MainWindow;
        }

		public ReactivateResourcePresentationModel Model
        {
            get
            {
				return this.DataContext as ReactivateResourcePresentationModel;
            }

            set
            {
                this.DataContext = value;
            }
        }

		protected override void OnClosing (System.ComponentModel.CancelEventArgs e)
		{
			this.Visibility = Visibility.Hidden;
			e.Cancel = true;
		}

		private void CancelButton_Click (object sender, RoutedEventArgs e)
		{
			Close ();
		}

		private void OkButton_Click (object sender, RoutedEventArgs e)
		{
            if (RPMSClinicList.SelectedItems.Count == 0)
            {
				this.Model.View.AlertUser ("Select at least one (1) clinic to activate", "Activate Clinic(s)");
            }
            else
            {
//                foreach (RPMSClinic item in RPMSClinicList.SelectedItems)
                foreach (SchdResource item in RPMSClinicList.SelectedItems)                    
                    this.Model.SelectedClinicList.Add(item);

                this.Model.ActivateClinics();

                if (!this.Model.ValidationMessage.IsValid)
				    this.Model.View.AlertUser (this.Model.ValidationMessage.Message, this.Model.ValidationMessage.Title);

                bDialogResult = true;
                Close ();
			}
		}

		private bool bDialogResult = false;
		public bool ConfirmUser (string message, string caption)
		{
            bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return bDialogResult;
        }

		public void AlertUser (string message, string caption)
		{
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

        private void paneManagementReactivateResource_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }
    }
}
