using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using Telerik.Windows.Controls;
using System.Windows.Input;

namespace ClinSchd.Modules.Management.ResourceGroups
{
    /// <summary>
	/// Interaction logic for ResourceGroupsView.xaml
    /// </summary>
	public partial class ResourceGroupsView : RadPane, IResourceGroupsView
    {
		public ResourceGroupsView()
        {
            InitializeComponent();
        }

		public ResourceGroupsPresentationModel Model
        {
            get
            {
				return this.DataContext as ResourceGroupsPresentationModel;
            }

            set
            {
                this.DataContext = value;
            }
        }

		private bool bDialogResult = false;
		public bool ConfirmUser (string message, string caption)
		{
            bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return bDialogResult;
        }   

		public void AlertUser (string message, string caption)
		{
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

		private void NewGroup_Click (object sender, RoutedEventArgs e)
		{
			this.Model.ExecuteAddResourceGroupCommand(string.Empty);
		}

		private void EditGroup_Click (object sender, RoutedEventArgs e)
		{
			this.Model.ExecuteEditResourceGroupCommand (string.Empty);
		}

		private void RemoveGroup_Click (object sender, RoutedEventArgs e)
		{
			this.Model.RemoveGroupByName ();
		}

		private void ClinicGroupListBox_SelectionChanged (object sender, SelectionChangedEventArgs e)
		{
			this.Model.LoadGroupedClinics ();
		}

		private void AddClinic_Click (object sender, RoutedEventArgs e)
		{
			this.Model.AddClinicToGroup ();
		}

		private void AddAll_Click (object sender, RoutedEventArgs e)
		{
			this.Model.AddAllClinicsToGroup ();
		}

		private void RemoveClinic_Click (object sender, RoutedEventArgs e)
		{
			this.Model.RemoveGroupedClinic ();
		}

		private void RemoveAll_Click (object sender, RoutedEventArgs e)
		{
			this.Model.RemoveAllGroupedClinics ();
		}

        private void FindResource_Click(object sender, RoutedEventArgs e)
        {
            if (String.IsNullOrEmpty(ResourceSearch.Text))
                return;

            this.Model.LoadClinics(ResourceSearch.Text);
        }

        private void paneManagementResourceGroups_PreviewKeyUp(object sender, System.Windows.Input.KeyEventArgs e)
        {
            Key key = (e.Key == Key.System ? e.SystemKey : e.Key);
            hotKeysHandler(key);
        }

        private void hotKeysHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                
                switch (k)
                {
                    case Key.N :
                        Keyboard.Focus(NewGroup);
                        break;
                    case Key.E:
                        Keyboard.Focus(EditGroup);
                        break;
                    case Key.R:
                        Keyboard.Focus(RemoveGroup);
                        break;
                    case Key.F:
                        Keyboard.Focus(FindResource);
                        break;
                    case Key.S:
                        Keyboard.Focus(AddClinic);
                        break;
                    case Key.L:
                        Keyboard.Focus(AddAll);
                        break;
                    case Key.D:
                        Keyboard.Focus(RemoveClinic);
                        break;
                    case Key.X:
                        Keyboard.Focus(RemoveAll);
                        break;
                    
                }
            }
        }

        private void paneManagementResourceGroups_Loaded(object sender, RoutedEventArgs e)
        {              
            ResourceSearch.Focus();
            Keyboard.Focus(ResourceSearch);
        }

        public void SchedulingManagementUserGrpHotKeys(Key k)
        {
            hotKeysHandler(k);
        }

        public void SchedulingManagementUserGrpTabToggle()
        {
            ResourceSearch.Focus();
            Keyboard.Focus(ResourceSearch);
        }

        public void SetResourceSearchFocus()
        {
            ResourceSearch.Focus();
            Keyboard.Focus(ResourceSearch);
        }
    }
}
