using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Management.Resources
{
    /// <summary>
    /// Interaction logic for ResourcesView.xaml
    /// </summary>
    public partial class ResourcesView : IResourcesView
    {
        public ResourcesView()
        {
            InitializeComponent();            
        }

        public ResourcesPresentationModel Model
        {
            get { return DataContext as ResourcesPresentationModel; }
            set { DataContext = value; }
        }

        private bool _bDialogResult;
        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }   

        public void AlertUser (string message, string caption)
        {
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

        private void AddClinic_Click (object sender, RoutedEventArgs e)
        {
            Model.ExecuteAddResourceCommand (string.Empty);
        }

        private void EditClinic_Click (object sender, RoutedEventArgs e)
        {
            Model.ExecuteEditResourceCommand (string.Empty);
        }

/*		private void RemoveClinic_Click (object sender, RoutedEventArgs e)
        {
            this.Model.ExecuteDeleteResourceCommand (string.Empty);
        }//*/

        private void EditUser_Click (object sender, RoutedEventArgs e)
        {
            Model.ExecuteEditUserCommand (string.Empty);
        }

        private void AddUser_Click (object sender, RoutedEventArgs e)
        {
            Model.ExecuteAddUserCommand (string.Empty);
        }

        private void ListBox_SelectionChanged (object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                var prohibClin = e.AddedItems[0] as ProhibitedClinic;
                if (prohibClin == null)
                    Model.LoadResourceUsers();
                else
                    Model.LoadResourceUsers(prohibClin.Ien);
            }
        }

        private void RemoveUser_Click (object sender, RoutedEventArgs e)
        {
            Model.RemoveResourceUser ();
        }

        private void RemoveAll_Click (object sender, RoutedEventArgs e)
        {
            Model.RemoveAllResourceUsers ();
        }

        private void ResourceUserListBox_SelectionChanged (object sender, SelectionChangedEventArgs e)
        {
            if (Model.ResourceUserIEN != null)
            {
                Model.ResourceUserSelectionChanged ();
            }
        }

        private void UserListBox_SelectionChanged (object sender, SelectionChangedEventArgs e)
        {
            if (Model.UserIEN != null)
            {
                Model.UserSelectionChanged ();
            }
        }

        private void ReactivateClinic_Click(object sender, RoutedEventArgs e)
        {
            Model.ExecuteReactivateResourceCommand(string.Empty);
        }

        private void FindClinic_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(ClinicSearch.Text))
                return;

            Model.LoadResources(ClinicSearch.Text);

            if (Model.ProhibitedClinicsFound == 0)
                Model.ResourceUserList = null;
        }

        private void hotKeysHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {   
                switch(k)
                {
                    case Key.A:
                        Keyboard.Focus(AddUser);
                        break;
                    case Key.R:
                        Keyboard.Focus(RemoveUser);
                        break;
                    case Key.E:
                        Keyboard.Focus(RemoveAll);
                        break;
                    case Key.F:
                        Keyboard.Focus(FindClinic);
                        break;
                    case Key.S:
                        Keyboard.Focus(ClinicSearch);
                        break;

                }
            }
        }

        private void UserListBox_Loaded(object sender, RoutedEventArgs e)
        {
            //var item = UserListBox.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;
            //if ( item != null )
            //{
            //    item.Focus();
            //    Keyboard.Focus(item);
            //}
            ClinicSearch.Focus();
            Keyboard.Focus(ClinicSearch);
        }

        public void SchedulingManagementUserHotKeys(Key k)
        {
            hotKeysHandler(k);
        }

        public void SchedulingManagementUserTabToggle()
        {
            ClinicSearch.Focus();
            Keyboard.Focus(ClinicSearch);
        }

        private void paneManagementResources_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            var key = e.Key == Key.System ? e.SystemKey : e.Key;
            hotKeysHandler(key);
        }

        public void SetProhibitedClinicFocus()
        {
            ClinicSearch.Focus();
            Keyboard.Focus(ClinicSearch);
        }
    }
}
