using System.ComponentModel;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Management.Controllers;
using ClinSchd.Modules.Management.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

namespace ClinSchd.Modules.Management.Group
{
    public class GroupPresentationModel : IGroupPresentationModel, INotifyPropertyChanged
    {
        private readonly IManagementService _managementService;
        public readonly IEventAggregator EventAggregator;
        public IDataAccessService DataAccessService;

        public GroupPresentationModel (IGroupView view, IManagementService managementService, IEventAggregator eventAggregator, IDataAccessService dataAccessService)
        {
            View = view;
            View.Model = this;
            DataAccessService = dataAccessService;
            _managementService = managementService;
            EventAggregator = eventAggregator;
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public IGroupView View { get; }

        public IManagementController Controller { get; set; }
        public CompositePresentationEvent<RPMSClinic> NewForwardingEvent { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
