using System;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.Management.Views;

namespace ClinSchd.Modules.Management.Group
{
    /// <summary>
    /// Interaction logic for GroupView.xaml
    /// </summary>
    public partial class GroupView : IGroupView
    {
        public GroupView()
        {
            InitializeComponent();
            Owner = Keys.MainWindow;            
        }

        public GroupPresentationModel Model
        {
            get
            {
                return DataContext as GroupPresentationModel;
            }

            set
            {
                DataContext = value;
            }
        }

        public event EventHandler<EventArgs> ShowManagement;

        protected override void OnClosing (System.ComponentModel.CancelEventArgs e)
        {
            Visibility = Visibility.Hidden;
            e.Cancel = true;
            Model.EventAggregator.GetEvent<ManagementItemAddedEvent>().Publish("");
        }

        private void Close_Click(object sender, RoutedEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Wait;

            try
            {
                Model.EventAggregator.GetEvent<RefreshProhibitedClinicsEvent>().Publish(string.Empty);
                Close();
                var updateResourceUsers = new UpdateResourceUsersView(Model.DataAccessService);
                CLaunchWindow.LaunchWindow(updateResourceUsers, "", 200, 400, "UpdateResourceUsers", false, true, false, false);
                //Keys.Tables.ProhibitedClinicsAndUsers = Model.DataAccessService.GetProhibitedClincs();
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Wait;

            try
            {
                Model.EventAggregator.GetEvent<RefreshProhibitedClinicsEvent>().Publish(string.Empty);
                Close();
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OKButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                    case Key.G:
                        ManagementGroupRegionGroup.SelectedIndex = 1;
                        break;
                    case Key.U:
                        ManagementGroupRegionGroup.SelectedIndex = 0;
                        break;
                    default :
                        if (ManagementGroupRegionGroup.SelectedPane.Name.Equals("paneManagementResources"))
                        {
                            Model.EventAggregator.GetEvent<SchedulingManagementUserHotKeys>().Publish(key);
                        }
                        else if (ManagementGroupRegionGroup.SelectedPane.Name.Equals("paneManagementResourceGroups"))
                        {
                            Model.EventAggregator.GetEvent<SchedulingManagementUserGrpHotKeys>().Publish(key);
                        }
                        break;
                }

            }
        }

        private void ManagementGroupRegionGroup_SelectionChanged(object sender, RoutedEventArgs e)
        {
            string paneName = ManagementGroupRegionGroup.SelectedPane.Name;

            if (paneName.Equals("paneManagementResources"))
            {
                Model.EventAggregator.GetEvent<SchedulingManagementUserTabToggle>().Publish("");
            }
            else if (paneName.Equals("paneManagementResourceGroups"))
            {
                Model.EventAggregator.GetEvent<SchedulingManagementUserGrpTabToggle>().Publish("");
            }
        }
    }
}
