using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using ClinSchd.Infrastructure;
using ClinSchd.Modules.Management.Group;
using ClinSchd.Modules.Management.Controllers;
using ClinSchd.Modules.Management.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

namespace ClinSchd.Modules.Management
{
    public class ManagementModule : IModule
    {
        private readonly IUnityContainer container;
        private readonly IEventAggregator eventAggregator;

        public ManagementModule (IUnityContainer container, IEventAggregator eventAggregator)
        {
            this.container = container;
            this.eventAggregator = eventAggregator;

        }

        public void Initialize()
        {
            RegisterViewsAndServices();

            eventAggregator.GetEvent<LaunchSchedulingManagementDialogEvent> ().Subscribe (LaunchSchedulingManagementDialog, ThreadOption.UIThread, true);

        }

        public void LaunchSchedulingManagementDialog (string title)
        {
            IManagementController controller = container.Resolve<IManagementController> ();
            controller.Run ();
        }

        protected void RegisterViewsAndServices()
        {
            container.RegisterType<IManagementController, ManagementController> (new ContainerControlledLifetimeManager ());
            container.RegisterType<IGroupView, GroupView>();
            container.RegisterType<IGroupPresentationModel, GroupPresentationModel>();
            container.RegisterType<IManagementService, ManagementService> (new ContainerControlledLifetimeManager ());
        }
    }
}
