using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ClinSchd.Modules.MarkAsNoShow.MarkAsNoShow;
using ClinSchd.Modules.MarkAsNoShow.Controllers;
using ClinSchd.Modules.MarkAsNoShow.Services;
using ClinSchd.Modules.MarkAsNoShow.Tests.Mocks;

namespace ClinSchd.Modules.MarkAsNoShow.Tests
{
	/// <summary>
	/// Summary description for UnitTest1
	/// </summary>
	[TestClass]
	public class MarkAsNoShowModuleFixture
	{
		private MockUnityResolver _container;
		private MockMarkAsNoShowController _controller;

		[TestMethod]
		public void MarkAsNoShowModuleRegistersMarkAsNoShowViewAndMarkAsNoShowService ()
		{
			var markAsNoShowModule = CreateTestableMarkAsNoShowModule ();

			markAsNoShowModule.InvokeRegisterViewsAndServices ();

			Assert.AreEqual (typeof (MarkAsNoShowView), _container.Types[typeof (IMarkAsNoShowView)]);
			Assert.AreEqual (typeof (MarkAsNoShowController), _container.Types[typeof (IMarkAsNoShowController)]);
			Assert.AreEqual (typeof (MarkAsNoShowPresentationModel), _container.Types[typeof (IMarkAsNoShowPresentationModel)]);
			Assert.AreEqual (typeof (MarkAsNoShowService), _container.Types[typeof (IMarkAsNoShowService)]);
#if !SILVERLIGHT
#endif
		}

		[TestMethod]
		public void InitCallsRunOnMarkAsNoShowController ()
		{
			var markAsNoShowModule = CreateTestableMarkAsNoShowModule ();

			markAsNoShowModule.Initialize ();

			Assert.IsTrue(_controller.RunCalled);
		}

		private TestableMarkAsNoShowModule CreateTestableMarkAsNoShowModule ()
		{
			_container = new MockUnityResolver();
			_controller = new MockMarkAsNoShowController ();

			_container.Bag.Add (typeof (IMarkAsNoShowController), _controller);
			_container.Bag.Add (typeof (IMarkAsNoShowView), null);

			return new TestableMarkAsNoShowModule (_container);
		}

		internal class TestableMarkAsNoShowModule : MarkAsNoShowModule
		{
			public TestableMarkAsNoShowModule (IUnityContainer container)
				: base(container, null)
			{
			}

			public void InvokeRegisterViewsAndServices()
			{
				RegisterViewsAndServices();
			}
		}
	}
}
