using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure.Events;

namespace ClinSchd.Modules.MarkAsNoShow.MarkAsNoShow
{
    /// <summary>
    /// Interaction logic for ChangeDivisionView.xaml
    /// </summary>
    public partial class MarkAsNoShowView : IMarkAsNoShowView
    {
        public MarkAsNoShowView ()
        {
            InitializeComponent();
            Owner = Infrastructure.Keys.MainWindow;
        }

        public MarkAsNoShowPresentationModel Model
        {
            get
            {
                return DataContext as MarkAsNoShowPresentationModel;
            }

            set
            {
                DataContext = value;
            }
        }

        protected override void OnClosing (System.ComponentModel.CancelEventArgs e)
        {
        } 

        private void Button_OK_Click(object sender, RoutedEventArgs e)
        {
            Model.ExecuteNoShowAppointmentCommand ("Mark As NoShow Appointment");
            if (Model.ValidationMessage.IsValid)
            {
                if (ConfirmUser("Print the Patient Letter?", "Print Letter?"))
                    Model.PrintLetterCommand.Execute("");

                var waitListIen = Model.DataAccessService.ReopenWaitListRecordAfterCancelAppointment(
                    Model.SelectedAppointment.APPTREQTYPE_REQTYPE,
                    Model.SelectedAppointment.APPTREQTYPE_REQIEN,
                    Model.SelectedAppointment.APPOINTMENTID,
                    Model.SelectedAppointment.PATIENTNAME,
                    Model.SelectedAppointment.PATIENTID,
                    Model.SelectedAppointment.DesiredAppointmentDate.HasValue ? Model.SelectedAppointment.DesiredAppointmentDate.Value.ToShortDateString() : string.Empty);
                Model.EventAggregator.GetEvent<RefreshWaitlistEvent>().Publish(waitListIen);
                Close();
            }
            else if (!Model.ValidationMessage.IsValid && Model.ValidationMessage.Title == "AutoRebook Appointment")
            {
                Model.View.AlertUser (Model.ValidationMessage.Message, Model.ValidationMessage.Title);
                Close ();
            }
            else
            {
                Model.View.AlertUser (Model.ValidationMessage.Message, Model.ValidationMessage.Title);
            }
        }

        private void Button_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private bool _bDialogResult;
        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }   

        public void AlertUser (string message, string caption)
        {
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

        private void AnyType_Checked (object sender, RoutedEventArgs e)
        {
            Model.AutoRebook.IsAccessTypeChecked = false;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OkButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }

    }
}
