using System;
using System.Collections.Generic;
using System.ComponentModel;
using ClinSchd.Infrastructure.Events;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Navigation.Controllers;
using ClinSchd.Modules.Navigation.Services;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Windows.Threading;
using System.Linq;
using System.Windows.Input;

namespace ClinSchd.Modules.Navigation.Group
{
    public class GroupPresentationModel : IGroupPresentationModel, INotifyPropertyChanged
    {
        private readonly INavigationService _navigationService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private readonly IDataAccessService _dataAccessService;
        private bool _patientSelectedByMe;
        //private bool _resourceListStale;
        private bool _resourceGroupListStale;
        //private bool _providerListStale;
        public string ScheduleRequestType { get; set; }

        private readonly Dispatcher _dispatcher;
        private readonly Factory<SchdResource> _schdResourceFactory;
        public System.Data.DataTable WaitList { get; set; }
        //private bool byPass = false;
        //private int _ctr;
        private DateTime _selectedDate = DateTime.Today;

        public DateTime SelectedDate
        {
            get { return _selectedDate; }
            set
            {
                _selectedDate = value;
                OnPropertyChanged( "SelectedDate" );
            }
        }

        private DateTime _displayDate = DateTime.Today;
        public DateTime DisplayDate
        {
            get { return _displayDate; }
            set
            {
                _displayDate = value;
                OnPropertyChanged("DisplayDate");
            }
        }

        private DateTime _displayDateStart = DateTime.MinValue; //new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);
        public DateTime DisplayDateStart
        {
            get { return _displayDateStart; }
            set
            {
                _displayDateStart = value;
                OnPropertyChanged("DisplayDateStart");
            }
        }

        private DateTime _displayDateEnd = DateTime.MaxValue;  // new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1).AddMonths(1).AddDays(-1);
        public DateTime DisplayDateEnd
        {
            get { return _displayDateEnd; }
            set
            {
                _displayDateEnd = value;
                OnPropertyChanged("DisplayDateEnd");
            }
        }

        public GroupPresentationModel()
        {
            
        }

        public GroupPresentationModel(IGroupView view,
            INavigationService navigationService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator,
            Factory<SchdResource> schdResourceFactory,
            Factory<SchdResourceGroup> schdResourceGroupFactory,
            Factory<Provider> providerFactory)
        {
            _dispatcher = Dispatcher.CurrentDispatcher;
            View = view;
            View.Model = this;
            _navigationService = navigationService;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
            AdmittedPatients = new ObservableCollection<Patient> ();
            Resource = schdResourceFactory.Create ();
            ResourceGroup = schdResourceGroupFactory.Create ();
            Provider = providerFactory.Create ();
            _schdResourceFactory = schdResourceFactory;

            SchedulerDisplayGroupCommand = new DelegateCommand<SchdResourceGroup> (ExecuteSchedulerDisplayGroupCommand, CanExecuteSchedulerDisplayGroupCommand);
            SchedulerDisplayCommand = new DelegateCommand<SchdResource> (ExecuteSchedulerDisplayCommand, CanExecuteSchedulerDisplayCommand);
            SchedulerDisplayProviderCommand = new DelegateCommand<Provider> (ExecuteSchedulerDisplayProviderCommand, CanExecuteSchedulerDisplayProviderCommand);

            _eventAggregator.GetEvent<ResourceSelectedEvent> ().Subscribe (ResourceSelected, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<ProviderSelectedEvent> ().Subscribe (ProviderSelected, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<ResourceGroupSelectedEvent> ().Subscribe (ResourceGroupSelected, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<ManagementItemAddedEvent> ().Subscribe (RefreshDataSources, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<SchedulerViewDateEvent>().Subscribe(ViewDate, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<ResourceSelectedFromRmGrid>().Subscribe(ResourceSelectedFromGrid, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<SchedulerDisplayProviderEvent>().Subscribe(ExecuteSchedulerDisplayProviderCommandForClinicGroups, ThreadOption.UIThread, false);
            _eventAggregator.GetEvent<UnselectClinicEvent>().Subscribe(UnselectClinic, ThreadOption.UIThread, false);
            eventAggregator.GetEvent<ResetCalendarEvent>().Subscribe(ResetCalendar, ThreadOption.UIThread, false);
            eventAggregator.GetEvent<GroupViewHotKeyEvents>().Subscribe(HotKeyHandler, ThreadOption.UIThread, false);
        }

        private void ResetCalendar(DateTime dt)
        {
            SelectedDate = dt.Date;
            DisplayDate = dt.Date;
        }

        private void UnselectClinic(string s)
        {
            View.UnselectClinic(s);
            Resource = null;
            ResourceList = null;

            if ("CLEAR-REFRESH".Contains(s))
            {
                Provider = null;
                ProviderList = null;
                ResourceGroup = null;
                ResourceGroupList = null;
            }
        }

        private void SetDates(DateTime newDate)
        {
            View.Use_Calendar_SelectionChanged(false);

            try
            {
/*
                DateTime startDate = new DateTime(newDate.Year, newDate.Month, 1);
                DateTime endDate = startDate.AddMonths(1).AddDays(-1);

                if (startDate > this.DisplayDateEnd)
                {
                    this.DisplayDateEnd = endDate;
                    this.DisplayDateStart = startDate;
                }
                else
                {
                    this.DisplayDateStart = startDate;
                    this.DisplayDateEnd = endDate;
                }
*/
                SelectedDate = newDate;
                DisplayDate = newDate;
            }
            finally
            {
                View.Use_Calendar_SelectionChanged(true);
            }
        }

        private void ViewDate(SchedulerViewDateEventArgs dateToView)
        {
            if (dateToView.Sender != "Navigation.GroupPresentationModel" && (SelectedDate.Date != dateToView.NewDate.Date))
            {
                SetDates(dateToView.NewDate);
            }
        }

        public void PatientSelected (Patient selectedPatient)
        {
            if ( selectedPatient == null )
                return;

            if (!_patientSelectedByMe)
            {
                var patientExistsInWorkspace = false;
                foreach (var item in AdmittedPatients)
                {
                    if (item.IEN == selectedPatient.IEN)
                    {
                        if (!string.IsNullOrEmpty (selectedPatient.IEN))
                        {
                            patientExistsInWorkspace = true;
                            selectedPatient = item;
                            break;
                        }
                        if (item.Name == selectedPatient.Name)
                        {
                            patientExistsInWorkspace = true;
                            selectedPatient = item;
                            break;
                        }
                    }
                }
                if (!patientExistsInWorkspace)
                {
                    AdmittedPatients.Add (selectedPatient);
                }
                SelectedPatient = selectedPatient;
                OnPropertyChanged ("SelectedPatient");
            } else {
                _patientSelectedByMe = false;
            }
        }

        public bool AreDropDownsOpen { get; set; }
        private void CollapseComboBoxes ()
        {
            AreDropDownsOpen = false;
            OnPropertyChanged ("AreDropDownsOpen");
        }

        public void SelectPatient (Patient selectedPatient)
        {
            _patientSelectedByMe = true;
            _eventAggregator.GetEvent<PatientSelectedEvent> ().Publish (selectedPatient);
        }

        public void ResourceSelected (SchdResource selectedResource)
        {
            Resource = selectedResource;
            CollapseComboBoxes ();
            OnPropertyChanged ("Resource");
        }

        public void ResourceSelectedFromGrid(SchdResource res)
        {
            ResourceList = new List<SchdResource> { res };
            Resource = res;
        }
        private void ProviderSelected (Provider provider)
        {
            Provider = provider;
            CollapseComboBoxes ();
        }

        public void ResourceGroupSelected (SchdResourceGroup selectedResourceGroup)
        {
            ResourceGroup = selectedResourceGroup;
            CollapseComboBoxes ();
        }

        public void RemovePatientFromWorkspace (Patient selectedPatient)
        {
            AdmittedPatients.Remove (selectedPatient);
            SelectedPatient = null;
        }

        public void ViewDayRange (DateTime dayToView)
        {
            _eventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("Navigation.GroupPresentationModel", dayToView));
        }

        private void RefreshDataSources (string arg)
        {
            /* This will force the lists to refresh when the next search is done */
            //_resourceListStale = true;
            _resourceGroupListStale = true;
            //_providerListStale = true;

            var resource = Resource;
            var resourceGroup = ResourceGroup;
            var provider = Provider;

            if (resource != null && !string.IsNullOrEmpty(resource.RESOURCE_NAME) && !string.IsNullOrEmpty(resource.RESOURCEID) && ResourceList != null)
            {
                ResourceStringChanged(resource.RESOURCE_NAME.Substring(0, 5));
                var newRes = ResourceList.FirstOrDefault(r => r.RESOURCEID == resource.RESOURCEID);
                View.UpdateSelectedResource(newRes, true);
            }

            if (provider != null && !string.IsNullOrEmpty(provider.Name) && !string.IsNullOrEmpty(provider.IEN))
            {
                ProviderStringChanged(provider.Name.Substring(0, 5));
                var newProv = ProviderList.First(p => p.IEN == provider.IEN);
                View.UpdateSelectedProvider(newProv);
            }

            if (resourceGroup != null && !string.IsNullOrEmpty(resourceGroup.Name) && !string.IsNullOrEmpty(resourceGroup.IEN))
            {
                ResourceGroupStringChanged(resourceGroup.Name.Substring(0, 5));
                var newGroup = ResourceGroupList.FirstOrDefault(r => r.IEN == resourceGroup.IEN);
                View.UpdateSelectedResourceGroup(newGroup);
            }
        }

        public ObservableCollection<Patient> AdmittedPatients { get; set; }
        public Patient SelectedPatient { get; set; }

        public IGroupView View { get; }

        public SchdResource Resource
        {
            get { return _resource; }
            set
            {
                _resource = value;
                OnPropertyChanged( "Resource" );
            }
        }

        private IList<SchdResource> _resourceList;

        public IList<SchdResource> ResourceList
        {
            get { return _resourceList1; }
            set
            {
                _resourceList1 = value;
                OnPropertyChanged( "ResourceList" );
            }
        }

        public SchdResourceGroup ResourceGroup
        {
            get { return _resourceGroup; }
            set
            {
                _resourceGroup = value; 
                OnPropertyChanged( "ResourceGroup" );
            }
        }

        private IList<SchdResourceGroup> _resourceGroupList;

        public IList<SchdResourceGroup> ResourceGroupList
        {
            get { return _resourceGroupList1; }
            set
            {
                _resourceGroupList1 = value;
                OnPropertyChanged( "ResourceGroupList" );
            }
        }

        public Provider Provider
        {
            get { return _provider; }
            set
            {
                _provider = value;
                OnPropertyChanged( "Provider" );
            }
        }

        private IList<Provider> _providerList;

        public IList<Provider> ProviderList
        {
            get { return _providerList1; }
            set
            {
                _providerList1 = value; 
                OnPropertyChanged( "ProviderList" );
            }
        }

        public INavigationController Controller { get; set; }

        public DelegateCommand<Provider> SchedulerDisplayProviderCommand { get; private set; }
        public bool ExecutingSchedulerDisplayProviderCommand;

        public void ExecuteSchedulerDisplayProviderCommand (Provider param)
        {
            var loadingID = Guid.NewGuid();
            ExecutingSchedulerDisplayProviderCommand = true;

            param.Resources = _dataAccessService.GetClinicsByProvider(param.IEN);
            Keys.ScheduleRequestType = "Provider";
            var providerGroup = new SchdResourceGroup( _dataAccessService )
            {
                LoadingID = loadingID,
                Name      = param.Name,
                Resources = param.Resources
            };
            _eventAggregator.GetEvent<ProviderSelectedEvent> ().Publish (param);
            if ( providerGroup.Resources.Count > 0 )
                _eventAggregator.GetEvent<SchedulerDisplayForProvidersEvent>().Publish( providerGroup );
            else
                _eventAggregator.GetEvent<SchedulerDisplayEmptyPaneEvent>().Publish( "There are no Clinics associated with " + param.Name + "." );
        }

        public void ExecuteSchedulerDisplayProviderCommandForClinicGroups(Provider param)
        {
            var providerGroup = new SchdResourceGroup( _dataAccessService )
            {
                Name = param.Name,
                Resources = param.Resources
            };
            //_eventAggregator.GetEvent<ProviderSelectedEvent>().Publish(param);
            if ( providerGroup.Resources.Count > 0 )
                _eventAggregator.GetEvent<SchedulerDisplayGroupEvent>().Publish( providerGroup );
            else
                _eventAggregator.GetEvent<SchedulerDisplayEmptyPaneEvent>().Publish( "There are no Clinics associated with " + param.Name + "." );
        }


        public bool CanExecuteSchedulerDisplayProviderCommand (Provider param)
        {
            return true;
        }

        public DelegateCommand<SchdResource> SchedulerDisplayCommand { get; private set; }
        public bool ExecutingSchedulerDisplayCommand;

        public void ExecuteSchedulerDisplayCommand (SchdResource param)
        {
            if ( Keys.SelectedAppointmentRequest != null && Keys.SelectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch )
                if ( !string.IsNullOrEmpty( param.HOSPITAL_LOCATION_ID ) && param.Clinic == null )
                {
                    Keys.AlertUser( View,
                        "Clinic Schedule for selected PtCSch Request cannot be displayed because the " +
                        "assigned clinic does not have a PtCSch Reminder Letter defined.",
                        "PtCSch Request Clinic Selection"
                    );
                    return;
                }

            param.LoadingID = Guid.NewGuid();
            ExecutingSchedulerDisplayCommand = true;
            Keys.ScheduleRequestType = "Clinic";
            /* rhh added */
            if (Keys.SelectedAppointmentRequest == null && !Keys.RequestKeys.IsPendingApptSelected)
            {
                ViewDayRange(DateTime.Now.Date);
                Keys.RequestKeys.CurrentSelectedDesiredDate = DateTime.Now.Date.ToShortDateString();
            }

            Keys.RequestKeys.IsPendingApptSelected = false;
            _eventAggregator.GetEvent<SchedulerDisplayEvent>().Publish(param);
            _eventAggregator.GetEvent<ResourceSelectedEvent> ().Publish (param);
            _eventAggregator.GetEvent<FocusSchedulerEvent> ().Publish (null);
        }

        public bool CanExecuteSchedulerDisplayCommand (SchdResource resourceID)
        {
            return true;
        }

        public DelegateCommand<SchdResourceGroup> SchedulerDisplayGroupCommand { get; private set; }
        public bool ExecutingSchedulerDisplayGroupCommand;

        public void ExecuteSchedulerDisplayGroupCommand (SchdResourceGroup param)
        {
            param.LoadingID = Guid.NewGuid();
            ExecutingSchedulerDisplayGroupCommand = true;

            if (Keys.SelectedAppointmentRequest == null)
            {
                ViewDayRange(DateTime.Now.Date);
                Keys.RequestKeys.CurrentSelectedDesiredDate = DateTime.Now.Date.ToShortDateString();
            }

            ResourceGroup = param;

            Keys.ScheduleRequestType = "ClinicGroups";
            _eventAggregator.GetEvent<ResourceGroupSelectedEvent> ().Publish (param);
            _eventAggregator.GetEvent<SchedulerDisplayGroupEvent> ().Publish (param);
        }

        public bool CanExecuteSchedulerDisplayGroupCommand (SchdResourceGroup param)
        {
            return true;
        }	
        
        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }

        public void ResourceStringChanged(string newSearchString)
        {
            if (newSearchString == null)
                return;

            if (newSearchString.Length < Keys.ClinicHResourceSearchCharMin)
                return;

            _resourceList = _dataAccessService.SearchSdecHResources(newSearchString);
            //_resourceListStale = false;

            if (_resourceList != null)
            {
                ResourceList = _resourceList;

                if (ResourceList.Count == 1)
                {
                    View.UpdateSelectedResource(ResourceList[0]);
                    ExecuteSchedulerDisplayCommand(ResourceList[0]);
                }
                else if (ResourceList.Count > 1)
                {
                    string lastSub = ResourceList[ResourceList.Count - 1].LastSub;

                    if ((ResourceList.Count == Keys.ClinicResourceSearchMaxRecords) && !string.IsNullOrEmpty(lastSub))
                    {
                        Keys.AlertUser(View, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "Clinic Search Criteria");
                        View.SetResourceComboBoxFocus();
                    }
                }

                View.SetResourceComboBoxOpen(ResourceList.Count > 1);
            }
        }

        private string _resourceVeryLastSub = string.Empty;
        private bool _resourceScrollJustRan = true;

        public void ResourceScrollChanged(string newSearchString, System.Windows.Controls.ScrollViewer scrollViewer)
        {
            var vOffSet = scrollViewer.VerticalOffset;

            if (!_resourceScrollJustRan)
            {
                _resourceScrollJustRan = !_resourceScrollJustRan;
                return;
            }

            if (ResourceList != null && ResourceList.Count > 0)
            {
                var lastSub = ResourceList[ResourceList.Count - 1].LastSub;
                // save previous very last sub for use in testing
                var veryLastSub = _resourceVeryLastSub;
                // save the current very last sub for future testing
                _resourceVeryLastSub = lastSub;

                // test using the saved previous last sub
                if (veryLastSub == lastSub)
                    return;

                var moreResources = _dataAccessService.SearchSdecHResources(newSearchString, lastSub);

                if (moreResources != null && moreResources.Count > 0)
                {
                    foreach (var moreResource in moreResources)
                    {
                        if (_resourceList.FirstOrDefault(r => r.RESOURCEID == moreResource.RESOURCEID) == null)
                            _resourceList.Add(moreResource);

                        if (ResourceList.FirstOrDefault(r => r.RESOURCEID == moreResource.RESOURCEID) == null)
                            ResourceList.Add(moreResource);
                    }
                    OnPropertyChanged( "ResourceList" );  // Need to do this because if you just Add() to the ResourceList, the OnPropertyChanged is not fired.
                    ((GroupView)View).ResourceComboBox.Items.Refresh();  // I don't think this is needed, I added the OnPropertyChanged which causes a refresh
                    scrollViewer.ScrollToVerticalOffset(vOffSet);
                }
            }
            _resourceScrollJustRan = !_resourceScrollJustRan;
        }

        public void ProviderStringChanged(string newSearchString)
        {
            if (newSearchString == null)
                return;

            if (newSearchString.Length < Keys.PatientSearchCharMin)
                return;

            _providerList = _dataAccessService.SearchSdecProviders(newSearchString);
            ProviderList = _providerList.Where(r => r.Name.StartsWith(newSearchString, true, new CultureInfo("en-us"))).ToList();

            if (ProviderList.Count == 1)
                Provider = ProviderList[0];
            else
                View.SetProviderComboBoxOpen(ProviderList.Count > 0);
        }

        private string _providerVeryLastSub = string.Empty;
        private bool _providerScrollJustRan = true;
        private IList<SchdResource> _resourceList1;
        private IList<SchdResourceGroup> _resourceGroupList1;
        private SchdResource _resource;
        private Provider _provider;
        private IList<Provider> _providerList1;
        private SchdResourceGroup _resourceGroup;

        public void ProvidersScrollChanged(string newSearchString, System.Windows.Controls.ScrollViewer scrollViewer)
        {
            var vOffSet = scrollViewer.VerticalOffset;

            if (!_providerScrollJustRan)
            {
                _providerScrollJustRan = !_providerScrollJustRan;
                return;
            }

            if (ProviderList != null && (ProviderList.Count > 0))
            {
                var lastSub = ProviderList[ProviderList.Count - 1].LastSub;
                // save previous very last sub for use in testing
                var veryLastSub = _providerVeryLastSub;
                // save the current very last sub for future testing
                _providerVeryLastSub = lastSub;

                // test using the saved previous last sub
                if (veryLastSub == lastSub)
                    return;

                var moreProviders = _dataAccessService.SearchSdecProviders(newSearchString, lastSub);

                if (moreProviders != null && moreProviders.Count > 0)
                {
                    foreach (var moreProvider in moreProviders)
                    {
                        if (_providerList.FirstOrDefault(p => p.IEN == moreProvider.IEN) == null)
                            _providerList.Add(moreProvider);

                        if (ProviderList.FirstOrDefault(p => p.IEN == moreProvider.IEN) == null)
                            ProviderList.Add(moreProvider);
                    }

                    ((GroupView)View).ProviderComboBox.Items.Refresh();
                    scrollViewer.ScrollToVerticalOffset(vOffSet);
                }
            }
            _providerScrollJustRan = !_providerScrollJustRan;
        }

        public void ResourceGroupStringChanged(string newSearchString)
        {
            if (newSearchString == null)
                return;

            if (newSearchString.Length < Keys.ClinicResourceSearchCharMin)
                return;

            if ( _resourceGroupList == null || _resourceGroupListStale || _resourceGroupList.Count == 0 )
            {
                ResourceGroup = new SchdResourceGroup( _dataAccessService )
                {
                    IsAsync = false
                };
                ResourceGroup.GetAllResourceGroups( ( s, args ) =>
                {
                    lock( args )
                    {
                        _resourceGroupList = args.Result as IList<SchdResourceGroup>;
                    }
                    _resourceGroupListStale = false;
                } );
            }
            ResourceGroupList = _resourceGroupList.Where(r => r.Name.StartsWith(newSearchString, true, new CultureInfo("en-us"))).ToList();

            if (ResourceGroupList.Count == 1)
                ResourceGroup = ResourceGroupList[0];
            else
                View.SetResourceGroupComboBoxOpen(ResourceGroupList.Count > 0);
        }

        public void HotKeyHandler(Key k)
        {
            View.HotKeyHandler(k);
        }
    }
}