using System;
using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Timers;
using ClinSchd.Infrastructure;
using Telerik.Windows.Controls;
using ClinSchd.Infrastructure.Models;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls.TreeView;

namespace ClinSchd.Modules.Navigation.Group
{
    /// <summary>
    /// Interaction logic for GroupView.xaml
    /// </summary>
    public partial class GroupView : RadPane, IGroupView
    {
        //Patient selectedPatient;
        public void UnselectClinic(string command)
        {
            ResourceComboBox.SelectedItem = null;
            UpdateSelectedResource(null);

            if ("CLEAR-REFRESH".Contains(command))
            {
                ProviderComboBox.SelectedItem = null;
                ProviderComboBox.Text = string.Empty;
                ResourceGroupComboBox.SelectedItem = null;
                ResourceGroupComboBox.Text = string.Empty;
            }
        }

        public GroupView()
        {
            TraceLog.LogInfo("GroupView", "Construct", "Loading Group View");
            InitializeComponent();


            _resourceSearchTimer = new Timer(Keys.SearchTimerInterval);
            _resourceSearchTimer.Elapsed += resourceSearchTimer_Elapsed;
            _providerSearchTimer = new Timer(Keys.SearchTimerInterval);
            _providerSearchTimer.Elapsed += providerSearchTimer_Elapsed;
            _resourceGroupSearchTimer = new Timer(Keys.SearchTimerInterval);
            _resourceGroupSearchTimer.Elapsed += resourceGroupSearchTimer_Elapsed;
        }

        private void OnGridViewDragInfo(object sender, DragDropEventArgs e)
        {
            if (e.Options.Source is GridViewRow)
            {
                if (e.Options.Status == DragStatus.DragInProgress)
                {
                    var radGridView = sender as RadGridView;
                    if (radGridView != null)
                    {
                        var cue = new TreeViewDragCue
                            {
                                ItemTemplate = radGridView.Resources["DraggedItemTemplate"] as DataTemplate,
                                ItemsSource = e.Options.Payload as IEnumerable
                            };
                        e.Options.DragCue = cue;
                    }
                    e.Handled = true;
                }
                else if (e.Options.Status == DragStatus.DragComplete)
                {
                    //IList source = this.orderView.ItemsSource as IList;
                    //foreach (object draggedItem in draggedItems)
                    //{
                    //    source.Remove(draggedItem);
                    //}
                    e.Handled = true;
                }
            }
        }
        private void OnGridViewDragQuery(object sender, DragDropQueryEventArgs e)
        {
            if (e.Options.Source is GridViewRow)
            {
                RadGridView gridView = sender as RadGridView;
                if (gridView != null)
                {
                    e.QueryResult = gridView.SelectedItems.Count > 0;
                    e.Options.Payload = gridView.SelectedItems;
                }
                else
                {
                    e.QueryResult = true;
                    e.Handled = true;
                }
            }
        }

        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
            e.Handled = true;
        }
        private void OnGridViewDropInfo(object sender, DragDropEventArgs e)
        {
        }

        public void UpdateSelectedResource(SchdResource resource, bool updateSelectedItem = false)
        {
            if (updateSelectedItem)
                ResourceComboBox.SelectedItem = resource;
            else
            {
                if (resource != null)
                {
                    ResourceComboBox.Text = resource.RESOURCE_NAME;
                    ResourceComboBox.EmptyText = string.Empty;
                    ResourceComboBox.IsDropDownOpen = false;
                }
                else
                {
                    ResourceComboBox.Text = string.Empty;
                    ResourceComboBox.EmptyText = "(Select a Clinic)";
                }
            }

            ResourceComboBox.IsDropDownOpen = false;
            RadOutlookBar1.SelectedIndex = 0;
        }

        public void UpdateSelectedResourceGroup(SchdResourceGroup resourceGroup)
        {
            ResourceGroupComboBox.SelectedItem = resourceGroup;
        }

        public void UpdateSelectedProvider(Provider provider)
        {
            ProviderComboBox.SelectedItem = provider;
        }
        public void HotKeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
            {
                switch (k)
                {
                    case Key.D:
                        Keyboard.Focus(Calendar);
                        break;
                }
            }
        }
        //private Point startPoint;
        public GroupPresentationModel Model
        {
            get { return DataContext as GroupPresentationModel; }
            set { DataContext = value; }
        }

        private void AdmittedPatients_SelectionChanged (object sender, SelectionChangeEventArgs e)
        {
            if (e.AddedItems.Count > 0) {
                Model.SelectPatient (e.AddedItems[0] as Patient);
            }
        }

        /*private void PatientContextMenu_ItemClick (object sender, Telerik.Windows.RadRoutedEventArgs e)
        {
            selectedPatient = PatientContextMenu.GetClickedElement<GridViewRow>().Item as Patient;
            switch ((e.Source as RadMenuItem).Header.ToString()) {
            case "Remove Patient":
                var waitListClose = new WaitListDisposition();
                    waitListClose.PatientToRemove = selectedPatient;
                CLaunchWindow.LaunchWindow(waitListClose, "Remove Paitent from Request Management", 170, 325, "", true);
                    if(waitListClose.DialogResult)
                        Model.RemovePatientFromWorkspace (PatientContextMenu.GetClickedElement<GridViewRow> ().Item as Patient);
                break;
            case "Clear Workspace":
                Model.AdmittedPatients.Clear ();
                break;
            default:
                break;
            }
        }

        private void PatientContextMenu_Opened (object sender, RoutedEventArgs e)
        {
            GridViewRow selectedRow = PatientContextMenu.GetClickedElement<GridViewRow> ();
            RadMenuItem RemovePatientMenuItem = (PatientContextMenu.Items[0] as RadMenuItem);
            if (selectedRow == null) {
                RemovePatientMenuItem.IsEnabled = false;
            } else {
                RemovePatientMenuItem.IsEnabled = true;
            }
        }*/

        public void Use_Calendar_SelectionChanged(bool useIt)
        {
            if (useIt)
                Calendar.SelectionChanged += Calendar_SelectionChanged;
            else
                Calendar.SelectionChanged -= Calendar_SelectionChanged;
        }

        private void Calendar_SelectionChanged (object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                Keys.RequestKeys.CurrentSelectedDesiredDate = e.AddedItems[0].ToString();
                Model.ViewDayRange((DateTime)e.AddedItems[0]);
            }
        }

        /*private void AdmittedPatients_PreviewMouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            startPoint = e.GetPosition(null);
        }

        private void AdmittedPatients_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            // Get the current mouse position
            Point mousePos = e.GetPosition(null);
            Vector diff = startPoint - mousePos;

            if (e.LeftButton == MouseButtonState.Pressed &&
                Math.Abs(diff.X) > SystemParameters.MinimumHorizontalDragDistance ||
                Math.Abs(diff.Y) > SystemParameters.MinimumVerticalDragDistance)
            {
                // Get the dragged ListViewItem
                var radGridView = sender as RadGridView;
                var radGridViewItem =
                    FindAnchestor<RadGridView>((DependencyObject)e.OriginalSource);

                // Find the data behind the ListViewItem
                Patient patient = (Patient)radGridView.ItemContainerGenerator.ItemFromContainer(radGridViewItem);

                // Initialize the drag & drop operation
                var dragData = new DataObject("myFormat", patient);
                DragDrop.DoDragDrop(radGridViewItem, dragData, DragDropEffects.Move);
            } 
        }*/
        // Helper to search up the VisualTree
        private static T FindAnchestor<T>(DependencyObject current)
            where T : DependencyObject
        {
            do
            {
                if (current is T)
                {
                    return (T)current;
                }
                current = VisualTreeHelper.GetParent(current);
            }
            while (current != null);
            return null;
        }

        /****************************************************************************************/
        #region resource (Clinics) ComboBox

        private readonly Timer _resourceSearchTimer;        
        public string ResourceSearchString { get; set; }

        void resourceSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _resourceSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => Model.ResourceStringChanged(ResourceSearchString)));
        }

        private void ResourceComboBox_OnKeyUp(object sender, KeyEventArgs e)
        {
            _resourceSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _resourceSearchTimer.Start();
        }

        private void ResourceComboBox_OnTextChanged(object sender, RoutedEventArgs e)
        {
            ResourceSearchString = ((RadComboBox)sender).Text;
        }

        private void ResourceCombo_GotFocus(object sender, RoutedEventArgs e)
        {
            ResourceComboBox.Text = string.Empty;
        }

        private void ResourceCombo_LostFocus(object sender, RoutedEventArgs e)
        {
//            SetResourceComboBoxOpen(false);
        }

        private void ResourceComboBox_OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                Model.Resource = e.AddedItems[0] as SchdResource;
                Model.ExecuteSchedulerDisplayCommand(e.AddedItems[0] as SchdResource);
            }
        }

        public void SetResourceComboBoxOpen(bool openDropDown)
        {
            ResourceComboBox.IsDropDownOpen = openDropDown;
        }

        public void SetResourceComboBoxFocus()
        {
            ResourceComboBox.Focus();
            Keyboard.Focus(ResourceComboBox);
        }

        //private void ResourceComboBox_ScrollChanged(object sender, ScrollChangedEventArgs e)
        //{
        //    if (string.IsNullOrEmpty(ResourceComboBox.Text))
        //        return;

        //    if (ResourceComboBox.Text.Length < Keys.ClinicHResourceSearchCharMin)
        //        return;

        //    var scrollViewer = e.OriginalSource as ScrollViewer;

        //    if (scrollViewer != null && scrollViewer.VerticalOffset > 0 && scrollViewer.VerticalOffset >= scrollViewer.ScrollableHeight - Keys.ClinicResourceSearchGetNextWithin)
        //        Dispatcher.BeginInvoke((Action)(() => Model.ResourceScrollChanged(ResourceSearchString, scrollViewer)));
        //}

        #endregion resource (Clinics) ComboBox

        /****************************************************************************************/
        #region Provider Combobox

        private readonly Timer _providerSearchTimer;
        public string ProviderSearchString { get; set; }

        void providerSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _providerSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => Model.ProviderStringChanged(ProviderSearchString)));
        }

        private void ProviderComboBox_OnKeyUp(object sender, KeyEventArgs e)
        {
            _providerSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _providerSearchTimer.Start();
        }

        private void ProviderCombo_GotFocus(object sender, RoutedEventArgs e)
        {
            ProviderComboBox.Text = string.Empty;
        }

        private void ProviderCombo_LostFocus(object sender, RoutedEventArgs e)
        {
            //SetProviderComboBoxOpen(false);
        }

        private void ProviderComboBox_OnTextChanged(object sender, RoutedEventArgs e)
        {
            ProviderSearchString = ((RadComboBox)sender).Text;
        }

        private void ProviderComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                if (Keys.SelectedAppointmentRequest == null)
                {
                    Model.ViewDayRange(DateTime.Now.Date);
                    Keys.RequestKeys.CurrentSelectedDesiredDate = DateTime.Now.Date.ToShortDateString();
                }
                
                Model.Provider = e.AddedItems[0] as Provider;
                Model.ExecuteSchedulerDisplayProviderCommand(e.AddedItems[0] as Provider);
            }
        }

        public void SetProviderComboBoxOpen(bool openDropDown)
        {
            ProviderComboBox.IsDropDownOpen = openDropDown;
        }

        private void ProviderComboBox_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            if (string.IsNullOrEmpty(ProviderComboBox.Text))
                return;

            if (ProviderComboBox.Text.Length < Keys.ProviderSearchCharMin)
                return;

            var scrollViewer = e.OriginalSource as ScrollViewer;

            if (scrollViewer != null && scrollViewer.VerticalOffset > 0 && scrollViewer.VerticalOffset >= scrollViewer.ScrollableHeight - Keys.ProviderSearchGetNextWithin)
                Dispatcher.BeginInvoke((Action)(() => Model.ProvidersScrollChanged(ResourceSearchString, scrollViewer)));
        }

        #endregion Provider Combobox

        /****************************************************************************************/
        #region Resource Group Combobox (Clinic Groups)

        private readonly Timer _resourceGroupSearchTimer;
        public string ResourceGroupSearchString { get; set; }

        void resourceGroupSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _resourceGroupSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => Model.ResourceGroupStringChanged(ResourceGroupSearchString)));
        }

        private void ResourceGroupCombo_GotFocus(object sender, RoutedEventArgs e)
        {
            ResourceGroupComboBox.Text = string.Empty;
        }

        private void ResourceGroupCombo_LostFocus(object sender, RoutedEventArgs e)
        {
            SetResourceGroupComboBoxOpen(false);
        }

        private void ResourceGroupComboBox_OnKeyUp(object sender, KeyEventArgs e)
        {
            _resourceGroupSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _resourceGroupSearchTimer.Start();
        }

        private void ResourceGroupComboBox_OnTextChanged(object sender, TextChangedEventArgs e)
        {
            ResourceGroupSearchString = ((RadComboBox)sender).Text;
        }

        private void ResourceGroupComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count > 0)
                Model.ExecuteSchedulerDisplayGroupCommand(e.AddedItems[0] as SchdResourceGroup);
        }

        public void SetResourceGroupComboBoxOpen(bool openDropDown)
        {
            ResourceGroupComboBox.IsDropDownOpen = openDropDown;
        }

        #endregion Resource Group Combobox (Clinic Groups)
    }
}
