﻿using System.Windows;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;
using Microsoft.Practices.Unity;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

using ClinSchd.Modules.PatientAppt.Services;
using ClinSchd.Modules.PatientAppt.ChildModules.Disabilities.Services;
using ClinSchd.Modules.PatientAppt.ChildModules.Disabilities.Disabilities;

namespace ClinSchd.Modules.PatientAppt.ChildModules.Disabilities.Controllers
{
    public class DisabilitiesController : IDisabilitiesController
    {
        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;
        private readonly IUnityContainer _container;
        private readonly IDisabilitiesService _service;
//        private Patient _selectedPatient;
        private IDisabilitiesView _disabilitiesView = null;
        private PatientDisabilitiesModel _disabilities = null;

        public DisabilitiesController(IEventAggregator eventAggregator, IUnityContainer container, IDataAccessService dataAccessService, IDisabilitiesService service)
        {
            this._eventAggregator = eventAggregator;
            this._container = container;
            this._dataAccessService = dataAccessService;
            this._service = service;
        }

        #region IDisabilitiesController members

        public void Run()
        {
//            _eventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(HandlePatientSelectedEvent, false);
//            _eventAggregator.GetEvent<PatientUnselectEvent>().Subscribe(HandlePatientUnselectedEvent, false);
            _eventAggregator.GetEvent<PatientDisabilitiesEvent>().Subscribe(HandlePatientDisabilitiesEvent, false);
        }

        #endregion IDisabilitiesController members

        #region Event Handlers

        public void HandlePatientDisabilitiesEvent(PatientDisabilitiesArgs args)
        {
            if (args != null)
            {
                // If there is no patient then make sure the disabilities dialog is not visible and return
                if (args.Patient == null)
                {
                    HandlePatientDisabilitiesEvent(null);
                    return;
                }

                Window owner = args.Owner;

                if ((_disabilities == null) || (_disabilities.PatientIEN != args.Patient.IEN))
                    _disabilities = _dataAccessService.GetPatientDisabilities(args.Patient.IEN);

                if (_disabilitiesView == null)
                    _disabilitiesView = _container.Resolve<IDisabilitiesView>();

                _disabilitiesView.Model.Disabilities = _disabilities;
                _disabilitiesView.Model.PatientName = args.Patient.Name;

                // Positioning Calculations
                Rect screenRect = Infrastructure.Keys.CurrentScreenDimensions();

                // Attempting to center the 2 forms
                // Width of both forms + 10 pixel separation
                double combinedWidth = owner.Width + _disabilitiesView.Width + 20;
                // Calculate disabilities viewer left position
                double childLeft = screenRect.Left + (((screenRect.Right - screenRect.Left + 1) - combinedWidth) / 2);

                // if disabilities viewer left is less than the current screen left make an adjustment
                if (childLeft < screenRect.Left)
                    childLeft = screenRect.Left + 10;

                // Set the owner form to be 20 pixels to the right of the disabilities viewer
                double ownerLeft = childLeft + _disabilitiesView.Width + 20;
                // Get the new right position of the owner form
                double ownerRight = ownerLeft + owner.Width;

                // if the owner form right position is greater than the current screen right make an adjustment 
                if (ownerRight > screenRect.Right)
                    ownerLeft = ownerLeft - (ownerRight - screenRect.Right + 10);

                // Set the disabilities viewer top the same as the owner form
                double childTop = owner.Top - 50;

                // if the new disabilties top is above the current screen top make an adjustment
                if (childTop < screenRect.Top)
                    childTop = screenRect.Top + 30;

                // Set the form positions
                owner.Left = ownerLeft;
                _disabilitiesView.Top = childTop;
                _disabilitiesView.Left = childLeft;
                _disabilitiesView.Owner = owner;
                _service.Show(_disabilitiesView, _disabilitiesView.Model);
            }
            else
            {
                if (_disabilitiesView != null)
                {
                    _disabilitiesView.Close();
                    _disabilitiesView = null;
                }            
            }
        }

        //public void HandlePatientSelectedEvent(Patient patient)
        //{
        //    _selectedPatient = patient;
        //}

        //public void HandlePatientUnselectedEvent(string command)
        //{
        //    HandlePatientDisabilitiesEvent(null);
        //    _selectedPatient = null;
        //}

        #endregion Event Handlers
    }
}

