﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Media;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.PatientAppt.ChildModules.Disabilities.Disabilities
{
    public class DisabilitiesPresentationModel : IDisabilitiesPresentationModel, INotifyPropertyChanged
    {
        public IDisabilitiesView View { get; set; }
        private PatientDisabilitiesModel _disabilities;
        private string _patientName = string.Empty;

        #region IDisabilitiesPresentationModel members

        public void Close()
        {
            View = null;
        }

        public void Show()
        {
            View.Show();
        }

        public PatientDisabilitiesModel Disabilities
        {
            get { return _disabilities; }
            set
            {
                _disabilities = value;
                OnPropertyChanged("DisabilitiesList");
                OnPropertyChanged("PrimaryName");
                OnPropertyChanged("SCPercent");
                OnPropertyChanged("Secondaries");
                OnPropertyChanged("HasDisabilities");
                OnPropertyChanged("ShowDisabilities");
                OnPropertyChanged("ShowNoDisabilities");
                OnPropertyChanged("DisabilitiesListBackground");
            }
        }

        private readonly IList<PatientDisabilityModel> _emptyDisabilitiesList = new List<PatientDisabilityModel>();

        public IList<PatientDisabilityModel> DisabilitiesList
        {
            get { return HasDisabilities ? Disabilities.Disabilities : _emptyDisabilitiesList; }
        }

        public Brush DisabilitiesListBackground
        {
            get { return HasDisabilities && DisabilitiesList.Count > 0 
                    ? new SolidColorBrush( Colors.White ) 
                    : (SolidColorBrush)new BrushConverter().ConvertFrom( "#A6BEE4" ); }
        }

        public bool HasDisabilities
        {
            get { return Disabilities != null && Disabilities.HasDisabilities; }
        }

        public string NoDisabilitiesText
        {
            get { return string.Format("Patient, {0}, has no current disabilities.", PatientName); }
        }

        public string PatientName
        {
            get { return _patientName; }
            set
            {
                _patientName = value;
                OnPropertyChanged("PatientName");
                OnPropertyChanged("NoDisabilitiesText");
            }
        }

        public string PrimaryName
        {
            get { return HasDisabilities ? Disabilities.PrimaryName : string.Empty; }
        }

        public string SCPercent
        {
            get { return HasDisabilities ? Disabilities.ServiceConnectedPercentage : string.Empty; }
        }

        private readonly IList<NameValue> _emptySecondaries = new List<NameValue>();
        public IList<NameValue> Secondaries
        {
            get { return HasDisabilities ? Disabilities.Secondaries : _emptySecondaries; }
        }

        public Visibility ShowDisabilities
        {
            get { return HasDisabilities && DisabilitiesList.Count > 0 ? Visibility.Visible : Visibility.Collapsed; }
        }

        public Visibility ShowNoDisabilities
        {
            get { return HasDisabilities && DisabilitiesList.Count > 0 ? Visibility.Collapsed : Visibility.Visible; }
        }

        public string Title
        {
            get { return "Patient's Eligibility Information:"; }
        }

        #endregion IDisabilitiesPresentationModel members

        #region INotifyPropertyChanged members

        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion INotifyPropertyChanged members

    }
}
