﻿using ClinSchd.Modules.PatientAppt.Group.BenefitEligibility;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ClinSchd.Modules.PatientAppt.Group.Factories
{
    public class EligibilityFactory
    {
        public static List<Eligibility> CreateEligiblityList(string sdecEligiblityRpc)
        {
            var rpcRecords = getRpcRecords(sdecEligiblityRpc);

            if (rpcRecords.Length == 0)
                return new List<Eligibility>();

            return createEligibilityList(rpcRecords);
        }

        #region HELPERS
        private static List<Eligibility> createEligibilityList(string[] rpcRecords)
        {
            var eligibilityList = new List<Eligibility>();

            foreach (var record in rpcRecords)
                eligibilityList.Add(createEligibility(record));

            eligibilityList.RemoveAll(item => item == null);

            return eligibilityList;
        }
        private static Eligibility createEligibility(string record)
        {
            var rpcPieces = getRpcStringPieces(record);

            if (rpcPieces.Length != 4)
                return null;

            return mapEligibility(rpcPieces);
        }
        private static Eligibility mapEligibility(string[] rpcPieces)
        {
            var eligibility = new Eligibility();
            eligibility.EligibilityIen = rpcPieces[0];
            eligibility.Name = rpcPieces[1];
            eligibility.NationalEligibilityName = rpcPieces[2];
            eligibility.VeteranEligibility = rpcPieces[3] == "VETERAN" ? true : false;
            return eligibility;
        }

        private static string[] getRpcStringPieces(string rpcRecord)
        {
            if (string.IsNullOrWhiteSpace(rpcRecord))
                return new string[0];

            return rpcRecord.Split('^');
        }
        private static string[] getRpcRecords(string rpcString)
        {
            if (string.IsNullOrWhiteSpace(rpcString))
                return new string[0];

            return rpcString.Split(':');
        }
        private static string[] getRpcPieces(string rpcString)
        {
            if (string.IsNullOrWhiteSpace(rpcString))
                return new string[0];

            return rpcString.Split('^');
        }
        #endregion
    }
}
