using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.PatientAppt.Controllers;
using ClinSchd.Modules.PatientAppt.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using ClinSchd.Modules.PatientAppt.Group.BenefitEligibility;

namespace ClinSchd.Modules.PatientAppt.Group
{
    public class GroupPresentationModel : IGroupPresentationModel, INotifyPropertyChanged
    {
        public AppointmentRequest AppointmentRequest { get; set; }

        private readonly IPatientApptService _patientApptService;
        public readonly IDataAccessService DataAccessService;
        public event PropertyChangedEventHandler PropertyChanged;
        public readonly IEventAggregator EventAggregator;
        private ValidationMessage _validationMessage;
        private AppointmentData _appointmentDate;
        private bool _isNoteEnabled = true;
        private string _paneTitle = "";
        public bool WantPatientNotifiedOfLabXRayOrEkgStops { get; set; }
        public bool WantPreviousXrayResultsSentToClinic { get; set; }
        public GroupPresentationModel(IGroupView view, IDataAccessService dataAccessService,
        IPatientApptService patientApptService, IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _patientApptService = patientApptService;
            EventAggregator = eventAggregator;
            DataAccessService = dataAccessService;
            ValidationMessage = new ValidationMessage();
            NewAppointmentCommand = new DelegateCommand<string>(ExecuteNewAppointmentCommand, CanExecuteNewAppointmentCommand);
            PrintLetterCommand = new DelegateCommand<string>(ExecutePrintLetterCommand, CanExecutePrintLetterCommand);
            if (EventAggregator != null)
                EventAggregator.GetEvent<AppointmentRequestSelectedEvent>().Subscribe(AppointmentRequestSelected, ThreadOption.UIThread, false);

            

            //LoadConflicts ();
        }

        public void OnClose()
        {
            View.Close();
        }
        public IGroupView View { get; }
        public IPatientApptController Controller { get; set; }
        public CompositePresentationEvent<string> NewForwardingEvent { get; set; }

        public PatientInformation PatientInformation
        {
            get { return _patientInformation; }
            set
            {
                _patientInformation = value;
                OnPropertyChanged("PatientInformation");
            }
        }

        public DelegateCommand<string> NewAppointmentCommand { get; private set; }
        public DelegateCommand<string> PrintLetterCommand { get; private set; }

        public IList<ConflictAppointments> ConflictAppointments
        {
            get { return _conflictAppointments; }
            set
            {
                _conflictAppointments = value;
                OnPropertyChanged("ConflictAppointments");
            }
        }

        public SchdAppointment SchdAppointment { get; set; }

        public string Command
        {
            get { return _command; }
            set
            {
                _command = value;
                OnPropertyChanged("Command");
            }
        }

        public bool IsNoteEnabled
        {
            get { return _isNoteEnabled; }
            set
            {
                if (_isNoteEnabled != value)
                {
                    _isNoteEnabled = value;
                    OnPropertyChanged("IsNoteEnabled");
                }
            }
        }

        public bool IsNewAppointment { get { return Command == "New Appointment" || Command == "WalkIn Appointment"; } }

        public bool IsStartTimeEnabled
        {
            get
            {
                if (Resource == null)
                    return false;

                if (AppointmentRequest == null)
                    return false;

                if (IsNewAppointment)
                    return !Keys.LoggedInUserSDECViewOnlyKey && !this.AppointmentRequest.IsMRTCChild;
                else
                    return false;
                //    return !Keys.LoggedInUserSDECViewOnlyKey && !this.AppointmentRequest.IsMRTCChild &&
                //    (
                //        SchdAppointment?.AppointmentStatus == AppointmentStatuses.New
                //    ||
                //        SchdAppointment?.AppointmentStatus == AppointmentStatuses.WalkIn_NotCheckedIn
                //    );
            }
        }

        public bool IsDurationEnabled
        {
            get
            {
                if (Resource == null)
                    return false;

                return Resource.VariableLengthAppointments && IsStartTimeEnabled;
            }
        }

        public AppointmentData AppointmentData
        {
            get { return _appointmentDate; }
            set
            {
                _appointmentDate = value;
                OnPropertyChanged("AppointmentData");
            }
        }

        public ValidationMessage ValidationMessage
        {
            get
            {
                return _validationMessage;
            }
            set
            {
                _validationMessage = value;
            }
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged("PaneTitle");
            }
        }

        private SchdResource GetSchdResource(SchdAppointment appointment)
        {
            if (appointment.SelectedResource != null)
                return appointment.SelectedResource;
            if (!string.IsNullOrEmpty(appointment.ResourceId))
                return DataAccessService.GetSdecResource(appointment.ResourceId);
            if (!string.IsNullOrEmpty(appointment.RESOURCENAME))
                return DataAccessService.GetSdecResourceByName(appointment.RESOURCENAME);
            return null;
        }

        public void GetEditAppointment(SchdAppointment newAppointment)
        {
            if (newAppointment == null || newAppointment.APPOINTMENTID == null)
                return;

            if (AppointmentRequest == null)
                AppointmentRequest = DataAccessService.GetAppointmentRequestByRequestIenAndType(newAppointment.APPTREQTYPE_REQIEN, newAppointment.APPTREQTYPE_REQTYPE);

            SchdAppointment = newAppointment;
            SchdAppointment.PATIENTID = newAppointment.PATIENTID;

            var a = new AppointmentData(DataAccessService, SchdAppointment.PATIENTID)
            {
                ResourceName = newAppointment.RESOURCENAME,
                Notes = newAppointment.NOTE,
                AppointmentTypeIen = newAppointment.APPTTYPE_IEN
            };
            SetExistingAppointmentTimes(newAppointment, a);
            a.AppointmentID = newAppointment.APPOINTMENTID;
            a.AppointmentCommand = "Edit Appointment";
            a.Duration = (Convert.ToDateTime(newAppointment.END_TIME) - Convert.ToDateTime(newAppointment.START_TIME)).TotalMinutes.ToString();
            AppointmentData = a;

            if (newAppointment.PATIENTID != null)
            {
                GetPatientInformation(newAppointment.PATIENTID);
                //a.IsServiceConnected = PatientInformation.
            }
            LoadConflicts(false);
        }

        public void GetViewAppointment(SchdAppointment newAppointment)
        {
            if (newAppointment.APPOINTMENTID == null)
            {
                return;
            }
            SchdAppointment = newAppointment;
            SchdAppointment.PATIENTID = newAppointment.PATIENTID;
            //SchdResource schdResource = GetSchdResource(newAppointment);
            var a = new AppointmentData
            {
                ResourceName = newAppointment.RESOURCENAME,
                Notes = newAppointment.NOTE,
                AppointmentTypeIen = newAppointment.APPTTYPE_IEN
            };
            SetExistingAppointmentTimes(newAppointment, a);
            a.AppointmentID = newAppointment.APPOINTMENTID;
            a.AppointmentCommand = "View Appointment";
            a.Duration = (Convert.ToDateTime(newAppointment.END_TIME) - Convert.ToDateTime(newAppointment.START_TIME)).TotalMinutes.ToString();
            //BJ TODO: add eligibility IEN assignment
            AppointmentData = a;

            if (newAppointment.PATIENTID != null)
            {
                GetPatientInformation(newAppointment.PATIENTID);
            }
            LoadConflicts(false);
        }

        public void GetWalkInAppointment(SchdAppointment newAppointment, bool isOverbook = false)
        {
            GetPatientInformation(newAppointment.PATIENTID);
            SchdAppointment = newAppointment;
            SchdAppointment.PATIENTID = newAppointment.PATIENTID;
            var a = new AppointmentData
            {
                IsOverbook = isOverbook,
                ResourceName = newAppointment.RESOURCENAME,
                Notes = "",
                StartTime = newAppointment.START_TIME,
                EndTime = newAppointment.END_TIME
            };

            // Set new appointment duration
            var duration = (Convert.ToDateTime(newAppointment.END_TIME) - Convert.ToDateTime(newAppointment.START_TIME)).TotalMinutes;

            if (duration > Keys.NewAppointmentMaxDuration)
                duration = newAppointment.SelectedResource.Clinic.Duration;

            a.Duration = duration.ToString();

            a.IsServiceConnected = SchdAppointment.AppointmentRequest.Patient.SCVisit != "NO";
            a.ServicePercentage = SchdAppointment.AppointmentRequest.Patient.ServiceConnectionPercentage;
            ChkSvcRelatedEnabled = PatientInformation.ServiceConnection != "NO";
            a.AppointmentCommand = "WalkIn Appointment";
            AppointmentData = a;
        }

        private string _patientId;
        public void GetNewAppointment(SchdAppointment newAppointment, bool isOverbook = false)
        {
            _patientId = newAppointment.PATIENTID;
            GetPatientInformation(newAppointment.PATIENTID);
            SchdAppointment = newAppointment;
            SchdAppointment.PATIENTID = newAppointment.PATIENTID;
            var a = new AppointmentData(DataAccessService, SchdAppointment.PATIENTID)
            {
                IsOverbook = isOverbook,
                ResourceName = newAppointment.RESOURCENAME,
                Notes = "",
                StartTime = newAppointment.START_TIME,
                EndTime = newAppointment.END_TIME                
            };

            // Set new appointment duration
            var duration = (Convert.ToDateTime(newAppointment.END_TIME) - Convert.ToDateTime(newAppointment.START_TIME)).TotalMinutes;

            if (duration > Keys.NewAppointmentMaxDuration)
                duration = newAppointment.SelectedResource.Clinic.Duration;

            a.Duration = duration.ToString();

            if (SchdAppointment.AppointmentRequest == null)
            {
                if (Keys.SelectedAppointmentRequest != null)
                    SchdAppointment.AppointmentRequest = Keys.SelectedAppointmentRequest;
                else
                {
                    MessageBox.Show("Please select a request before making appointments");
                    return;
                }
            }
            a.IsServiceConnected = SchdAppointment.AppointmentRequest.Patient.SCVisit != "NO";
            a.ServicePercentage = SchdAppointment.AppointmentRequest.Patient.ServiceConnectionPercentage;
            ChkSvcRelatedEnabled = PatientInformation.ServiceConnection != "NO";
            a.AppointmentCommand = "New Appointment";
            AppointmentData = a;

            LoadConflicts(true);
        }

        private void SetExistingAppointmentTimes(SchdAppointment appointment, AppointmentData data)
        {
            if (Keys.Methods.IsDate(appointment.START_TIME))
                data.StartTime = appointment.START_TIME;
            else
            {
                MessageBox.Show("Invalid Start Time");
                return;
            }

            data.Duration = ComputeActualDuration(appointment).ToString();

            if (Keys.Methods.IsDate(appointment.START_TIME))
                data.EndTime = appointment.END_TIME;
            else
                MessageBox.Show("Invalid End Time");
        }

        public int ComputeActualDuration(SchdAppointment schdAppointment)
        {
            try
            {
                var endTime = Convert.ToDateTime(schdAppointment.END_TIME);
                var startTime = Convert.ToDateTime(schdAppointment.START_TIME);
                return Convert.ToInt32((endTime - startTime).TotalMinutes);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Duration Calculation Error");
            }
            return 30;
        }

        public void GetPatientInformation(string patientIEN)
        {
            PatientInformation = DataAccessService.GetPatientInfo(patientIEN);
        }

        public void LoadConflicts(bool isNewAppointment)
        {
            ConflictAppointments = DataAccessService.GetConflictAppointment(SchdAppointment.RESOURCENAME, SchdAppointment.PATIENTID, SchdAppointment.START_TIME, SchdAppointment.END_TIME, isNewAppointment, SchdAppointment.APPOINTMENTID);
        }

        private void AppointmentRequestSelected(AppointmentRequest appointmentRequest)
        {
            AppointmentRequest = appointmentRequest;
        }

        public void ExecuteNewAppointmentCommand(string command)
        {
            var errorMessage = string.Empty;
            if (command == "New Appointment")
            {
                AppointmentData.AppointmentID = "0";
                List<string> errorMessages;
                if (AppointmentRequest == null)
                    errorMessages = DataAccessService.AddNewAppointment(AppointmentData, PatientInformation, false);
                else
                    errorMessages = DataAccessService.AddNewAppointment(AppointmentData, PatientInformation, false, AppointmentRequest);
                if (errorMessages.Count > 0)
                    errorMessage = errorMessages[1];
            }
            else if (command == "WalkIn Appointment")
            {
                AppointmentData.AppointmentID = "WALKIN";
                errorMessage = CreateWalkInAppointment(AppointmentData, PatientInformation, false);
            }
            else if (command == "Edit Appointment")
            {
                errorMessage = DataAccessService.EditAppointment(AppointmentData);
            }

            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            if (errorMessage == string.Empty)
            {
                // Setting the event RefreshScheduler payload to null allows the schedule to be refreshed
                EventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(String.Empty);
                EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Publish(null);
            }
            else
            {
                // If the overbook failed a message was already displayed
                _validationMessage.IsValid = false;
                _validationMessage.Title = command;
                _validationMessage.Message = errorMessage;
            }
        }

        public string CreateWalkInAppointment(AppointmentData a, PatientInformation p, bool bMakeChartRequest)
        {
            var errorMessage = string.Empty;
            List<string> errorMessages;

            if (AppointmentRequest == null)
                errorMessages = DataAccessService.AddNewAppointment(AppointmentData, PatientInformation, false);
            else
                errorMessages = DataAccessService.AddNewAppointment(AppointmentData, PatientInformation, false, AppointmentRequest);

            //			errorMessages = this.dataAccessService.AddNewAppointment (a, p, bMakeChartRequest);
            if (errorMessages[0] != "0" && errorMessages[1] == string.Empty)
            {
                //prompt check in dialog
                var sAppt = new SchdAppointment
                {
                    IsNewAppointment = true,
                    START_TIME = a.StartTime,
                    APPOINTMENTID = errorMessages[0],
                    PATIENTID = p.IEN,
                    PATIENTNAME = p.Name,
                    RESOURCENAME = a.ResourceName,
                    WALKIN = "1"
                };
                EventAggregator.GetEvent<LaunchCheckInDialogEvent>().Publish(sAppt);
            }
            else
            {
                errorMessage = errorMessages[1];
            }

            return errorMessage;
        }

        public bool CanExecuteNewAppointmentCommand(string command)
        {
            return true;
        }

        public void ViewPatientAppointments(int patientID)
        {
            var args = new object[2];
            args[0] = "Patient Appointments";
            args[1] = patientID;
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
            //DPatientApptDisplay dPa = new DPatientApptDisplay (!m_bNewAppointment);
        }

        private void PrintPatientLetter(int apptID)
        {
            var args = new object[3];
            args[0] = "Patient Appointment Letter";
            args[1] = apptID.ToString();
            args[2] = "P";
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
            //DPatientLetter dPa = new DPatientLetter ();
        }

        public void ViewPatientHistory(int patientID)
        {
            //DPatientHistoryDisplay dHi = new DPatientHistoryDisplay ();
        }

        private void ViewPatientHSMerge(int patientID)
        {
            var args = new object[2];
            args[0] = "Patient HS Merge";
            args[1] = patientID;
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
            //DPatientHSMergeTextDisplay dHs = new DPatientHSMergeTextDisplay ();
        }

        private void ExecutePrintLetterCommand(string arg)
        {
            var nPatientID = Convert.ToInt32(PatientInformation.IEN);
            var nApptID = Convert.ToInt32(AppointmentData.AppointmentID);

            switch (View.ReportComboBox.SelectedItem.ToString())
            {
                case "Patient Letter":
                    PrintPatientLetter(nApptID);
                    break;
                case "Patient Appointments":
                    ViewPatientAppointments(nPatientID);
                    break;
                case "Patient History":
                    ViewPatientHistory(nPatientID);
                    break;
                case "Patient HS Merge":
                    ViewPatientHSMerge(nPatientID);
                    break;
            }
        }

        private bool CanExecutePrintLetterCommand(string arg)
        {
            return true;
        }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        public void PromptToRemoveRequest(string requestIen)
        {
            EventAggregator.GetEvent<RemoveRequestEvent>().Publish(requestIen);
        }

        public bool IsConsultOrPtCSch { get; set; }
        private IList<AppointmentType> _appointmentTypes;
        public IList<AppointmentType> AppointmentTypes
        {
            get
            {
                if (AppointmentRequest != null)
                {
                    if (AppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Consult &&
                        AppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Procedure &&
                        AppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.PtCSch)
                        IsConsultOrPtCSch = false;
                    else
                        IsConsultOrPtCSch = true;
                }
                if (_appointmentTypes == null)
                {
                    if (_patientId == null && AppointmentRequest != null && AppointmentRequest.Patient != null)
                        _patientId = AppointmentRequest.Patient.IEN;
                    if (string.IsNullOrEmpty(_patientId))
                        return null;
                    return DataAccessService.GetAppointmentTypes(_patientId);
                }
                return _appointmentTypes;
            }
            set { _appointmentTypes = value; }

        }

        public bool IsServiceConnected
        {
            get { return AppointmentData.IsServiceConnected; }
            set
            {
                AppointmentData.IsServiceConnected = value;
                OnPropertyChanged("IsServiceConnected");
                View.SetAppointmentType();
            }
        }

        public bool ChkSvcRelatedEnabled { get; private set; }

        public Visibility PrintLetterVisibility
        {
            get { return AppointmentData != null ? (string.IsNullOrEmpty(AppointmentData.AppointmentID) ? Visibility.Hidden : Visibility.Visible) : Visibility.Hidden; }
        }

        private SchdResource _resource;
        private string _command;
        private PatientInformation _patientInformation;
        private IList<ConflictAppointments> _conflictAppointments;

        public SchdResource Resource
        {
            get
            {
                if (_resource == null)
                    _resource = DataAccessService.GetSdecResourceByName(AppointmentData.ResourceName);

                return _resource;
            }
        }
        
        public TimeSpan ClockStartTime
        {
            get
            {
                if (Resource == null)
                    return TimeSpan.FromHours(8);

                if (Resource.Clinic == null)
                    return TimeSpan.FromHours(8);

                return TimeSpan.FromHours(Convert.ToInt32(Resource.Clinic.HOUR_DISPLAY_BEGIN));
            }
        }

        public TimeSpan ClockEndTime
        {
            get
            {
                var results = ClockStartTime.Add(ClockTimeInterval);

                while (results < TimeSpan.FromDays(1).Subtract(ClockTimeInterval))
                    results = results.Add(ClockTimeInterval);

                return results;
            }
        }

        public TimeSpan ClockTimeInterval
        {
            get
            {
                if (Resource == null)
                    return TimeSpan.FromMinutes(30);

                if (Resource.Clinic == null)
                    return TimeSpan.FromMinutes(30);

                return TimeSpan.FromMinutes(Resource.Clinic.Duration);
            }
        }

        public IList<string> DurationsList
        {
            get
            {
                var results = new List<string>();

                var displayIncrement = 15;

                if (Resource != null && Resource.Clinic != null)
                    displayIncrement = 60 / Resource.Clinic.DISPLAY_INCREMENTS;

                for (var duration = displayIncrement; duration <= Keys.NewAppointmentMaxDuration; duration += displayIncrement)
                    results.Add(duration.ToString());

                return results;
            }
        }

        public Keys.OverbookingStatus CheckOverbook()
        {
            var newAppt = new SchdAppointment
            {
                SelectedResource = Resource,
                START_TIME = AppointmentData.StartTime,
                MRTCParentRequestIEN = AppointmentRequest.MRTCParentRequestIEN
            };
            return DataAccessService.ClinicOverbook(newAppt);
        }

        #region BJ'S BENEFIT/ELIGIBILITY SELECTION PROPERTIES

        private List<Eligibility> patientEligibilitiesList;
        public List<Eligibility> PatientEligibilitiesList
        {
            get { return patientEligibilitiesList; }
            set
            {
                if (value == patientEligibilitiesList)
                    return;

                patientEligibilitiesList = value;
                if (value.Count == 1)
                    disableEligibilitySelection();
                    
                OnPropertyChanged("PatientEligibilitiesList");
            }
        }


        private Eligibility selectedPatientEligibility;
        public Eligibility SelectedPatientEligibility
        {
            get { return selectedPatientEligibility; }
            set
            {
                if (value == selectedPatientEligibility)
                    return;

                selectedPatientEligibility = value;
                if (value != null)
                {
                    AppointmentData.EligibilityIen = value.EligibilityIen;
                    if(Command.ToUpper() != "VIEW APPOINTMENT" && Command.ToUpper() != "EDIT APPOINTMENT")
                        SelectedAppointmentType = DetermineDefaultAppointmentType(value);
                }
                OnPropertyChanged("SelectedPatientEligibility");
            }
        }

        
        private bool eligibilitySelectionEnabled = true; 
        public bool EligibilitySelectionEnabled
        {
            get { return eligibilitySelectionEnabled; }
            set
            {
                if (value == eligibilitySelectionEnabled)
                    return;

                eligibilitySelectionEnabled = value;
                OnPropertyChanged("EligibilitySelectionEnabled");
            }
        }


        private AppointmentType selectedAppointmentType;
        public AppointmentType SelectedAppointmentType
        {
            get { return selectedAppointmentType; }
            set
            {
                if (value == selectedAppointmentType)
                    return;

                selectedAppointmentType = value;
                OnPropertyChanged("SelectedAppointmentType");
            }
        }
        

        #region HELPERS
        private void disableEligibilitySelection()
        {
            SelectedPatientEligibility = PatientEligibilitiesList[0];
            EligibilitySelectionEnabled = false;
        }

        public AppointmentType DetermineDefaultAppointmentType(Eligibility value)
        {
            var defaultAppointmentTypeName = 
                EligibilityDefaultAppointmentTypes
                    .GetDefaultAppointmentTypeName(value.NationalEligibilityName);

            foreach (var appointmentType in AppointmentTypes)
                if (appointmentType.Name.ToUpper() == defaultAppointmentTypeName.ToUpper())
                    return appointmentType;

            return DataAccessService.GetDefaultClinicAppointmentType(AppointmentRequest.ClinicIen, null);
        }
        #endregion

        #endregion

        
    }
}
