using System;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using Telerik.Windows.Controls;
using ClinSchd.Modules.PatientAppt.Group.Factories;

namespace ClinSchd.Modules.PatientAppt.Group
{
    /// <summary>
    /// Interaction logic for GroupView.xaml
    /// </summary>
    public partial class GroupView : IGroupView
    {
        private bool _isLoaded;
        public GroupView()
        {
            InitializeComponent();
            LoadReportSelection();
        }

        public string RemoveRequestCommand { get; set; }

        public GroupPresentationModel Model
        {
            get { return DataContext as GroupPresentationModel; }
            set { DataContext = value; }
        }

        public bool? DialogResult { get; set; }
        public event EventHandler Closed;
        public bool? ShowDialog()
        {
            throw new NotImplementedException();
        }

        public event EventHandler<EventArgs> ShowPatientAppt;
        public DateTime m_dStartTime = DateTime.Today;

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void OKButton_Click(object sender, RoutedEventArgs e)
        {
            if (Model.SelectedPatientEligibility == null)
            {
                MessageBox.Show("You must select a 'Benefit/Eligibility' before making this appointment.");
                return;
            }
            if (cmbAppointmentType.SelectedValue != null)
                Model.AppointmentData.AppointmentTypeIen = cmbAppointmentType.SelectedValue.ToString();

            //FIX TO MANAGE MISSING APPOINTMENT TYPE IEN WHEN EDITING APPOINTMENT (SHOULD BE REDESIGNED WHEN TIME ALLOWS)
            if (Model.AppointmentData.AppointmentCommand != "Edit Appointment")
            {
                if (Model.IsConsultOrPtCSch && string.IsNullOrEmpty(Model.AppointmentData.AppointmentTypeIen))
                {
                    MessageBox.Show("Please provide an appointment type.");
                    return;
                }
            }
            //check conflict appointment
            bool isTelehealth;
            if (Model.DataAccessService.CheckConflictAppointment("Save Appointment", Model.AppointmentData.ResourceName, Model.AppointmentData.PatientId, Model.PatientInformation.Name, Model.AppointmentData.StartTime, Model.AppointmentData.EndTime, out isTelehealth, Model.AppointmentData.AppointmentID, false, true))
            {
                Model.AppointmentData.LastSaveAttemptStartTime = Model.AppointmentData.StartTimeValue;
                Model.AppointmentData.OKButtonIsEnabled = false;
                return;
            }

            // Do we need to ptCSch overbooking ??
            if ((Model.AppointmentData.ValueHasChanged("StartTime") || Model.AppointmentData.ValueHasChanged("Duration")) && Model.Command != "WalkIn Appointment")
            {
                switch (Model.CheckOverbook())
                {
                    case Keys.OverbookingStatus.Cannot:
                        return;
                    case Keys.OverbookingStatus.NotRequired:
                        break;
                    case Keys.OverbookingStatus.Required:
                        Model.AppointmentData.IsOverbook = true;
                        break;
                }
            }

            // Save the new Appointment
            Model.ExecuteNewAppointmentCommand(Model.AppointmentData.AppointmentCommand);

            if (Model.ValidationMessage.IsValid)
            {
                if (Model.AppointmentData.AppointmentCommand == "New Appointment" && !string.IsNullOrEmpty(Model.AppointmentData.AppointmentID) && Model.AppointmentData.AppointmentID != "0")
                {
                    if (ConfirmUser("Print the Patient Letter?", "Print Letter?"))
                        Model.PrintLetterCommand.Execute(Model.AppointmentData.AppointmentID);
                }
                Close();
                if (Model.AppointmentRequest != null && Model.AppointmentRequest.ConsultationRequestIen > 0)
                {
                    Thread.Sleep(500);
                    Keys.RemoveConsultRequest = true;
                    Model.EventAggregator.GetEvent<RemoveRequestEvent>().Publish(Model.AppointmentRequest.ConsultationRequestIen.ToString());
                }
                if (Model.IsConsultOrPtCSch && Model.AppointmentData != null && !string.IsNullOrEmpty(Model.AppointmentData.AppointmentTypeIen))
                {
                    Model.AppointmentData.Appointment = Model.SchdAppointment;
                    Model.EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("PatientApptView");
                    return;
                }
                if (Model.AppointmentData.AppointmentCommand == "New Appointment" || Model.AppointmentData.AppointmentCommand == "WalkIn Appointment")
                {
                    Model.AppointmentData.Appointment = Model.SchdAppointment;
                    Model.EventAggregator.GetEvent<AppointmentAddedEvent>().Publish(Model.AppointmentData);

                    if ((string.IsNullOrEmpty(RemoveRequestCommand) || RemoveRequestCommand == "REMOVE") && Model.AppointmentRequest != null)
                    {
                        if (Model.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl)
                            Model.PromptToRemoveRequest(Model.AppointmentRequest == null ? "" : Model.AppointmentRequest.AppointmentRequestType + "|" + Model.AppointmentRequest.RequestManagmentIen);
                        else if (Keys.Methods.IsApptOrVet(Model.AppointmentRequest.AppointmentRequestType))
                            Model.PromptToRemoveRequest(Model.AppointmentRequest == null ? "" : Model.AppointmentRequest.AppointmentRequestType + "|" + Model.AppointmentRequest.ApptRequestIen);
                    }

                    Model.EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("PatientApptView");
                }

                DialogResult = true;
            }
            else
            {
                if (Model.ValidationMessage.Message == "OVERBOOK FAILED")
                {
                    DialogResult = false;
                    Close();
                }
                else
                    Model.View.AlertUser(Model.ValidationMessage.Message, Model.ValidationMessage.Title);
            }
        }

        private void LoadReportSelection()
        {
            if (ReportCombo.Items.Count == 0)
            {
                if (!(m_dStartTime < DateTime.Today))
                {
                    ReportCombo.Items.Add("Patient Letter");
                }
                //ReportCombo.Items.Add ("Patient Appointments");
                //ReportCombo.Items.Add ("Patient History");
                //ReportCombo.Items.Add ("Patient HS Merge");
                ReportCombo.SelectedIndex = 0;
            }
        }

        public void Refresh()
        {
            AppointmentsPatientApptGrid.Rebind();
        }

        private bool _bDialogResult;
        public void Close()
        {
            Keys.IsAddAppointmentOrLeftClickOnScheduler = false;
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
            System.Windows.Forms.Application.DoEvents();
        }

        public bool ConfirmUser(string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }

        public void AlertUser(string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        public RadComboBox ReportComboBox
        {
            get
            {
                return ReportCombo;
            }
        }

        private void GroupView_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            System.Windows.Forms.Application.DoEvents();
            Window thisWindow = Window.GetWindow(this);

            
            if (Model.Command.ToUpper() == "VIEW APPOINTMENT" || Model.Command.ToUpper() == "EDIT APPOINTMENT")
            {
                ChkSvcRelated.Visibility = Visibility.Hidden;
                TxtAppointmentTypes.Visibility = Visibility.Hidden;
                cmbAppointmentType.Visibility = Visibility.Hidden;



                Model.PatientEligibilitiesList = EligibilityFactory.CreateEligiblityList(
                    Model.DataAccessService.GetEligibility(Model.PatientInformation.IEN,
                    Model.Resource.HOSPITAL_LOCATION_ID,
                    ExtensionMethods.ConvertToFilemanDateTime(
                        Convert.ToDateTime(Model.SchdAppointment.START_TIME))));

                if (Model.PatientEligibilitiesList.Count < 1)
                {
                    AlertUser("This appointment was scheduled without an eligibility.  Please cancel and reschedule this appointment with the correct eligibility.", "No Eligibility Assigned");
                    this.Close(); 
                }
                else
                    Model.SelectedPatientEligibility = Model.PatientEligibilitiesList.First();

                Model.EligibilitySelectionEnabled = false;

            }
            else
            {
                Model.PatientEligibilitiesList =
                    EligibilityFactory.CreateEligiblityList(
                        Model.DataAccessService.GetPatientEligibilities(Model.AppointmentRequest.Patient.IEN));

                SetAppointmentType();

                // Open the Patient Disabilities Viewer
                if (Model.AppointmentRequest != null && Model.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt)
                {
                    if (Model.AppointmentRequest.WaitTime > 0)
                        Model.EventAggregator.GetEvent<PatientDisabilitiesEvent>().Publish(new PatientDisabilitiesArgs(thisWindow, Model.AppointmentRequest.Patient));
                }
                else
                    Model.EventAggregator.GetEvent<PatientDisabilitiesEvent>().Publish(new PatientDisabilitiesArgs(thisWindow, Model.AppointmentRequest.Patient));
            }
            //            System.Windows.Forms.Application.DoEvents();
            //            Thread.Sleep(200);
            //            System.Windows.Forms.Application.DoEvents();
            thisWindow?.BringIntoView();
            thisWindow?.Focus();
            ChkSvcRelated.Focus();
            Keyboard.Focus(ChkSvcRelated);


            _isLoaded = true;
        }

        private void GroupView_Unloaded(object sender, RoutedEventArgs e)
        {
            // Close the Patient Disabilities Viewer
            if (Model.AppointmentRequest != null && Model.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt)
            {
                if (Model.AppointmentRequest.WaitTime > 0)
                    Model.EventAggregator.GetEvent<PatientDisabilitiesEvent>().Publish(null);
            }
            else
                Model.EventAggregator.GetEvent<PatientDisabilitiesEvent>().Publish(null);
        }

        public void SetAppointmentType()
        {
            string apptType = "";
            if (Model.AppointmentRequest == null)
            {
                if (Keys.SelectedAppointmentRequest == null)
                {
                    MessageBox.Show("Please select an appointment request");
                    return;
                }
                Model.AppointmentRequest = Keys.SelectedAppointmentRequest;
            }

            if (Model.AppointmentRequest.AppointmentType != null && !_isLoaded)
            {
                if (Model.AppointmentTypes.FirstOrDefault(t => t.Name == Model.AppointmentRequest.AppointmentType) != null)
                    apptType = Model.AppointmentRequest.AppointmentType;
            }
            else
            {
                var defaultApptType = Model.DataAccessService.GetDefaultClinicAppointmentType(Model.AppointmentRequest.ClinicIen, null);

                if (defaultApptType != null)
                    apptType = defaultApptType.Name;
            }
            if (ChkSvcRelated.IsChecked != null && ChkSvcRelated.IsChecked == true && Model.AppointmentRequest.Patient.ServiceConnection == "YES")
            {
                if (Model.AppointmentRequest.Patient.ServiceConnectionPercent >= 50 && Model.AppointmentTypes.Any(at => at.Name == "SERVICE CONNECTED"))
                    apptType = "SERVICE CONNECTED";

                if ((string.IsNullOrEmpty(apptType) || apptType == "REGULAR") && Model.AppointmentTypes.Any(at => at.Name == "SERVICE CONNECTED"))
                    apptType = "SERVICE CONNECTED";
            }
            cmbAppointmentType.Text = apptType;
        }

        private void HotKeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                switch (k)
                {
                    case Key.O:
                        Keyboard.Focus(OKButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                    case Key.S:
                        Keyboard.Focus(ChkSvcRelated);
                        break;
                }
            }
        }

        private void FocusFirstRowInConflicts()
        {
            if (AppointmentsPatientApptGrid.HasItems)
            {
                AppointmentsPatientApptGrid.SelectedItem = AppointmentsPatientApptGrid.Items[0];
                AppointmentsPatientApptGrid.ScrollIntoView(AppointmentsPatientApptGrid.SelectedItem);
                AppointmentsPatientApptGrid.Focus();
            }
        }

        private void NotesTextBox_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Tab)
            {

                if (AppointmentsPatientApptGrid.HasItems)
                {
                    e.Handled = true;
                    FocusFirstRowInConflicts();
                }
                else
                {
                    e.Handled = true;
                    Keyboard.Focus(OKButton);
                }

            }
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            Key key = e.Key == Key.System ? e.SystemKey : e.Key;
            HotKeyHandler(key);
        }
    }
}
