using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ClinSchd.Modules.PatientSelection.PatientSelection;
using ClinSchd.Modules.PatientSelection.Controllers;
using ClinSchd.Modules.PatientSelection.Services;
using ClinSchd.Modules.PatientSelection.Tests.Mocks;

namespace ClinSchd.Modules.PatientSelection.Tests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class PatientSelectionModuleFixture
    {
        private MockUnityResolver _container;
        private MockPatientSelectionController _controller;

        [TestMethod]
        public void PatientSelectionModuleRegistersPatientSelectionViewAndPatientSelectionService()
        {
            var patientSelectionModule = CreateTestablePatientSelectionModule();

            patientSelectionModule.InvokeRegisterViewsAndServices();

            Assert.AreEqual(typeof(PatientSelectionView), _container.Types[typeof(IPatientSelectionView)]);
            Assert.AreEqual(typeof(PatientSelectionController), _container.Types[typeof(IPatientSelectionController)]);
            Assert.AreEqual(typeof(PatientSelectionPresentationModel), _container.Types[typeof(IPatientSelectionPresentationModel)]);
            Assert.AreEqual(typeof(PatientSelectionService), _container.Types[typeof(IPatientSelectionService)]);
#if !SILVERLIGHT
#endif
        }

        [TestMethod]
        public void InitCallsRunOnPatientSelectionController()
        {
            var patientSelectionModule = CreateTestablePatientSelectionModule();

            patientSelectionModule.Initialize();

            Assert.IsTrue(_controller.RunCalled);
        }

        private TestablePatientSelectionModule CreateTestablePatientSelectionModule()
        {
            _container = new MockUnityResolver();
            _controller = new MockPatientSelectionController();

            _container.Bag.Add(typeof(IPatientSelectionController), _controller);
            _container.Bag.Add(typeof(IPatientSelectionView), null);

            return new TestablePatientSelectionModule(_container);
        }

        internal class TestablePatientSelectionModule : PatientSelectionModule
        {
            public TestablePatientSelectionModule(IUnityContainer container)
                : base(container, null)
            {
            }

            public void InvokeRegisterViewsAndServices()
            {
                RegisterViewsAndServices();
            }
        }
    }
}
