using System;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Regions;
using Microsoft.Practices.Unity;
using ClinSchd.Modules.PatientSelection.Services;
using ClinSchd.Modules.PatientSelection.PatientSelection;

namespace ClinSchd.Modules.PatientSelection.Controllers
{
    public class PatientSelectionController : IPatientSelectionController
    {
        private readonly IRegionManager _regionManager;
        private readonly IPatientSelectionPresentationModel _patientSelectionPresentationModel;
        private readonly IPatientSelectionService _patientSelectionService;
        private readonly IEventAggregator _eventAggregator;
        private readonly IUnityContainer _container;

        public PatientSelectionController(IRegionManager regionManager,
            IPatientSelectionPresentationModel patientSelectionPresentationModel,
            IPatientSelectionService patientSelectionService,
            IEventAggregator eventAggregator,
            IUnityContainer container)
        {
            _regionManager                     = regionManager;
            _patientSelectionPresentationModel = patientSelectionPresentationModel;
            _patientSelectionService           = patientSelectionService;
            _eventAggregator                   = eventAggregator;
            SelectedPatientIEN                 = string.Empty;
            _container                         = container;
        }
        
        public IPatientSelectionPresentationModel Model { get { return _patientSelectionPresentationModel; } }

        public void Run()
        {
            _patientSelectionService.ShowDialog(
                _patientSelectionPresentationModel.View,
                _patientSelectionPresentationModel, () => _patientSelectionPresentationModel.OnClose() );
        }

        public string SelectedPatientIEN { get; set; }

        public void RunDemographicEditor(string command)
        {
            if (!string.IsNullOrEmpty(SelectedPatientIEN))
            {
                IPatientInformationPresentationModel model = _container.Resolve<IPatientInformationPresentationModel>();
                model.SetPatient(SelectedPatientIEN);
                _patientSelectionService.ShowDemographicEditor(command, model);
            }
        }
    }
}
