using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using System.Linq;
using System.Text.RegularExpressions;
using ClinSchd.Modules.PatientSelection.PatientSelection.ChildModules.PatientDemographics;

namespace ClinSchd.Modules.PatientSelection.PatientSelection
{
    public class PatientInformationPresentationModel : BaseViewModel, IPatientInformationPresentationModel
    {
        #region private fields
//        public const string VALID_PHONE_INPUT = "0123456789 +-x()";
//        public const string VALID_CELL_PHONE_INPUT = "0123456789 +-()";

        public IDataAccessService DataAccessService { get; }

        private readonly IEventAggregator _eventAggregator;
        private Patient _selectedPatient;
        private IList<MaritalStatus> _maritalStatusList;
        private IList<Ethnicity> _ethnicitiesList;
        private IList<Religion> _religionList;
        private IList<State> _stateList;
        private IList<Country> _countryList;
        private string _primaryZipPlus4 = "";
        private string _primaryCity = "";
        private string _primaryCounty = "";
        private string _primaryState = "";
        private string _temporaryZipPlus4 = "";
        private string _temporaryCity = "";
        private string _temporaryCounty = "";
        private string _temporaryState = "";
        private bool _temporaryIsActive;

        private ObservableCollection<string> _cityList = new ObservableCollection<string>();
        private ObservableCollection<string> _tempCityList = new ObservableCollection<string>();
        private bool _hasMultipleCities;
        private bool _tempHasMultipleCities;

        #endregion private fields

        #region constructor

        public PatientInformationPresentationModel(
            IPatientInformation view,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _eventAggregator = eventAggregator;
            DataAccessService = dataAccessService;
        }

        public void InitializeStateValues()
        {
            PrimaryCounty   = SelectedPatient.PrimaryContactInfo.County;
            PrimaryCity     = SelectedPatient.PrimaryContactInfo.City;
            CityList.Add( PrimaryCity );

            if (SelectedPatient.PrimaryContactInfo.State == null)
                PrimaryState = String.Empty;
            else
                PrimaryState    = SelectedPatient.PrimaryContactInfo.State.Name;

            PrimaryZipPlus4 = SelectedPatient.PrimaryContactInfo.ZipPlus4;

            TemporaryCounty   = SelectedPatient.TemporaryContactInfo.County;
            TemporaryCity     = SelectedPatient.TemporaryContactInfo.City;
            TempCityList.Add( TemporaryCity );

            if (SelectedPatient.TemporaryContactInfo.State == null)
                TemporaryState = String.Empty;
            else
                TemporaryState = SelectedPatient.TemporaryContactInfo.State.Name;

            TemporaryZipPlus4 = SelectedPatient.TemporaryContactInfo.ZipPlus4;

            TemporaryIsActive = SelectedPatient.TemporaryContactInfo.IsActive;
        }

        #endregion constructor

        #region IPatientInformationPresentationModel Members

        public string PrimaryZipPlus4
        {
            get
            {
                if (_primaryZipPlus4.Length <= 5)
                    return _primaryZipPlus4;

                return _primaryZipPlus4.Substring(0, 5) + "-" + _primaryZipPlus4.Substring(5);
            }
            set
            {
                _primaryZipPlus4 = value.Replace("-","");
                if ( (_primaryZipPlus4.Length == 0 || _primaryZipPlus4.Length == 5 || _primaryZipPlus4.Length == 9 ) && ChangingZipCode ) // Lookup City and State
                {
                    UpdatePrimaryContactLinkedInfo( _primaryZipPlus4 );
                    ChangingZipCode = false;
                }
                else
                {
                    ChangingZipCode = true;
                }
                NotifyPropertyChanged( "PrimaryZipPlus4" );
            }
        }

        public bool ChangingZipCode { get; set; }

        public string PrimaryCity
        {
            get { return _primaryCity; }
            set
            {
                _primaryCity = value;
                NotifyPropertyChanged( "PrimaryCity" );
            }
        }

        public string PrimaryCounty
        {
            get { return _primaryCounty; }
            set
            {
                _primaryCounty = value;
                NotifyPropertyChanged( "PrimaryCounty" );
            }
        }

        public string PrimaryState
        {
            get { return _primaryState; }
            set
            {
                _primaryState = value;
                NotifyPropertyChanged( "PrimaryState" );
            }
        }

        public string TemporaryZipPlus4
        {
            get
            {
                if (_temporaryZipPlus4.Length <=5)
                    return _temporaryZipPlus4;

                return _temporaryZipPlus4.Substring(0, 5) + "-" + _temporaryZipPlus4.Substring(5);
            }
            set
            {
                _temporaryZipPlus4 = value.Replace("-","");
                if ( ( _temporaryZipPlus4.Length == 0 || _temporaryZipPlus4.Length == 5 || _temporaryZipPlus4.Length == 9 ) && ChangeingTempZipCode ) // Lookup City and State
                {
                    UpdateTemporaryContactLinkedInfo( _temporaryZipPlus4 );
                    ChangeingTempZipCode = false;
                }
                else
                {
                    ChangeingTempZipCode = true;
                }
                NotifyPropertyChanged( "TemporaryZipPlus4" );
            }
        }

        public bool ChangeingTempZipCode { get; set; }

        public string TemporaryCity
        {
            get
            {
                return _temporaryCity;
            }
            set
            {
                _temporaryCity = value;
                NotifyPropertyChanged( "TemporaryCity" );
            }
        }

        public string TemporaryCounty
        {
            get { return _temporaryCounty; }
            set
            {
                _temporaryCounty = value;
                NotifyPropertyChanged( "TemporaryCounty" );
            }
        }

        public string TemporaryState
        {
            get { return _temporaryState; }
            set
            {
                _temporaryState = value;
                NotifyPropertyChanged( "TemporaryState" );
            }
        }

        public DateTime? TemporaryStartDate
        {
            get { return SelectedPatient.TemporaryContactInfo.Start; }
            set
            {
                SelectedPatient.TemporaryContactInfo.Start = value;
                NotifyPropertyChanged("TemporaryStartDate");
                NotifyPropertyChanged("TemporaryEndDate");
                NotifyPropertyChanged("TemporaryMinEndDate");
                NotifyPropertyChanged("TemporaryEndDateIsActive");
            }
        }

        public DateTime? TemporaryEndDate
        {
            get { return SelectedPatient.TemporaryContactInfo.End; }
            set
            {
                SelectedPatient.TemporaryContactInfo.End = value;
                NotifyPropertyChanged("TemporaryEndDate");
            }
        }

        public DateTime? TemporaryMinEndDate
        {
            get { return SelectedPatient.TemporaryContactInfo.MinEndDate; }
        }

        public bool TemporaryEndDateIsActive
        {
            get
            {
                return TemporaryIsActive && SelectedPatient.TemporaryContactInfo.Start.HasValue; ;
            }
        }

        public bool TemporaryIsActive
        {
            get { return _temporaryIsActive; }
            set
            {
                _temporaryIsActive = value;
                NotifyPropertyChanged( "TemporaryIsActive" );
                NotifyPropertyChanged("TemporaryEndDateIsActive");

                if (SelectedPatient.TemporaryContactInfo.IsActive && !value)
                    if (Keys.ConfirmUser(null, "Do you want to delete all temporary address data ? ", "Temporary Address", true))
                    {
                        SelectedPatient.TemporaryContactInfo.Street1 = string.Empty;
                        SelectedPatient.TemporaryContactInfo.Street2 = string.Empty;
                        SelectedPatient.TemporaryContactInfo.Start = null;
                        SelectedPatient.TemporaryContactInfo.End = null;
                        //TemporaryCity = string.Empty;
                        //TemporaryState = string.Empty;
                        TemporaryZipPlus4 = string.Empty;
                        //TemporaryCounty = string.Empty;
                        SelectedPatient.TemporaryContactInfo.Country = null;
                        FormattedTemporaryContactInfoPhone = string.Empty;
                    }
            }
        }

        public bool DialogResult { get { return View.DialogResult; } }

        public MaritalStatus SelectedMaritalStatus
        {
            get
            {
                if (SelectedPatient == null)
                    return null;

                if (string.IsNullOrEmpty(SelectedPatient.MaritalStatus))
                    return _maritalStatusList.FirstOrDefault(ms => ms.code == "U");

                return _maritalStatusList.FirstOrDefault(ms => ms.name == SelectedPatient.MaritalStatus);
            }
            set
            {
                if (SelectedPatient != null)
                {
                    SelectedPatient.MaritalStatus = value.name;
                    NotifyPropertyChanged( "SelectedMaritalStatus");
                }
            }
        }

        public IList<MaritalStatus> MaritalStatusList
        {
            get
            {
                if (_maritalStatusList == null)
                    _maritalStatusList  = DataAccessService.GetMaritalStatusList();

                return _maritalStatusList;
            }
        }

        public Patient SelectedPatient 
        {
            get { return _selectedPatient; }
            set
            {
                _selectedPatient = value;
                NotifyPropertyChanged( "SelectedPatient");
            }
        }

        public void OnClose()
        {
//bg			View.Close();
        }

        public IPatientInformation View { get; }

        public string Command { get; set; }

        public string Gender
        {
            get
            {
                if (SelectedPatient == null)
                    return string.Empty;

                if (SelectedPatient.Gender == "M")
                    return "Male";
                if (SelectedPatient.Gender == "F")
                    return "Female";
                return "Unknown";
            }
        }

        public bool IsBadAddress
        {
            get
            {
                if (SelectedPatient != null)
                    return SelectedPatient.IsBadAddress;

                return true;
            }
            set
            {
                if (SelectedPatient != null)
                {
                    SelectedPatient.IsBadAddress = value;

                    if (!value)
                        SelectedPatient.BadAddressType = string.Empty;

                    NotifyPropertyChanged( "IsBadAddress");
                    NotifyPropertyChanged( "BadAddressEnabled");
                }

            }
        }

        public bool BadAddressEnabled
        {
            get { return IsBadAddress && !Keys.LoggedInUserSDECViewOnlyKey; }
        }

        public string PriorityGrouping
        {
            get
            {
                if (SelectedPatient == null)
                    return string.Empty;

                if (!string.IsNullOrEmpty(SelectedPatient.PriorityGroup) && !string.IsNullOrEmpty(SelectedPatient.EnrollmentSubGroup))
                    return SelectedPatient.PriorityGroup.Substring( 6 ) + SelectedPatient.EnrollmentSubGroup;  // Remove "GROUP "
                if (!string.IsNullOrEmpty(SelectedPatient.PriorityGroup))
                    return SelectedPatient.PriorityGroup;
                if (!string.IsNullOrEmpty(SelectedPatient.EnrollmentSubGroup))
                    return SelectedPatient.EnrollmentSubGroup;

                return string.Empty;
            }
        }

        public Ethnicity SelectedEthnicity
        {
            get
            {
                if (SelectedPatient == null)
                    return null;

                return EthnicityList.FirstOrDefault(e => e.Ien == SelectedPatient.Ethnicity.Ien);
            }
            set
            {
                SelectedPatient.Ethnicity = value;
                NotifyPropertyChanged( "SelectedEthnicity");
            }
        }

        public IList<Ethnicity> EthnicityList
        {
            get
            {
                if (_ethnicitiesList == null)
                    _ethnicitiesList = DataAccessService.GetEthnicities();

                return _ethnicitiesList;
            }
        }

        private Race _selectedRace = null;
        public Race SelectedRace
        {
            get { return _selectedRace; }
            set
            {
                _selectedRace = value;
                NotifyPropertyChanged("SelectedRace");
            }
        }

        public Religion SelectedReligion
        {
            get
            {
                if (SelectedPatient == null)
                    return null;

                if (string.IsNullOrEmpty(SelectedPatient.ReligiousPreference))
                    return _religionList.FirstOrDefault(r => r.Ien == "29");

                return _religionList.FirstOrDefault(r => r.Ien == SelectedPatient.ReligiousPreference || r.Name == SelectedPatient.ReligiousPreference || r.Code == SelectedPatient.ReligiousPreference);
            }
            set
            {
                SelectedPatient.ReligiousPreference = value.Name;
                NotifyPropertyChanged( "SelectedReligion");
            }
        }

        public IList<Religion> ReligionList
        {
            get
            {
                if (_religionList == null)
                    _religionList = DataAccessService.GetReligionList();

                return _religionList;
            }
        }

        public bool SDECViewOnly
        {
            get { return !Keys.LoggedInUserSDECViewOnlyKey; }
        }

        //        public ObservableCollection<Race> RaceList
        //        {
        //            get
        //            {
        //                if (raceList == null)
        //                {
        //                    raceList = new ObservableCollection<Race>();

        ////                    foreach (Race item in this.dataAccessService.GetRaces())
        //                    foreach (Race item in selectedPatient.RaceList)
        //                        raceList.Add(item);
        //                }

        //                return raceList;
        //            }
        //        }

        public IList<State> StateList
        {
            get
            {
                if (_stateList == null)
                    _stateList = DataAccessService.GetStates();

                return _stateList;
            }
        }

        public IList<Country> CountryList
        {
            get
            {
                if (_countryList == null)
                    _countryList = DataAccessService.GetCountries();

                return _countryList;
            }
        }

        public ObservableCollection<string> CityList
        {
            get { return _cityList; }
            set
            {
                _cityList = value;
                NotifyPropertyChanged( "CityList" );
            }
        }

        public ObservableCollection<string> TempCityList
        {
            get { return _tempCityList; }
            set
            {
                _tempCityList = value;
                NotifyPropertyChanged( "TempCityList" );
            }
        }

        public string FormattedPrimaryContactInfoPhonePager
        {
            get { return FormattedPhoneNumber( SelectedPatient.PrimaryContactInfo.PhonePager, true ); }
            set { SelectedPatient.PrimaryContactInfo.PhonePager = CleanMobilePhoneNumber( value ); }
        }

        public string FormattedPrimaryContactInfoPhoneCell
        {
            get { return FormattedPhoneNumber( SelectedPatient.PrimaryContactInfo.PhoneCell, true ); }
            set { SelectedPatient.PrimaryContactInfo.PhoneCell = CleanMobilePhoneNumber( value ); }
        }

        public string FormattedPrimaryContactInfoPhoneResidence
        {
            get { return FormattedPhoneNumber( SelectedPatient.PrimaryContactInfo.PhoneResidence ); }
            set { SelectedPatient.PrimaryContactInfo.PhoneResidence = CleanPhoneNumber( value ); }
        }

        public string FormattedPrimaryContactInfoPhoneWork
        {
            get { return FormattedPhoneNumber( SelectedPatient.PrimaryContactInfo.PhoneWork ); }
            set { SelectedPatient.PrimaryContactInfo.PhoneWork = CleanPhoneNumber( value ); }
        }

        public string FormattedTemporaryContactInfoPhone
        {
            get { return FormattedPhoneNumber( SelectedPatient.TemporaryContactInfo.Phone ); }
            set
            {
                SelectedPatient.TemporaryContactInfo.Phone = CleanPhoneNumber( value );
                NotifyPropertyChanged("FormattedTemporaryContactInfoPhone");
            }
        }
        

        public string FormattedPrimaryNextOfKinContactInfoPhone
        {
            get { return FormattedPhoneNumber( SelectedPatient.PrimaryNextOfKin.ContactInfo.Phone ); }
            set
            {
                if ( value != null )
                    SelectedPatient.PrimaryNextOfKin.ContactInfo.Phone = CleanPhoneNumber( value );
            }
        }

        private string CleanPhoneNumber( string phone )
        {
            var tphone = "";
            for ( var i = 0; i < phone.Length; i++ )
                if(Keys.Methods.VALID_PHONE_INPUT.Contains(phone[i]))
                    tphone += phone[i];
            return tphone;
        }

        private string CleanMobilePhoneNumber(string phone)
        {
            var tphone = "";
            for (var i = 0; i < phone.Length; i++)
                if (Keys.Methods.VALID_MOBILE_PHONE_INPUT.Contains(phone[i]))
                    tphone += phone[i];
            return tphone;
        }

        public string FormattedPhoneNumber( string phoneNumber, bool isMobilePhone = false )
        {
            if ( phoneNumber == null )
                return string.Empty;

            return Keys.Methods.StandardPhoneFormat(phoneNumber, isMobilePhone);
        }

        public string FormattedSsnRmGrid
        {
            get { return FormattedSSN( SelectedPatient.SSNFormatted ); }
        }

        public string FormattedSSN( string ssn )
        {
            if ( ssn == null )
                return string.Empty;

            switch ( ssn.Length )
            {
                case 9:
                    return Regex.Replace( ssn, @"(\d{3})(\d{2})(\d{4})", "$1-$2-$3" );
                default:
                    return ssn;
            }
        }

        public bool SavePatientData()
        {
            Cursor oldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Wait;

            try
            {
                if ((TemporaryIsActive) && (!TemporaryStartDate.HasValue))
                {
                    Keys.AlertUser(View, "A Start Date is required when activating Temporary Contact Information.", "Temporary Contacnt Iformation Validation");
                    return false;
                }

                // Validate minimum phone number length
                if (!ValidatePhoneNumberLengths())
                    return false;

                if (!ValidateZipCodeLengths())
                    return false;

                // Update the SelectedPatient values with the state values
                SelectedPatient.PrimaryContactInfo.ZipPlus4 = PrimaryZipPlus4;
                SelectedPatient.PrimaryContactInfo.State    = StateList.FirstOrDefault( sl => sl.Name.ToUpper() == PrimaryState.ToUpper() );
                SelectedPatient.PrimaryContactInfo.County   = PrimaryCounty;
                SelectedPatient.PrimaryContactInfo.City     = PrimaryCity;

                SelectedPatient.TemporaryContactInfo.ZipPlus4 = TemporaryZipPlus4;
                SelectedPatient.TemporaryContactInfo.State    = StateList.FirstOrDefault( sl => sl.Name.ToUpper() == TemporaryState.ToUpper() );
                SelectedPatient.TemporaryContactInfo.County   = TemporaryCounty;
                SelectedPatient.TemporaryContactInfo.City     = TemporaryCity;

                SelectedPatient.TemporaryContactInfo.IsActive = TemporaryIsActive;

                bool patientSaved = DataAccessService.SetPatientDemographics(SelectedPatient);
                // If the Special Needs have not changed that is equivalent to specialNeedsSaved
                bool specialNeedsSaved = !_selectedPatient.SpecialNeedsPreferences.HasChanged();

                if (patientSaved && !specialNeedsSaved)
                    specialNeedsSaved = DataAccessService.SetPatientSpecialNeedsPreferences(_selectedPatient.IEN, _selectedPatient.SpecialNeedsPreferences);

                if (patientSaved || specialNeedsSaved)
                    _eventAggregator.GetEvent<PatientUpdatedEvent>().Publish(SelectedPatient);

                return patientSaved && specialNeedsSaved;
            }
            finally
            {
                Mouse.OverrideCursor = oldCursor;
            }
        }

        private bool ValidatePhoneNumberLengths()
        {
            // Validate phones number length  > 4
            List<string> problems = new List<string>();

            if (!String.IsNullOrEmpty(SelectedPatient.PrimaryContactInfo.PhoneCell) && (SelectedPatient.PrimaryContactInfo.PhoneCell.Length < 4))
                problems.Add("    Primary Contact Info Cell Phone Number");

            if (!String.IsNullOrEmpty(SelectedPatient.PrimaryContactInfo.PhoneResidence) && (SelectedPatient.PrimaryContactInfo.PhoneResidence.Length < 4))
                problems.Add("    Primary Contact Info Home Phone Number");

            if (!String.IsNullOrEmpty(SelectedPatient.PrimaryContactInfo.PhonePager) && (SelectedPatient.PrimaryContactInfo.PhonePager.Length < 4))
                problems.Add("    Primary Contact Info Pager Phone Number");

            if (!String.IsNullOrEmpty(SelectedPatient.PrimaryContactInfo.PhoneWork) && (SelectedPatient.PrimaryContactInfo.PhoneWork.Length < 4))
                problems.Add("    Primary Contact Info Work Phone Number");

            if (!String.IsNullOrEmpty(SelectedPatient.TemporaryContactInfo.Phone) && (SelectedPatient.TemporaryContactInfo.Phone.Length < 4))
                problems.Add("    Temporary Contact Info Phone Number");

            if (problems.Count > 0)
                Keys.AlertUser(View, "The listed phone numbers need to be at least 4 characters long.\n\n" + String.Join("\n", problems.ToArray()), "Phone Number Validation");

            return (problems.Count == 0);
        }

        private bool ValidateZipCodeLengths()
        {
            // Validate phones number length  > 4
            List<string> problems = new List<string>();

            // We test the underlying variable because any "-" included in the zipcode is stripped when the value is saved in the property setter
            if (!String.IsNullOrEmpty(_primaryZipPlus4) && (_primaryZipPlus4.Length != 5) && (_primaryZipPlus4.Length != 9))
                problems.Add("    Primary Contact Info");

            if (!String.IsNullOrEmpty(_temporaryZipPlus4) && (_temporaryZipPlus4.Length != 5) && (_temporaryZipPlus4.Length != 9))
                problems.Add("    Temporary Contact Info");

            if (problems.Count > 0)
                Keys.AlertUser(View, "The listed contact info sections have invalid zip codes.\nZip Codes must be blank or 5 or 9 digits in length\n\n" + String.Join("\n", problems.ToArray()), "Zip Code Validation");

            return (problems.Count == 0);
        }

        public void UpdatePrimaryContactLinkedInfo( string zipcode )
        {
            CityList   = new ObservableCollection<string>();

            // Lookup City and State
            var zipInfoList = DataAccessService.GetLinkedCityState( zipcode );
            if ( zipInfoList.Count == 0 )
            {
                PrimaryCity   = "";
                PrimaryState  = "";
                PrimaryCounty = "";
                return;
            }
            foreach ( var zipInfo in zipInfoList )
            {
                zipInfo.City = zipInfo.City.Trim( '*' );
                if ( zipInfo == zipInfoList[0] )
                {
                    PrimaryCity = zipInfo.City;
                    PrimaryState = zipInfo.State;
                    PrimaryCounty = zipInfo.County;
                }
                if ( !CityList.Contains( zipInfo.City ) )
                    CityList.Add( zipInfo.City );
            }
            HasMultipleCities = CityList.Count > 1;
        }

        public bool HasMultipleCities
        {
            get { return _hasMultipleCities; }
            set
            {
                _hasMultipleCities = value; 
                NotifyPropertyChanged( "HasMultipleCities" );
            }
        }

        public void UpdateTemporaryContactLinkedInfo( string zipcode )
        {
            TempCityList = new ObservableCollection<string>();

            // Lookup City and State
            var zipInfoList = DataAccessService.GetLinkedCityState( zipcode );
            if ( zipInfoList.Count == 0 )
            {
                TemporaryCity = "";
                TemporaryState = "";
                TemporaryCounty = "";
                return;
            }
            foreach ( var zipInfo in zipInfoList )
            {
                zipInfo.City = zipInfo.City.Trim( '*' );
                if ( zipInfo == zipInfoList[0] )
                {
                    TemporaryCity = zipInfo.City;
                    TemporaryState = zipInfo.State;
                    TemporaryCounty = zipInfo.County;
                }
                if ( !TempCityList.Contains( zipInfo.City ) )
                    TempCityList.Add( zipInfo.City );
            }
            TempHasMultipleCities = TempCityList.Count > 1;
        }

        public bool TempHasMultipleCities
        {
            get { return _tempHasMultipleCities; }
            set
            {
                _tempHasMultipleCities = value; 
                NotifyPropertyChanged( "TempHasMultipleCities" );
            }
        }

        public ObservableCollection<State> TempStateList { get; set; }

        public void SetPatient(string patientIEN)
        {
            if (string.IsNullOrEmpty(patientIEN))
                _selectedPatient = new Patient(DataAccessService);
            else
            {
                IList<Patient> patientList = DataAccessService.GetPatients("#" + patientIEN, 1);

                if (patientList.Count > 0)
                    _selectedPatient = patientList[0];
                else
                    throw new Exception(string.Format("Patient record for record number {0} not found.", patientIEN));

                //foreach (Race r in RaceList)
                //    r.IsSelected = false;

                //if (selectedPatient != null)
                //{
                //    RaceList.Clear();

                //    foreach (Race r in selectedPatient.RaceList)
                //        RaceList.Add(r);

                //    OnPropertyChanged("RaceList");
                //}
            }

            //if (_selectedPatient.PrimaryContactInfo.State != null && !string.IsNullOrEmpty(_selectedPatient.PrimaryContactInfo.State.Ien))
            //{
            //    _selectedPatient.PrimaryContactInfo.State = StateList.FirstOrDefault(s => s.Ien == _selectedPatient.PrimaryContactInfo.State.Ien);
            //}

            if (_selectedPatient.PrimaryContactInfo.Country != null && !string.IsNullOrEmpty(_selectedPatient.PrimaryContactInfo.Country.Ien))
            {
                _selectedPatient.PrimaryContactInfo.Country = CountryList.FirstOrDefault(c => c.Ien == _selectedPatient.PrimaryContactInfo.Country.Ien);
            }

            //if (_selectedPatient.TemporaryContactInfo.State != null && !string.IsNullOrEmpty(_selectedPatient.TemporaryContactInfo.State.Ien))
            //{
            //    _selectedPatient.TemporaryContactInfo.State = StateList.FirstOrDefault(s => s.Ien == _selectedPatient.TemporaryContactInfo.State.Ien);
            //}

            if (_selectedPatient.TemporaryContactInfo.Country != null && !string.IsNullOrEmpty(_selectedPatient.TemporaryContactInfo.Country.Ien))
            {
                _selectedPatient.TemporaryContactInfo.Country = CountryList.FirstOrDefault(c => c.Ien == _selectedPatient.TemporaryContactInfo.Country.Ien);
            }

            if (_selectedPatient.PrimaryNextOfKin.ContactInfo.State != null && !string.IsNullOrEmpty(_selectedPatient.PrimaryNextOfKin.ContactInfo.State.Ien))
            {
                _selectedPatient.PrimaryNextOfKin.ContactInfo.State = StateList.FirstOrDefault(s => s.Ien == _selectedPatient.PrimaryNextOfKin.ContactInfo.State.Ien);
            }

            if (_selectedPatient.PrimaryNextOfKin.ContactInfo.Country != null && !string.IsNullOrEmpty(_selectedPatient.PrimaryNextOfKin.ContactInfo.Country.Ien))
            {
                _selectedPatient.PrimaryNextOfKin.ContactInfo.Country = CountryList.FirstOrDefault(c => c.Ien == _selectedPatient.PrimaryNextOfKin.ContactInfo.Country.Ien);
            }

            if (_selectedPatient.SecondaryNextOfKin.ContactInfo.State != null && !string.IsNullOrEmpty(_selectedPatient.SecondaryNextOfKin.ContactInfo.State.Ien))
            {
                _selectedPatient.SecondaryNextOfKin.ContactInfo.State = StateList.FirstOrDefault(s => s.Ien == _selectedPatient.SecondaryNextOfKin.ContactInfo.State.Ien);
            }

            if (_selectedPatient.SecondaryNextOfKin.ContactInfo.Country != null && !string.IsNullOrEmpty(_selectedPatient.SecondaryNextOfKin.ContactInfo.Country.Ien))
            {
                _selectedPatient.SecondaryNextOfKin.ContactInfo.Country = CountryList.FirstOrDefault(c => c.Ien == _selectedPatient.SecondaryNextOfKin.ContactInfo.Country.Ien);
            }

            SpecialNeedsPreferences prefs = _selectedPatient.SpecialNeedsPreferences;
            DataAccessService.GetPatientSpecialNeedsPreferences(_selectedPatient.IEN, ref prefs);
            _selectedPatient.SpecialNeedsPreferences = prefs;
            InitializeStateValues();
        }

        #endregion IPatientInformationPresentationModel Members

        //#region ICloseCommandProvider Members

        //public bool IsClosing { get; private set; }

        //public ICommand CloseCommand
        //{
        //    get
        //    {
        //        return GetCommand( "CloseCommand",
        //            _ =>
        //            {
        //                IsClosing = true;
        //                CloseCallback?.Invoke( null );
        //            },
        //            _ => true );
        //    }
        //}

        //#endregion ICloseCommandProvider Members

        //#region ISaveCommandProvider Members

        //public ICommand SaveCommand
        //{
        //    get
        //    {
        //        return GetCommand(
        //            "SaveCommand",
        //            _ =>
        //            {
        //                SavePatientData();
        //                CloseCallback?.Invoke( SelectedPatient );
        //            },
        //            _ => true );
        //    }
        //}

        //#endregion ISaveCommandProvider Members

    }
}
