using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.PatientSelection.PatientSelection;
using ClinSchd.Modules.PatientSelection.Controllers;
using ClinSchd.Modules.PatientSelection.PatientSelection.ChildModules.PatientDemographics;
using ClinSchd.Modules.PatientSelection.Services;
using PatientInformation = ClinSchd.Infrastructure.Models.PatientInformation;

namespace ClinSchd.Modules.PatientSelection
{
    public class PatientSelectionModule : IModule
    {
        private readonly IUnityContainer container;
        private readonly IEventAggregator eventAggregator;
        private IPatientSelectionController controller;

        public PatientSelectionModule(IUnityContainer container, IEventAggregator eventAggregator)
        {
            this.container = container;
            this.eventAggregator = eventAggregator;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();
            controller = container.Resolve<IPatientSelectionController>();
            eventAggregator.GetEvent<LaunchPatientSelectionDialogEvent> ()
                .Subscribe(LaunchPatientSelectionDialog, ThreadOption.UIThread, true);
            eventAggregator.GetEvent<LaunchDemographicEditor> ().Subscribe(LaunchDemographicEditor, ThreadOption.UIThread, true);
            eventAggregator.GetEvent<PassAppointmentsEvent> ().Subscribe (PassAppointments, ThreadOption.UIThread, true);
            eventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(PatientSelectedHandler, ThreadOption.UIThread, true);
            eventAggregator.GetEvent<PatientUnselectEvent>().Subscribe(PatientUnselectedHandler, ThreadOption.UIThread, true);
            container.RegisterType<IPatientInformationPresentationModel, PatientInformationPresentationModel>();
            container.RegisterType<IPatientInformation, PatientSelection.ChildModules.PatientDemographics.PatientInformation>();
        }

        public void PassAppointments (SchdAppointment newAppointment)
        {
            controller.Model.SelectedAppointment = newAppointment;
        }

        public void LaunchPatientSelectionDialog(string title)
        {
            controller.Model.Command = title;
            controller.Run ();
        }

        public void LaunchDemographicEditor(string command)
        {
            controller.RunDemographicEditor(command);
        }

        public void PatientSelectedHandler(Patient patient)
        {
            if (patient == null)
                controller.SelectedPatientIEN = string.Empty;
            else
                controller.SelectedPatientIEN = patient.IEN;
        }

        public void PatientUnselectedHandler(string command)
        {
            PatientSelectedHandler(null);
        }

        protected void RegisterViewsAndServices()
        {
            container.RegisterType<IPatientSelectionController, PatientSelectionController>();
            container.RegisterType<IPatientSelectionView, PatientSelectionView>();
            container.RegisterType<IPatientSelectionPresentationModel, PatientSelectionPresentationModel>();
            container.RegisterType<IPatientSelectionService, PatientSelectionService>();
            container.RegisterType<IPatientInformationPresentationModel, PatientInformationPresentationModel>();
        }
    }
}
