﻿using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Prerequisites.Prerequisites;
using ClinSchd.Modules.Prerequisites.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Unity;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ClinSchd.Modules.Prerequisites.Controllers
{
    public class PrerequisitesController : IPrerequisitesController
    {
        private readonly IPrerequisitesService _PrerequisitesService;
        private readonly IEventAggregator _eventAggregator;
        private readonly IUnityContainer _container;

        public PrerequisitesController(IUnityContainer container,
            IPrerequisitesService PrerequisitesService,
            IEventAggregator eventAggregator)
        {
            this._PrerequisitesService = PrerequisitesService;
            this._eventAggregator = eventAggregator;
            this._container = container;
        }

        public void Run()
        {
            this._eventAggregator.GetEvent<LaunchPrerequisitesDialogEvent>().Subscribe(LaunchPrerequisitesDialog, ThreadOption.UIThread, true);
        }

        public void LaunchPrerequisitesDialog(AppointmentRequest appointmentRequest)
        {
            IPrerequisitesPresentationModel model = _container.Resolve<IPrerequisitesPresentationModel>();

            model.AppointmentRequest = appointmentRequest;
            if (model.Prerequisites != null && model.Prerequisites.Count > 0)
            {
                if (model.Prerequisites.First() == "-1")
                {
                    model.View.AlertUser("There was an error retrieving the prerequisites for this Appointment Request.  Please contact you system administrator.", "Error");
                    return;
                }
                //this._PrerequisitesService.ShowDialog(Model.View, Model, () => Model.OnClose());
                if (Keys.PrerequisiteWindowRequestIen != appointmentRequest.ApptRequestIen)
                {
                    Keys.closePrerequisitesWindows(); //close any prereq windows from previous Appt request(s) before opening new one
                    Keys.PrerequisiteWindowRequestIen = appointmentRequest.ApptRequestIen;
                    var view = new PrerequisitesView();
                    view.DataContext = model;
                    view.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
                    view.Show();
                }
            }
            else Keys.closePrerequisitesWindows(); //close any prereq windows from previous Appt request(s); we don't need one for this request

        }
    }
}
