﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ClinSchd.Infrastructure.Models;
using System.ComponentModel;
using ClinSchd.Modules.Prerequisites.Services;
using ClinSchd.Infrastructure.Interfaces;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Modules.Prerequisites.Prerequisites;

namespace ClinSchd.Modules.Prerequisites.Prerequisites
{
    public class PrerequisitesPresentationModel : IPrerequisitesPresentationModel, INotifyPropertyChanged
    {
        private readonly IPrerequisitesService _PrerequisitesService;
        public readonly IDataAccessService _DataAccessService;
        public readonly IEventAggregator _EventAggregator;


        public PrerequisitesPresentationModel(
            IPrerequisitesView view,
            IPrerequisitesService PrerequisitesService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            _EventAggregator = eventAggregator;
            _PrerequisitesService = PrerequisitesService;
            _DataAccessService = dataAccessService;

            View = view;
            View.Model = this;

            ValidationMessage = new ValidationMessage();
            InitialErrorMessage();
        }


        private List<string> prerequisites; 
        public List<string> Prerequisites
        {
            get
            {
                return prerequisites;
            }

            set
            {
                if (value == prerequisites)
                    return;

                prerequisites = value;
                OnPropertyChanged("Prerequisites");
            }
        }

        private AppointmentRequest appointmentRequest; 
        public AppointmentRequest AppointmentRequest
        {
            get
            {
                return appointmentRequest; 
            }

            set
            {
                if (value == appointmentRequest)
                    return;

                appointmentRequest = value;
                Prerequisites = getPrerequisites(value.ApptRequestIen.ToString());  
                OnPropertyChanged("AppointmentRequest");
            }
        }

        #region HELPERS
        private List<string> getPrerequisites(string reqIen)
        {
            var prerequisites = new List<string>();

            var x = _DataAccessService.GetPrerequisites(reqIen).Split('^');
            foreach (var p in _DataAccessService.GetPrerequisites(reqIen).Split('^'))
                prerequisites.Add(p);

            if (prerequisites.Count == 1 && string.IsNullOrEmpty(prerequisites.First()))
                prerequisites = null; 

            return prerequisites;
        }
        #endregion

        #region VSE MODULE PATTERN PROPERTIES
        public IPrerequisitesView View { get; }

        public ValidationMessage ValidationMessage { get; set; }

        public void InitialErrorMessage()
        {
            ValidationMessage.IsValid = true;
            ValidationMessage.Title = string.Empty;
            ValidationMessage.Message = string.Empty;
        }

        public void OnClose()
        {
            View.Close();
        }
        #endregion

        #region PROPERTY CHANGED
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
