using System;
using System.Collections.Generic;
using System.Reflection;
using System.Xml;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Reports.Helper_Classes
{
    /// <summary>
    ///
    /// </summary>
    public class ReportsModel : AsyncValidatableModel, IReportsModel
    {
        #region Properties

        public List<ReportParameter> ParameterDictionary { get; set; }
        private readonly IDataAccessService _dataAccessService;

        public ReportParameterCreator ReportParameterCreator { get; set; }
        public Dictionary<string, ReportParameter> Params { get; set; }
        public List<ReportParameter> HardCodedParams { get; set; }

        //public IList<NameValue> WardLocations { get { return DataAccessService.GetWardLocations(); } }
        //public IList<NameValue> TreatingSpecialities { get { return DataAccessService.GetTreatingSpecialities(); } }
        //public IList<NameValue> TreatingSpecialitiesAll { get { return DataAccessService.GetTreatingSpecialitiesAll(); } }
        //public IList<NameValue> Providers { get { return DataAccessService.GetProviders(); } }
        //public IList<NameValue> ProvidersAll { get { return DataAccessService.GetProvidersAll(); } }
        //public IList<NameValue> Religions { get { return DataAccessService.GetReligions(); } }
        //public IList<NameValue> HospitalLocations { get { return DataAccessService.GetClinicList(); } }
        public IList<NameValue> ServerPrinterList { get { return _dataAccessService.GetServerPrinters(); } }

        //public IList<NameValue> DynamicServerPrinterList { get { return DataAccessService.GetDynamicServerPrinters(); } }
        //public ObservableCollection<Movement> PatientAdmissionsList { get; private set; }
        //public ObservableCollection<IncompleteChartModel.IncompleteChart> PatientIncompleteChartList { get; private set; }

        //private void LoadPatientAdmissions()
        //{
        //    PatientAdmissionsList.Clear();
        //    foreach (Movement item in DataAccessService.GetMovementList(PatientIEN, Enums.MovementTypes.Admission))
        //    {
        //        PatientAdmissionsList.Add(item);
        //    }
        //}

        //private void LoadIncompleteCharts()
        //{
        //    PatientIncompleteChartList.Clear();
        //    foreach (IncompleteChartModel.IncompleteChart item in DataAccessService.
        //        GetIncompleteChartList(PatientIEN))
        //    {
        //        PatientIncompleteChartList.Add(item);
        //    }
        //}

        private string _reportName;

        public string ReportDisplayName
        {
            get { return _reportName; }
            set
            {
                _reportName = value;
                InitializeReport();
            }
        }

        private string _patientIEN;

        public string PatientIEN
        {
            get { return _patientIEN; }
            set
            {
                _patientIEN = value;
                //LoadPatientAdmissions ();
            }
        }

        public bool? PatientHasBadAddress
        {
            get
            {
                if (string.IsNullOrEmpty(_patientIEN))
                    return null;

                Patient patient = new Patient(_dataAccessService)
                {
                    IEN = _patientIEN
                };

                _dataAccessService.GetPatient(ref patient);
                return patient.IsBadAddress;
            }
        }

        public bool IsPatientLetter { get; set; }
        //{
        //    get { return ("Patient Appointment Letter^Patient Cancellation Letter^Patient No Show Letter").Contains(ReportDisplayName); }
        //}

        public string ReportType { get; set; }

        /// <summary>
        /// Gets the name of the RPC that retrieves the selected report.
        /// </summary>
        public string ReportRPC { get; set; }

        /// <summary>
        /// Gets the list of required parameters for the selected report.
        /// </summary>
        public List<ReportParameter> RequiredParams { get; private set; }

        private bool _reportTextIsStale = true;
        private string _reportText;

        /// <summary>
        /// Gets the full text of the selected report.
        /// </summary>
        public string ReportText
        {
            get
            {
                if ( _reportTextIsStale )
                {
                    if ( _reportText == null && ReportType != null )
                    {
                        DoWorkAsync( ( s, args ) =>
                        {
                            _reportText = _dataAccessService.GetReportText( ReportRPC, GetRPCParams() );
                        }, ( s, args ) =>
                        {
                            ReportText = _reportText;
                            _reportTextIsStale = false;
                            NotifyPropertyChanged( "ReportText" );
                        } );
                        return string.Empty;
                    }
                    return null;
                }
                _reportTextIsStale = true;
                return _reportText;
            }
            private set
            {
                _reportText = value;
            }
        }

        private List<List<string>> GetRPCParams()
        {
            var parms = new List<List<string>>();
            foreach ( var item in RequiredParams )
            {
                if ( !Params.ContainsKey( item.Description ) )
                {
                    throw new ArgumentException( "Required parameter " + item + " was not passed." );
                }
                if ( !( Params[item.Description].Value is List<string> ) )
                {
                    var newList = new List<string>
                    {
                        Params[item.Description].Value as string
                    };
                    parms.Add( newList );
                }
                else
                {
                    parms.Add( Params[item.Description].Value as List<string> );
                }
            }
            return parms;
        }

        #endregion Properties

        #region Methods

        public ReportsModel( IDataAccessService dataAccessService )
        {
            _dataAccessService = dataAccessService;
            ReportParameterCreator = new ReportParameterCreator( this );
            //PatientAdmissionsList = new ObservableCollection<Movement>();
            //PatientIncompleteChartList = new ObservableCollection<IncompleteChartModel.IncompleteChart>();
        }

        private void InitializeReport()
        {
            ReportLoader.LoadReport( this );
        }

        public void InjectManualMappings( string propertyChanged )
        {
            switch ( propertyChanged )
            {
                case "VisitType":
                    Params["SortType"].PossibleValues = new ObservableDictionary<string, string>
                    {
                        {"Authorizing Provider", "1"},
                        { "Case Manager", "2"},
                        { "Community", "4"},
                        { "Date Expected", "5"},
                        { "Patient Name", "6"},
                        { "Visit Disposition", "8"}
                    };

                    switch ( Params["VisitType"].Value as string )
                    {
                        case "A":
                            Params["SortType"].PossibleValues.Add( "Service", "7" );
                            Params["SortType"].PossibleValues.Add( "Ward", "9" );
                            break;

                        case "D":
                            Params["SortType"].PossibleValues.Add( "Service", "7" );
                            break;

                        case "O":
                            Params["SortType"].PossibleValues.Add( "Clinic", "3" );
                            break;

                        default:
                            break;
                    }
                    Params["SubSortType"].PossibleValues = Params["SortType"].PossibleValues;
                    NotifyPropertyChanged( "Params" );
                    break;

                case "ReportToDisplay":
                    if ( Params.ContainsKey( "ReportTypeSort" ) )
                    {
                        Params["ReportTypeSort"].PossibleValues = new ObservableDictionary<string, string>();
                        switch ( Params["ReportToDisplay"].Value as string )
                        {
                            case "1":
                            case "2":
                            case "3":
                            case "5":
                            case "7":
                            default:
                                Params["ReportTypeSort"].PossibleValues.Add( "By DATE Only", "1" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By WARD", "2" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By SERVICE", "3" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By ADMITTING Provider", "4" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By Provider's SERVICE", "5" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By Community", "6" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By Service Unit", "7" );
                                Params["ReportTypeSort"].PossibleValues.Add( "By Patient Name", "8" );
                                break;

                            case "4":
                                Params["ReportTypeSort"].PossibleValues.Add( "TRANSFERS to ICU", "1" );
                                Params["ReportTypeSort"].PossibleValues.Add( "RETURNS to ICU", "2" );
                                break;

                            case "6":
                                Params["ReportTypeSort"].PossibleValues.Add( "LISTING Only", "1" );
                                Params["ReportTypeSort"].PossibleValues.Add( "STATISTICS Only", "2" );
                                Params["ReportTypeSort"].PossibleValues.Add( "BOTH Listings and Stats", "3" );
                                break;
                        }
                        NotifyPropertyChanged( "Params" );
                    }
                    break;

                default:
                    break;
            }
        }

        #endregion Methods

        public static class ReportLoader
        {
            private static XmlDocument ReadInConfigFile()
            {
                var configXMLDoc = new XmlDocument();
                var names = Assembly.GetExecutingAssembly().GetManifestResourceNames();
                var xmlStream = Assembly.GetExecutingAssembly().
                    GetManifestResourceStream( "ClinSchd.Modules.Reports.ReportsConfiguration.xml" );
                var input = new XmlTextReader( xmlStream )
                {
                    WhitespaceHandling = WhitespaceHandling.None
                };
                configXMLDoc.Load( input );
                return configXMLDoc;
            }

            private static List<ReportParameter> GetParameterDictionary( XmlDocument configXMLDoc, ReportParameterCreator reportParameterCreator )
            {
                var parameterDictionary = new List<ReportParameter>();
                foreach ( XmlNode item in configXMLDoc["ReportsConfiguration"].ChildNodes )
                {
                    switch ( item.Name )
                    {
                        case "Parameters":
                            foreach ( XmlNode parameterNode in item.ChildNodes )
                            {
                                var parameterName = parameterNode.Attributes["Name"].Value;
                                var newParam = reportParameterCreator.GetNewWrapper( null, parameterName );
                                newParam.PossibleValues = new ObservableDictionary<string, string>();
                                if ( parameterNode.Attributes["ValueType"] != null )
                                {
                                    newParam.ValueType = parameterNode.Attributes["ValueType"].Value;
                                }
                                foreach ( XmlNode parameterValuesNode in parameterNode.ChildNodes )
                                {
                                    if ( parameterValuesNode.FirstChild != null )
                                    {
                                        newParam.PossibleValues.Add( parameterValuesNode.Attributes["ID"].Value, parameterValuesNode.FirstChild.Value );
                                    }
                                }
                                parameterDictionary.Add( newParam );
                            }
                            break;

                        default:
                            break;
                    }
                }
                return parameterDictionary;
            }

            public static void LoadReport( ReportsModel model )
            {
                var configXMLDoc = ReadInConfigFile();
                model.ParameterDictionary = GetParameterDictionary( configXMLDoc, model.ReportParameterCreator );
                model.HardCodedParams = new List<ReportParameter>();
                model.RequiredParams = new List<ReportParameter>();

                foreach ( XmlNode item in configXMLDoc["ReportsConfiguration"].ChildNodes )
                {
                    switch ( item.Name )
                    {
                        case "Reports":
                            foreach ( XmlNode reportNode in item.ChildNodes )
                            {
                                if ( reportNode.Attributes["DisplayName"].Value == model.ReportDisplayName )
                                {
                                    model.ReportType = reportNode.Attributes["ID"].Value;
                                    model.ReportRPC = reportNode.Attributes["RPC"].Value;
                                    model.IsPatientLetter = Convert.ToBoolean(reportNode.Attributes["IsPatientLetter"].Value);

                                    foreach ( XmlNode reportParametersNode in reportNode.ChildNodes )
                                    {
                                        switch ( reportParametersNode.Name )
                                        {
                                            case "RequiredParams":
                                                foreach ( XmlNode requiredParamNode in reportParametersNode.ChildNodes )
                                                {
                                                    foreach ( var requiredParam in model.ParameterDictionary )
                                                    {
                                                        if ( requiredParam.Description == requiredParamNode.FirstChild.Value )
                                                        {
                                                            model.RequiredParams.Add( requiredParam );
                                                        }
                                                    }
                                                }
                                                break;

                                            case "HardCodedParams":
                                                foreach ( XmlNode hardCodedParamNode in reportParametersNode.ChildNodes )
                                                {
                                                    model.HardCodedParams.Add( model.ReportParameterCreator.GetNewWrapper( hardCodedParamNode.FirstChild.Value, hardCodedParamNode.Attributes["ID"].Value ) );
                                                }
                                                break;

                                            default:
                                                break;
                                        }
                                    }
                                }
                            }
                            break;

                        default:
                            break;
                    }
                }

                InitializeParams( model );
            }

            private static void InitializeParams( ReportsModel model )
            {
                model.Params = new Dictionary<string, ReportParameter>();
                foreach ( var item in model.RequiredParams )
                {
                    model.Params.Add( item.Description, item );
                }
                foreach ( var item in model.HardCodedParams )
                {
                    model.Params[item.Description].Value = item.Value;
                }
            }
        }
    }
}