﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure.Interfaces;
using Telerik.Windows.Controls;
using Telerik.Windows.Data;
using Cursors = System.Windows.Input.Cursors;
using UserControl = System.Windows.Controls.UserControl;

namespace ClinSchd.Modules.Reports.Reports
{
    /// <summary>
    /// Interaction logic for AuditReportView.xaml
    /// </summary>
    public partial class AuditReportView : UserControl
    {
        public AuditReportViewModel ViewModel { get; private set; }

        public AuditReportView(IDataAccessService dataAccessService)
        {
            ViewModel = new AuditReportViewModel(dataAccessService, this);
            //ViewModel.AsyncWrapper = asyncWrapper;
            DataContext = ViewModel;
            InitializeComponent();
        }

        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void CloseWindow()
        {
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void AuditReportView_Unloaded(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void ViewAuditReport_Click(object sender, RoutedEventArgs e)
        {
            ViewModel.GetAuditData();
        }

        private void ViewAuditGraph_Click(object sender, RoutedEventArgs e)
        {
            //Mouse.OverrideCursor = Cursors.Wait;
            //AuditReportGrid.Visibility = Visibility.Hidden;
            //AuditReportChart.Visibility = Visibility.Visible;
            //ViewModel.CreateChart();
            //Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void ExportAuditReport_Click(object sender, RoutedEventArgs e)
        {
            RadGridView grid = (RadGridView)AuditReportGrid;
            ObservableCollection<AuditRecord> temp = new ObservableCollection<AuditRecord>();
            /* Need to calulate the totals for the report */
            AuditRecord totals = new AuditRecord() { Scheduler = "Totals" };

            foreach (var item in grid.Items)
            {
                var req = item as AuditRecord;
                temp.Add(req);
                totals.PatientContacts += req.PatientContacts;
                totals.APPTEntries += req.APPTEntries;
                totals.APPTAppointmentsMade += req.APPTAppointmentsMade;
                totals.APPTClosed += req.APPTClosed;
                totals.EWLEntries += req.EWLEntries;
                totals.EWLAppointmentsMade += req.EWLAppointmentsMade;
                totals.EWLClosed += req.EWLClosed;
                totals.PtCSchEntries += req.PtCSchEntries;
                totals.PtCSchAppointmentsMade += req.PtCSchAppointmentsMade;
                //totals.ConsultEntries += req.ConsultEntries;
                totals.ConsultAppointmentsMade += req.ConsultAppointmentsMade;
                totals.AppointmentsCancelled += req.AppointmentsCancelled;
            }

            temp.Add(totals);
            IEnumerable<AuditRecord> reportData = temp;
            ExportReportToExcel.ExportReport<AuditRecord>("Audit Report", reportData, true);
        }

        private void AuditReportGrid_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);

            foreach (var col in AuditReportGrid.Columns)
            {
                switch (((string)(col.Header)))
                {
                    case "SchedulerIEN":
                        break;
                    case "Scheduler":
                        col.Footer = "Totals";
                        break;
                    default:
                        col.AggregateFunctions.Add(new SumFunction());
                        break;
                }
            }

            AuditReportGrid.UpdateLayout();
        }

        private void UserControl_Initialized(object sender, EventArgs e)
        {
            ViewModel.LoadScheduleUsers();
            CloseBtn.Focus();
            Keyboard.Focus(CloseBtn);
        }

        private void UserControl_PreviewKeyUp(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = (e.Key == Key.System ? e.SystemKey : e.Key);

                switch (key)
                {
                    case Key.S:
                        Keyboard.Focus(CmbSchedulerComboBox);
                        break;
                    case Key.V:
                        Keyboard.Focus(BtnViewAuditReport);
                        break;
                    case Key.E:
                        Keyboard.Focus(BtnExportAuditReport);
                        break;
                    case Key.C:
                        Keyboard.Focus(CloseBtn);
                        break;
                    case Key.G:
                        if (AuditReportGrid.Items.Count > 0)
                        {
                            AuditReportGrid.SelectedItem = AuditReportGrid.Items[0];
                            AuditReportGrid.ScrollIntoView(AuditReportGrid.SelectedItem);
                            AuditReportGrid.Focus();
                        }
                        break;
                }
            }
        }

        private void CloseBtn_LostFocus(object sender, RoutedEventArgs e)
        {
            CmbSchedulerComboBox.TabIndex = 100;
        }

        private void CloseBtn_GotFocus(object sender, RoutedEventArgs e)
        {
            CmbSchedulerComboBox.TabIndex = CloseBtn.TabIndex + 1;
        }

        private void AuditReportGrid_DataLoaded(object sender, EventArgs e)
        {
            if (AuditReportGrid.Items.Count > 0)
            {
                AuditReportGrid.SelectedItem = AuditReportGrid.Items[0];
                AuditReportGrid.ScrollIntoView(AuditReportGrid.SelectedItem);
                AuditReportGrid.Focus();
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            BtnViewAuditReport.Focus();
            Keyboard.Focus(BtnViewAuditReport);
        }
#if DEBUG
        private void ImportAuditReport_Click(object sender, RoutedEventArgs e)
        {
            ViewModel.ImportReportData();
        }
#endif
    }
}
