﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

//using Telerik.Windows.Controls.Charting;
using HorizontalAlignment = System.Windows.HorizontalAlignment;
using Keys = ClinSchd.Infrastructure.Keys;

#if DEBUG
using System.IO;
using System.Windows.Forms;
#endif

namespace ClinSchd.Modules.Reports.Reports
{
    public class AuditReportViewModel : AsyncValidatableModel
    {
        private readonly IDataAccessService _dataAccessService;
        private ObservableCollection<AuditRecord> _auditRecords = new ObservableCollection<AuditRecord>();
        private Dictionary<string, AuditRecord> _auditRecordsByUser = new Dictionary<string, AuditRecord>();
        private AuditReportView _view;
        private List<string> dispositionedEntries = new List<string>();
        private bool _isFormEnabled = true;
        private bool _isLoadingData = false;
        private bool _isBusy = false;
        private string _busyMessage = String.Empty;

        public AuditReportViewModel(IDataAccessService dataAccessService, AuditReportView view)
        {
            _dataAccessService = dataAccessService;
            _view = view;
        }
        
        public ObservableCollection<AuditRecord> AuditRecords
        {
            get { return _auditRecords; }
            set { _auditRecords = value; }
        }

//        public RadChart AuditChart { get; set; }

        //public void CreateChart()
        //{
        //    Telerik.Windows.Controls.RadChart telerikChart = new Telerik.Windows.Controls.RadChart();    //Chart Title
        //    telerikChart.DefaultView.ChartTitle.Content = "Year 2014";
        //    telerikChart.DefaultView.ChartTitle.HorizontalAlignment = HorizontalAlignment.Center;    //Chart Legend
        //    telerikChart.DefaultView.ChartLegend.UseAutoGeneratedItems = true;    //Line Chart
        //    DataSeries lineSeries = new DataSeries();
        //    lineSeries.LegendLabel = "Turnover";
        //    lineSeries.Definition = new LineSeriesDefinition();
        //    lineSeries.Add(new DataPoint() { YValue = 154, XCategory = "Jan" });
        //    lineSeries.Add(new DataPoint() { YValue = 138, XCategory = "Feb" });
        //    lineSeries.Add(new DataPoint() { YValue = 143, XCategory = "Mar" });
        //    lineSeries.Add(new DataPoint() { YValue = 120, XCategory = "Apr" });
        //    lineSeries.Add(new DataPoint() { YValue = 135, XCategory = "May" });
        //    lineSeries.Add(new DataPoint() { YValue = 125, XCategory = "Jun" });
        //    lineSeries.Add(new DataPoint() { YValue = 179, XCategory = "Jul" });
        //    lineSeries.Add(new DataPoint() { YValue = 170, XCategory = "Aug" });
        //    lineSeries.Add(new DataPoint() { YValue = 198, XCategory = "Sep" });
        //    lineSeries.Add(new DataPoint() { YValue = 187, XCategory = "Oct" });
        //    lineSeries.Add(new DataPoint() { YValue = 193, XCategory = "Nov" });
        //    lineSeries.Add(new DataPoint() { YValue = 176, XCategory = "Dec" });
        //    telerikChart.DefaultView.ChartArea.DataSeries.Add(lineSeries);    //Bar Chart
        //    DataSeries barSeries = new DataSeries();
        //    barSeries.LegendLabel = "Expenses";
        //    barSeries.Definition = new BarSeriesDefinition();
        //    barSeries.Add(new DataPoint() { YValue = 45, XCategory = "Jan" });
        //    barSeries.Add(new DataPoint() { YValue = 48, XCategory = "Feb" });
        //    barSeries.Add(new DataPoint() { YValue = 53, XCategory = "Mar" });
        //    barSeries.Add(new DataPoint() { YValue = 41, XCategory = "Apr" });
        //    barSeries.Add(new DataPoint() { YValue = 32, XCategory = "May" });
        //    barSeries.Add(new DataPoint() { YValue = 28, XCategory = "Jun" });
        //    barSeries.Add(new DataPoint() { YValue = 63, XCategory = "Jul" });
        //    barSeries.Add(new DataPoint() { YValue = 74, XCategory = "Aug" });
        //    barSeries.Add(new DataPoint() { YValue = 77, XCategory = "Sep" });
        //    barSeries.Add(new DataPoint() { YValue = 85, XCategory = "Oct" });
        //    barSeries.Add(new DataPoint() { YValue = 89, XCategory = "Nov" });
        //    barSeries.Add(new DataPoint() { YValue = 80, XCategory = "Dec" });
        //    telerikChart.DefaultView.ChartArea.DataSeries.Add(barSeries);
        //    AuditChart = telerikChart;
        //}

        private List<ScheduleUser> _scheduleUsers = null;

        public List<ScheduleUser> ScheduleUsers
        {
            get { return _scheduleUsers; }
        }

        private DateTime? _startDate = DateTime.Today.AddDays(-6);
        private DateTime _auditReportEndDateSelectableDateStart = DateTime.Today.AddDays(-6);
        private DateTime _auditReportEndDateSelectableDateEnd = DateTime.Today;
        private DateTime? _endDate = DateTime.Today;
        private bool _criteriaHasChanged = true;
        private bool _hasAllRecords = false;
        private DateTime _lastAllRead = DateTime.MinValue;

        public DateTime? StartDate
        {
            get { return _startDate; }
            set
            {
                if (value != _startDate)
                {
                    _startDate = value;
                    _criteriaHasChanged = true;
                    NotifyPropertyChanged("StartDate");

                    if (_startDate.HasValue)
                    {
                        DateTime newEndDate = _startDate.Value.AddDays(6);

                        if (DateTime.Compare(newEndDate, DateTime.Today) > 0)
                            newEndDate = DateTime.Today;

                        AuditReportEndDateSelectableDateStart = _startDate.Value;
                        AuditReportEndDateSelectableDateEnd = newEndDate;
                        EndDate = newEndDate;
                    }
                }
            }
        }

        public DateTime AuditReportEndDateSelectableDateEnd
        {
            get { return _auditReportEndDateSelectableDateEnd; }
            set
            {
                _auditReportEndDateSelectableDateEnd = value;
                NotifyPropertyChanged("AuditReportEndDateSelectableDateEnd");
            }
        }

        public DateTime AuditReportEndDateSelectableDateStart
        {
            get { return _auditReportEndDateSelectableDateStart; }
            set
            {
                _auditReportEndDateSelectableDateStart = value;
                NotifyPropertyChanged("AuditReportEndDateSelectableDateStart");
            }
        }

        public DateTime? EndDate
        {
            get { return _endDate; }
            set
            {
                if (value != _endDate)
                {
                    _endDate = value;
                    _criteriaHasChanged = true;
                    NotifyPropertyChanged("EndDate");
                }
            }
        }

        private ScheduleUser _selectedScheduleUser = null;

        public ScheduleUser SelectedScheduleUser
        {
            get { return _selectedScheduleUser; }
            set
            {
                if (_selectedScheduleUser.Ien != value.Ien)
                {
                    _selectedScheduleUser = value;
                    _criteriaHasChanged = true;
                    NotifyPropertyChanged("SelectedScheduleUser");
                    GetAuditData();
                }
            }
        }

        private void CompileAuditRecord(AuditData aData)
        {
            // Find the Audit Record for the primary scheduler in this data record
            AuditRecord auditRec = FindAuditRecord(aData.UserIEN, aData.UserName);

            // Patient Contacts - Each Patient Contact record has its own Scheduler Information, 
            //      so need to iterate through the list and accumulate for each individual Scheduler
            //      Patient Contacts are Request Type independent.
            if (aData.Contacts != null)
                foreach (var pt in aData.Contacts)
                    FindAuditRecord(pt.EnteredByUserIEN, pt.EnteredByUserName).PatientContacts++;

            switch (aData.RequestType)
            {
                case Keys.AppointmentRequestTypes.Appt:
                case Keys.AppointmentRequestTypes.Veteran:
                case Keys.AppointmentRequestTypes.WelcomeToVa:
                case Keys.AppointmentRequestTypes.ReturnToClinic:
                    // APPT Entry - based on the scheduler in the current data record
                    if (aData.Date.HasValue)
                    {
                        if (aData.IsMRTCParent)
                            auditRec.MRTCOpened++;
                        else
                            auditRec.APPTEntries++;
                    }

                    // APPT Appointment
                    if (aData.Date2 != null)
                    {
                        if (String.IsNullOrEmpty(aData.Date2.ClerkIEN))
                            auditRec.APPTAppointmentsMade++;
                        else
                            FindAuditRecord(aData.Date2.ClerkIEN, aData.Date2.ClerkName).APPTAppointmentsMade++;
                    }

                    // APPT Requests Closed - based on the scheduler in the current data record
                    if (aData.Date1.HasValue)
                    {
                        string key = String.Format("{0}^{1}^{2}^{3}", aData.RequestType, aData.UserIEN, aData.IEN, aData.Date1);

                        if (!dispositionedEntries.Contains(key))
                        {
                            if (aData.IsMRTCParent)
                                auditRec.MRTCClosed++;
                            else
                                auditRec.APPTClosed++;

                            dispositionedEntries.Add(key);
                        }
                    }

                    break;
                case Keys.AppointmentRequestTypes.Ewl:
                    // EWL Entry - based on the scheduler in the current data record
                    if (aData.Date.HasValue)
                        auditRec.EWLEntries++;

                    // EWL Appointment
                    if (aData.Date2 != null)
                    {
                        if (String.IsNullOrEmpty(aData.Date2.ClerkIEN))
                            auditRec.EWLAppointmentsMade++;
                        else
                            FindAuditRecord(aData.Date2.ClerkIEN, aData.Date2.ClerkName).EWLAppointmentsMade++;
                    }

                    // EWL Requests Closed - based on the scheduler in the current data record
                    if (aData.Date1.HasValue)
                    {
                        string key = String.Format("{0}^{1}^{2}^{3}", aData.RequestType, aData.UserIEN, aData.IEN, aData.Date1);

                        if (!dispositionedEntries.Contains(key))
                        {
                            auditRec.EWLClosed++;
                            dispositionedEntries.Add(key);
                        }
                    }

                    break;
                case Keys.AppointmentRequestTypes.PtCSch:
                case Keys.AppointmentRequestTypes.Procedure:
                    // PtCSch Entry - based on the scheduler in the current data record
                    if (aData.Date.HasValue)
                        auditRec.PtCSchEntries++;

                    // PtCSch Appointments Made
                    if (aData.PtCSch != null)
                    {
                        if (String.IsNullOrEmpty(aData.PtCSch.ClerkIEN))
                            auditRec.PtCSchAppointmentsMade++;
                        else
                            FindAuditRecord(aData.PtCSch.ClerkIEN, aData.PtCSch.ClerkName).PtCSchAppointmentsMade++;
                    }

                    break;
                case Keys.AppointmentRequestTypes.Consult:
                    // CONSULT Appointments Made
                    if (aData.Date2 != null)
                    {
                        if (String.IsNullOrEmpty(aData.Date2.ClerkIEN))
                            auditRec.ConsultAppointmentsMade++;
                        else
                            FindAuditRecord(aData.Date2.ClerkIEN, aData.Date2.ClerkName).ConsultAppointmentsMade++;
                    }

                    break;
            }
        }

        public bool IsExportButtonEnabled
        {
            get { return (!_isLoadingData && (AuditRecords.Count > 0)); }
        }

#if DEBUG
        public void GetAuditData(string sdecSummGetFileName = "", string sdecSummAGetFileName = "")
#else
        public void GetAuditData()
#endif
        {
            // Is there a reason to load\reload the data ?
            if (!_criteriaHasChanged && !_hasAllRecords && (_lastAllRead.AddMinutes(30).CompareTo(DateTime.Now) >= 0))
                return;

            _isLoadingData = true;
            NotifyPropertyChanged("IsExportButtonEnabled");

            // Clear the data store
            _auditRecords.Clear();

            foreach (KeyValuePair<string, AuditRecord> kvp in _auditRecordsByUser)
                kvp.Value.Clear();

            // initialize a filter
            string selectedFilter = String.Empty;

            if (SelectedScheduleUser != null)
                if (SelectedScheduleUser.Name.ToUpper() != "ALL")
                    selectedFilter = SelectedScheduleUser.Ien;

            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
#if DEBUG
                        if (string.IsNullOrEmpty(sdecSummGetFileName) || string.IsNullOrEmpty(sdecSummAGetFileName))
#endif
                            args.Result = _dataAccessService.GetAuditReportData(StartDate, EndDate, selectedFilter);
#if DEBUG
                        else
                            args.Result = ReadReportFiles(sdecSummGetFileName, sdecSummAGetFileName);
#endif
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        try
                        {
                            if (args.Result != null)
                            {
                                List<DataTable> resultsList = (List<DataTable>)args.Result;

                                if (resultsList.Count >= 1)
                                {
                                    DataTable dt = resultsList[0];
                                    _hasAllRecords = ((dt.Rows.Count > 0) && String.IsNullOrEmpty(selectedFilter));

                                    if (_hasAllRecords)
                                        _lastAllRead = DateTime.Now;

                                    _criteriaHasChanged = false;
                                    List<AuditData> auditDataList = new List<AuditData>();
                                    dispositionedEntries.Clear();

                                    foreach (DataRow row in dt.Rows)
                                    {
                                        AuditData aData = new AuditData(row, (StartDate.HasValue ? StartDate.Value : DateTime.MinValue), (EndDate.HasValue ? EndDate.Value : DateTime.MaxValue));
                                        CompileAuditRecord(aData);
                                    }

                                    // Add Appointments Cancelled count by Scheduler
                                    if (resultsList.Count > 1)
                                    {
                                        DataTable dt1 = resultsList[1];

                                        foreach (KeyValuePair<string, AuditRecord> kvp in _auditRecordsByUser)
                                            kvp.Value.AppointmentsCancelled = dt1.Select("USERIEN = '" + kvp.Key + "'").Count();
                                    }

                                    foreach (KeyValuePair<string, AuditRecord> kvp in _auditRecordsByUser)
                                    {
                                        if (String.IsNullOrEmpty(selectedFilter))
                                            _auditRecords.Add(kvp.Value);
                                        else
                                        {
                                            if (kvp.Key == selectedFilter)
                                                _auditRecords.Add(kvp.Value);
                                        }
                                    }
                                }
                            }
                        }
                        finally
                        {
                            if (_auditRecords.Count == 0)
                                Keys.AlertUser(_view, "No audit data available.", "Audit Report");

                            _isLoadingData = false;
                            NotifyPropertyChanged("IsExportButtonEnabled");
                        }
                    }
                },
                true,
                false,
                "Loading Audit Data",
                String.Empty/*,
                TimeSpan.FromSeconds(3)*/
            );
        }

        private AuditRecord FindAuditRecord(string userIen, string userName)
        {
            AuditRecord auditRecord;
            userIen = (String.IsNullOrEmpty(userIen) ? "UNKNOWN" : userIen);

            if (_auditRecordsByUser.ContainsKey(userIen))
                auditRecord = _auditRecordsByUser[userIen];
            else
            {
                auditRecord = new AuditRecord();

                if (userIen == "UNKNOWN")
                {
                    auditRecord.Scheduler = "UNKNOWN SCHEDULER";
                    auditRecord.SchedulerIEN = String.Empty;
                }
                else
                {
                    auditRecord.Scheduler = userName;
                    auditRecord.SchedulerIEN = userIen;
                }

                _auditRecordsByUser.Add(userIen, auditRecord);
            }

            return auditRecord;
        }

#if DEBUG
        public void LoadScheduleUsers(string usersFileName = "")
#else
        public void LoadScheduleUsers()
#endif
        {
            IsFormEnabled = false;
            _view.asyncWrapper.ShowBusyIndicator("Loading Scheduler Users", String.Empty, false);
            _auditRecordsByUser.Clear();

            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
#if DEBUG
                        if (string.IsNullOrEmpty(usersFileName))
#endif
                            args.Result = _dataAccessService.GetSchedulers();
#if DEBUG
                        else
                            args.Result = ReadUserFile(usersFileName);
#endif
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        if (args.Result != null)
                        {
                            List<ScheduleUser> scheduleUsers = (List<ScheduleUser>)args.Result;

                            foreach (ScheduleUser user in scheduleUsers)
                                FindAuditRecord(user.Ien, user.Name);

                            var sUser = new ScheduleUser();
                            sUser.Name = "ALL";
                            sUser.Ien = "0";
                            scheduleUsers.Add(sUser);
                            _scheduleUsers = scheduleUsers.OrderBy(c => c.Name).ToList();
                            _selectedScheduleUser = sUser;
                            NotifyPropertyChanged("ScheduleUsers");
                        }

                        IsFormEnabled = true;
                        _view.asyncWrapper.HideBusyIndicator();
                    }
                },
                false
            );
        }

        public bool IsFormEnabled
        {
            get { return _isFormEnabled; }
            set
            {
                _isFormEnabled = value;
                NotifyPropertyChanged("IsFormEnabled");
            }
        }

        private class AuditData
        {
            private DateTime StartDate;
            private DateTime EndDate;

            public class ApptInfo
            {
                public string ApptDate { get; private set; }
                public string ApptClerkIen { get; private set; }
                public string ApptClerkName { get; private set; }
                public string EnteredDate { get; private set; }

                public ApptInfo(string source)
                {
                    ApptDate = String.Empty;
                    ApptClerkIen = String.Empty;
                    ApptClerkName = String.Empty;
                    EnteredDate = String.Empty;

                    if (!String.IsNullOrEmpty(source))
                        if (source.Contains("~~"))
                        {
                            string[] sourceList = source.Split(new string[] {"~~"}, StringSplitOptions.None);
                            ApptDate = sourceList[0];
                            ApptClerkIen = sourceList[11];
                            ApptClerkName = sourceList[12];
                            EnteredDate = sourceList[16];
                        }
                }
            }

            public class ContactInfo
            {
                public string DateEntered { get; private set; }
                public string EnteredByUserIEN { get; private set; }
                public string EnteredByUserName { get; private set; }
                public string Action { get; private set; }

                public ContactInfo(string source)
                {
                    DateEntered = String.Empty;
                    EnteredByUserIEN = String.Empty;
                    EnteredByUserName = String.Empty;
                    Action = String.Empty;

                    if (!String.IsNullOrEmpty(source))
                        if (source.Contains("~~"))
                        {
                            string[] sourceList = source.Split(new string[] { "~~" }, StringSplitOptions.None);
                            DateEntered = sourceList[0];
                            EnteredByUserIEN = sourceList[1];
                            EnteredByUserName = sourceList[2];
                            Action = sourceList[3];
                        }
                        else
                            DateEntered = source;
                }
            }

            public class PtCSchInfo
            {
                public string AppointmentDate { get; private set; }
                public string DeleteDate { get; private set; }
                public string ClerkIEN { get; private set; }
                public string ClerkName { get; private set; }

                public PtCSchInfo(string source)
                {
                    AppointmentDate = String.Empty;
                    DeleteDate = String.Empty;
                    ClerkIEN = String.Empty;
                    ClerkName = String.Empty;

                    if (!String.IsNullOrEmpty(source))
                    {
                        if (source.Contains("|"))
                        {
                            string[] sourceList = source.Split('|');
                            AppointmentDate = sourceList[0];
                            DeleteDate = sourceList[1];
                            ClerkIEN = sourceList[2];
                            ClerkName = sourceList[3];
                        }
                        else
                            AppointmentDate = source;
                    }
                }
            }

            public class ActivityInfo
            {
                public string Request_ConsultIEN { get; private set; }
                public string ProcessActivityIEN { get; private set; }
                public string ActivityDate { get; private set; }
                public string Activity { get; private set; }
                public string EnteredByIEN { get; private set; }
                public string EnteredByName { get; private set; }

                public ActivityInfo(string source)
                {
                    Request_ConsultIEN = String.Empty;
                    ProcessActivityIEN = String.Empty;
                    ActivityDate = String.Empty;

                    Activity = String.Empty;
                    EnteredByIEN = String.Empty;
                    EnteredByName = String.Empty;

                    if (!String.IsNullOrEmpty(source))
                        if (source.Contains(";;"))
                        {
                            string[] sourceList = source.Split(new string[] { ";;" }, StringSplitOptions.None);
                            Request_ConsultIEN = sourceList[0];
                            ProcessActivityIEN = sourceList[1];
                            ActivityDate = sourceList[2];
                            Activity = sourceList[3];
                            EnteredByIEN = sourceList[4];
                            EnteredByName = sourceList[5];
                        }
                        else
                            Request_ConsultIEN = source;
                }
            }
            private string requestType;
            public string RequestType
            {
                get { return requestType; }
                private set
                {
                    if (value.ToUpper() == "RECALL")
                        requestType = "PtCSch";
                }
            }
            public string PatientIEN { get; private set; }
            public string PatientName { get; private set; }
            public DateTime? Date { get; private set; }
            public string UserIEN { get; private set; }
            public string UserName { get; private set; }
            public DateTime? Date1 { get; private set; }
            public string ProviderIEN { get; private set; }
            public string ProviderName { get; private set; }
            public IList<ContactInfo> Contacts { get; private set; }
            public ApptInfo SchedAppt { get; private set; }
            public PtCSchInfo PtCSch { get; private set; }
            public PtCSchInfo Date2 { get { return PtCSch; } }
            public string ClinicIEN { get; private set; }
            public string ClinicName { get; private set; }
            public IList<ActivityInfo> Activity { get; private set; }
            public string LastSub { get; private set; }
            public string IEN { get; private set; }
            public bool IsMRTCParent { get; private set; }

            public AuditData(DataRow row, DateTime startDate, DateTime endDate)
            {
                StartDate = startDate;
                EndDate = endDate;
                RequestType = row["REQUESTTYPE"].ToString();
                PatientIEN = row["DFN"].ToString();
                PatientName = row["NAME"].ToString();

                if (!String.IsNullOrEmpty(row["DATE"].ToString()))
                {
                    Date = ConvertExternalDateFormat(row["DATE"].ToString());

                    // Only include if within the report date range
                    if (Date.HasValue)
                        if (!IncludeInReport(Date.Value))
                            Date = null;
                }
                else
                    Date = null;

                UserIEN = row["USERIEN"].ToString();
                UserName = row["USERNAME"].ToString();

                if (!String.IsNullOrEmpty(row["DATE1"].ToString()))
                {
                    Date1 = ConvertExternalDateFormat(row["DATE1"].ToString());

                    // Only include if within the report date range
                    if (Date1.HasValue)
                        if (!IncludeInReport(Date1.Value))
                            Date1 = null;
                }
                else
                    Date1 = null;

                ProviderIEN = row["PROVIEN"].ToString();
                ProviderName = row["PROVNAME"].ToString();
                LastSub = row["LASTSUB"].ToString();
                IEN = row["IEN"].ToString();

                Contacts = new List<ContactInfo>();

                if (!String.IsNullOrEmpty(row["PCONTACT"].ToString()))
                {
                    string[] contacts = row["PCONTACT"].ToString().Split(new string[] { "::" }, StringSplitOptions.None);

                    foreach(string contact in contacts)
                        if (!String.IsNullOrEmpty(contact))
                        {
                            ContactInfo ci = new ContactInfo(contact);
                            DateTime dateEntered = Convert.ToDateTime(ci.DateEntered.Replace("@", " "));

                            // Only add entries that are within the filter date range
                            if (IncludeInReport(dateEntered))
                                Contacts.Add(ci);
                        }
                }

                if (!String.IsNullOrEmpty(row["APPT_SCHED_DATE"].ToString()))
                {
                    SchedAppt = new ApptInfo(row["APPT_SCHED_DATE"].ToString());

                    // Do not keep this is it was not entered within the report date range
                    if (!String.IsNullOrEmpty(SchedAppt.EnteredDate))
                        if (!IncludeInReport(ConvertExternalDateFormat(SchedAppt.EnteredDate).Value.Date))
                            SchedAppt = null;
                }
                else
                    SchedAppt = null;

                PtCSch = null;

                if (!String.IsNullOrEmpty(row["DATE2"].ToString()))
                {
                    PtCSchInfo ptCSch = new PtCSchInfo(row["DATE2"].ToString());

                    if (!String.IsNullOrEmpty(ptCSch.AppointmentDate))
                        if (IncludeInReport(Keys.Methods.ConvertFileManDateToDateTime(ptCSch.AppointmentDate)))
                            PtCSch = ptCSch;
                }

                ClinicIEN = row["CLINIEN"].ToString();
                ClinicName = row["CLINNAME"].ToString();
                Activity = new List<ActivityInfo>();

                if (!String.IsNullOrEmpty(row["ACTIVITY"].ToString()))
                {
                    string[] activities = row["ACTIVITY"].ToString().Split('|');

                    foreach(string activity in activities)
                        if (!String.IsNullOrEmpty(activity))
                        {
                            ActivityInfo a = new ActivityInfo(activity);
                            DateTime? activityDate = ConvertExternalDateFormat(a.ActivityDate);
                            
                            // Only include Activities entered within the date filter range
                            if (activityDate.HasValue)
                                if (IncludeInReport(activityDate.Value))
                                    Activity.Add(a);
                        }
                }

                if (row.Table.Columns.Contains("MRTC"))
                    IsMRTCParent = (row["MRTC"].ToString() == "1");
                else
                    IsMRTCParent = false;
            }

            private DateTime? ConvertExternalDateFormat(string source)
            {
                DateTime result;
                
                if (Keys.Methods.TryParseDateTime(source, out result))
                    return (DateTime?)result;
                else
                    return null;
            }

            private bool IncludeInReport(DateTime dateTime)
            {
                return ((dateTime.Date.CompareTo(StartDate.Date) >= 0) && (dateTime.Date.CompareTo(EndDate.Date) <= 0));
            }
        }

#if DEBUG
        private string _targetFilePath = "c:\\";

        private List<DataTable> ReadReportFiles(string sdecSummGetFileName, string sdecSummAGetFileName)
        {
            List<DataTable> results = new List<DataTable>();

            string[] summGet_FileData = File.ReadAllLines(sdecSummGetFileName);
            string[][] summGet_res = ParseData(summGet_FileData);
            results.Add(PopulateDataTable(summGet_res));

            string[] summAGet_FileData = File.ReadAllLines(sdecSummAGetFileName);
            string[][] sumAGet_res = ParseData(summAGet_FileData);
            results.Add(PopulateDataTable(sumAGet_res));

            return results;
        }

        private string[][] ParseData(string[] data)
        {
            List<string[]> result = new List<string[]>();

            for(int lineNo = 0; lineNo < data.Count(); lineNo++)
            {
                string[] parts = data[lineNo].Split('^');

                if (lineNo == 0)
                    for (int idx = 0; idx < parts.Count(); idx++)
                        parts[idx] = parts[idx].Substring(6);

                result.Add(parts);
            }

            return result.ToArray();
        }

        private DataTable PopulateDataTable(string[][] data_res)
        {
            if (data_res == null)
                return null;
            DataTable dt = new DataTable();

            // used to get the header information returned from the RPC command.
            int count = 0;

            // loop through the RPC command results array and add data to the DataTable
            foreach (var p in data_res)
            {
                if (p == null)
                    continue;

                // Check for DataTable header information
                if (count == 0)
                {

                    // loop through each header item
                    foreach (string st in p)
                    {

                        // add each header item to the DataTable object
                        dt.Columns.Add(st, typeof(string));
                    }

                    // increment the count to avoid adding header info again
                    count++;
                }
                else if (count > 2)
                {

                    // DataTable row
                    DataRow dr = dt.NewRow();
                    for (int i = 0; i < p.Length; i++)
                    {
                        dr[i] = p[i];
                    }

                    // add the DataRow to the DataTable
                    dt.Rows.Add(dr);
                }
                else
                {

                    // increment the count to avoid rows 1 and 2
                    count++;
                }
            }

            ////Console.WriteLine("Call Successful");
            //WriteToConsole("Call Successful");

            // return the DataTable
            return dt;
        }

        private List<ScheduleUser> ReadUserFile(string usersFileName)
        {
            //Get Schedule Users
            List<string> lines = File.ReadAllLines(usersFileName).ToList();
            var schedUsers = new List<ScheduleUser>();
            bool isFirstLine = true;

            foreach (string row in lines)
            {
                if (isFirstLine)
                {
                    isFirstLine = false;
                    continue;
                }

                string[] parts = row.Split('^');
                var schedUser = new ScheduleUser
                {
                    Ien = parts[0],
                    Name = parts[1]
                };

                schedUsers.Add(schedUser);
            }

            return schedUsers;
        }

        public void ImportReportData()
        {
            string sdecSummGetFileName = string.Empty;
            string sdecSummAGetFileName = string.Empty;
            string usersFileName = string.Empty;

            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.Title = "Select Users File";
                openFileDialog.InitialDirectory = _targetFilePath;
                openFileDialog.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
                openFileDialog.FilterIndex = 0;
                openFileDialog.RestoreDirectory = true;

                if (openFileDialog.ShowDialog() != DialogResult.OK)
                    return;

                usersFileName = openFileDialog.FileName;
                _targetFilePath = Path.GetDirectoryName(usersFileName);
                openFileDialog.InitialDirectory = _targetFilePath;
                LoadScheduleUsers(usersFileName);

                openFileDialog.Title = "Select SDEC SUMMGET File Name";

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                    sdecSummGetFileName = openFileDialog.FileName;

                openFileDialog.Title = "Select SDEC SUMMAGET File Name";

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                    sdecSummAGetFileName = openFileDialog.FileName;
            }

            if (string.IsNullOrEmpty(sdecSummGetFileName) || string.IsNullOrEmpty(sdecSummAGetFileName))
                return;

            GetAuditData(sdecSummGetFileName, sdecSummAGetFileName);
        }
#endif

        public System.Windows.Visibility ImportButtonVisibility
        {
            get
            {
#if DEBUG
                return System.Windows.Visibility.Visible;
#else
                return System.Windows.Visibility.Collapsed;
#endif
            }
        }
    }
}
