﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Input;
using Cursors = System.Windows.Input.Cursors;

using ClosedXML.Excel;

namespace ClinSchd.Modules.Reports.Reports
{
    public static class ExportReportToExcel
    {
        public static void ExportReport<T>(string reportName, IEnumerable<T> reportData, bool colorFillLastRow = false)
        {
            System.Threading.Thread.Sleep(200);
            string extension = "xlsx";
            string validExtensions = ".xlsx or .xlsm ";
            string fileName= string.Empty;

            var dialog = new SaveFileDialog()
            {
                FileName = reportName + "." + extension,
                DefaultExt = extension,
                Filter = String.Format("{1} files (*.{0})|*.{0}", extension, "Excel"),
                FilterIndex = 1,
                AddExtension = true
            };

            while (true)
            {
                bool? save = dialog.ShowDialog();

                if (!save.HasValue || (save.HasValue && !save.Value))
                    return;

                string ext = Path.GetExtension(dialog.FileName);

                if (validExtensions.Contains(ext + " "))
                    break;

                System.Windows.MessageBox.Show(
                    String.Format("\"{0}\" is an invalid export file extension, please use one of \"{1}\".", ext, validExtensions.TrimEnd()),
                    "Invalid Export File Name",
                    System.Windows.MessageBoxButton.OK,
                    System.Windows.MessageBoxImage.Error
                );
            }

            System.Windows.Input.Cursor holdCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Wait;

            try
            {

                int lastRow = reportData.Count() + 1;
                var workbook = new XLWorkbook();
                var worksheet = workbook.Worksheets.Add(reportName);
                worksheet.Cell("A1").InsertTable(reportData);
                worksheet.Columns().AdjustToContents(10, 100);

                foreach (var col in worksheet.Columns())
                {
                    int colNumber = col.ColumnNumber();
                    //int minWidth = worksheet.Cell(1, colNumber).Value.ToString().Length;
                    double maxDigitWidth = worksheet.Cell(1, colNumber).Style.Font.GetWidth("W", new Dictionary<IXLFontBase, Font>());
                    double minWidth = (((worksheet.Cell(1, colNumber).Value.ToString().Length * maxDigitWidth) + 10) / maxDigitWidth * 256) /256;
                    col.AdjustToContents(minWidth, 100);
                    worksheet.Cell(1, colNumber).Style.Fill.BackgroundColor = XLColor.BabyBlue;
                    worksheet.Cell(1, colNumber).Style.Font.FontColor = XLColor.Black;

                    string tmp1;
                    worksheet.Cell(1, colNumber).TryGetValue(out tmp1);
                    if (tmp1 == "DESIRED_PtCSch_DATE")
                    {
                        worksheet.Cell(1, colNumber).Value = "CID/PREFERRED DATE";
                    }
                    

                    if (colorFillLastRow)
                    {
                        worksheet.Cell(lastRow, colNumber).Style.Fill.BackgroundColor = XLColor.BabyBlue;
                        worksheet.Cell(lastRow, colNumber).Style.Font.FontColor = XLColor.Black;
                    }
                }

                workbook.SaveAs(dialog.FileName);
            }
            finally
            {
                if (holdCursor == null)
                    Mouse.OverrideCursor = Cursors.Arrow;
                else
                    Mouse.OverrideCursor = holdCursor;
            }
        }
    }
}
