﻿using System.Collections.Generic;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using Telerik.Windows.Controls;

namespace ClinSchd.Modules.Reports.Reports
{
    /// <summary>
    /// Interaction logic for NonDisplayPatients.xaml
    /// </summary>
    public partial class NonDisplayPatients : UserControl
    {
        public NonPatientsDisplayViewModel ViewModel { get; }
        private readonly Timer _patientSearchTimer;
        public string PatientSearchString { get; set; }
        public NonDisplayPatients(IDataAccessService dataAccessService)
        {
            InitializeComponent();
            ViewModel = new NonPatientsDisplayViewModel(dataAccessService)
                {NonDisplayPatientRequests = new List<NonDisplayPatientRequest>()};
            DataContext = ViewModel;
            _patientSearchTimer = new Timer(Keys.SearchTimerInterval);
            _patientSearchTimer.Elapsed += patientSearchTimer_Elapsed;
        }

        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void CloseWindow()
        {
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        void patientSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            TimerFired();
        }
        private void TimerFired()
        {
            //newcode to work with TimeTrader APIs
            //patientSearchTimer.Stop();
            //var ttWebServices = new ClinSchd.Infrastructure.TimeTraderServices.TimeTraderWebServices();
            //var result = ttWebServices.GetClientServices("John");



            // Current Code to work with RPMS..
            _patientSearchTimer.Stop();
            //Dispatcher.BeginInvoke((Action)(() => ViewModel.SearchStringChanged(PatientSearchString)));
            ViewModel.SearchStringChanged(PatientSearchString);

        }

        private void RadComboBox_KeyUp(object sender, KeyEventArgs e)
        {
            _patientSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _patientSearchTimer.Start();
        }
        private void PatientSelection_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (((RadComboBox)sender).Text == "ClinSchd.Infrastructure.Models.Patient")
            {
                ((RadComboBox)sender).Text = "";
                return;
            }
            if (PatientSearchString == "ClinSchd.Infrastructure.Models.Patient")
                PatientSearchString = "";

            //patientSearchString = ((RadComboBox)sender).Text.Replace(" ", "");
            PatientSearchString = ((RadComboBox)sender).Text;
            if (!((RadComboBox)sender).IsDropDownOpen)
            {
                ((RadComboBox)sender).IsDropDownOpen = true;
            }
        }

        private void View_Click(object sender, RoutedEventArgs e)
        {
            //MessageBox.Show(CmbClinicComboBox.SelectedValue.ToString());
            Mouse.OverrideCursor = Cursors.Wait;
            var patientIen = "";
            var clinicIen = "";

             if (PatientSearchResults.SelectedValue == null && CmbClinicComboBox.SelectedValue == null)
             {
                MessageBox.Show("Please select a patient or clinic.");
                return;
             }
            

            if (PatientSearchResults.SelectedValue != null)
                patientIen = PatientSearchResults.SelectedValue.ToString();
            if (CmbClinicComboBox.SelectedValue != null)
                clinicIen = CmbClinicComboBox.SelectedValue.ToString();


            ViewModel.NonDisplayPatientRequests = ViewModel.DataAccessService.GetNonDisplayPatientRequests(patientIen, clinicIen);
            Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void Print_Click(object sender, RoutedEventArgs e)
        {

        }

        private void Clinics_Changed(object sender, SelectionChangedEventArgs e)
        {
            //cmbPatientSearchComboBox.Text = "";
        }

        private void FindPatient_Click(object sender, RoutedEventArgs e)
        {
            //PatientSearchResults.Items.Clear();
            if(ViewModel.PatientList != null)
                ViewModel.PatientList.Clear();
            ViewModel.PatientList = ViewModel.DataAccessService.GetPatients(PatientSearchText.Text, Keys.MaxPatientsToReturn);
        }
        private void GetMatchingPatients()
        {
            //PatientSearchResults.Items.Clear();
            if (ViewModel.PatientList != null)
                ViewModel.PatientList.Clear();

            var blankPat = new Patient(ViewModel.DataAccessService) { Name = "" };
            ViewModel.PatientList = ViewModel.DataAccessService.GetPatients(PatientSearchText.Text, Keys.MaxPatientsToReturn);
            ViewModel.PatientList.Add(blankPat);
        }
        private void PatientSearchKey_Down(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Return || e.Key == Key.Tab)
            {
                if (PatientSearchText.Text.Length > 3)
                    GetMatchingPatients();
                e.Handled = true;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
        }
    }
}
