using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Threading;
using System.Timers;
using Telerik.Windows.Controls;
using ClinSchd.Infrastructure.Interfaces;
using KeyEventArgs = System.Windows.Input.KeyEventArgs;
using Keys = ClinSchd.Infrastructure.Keys;
using TextBox = System.Windows.Controls.TextBox;

namespace ClinSchd.Modules.Reports.Reports
{
    /// <summary>
    /// Interaction logic for ReportsView.xaml
    /// </summary>
    public partial class ReportsView : IReportsView
    {
        public ReportsView(IDataAccessService dataAccessService)
        {
            InitializeComponent();
            Owner = Keys.MainWindow;
            _dataAccessService = dataAccessService;

            _servicePrinterSearchTimer = new System.Timers.Timer(Keys.SearchTimerInterval);
            _servicePrinterSearchTimer.Elapsed += resourceSearchTimer_Elapsed;

        }

        private readonly IDataAccessService _dataAccessService;

        private IReportsPresentationModel Model
        {
            get { return (IReportsPresentationModel)DataContext; }
        }

        public TextBox ReportsTextBox
        {
            get { return ReportTextBox; }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
        }

        private void ServerPrinterCmb_KeyUp(object sender, KeyEventArgs e)
        {
            ServerPrinterCmb.IsDropDownOpen = false;
            _servicePrinterSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _servicePrinterSearchTimer.Start();
        }

        private void ServerPrinterCmb_TextChanged(object sender, TextChangedEventArgs e)
        {
            var cbx = (RadComboBox)sender;
            _printerSearchString = cbx.Text;
        }

        private void ServerPrinterCmb_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            SetPrinterType();
        }

        private readonly System.Timers.Timer _servicePrinterSearchTimer;
        private string _printerSearchString;

        void resourceSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _servicePrinterSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => ServicePrinterSearchChanged(_printerSearchString)));
        }

        private void ServicePrinterSearchChanged(string newSearchString)
        {
            if (string.IsNullOrEmpty(newSearchString))
                return;

            if (newSearchString.Length < 4)
                return;

            Model.ServerPrinterList = _dataAccessService.GetDynamicServerPrinters(newSearchString.ToUpper());
            System.Windows.Forms.Application.DoEvents();
            Thread.Sleep(100);

            DeselectComboBoxText(ServerPrinterCmb);

            if (Model.ServerPrinterList.Count == 1)
            {
                Model.SelectedServerPrinter = Model.ServerPrinterList[0];
                //ServerPrinterCmb.SelectedValue = Model.ServerPrinterList[0].Name;
                SetPrinterType();
            }
            else
                ServerPrinterCmb.IsDropDownOpen = true; 
        }

        public void SetPrinterType()
        {
            var defaultPrinterType = string.Empty;
            var selectedPrinter = ServerPrinterCmb.SelectedValue == null ? "" : ServerPrinterCmb.SelectedValue.ToString();

            if (!string.IsNullOrEmpty(selectedPrinter))
            {
                defaultPrinterType = "SAT MESSAGE P-MESSAGE";
            }
        }

        private void DeselectComboBoxText(RadComboBox cbx)
        {
            //  Calling this method insures the CombBox text is not selected and the cursor is positioned for additional typing (ComboBox ONLY)
            var tbx = (TextBox)cbx.Template.FindName("PART_EditableTextBox", cbx);
            if (tbx != null)
            {
                tbx.SelectionStart = tbx.Text.Length == 0 ? 0 : tbx.Text.Length;
                tbx.SelectionLength = 0;
            }
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(OK);
                        break;
                    case Key.P:
                        Keyboard.Focus(PrintServer);
                        break;
                    case Key.S:
                        Keyboard.Focus(Print);
                        break;
                }
            }
        }

        private void ServerPrinterCmb_TextChanged(object sender, RoutedEventArgs e)
        {

        }
    }
}

