using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ClinSchd.Modules.ResourceSelection.ResourceSelection;
using ClinSchd.Modules.ResourceSelection.Controllers;
using ClinSchd.Modules.ResourceSelection.Services;
using ClinSchd.Modules.ResourceSelection.Tests.Mocks;

namespace ClinSchd.Modules.ResourceSelection.Tests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class ResourceSelectionModuleFixture
    {
        private MockUnityResolver _container;
        private MockResourceSelectionController _controller;

        [TestMethod]
        public void PatientSelectionModuleRegistersPatientSelectionViewAndPatientSelectionService()
        {
            var resourceSelectionModule = CreateTestablePatientSelectionModule ();

            resourceSelectionModule.InvokeRegisterViewsAndServices ();

            Assert.AreEqual (typeof (ResourceSelectionView), _container.Types[typeof (IResourceSelectionView)]);
            Assert.AreEqual(typeof(ResourceSelectionController), _container.Types[typeof(IResourceSelectionController)]);
            Assert.AreEqual (typeof (ResourceSelectionPresentationModel), _container.Types[typeof (IResourceSelectionPresentationModel)]);
            Assert.AreEqual(typeof(ResourceSelectionService), _container.Types[typeof(IResourceSelectionService)]);
#if !SILVERLIGHT
#endif
        }

        [TestMethod]
        public void InitCallsRunOnPatientSelectionController()
        {
            var resourceSelectionModule = CreateTestablePatientSelectionModule ();

            resourceSelectionModule.Initialize ();

            Assert.IsTrue(_controller.RunCalled);
        }

        private TestablePatientSelectionModule CreateTestablePatientSelectionModule()
        {
            _container = new MockUnityResolver();
            _controller = new MockResourceSelectionController ();

            _container.Bag.Add(typeof(IResourceSelectionController), _controller);
            _container.Bag.Add (typeof (IResourceSelectionView), null);

            return new TestablePatientSelectionModule(_container);
        }

        internal class TestablePatientSelectionModule : ResourceSelectionModule
        {
            public TestablePatientSelectionModule(IUnityContainer container)
                : base(container, null)
            {
            }

            public void InvokeRegisterViewsAndServices()
            {
                RegisterViewsAndServices();
            }
        }
    }
}
