using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.ResourceSelection.Services;
using Microsoft.Practices.Unity;

namespace ClinSchd.Modules.ResourceSelection.ResourceSelection
{
    public class ResourceSelectionPresentationModel : IResourceSelectionPresentationModel, INotifyPropertyChanged
    {
        private readonly IResourceSelectionService _resourceSelectionService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private string _searchString;
        private IList<SchdResource> _resourceList;
        private SchdResource _selectedResource;
        //private RPMSClinic selectedClinic;
        private string _paneTitle = "";
        //private IList<ResourceUser> _resourceUsers;

        public ResourceSelectionPresentationModel (
            IResourceSelectionView view,
            IResourceSelectionService resourceSelectionService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _eventAggregator = eventAggregator;
            _resourceSelectionService = resourceSelectionService;
            _dataAccessService = dataAccessService;
        }

        public void SearchStringChanged(string newSearchString)
        {
            SearchString = newSearchString;
            //IList<SchdResource> newResourcesList = this.dataAccessService.GetResourcesByType("H", newSearchString == string.Empty ? null : newSearchString, false);
            var newResourceList = _dataAccessService.SearchSdecHResources(newSearchString);

            if (newResourceList.Count > 0)
            {
                string lastSub = newResourceList[newResourceList.Count - 1].LastSub;

                if ((newResourceList.Count == Keys.ClinicResourceSearchMaxRecords) && !string.IsNullOrEmpty(lastSub))
                {
                    Keys.AlertUser(View, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search to select other clinics.", "Clinic Search Criteria");
                    View.SetSearchFocus();
                }
            }

            ResourceList = newResourceList;

            if (ResourceList.Count == 1)
                SelectedResource = ResourceList[0];
        }
        
        public void OnClose()
        {
            View.Close();
            if (View.DialogResult != null && View.DialogResult.Value && _selectedResource != null)
            {
                // Forward the sender's event
                NewForwardingEvent.Publish (SelectedResource);
                
                // Notify anyone who cares that a resource has been selected
                _eventAggregator.GetEvent<ResourceSelectedEvent> ().Publish (SelectedResource);
            }
        }

        [Dependency]
        public Factory<SchdResource> ResourceFactory { get; set; }

        public void ResetResource ()
        {
            SelectedResource = ResourceFactory.Create ();
        }

        public bool IsOkButtonEnabled => (SelectedResource != null);

        #region IResourceSelectionPresentationModel Members

        public IResourceSelectionView View { get; }
        public CompositePresentationEvent<SchdResource> NewForwardingEvent { get; set; }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public string SearchString
        {
            get { return _searchString; }
            set
            {
                if ( _searchString != value )
                {
                    _searchString = value;
                    OnPropertyChanged( "SearchString" );
                }
            }
        }

        public IList<SchdResource> ResourceList
        {
            get { return _resourceList; }
            private set
            {
                if ( _resourceList != value )
                {
                    _resourceList = value;
                    OnPropertyChanged( "ResourceList" );
                }
            }
        }

        public SchdResource SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                if ( !Equals( _selectedResource, value ) && value != null )
                {
                    _selectedResource = value;
                    OnPropertyChanged( "SelectedResource" );
                    OnPropertyChanged( "IsOkButtonEnabled" );
                }
            }
        }

        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
                handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
