using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Timers;
using ClinSchd.Infrastructure;

namespace ClinSchd.Modules.ResourceSelection.ResourceSelection
{
    /// <summary>
    /// Interaction logic for PatientSelectionView.xaml
    /// </summary>
    public partial class ResourceSelectionView : IResourceSelectionView
    {
        public ResourceSelectionView ()
        {
            InitializeComponent();
            Owner = Keys.MainWindow;
            _resourceSearchTimer = new Timer (1000);
            _resourceSearchTimer.Elapsed += resourceSearchTimer_Elapsed;            
        }

        public ResourceSelectionPresentationModel Model
        {
            get { return DataContext as ResourceSelectionPresentationModel; }
            set { DataContext = value; }
        }

        public void SetSearchFocus()
        {
            txtSearch.Focus();
            Keyboard.Focus(txtSearch);
        }

        private void PatientSelection_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            DialogResult = true;
            Model.ResetResource ();
            Close();
        }

        private void Button_OK_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
            Model.ResetResource ();
            Close ();
        }

        private void Button_Cancel_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = false;
            Model.SelectedResource = null;
            Close();
        }

        private readonly Timer _resourceSearchTimer;
        private string _resourceSearchString;
        private void PatientSelection_TextChanged (object sender, TextChangedEventArgs e)
        {
            if ( sender is TextBox )
            {
                _resourceSearchString = ((TextBox)sender).Text;
            }
        }

        private void txtSearch_KeyUp (object sender, KeyEventArgs e)
        {
            _resourceSearchTimer.Stop ();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _resourceSearchTimer.Start();
        }

        void resourceSearchTimer_Elapsed( object sender, ElapsedEventArgs e )
        {
            _resourceSearchTimer.Stop();
            if ( _resourceSearchString.Length < Keys.ClinicHResourceSearchCharMin )
                return;

            Dispatcher.Invoke( (Action)( () => { Model.SearchStringChanged( _resourceSearchString ); } ) );
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            txtSearch.Focus();
            Keyboard.Focus(txtSearch);
        }
        
        private void Window_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;

                switch (key)
                {
                    case Key.O:
                        BtnOk.Focus();
                        Keyboard.Focus(BtnOk);
                        break;
                    case Key.C:
                        BtnCancel.Focus();
                        Keyboard.Focus(BtnCancel);
                        break;
                    case Key.S:
                        txtSearch.Focus();
                        Keyboard.Focus(txtSearch);
                        break;
                    case Key.G:
                        if (Clinics.Items.Count > 0)
                        {
                            Clinics.SelectedItem = Clinics.Items[0];
                            Clinics.ScrollIntoView(Clinics.SelectedItem);
                            Clinics.Focus();
                        }
                        break;
                }
            }
        }

        private void Window_Activated(object sender, EventArgs e)
        {
            BtnOk.Focus();
            Keyboard.Focus(BtnOk);
        }

        private void BtnCancel_GotFocus(object sender, RoutedEventArgs e)
        {
            //txtSearch.TabIndex = BtnCancel.TabIndex + 1;
        }

        private void BtnCancel_LostFocus(object sender, RoutedEventArgs e)
        {
            //txtSearch.TabIndex = 0;
        }

        private void FocusFirstRowInGrid()
        {
            if (Clinics.Items.Count > 0)
            {
                Clinics.SelectedItem = Clinics.Items[0];
                Clinics.ScrollIntoView(Clinics.SelectedItem);
                Clinics.Focus();
            }
        }

        private void txtSearch_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Tab)
            {
                e.Handled = true;
                FocusFirstRowInGrid();
            }
        }

        private void BtnOk_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            //if (Keyboard.Modifiers == ModifierKeys.Shift && e.Key == Key.Tab)
            //{
            //    e.Handled = true;
            //    FocusFirstRowInGrid();
            //}
        }

        private void Clinics_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.None)
            {
                if (e.Key == Key.Tab)
                {
                    e.Handled = true;
                    Keyboard.Focus(BtnOk);
                }
                else if (e.Key == Key.Enter)
                {
                    e.Handled = true;
                    Button_OK_Click(BtnOk, new RoutedEventArgs());
                }
            }
            if (Keyboard.Modifiers == ModifierKeys.Shift && e.Key == Key.Tab)
            {
                e.Handled = true;
                Keyboard.Focus(txtSearch);
            }
        }
    }
}
