using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.ResourceSelection.ResourceSelection;
using ClinSchd.Modules.ResourceSelection.Controllers;
using ClinSchd.Modules.ResourceSelection.Services;

namespace ClinSchd.Modules.ResourceSelection
{
    public class ResourceSelectionModule : IModule
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;

        public ResourceSelectionModule(IUnityContainer container, IEventAggregator eventAggregator)
        {
            _container = container;
            _eventAggregator = eventAggregator;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();
            _eventAggregator.GetEvent<LaunchResourceSelectionDialogEvent> ()
                .Subscribe (LaunchResourceSelectionDialog, ThreadOption.UIThread, true);
            _eventAggregator.GetEvent<LaunchResourceAvailabilityDialogEvent> ()
                .Subscribe (LaunchResourceAvailibilityDialog, ThreadOption.UIThread, true);
        }

        public void LaunchResourceSelectionDialog (CompositePresentationEvent<SchdResource> newforwardingEvent)
        {
            IResourceSelectionController controller = _container.Resolve<IResourceSelectionController> ();
            controller.Model.NewForwardingEvent = newforwardingEvent;
            controller.Run ("Resource Selection");
        }

        public void LaunchResourceAvailibilityDialog (CompositePresentationEvent<SchdResource> newforwardingEvent)
        {
            IResourceSelectionController controller = _container.Resolve<IResourceSelectionController> ();
            controller.Model.NewForwardingEvent = newforwardingEvent;
            controller.Run ("Availability Selection");
        }

        protected void RegisterViewsAndServices()
        {
            _container.RegisterType<IResourceSelectionController, ResourceSelectionController>();
            _container.RegisterType<IResourceSelectionView, ResourceSelectionView> ();
            _container.RegisterType<IResourceSelectionPresentationModel, ResourceSelectionPresentationModel> ();
            _container.RegisterType<IResourceSelectionService, ResourceSelectionService>();
        }
    }
}
