using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Ribbon.Schedule;
using ClinSchd.Modules.Ribbon.Controllers;
using ClinSchd.Modules.Ribbon.Services;
using ClinSchd.Modules.Ribbon.Tests.Mocks;

namespace ClinSchd.Modules.Ribbon.Tests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class RibbonModuleFixture
    {
        private MockUnityResolver _container;
        private MockRibbonController _controller;

        [TestMethod]
        public void RibbonModuleRegistersRibbonViewAndRibbonService()
        {
            TestableRibbonModule ribbonModule = CreateTestableRibbonModule();

            ribbonModule.InvokeRegisterViewsAndServices();

            Assert.AreEqual (typeof (ScheduleView), _container.Types[typeof (IScheduleView)]);
            Assert.AreEqual(typeof(RibbonController), _container.Types[typeof(IRibbonController)]);
            Assert.AreEqual (typeof (SchedulePresentationModel), _container.Types[typeof (ISchedulePresentationModel)]);
            Assert.AreEqual(typeof(RibbonService), _container.Types[typeof(IRibbonService)]);
#if !SILVERLIGHT
#endif
        }

        [TestMethod]
        public void InitCallsRunOnRibbonController()
        {
            var ribbonModule = CreateTestableRibbonModule();

            ribbonModule.Initialize();

            Assert.IsTrue(_controller.RunCalled);
        }

        private TestableRibbonModule CreateTestableRibbonModule()
        {
            _container = new MockUnityResolver();
            _controller = new MockRibbonController();

            _container.Bag.Add(typeof(IRibbonController), _controller);

            return new TestableRibbonModule(_container);
        }

        internal class TestableRibbonModule : RibbonModule
        {
            public TestableRibbonModule(IUnityContainer container)
                : base(container)
            {
            }

            public void InvokeRegisterViewsAndServices()
            {
                RegisterViewsAndServices();
            }
        }
    }
}
