using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Ribbon.Schedule;
using ClinSchd.Modules.Ribbon.Controllers;
using ClinSchd.Modules.Ribbon.Services;

namespace ClinSchd.Modules.Ribbon
{
    public class RibbonModule : IModule
    {
        private readonly IUnityContainer container;

        public RibbonModule(IUnityContainer container)
        {
            this.container = container;
        }

        public void Initialize()
        {
            RegisterViewsAndServices();

            IRibbonController controller = container.Resolve<IRibbonController>();
            controller.Run();
        }

        protected void RegisterViewsAndServices()
        {
            container.RegisterType<IRibbonController, RibbonController>(new ContainerControlledLifetimeManager());
            container.RegisterType<IScheduleView, ScheduleView>();
            container.RegisterType<IPatientQueueView, PatientQueueView>();
            container.RegisterType<ISchedulePresentationModel, SchedulePresentationModel>();
            container.RegisterType<IRibbonService, RibbonService>(new ContainerControlledLifetimeManager());
            container.RegisterType<RequestManagement, RequestManagement>();
            container.RegisterType<RequestManagement>();
        }
    }
}
