using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.DataAccess.Services;
using ClinSchd.Modules.Navigation.Views;
using ClinSchd.Infrastructure.Models;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.GridView;
using DragEventArgs = System.Windows.DragEventArgs;
using MessageBox = System.Windows.MessageBox;
using Keys = ClinSchd.Infrastructure.Keys;
using Label = Telerik.Windows.Controls.Label;
using ClinSchd.Infrastructure.Factories;

namespace ClinSchd.Modules.Ribbon.Schedule
{
    /// <summary>
    /// Interaction logic for ScheduleView.xaml
    /// </summary>
    public partial class PatientQueueView : IPatientQueueView
    {
        AppointmentRequest _selectedAppointmentRequest;
        //private string patientSearchString;
        //private GridLength previousExpanderHeight;
        public bool IsRefreshOrClear { get; set; }
        private bool _isRightClickSelect;

        public PatientQueueView(ISchedulePresentationModel model)
        {
            Model = (SchedulePresentationModel)model;
            InitializeComponent();
            RadDragAndDropManager.AddDropQueryHandler( GrAdmittedPatients, OnDropQuery );
            RadDragAndDropManager.AddDropInfoHandler( GrAdmittedPatients, OnGridViewDropInfo );
            RadDragAndDropManager.AddDragQueryHandler( GrAdmittedPatients, OnGridViewDragQuery );
            RadDragAndDropManager.AddDragInfoHandler( GrAdmittedPatients, OnGridViewDragInfo );
            Keys.LastRecordUnlocked = " ";
        }

        public SchedulePresentationModel Model
        {
            get { return DataContext as SchedulePresentationModel; }
            set { DataContext = value; }
        }

        public RadGridView PatientQueueRadGridView 
        {
            get { return GrAdmittedPatients; }
        }

        private bool _bDialogResult;
        public bool ConfirmUser (string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }

        public void AlertUser (string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        private void DisplayUnableToLockMessage(string msg)
        {
            var reqType = "";
            var ret = _lockedRecord.Split('|')[0].ToUpper();
            switch (ret)
            {
                case "A":
                    reqType = "Appointment";
                    break;
                case "E":
                    reqType = "EWL";
                    break;
                case "R":
                    reqType = "PtCSch";
                    break;
                case "P":
                case "C":
                    reqType = "Consult";
                    break;
            }
            var message = "The selected " + reqType + " request can not be accessed.  " + msg;
            MessageBox.Show(message);
        }
        private void DisplayFailToUnlockRecordMessage()
        {
            var reqType = "";
            var ret = _lockedRecord.Split('|')[0].ToUpper();
            switch (ret)
            {
                case "A":
                    reqType = "Appointment";
                    break;
                case "E":
                    reqType = "EWL";
                    break;
                case "R":
                    reqType = "PtCSch";
                    break;
                case "P":
                case "C":
                    reqType = "Consult";
                    break;
            }
            var message = "The selected " + reqType + " request record failed to Unlock.";
            MessageBox.Show(message);
        }
        private string _lockedRecord;

        private void WaitListPatients_SelectionChanged( object sender, SelectionChangeEventArgs e )
        {
            try
            {
                //Lock Wait List
                if (Keys.IsEwlTransfer)
                {
                    PatientQueueRadGridView.SelectedItem = null;
                    _isRightClickSelect = false;
                    return;
                }

                if (e.AddedItems.Count > 0)
                {
                    var selectedAppointmentRequest = e.AddedItems[0] as AppointmentRequest;

                    if (_isRightClickSelect)
                    {
                        _isRightClickSelect = false;

                        if (selectedAppointmentRequest != null)
                            Model.SelectedAppointmentRequest = selectedAppointmentRequest;

                        return;
                    }

                    Mouse.OverrideCursor = Cursors.Wait;

                    try
                    {
                        // The bulk of this method was moved to an ancillary method so the patient update can be called from other locations
                        WaitListPatients_SelectionChanged(selectedAppointmentRequest);
                    }
                    finally
                    {
                        Mouse.OverrideCursor = Cursors.Arrow;
                    }
                }
                else
                {
                    _isRightClickSelect = false;
                    Keys.SelectedAppointmentRequest = null;
                    Keys.SelectedAppointmentInitialDesiredDate = null;

                    if (!string.IsNullOrEmpty(_lockedRecord))
                        if (!Model.DataAccessService.UnlockAppointmentRequest(_lockedRecord))
                            DisplayFailToUnlockRecordMessage();

                    _lockedRecord = "";
                }
            }
            catch (Exception ex)
            {
                //think this is causing exception in telerik.windows.controls.dll
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private bool _waitListSelectionCancelled = false;
        private void WaitListPatients_SelectionChanged(AppointmentRequest selectedAppointmentRequest, bool launchSchedulerParam = true)
        {
            Model.EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("SelectedRequestChanging");
            // Error handling for this proccess should be handled by the calling method/
            Keys.SendPatientData = true;
            _waitListSelectionCancelled = false;
            var dataAccessService = (DataAccessService)Model.DataAccessService;

            if (selectedAppointmentRequest != null)
            {
                Model.SelectedAppointmentRequest = selectedAppointmentRequest;

                //Lock Wait List record.
                if (!string.IsNullOrEmpty(_lockedRecord) && !_lockedRecord.Contains(Keys.LastRecordUnlocked))
                {
                    Keys.LastRecordUnlocked = " ";
                    if (!dataAccessService.UnlockAppointmentRequest(_lockedRecord))
                    {
                        DisplayFailToUnlockRecordMessage();
                        _lockedRecord = "";
                        _waitListSelectionCancelled = true;
                        return;
                    }
                }
                _lockedRecord = "";
                var msg = "";
                var isLocked = dataAccessService.LockAppointmentRequest(selectedAppointmentRequest, ref _lockedRecord, ref msg);
                if (!isLocked)
                {
                    DisplayUnableToLockMessage(msg);
                    _lockedRecord = "";
                    GrAdmittedPatients.SelectedItem = GrAdmittedPatients.Items[0];
                    _waitListSelectionCancelled = true;
                    return;
                }

                if (selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Consult && selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Procedure)
                {
                    selectedAppointmentRequest.ClinicResourceId = Model.DataAccessService.GetResourceIdByClinicIdAndName(selectedAppointmentRequest.ClinicIen, selectedAppointmentRequest.ClinName, false, false);
                }

                if (selectedAppointmentRequest.IsMRTCParent && Keys.LoggedInUserSDECViewOnlyKey)
                {
                    _waitListSelectionCancelled = true;
                    return;
                }

                Keys.SelectedAppointmentRequest = selectedAppointmentRequest;
                if (Keys.SelectedPatient == null || selectedAppointmentRequest.Patient.IEN != Keys.SelectedPatient.IEN)
                {
                    Keys.SelectedPatient = selectedAppointmentRequest.Patient;
                    var tempPatient = selectedAppointmentRequest.Patient;
                    var promptRslt = Model.DataAccessService.GetPatientSelectionPrompts(selectedAppointmentRequest.Patient.IEN, ref tempPatient, false);
                    selectedAppointmentRequest.Patient = tempPatient;
                    if (promptRslt == "0")
                    {
                        Model.PatientList = new List<Patient>();
                        Model.View2.PatientSearchString = "";
                        Model.View2.PatientSearchComboBox.Text = "";
                        _waitListSelectionCancelled = true;
                        return;
                    }
                    selectedAppointmentRequest.Patient.LoadRMGrid = true;
                    var futureAppointments = dataAccessService.GetFutureAppointments(selectedAppointmentRequest.Patient.IEN);
                    Model.FuturePatientAppointments = futureAppointments;
                    Model.View.UpdateFutureAppointments(futureAppointments);
                    var specialNeeds = dataAccessService.GetPatientSpecialNeedsPreferences(selectedAppointmentRequest.Patient.IEN);
                    selectedAppointmentRequest.Patient.Preferences.Clear();
                    foreach (DataRow dr in specialNeeds.Rows)
                    {
                        if (specialNeeds.Columns.Contains("RETURNCODE"))
                            continue;
                        selectedAppointmentRequest.Patient.Preferences.Add(dr["PREF_TEXT"].ToString());
                    }
                    Model.View.UpdateSpecialNeedsRibbon(selectedAppointmentRequest.Patient.Preferences);
                    Model.EventAggregator.GetEvent<PatientSelectedEvent>().Publish(selectedAppointmentRequest.Patient);
                    // Because the just above event call will trigger an RM Grid Reload and reselect of the current request, we will return here.
                    // The reselect of the current request will bypass this block of code, because the patient is already selected, and trigger 
                    // any required schedule loading.
                    return;
                }

                if (Keys.SelectedAppointmentRequest != null && selectedAppointmentRequest.ClinName == Keys.SelectedAppointmentRequest.ClinName)
                    Keys.SameClinicSelected = true;

                if ((string.IsNullOrEmpty(selectedAppointmentRequest.DesiredApptDateShortString) || Keys.BooleanFlags.ClearButtonPressed) && (selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Consult) && (selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Procedure))
                {
                    Keys.BooleanFlags.ClearButtonPressed = false;
                    _waitListSelectionCancelled = true;
                    return;
                }

                Model.SelectPatient(selectedAppointmentRequest.Patient);
                Keys.RequestKeys.CurrentSelectedDesiredDate = selectedAppointmentRequest.DesiredApptDateShortString;
                Model.AppointmentSelectedEvent(selectedAppointmentRequest);

                Model.View.UpdatePatientSearchText(Model.SelectedAppointmentRequest.Patient.Name);
                if (!Model.ArePendingAppointmentsVisible)
                {
                    Model.View.FocusPatientSearch();
                    Model.ArePendingAppointmentsVisible = true;
                }

                _selectedAppointmentRequest = selectedAppointmentRequest;

                if (string.IsNullOrEmpty(Keys.RequestKeys.CurrentSelectedDesiredDate))
                    Keys.RequestKeys.CurrentSelectedDesiredDate = DateTime.Today.ToShortDateString();

                SchdResource schedRes = null;

                if (selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Consult && selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Procedure)
                {
                    schedRes = Model.DataAccessService.GetSdecResource(selectedAppointmentRequest.ClinicResourceId);
                    if (schedRes != null)
                        schedRes.IsAsync = false;
                }

                if (selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Consult || selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Procedure)
                {
                    if (Model.Resource != null && !string.IsNullOrEmpty(Model.Resource.HOSPITAL_LOCATION_ID))
                        Model.EventAggregator.GetEvent<CloseSchedulesEvent>().Publish(Model.Resource.HOSPITAL_LOCATION_ID);
                }
                else if (!Model.AppointmetRequestTypeViewNoneChecked || ((schedRes != null) && (schedRes.Clinic == null)))
                {
                    if (string.IsNullOrEmpty(selectedAppointmentRequest.ClinicIen) && selectedAppointmentRequest.Clinic != null)
                    {
                        selectedAppointmentRequest.ClinicIen = selectedAppointmentRequest.Clinic.ClinicIen;
                        selectedAppointmentRequest.ClinName = selectedAppointmentRequest.Clinic.ClinicName;
                    }
                }

                bool sendCID = true;

                if (!IsRefreshOrClear)
                    if (selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Consult && selectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Procedure)
                    {
                        var launchSchedule = true;

                        // Check if can launch Scheduler
                        if (schedRes != null)
                        {
                            if (Keys.Methods.IsApptOrVet(selectedAppointmentRequest.AppointmentRequestType))
                            {
                                // APPT/W2VA/VETERAN requests can have MRTC
                                if (schedRes.Clinic != null && schedRes.Clinic.HOSPITAL_LOCATION_ID != null)
                                    launchSchedule = selectedAppointmentRequest.ClinicServiceSpecialtyMerge.ToUpper() == schedRes.ClinicName.ToUpper();
                            }
                            else if (selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch)
                            {
                                // Checking PtCSch Requests.  If the Resource has a Hospital Location ID but the Clinic was not found it is assummed
                                // the Clinic has no PtCSch Reminder Letter.  Tell the user.
                                if (!string.IsNullOrEmpty(schedRes.HOSPITAL_LOCATION_ID) && schedRes.Clinic == null)
                                {
                                    AlertUser(
                                        "Clinic Schedule for selected PtCSch Request cannot be displayed because the " +
                                        "assigned clinic does not have a PtCSch Reminder Letter defined.",
                                        "PtCSch Request Selection"
                                        );
                                    launchSchedule = false;
                                }
                            }
                        }

                        if (launchSchedule && launchSchedulerParam && (schedRes != null))
                        {
                            Model.ExecuteSchedulerDisplayCommand(schedRes);
                            sendCID = false;
                        }
                    }

                if (sendCID && selectedAppointmentRequest.DesiredApptDate.HasValue)
                {
                    DateTime sendDate = selectedAppointmentRequest.DesiredApptDate.Value.Date;

                    if (selectedAppointmentRequest.DesiredApptDate.Value.Date.CompareTo(DateTime.Today) < 0)
                        sendDate = DateTime.Today;

                    Model.EventAggregator.GetEvent<ResetCalendarEvent>().Publish(sendDate);
                }

                IsRefreshOrClear = false;
            }

            if (Model.AppointmetRequestTypeViewNoneChecked)
                Model.AppointmetRequestTypeViewNoneChecked = false;

            _waitListSelectionCancelled = false;
        }

        private void PatientContextMenu_ItemClick(object sender, Telerik.Windows.RadRoutedEventArgs e)
        {
            try
            {
                if (Model.SelectedAppointmentRequest == null)
                    return;

                if (PatientContextMenu.GetClickedElement<GridViewRow>().Item == null)
                    return;

                _selectedAppointmentRequest = PatientContextMenu.GetClickedElement<GridViewRow>().Item as AppointmentRequest;
                _selectedAppointmentRequest.PatientContact = PatientContactFactory.CreatePatientContact(
                    Model.DataAccessService.GetPatientContact(PatientContactRequestFactory.CreateReadRequestParams(
                        _selectedAppointmentRequest))); 

                if (_selectedAppointmentRequest.PatientContact != null)
                    _selectedAppointmentRequest.PatientContact.ContactAttempts = ContactAttemptFactory.CreateContactAttempts(
                        _selectedAppointmentRequest.PatientContact, Model.DataAccessService.GetContactAttempts(
                            PatientContactRequestFactory.CreateReadMulipleRequestParams(
                                _selectedAppointmentRequest.PatientContact)));

                if (_selectedAppointmentRequest == null)
                    return;

                try
                {
                    Mouse.OverrideCursor = Cursors.Wait;

                    try
                    {
                        WaitListPatients_SelectionChanged(_selectedAppointmentRequest, false);

                    }
                    finally
                    {
                        Mouse.OverrideCursor = Cursors.Arrow;
                    }

                    if (_waitListSelectionCancelled)
                        return;
                }
                catch (Exception ex)
                {
                    //think this is causing exception in telerik.windows.controls.dll
                    TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                    //MessageBox.Show(ex.Source + ": " + ex.Message);
                }

                var selectedCause = (e.Source as RadMenuItem).Header.ToString().ToUpper();

                switch (selectedCause)
                {
                    case "DEATH":
                    case "REMOVED/NON-VA CARE":
                    case "REMOVED/SCHEDULED-ASSIGNED":
                    case "REMOVED/VA CONTRACT CARE":
                    case "REMOVED/NO LONGER NECESSARY":
                    case "ENTERED IN ERROR":
                    case "TRANSFERRED":
                    case "CHANGED CLINIC":
                        if (_selectedAppointmentRequest.MultipleAppointmentsRequired)
                        {
                            if (Keys.WaitListDispositions.IsValidMRTCDisposition(selectedCause))
                                //                                    (selectedCause == Keys.WaitListDispositions.EnteredInError)
                                //                                ||
                                //                                    (selectedCause == Keys.WaitListDispositions.RemovedNoLongerNecessary)
                                //                                ||
                                //                                    (selectedCause == Keys.WaitListDispositions.RemovedNonVACare)
                                //                                ||
                                //                                    (selectedCause == Keys.WaitListDispositions.RemovedDeath)
                                //                                )
                                Model.EventAggregator.GetEvent<LaunchMRTCCancelDialog>().Publish(new LaunchMRTCCancelDialogArgs(_selectedAppointmentRequest, null, selectedCause, null));
                        }
                        else
                        {
                            if (selectedCause == "REMOVED/NO LONGER NECESSARY")
                                if (patientContactWarning() == MessageBoxResult.No)
                                    return;                                                   

                            Model.RemoveAppointmentRequest(_selectedAppointmentRequest, selectedCause); //RemovePatientFromWorkspace(PatientContextMenu.GetClickedElement<GridViewRow>().Item as Patient);
                        }
                        break;
                    case "CLEAR WORKSPACE":
                        Model.AdmittedPatients.Clear();
                        break;
                    #region BJ'S REPLACMENT CODE FOR CONTACT LOGIC
                    case "CONTACT ATTEMPTS":
                        Model.EventAggregator.GetEvent<LaunchContactAttemptDialogEvent>().Publish(Model.SelectedAppointmentRequest);
                        break;
                    #endregion
                    #region HPE OLD CONTACT CODE (COMMENTED OUT)
                    //case "CALL ATTEMPT 1":
                    //case "CALL ATTEMPT 2":
                    //case "CALL ATTEMPT 3":
                    //case "LETTER SENT TO PATIENT":
                    //    var patientContactDialog = new PatientContactDateTime();
                    //    CLaunchWindow.LaunchWindow(patientContactDialog, "Patient Contact", 300, 325);
                    //    if (patientContactDialog.DialogResult)
                    //    {
                    //        var selectedDateTime = patientContactDialog.viewModel.ContactDateTime;
                    //        var selectedAction = selectedCause == "LETTER SENT TO PATIENT" ? "LETTER" : (patientContactDialog.viewModel.LeftMessage ? "Message Left" : "Called");
                    //        var selectedPhoneNumber = patientContactDialog.viewModel.ContactPhoneNumber;
                    //        var selectedAppointmentRequestManagementIen = _selectedAppointmentRequest.RequestManagmentIen;

                    //        if (selectedAppointmentRequestManagementIen == null)
                    //        {
                    //            selectedAppointmentRequestManagementIen = _selectedAppointmentRequest.ConsultationRequestIen.ToString();
                    //        }
                    //        if (Keys.Methods.IsApptOrVet(_selectedAppointmentRequest.AppointmentRequestType))
                    //            Model.ContactPatient(_selectedAppointmentRequest.ApptRequestIen.ToString(),
                    //                selectedDateTime.ConvertToExternalDateTimeFormat(false, false), selectedPhoneNumber,
                    //                selectedAction, Keys.LoggedInUserIen, true); // _selectedAppointmentRequest.OriginationUser, true);
                    //        if (_selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl ||
                    //            _selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Consult)
                    //            Model.ContactPatient(selectedAppointmentRequestManagementIen,
                    //                selectedDateTime.ConvertToExternalDateTimeFormat(false, false), selectedPhoneNumber,
                    //                selectedAction, Keys.LoggedInUserIen); // _selectedAppointmentRequest.OriginationUser);
                    //    }
                    //    break;
                    #endregion
                    case "FAILURE TO RESPOND":
                    case "MOVED":
                    case "DECEASED":
                    case "DOESN'T WANT VA SERVICES":
                    case "RECEIVED CARE AT ANOTHER VA":
                    case "OTHER":
                        if ((selectedCause == "OTHER" || selectedCause == "FAILURE TO RESPOND"))                        
                            if (patientContactWarning() == MessageBoxResult.No)
                                return;
                        
                        Model.RemovePtCSchRequest(_selectedAppointmentRequest, selectedCause);
                        break;
                    case "TRANSFER TO EWL":
                        TransferApptRequestToEwl();
                        break;
                    //case "TRANSFER TO PTCSCH":
                    //    TransferApptRequestToPtCSch();
                    //    break;
                    case "VIEW REQUEST":
                        EditViewRequest(_selectedAppointmentRequest, false);
                        break;
                    case "EDIT REQUEST":
                        EditViewRequest(_selectedAppointmentRequest, true);
                        break;
                    case "AUDIT REPORT":
                        EditViewRequest( _selectedAppointmentRequest, true );
                        break;
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private bool isNonMentalHealthContactsSatisfied()
        {
            if (_selectedAppointmentRequest.PatientContact != null
                && _selectedAppointmentRequest.PatientContact.IsNonMentalHealthContactsSatisfied)
                return true;

            return false;
        }

        private bool isMentalHealthContactsSatisfied()
        {
            if (_selectedAppointmentRequest.PatientContact != null
                && _selectedAppointmentRequest.PatientContact.IsMentalHealthContactsSatisfied)
                return true;

            return false; 
        }

        private bool isMentalHealthClinic()
        {
            string clinicCodesString = Model.DataAccessService.GetClinicStopCodes(
                PatientContactRequestFactory.CreateStopCodeRequestParams(
                _selectedAppointmentRequest));
            var clinicStopCodesArray = clinicCodesString.Split('^'); 
            int stopCode = 0;
            int creditStop = 0;
            int.TryParse(clinicStopCodesArray[0], out stopCode);
            int.TryParse(clinicStopCodesArray[1], out creditStop);

            if (Enumerable.Range(500, 599).Contains(stopCode))
                return true;

            if (creditStop == 125)
                return true; 

            return false; 
        }

        private MessageBoxResult patientContactWarning()
        {
            if (isMentalHealthClinic() && !isMentalHealthContactsSatisfied())
                return showPatientContactWarning(); 
            else if (!isNonMentalHealthContactsSatisfied())
                return showPatientContactWarning();

            return MessageBoxResult.Yes;   
        }

        private MessageBoxResult showPatientContactWarning()
        {
            return MessageBox.Show("The required number of Patient Contacts has NOT been completed for this request." +
                        Environment.NewLine + Environment.NewLine + "Are you sure you want to remove this request?", 
                        "Patient Contacts", MessageBoxButton.YesNo, MessageBoxImage.Warning, MessageBoxResult.No);
        }

        private void EditViewRequest (AppointmentRequest selectedAppointmentRequest, bool isEdit)
        {

            if (isEdit == true && selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.ReturnToClinic)
            {
                MessageBox.Show("You cannot edit a RTC Request!", "Not Allowed", MessageBoxButton.OK, MessageBoxImage.Stop);
                return;
            }

            if (Keys.SelectedPatient == null || selectedAppointmentRequest.Patient.IEN != Keys.SelectedPatient.IEN)
            {
                Keys.SelectedPatient = selectedAppointmentRequest.Patient;
                var tempPatient = selectedAppointmentRequest.Patient;
                var promptRslt = Model.DataAccessService.GetPatientSelectionPrompts( selectedAppointmentRequest.Patient.IEN, ref tempPatient, false);
                selectedAppointmentRequest.Patient = tempPatient;
                Mouse.OverrideCursor = Cursors.Arrow;

                if (promptRslt == "0")
                {
//                    Model.PatientList = new List<Patient>();
//                    Model.View2.patientSearchString = "";
//                    Model.View2.patientSearchComboBox.Text = "";
//                    Mouse.OverrideCursor = Cursors.Arrow;
                    return;
                }
            }

            try
            {
                var das = Model.DataAccessService;
                var patTemp = _selectedAppointmentRequest.Patient;
                var patient = das.GetPatient( ref patTemp );
                IList<Patient> emptyList = new List<Patient>();

                //selectedAppointmentRequest.ClinicServiceSpecialtyMerge = selectedAppointmentRequest.ClinName;

                var requestManagement = new RequestManagement(Model.DataAccessService, Model.EventAggregator, selectedAppointmentRequest, isEdit ? RequestManagement.Mode.EditRequest : RequestManagement.Mode.ViewRequest );
                LoadSpecialNeeds( _selectedAppointmentRequest );
                var height = requestManagement.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch ? 575 : 625;
                string sTitle;
                switch ( requestManagement.AppointmentRequest.AppointmentRequestType )
                {
                    case Keys.AppointmentRequestTypes.Ewl :
                        sTitle = "EWL Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.Appt :
                        sTitle = "Appointment Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.PtCSch :
                        sTitle = "PtCSch Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.Consult :
                        sTitle = "Consult Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.Veteran :
                        sTitle = "Veteran Appointment Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.Procedure :
                        sTitle = "Procedure Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.Mobile :
                        sTitle = "Mobile Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.WelcomeToVa :
                        sTitle = "WelcomeToVA Request for ";
                        break;
                    case Keys.AppointmentRequestTypes.ReturnToClinic:
                        sTitle = "RTC Request for ";
                        break;
                    default :
                        sTitle = "Undefined type in EditViewRequest ";
                        break;
                }

                CLaunchWindow.LaunchWindow( requestManagement, sTitle + requestManagement.AppointmentRequest.Patient.Name, height, 700 );
                Mouse.OverrideCursor = Cursors.Wait;

                try
                {
                    System.Windows.Forms.Application.DoEvents();
                    if (requestManagement.DialogResult == false)
                    {
                        Model.View2.UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                        Model.PatientList = emptyList;
                        Model.PatientList.Add(patient);
                        //patientSearchString = patient.Name;
                        Model.View2.PatientSearchComboBox.Text = patient.Name;
                    }
                    else if (isEdit)
                    {
                        Keys.BooleanFlags.RefreshWaitList = false;
                        Model.AdmittedPatients.Clear();

                        foreach (var rec in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
                            Model.AdmittedPatients.Add(rec);

                        Model.SelectedAppointmentRequest = null;
                        System.Windows.Forms.Application.DoEvents();
                        Model.SelectedAppointmentRequest = Model.AdmittedPatients.FirstOrDefault(r => (r.UniqueID == requestManagement.UpdatedAppointmentRequest.UniqueID));

                        if (Model.SelectedAppointmentRequest != null)
                            GrAdmittedPatients.ScrollIntoViewAsync(Model.SelectedAppointmentRequest, GrAdmittedPatients.Columns[0], null, null);
                    }

                    //TransferApptRequestToEwl does this magical incantation, so we might as well 
                    System.Windows.Forms.Application.DoEvents();
                    Thread.Sleep(250);
                    System.Windows.Forms.Application.DoEvents();
                }
                finally
                {
                    Mouse.OverrideCursor = Cursors.Arrow;
                }
            }
            catch ( Exception ex )
            {
                TraceLog.LogInfo( ex.Source, ex.GetType().ToString(), ex.Message );
            }
        }

        //private bool TransferApptRequestToPtCSch()
        //{
        //    try
        //    {
        //        var das = Model.DataAccessService;
        //        var patTemp = _selectedAppointmentRequest.Patient;
        //        var patient = das.GetPatient(ref patTemp);
        //        IList<Patient> emptyList = new List<Patient>();
        //        var appointmentRequest = _selectedAppointmentRequest;

        //        //PtCSch specific values
        //        appointmentRequest.PtCSchDate = appointmentRequest.DesiredApptDate;
        //        appointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.PtCSch;

        //        if (appointmentRequest.RequestedBy.Name == "PATIENT")
        //            appointmentRequest.PtCSchDatePerPatient = appointmentRequest.DesiredApptDate; 
                

        //        var requestManagement = new RequestManagement(
        //            Model.DataAccessService, Model.EventAggregator, appointmentRequest, RequestManagement.Mode.PtCSchTransfer);
        //        var originalRequestType = requestManagement.AppointmentRequest.AppointmentRequestType;
                

        //        requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.PtCSch;

        //        if (string.IsNullOrEmpty(appointmentRequest.Patient.SCVisit))                
        //            // If the _appointmentRequest.Patient.SCVisit is NOT set in the original request then set it to the patient default
        //            requestManagement.AppointmentRequest.Patient.ServiceRelated = requestManagement.AppointmentRequest.Patient.ServiceConnectionPercent >= 50;                
        //        else
        //            requestManagement.AppointmentRequest.Patient.ServiceRelated = appointmentRequest.Patient.SCVisit != "NO";

        //        //Get Patients SpecialNeeds/Preferences....................................................
        //        LoadSpecialNeeds(appointmentRequest);

        //        var height = requestManagement.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch ? 575 : 625;
        //        CLaunchWindow.LaunchWindow(requestManagement, "PtCSch request for " + requestManagement.AppointmentRequest.Patient.Name, height, 700);
        //        System.Windows.Forms.Application.DoEvents();

        //        if (requestManagement.DialogResult == false)
        //        {
        //            requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Appt;
        //            Model.View2.UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
        //            Model.PatientList = emptyList;
        //            Model.PatientList.Add(patient);
        //            Model.View2.PatientSearchComboBox.Text = patient.Name;
        //        }
        //        else
        //        {
        //            requestManagement.UpdatedAppointmentRequest.AppointmentRequestType = originalRequestType; // Keys.AppointmentRequestTypes.Appt;
        //            Model.RemoveAppointmentRequest(_selectedAppointmentRequest, Keys.WaitListDispositions.TransferredToPtCSch);
        //            requestManagement.UpdatedAppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.PtCSch;
        //            Model.GetWaitListRecordsForSelectedPatient(
        //                requestManagement.UpdatedAppointmentRequest.Patient.IEN,
        //                "",
        //                true,
        //                true,
        //                requestManagement.UpdatedAppointmentRequest.AppointmentRequestType,
        //                requestManagement.UpdatedAppointmentRequest.RequestManagmentIen,
        //                requestManagement.UpdatedAppointmentRequest
        //            );
        //            Model.EventAggregator.GetEvent<SelectRequestEvent>().Publish("");
        //        }
        //        System.Windows.Forms.Application.DoEvents();
        //        Thread.Sleep(250);
        //        System.Windows.Forms.Application.DoEvents();
        //        //Keys.IsEwlTransfer = false;
        //        return true;
        //    }
        //    catch (Exception ex)
        //    {
        //        TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
        //    }
        //    return false;
        //}

        private bool TransferApptRequestToEwl()
        {
            try
            {
                var das = Model.DataAccessService;
                var patTemp = _selectedAppointmentRequest.Patient;
                var patient = das.GetPatient(ref patTemp);
                IList<Patient> emptyList = new List<Patient>();
                var appointmentRequest = _selectedAppointmentRequest;
                var requestManagement = new RequestManagement(Model.DataAccessService, Model.EventAggregator, appointmentRequest, RequestManagement.Mode.EwlTransfer);
                var originalRequestType = requestManagement.AppointmentRequest.AppointmentRequestType;

                requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Ewl;

                if (string.IsNullOrEmpty(appointmentRequest.Patient.SCVisit))
                {
                    // If the _appointmentRequest.Patient.SCVisit is NOT set in the original request then set it to the patient default
                    requestManagement.AppointmentRequest.Patient.ServiceRelated = requestManagement.AppointmentRequest.Patient.ServiceConnectionPercent >= 50;
                }
                else
                    requestManagement.AppointmentRequest.Patient.ServiceRelated = appointmentRequest.Patient.SCVisit != "NO";
                
                //Get Patients SpecialNeeds/Preferences....................................................
                LoadSpecialNeeds (appointmentRequest);

                var height = requestManagement.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch ? 575 : 625;
                CLaunchWindow.LaunchWindow(requestManagement, "EWL request for " + requestManagement.AppointmentRequest.Patient.Name, height, 700);
                System.Windows.Forms.Application.DoEvents();

                if (requestManagement.DialogResult == false)
                {
                    requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Appt;
                    Model.View2.UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                    Model.PatientList = emptyList;
                    Model.PatientList.Add(patient);
                    Model.View2.PatientSearchComboBox.Text = patient.Name;
                }
                else
                {
                    Keys.IsEwlTransfer = true;
                    requestManagement.UpdatedAppointmentRequest.AppointmentRequestType = originalRequestType; // Keys.AppointmentRequestTypes.Appt;
                    Model.RemoveAppointmentRequest(_selectedAppointmentRequest, Keys.WaitListDispositions.TransferredToEwl);
                    requestManagement.UpdatedAppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Ewl;
                    Model.GetWaitListRecordsForSelectedPatient(
                        requestManagement.UpdatedAppointmentRequest.Patient.IEN,
                        "",
                        true,
                        true,
                        requestManagement.UpdatedAppointmentRequest.AppointmentRequestType,
                        requestManagement.UpdatedAppointmentRequest.RequestManagmentIen,
                        requestManagement.UpdatedAppointmentRequest
                    );
                    Keys.IsEwlTransfer = false;
                    Model.EventAggregator.GetEvent<SelectRequestEvent>().Publish("");
                }
                System.Windows.Forms.Application.DoEvents();
                Thread.Sleep(250);
                System.Windows.Forms.Application.DoEvents();
                //Keys.IsEwlTransfer = false;
                return true;
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
            }
            return false;
        }
 
        private void LoadSpecialNeeds(AppointmentRequest apptReq)
        {
            var das = Model.DataAccessService;
            //Get Patients SpecialNeeds/Preferences....................................................
            var dasResult = das.GetPatientSpecialNeedsPreferences (apptReq.Patient.IEN);
            apptReq.Patient.SpecialNeedsPreferences = new SpecialNeedsPreferences ();
            foreach (DataRow dr in dasResult.Rows)
            {
                if (dasResult.Columns.Contains ("RETURNCODE"))
                    continue;

                apptReq.Patient.SpecialNeedsPreferences.Remarks               = dr["REMARKS"].ToString ();
                apptReq.Patient.SpecialNeedsPreferences.DateAdded             = dr["DATE_ADDED"].ToString ();
                apptReq.Patient.SpecialNeedsPreferences.AddedByUserIen        = dr["ADDED_BY_USER_IEN"].ToString ();
                apptReq.Patient.SpecialNeedsPreferences.AddedByUserName       = dr["ADDED_BY_USER_NAME"].ToString ();
                apptReq.Patient.SpecialNeedsPreferences.DateInactivated       = dr["DATE_INACTIVATED"].ToString ();
                apptReq.Patient.SpecialNeedsPreferences.InactivatedByUserIen  = dr["INACTIVATED_BY_USER_IEN"].ToString ();
                apptReq.Patient.SpecialNeedsPreferences.InactivatedByUserName = dr["INACTIVATED_BY_USER_NAME"].ToString ();

                apptReq.Patient.Preferences.Add (dr["PREF_TEXT"].ToString ());
                var prefText = dr["PREF_TEXT"].ToString ();
                switch (prefText) {
                    case "ON STRETCHER":
                        apptReq.Patient.SpecialNeedsPreferences.OnStretcher = true;
                        break;
                    case "NEEDS ESCORT":
                        apptReq.Patient.SpecialNeedsPreferences.NeedsEscort = true;
                        break;
                    case "GENDER SPECIFIC PROVIDER":
                        apptReq.Patient.SpecialNeedsPreferences.GenderSpecificProvider = true;
                        break;
                    case "HEARING IMPAIRED":
                        apptReq.Patient.SpecialNeedsPreferences.HearingImpaired =
                            true;
                        break;
                    case "SPECIAL MODE OF TRANSPORTATION":
                        apptReq.Patient.SpecialNeedsPreferences.SpecialModeOfTransportation = true;
                        break;
                    case "LANGUAGE PREFERENCE":
                        apptReq.Patient.SpecialNeedsPreferences.LanguagePreference = true;
                        break;
                    case "MORNING":
                        apptReq.Patient.SpecialNeedsPreferences.Am = true;
                        break;
                    case "AFTERNOON":
                        apptReq.Patient.SpecialNeedsPreferences.Pm = true;
                        break;
                    case "MONDAY":
                        apptReq.Patient.SpecialNeedsPreferences.Monday = true;
                        break;
                    case "TUESDAY":
                        apptReq.Patient.SpecialNeedsPreferences.Tuesday = true;
                        break;
                    case "WEDNESDAY":
                        apptReq.Patient.SpecialNeedsPreferences.Wednesday = true;
                        break;
                    case "THURSDAY":
                        apptReq.Patient.SpecialNeedsPreferences.Thursday = true;
                        break;
                    case "FRIDAY":
                        apptReq.Patient.SpecialNeedsPreferences.Friday = true;
                        break;
                    case "SATURDAY":
                        apptReq.Patient.SpecialNeedsPreferences.Saturday = true;
                        break;
                }
            }
        }

        private void PatientContextMenu_Opened(object sender, RoutedEventArgs e)
        {
            try
            {
                var selectedRow = PatientContextMenu.GetClickedElement<GridViewRow>();
                var selectedAppointmentRequest = selectedRow.Item as AppointmentRequest;
                if (selectedAppointmentRequest == null)
                {
                    PatientContextMenu.IsEnabled = false;
                    return;
                }

                //foreach(RadMenuItem item in PatientContextMenu.Items)
                //Console.WriteLine(item.Header);
                PatientContextMenu.IsEnabled = true;

                var apptDispositionPatientMenuItem = PatientContextMenu.Items[0] as RadMenuItem;
                if (apptDispositionPatientMenuItem == null)
                    return;

                foreach (RadMenuItem item in apptDispositionPatientMenuItem.Items)
                    if (item.Header != null)
                    {
                        var header = item.Header.ToString().ToUpper();

                        if (header == "TRANSFER TO EWL")
                        {
                            bool sdwlMenuAccess = ((selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt) || (selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Veteran));
                            item.IsEnabled = !selectedAppointmentRequest.MultipleAppointmentsRequired && sdwlMenuAccess && !Keys.LoggedInUserSDECViewOnlyKey;
                        }
                        else if (!Keys.WaitListDispositions.IsValidMRTCDisposition(header))
                        {
                            if (header == "VIEW REQUEST")
                                item.IsEnabled = !selectedAppointmentRequest.MultipleAppointmentsRequired;
                            else
                                item.IsEnabled = !selectedAppointmentRequest.MultipleAppointmentsRequired && !Keys.LoggedInUserSDECViewOnlyKey;
                        }
                    }

                var ewlDispositionPatientMenuItem = PatientContextMenu.Items[1] as RadMenuItem;
                var ptCSchDispositionPatientMenuItem = PatientContextMenu.Items[5] as RadMenuItem;
                var patientContactOptions = PatientContextMenu.Items[3] as RadMenuItem;
                if (patientContactOptions != null && !Keys.LoggedInUserSDECViewOnlyKey)
                    //HPE OLD CONTACT FUCTIONALITY
                    //patientContactOptions.IsEnabled = selectedAppointmentRequest.IsApptOrEwl;
                    patientContactOptions.IsEnabled = true; 
                apptDispositionPatientMenuItem.IsEnabled = Keys.Methods.IsApptOrVet(selectedAppointmentRequest.AppointmentRequestType) 
                                                           || selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.WelcomeToVa;
                if (ewlDispositionPatientMenuItem != null)
                    ewlDispositionPatientMenuItem.IsEnabled = selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl;

                /****** DISABLES EWL REQUEST EDIT MENU ITEM ******/
                //if (ewlDispositionPatientMenuItem.IsEnabled)
                //    ((RadMenuItem)ewlDispositionPatientMenuItem.Items[1]).IsEnabled = false;
                /****** DISABLES EWL REQUEST EDIT MENU ITEM ******/

                if (ptCSchDispositionPatientMenuItem != null)
                    ptCSchDispositionPatientMenuItem.IsEnabled = selectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch;
                if (!selectedAppointmentRequest.IsApptOrEwl) return;

                //CONTACT LOGIC NO LONGER NEEDED ON THE CONTEXT MENU 
                //THE LOGIC IS NOW EXECUTED WHEN YOU OPEN THE 
                //CONTACT ATTEMPT MODULE
                #region HPE OLD CONTACT LOGIC (COMMENTED OUT)
                //if (patientContactOptions != null && patientContactOptions.IsEnabled)
                //{
                //    if (selectedAppointmentRequest.PatientContacts != null)
                //    {

                //        var bLetterSent = false;
                //        foreach (var pc in selectedAppointmentRequest.PatientContacts)
                //        {
                //            if (bLetterSent)
                //                break;
                //            if (pc.Action.ToUpper() == Keys.PatientContactActions.Letter.ToUpper())
                //            {
                //                foreach (RadMenuItem item in patientContactOptions.Items)
                //                {
                //                    if (item.Header.ToString() == Keys.PatientContactActions.LetterSent)
                //                    {
                //                        item.Visibility = Visibility.Visible;
                //                        item.IsEnabled = false;
                //                        bLetterSent = true;
                //                        break;
                //                    }
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = true;
                //                }
                //            }
                //        }

                //        if ((selectedAppointmentRequest.PatientContacts.Count == 0) || (selectedAppointmentRequest.PatientContacts.Count == 1 && bLetterSent))
                //        {
                //            foreach (RadMenuItem item in patientContactOptions.Items)
                //            {
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt1)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = true;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt2)
                //                {
                //                    item.Visibility = Visibility.Hidden;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt3)
                //                    item.Visibility = Visibility.Hidden;
                //            }
                //        }
                //        if ((selectedAppointmentRequest.PatientContacts.Count == 1 && !bLetterSent) || (selectedAppointmentRequest.PatientContacts.Count == 2 && bLetterSent))
                //        {
                //            foreach (RadMenuItem item in patientContactOptions.Items)
                //            {
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt1)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = false;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt2)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = true;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt3)
                //                    item.Visibility = Visibility.Hidden;
                //            }
                //        }
                //        if ((selectedAppointmentRequest.PatientContacts.Count == 2 && !bLetterSent) || (selectedAppointmentRequest.PatientContacts.Count == 3 && bLetterSent))
                //        {
                //            foreach (RadMenuItem item in patientContactOptions.Items)
                //            {
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt1)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = false;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt2)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = false;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt3)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = true;
                //                }
                //            }
                //        }
                //        if ((selectedAppointmentRequest.PatientContacts.Count > 2 && !bLetterSent) || (selectedAppointmentRequest.PatientContacts.Count > 3 && bLetterSent))
                //        {
                //            foreach (RadMenuItem item in patientContactOptions.Items)
                //            {
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt1)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = false;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt2)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = false;
                //                    continue;
                //                }
                //                if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt3)
                //                {
                //                    item.Visibility = Visibility.Visible;
                //                    item.IsEnabled = false;
                //                }
                //            }
                //        }
                //    }
                //    else
                //    {
                //        foreach (RadMenuItem item in patientContactOptions.Items)
                //        {
                //            if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt1)
                //            {
                //                item.Visibility = Visibility.Visible;
                //                item.IsEnabled = true;
                //                continue;
                //            }
                //            if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt2)
                //            {
                //                item.Visibility = Visibility.Hidden;
                //                continue;
                //            }
                //            if (item.Header.ToString() == Keys.PatientContactActions.ContactAttempt3)
                //            {
                //                item.Visibility = Visibility.Hidden;
                //                continue;
                //            }
                //            if (item.Header.ToString() == Keys.PatientContactActions.LetterSent)
                //            {
                //                item.Visibility = Visibility.Visible;
                //                item.IsEnabled = true;
                //            }
                //        }
                //    }
                //} 
                #endregion

            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
            e.Handled = true;
        }

        private void OnGridViewDropInfo(object sender, DragDropEventArgs e)
        {
        }

        private void OnGridViewDragQuery(object sender, DragDropQueryEventArgs e)
        {
            try
            {
                if ( !( e.Options.Source is GridViewRow ) )
                    return;

                var gridView = sender as RadGridView;
                // IEnumerable draggedItems = e.Options.Payload as IEnumerable;
                if (gridView != null)
                {
                    e.QueryResult = gridView.SelectedItems.Count > 0;
                    e.Options.Payload = gridView.SelectedItems;
                }
                else
                {
                    e.QueryResult = true;
                    e.Handled = true;
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private void OnGridViewDragInfo(object sender, DragDropEventArgs e)
        {
            try
            {
                if ( !( e.Options.Source is GridViewRow ) )
                    return;

                if (e.Options.Status == DragStatus.DragInProgress)
                {
                    var radGridView = sender as RadGridView;
                    if (radGridView != null)
                    {
                        var cue2 = new ContentControl();
                        var content = radGridView.SelectedItems[0] as AppointmentRequest;
                        if (content != null)
                            cue2.Content = new Label { Background = new SolidColorBrush(Colors.Cornsilk), Content = content.Patient.Name };
                        else
                            cue2.Content = e.Options.Payload as IEnumerable;
                        e.Options.DragCue = cue2;
                    }
                    e.Handled = true;
                }
                else if (e.Options.Status == DragStatus.DragComplete)
                {
                    e.Handled = true;
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "OnGridViewDragInfo", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private void WaitTimeValues_Change(object sender, SelectionChangedEventArgs e)
        {
            Model.View.UpdatePatientSearchText(string.Empty);
        }

        public void UpdateWaitTimeValue(string value)
        {
            if (string.IsNullOrEmpty(value))
                value = "WAIT TIME All Days";
            WaitTimesValue.SelectedValue = value;
        }

        private void AdmittedPatients_ElementExporting(object sender, GridViewElementExportingEventArgs e)
        {
            if (e.Context is GridViewDataColumn && e.Value is RadComboBox)
            {
                e.Value = "WAIT TIME";
            }
        }

        private int _curPageIndex;
        private void WaitList_Changed(object sender, PageIndexChangedEventArgs e)
        {
            try
            {
                if ( !string.IsNullOrEmpty( Keys.WaitListQueryLastRec ) && _curPageIndex >= 0 && AdmittedPatientsPager.PageIndex > 0 )
                    AdmittedPatientsPager.PageIndex = _curPageIndex;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "WaitList_Changed", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private void WaitList_Changing(object sender, PageIndexChangingEventArgs e)
        {
            if (!string.IsNullOrEmpty(Keys.WaitListQueryLastRec))
                _curPageIndex = e.NewPageIndex;

            if ( e.NewPageIndex <= AdmittedPatientsPager.PageCount - 1 || string.IsNullOrEmpty( Keys.WaitListQueryLastRec ) )
                return;

            Model.WaitList_Changing();
            return;
        }

        private void RequestGrid_MouseRightButtonUpEvent(object sender, MouseButtonEventArgs e)
        {
            try
            {
                var s = e.OriginalSource as FrameworkElement;
                //var selectedRow = sender as AppointmentRequest;
                if (s is TextBlock || s is Border)
                {
                    var parentRow = s.ParentOfType<GridViewRow>();
                    if (parentRow != null)
                    {
                        _isRightClickSelect = true;
                        parentRow.IsSelected = true;
                    }
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "RequestGrid_MouseRightButtonUpEvent", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        public void HotKeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
            {
                switch (k)
                {
                    case Key.G:
                        if (GrAdmittedPatients.HasItems)
                        {
                            GrAdmittedPatients.SelectedItem = GrAdmittedPatients.Items[0];
                            GrAdmittedPatients.ScrollIntoView(GrAdmittedPatients.SelectedItem);
                            GrAdmittedPatients.Focus();
                        }
                        break;
                    case Key.Up:
                        MoveFocusToPrevRow();
                        break;
                    case Key.Down:
                        MoveFocusToNextRow();
                        break;
                }
            }
        }

        private void MoveFocusToNextRow()
        {
            if ( GrAdmittedPatients.HasItems )
            {
                var currIndex = GrAdmittedPatients.Items.IndexOf( GrAdmittedPatients.SelectedItem );
                if ( GrAdmittedPatients.Items.Count > currIndex + 1 )
                {
                    GrAdmittedPatients.SelectedItem = GrAdmittedPatients.Items[currIndex + 1];
                    GrAdmittedPatients.ScrollIntoView( GrAdmittedPatients.SelectedItem );
                    GrAdmittedPatients.Focus();
                }
            }
        }

        private void MoveFocusToPrevRow()
        {
            if (GrAdmittedPatients.HasItems)
            {
                var currIndex = GrAdmittedPatients.Items.IndexOf(GrAdmittedPatients.SelectedItem);
                if (currIndex-1>=0)
                {
                    GrAdmittedPatients.SelectedItem = GrAdmittedPatients.Items[currIndex - 1];
                    GrAdmittedPatients.ScrollIntoView(GrAdmittedPatients.SelectedItem);
                    GrAdmittedPatients.Focus();
                }
            }
        }

        public void ScrollSelectedRowIntoView()
        {
            if (GrAdmittedPatients.SelectedItem != null)
                GrAdmittedPatients.ScrollIntoViewAsync(GrAdmittedPatients.SelectedItem, GrAdmittedPatients.Columns[0], null);
        }

        private void GrAdmittedPatients_Drop( object sender, DragEventArgs e )
        {

        }
    }
}
