﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Timers;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.DataAccess.Services;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Telerik.Windows.Controls;
using ComboBox = System.Windows.Controls.ComboBox;
using KeyEventArgs = System.Windows.Input.KeyEventArgs;
using Keys = ClinSchd.Infrastructure.Keys;
using MessageBox = System.Windows.MessageBox;
using TextBox = System.Windows.Controls.TextBox;

namespace ClinSchd.Modules.Ribbon.Schedule
{
    /// <summary>
    /// Interaction logic for RequestManagement.xaml
    /// </summary>
    [System.Runtime.InteropServices.Guid( "06EEEA9F-DAD7-4539-8110-87CF2C45D92D" )]
    public partial class RequestManagement
    {
        public const string SERVICE_CONNECTED = "SERVICE CONNECTED";
        public const string REGULAR = "REGULAR";

        public bool DialogResult { get; set; }

        public AppointmentRequest AppointmentRequest
        {
            get { return DataContext as AppointmentRequest; }
            private set { DataContext = value; }
        }

        public DateTime? OriginalDesiredDate { get; set; }
        private OVIDDataTableRepository _adt;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private readonly Mode _mode;
        private readonly bool _isTransferToEwl;
        private readonly bool _isTransferToPtCSch;
        //public bool IsProhibited { get; set; }
        private bool _isLoaded;

        public RequestManagement(IDataAccessService dataAccessService, IEventAggregator eventAggregator)
            : this (dataAccessService, eventAggregator, null, Mode.NewRequest)
        {
        }

        public IList<AppointmentType> AppointmentTypes
        {
            get { return _dataAccessService.GetAppointmentTypes( AppointmentRequest.Patient.IEN ); }
        }

        public RequestManagement(IDataAccessService dataAccessService, 
            IEventAggregator eventAggregator, AppointmentRequest appointmentRequest, Mode mode)
        {
            _dataAccessService = dataAccessService;
            InitializeComponent();

            _ptCSchClinicSearchTimer       = new System.Timers.Timer(Keys.SearchTimerInterval);
            _ptCSchClinicSearchTimer.Elapsed += ptCSchSearchTimer_Elapsed;
            _serviceClinicSearchTimer      = new System.Timers.Timer(Keys.SearchTimerInterval);
            _serviceClinicSearchTimer.Elapsed += resourceSearchTimer_Elapsed;
            _providersSearchTimer          = new System.Timers.Timer(Keys.SearchTimerInterval);
            _providersSearchTimer.Elapsed += providersSearchTimer_Elapsed;
            _cmbPtCSchProvidersSearchTimer = new System.Timers.Timer(Keys.SearchTimerInterval);
            _cmbPtCSchProvidersSearchTimer.Elapsed += cmbPtCSchProvidersSearchTimer_Elapsed;

            _mode = mode;

            _isTransferToEwl = mode == Mode.EwlTransfer;
            _isTransferToPtCSch = mode == Mode.PtCSchTransfer;

            

            _eventAggregator = eventAggregator;

            if ( appointmentRequest == null )
                AppointmentRequest = new AppointmentRequest( _dataAccessService );
            else
            {
                if (appointmentRequest.IsAppt || appointmentRequest.IsVeteran)
                    AppointmentRequest = _dataAccessService.GetApptRequestByRequestIEN(appointmentRequest.ApptRequestIen.ToString());
                else if (appointmentRequest.IsEwl)
                    AppointmentRequest = _dataAccessService.GetEwlApptRequest(appointmentRequest.RequestManagmentIen);
                else if (appointmentRequest.IsPtCSch && !_isTransferToPtCSch)
                    AppointmentRequest = _dataAccessService.GetPtCSchApptRequest(appointmentRequest.PtCSchRequestIen.ToString());
                else if (_isTransferToPtCSch)
                    AppointmentRequest = appointmentRequest;

                AppointmentRequest.Patient = appointmentRequest.Patient;  // added Patient info - interim until calls above are working.
            }
            for ( var i = 2; i <= 60; i++)
                NumberOfAppointmentsRequired.Items.Add(i);
            for (var i = 1; i <= 30; i++)
                IntervalBetweenAppointments.Items.Add(i);

            eventAggregator.GetEvent<PatientUpdatedEvent>().Subscribe( AppointmentRequest.PatientUpdate, ThreadOption.UIThread, true );
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                var key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.O:
                        Keyboard.Focus(BtnOk);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnCancel);
                        break;
                    case Key.R:
                        Keyboard.Focus(RbClinic);
                        break;
                }
            }
        }

        private void Validation_Error(object sender, ValidationErrorEventArgs e)
        {
            if ( e.Action == ValidationErrorEventAction.Added )
                AppointmentRequest.Errors += 1;
            if ( e.Action == ValidationErrorEventAction.Removed )
                AppointmentRequest.Errors -= 1;
        }

        public AppointmentRequest UpdatedAppointmentRequest { get; set; }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            if (_mode == Mode.ViewRequest)
            {
                DialogResult = true;
                CloseThisWindow();
                return;
            }

            _adt = OVIDDataTableRepository.Instance;

            //Check if mrtc request is valid..
            if (AppointmentRequest.MultipleAppointmentsRequired && (AppointmentRequest.NumberOfAppointmentsRequired == 0 || AppointmentRequest.AppointmentIntervalInDays == 0))
            {
                MessageBox.Show("If this request requires multiple appointments, please supply the number of appointments required, and the interval between appointments (in days).");
                return;
            }

            /*Validate these fields..
        .01   PATIENT
            2   INSTITUTION
            4   WAIT LIST TYPE
            8   WL SPECIFIC CLINIC
            10   PRIORITY
            11   REQUEST BY
            22   DESIRED DATE OF APPOINTMENT*/

            if (!AppointmentRequest.MultipleAppointmentsRequired || AppointmentRequest.NumberOfAppointmentsRequired == 0 || AppointmentRequest.AppointmentIntervalInDays == 0)
            {
                AppointmentRequest.MultipleAppointmentsRequired = false;
                ChkIsMultipleAppt.IsChecked = false;
            }

            if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch)
            {
                if (string.IsNullOrEmpty(CmbPtCSchApptTypes.Text))
                {
                    MessageBox.Show("Please provide a PtCSch Appointment Type for this appointment request.");
                    CmbPtCSchApptTypes.Focus();
                    return;
                }
                var bFound = false;
                if (!string.IsNullOrEmpty(CmbPtCSchProvider.Text))
                    bFound = CmbPtCSchProvider.Items.Cast<Provider>().Any(prov => prov.Name.ToUpper() == CmbPtCSchProvider.Text.ToUpper());
                if (!bFound)
                {
                    MessageBox.Show("Please provide a PtCSch Provider for this appointment request.");
                    CmbPtCSchProvider.Focus();
                    return;
                }
                if (string.IsNullOrEmpty(CmbPtCSchClinic.Text))
                {
                    MessageBox.Show("Please provide a PtCSch Clinic or Service for this appointment request.");
                    CmbPtCSchClinic.Focus();
                    return;
                }
                if (AppointmentRequest.PtCSchDate == null)
                {
                    MessageBox.Show("Please provide a PtCSch Date for this appointment request.");
                    PtCSchDatePicker.Focus();
                    return;
                }
                if (txtAppointmentDuration.Text.Length > 0)
                {
                    if (!Keys.Methods.IsNumeric(txtAppointmentDuration.Text))
                    {
                        MessageBox.Show(Keys.AppointmentLengthValidationErrorMsg);
                        txtAppointmentDuration.Focus();
                        return;
                    }
                    long testNumber;
                    if (!long.TryParse(txtAppointmentDuration.Text, NumberStyles.None, null, out testNumber))
                    {
                        MessageBox.Show(Keys.AppointmentLengthValidationErrorMsg);
                        txtAppointmentDuration.Focus();
                        return;
                    }
                    if (Convert.ToInt32(txtAppointmentDuration.Text) < 10 || Convert.ToInt32(txtAppointmentDuration.Text) > 120)
                    {
                        MessageBox.Show(Keys.AppointmentLengthValidationErrorMsg);
                        txtAppointmentDuration.Focus();
                        return;
                    }
                }
            }
            else
            {
                if (string.IsNullOrEmpty(CmbServiceClinic.Text))
                {
                    MessageBox.Show("Please provide a clinic, service or specialty clinic for the appointment request.");
                    CmbServiceClinic.Focus();
                    return;
                }
                if (AppointmentRequest.RequestedBy == null)
                //if (string.IsNullOrEmpty(CmbRequestedBy.Text))
                {
                    MessageBox.Show("Please provide who requested the appointment request.");
                    CmbRequestedBy.Focus();
                    return;
                }
                if (CalDesiredApptDate.SelectedDate == null && AppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.Ewl)
                {
                    MessageBox.Show("Please select a CID/Preferred Date for the appointment request.");
                    CalDesiredApptDate.Focus();
                    return;
                }
                if (string.IsNullOrEmpty(CmbEnrolleeStatus.Text))
                {
                    MessageBox.Show("Please provide a status for this appointment request.");
                    CmbEnrolleeStatus.Focus();
                    return;
                }
                if (CmbRequestedBy.Text.ToUpper() == "PROVIDER")
                {
                    var bFound = false;
                    if (!string.IsNullOrEmpty(CmbProviders.Text))
                        bFound = CmbProviders.Items.Cast<Provider>().Any(prov => prov.Name.ToUpper() == CmbProviders.Text.ToUpper());
                    if (!bFound)
                    {
                        MessageBox.Show("Please select a Provider.");
                        CmbProviders.Focus();
                        return;
                    }
                }
                if (string.IsNullOrEmpty(CmbAppointmentTypes.Text))
                {
                    MessageBox.Show("Please select an Appointment Type.");
                    CmbAppointmentTypes.Focus();
                    return;
                }
            }

            if (AppointmentRequest.MultipleAppointmentsRequired)
            {
                // All other warnings are done so let's check the overall duration of the MRTC request to insure it is less than 24 months
                // _appointmentRequest.NumberOfAppointmentsRequired == 0 || _appointmentRequest.AppointmentIntervalInDays
                var firstDate = AppointmentRequest.DesiredApptDate.Value.Date;
                var finalDate = firstDate.AddDays(AppointmentRequest.NumberOfAppointmentsRequired*AppointmentRequest.AppointmentIntervalInDays);
                var monthsBetween = finalDate.Month - firstDate.Month + 12*(finalDate.Year - firstDate.Year);

                if (monthsBetween > 24)
                {
                    MessageBox.Show(
                        "The combination of requested appointments and the interval between appointments exceeds 24 months.  " +
                        "Please adjust the number of requested appointmentst and\\or the interval between appointments so that the combined duration is less than 24 months.",
                        "MRTC Interval/Appt. #");
                    NumberOfAppointmentsRequired.Focus();
                    return;
                }
            }

            AppointmentRequest.Patient.ServiceRelated = ChkSvcRelated.IsChecked == true;
            if (AppointmentRequest.Patient.ServiceRelated)
                AppointmentRequest.Patient.SCVisit = "YES";

            if (_mode == Mode.NewRequest)
                AppointmentRequest.OriginatingDate = DateTime.Now.Date;

            if (AppointmentRequest.IsPtCSch)
            {
                try
                {
                    var ptCSchParams = GetPtCSchRequestTypeParams();
                    var rslt = _adt.callOVIDRPC(Keys.RpcCalls.PtCSchSet, ptCSchParams);
                    foreach (DataRow row in rslt.Rows)
                    {
                        var errorId = row["ERRORID"].ToString();
                        var errorText = row["ERRORTEXT"].ToString();

                        if (errorId != "0")
                        {
                            if (!string.IsNullOrEmpty(errorText))
                            {
                                MessageBox.Show(errorText);
                                return;
                            }
                        }
                        else
                        {
                            if (Keys.Methods.IsNumeric(errorText))
                            {
                                AppointmentRequest.PtCSchRequestIen = Convert.ToInt32(errorText);
                                AppointmentRequest.RequestManagmentIen = errorText;

                                var parms1 = new List<string>
                                {
                                    null,
                                    null,
                                    null,
                                    "1",
                                    null,
                                    AppointmentRequest.PtCSchRequestIen.ToString(CultureInfo.InvariantCulture)
                                };

                                DataTable patientData = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, parms1);

                                if (patientData.Rows.Count > 0)
                                {
                                    DataRow patientRow = patientData.Rows[0];
                                    IList<AppointmentRequest> newRequestList = _dataAccessService.GetPtCSchAppointmentRequests(patientData);
                                    AppointmentRequest newRequest = newRequestList[0];

                                    if (_mode == Mode.EditRequest)
                                    {
                                        bool imported = false;

                                        foreach (DataRow ptCSchRow in Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows)
                                        {
                                            if (ptCSchRow["IEN"].ToString() == patientRow["IEN"].ToString())
                                            {
                                                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows.Remove(ptCSchRow);
                                                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.ImportRow(patientRow);
                                                imported = true;
                                                break;
                                            }
                                        }

                                        if (!imported)
                                            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.ImportRow(patientRow);

                                        AppointmentRequest oldRequest = Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.FirstOrDefault(r => (r.UniqueID == newRequest.UniqueID));
                                        
                                        if (oldRequest != null)
                                        {
                                            int idx = Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.IndexOf(oldRequest);
                                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[idx] = newRequest;
                                        }

                                        UpdatedAppointmentRequest = newRequest;
                                    }
                                    else if (_mode == Mode.NewRequest)
                                    {
                                        Keys.AppointmentRequestWaitListTables.PtCSchWaitList.ImportRow(patientRow);
                                        Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(newRequest);
                                    }
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Request Save Error");
                }
                DialogResult = true;
                CloseThisWindow();
                return;
            }

            if (_mode != Mode.NewRequest)
                AppointmentRequest.DateEntered = AppointmentRequest.OriginatingDate.Value.AddMinutes(1);
            else
                AppointmentRequest.DateEntered = DateTime.Now;

            //if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl || Keys.Methods.IsApptOrVet(AppointmentRequest.AppointmentRequestType) || AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.WelcomeToVa)
            //{
            //}

            AppointmentRequest.Patient.Ethnicity = AppointmentRequest.Patient.Ethnicities.FirstOrDefault(eth => eth.Name == CmbEthnicities.Text);

            AppointmentRequest.Patient.PrimaryContactInfo.ZipPlus4 = TxtZipCode.Text;
            AppointmentRequest.Patient.PrimaryContactInfo.Street1 = TxtAddress.Text;
            AppointmentRequest.Patient.PrimaryContactInfo.PhoneResidence = TxtPhoneResidence.Text;
            AppointmentRequest.Patient.PrimaryContactInfo.PhoneWork = TxtPhoneWork.Text;
            AppointmentRequest.MultipleAppointmentsRequired = ChkIsMultipleAppt.IsChecked == true;
            AppointmentRequest.ClinicServiceSpecialtyMerge = CmbServiceClinic.Text;

            Clinic selectedClinic = null;

            if (RbService.IsChecked != true && CmbServiceClinic.SelectedItem != null)
            {
                var list = _dataAccessService.GetPtCSchClinics(((NameValue) CmbServiceClinic.SelectedItem).Value, null);
                if (list.Count > 0)
                    selectedClinic = list[0];

                AppointmentRequest.Clinic = selectedClinic;
            }

            var serviceClinic = (NameValue) CmbServiceClinic.SelectedItem;
            if (serviceClinic == null)
            {
                MessageBox.Show(Keys.InvalidClinicRequestMessage);
                return;
            }

            if (!string.Equals(serviceClinic.Name, CmbServiceClinic.Text, StringComparison.CurrentCultureIgnoreCase))
            {
                if (RbClinic.IsChecked == true)
                {
                    if (!_dataAccessService.ClinicExists(CmbServiceClinic.Text))
                    {
                        MessageBox.Show(Keys.InvalidClinicRequestMessage);
                        return;
                    }
                }
                else
                {
                    if (!_dataAccessService.ServiceExists(CmbServiceClinic.Text))
                    {
                        MessageBox.Show(Keys.InvalidClinicRequestMessage);
                        return;
                    }
                }
            }
            try
            {
                DataTable dt;
                List<string> parms;
                if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt 
                    || AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.WelcomeToVa 
                    || AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Veteran
                    || AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.ReturnToClinic)
                {
                    parms = GetApptRequestTypeParams();
                    dt = _adt.callOVIDRPC(Keys.RpcCalls.AppointmentRequestSet, parms);
                }
                else if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl)
                {
                    parms = GetEwlRequestTypeParms(serviceClinic);
                    dt = _adt.callOVIDRPC(Keys.RpcCalls.WaitListSet, parms);
                }
                else
                    dt = null;

                if (dt != null && dt.Rows.Count == 1)
                {
                    var r = dt.Rows[0];
                    if (r["ERRORTEXT"].ToString() != string.Empty)
                    {
                        var sError = r["ERRORTEXT"].ToString();
                        MessageBox.Show("Unable to Create Appointment Request.  " + sError);
                    }
                    else
                    {
                        if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt && ChkIsVeteran.IsChecked == true)
                            AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Veteran;

                        AppointmentRequest.RequestManagmentIen = r["ERRORID"].ToString();

                        if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt 
                            || AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Veteran)
                            AppointmentRequest.ApptRequestIen = Convert.ToInt32(r["ERRORID"]);

                        if (AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl)
                        {
                            var parms1 = new List<string> {AppointmentRequest.RequestManagmentIen};
                            DataTable patientData = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, parms1);

                            if (patientData.Rows.Count > 0)
                            {
                                DataRow patientRow = patientData.Rows[0];
                                IList<AppointmentRequest> newRequestList = _dataAccessService.GetEwlAppointmentRequests(patientData);
                                AppointmentRequest newRequest = newRequestList[0];

                                if ((_mode == Mode.EditRequest) || (_mode == Mode.EwlTransfer))
                                {
                                    bool imported = false;

                                    foreach (DataRow row in Keys.AppointmentRequestWaitListTables.EwlWaitList.Rows)
                                    {
                                        if (row["IEN"].ToString() == patientRow["IEN"].ToString())
                                        {
                                            Keys.AppointmentRequestWaitListTables.EwlWaitList.Rows.Remove(row);
                                            Keys.AppointmentRequestWaitListTables.EwlWaitList.ImportRow(patientRow);
                                            imported = true;
                                            break;
                                        }
                                    }

                                    if (!imported)
                                        Keys.AppointmentRequestWaitListTables.EwlWaitList.ImportRow(patientRow);

                                    for (var i = 0; i < Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count; i++)
                                    {
                                        if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i].UniqueID == newRequest.UniqueID)
                                        {
                                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i] = newRequest;
                                            break;
                                        }
                                    }

                                    UpdatedAppointmentRequest = newRequest;
                                }
                            }
                        }
                        else if (AppointmentRequest.IsAppt || AppointmentRequest.IsVeteran)
                        {
                            var parms1 = new List<string> {AppointmentRequest.ApptRequestIen.ToString(CultureInfo.InvariantCulture), "1", "", "", ""};
                            DataTable patientData = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, parms1);

                            if (patientData.Rows.Count > 0)
                            {
                                DataRow patientRow = patientData.Rows[0];
                                IList<AppointmentRequest> newRequestList = _dataAccessService.GetApptAppointmentRequests(patientData);
                                AppointmentRequest newRequest = newRequestList[0];

                                if (_mode == Mode.EditRequest)
                                {
                                    bool imported = false;

                                    foreach (DataRow row in Keys.AppointmentRequestWaitListTables.ApptWaitList.Rows)
                                    {
                                        if (row["IEN"].ToString() == patientRow["IEN"].ToString())
                                        {
                                            Keys.AppointmentRequestWaitListTables.ApptWaitList.Rows.Remove(row);
                                            Keys.AppointmentRequestWaitListTables.ApptWaitList.ImportRow(patientRow);
                                            imported = true;
                                            break;
                                        }
                                    }

                                    if (!imported)
                                        Keys.AppointmentRequestWaitListTables.ApptWaitList.ImportRow(patientRow);

                                    for (var i = 0; i < Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count; i++)
                                    {
                                        if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i].UniqueID == newRequest.UniqueID)
                                        {
                                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i] = newRequest;
                                            break;
                                        }
                                    }

                                    UpdatedAppointmentRequest = newRequest;
                                }
                            }
                        }
                        else if (AppointmentRequest.IsPtCSch)
                        {
                            var parms1 = new List<string> {"", "", "", "1", "", AppointmentRequest.PtCSchRequestIen.ToString(CultureInfo.InvariantCulture)};
                            DataTable patientData = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, parms1);

                            if (patientData.Rows.Count > 0)
                            {
                                DataRow patientRow = patientData.Rows[0];
                                IList<AppointmentRequest> newRequestList = _dataAccessService.GetPtCSchAppointmentRequests(patientData);
                                AppointmentRequest newRequest = newRequestList[0];

                                if (_mode == Mode.EditRequest)
                                {
                                    bool imported = false;

                                    foreach (DataRow row in Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows)
                                    {
                                        if (row["IEN"].ToString() == patientRow["IEN"].ToString())
                                        {
                                            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows.Remove(row);
                                            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.ImportRow(patientRow);
                                            imported = true;
                                            break;
                                        }
                                    }

                                    if (!imported)
                                        Keys.AppointmentRequestWaitListTables.PtCSchWaitList.ImportRow(patientRow);

                                    for (var i = 0; i < Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count; i++)
                                    {
                                        if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i].UniqueID == newRequest.UniqueID)
                                        {
                                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i] = newRequest;
                                            break;
                                        }
                                    }

                                    UpdatedAppointmentRequest = newRequest;
                                }
                            }
                        }
                    }
                }
                else
                    MessageBox.Show("Unable to Create\\Update Appointment Request.");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Appointment Request Save Error");
            }

            DialogResult = true;
            CloseThisWindow();
        }

        private List<string> GetEwlRequestTypeParms( NameValue serviceClinic )
        {
            var waitListStr = "";
            if ( AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
            {
                waitListStr = RbClinic.IsChecked == true ? "SPECIFIC CLINIC" : "SERVICE/SPECIALITY";
            }
            else if ( Keys.Methods.IsApptOrVet( AppointmentRequest.AppointmentRequestType ) )
            {
                waitListStr = "APPOINTMENT";
                if ( RbService.IsChecked == true )
                {
                    AppointmentRequest.ClinicStopName = CmbServiceClinic.Text;
                    foreach ( var spec in AppointmentRequest.GetApptServiceSpecialties( AppointmentRequest.ClinicStopName ) )
                        if ( spec.Name.ToUpper() == AppointmentRequest.ClinicStopName.ToUpper() )
                            AppointmentRequest.ClinicStopIen = spec.Ien;
                }
            }

            var parms = new List<string>
            {
                // INP(1) = Wait List IEN point to SD WAIT LIST file 409.3.
                //         If null, a new entry will be added
                _mode == Mode.EditRequest ? AppointmentRequest.RequestManagmentIen : null,

                // INP(2) = Patient NAME - name field from the PATIENT file
                AppointmentRequest.Patient.IEN,

                // INP(3) = Originating Date and Time in external date form 
                AppointmentRequest.DateEntered.Value.ToString( "M/d/yyyy@H:mm" ),

                // INP(4) = Institution name NAME field from the INSTITUTION file
                AppointmentRequest.Patient.Institution,

                // INP(5) = Wait List Type 
                //            PCMM TEAM ASSIGNMENT 
                //            PCMM POSITION ASSIGNMENT
                //            SERVICE /SPECIALITY
                //            SPECIFIC CLINIC 
                waitListStr,

                // INP(6) = WL Specific Team name - NAME field in the TEAM file 404.51
                null,

                // INP(7) = WL Specific Position name - NAME field in The TEAM POSITION file 404.57.
                null
            };


            if ( RbClinic.IsChecked == true )
            {
                // INP( 8 ) = WL Service / Specialty name - NAME field in SD WL SERVICE / SPECIALTY file 409.31.
                AppointmentRequest.Specialty = null;
                parms.Add( null );
                if ( serviceClinic == null )
                {
                    AppointmentRequest.ClinName = CmbServiceClinic.Text;
                    AppointmentRequest.ClinicIen = CmbServiceClinic.Text;
                }
                else
                {
                    AppointmentRequest.ClinName = serviceClinic.Name;
                    AppointmentRequest.ClinicIen = serviceClinic.Value;
                }
                AppointmentRequest.Clinic = new Clinic( _dataAccessService )
                {
                    ClinicIen = serviceClinic.Value,
                    ClinicName = serviceClinic.Name
                };

                // INP(9) = WL Specific Clinic – Numeric will be used as the ID to the SD WL CLINIC LOCATION file
                //                               Beginning with Alpha will be used as the NAME from the SD WL CLINIC LOCATION file 409.32.
                if ( AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
                {
                    if (string.IsNullOrEmpty(serviceClinic.AlternateValue))
                    {
                        string clinIen;

                        if (string.IsNullOrEmpty(serviceClinic.AlternateName))
                            clinIen = _dataAccessService.GetEwlClinicIen(serviceClinic.Name);
                        else
                            clinIen = _dataAccessService.GetEwlClinicIen( serviceClinic.AlternateName );

                        parms.Add( !string.IsNullOrEmpty( clinIen ) ? clinIen : serviceClinic.Value );
                    }
                    else
                        parms.Add( serviceClinic.AlternateValue );
                }
                else
                    parms.Add( serviceClinic.Value );
            }
            else
            {
                // INP( 8 ) = WL Service / Specialty name - NAME field in SD WL SERVICE / SPECIALTY file 409.31.
                if ( serviceClinic == null )
                {
                    AppointmentRequest.Specialty = CmbServiceClinic.Text;
                    AppointmentRequest.SpecialtyIen = CmbServiceClinic.Text;
                    parms.Add( CmbServiceClinic.Text );
                }
                else
                {
                    AppointmentRequest.Specialty = serviceClinic.Name;
                    AppointmentRequest.SpecialtyIen = serviceClinic.Value;
                    parms.Add( serviceClinic.Value );
                }

                // INP(9) = WL Specific Clinic – Numeric will be used as the ID to the SD WL CLINIC LOCATION file
                //                               Beginning with Alpha will be used as the NAME from the SD WL CLINIC LOCATION file 409.32.
                AppointmentRequest.ClinName = null;
                AppointmentRequest.ClinicIen = null;
                AppointmentRequest.Clinic = null;
                parms.Add( null );
            }

            // INP(10) = Originating User name - NAME field in NEW PERSON file 200
            parms.Add( AppointmentRequest.Patient.OriginatingUserName );

            // INP(11) = <NOT USED> Priority - 'ASAP' or 'FUTURE' Priority is set based on 14 Desired Date of Appt. 
            parms.Add( AppointmentRequest.Priority );

            // INP(12) = Request By - 'PROVIDER' or 'PATIENT' 
            //AppointmentRequest.RequestedBy = CmbRequestedBy.Text == "PROVIDER" ? 1 : 2;
            parms.Add( AppointmentRequest.RequestedByText );

            // INP(13) = Provider name - NAME field in NEW PERSON file 200 
            var providerName = CmbProviders.SelectedValue == null ? "" : ( (Provider)CmbProviders.SelectedValue ).Name;
            parms.Add( providerName );

            // INP(14) = Desired Date of appointment in external format. 
            AppointmentRequest.Provider = new Provider { Name = providerName };
            var desiredApptDate = CalDesiredApptDate.SelectedDate == null
                ? ""
                : CalDesiredApptDate.SelectedDate.ToString();
            var dateStuff = desiredApptDate.Split( ' ' );
            var dateOnly = dateStuff[0];
            AppointmentRequest.DesiredApptDate = Convert.ToDateTime( dateOnly );
            OriginalDesiredDate = AppointmentRequest.DesiredApptDate;
            parms.Add( dateOnly ); //DESIRED DATE OF APPOINTMENT

            // INP(15) = comment must be 1-60 characters 
            var comment = TxtComment.Text;
            AppointmentRequest.RequestManagmentComment = comment;
            parms.Add( comment ); //COMMENTS

            // INP(16) = EWL Enrollee Status 
            //                NEW 
            //                ESTABLISHED
            //                PRIOR
            //                UNDETERMINED 
            parms.Add( AppointmentRequest.EnrolleeStatus );

            // INP(17) = NOT USED - PATIENT TELEPHONE 4-20 characters 
            parms.Add( AppointmentRequest.Patient.PrimaryContactInfo.Phone );

            // INP(18) = ENROLLMENT PRIORITY - Valid Values are: 
            //              GROUP 1 
            //              GROUP 2 
            //              GROUP 3 
            //              GROUP 4
            //              GROUP 5
            //              GROUP 6
            //              GROUP 7
            //              GROUP 8 
            AppointmentRequest.Patient.PriorityGroup = CmbPriorityGroup.Text;
            parms.Add( AppointmentRequest.Patient.PriorityGroup );

            // INP(19) = NOT USED - APPT SCHEDULED 
            parms.Add( null );

            // INP(20) = MULTIPLE APPOINTMENT RTC NO; YES 
            if ( AppointmentRequest.MultipleAppointmentsRequired && AppointmentRequest.NumberOfAppointmentsRequired > 0 && AppointmentRequest.AppointmentIntervalInDays > 0 )
                parms.Add( "YES" );
            else
            {
                parms.Add( "NO" );
                AppointmentRequest.MultipleAppointmentsRequired = false;
            }

            // INP(21) = MULT APPT RTC INTERVAL integer between 1-365 
            if ( !AppointmentRequest.MultipleAppointmentsRequired )
            {
                AppointmentRequest.NumberOfAppointmentsRequired = 0;
                AppointmentRequest.AppointmentIntervalInDays = 0;
            }
            parms.Add( AppointmentRequest.AppointmentIntervalInDays.ToString( CultureInfo.InvariantCulture ) );

            // INP(22) = MULT APPT NUMBER integer between 1-100 
            parms.Add( AppointmentRequest.NumberOfAppointmentsRequired.ToString( CultureInfo.InvariantCulture ) );

            // INP(23) = Patient Contacts separated by ::
            //    Each :: piece has the following ~~ pieces:
            //      1) = DATE ENTERED external date/time
            //      2) = PC ENTERED BY USER ID or NAME - Pointer to NEW PERSON file or NAME
            //      4) = ACTION - 'CALLED', 'MESSAGE LEFT', or "LETTER"
            //      5) = (optional)PATIENT PHONE Free-Text 4 - 20 characters
            //      6) = NOT USED( optional ) Comment 1 - 160 characters
            parms.Add( null );

            // INP(24) = (optional) SERVICE CONNECTED PRIORITY valid values are NO YES
            AppointmentRequest.Patient.ServiceConnection = ChkSvcRelated.IsChecked == true ? "YES" : "NO";
            AppointmentRequest.Patient.SCVisit = ChkSvcRelated.IsChecked == true ? "YES" : "NO";
            parms.Add( AppointmentRequest.Patient.ServiceConnection );

            // INP( 25 ) = (optional)SERVICE CONNECTED PERCENTAGE = numeric 0 - 100
            parms.Add( AppointmentRequest.Patient.ServiceConnectionPercentage );

            // INP(26) = (optional) MRTC calculated preferred dates separated by pipe | 
            //                       Each date can be in external format with no time.
                        var desiredApptDatesStr = "";
            foreach ( var dat in AppointmentRequest.DesiredApptDates )
                desiredApptDatesStr += dat.Value.ToShortDateString() + "|";
            desiredApptDatesStr = desiredApptDatesStr.Substring( 0, desiredApptDatesStr.Length - 1 );

            if ( _isTransferToEwl && AppointmentRequest.MultipleAppointmentsRequired )
            {
                var desDates = desiredApptDatesStr.Split( '|' )[0];
                if ( desDates.Length > 0 )
                    desiredApptDatesStr = desDates;

            }
            if ( AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
                parms.Add( null );
            else
                parms.Add( desiredApptDatesStr );

            // INP(27) = (optional) Appointment Type ID pointer to APPOINTMENT TYPE file 409.1
            AppointmentRequest.AppointmentType = CmbAppointmentTypes.Text;
            var apptType = AppointmentTypes.FirstOrDefault(at => at.Name.ToUpper() == AppointmentRequest.AppointmentType.ToUpper());

            if (apptType != null)
                parms.Add(apptType.Ien);
            else
                parms.Add(null);

            return parms;
        }

        private List<string> GetApptRequestTypeParams()
        {
            var requestType = AppointmentRequest.AppointmentRequestType;
            if (ChkIsVeteran.IsChecked == true)
                requestType = Keys.AppointmentRequestTypes.Veteran;

            if ( _mode != Mode.NewRequest )
                AppointmentRequest.DateEntered = AppointmentRequest.OriginatingDate.Value.AddMinutes( 1 );
            else
                AppointmentRequest.DateEntered = DateTime.Now;

            var parms = new List<string>
            {
                // INP(1) = (optional) Wait List IEN point to SDEC APPT REQUEST file 409.85. If null, a new entry will be added 
                _mode == Mode.EditRequest ? AppointmentRequest.ApptRequestIen.ToString() : null,

                // INP(2) = (required) DFN Pointer to the PATIENT file 2
                AppointmentRequest.Patient.IEN,

                // INP(3) = Originating Date/time in external date form 
                AppointmentRequest.DateEntered.Value.ToString( "M/d/yyyy@H:mm" ),

                // INP(4) = Institution name NAME field from the INSTITUTION file
                AppointmentRequest.Patient.Institution,

                // INP(5) = Request Type 
                requestType
            };

            // INP(6) = REQ Specific Clinic name - NAME field in file 44, NULL otherwise
            if ( RbClinic.IsChecked.HasValue && RbClinic.IsChecked.Value )
            {
                var selectedServiceClinic = (NameValue) CmbServiceClinic.SelectedItem;
                parms.Add(selectedServiceClinic == null ? null : selectedServiceClinic.Value);
            }
            else
                parms.Add( null );

            // INP(7) = Originating User name - NAME field in NEW PERSON file 200
            parms.Add(null);

            // INP(8) = Priority - 'ASAP' or 'FUTURE' 
            parms.Add(AppointmentRequest.Priority);

            // INP(9) = Request By - 'PROVIDER' or 'PATIENT' 
            //AppointmentRequest.RequestedBy = CmbRequestedBy.Text == "PROVIDER" ? 1 : 2;
            parms.Add(AppointmentRequest.RequestedByText);

            // INP(10) = Provider name - NAME field in NEW PERSON file 200 
            var providerName = CmbProviders.SelectedValue == null ? "" : ((Provider)CmbProviders.SelectedValue).Name;
            parms.Add(providerName);

            // INP(11) = Desired Date of appointment in external format. 
            var desiredApptDate = CalDesiredApptDate.SelectedDate == null ? "" : CalDesiredApptDate.SelectedDate.ToString();
            var dateOnly = desiredApptDate.Piece( " ", 1 );
            AppointmentRequest.DesiredApptDate = Convert.ToDateTime(dateOnly);
            OriginalDesiredDate = AppointmentRequest.DesiredApptDate;
            parms.Add(dateOnly);

            // INP(12) = comment must be 1-60 characters 
            var comment = TxtComment.Text;
            AppointmentRequest.RequestManagmentComment = comment;
            parms.Add(comment);

            // INP(13) = ENROLLMENT PRIORITY - Valid Values are: GROUP 1 GROUP 2 GROUP 3 GROUP 4 GROUP 5 GROUP 6 GROUP 7 GROUP 8 
            AppointmentRequest.Patient.PriorityGroup = CmbPriorityGroup.Text;
            parms.Add(AppointmentRequest.Patient.PriorityGroup);

            // INP(14) = MULTIPLE APPOINTMENT RTC NO; YES 
            parms.Add(AppointmentRequest.MultipleAppointmentsRequired ? "YES" : "NO");
            if (!AppointmentRequest.MultipleAppointmentsRequired)
            {
                AppointmentRequest.NumberOfAppointmentsRequired = 0;
                AppointmentRequest.AppointmentIntervalInDays = 0;
            }

            // INP(15) = MULT APPT RTC INTERVAL integer between 1-365 
            parms.Add(AppointmentRequest.AppointmentIntervalInDays.ToString(CultureInfo.InvariantCulture));

            // INP(16) = MULT APPT NUMBER integer between 1-100 - Number of Appointments required
            parms.Add(AppointmentRequest.NumberOfAppointmentsRequired.ToString(CultureInfo.InvariantCulture));

            // INP(17) = Patient Contacts separated by ::
            //    Each :: piece has the following ~~ pieces:
            //    1) = (date) DATE ENTERED external date/time
            //    2) = (text) PC ENTERED BY USER ID or NAME - Pointer to NEW PERSON file or NAME
            //    4) = (optional)ACTION - valid values are:
            //        CALLED
            //        MESSAGE LEFT
            //        LETTER
            //    5) = (optional)PATIENT PHONE Free-Text 4 - 20 characters
            //    6) = NOT USED( optional ) Comment 1 - 160 characters
            parms.Add(null);

            // INP(18) = (optional) SERVICE CONNECTED PRIORITY valid values are NO YES
            AppointmentRequest.Patient.ServiceConnection = ChkSvcRelated.IsChecked == true ? "YES" : "NO";
            AppointmentRequest.Patient.ServiceRelated = ChkSvcRelated.IsChecked == true;  //MES artf20390
            AppointmentRequest.Patient.SCVisit = ChkSvcRelated.IsChecked == true ? "YES" : "NO";

            parms.Add(AppointmentRequest.Patient.ServiceConnection);

            // INP(19) = (optional) SERVICE CONNECTED PERCENTAGE = numeric 0-100 
            parms.Add(AppointmentRequest.Patient.ServiceConnectionPercentage);

            // INP(20) = (optional) MRTC calculated preferred dates separated by pipe |:
            //   Each date can be in external format with no time
            //  ... original desired date only at this time... will calculate subsequent dates after first booking.. ShowStopper #5
            parms.Add(dateOnly);

            // INP(21) = (optional) CLINIC STOP pointer to CLINIC STOP file 40.7
            //            used to populate the REQ SERVICE/SPECIALTY field in 409.85
            //    Service Request ONLY
            if ( RbService.IsChecked == true )
            {
                AppointmentRequest.ClinicStopName = CmbServiceClinic.Text;
                foreach ( var spec in AppointmentRequest.GetApptServiceSpecialties( AppointmentRequest.ClinicStopName ) )
                    if ( spec.Name.ToUpper() == AppointmentRequest.ClinicStopName.ToUpper() )
                    {
                        AppointmentRequest.ClinicStopIen = spec.Ien;
                        break;
                    }
            }

            if ( RbService.IsChecked.HasValue && RbService.IsChecked.Value && !string.IsNullOrEmpty(AppointmentRequest.ClinicStopName))
                parms.Add(AppointmentRequest.ClinicStopIen);
            else
                parms.Add(null);

            // INP(22) = (optional) Appointment Type ID pointer to APPOINTMENT TYPE file 409.1
            AppointmentRequest.AppointmentType = CmbAppointmentTypes.Text;
            var apptType = AppointmentTypes.FirstOrDefault( at => at.Name.ToUpper() == AppointmentRequest.AppointmentType.ToUpper() );

            if ( apptType != null )
                parms.Add(apptType.Ien);
            else 
                parms.Add( null );

            // INP(23) = (optional) Patient Status
            //           N = NEW
            //           E = ESTABLISHED
            parms.Add(AppointmentRequest.EnrolleeStatus);

            // INP(24) = (optional) MULT APPTS MADE separated by pipe
            //               ( child appointments)
            //               ( no more ~pieces)
            //   1.Pointer to SDEC APPOINTMENT file 409.84
            // MRTC Parent\Master request ONLY) MRTC Child Appointment IEN list, separated by '|' - Not used here, set by Find Appt process and SDEC APPADD
            parms.Add(null);

            // INP(25) = (optional) PARENT REQUEST pointer to SDEC APPT REQUEST
            //           file 409.85
            // MRTC Cild Request ONLY) MRTC Parent Request IEN - Child requests are not created here so this is NOT used
            parms.Add(null);

            return parms;
        }

        private List<string> GetPtCSchRequestTypeParams()
        {
            var bFound = false;
            var parms = new List<string>
            {
                _mode == Mode.NewRequest || _mode == Mode.PtCSchTransfer? null : AppointmentRequest.PtCSchRequestIen.ToString(),
                AppointmentRequest.Patient.IEN,
                AppointmentRequest.Accession
            };
            AppointmentRequest.PtCSchComment = TxtPtCSchComment.Text;

            AppointmentRequest.RequestedByText = "PROVIDER";
            AppointmentRequest.DesiredApptDate = PtCSchDatePicker.SelectedDate;

            parms.Add( TxtPtCSchComment.Text );
            if ( ChkFasting.IsChecked == true )
                AppointmentRequest.Fasting = "FASTING";
            else
                AppointmentRequest.Fasting = "";
            parms.Add( AppointmentRequest.Fasting );

            //PtCSch Appt Types....
            if ((AppointmentRequest.SelectedPtCSchAppointmentType != null) && (!String.IsNullOrEmpty(AppointmentRequest.SelectedPtCSchAppointmentType.Ien)))
                parms.Add(AppointmentRequest.SelectedPtCSchAppointmentType.Ien);
            else
                parms.Add(null);

            //var recType = AppointmentRequest.PtCSchApptTypes.FirstOrDefault( rt => rt.Name.ToUpper() == CmbPtCSchApptTypes.Text.ToUpper() );
            //if ( recType != null )
            //    parms.Add( recType.Ien );
            //else
            //    parms.Add( null );
            
            //Provider
            var provider = AppointmentRequest.PtCSchProviders.FirstOrDefault( pr => pr.Name.ToUpper() == CmbPtCSchProvider.Text.ToUpper() );
            if ( provider != null )
            {
                AppointmentRequest.Provider = new Provider
                {
                    IEN = provider.IEN,
                    Name = provider.Name
                };
                AppointmentRequest.RequestedByProviderMerge = provider.Name;
                parms.Add( provider.IEN );
            }
            else
                parms.Add( null );

            //Clinic
            foreach ( var item in CmbPtCSchClinic.Items )
            {
                var clinic = (Clinic)item;

                if ( clinic.ClinicName.ToUpper() == CmbPtCSchClinic.Text.ToUpper() )
                {
                    AppointmentRequest.Clinic = new Clinic( _dataAccessService )
                    {
                        ClinicIen = clinic.ClinicIen,
                        ClinicName = clinic.ClinicName,
                        HospitalLocationId = clinic.HospitalLocationId
                    };
                    AppointmentRequest.ClinicServiceSpecialtyMerge = clinic.ClinicName;
                    parms.Add( clinic.ClinicIen );
                    bFound = true;
                    break;
                }
            }
            if ( !bFound )
                parms.Add( null );
            parms.Add( txtAppointmentDuration.Text ); //Appointment Duration..
            if ( AppointmentRequest.PtCSchDate == null )
                parms.Add( null );
            else
            {
                var ptCSchDate = Convert.ToDateTime( AppointmentRequest.PtCSchDate );
                AppointmentRequest.DesiredApptDate = ptCSchDate;
                OriginalDesiredDate = AppointmentRequest.DesiredApptDate;
                parms.Add( ptCSchDate.ToShortDateString() );
            }

            if ( AppointmentRequest.PtCSchDatePerPatient == null )
                parms.Add( null );
            else
            {
                var ptCSchDatePerPatient = Convert.ToDateTime( AppointmentRequest.PtCSchDatePerPatient );
                parms.Add( ptCSchDatePerPatient.ToShortDateString() );
            }
            parms.Add( null );
            parms.Add( Keys.LoggedInUserIen );
            parms.Add( null );
            parms.Add( DateTime.Now.ToShortDateString() );
            return parms;
        }

        private void CloseThisWindow()
        {
            var window = ParentWindow;
            if (window != null)
                window.Close();
            Thread.Sleep(200);
        }

        private Window ParentWindow { get { return Window.GetWindow(this); } }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = false;
            CloseThisWindow();
        }

        private void RequestBy_Change( object sender, SelectionChangedEventArgs e )
        {
            CmbProviders.IsEnabled = ((AppointmentRequest.RequestedBy != null) && (AppointmentRequest.RequestedBy.Value == "1"));

            if ( !CmbProviders.IsEnabled )
            {
                CmbProviders.SelectedIndex = -1;
                CmbProviders.SelectedValue = null;
                CmbProviders.Text = string.Empty;
            }
        }

        private void IsMultipleAppt_Click(object sender, RoutedEventArgs e)
        {
            MultipleExpander.IsExpanded = ChkIsMultipleAppt.IsChecked == true;
            //MultipleApptFields.Visibility = ChkIsMultipleAppt.IsChecked == true ? Visibility.Visible : Visibility.Hidden;
            //NumberOfAppointmentsRequired.IsEnabled = (ChkIsMultipleAppt.IsChecked.HasValue ? ChkIsMultipleAppt.IsChecked.Value : false);
            //IntervalBetweenAppointments.IsEnabled = (ChkIsMultipleAppt.IsChecked.HasValue ? ChkIsMultipleAppt.IsChecked.Value : false);
            //MultipleApptFields.IsExpanded = IsMultipleAppt.IsChecked == true;
        }

        private void RequestManagementDialog_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                Keys.SetRenderingMode(this);

                AppointmentRequest.Patient.OriginatingUserName = Keys.LoggedInUser;
                AppointmentRequest.Patient.OriginatingUser = Keys.LoggedInUser;

                if (!AppointmentRequest.IsPtCSch)
                    AppointmentRequest.ChkSvcRelatedEnabled = (AppointmentRequest.Patient.ServiceConnectionBool && (_mode != Mode.ViewRequest));

                if (AppointmentRequest.IsAppt)
                    CmbProviders.IsDropDownOpen = false;

                if (AppointmentRequest.MultipleAppointmentsRequired)
                {
                    MultipleExpander.IsExpanded = true;
                    MultipleApptFields.Visibility = Visibility.Visible;
                    NumberOfAppointmentsRequired.Visibility = Visibility.Visible;
                }

                AppointmentRequest.OriginatingUser = Keys.LoggedInUser;
                TxtOriginatingUser.Text = AppointmentRequest.OriginatingUser;
                /********************************/
                if (_mode != Mode.NewRequest)
                {
                    var searchString = string.Empty;
                    AppointmentRequest.Patient.ServiceRelated = AppointmentRequest.Patient.SCVisitBool;
                    SetAppointmentType();
                    NameValue selectedClinicService = null;

                    if (!string.IsNullOrEmpty(AppointmentRequest.ClinName) && !string.IsNullOrEmpty(AppointmentRequest.ClinicIen))
                    {
                        RbService.IsChecked = false;
                        RbClinic.IsChecked = true;
                        selectedClinicService = new NameValue(AppointmentRequest.ClinName, AppointmentRequest.ClinicIen);
                    }
                    else if (!string.IsNullOrEmpty(AppointmentRequest.ClinicStopIen) && !string.IsNullOrEmpty(AppointmentRequest.ClinicStopName))
                    {
                        RbService.IsChecked = true;
                        RbClinic.IsChecked = false;
                        selectedClinicService = new NameValue(AppointmentRequest.ClinicStopName, AppointmentRequest.ClinicStopIen);
                    }
                    else if (!string.IsNullOrEmpty(AppointmentRequest.SpecialtyIen) && !string.IsNullOrEmpty(AppointmentRequest.Specialty))
                    {
                        RbService.IsChecked = true;
                        RbClinic.IsChecked = false;
                        selectedClinicService = new NameValue(AppointmentRequest.Specialty, AppointmentRequest.SpecialtyIen);
                    }

                    if (_mode == Mode.EwlTransfer)
                        if (RbClinic.IsChecked.HasValue && RbClinic.IsChecked.Value && (selectedClinicService != null))
                            if (string.IsNullOrEmpty(_dataAccessService.GetEwlClinicIen(selectedClinicService.Name)))
                                selectedClinicService = null;

                    if (selectedClinicService != null)
                    {
                        if (RbService.IsChecked.Value && (_mode == Mode.EwlTransfer))
                        {
                            ServiceSpecialty ss = AppointmentRequest.ServiceSpecialties.FirstOrDefault(i => (i.Name.ToUpper() == selectedClinicService.Name.ToUpper()));

                            if (ss == null)
                                selectedClinicService = null;
                            else
                                selectedClinicService.Value = ss.Ien;
                        }

                        if (selectedClinicService != null)
                        {
                            CmbServiceClinic.Items.Add(selectedClinicService);
                            CmbServiceClinic.SelectedItem = selectedClinicService;
                            CmbServiceClinic.Text = selectedClinicService.Name;
                        }
                    }

                    CmbRequestedBy.Text = AppointmentRequest.RequestedByText;
                    CalDesiredApptDate.IsEnabled = false;
                    CmbRequestedBy.IsEnabled = false;
                    CmbServiceClinic.IsEnabled = true;
                    GrpSpecialNeeds.IsEnabled = false;

                    if (AppointmentRequest.IsPtCSch)
                    {
                        PtCSchClinicSearchChanged(AppointmentRequest.ClinName);
                        PtCSchProvidersSearchChanged(AppointmentRequest.Patient.Provider.Name);
                    }
                    else
                    { 
                        if (!string.IsNullOrEmpty(searchString))
                            ServiceClinicSearchChanged(searchString);

                        if (AppointmentRequest.Patient.Provider != null)
                            if (!string.IsNullOrEmpty(AppointmentRequest.Patient.Provider.Name))
                                ProvidersSearchChanged(AppointmentRequest.Patient.Provider.Name);
                    }
                }
                /***********************/
                else
                {
                    if ( !string.IsNullOrEmpty( AppointmentRequest.ClinName ) )
                    {
                        ServiceClinicSearchChanged( AppointmentRequest.ClinName );

                        if ( CmbServiceClinic.Items.Count == 1 )
                        {
                            CmbServiceClinic.SelectedItem = CmbServiceClinic.Items[0];
                            CmbServiceClinic.Text = ( (NameValue)CmbServiceClinic.Items[0] ).Name;
                            CmbServiceClinic.IsDropDownOpen = false;
                        }
                    }

                    if (AppointmentRequest.IsVeteran)
                        CmbRequestedBy.Text = "PATIENT";
                }

                if ( _mode == Mode.ViewRequest || _mode == Mode.EditRequest )
                {
                    CmbPriorityGroup.IsEnabled  = false;
                    CmbEthnicities.IsEnabled    = false;
                    CmbRace.IsEnabled           = false;
                    TxtAddress.IsEnabled        = false;
                    TxtCity.IsEnabled           = false;
                    CmbState.IsEnabled          = false;
                    TxtZipCode.IsEnabled        = false;
                    TxtPhoneResidence.IsEnabled = false;
                    TxtPhoneWork.IsEnabled      = false;
                    TxtCountry.IsEnabled        = false;
                    TxtComment.IsEnabled        = false;
                    ChkBadAddress.IsEnabled     = false;
                    CmbBadAddress.IsEnabled     = false;
                    MultipleExpander.IsEnabled  = false;
                    GrpSpecialNeeds.IsEnabled   = false;
                }

                if ( _mode == Mode.ViewRequest )
                {
                    RmGrid.IsEnabled = false;
                }
                else if ( _mode == Mode.EditRequest )
                    AppointmentRequest.MultipleAppointmentsRequired = false;

                if ( AppointmentRequest.IsAppt || AppointmentRequest.IsVeteran )
                {
                    RbClinic.IsEnabled = true;
                    RbService.IsEnabled = true;

                    CmbAppointmentTypes.IsEnabled = true;
                    CmbRequestedBy.IsEnabled = !AppointmentRequest.IsVeteran;
                    CmbRequestedBy.IsEditable = !AppointmentRequest.IsVeteran;
                    CmbProviders.IsEditable = true;
                    CmbEnrolleeStatus.IsEditable = false;
                    MultipleExpander.IsEnabled = true;

                    TxtComment.IsEnabled = true;
                }
                else if ( AppointmentRequest.IsEwl )
                {
                    RbClinic.IsEnabled         = _mode == Mode.NewRequest || _mode == Mode.EwlTransfer;
                    RbService.IsEnabled        = _mode == Mode.NewRequest || _mode == Mode.EwlTransfer;
                    CmbServiceClinic.IsEnabled = _mode == Mode.NewRequest || _mode == Mode.EwlTransfer;

                    CmbAppointmentTypes.IsEnabled = true;
                    CmbRequestedBy.IsEnabled = true;
                    CmbRequestedBy.IsEditable = true;
                    CmbProviders.IsEditable = true;
                    CmbEnrolleeStatus.IsEditable = false;
                    TxtComment.IsEnabled = true;
                }
                else if ( AppointmentRequest.IsPtCSch )
                {
                    PtCSchDatePicker.IsEnabled        = _mode == Mode.NewRequest;
                    PtCSchDatePatientPicker.IsEnabled = _mode == Mode.NewRequest;
                    CmbPtCSchApptTypes.IsEnabled = true;
                    ChkFasting.IsEnabled = true;
                    txtAppointmentDuration.IsEnabled = true;
                    CmbPtCSchClinic.IsEnabled = true;
                    CmbPtCSchProvider.IsEnabled = true;
                    TxtComment.IsEnabled = true;
                }

                // Open the Patient Disabilities Viewer for non-ptCSch Requests only
                if ( !AppointmentRequest.IsPtCSch )
                    _eventAggregator.GetEvent<PatientDisabilitiesEvent>().Publish( new PatientDisabilitiesArgs( Window.GetWindow( this ), AppointmentRequest.Patient ) );

                if ( Keys.IsCallCenterUser )
                {
                    AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.WelcomeToVa;

                    RbService.IsChecked           = true;
                    RbService.IsEnabled           = false;
                    RbClinic.IsChecked            = false;
                    RbClinic.IsEnabled            = false;
                    CmbServiceClinic.Text         = "PRIMARY CARE";
                    CmbServiceClinic.IsEnabled    = false;
                    CmbAppointmentTypes.Text      = REGULAR;
                    CmbAppointmentTypes.IsEnabled = false;
                    CmbRequestedBy.Text           = "PATIENT";
                    CmbRequestedBy.IsEnabled      = false;
                    CmbEnrolleeStatus.Text        = "NEW";
                    CmbEnrolleeStatus.IsEnabled   = false;
                    ChkSvcRelated.Visibility      = Visibility.Hidden;
                    ChkIsMultipleAppt.Visibility  = Visibility.Hidden;
                    MultipleExpander.Visibility   = Visibility.Hidden;
                }

                SetIsMultipleAppointmentEnabled();
                _isLoaded = true;

                if (AppointmentRequest.IsPtCSch)
                {
                    PtCSchDatePicker.Focus();
                    Keyboard.Focus(PtCSchDatePicker);
                }
                else
                {
                    CmbServiceClinic.Focus();
                    Keyboard.Focus(CmbServiceClinic);
                }
            }
            catch ( Exception ex )
            {
                Keys.AlertUserError(this, ex, "Request Management Load Error");
            }
        }

        private void RequestManagementDialog_Unloaded(object sender, RoutedEventArgs e)
        {
            // Close the Patient Disabilities Viewer
            _eventAggregator.GetEvent<PatientDisabilitiesEvent>().Publish(null);
        }

        private void radServiceClinic_Click(object sender, RoutedEventArgs e)
        {
            if (CmbServiceClinic == null)
                return;

            ClearServiceClinicComboItems();
            SetIsMultipleAppointmentEnabled();
        }

        private void ClearServiceClinicComboItems()
        {
            CmbServiceClinic.Items.Clear();
            CmbServiceClinic.SelectedItem = null;
            CmbServiceClinic.Text = string.Empty;
        }

        private void chkBadAddress_Click(object sender, RoutedEventArgs e)
        {
            CmbBadAddress.IsEnabled = ChkBadAddress.IsChecked == true;
            if (ChkBadAddress.IsChecked == false)
                CmbBadAddress.Text = string.Empty;
        }

        private void Veteran_Click(object sender, RoutedEventArgs e)
        {
            if (AppointmentRequest.IsVeteran)
            {
                CmbRequestedBy.Text = "PATIENT";
                AppointmentRequest.MultipleAppointmentsRequired = false;
                AppointmentRequest.NumberOfAppointmentsRequired = 0;
                AppointmentRequest.AppointmentIntervalInDays = 0;
                NumberOfAppointmentsRequired.Text = "";
                IntervalBetweenAppointments.Text = "";
            }

            SetIsMultipleAppointmentEnabled();
        }

        private AppointmentType _lastApptType = null;
        private bool _svcRelatedChanged = false;

        private string ClinicDefaultApptType
        {
            get
            {
                var selectedClinic = CmbServiceClinic.SelectedValue == null ? string.Empty : CmbServiceClinic.SelectedValue.ToString();

                if (!string.IsNullOrEmpty(selectedClinic))
                {
                    AppointmentType appointmentType = 
                        AppointmentRequest.IsPtCSch ?
                        _dataAccessService.GetDefaultClinicAppointmentType(null, selectedClinic, "1") :
                        _dataAccessService.GetDefaultClinicAppointmentType(null, selectedClinic);

                    return (appointmentType == null) ? string.Empty : appointmentType.Name;
                }

                return string.Empty;
            }
        }

        private void CmbAppointmentTypes_Changed(object sender, SelectionChangedEventArgs e)
        {
            AppointmentType selectedApptType = null;

            if (e.AddedItems.Count > 0)
                selectedApptType = (AppointmentType)e.AddedItems[0];

            // The selected type is SERVICE CONNECTED
            if (selectedApptType.Name == SERVICE_CONNECTED)
            {
                // The Svc Related checkbox is checked
                if (ChkSvcRelated.IsChecked.HasValue && ChkSvcRelated.IsChecked.Value && !_svcRelatedChanged)
                    // Clear the last selected
                    _lastApptType = null;

                // we are done here
                return;
            }

            string clinicDefaultApptType = ClinicDefaultApptType;

            // The selected Clinic Default Appointment type is the same as the selected appointment type
            if (!string.IsNullOrEmpty(clinicDefaultApptType) && (selectedApptType.Name == clinicDefaultApptType))
            {
                // The Svc Related checkbox is checked
                if ((!ChkSvcRelated.IsChecked.HasValue || !ChkSvcRelated.IsChecked.Value) && !_svcRelatedChanged)
                    // Clear the last selected
                    _lastApptType = null;

                // we are done here
                return;
            }

            // Save the selected appointment type
            _lastApptType = selectedApptType;
        }

        public void SetAppointmentType()
        {
            string apptType = "";
            string clinicDefaultApptType = ClinicDefaultApptType;

            if ( _isTransferToEwl && !_isLoaded && !string.IsNullOrEmpty( AppointmentRequest.AppointmentType ) )
                apptType = AppointmentRequest.AppointmentType;
            else
            {
                // set the appointment type to the clinic default
                apptType = clinicDefaultApptType;

                // if we are editing, or ewl xfer
                if ((_mode == Mode.EditRequest) || (_mode == Mode.EwlTransfer) || (_mode == Mode.ViewRequest))
                    // if we have an old appointment request and that requests's appointment type is NOT SERVICE CONNECTED
                    if ( !string.IsNullOrEmpty( AppointmentRequest.AppointmentType ) && (AppointmentRequest.AppointmentType != SERVICE_CONNECTED))
                        // use the old request appointment type
                        apptType = AppointmentRequest.AppointmentType;
            }

            // The patient is SERVICE CONNECTED
            if (AppointmentRequest.Patient.ServiceConnection == "YES")
            {
                // The Svc Related checkbox is checked
                if (ChkSvcRelated.IsChecked.HasValue && ChkSvcRelated.IsChecked.Value)
                {
                    // The patient Service Connected % is greater than 50%
                    //if (AppointmentRequest.Patient.ServiceConnectionPercent >= 50 && AppointmentTypes.Any(at => at.Name == SERVICE_CONNECTED))
                    if ( AppointmentTypes.Any(at => at.Name == SERVICE_CONNECTED))
                        // Set the appointment Type tp SERVICE CONNECTED
                        apptType = SERVICE_CONNECTED;

                    // The appointment Type is not set, or is REGULAR and SERVICE CONNECTED is part of the patient appointment types list
                    if ((string.IsNullOrEmpty(apptType) || apptType == REGULAR) && AppointmentTypes.Any(at => at.Name == SERVICE_CONNECTED))
                        // Set the appointment Type tp SERVICE CONNECTED
                        apptType = SERVICE_CONNECTED;
                }
                else
                {
                    // A non SERVICE CONNECTED or Clinic Default appointment type was previosly selected
                    if (_lastApptType != null)
                        // Use the selected
                        apptType = _lastApptType.Name;
                }
            }

            if (string.IsNullOrEmpty(apptType))
                // The Svc Related checkbox is checked
                if (ChkSvcRelated.IsChecked.HasValue && ChkSvcRelated.IsChecked.Value)
                    apptType = SERVICE_CONNECTED;
                else
                    apptType = REGULAR;

            var ind = 0;
            foreach (var appointmentType in AppointmentTypes)
            {
                if (appointmentType.Name == apptType)
                {
                    CmbAppointmentTypes.SelectedIndex = ind;
                    break;
                }
                ind++;
            }

            CmbAppointmentTypes.Text = apptType;
            _svcRelatedChanged = false;
        }

        private void ChkSvcRelated_Checked(object sender, RoutedEventArgs e)
        {
            _svcRelatedChanged = true;
            //_lastApptType = null;
            SetAppointmentType();
        }

        private void ChkSvcRelated_Unchecked(object sender, RoutedEventArgs e)
        {
            _svcRelatedChanged = true;
            SetAppointmentType();
        }

        #region CmbPtCSchClinic members

        private readonly System.Timers.Timer _ptCSchClinicSearchTimer;
        private string _ptCSchClinicSearchString;

        void ptCSchSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _ptCSchClinicSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => PtCSchClinicSearchChanged(_ptCSchClinicSearchString)));
        }

        private void CmbPtCSchClinic_OnLostFocus(object sender, RoutedEventArgs e)
        {
            SetComboBoxDropDownOpen(CmbPtCSchClinic, false);
        }

        private void CmbPtCSchClinic_KeyUp(object sender, KeyEventArgs e)
        {
            _ptCSchClinicSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _ptCSchClinicSearchTimer.Start();
        }

        private void CmbPtCSchClinic_TextChanged(object sender, RoutedEventArgs e)
        {
            var cbx = (ComboBox)sender;
            _ptCSchClinicSearchString = cbx.Text;
        }

        private void CmbPtCSchClinic_SelectionChanged( object sender, SelectionChangedEventArgs e )
        {
            var cbx = (ComboBox)sender;
            if ( cbx.SelectedValue != null )
                CmbPtCSchClinic.Text = cbx.SelectedValue.ToString();
        }

        private void PtCSchClinicSearchChanged(string newSearchString)
        {
            if (string.IsNullOrEmpty(newSearchString))
                return;

            if (newSearchString.Length < Keys.ClinSetSearchCharMin)
                return;

            CmbPtCSchClinic.Items.Clear();
            var list = _dataAccessService.SearchPtCSchClinics(newSearchString, "1");

            if (list.Count > 0)
            {
                if ((list.Count == Keys.ClinicResourceSearchMaxRecords) && list[list.Count - 1].MoreRecords)
                {
                    Keys.AlertUser(ParentWindow, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "PtCSch Clinic Search Criteria");
                    CmbPtCSchClinic.Focus();
                    Keyboard.Focus(CmbPtCSchClinic);
                }
            }

            foreach (var item in list)
                CmbPtCSchClinic.Items.Add(item);

            System.Windows.Forms.Application.DoEvents();
            Thread.Sleep( 75 );
            DeselectComboBoxText( CmbPtCSchClinic );

            if ( CmbPtCSchClinic.Items.Count == 1 )
            {
                CmbPtCSchClinic.SelectedItem = CmbPtCSchClinic.Items[0];
                CmbPtCSchClinic.Text = ( (Clinic)CmbPtCSchClinic.Items[0] ).ClinicName;
                CmbPtCSchClinic.IsDropDownOpen = false;
            }
            else
            {
                SetComboBoxDropDownOpen( CmbPtCSchClinic, CmbPtCSchClinic.Items.Count > 1 );
            }
        }

        #endregion CmbPtCSchClinic members

        #region CmbServiceClinic search members

        private readonly System.Timers.Timer _serviceClinicSearchTimer;
        private string _serviceClinicSearchString;

        void resourceSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _serviceClinicSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => ServiceClinicSearchChanged(_serviceClinicSearchString)));
        }

        private void CmbServiceClinic_OnLostFocus(object sender, RoutedEventArgs e)
        {
            SetComboBoxDropDownOpen(CmbServiceClinic, false);
        }

        private void CmbServiceClinic_KeyUp(object sender, KeyEventArgs e)
        {
            _serviceClinicSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _serviceClinicSearchTimer.Start();
        }

        private void CmbServiceClinic_TextChanged(object sender, RoutedEventArgs e)
        {
            var cbx = (ComboBox)sender;
            _serviceClinicSearchString = cbx.Text;
        }

        private void CmbServiceClinic_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            // Clear the last Selecte Appointment Type
            _lastApptType = null;
            var selectedItem = (NameValue)e.AddedItems[0];
            SetAppointmentType();
            bool? isNew = null;
            AppointmentRequest.EnrolleeStatus = "";
            CmbEnrolleeStatus.Text = "";

            if (RbClinic.IsChecked == true)
                isNew = _dataAccessService.GetClinicStopInfo(AppointmentRequest.Patient.IEN, selectedItem.Value);
            else if (RbService.IsChecked == true)
                isNew = _dataAccessService.GetServiceStopInfo(AppointmentRequest.Patient.IEN, AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl ? selectedItem.Name : selectedItem.Value);

            if (isNew.HasValue)
            {
                if (isNew.Value)
                {
                    AppointmentRequest.EnrolleeStatus = "NEW";
                    CmbEnrolleeStatus.Text = "NEW";
                }
                else
                {
                    AppointmentRequest.EnrolleeStatus = "ESTABLISHED";
                    CmbEnrolleeStatus.Text = "ESTABLISHED";
                }
            }
            if (string.IsNullOrEmpty(CmbAppointmentTypes.Text) && CmbAppointmentTypes.IsEnabled == false)
                CmbAppointmentTypes.IsEnabled = true;
        }

        private void ServiceClinicSearchChanged( string newSearchString )
        {
            if ( string.IsNullOrEmpty( newSearchString ) )
                return;

            if ( newSearchString.Length < Keys.ClinStopSearchCharMin )
                return;

            CmbServiceClinic.Items.Clear();

            if ( RbService.IsChecked == true )
            {
                if ( AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
                {
                    foreach ( var item in AppointmentRequest.ServiceSpecialties )
                    {
                        if ( item.Name.ToUpper().StartsWith( newSearchString.ToUpper() ) )
                            CmbServiceClinic.Items.Add( new NameValue( item.Name, item.Ien ) );
                    }
                }
                else if ( Keys.Methods.IsApptOrVet( AppointmentRequest.AppointmentRequestType ) )
                {
                    foreach ( var item in AppointmentRequest.GetApptServiceSpecialties( newSearchString ) )
                    {
                        if ( item.Name.ToUpper().StartsWith( newSearchString.ToUpper() ) )
                            CmbServiceClinic.Items.Add( new NameValue( item.Name, item.Ien ) );
                    }
                }

                System.Windows.Forms.Application.DoEvents();
                Thread.Sleep( 100 );
            }

            if ( RbClinic.IsChecked == true )
            {
                if ( AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
                {
                    IList<NameValue> list = _dataAccessService.GetEwlClinics(newSearchString);

                    if (list.Count > 0)
                    {
                        if ((list.Count == Keys.ClinicResourceSearchMaxRecords) && list[list.Count - 1].MoreRecords)
                        {
                            Keys.AlertUser(ParentWindow, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "EWL Clinic Search Criteria");
                            CmbServiceClinic.Focus();
                            Keyboard.Focus(CmbServiceClinic);
                        }
                    }

                    foreach (var item in list)
                        CmbServiceClinic.Items.Add(item);
                }
                else if ( Keys.Methods.IsApptOrVet( AppointmentRequest.AppointmentRequestType ) )
                {
                    var list = _dataAccessService.SearchPtCSchClinics( newSearchString );

                    if (list.Count > 0)
                    {
                        if ((list.Count == Keys.ClinicResourceSearchMaxRecords) && list[list.Count - 1].MoreRecords)
                        {
                            Keys.AlertUser(ParentWindow, $"The current search criteria returns more than {Keys.ClinicResourceSearchMaxRecords} records.\nPlease refine your search and try again.", "Clinic Search Criteria");
                            CmbServiceClinic.Focus();
                            Keyboard.Focus(CmbServiceClinic);
                        }
                    }

                    foreach ( var item in list )
                    {
                        CmbServiceClinic.Items.Add( new NameValue( item.ClinicName, item.ClinicIen ) );
                    }
                }
                else if ( AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch )
                    return;
            }
            System.Windows.Forms.Application.DoEvents();
            Thread.Sleep( 75 );
            DeselectComboBoxText(CmbServiceClinic);

            //if ((CmbServiceClinic.Items.Count == Keys.ClinicResourceSearchMaxRecords) && !string.IsNullOrEmpty(CmbServiceClinic.Items[CmbServiceClinic.Items.Count - 1].LastSub))
            //{
            //    if (isNameSearch)
            //    {
            //        View.AlertUser("The current search criteria returns too many records.\nPlease refine your search and try again.", "Patient Search Criteria");
            //        View.FocusPatientSearch();
            //        return;
            //    }
            //}

            if (CmbServiceClinic.Items.Count == 1)
            {
                CmbServiceClinic.SelectedItem = CmbServiceClinic.Items[0];
                CmbServiceClinic.Text = ((NameValue)CmbServiceClinic.Items[0]).Name;
                CmbServiceClinic.IsDropDownOpen = false;
            }
            else
            {
                CmbServiceClinic.IsDropDownOpen = CmbServiceClinic.Items.Count > 1;
            }
        }

        #endregion CmbServiceClinic search members

        #region CmbProviders members

        private readonly System.Timers.Timer _providersSearchTimer;
        private string _providersSearchString;

        void providersSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _providersSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => ProvidersSearchChanged(_providersSearchString)));
        }

        private void CmbProviders_KeyUp(object sender, KeyEventArgs e)
        {
            _providersSearchTimer.Stop();
            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _providersSearchTimer.Start();
        }

        private void Provider_LostFocus(object sender, RoutedEventArgs e)
        {
            SetComboBoxDropDownOpen(CmbProviders, false);
        }

        private void CmbProviders_TextChanged(object sender, RoutedEventArgs e)
        {
            var cbx = (ComboBox)sender;
            _providersSearchString = cbx.Text;
        }

        private void ProvidersSearchChanged(string newSearchString)
        {
            if (string.IsNullOrEmpty(newSearchString))
                return;

            if (newSearchString.Length < Keys.PatientSearchCharMin)
                return;

            //AppointmentRequest.Providers.Clear();
            CmbProviders.Items.Clear();
            var list = _dataAccessService.GetProviders(newSearchString);

            foreach (var item in list)
                //AppointmentRequest.Providers.Add(item);
                CmbProviders.Items.Add(item);
            
            System.Windows.Forms.Application.DoEvents();
            Thread.Sleep(75);
            DeselectComboBoxText(CmbProviders);

            if (CmbProviders.Items.Count == 1)
            {
                CmbProviders.SelectedItem = CmbProviders.Items[0];
                CmbProviders.Text = ((Provider)CmbProviders.Items[0]).Name;
                CmbProviders.IsDropDownOpen = false;
            }
            else
            {
                SetComboBoxDropDownOpen(CmbProviders, CmbProviders.Items.Count > 1);
            }
        }

        #endregion CmbProviders members

        #region CmbPtCSchProvider

        private readonly System.Timers.Timer _cmbPtCSchProvidersSearchTimer;
        private string _cmbPtCSchProvidersSearchString;

        void cmbPtCSchProvidersSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _cmbPtCSchProvidersSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => PtCSchProvidersSearchChanged(_cmbPtCSchProvidersSearchString)));
        }

        private void CmbPtCSchProvider_OnLostFocus(object sender, RoutedEventArgs e)
        {
            SetComboBoxDropDownOpen(CmbPtCSchProvider, false);
        }

        private void CmbPtCSchProvider_KeyUp(object sender, KeyEventArgs e)
        {
            _serviceClinicSearchTimer.Stop();
           
            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _cmbPtCSchProvidersSearchTimer.Start();
        }

        private void CmbPtCSchProvider_TextChanged(object sender, RoutedEventArgs e)
        {
            var cbx = (ComboBox)sender;
            _cmbPtCSchProvidersSearchString = cbx.Text;
        }

        private void CmbPtCSchProvider_SelectionChanged( object sender, SelectionChangedEventArgs e )
        {
            var cbx = (ComboBox)sender;
            if ( cbx.SelectedValue != null )
                CmbProviders.Text = cbx.SelectedValue.ToString();
        }

        private void PtCSchProvidersSearchChanged( string newSearchString )
        {
            if ( string.IsNullOrEmpty( newSearchString ) )
                return;

            if ( newSearchString.Length < Keys.ProviderSearchCharMin )
                return;

            AppointmentRequest.PtCSchProviders.Clear();
            var list = _dataAccessService.GetPtCSchProviders( newSearchString );

            foreach ( var item in list )
                AppointmentRequest.PtCSchProviders.Add( item );

            System.Windows.Forms.Application.DoEvents();
            Thread.Sleep( 75 );
            DeselectComboBoxText( CmbPtCSchProvider );

            if ( CmbPtCSchProvider.Items.Count == 1 )
            {
                CmbPtCSchProvider.SelectedItem = CmbPtCSchProvider.Items[0];
                CmbPtCSchProvider.Text = ( (Provider)CmbPtCSchProvider.Items[0] ).Name;
                CmbPtCSchProvider.IsDropDownOpen = false;
            }
            else
            {
                SetComboBoxDropDownOpen( CmbPtCSchProvider, AppointmentRequest.PtCSchProviders.Count > 1 );
            }
        }

        #endregion CmbPtCSchProvider

        public void SetComboBoxDropDownOpen(ComboBox cbx, bool openDropDown)
        {
            cbx.IsDropDownOpen = openDropDown;
        }

        private void DeselectComboBoxText(ComboBox cbx)
        {
            //  Calling this method insures the CombBox text is not selected and the cursor is positioned for additional typing (ComboBox ONLY)
            var tbx = (TextBox)cbx.Template.FindName("PART_EditableTextBox", cbx);
            if ( tbx != null )
            {
                tbx.SelectionStart = tbx.Text.Length == 0 ? 0 : tbx.Text.Length;
                tbx.SelectionLength = 0;
            }
        }

        private void CalDesiredApptDate_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
        }

        private void RadDatePicker_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter || e.Key == Key.Return)
            {
                CalDesiredApptDate.IsDropDownOpen = true;
            }
        }

        private void RadDatePicker_GotFocus(object sender, RoutedEventArgs e)
        {
            BtnOk.IsDefault = false;
        }

        private void RadDatePicker_LostFocus(object sender, RoutedEventArgs e)
        {
            BtnOk.IsDefault = true;
        }

        public enum Mode
        {
            NewRequest,
            EditRequest,
            ViewRequest,
            EwlTransfer,
            PtCSchTransfer
        }

        private void SetIsMultipleAppointmentEnabled()
        {
            ChkIsMultipleAppt.IsEnabled = (
                    _mode == Mode.NewRequest
                &&
                    !AppointmentRequest.IsVeteran
                &&
                    !AppointmentRequest.IsEwl
                &&
                    !_isTransferToEwl
                &&
                    RbClinic.IsChecked.HasValue ? RbClinic.IsChecked.Value : false
            );

            if (!ChkIsMultipleAppt.IsEnabled)
                ChkIsMultipleAppt.IsChecked = false;
        }
    }
}
