using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.IO;
using System.Windows.Controls;
using ClinSchd.Modules.DataAccess.Services;
using ClinSchd.Modules.Navigation.Views;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Events;
using System.Windows;
using System.Windows.Input;
using System.Linq;
using System.Reflection;
using System.Runtime.InteropServices;
using Telerik.Windows.Controls;
using System.Text.RegularExpressions;
using System.Windows.Media;
using ClinSchd.Modules.Ribbon.ViewModels;
using ClinSchd.Modules.Ribbon.Views;
using ClinSchd.Infrastructure.Factories;

namespace ClinSchd.Modules.Ribbon.Schedule
{
    public class SchedulePresentationModel : AsyncValidatableModel, ISchedulePresentationModel, IAppointmentRequestViewOperations
    {
        public readonly IEventAggregator EventAggregator;
        public readonly IDataAccessService DataAccessService;
        private bool _isViewAppointmentEnabled;
        private bool _isEditAppointmentEnabled;
        private bool _isNewAppointmentEnabled;
        private bool _isExistAppointmentEnabled;
        private bool _isNoShowAppointmentEnabled;
        private bool _isCheckedInAppointmentEnabled;
        private bool _isCheckedOutAppointmentEnabled;
        private bool _isCancelledAppointmentEnabled;
        private bool _isNotNoShowAppointmentEnabled;
        private bool _isNotCheckedInAppointmentEnabled;
        private bool _isNotCheckedOutAppointmentEnabled;
        private bool _isNotCancelledAppointmentEnabled;
        private bool _isPasteAvailability;
        private bool _isManagerUser;
        private IList<Patient> _patientList;
        private string _paneTitle = "";
        private int _timeScaleIndex = 2;
        private List<AppointmentRequest> _unfilteredAdmittedPatients;
        private ObservableCollection<AppointmentRequest> _admittedPatients;

        public ObservableCollection<FuturePatientAppointment> FuturePatientAppointments { get; set; }

        private OVIDDataTableRepository _adt;
        public bool AppointmetRequestTypeViewNoneChecked;

        public string PatientLastSub { get; set; }

        public ObservableCollection<AppointmentRequest> AdmittedPatients
        {
            get { return _admittedPatients; }
            private set
            {
                _unfilteredAdmittedPatients = value.ToList();
                _admittedPatients = value;
                NotifyPropertyChanged("AdmittedPatients");
            }
        }

        public bool RowSelected
        {
            get { return SelectedAppointmentRequest != null; }
        }

        public List<AppointmentRequest> UnfilteredAdmittedPatients
        {
            get { return _unfilteredAdmittedPatients; }
        }

        public DataTable WaitList { get; set; }

        public bool MultipleAppointmentsRequired { get; set; }

        public List<string> WaitTimeValues { get; } = new List<string>(new[]
        {
            Keys.RmGridColumns.WaitTimeAll, Keys.RmGridColumns.WaitTimeLessThan30, Keys.RmGridColumns.WaitTimeBetween30And60, Keys.RmGridColumns.WaitTimeBetween60And90, Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90
        });

        private string _waitTimeValue = Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90;

        public string WaitTimeValue
        {
            get { return _waitTimeValue; }
            set
            {
                if (value != _waitTimeValue)
                {
                    _waitTimeValue = value;
                    /* if (Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded == false && _waitTimeValue != Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90)
                        {
                            MessageBox.Show("Your requet cannot be completed right now please try again.");
                            return;
                        }*/
                    Keys.BooleanFlags.WaitTimeValueChanged = true;
                    FilterAdmittedPatientsByWaitTime(_waitTimeValue);
                    if (!Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
                    {
                        DataAccessService.LoadWaitList((s, args) =>
                       {
                           lock (args)
                           {
                               Keys.AppointmentRequestWaitListTables.EwlWaitListTemp = args.Result as DataTable;
                           }
                       });
                    }
                }
            }
        }

        private FuturePatientAppointment _selectedPendingAppointment;

        public FuturePatientAppointment SelectedPendingAppointment
        {
            get { return _selectedPendingAppointment; }
            set
            {
                if (_selectedPendingAppointment != value)
                {
                    _selectedPendingAppointment = value;

                    if (PendingAppointmentsRightClick)
                    {
                        PendingAppointmentsRightClick = false;
                        return;
                    }

                    if (_selectedPendingAppointment != null && !_selectedPendingAppointment.IsCancelled)
                    {
                        SelectedAppointmentRequest = null;
                        Keys.SelectedAppointmentRequest = null;
                        Keys.RequestKeys.CurrentSelectedDesiredDate = _selectedPendingAppointment.AppointmentDate;
                        Keys.RequestKeys.IsPendingApptSelected = true;
                        var resourceId = DataAccessService.GetResourceIdByClinicIdAndName(_selectedPendingAppointment.ClinicIen, _selectedPendingAppointment.Clinic, true);
                        SchdResource schedRes = null;

                        if (!string.IsNullOrEmpty(resourceId))
                            schedRes = DataAccessService.GetSdecResource(resourceId);

                        if (schedRes == null)
                            View.AlertUser(
                                "The Clinic information associated with the selected appointment is not present or refers to a Clinic that is unavailable.\n" +
                                "The Clinic Schedule for the selected appointment cannot be displayed.",
                                "Pending Appointment Selection"
                            );
                        else
                        {
                            // TO DO: Convert.ToDateTime does not work, incorrect time format, is 'DEC 02, 2015@13:00'
                            DateTime startTime;

                            if (!Keys.Methods.TryParseDateTime(_selectedPendingAppointment.AppointmentDate, out startTime))
                            {
                                View.AlertUser(string.Format("'{0}' is an incorrect date time format.", _selectedPendingAppointment.AppointmentDate), "Pending Appointment Selection");
                                return;
                            }

                            var endTime = startTime.Date.AddDays(1).AddMinutes(-1);
                            // This gets Appointments, AppSlots and Availability for the dates in question
                            var appointments = schedRes.GetAppointments(startTime.Date, endTime);
                            var found = false;

                            // See if the selected appointment is in the the appointment schedule
                            foreach (var appt in appointments.Where(a => a.RESOURCENAME == schedRes.RESOURCE_NAME).ToList())
                            {
                                var apptStartTime = Convert.ToDateTime(appt.START_TIME);
                                found = apptStartTime == startTime;

                                if (found)
                                    break;
                            }

                            if (found)
                                ExecuteSchedulerDisplayCommand(schedRes);
                            else
                            {
                                View.AlertUser(
                                    "The information associated with the selected appointment is not present or refers " +
                                    "to an appointment created prior to the VistA Scheduling installation.  " +
                                    "The Clinic Schedule for the selected appointment cannot be displayed.",
                                    "Pending Appointment Selection"
                                );
                            }
                        }

                        // Send the selected pending appointment to the clinic schedule
                        EventAggregator.GetEvent<PendingAppointmentSelectedEvent>().Publish(_selectedPendingAppointment);
                        EventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(
                                           new SchedulerViewDateEventArgs("PendingAppointments", _selectedPendingAppointment.AppointmentDateTime));                                                
                    }
                }
            }
        }

        public SchedulePresentationModel(
            IScheduleView view,
            IRibbonService ribbonService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator,
            ScheduleView view2)
        {
            //            Keys.RMGridBusyIndicator = delegate(string s) { this.BusyIndicatorEventHandler(s); };

            View = view;
            View2 = view2;
            View.Model = this;
            PatientQueueView = new PatientQueueView(this);// patientQueueView;
            //PatientQueueView.Model    = this;
            EventAggregator = eventAggregator;
            DataAccessService = dataAccessService;
            AdmittedPatients = new ObservableCollection<AppointmentRequest>();
            FuturePatientAppointments = new ObservableCollection<FuturePatientAppointment>();
            Resource = new SchdResource(dataAccessService);
            ResourceGroup = new SchdResourceGroup(DataAccessService);
            UserPrefsAreSet = false;

            ExpandedEntryCommand = new DelegateCommand<string>(ExecuteExpandedEntryCommand);
            NewRequestCommand = new DelegateCommand<string>(ExecuteNewRequestCommand, CanExecuteNewRequestCommand);
            NewAppointmentCommand = new DelegateCommand<string>(ExecuteNewAppointmentCommand, CanExecuteNewAppointmentCommand);
            WalkInAppointmentCommand = new DelegateCommand<string>(ExecuteWalkInAppointmentCommand, CanExecuteWalkInAppointmentCommand);
            ViewAppointmentCommand = new DelegateCommand<string>(ExecuteViewAppointmentCommand, CanExecuteViewAppointmentCommand);
            EditAppointmentCommand = new DelegateCommand<string>(ExecuteEditAppointmentCommand, CanExecuteEditAppointmentCommand);
            SelectPatientCommand = new DelegateCommand<string>(ExecuteSelectPatientCommand, CanExecuteSelectPatientCommand);
            //ChangeServerCommand           = new DelegateCommand<string> (ExecuteChangeServerCommand, CanExecuteChangeServerCommand);
            ChangeUserCommand = new DelegateCommand<string>(ExecuteChangeUserCommand, CanExecuteChangeUserCommand);
            ChangeDivisionCommand = new DelegateCommand<string>(ExecuteChangeDivisionCommand, CanExecuteChangeDivisionCommand);
            MarkAsNoShowCommand = new DelegateCommand<string>(ExecuteMarkAsNoShowCommand, CanExecuteMarkAsNoShowCommand);
            UndoANoShowCommand = new DelegateCommand<string>(ExecuteUndoANoShowCommand, CanExecuteUndoANoShowCommand);
            CheckInPatientCommand = new DelegateCommand<string>(ExecuteCheckInPatientCommand, CanExecuteCheckInPatientCommand);
            UndoCheckInPatientCommand = new DelegateCommand<string>(ExecuteUndoCheckInPatientCommand, CanExecuteUndoCheckInPatientCommand);
            CheckOutPatientCommand = new DelegateCommand<string>(ExecuteCheckOutPatientCommand, CanExecuteCheckOutPatientCommand);
            UndoCheckOutPatientCommand = new DelegateCommand<string>(ExecuteUndoCheckOutPatientCommand, CanExecuteUndoCheckOutPatientCommand);
            CancelAppointmentCommand = new DelegateCommand<string>(ExecuteCancelAppointmentCommand, CanExecuteCancelAppointmentCommand);
            UndoCancelAppointmentCommand = new DelegateCommand<string>(ExecuteUndoCancelAppointmentCommand, CanExecuteUndoCancelAppointmentCommand);
            FindAppointmentCommand = new DelegateCommand<string>(ExecuteFindAppointmentCommand, CanExecuteFindAppointmentCommand);
            SelectResourceCommand = new DelegateCommand<string>(ExecuteSelectResourceCommand, CanExecuteSelectResourceCommand);
            AvailabilityResourceCommand = new DelegateCommand<string>(ExecuteAvailabilityResourceCommand, CanExecuteAvailabilityResourceCommand);
            FileVersionsCommand = new DelegateCommand<string>(ExecuteFileVersionsCommand, CanExecuteFileVersionsCommand);
            ViewDayRangeCommand = new DelegateCommand<string>(ExecuteViewDayRangeCommand, CanExecuteViewDayRangeCommand);
            ViewTimeScaleCommand = new DelegateCommand<string>(ExecuteViewTimeScaleCommand, CanExecuteViewTimeScaleCommand);
            SchedulingManagementCommand = new DelegateCommand<string>(ExecuteSchedulingManagementCommand, CanExecuteSchedulingManagementCommand);
            CopyClipboardCommand = new DelegateCommand<string>(ExecuteHelpCommand, CanExecuteHelpCommand);
            DisplayWalkInCommand = new DelegateCommand<string>(ExecuteCopyClipboardCommand, CanExecuteCopyClipboardCommand);
            ChangeViewModeCommand = new DelegateCommand<string>(ExecuteChangeViewModeCommand, CanExecuteChangeViewModeCommand);
            CopyAppointmentCommand = new DelegateCommand<string>(ExecuteCopyAppointmentCommand, CanExecuteCopyAppointmentCommand);
            PasteAppointmentCommand = new DelegateCommand<string>(ExecutePasteAppointmentCommand, CanExecutePasteAppointmentCommand);
            HelpCommand = new DelegateCommand<string>(ExecuteHelpCommand, CanExecuteHelpCommand);
            ExitCommand = new DelegateCommand<string>(ExecuteExitCommand, CanExecuteExitCommand);
            PrintCurrentSchedulerCommand = new DelegateCommand<string>(ExecutePrintCurrentSchedulerCommand, CanExecutePrintCurrentSchedulerCommand);
            ReportPatientHSMergeCommand = new DelegateCommand<string>(ExecuteReportPatientHSMergeCommand, CanExecuteReportPatientCommand);
            ReportPatientHistoryCommand = new DelegateCommand<string>(ExecuteReportPatientHistoryCommand, CanExecuteReportPatientCommand);
            ReportPatientAppointmentCommand = new DelegateCommand<string>(ExecuteReportPatientAppointmentCommand, CanExecuteReportPatientCommand);
            ReportPatientLetterCommand = new DelegateCommand<string>(ExecuteReportPatientLetterCommand, CanExecuteReportLetterCommand);
            ReportReminderLetterCommand = new DelegateCommand<string>(ExecuteReportReminderLetterCommand, CanExecuteReportLetterCommand);
            ReportRebookLetterCommand = new DelegateCommand<string>(ExecuteReportRebookLetterCommand, CanExecuteReportLetterCommand);
            ReportCancellationLetterCommand = new DelegateCommand<string>(ExecuteReportCancellationLetterCommand, CanExecuteReportLetterCommand);
            ReportClinicSchedulerCommand = new DelegateCommand<string>(ExecuteReportClinicSchedulerCommand, CanExecuteReportLetterCommand);
            ReportClinicWalkInCommand = new DelegateCommand<string>(ExecuteReportClinicWalkInCommand, CanExecuteReportLetterCommand);
            LaunchBrowserCommand = new DelegateCommand<string>(ExecuteLaunchBrowserCommand, CanLaunchBrowserCommand);
            UpdateUserPrefIconCommand = new DelegateCommand<string>(ExecuteUpdateUserPrefIconCommand, CanExecuteUpdateUserPrefIconCommand);
            ProcessUserPreferencesCommand = new DelegateCommand<string>(ExecuteProcessUserPreferencesCommand, CanExecuteProcessUserPreferencesCommand);

            EventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(ProcessPatientChange, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<PatientSelectedEvent>().Subscribe(PatientSelected, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<PatientUpdatedEvent>().Subscribe(PatientSelected, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<PatientUnselectEvent>().Subscribe(PatientUnselect, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<SchedulerDisplayEvent>().Subscribe(SchedulerDisplay, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<ResourceSelectedEvent>().Subscribe(ResourceSelected, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<ResourceGroupSelectedEvent>().Subscribe(ResourceGroupSelected, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<EnableAppointmentButtonsEvent>().Subscribe(EnableAppointmentButtons, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<PassAppointmentsEvent>().Subscribe(PassAppointments, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<FocusPatientSearchEvent>().Subscribe(FocusPatientSearch, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<RibbonBarHotKeysEvent>().Subscribe(RibbonBarHotKeysHandler, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<CopyKeysPressedEvent>().Subscribe(CopyKeysPressed, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<PasteKeysPressedEvent>().Subscribe(PasteKeysPressed, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<IsPasteAvailabilityEvent>().Subscribe(IsPasteAvailability, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<IsManagerUserEvent>().Subscribe(SetManagerUser, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<DisplayDefaultDivisionEvent>().Subscribe(DisplayDefaultDivision, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<SelectTimeScaleEvent>().Subscribe(SelectTimeScale, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe(RefreshWaitList, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<LoadGlobalRecordsetsEvent>().Subscribe(LoadGlobalRecordsets, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<RemoveRequestEvent>().Subscribe(RemoveSelectedPatientFromWaitList, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<UpdateFutureAppointmentsEvent>().Subscribe(UpdateFutureAppointments, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<InsertAppointmentRequestEvent>().Subscribe(InsertAppointmentRequest, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<OpenNewAppointmentRequest>().Subscribe(OpenNewAppointmentRequestEvent, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<CloseFindAppointmentEvent>().Subscribe(CloseFindAppointmentEventHandler, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<UpdateUserPrefIconEvent>().Subscribe(UpdateUserPrefIconEventHandler, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<ProcessUserPreferencesEvent>().Subscribe(ProcessUserPreferencesEventHandler, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<LaunchFileVersionsEvent>().Subscribe(DisplayFileVersionsEventHandler, ThreadOption.UIThread, false);
            EventAggregator.GetEvent<ChangeViewModeEvent>().Subscribe(SchedulerViewModeChanged, ThreadOption.UIThread, false);
            // VAR Mobile Request Event Handlers
            EventAggregator.GetEvent<VAR_ButtonViewEvent>().Subscribe(VAR_ButtonViewEventHandler, ThreadOption.UIThread, false);
            eventAggregator.GetEvent<SelectRequestEvent>().Subscribe(SelectRequestAfterEwlTransfer, ThreadOption.UIThread, false);
            eventAggregator.GetEvent<RefreshWaitlistEvent>().Subscribe(UpdateWaitList, ThreadOption.UIThread, false);
            eventAggregator.GetEvent<PatientQueueViewHotKeysEvent>().Subscribe(PatientQueueViewHotKeyHandler, ThreadOption.UIThread, false);
            eventAggregator.GetEvent<ClearClinicInfoEvent>().Subscribe(ClearClinicInfo, ThreadOption.UIThread, false);

            _selectedPatientQuery.Clear();
        }

        private void ProcessPatientChange(Patient selectedPatient)
        {
            if (Keys.SendPatientData)
            {
                DataAccessService.SetCurrentPatient(selectedPatient);
            }
        }        

        private void ClearClinicInfo(string s)
        {
            Clinic = null;
        }

        private void UpdateWaitList(string wlIen)
        {
            //Console.WriteLine("Entering UpdateWaitList");
            PatientQueueView.PatientQueueRadGridView.ToolTip = "Viewing records 1 thru " + Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count + " of " + Keys.WaitListQueryTotalRecords;
            AdmittedPatients.Clear();
            foreach (var req in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
                AdmittedPatients.Add(req);

            foreach (AppointmentRequest ar in PatientQueueView.PatientQueueRadGridView.Items)
            {
                System.Windows.Forms.Application.DoEvents();
                if (ar.RequestManagmentIen == wlIen || ar.ApptRequestIen.ToString() == wlIen || ar.PtCSchRequestIen.ToString() == wlIen || ar.ConsultationRequestIen.ToString() == wlIen)
                {
                    PatientQueueView.PatientQueueRadGridView.ScrollIntoView(ar);
                    PatientQueueView.PatientQueueRadGridView.SelectedItem = ar;
                    return;
                }
            }
        }

        private void OpenNewAppointmentRequestEvent(NewAppointmentRequestParams requestParms)
        {
            var patient = DataAccessService.GetPatients(requestParms.PatientName, 1).FirstOrDefault();
            View.OpenNewRequestForPatient(patient, requestParms.DesiredDateTime, requestParms.ResourceIEN, requestParms.ResourceName, Keys.AppointmentRequestTypes.Appt);
        }

        private void PatientUnselect(string s)
        {
            Patient = null;
            View.UpdatePatientFlags(null);

            if (s == "VARPatientSearch")
            {
                View.UpdatePatientSearchText(string.Empty);
                EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish(string.Empty);
                Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = false;
                DoRefreshWaitList("");
            }
        }

        private void InsertAppointmentRequest(AppointmentRequest ar)
        {
            if (_waitTimeValue == Keys.RmGridColumns.WaitTimeAll)
                AdmittedPatients.Add(ar);
        }


        private void SelectRequestAfterEwlTransfer(string s)
        {
            if (PatientQueueView.PatientQueueRadGridView.Items.Count == 0)
                return;
            var selectedItem = PatientQueueView.PatientQueueRadGridView.Items[0] as AppointmentRequest;
            PatientQueueView.PatientQueueRadGridView.SelectedItem = null;
            System.Windows.Forms.Application.DoEvents();
            PatientQueueView.PatientQueueRadGridView.SelectedItem = selectedItem;
            //PatientQueueView.PatientQueueRadGridView.ScrollIntoView(selectedItem);
        }

        public void UpdateFutureAppointments(Patient patient)
        {
            View.UpdateFutureAppointmentsOnly(patient);
        }

        public void AppointmentSelectedEvent(AppointmentRequest appointmentRequest)
        {
            EventAggregator.GetEvent<AppointmentRequestSelectedEvent>().Publish(appointmentRequest);
        }

        private void RemoveSelectedPatientFromWaitList(string requestIen)
        {
            //Console.WriteLine("Entering RemoveSelectedPatientFromWaitList");
            // Must have either selected request or incoming requestIEN
            if (_selectedAppointmentRequest == null && string.IsNullOrEmpty(requestIen))
                return;

            // Removing a consult request ?
            if (Keys.RemoveConsultRequest)
            {
                Keys.RemoveConsultRequest = false;
                if (SelectedAppointmentRequest != null)
                    RemoveAppointmentRequest(SelectedAppointmentRequest);
                return;
            }

            var reqType = string.Empty;
            var reqIEN = requestIen;
            var selectedCause = string.Empty;
            var isSilent = false;
            AppointmentRequest request = null;

            if (string.IsNullOrEmpty(requestIen))
            {
                // No incoming request IEN, use the selected request
                request = _selectedAppointmentRequest;
            }
            else
            {
                if (requestIen.Contains('|'))
                {
                    var tmpData = requestIen.Split('|');
                    reqType = tmpData[0];
                    reqIEN = tmpData[1];

                    if (tmpData.Length > 2)
                        selectedCause = tmpData[2];

                    if (tmpData.Length > 3)
                        isSilent = tmpData[3] == "1";

                    if (!Keys.Methods.IsNumeric(tmpData[1]))
                    {
                        MessageBox.Show("Invalid Request Ien: " + tmpData[1]);
                        return;
                    }
                }

                // Check the incoming request IEN against the selected request
                if (_selectedAppointmentRequest != null)
                {
                    if (reqType == Keys.AppointmentRequestTypes.Ewl)
                    {
                        if (_selectedAppointmentRequest.RequestManagmentIen == reqIEN)
                            request = _selectedAppointmentRequest;
                    }

                    if (Keys.Methods.IsApptOrVet(reqType))
                    {
                        if (_selectedAppointmentRequest.ApptRequestIen == Convert.ToInt32(reqIEN))
                            request = _selectedAppointmentRequest;
                    }

                    if (request == null)
                    {
                        if (_selectedAppointmentRequest.RequestManagmentIen == reqIEN)
                            request = _selectedAppointmentRequest;
                    }
                }

                // Selected request was not what was asked for, look through cached requests
                if (request == null)
                {
                    // unfiltered requests
                    foreach (var pat in _unfilteredAdmittedPatients)
                    {
                        if (Keys.Methods.IsApptOrVet(reqType))
                        {
                            if (pat.ApptRequestIen.ToString() != reqIEN)
                                continue;
                            request = pat;
                            break;
                        }
                        else
                        {
                            if (pat.RequestManagmentIen != reqIEN)
                                continue;
                            request = pat;
                            break;
                        }
                    }
                }

                if (request == null)
                {
                    // filtered requests
                    foreach (var pat in AdmittedPatients)
                    {
                        if (Keys.Methods.IsApptOrVet(reqType))
                        {
                            if (pat.ApptRequestIen.ToString() != reqIEN)
                                continue;
                            request = pat;
                            break;
                        }
                        else
                        {
                            if (pat.RequestManagmentIen != reqIEN)
                                continue;
                            request = pat;
                            break;
                        }
                    }
                }

                // if the asked for request was not found in the cached request lists and the request is APPT\VETEREAN\W2VA then retrieve the request from the database
                if (request == null)
                {
                    if (Keys.Methods.IsApptOrVet(reqType))
                        request = DataAccessService.GetApptRequestByRequestIEN(reqIEN);
                    else if (reqType == Keys.AppointmentRequestTypes.Ewl)
                        request = DataAccessService.GetEwlApptRequest(reqIEN);
                    else if (reqType == Keys.AppointmentRequestTypes.PtCSch)
                        request = DataAccessService.GetPtCSchApptRequest(reqIEN);
                }
            }

            if (request != null)
            {
                var alertText = "This request will be closed.";
                var selectedClause = string.Empty;
                if (request.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl || Keys.Methods.IsApptOrVet(request.AppointmentRequestType))
                {
                    var bRemove = true;
                    if (request.MultipleAppointmentsRequired)
                    {
                        if (request.IsMRTCParent)
                        {
                            if (Keys.MrtcCloseRequestIen != request.ApptRequestIen.ToString())
                                bRemove = false;

                            Keys.MrtcCloseRequestIen = "";

                            if (string.IsNullOrEmpty(selectedCause))
                                selectedClause = Keys.WaitListDispositions.MRTCParentClosed;

                            alertText = "The parent MRTC request will be closed.";
                        }
                    }
                    if (bRemove)
                    {
                        if (!isSilent)
                            View.AlertUser(alertText, "Closing Request");


                        //Console.WriteLine("Removing request IEN" + _selectedAppointmentRequest.ApptRequestIen);
                        RemoveAppointmentRequest(request, selectedClause);

                        if (Keys.Methods.IsApptOrVet(request.AppointmentRequestType))
                        {
                            if (request.MultipleAppointmentsRequired)
                            {
                                if (request.IsMRTCChild)
                                {
                                    //GetNumber of Appts Booked for this request..
                                    var count = DataAccessService.GetMRTCParentBookedAppointmentsCount(request.MRTCParentRequestIEN);

                                    if (count >= request.NumberOfAppointmentsRequired)
                                    {
                                        Keys.MrtcCloseRequestIen = request.MRTCParentRequestIEN;
                                        var holdSelection = _selectedAppointmentRequest;

                                        try
                                        {
                                            _selectedAppointmentRequest = null;
                                            // Recursive Call
                                            RemoveSelectedPatientFromWaitList(Keys.AppointmentRequestTypes.Appt + "|" + request.MRTCParentRequestIEN + "|" + selectedCause + "|" + (isSilent ? "1" : "0"));
                                        }
                                        finally
                                        {
                                            _selectedAppointmentRequest = holdSelection;
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                }
            }
            // Console.WriteLine("Leaving RemoveSelectedPatientFromWaitList");
            Mouse.OverrideCursor = Cursors.Arrow;
        }

        private void PromptOnClosed(object sender, WindowClosedEventArgs e)
        {
            if (e.DialogResult == true)
            {
                /*var x = Keys.SelectedAppointmentRequest.ClinName;
                //if (x == SelectedAppointmentRequest.ClinName)
                    //MessageBox.Show("Hello");
                var parms = new List<string> { SelectedAppointmentRequest.RequestManagmentIen, Keys.WaitListDispositions.RemovedScheduledAssigned };
                _adt = OVIDDataTableRepository.Instance;
                var rslt = _adt.callOVIDRPC("SDEC WLCLOSE", parms);
                foreach (DataRow row in rslt.Rows)
                {
                    var errorId = row["ERRORID"].ToString();
                    var errorText = row["ERRORTEXT"].ToString();

                    if (errorId != "0")
                    {
                        if (!string.IsNullOrEmpty(errorText))
                        {
                            MessageBox.Show(errorText);
                            return;
                        }
                    }
                }*/
                RemoveAppointmentRequest(_selectedAppointmentRequest);
            }
        }

        private void LoadGlobalRecordsets(string T)
        {
            // this.DataAccessService.LoadGlobalRecordsets();
            EventAggregator.GetEvent<ManagementItemAddedEvent>().Publish(null);
        }

        private DateTime? _ewlListStartDate;
        private DateTime? _ewlListEndDate;

        public void RefreshWaitList(string arg)
        {
            if ((Keys.BooleanFlags.RefreshWaitList.HasValue && Keys.BooleanFlags.RefreshWaitList.Value) || Keys.BooleanFlags.ClearButtonPressed)
            {
                _waitTimeValue = "";
                WaitTimeValue = Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90;
            }
            else
            {
                // This forces a refresh to the current values
                var oldValue = WaitTimeValue;
                _waitTimeValue = "";
                WaitTimeValue = oldValue;
            }
        }

        bool _processingDoRefreshWaitList;

        public void DoRefreshWaitList(string arg)
        {
            if (_processingDoRefreshWaitList)
                return;

            _processingDoRefreshWaitList = true;

            if (Keys.UserPreferences.AreDefined)
                ReorderColumns();

            AdmittedPatients.Clear();
            PatientQueueView.PatientQueueRadGridView.SortDescriptors.Clear();
            PatientQueueView.PatientQueueRadGridView.FilterDescriptors.Clear();

            _admittedPatients.Clear();
            PatientQueueView.PatientQueueRadGridView.SortDescriptors.Clear();
            PatientQueueView.PatientQueueRadGridView.FilterDescriptors.Clear();

            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
                        WorkAsyncResults results = new WorkAsyncResults();
                        string action = "Initializing";
                        List<string> messages = new List<string>();

                        try
                        {
                            if (Keys.UserPreferences.AreDefined)
                            {
                                action = "Getting Filtered Requests";
                                DataAccessService.GetFilteredData(Keys.WaitListQueryLastRec, true, true, messages);
                            }
                            else
                            {
                                action = "Getting Wait List Requests";
                                DataAccessService.GetWaitListRecords(Keys.WaitListQueryLastRec, messages);
                            }

                            if (!Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
                            {
                                action = "Getting Ewl Appointment Requests";
                                results.List.Add(DataAccessService.GetEwlAppointmentRequests(null, true));
                                action = "Getting Consult Appointment Requests";
                                results.List.Add(DataAccessService.GetConsultAppointmentRequests(true));
                                action = "Getting PtCSch Appointment Requests";
                                results.List.Add(DataAccessService.GetPtCSchAppointmentRequests(null, true, messages).ToList());
                            }

                            results.Messages = messages;
                            results.Success = true;
                        }
                        catch (Exception ex)
                        {
                            results.Error = new Exception(string.Format("While loading Patient Requests there was a problem {0}.", action), ex);
                        }

                        args.Result = results;
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        if (args.Result != null)
                        {
                            WorkAsyncResults results = (WorkAsyncResults)args.Result;

                            if (results.Error != null)
                            {
                                Keys.AlertUserError(View, results.Error, "Refresh Wait List Error");
                                return;
                            }

                            if (results.Messages != null)
                                if (results.Messages.Count > 0)
                                    View.AlertUser(string.Join("\n", results.Messages.ToArray()), "Refresh Wait List");

                            if (results.Success)
                            {
                                PatientQueueView.PatientQueueRadGridView.ToolTip = "Viewing records 1 thru " + Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count + " of " + Keys.WaitListQueryTotalRecords;

                                if (!Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
                                {
                                    if (results.List.Count > 0)
                                        foreach (var eReq in (List<AppointmentRequest>)results.List[0])
                                            AdmittedPatients.Add(eReq);

                                    if (results.List.Count > 1)
                                        foreach (var cReq in (List<AppointmentRequest>)results.List[1])
                                            AdmittedPatients.Add(cReq);

                                    if (results.List.Count > 2)
                                        foreach (var rReq in (List<AppointmentRequest>)results.List[2])
                                            AdmittedPatients.Add(rReq);
                                }
                                else
                                    foreach (var req in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
                                        AdmittedPatients.Add(req);

                                _unfilteredAdmittedPatients = _admittedPatients.ToList();
                                PatientQueueView.UpdateWaitTimeValue(WaitTimeValue); //Keys.RmGridColumns.WaitTimeAll);
                                OnPropertyChanged("AdmittedPatients");
                                PatientQueueView.PatientQueueRadGridView.SelectedItem = null;
                                View.UpdatePatientSearchText(string.Empty);
                                View.UpdateSpecialNeedsRibbon(new List<string>());
                            }
                            else
                            {
                                TraceLog.LogException(GetType().Name, "DoRefreshWaitList", results.Error);
                                Keys.AlertUserError(PatientQueueView, results.Error, "Wait List Refresh Error");
                            }
                        }

                        _processingDoRefreshWaitList = false;
                        Mouse.OverrideCursor = Cursors.Arrow;
                    }
                },
                true,
                false,
                "Loading Patient Requests"
            );
        }
        

        private struct SelectedPatientQuery
        {
            public string patientIen;
            public string lastRec;
            public bool isNewQuery;
            public bool byDescendingRequestIen;
            public string requestType;
            public string newlyOpenedRequestIen;
            public string oldSearchType;
            public bool DoSecondaryLoad;

            public void Clear()
            {
                patientIen = string.Empty;
                lastRec = Keys.WaitListQueryLastRec;
                isNewQuery = false;
                byDescendingRequestIen = false;
                requestType = string.Empty;
                newlyOpenedRequestIen = string.Empty;
                DoSecondaryLoad = true;
            }
        }

        private SelectedPatientQuery _selectedPatientQuery;
        private AppointmentRequest _selectedPatient_selectedRequest;

        public void GetWaitListRecordsForSelectedPatient(string patientIen, string lastRec = "", bool isNewQQuery = false, bool byDescendingRequestIen = false, string requestType = "", string newlyOpenedRequestIen = "", AppointmentRequest selectedRequest = null)
        {
            _selectedPatientQuery.Clear();
            _selectedPatientQuery.patientIen = string.IsNullOrEmpty(patientIen) ? Patient.IEN : patientIen;
            _selectedPatientQuery.lastRec = lastRec;
            _selectedPatientQuery.isNewQuery = isNewQQuery;
            _selectedPatientQuery.byDescendingRequestIen = byDescendingRequestIen;
            _selectedPatientQuery.requestType = requestType;
            _selectedPatientQuery.newlyOpenedRequestIen = newlyOpenedRequestIen;
            _selectedPatientQuery.oldSearchType = Keys.WaitListQueryType;
            _selectedPatient_selectedRequest = selectedRequest;
            Keys.WaitListQueryType = Keys.WaitListQueryTypes.SelectPatient;
            WaitList_Changing();
        }

        bool _processingWaitListChanging;

        public void WaitList_Changing()
        {
            if (_processingWaitListChanging)
                return;

            _processingWaitListChanging = true;
            IsAsync = true;
            DoWorkAsync(
                (s, args) =>
                {
                    lock (args)
                    {
                        WorkAsyncResults results = new WorkAsyncResults();
                        string action = "Initializing";
                        List<string> messages = new List<string>();

                        try
                        {
                            //Initial Pull Query....
                            if (Keys.WaitListQueryType == Keys.WaitListQueryTypes.InitialPull)
                            {
                                if (Keys.UserPreferences.AreDefined)
                                {
                                    action = "Getting Filtered Data";
                                    DataAccessService.GetFilteredData(Keys.WaitListQueryLastRec, false, true, messages);
                                }
                                else
                                {
                                    action = "Getting Wait List Records";
                                    DataAccessService.GetWaitListRecords(Keys.WaitListQueryLastRec, messages);
                                }
                            }
                            else if (Keys.WaitListQueryType == Keys.WaitListQueryTypes.SelectPatient)
                            {
                                action = "Getting Wait List for Selected Patient";
                                string lastRec = _selectedPatientQuery.lastRec;
                                bool isNewQuery = _selectedPatientQuery.isNewQuery;
                                bool byDescendingRequestIen = _selectedPatientQuery.byDescendingRequestIen;
                                string requestType = _selectedPatientQuery.requestType;
                                string newlyOpenedRequestIen = _selectedPatientQuery.newlyOpenedRequestIen;
                                bool doSecondaryLoad = _selectedPatientQuery.DoSecondaryLoad;

                                DataAccessService.GetWaitListRecordsForSelectedPatient(Patient.IEN, lastRec, isNewQuery, byDescendingRequestIen, requestType, newlyOpenedRequestIen, true, messages);
                                _selectedPatientQuery.lastRec = Keys.WaitListQueryLastRec;
                                _selectedPatientQuery.isNewQuery = false;
                            }
                            else if (Keys.WaitListQueryType == Keys.WaitListQueryTypes.ServerQuery)
                            {
                                action = "Getting Filtered Wait List Records";
                                if (!DataAccessService.GetFilteredWaitListRecords(Keys.WaitListCurrentFilter, Keys.WaitListQueryLastRec, false, null, messages))
                                    return;
                            }

                            results.Success = true;
                        }
                        catch (Exception ex)
                        {
                            results.Error = new Exception(string.Format("While updating Patient Requests there was a problem {0}.", action), ex);
                        }

                        args.Result = results;
                    }
                },
                (s, args) =>
                {
                    lock (args)
                    {
                        if (args.Result != null)
                        {
                            WorkAsyncResults results = (WorkAsyncResults)args.Result;

                            if (results.Error != null)
                            {
                                Keys.AlertUserError(View, results.Error, "Wait List Changing Error");
                                return;
                            }

                            if (results.Messages != null && results.Messages.Count > 0)
                                View.AlertUser(string.Join("\n", results.Messages.ToArray()), "Wait List Changing");

                            if (results.Success)
                            {
                                AdmittedPatients.Clear();

                                if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count >= Keys.WaitListQueryTotalRecords)
                                    Keys.WaitListQueryLastRec = "";

                                foreach (var rec in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
                                    AdmittedPatients.Add(rec);

                                OnPropertyChanged("AdmittedPatients");
                                PatientQueueView.PatientQueueRadGridView.ToolTip = "Viewing records 1 thru " + Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count + " of " + Keys.WaitListQueryTotalRecords;
                                AppointmentRequest oldRequest = null;

                                if (_selectedPatient_selectedRequest != null)
                                    oldRequest = _selectedPatient_selectedRequest;
                                else if (Keys.SelectedPatientAppointmentRequest != null)
                                    oldRequest = Keys.SelectedPatientAppointmentRequest;

                                if (oldRequest != null)
                                {
                                    AppointmentRequest newRequest = AdmittedPatients.FirstOrDefault(r => (r.UniqueID == oldRequest.UniqueID));

                                    if (newRequest == null)
                                    {
                                        if (oldRequest.IsAppt || oldRequest.IsVeteran)
                                            newRequest = DataAccessService.GetApptRequestByRequestIEN(oldRequest.ApptRequestIen.ToString());
                                        else if (oldRequest.IsEwl)
                                            newRequest = DataAccessService.GetEwlApptRequest(oldRequest.RequestManagmentIen);
                                        else if (oldRequest.IsPtCSch)
                                            newRequest = DataAccessService.GetPtCSchApptRequest(oldRequest.PtCSchRequestIen.ToString());

                                        if (newRequest != null)
                                            AdmittedPatients.Insert(0, newRequest);
                                    }

                                    if (newRequest != null)
                                    {
                                        SelectedAppointmentRequest = null;
                                        System.Windows.Forms.Application.DoEvents();
                                        SelectedAppointmentRequest = newRequest;
                                        Keys.SelectedAppointmentRequest = newRequest;
                                        PatientQueueView.ScrollSelectedRowIntoView();
                                    }
                                }

                                Keys.SelectedPatientAppointmentRequest = null;
                                _selectedPatient_selectedRequest = null;
                            }
                            else
                            {
                                TraceLog.LogException(GetType().Name, "WaitList_Changing", results.Error);
                                Keys.AlertUserError(this, results.Error, "Wait List Update Error");
                            }
                        }

                        _processingWaitListChanging = false;
                        Mouse.OverrideCursor = Cursors.Arrow;
                    }
                },
                true,
                false,
                "Loading Patient Requests"
            );
        }

        private void FilterAdmittedPatientsByWaitTime(string waitTime)
        {
            IEnumerable<AppointmentRequest> newList;
            // Defaule dates for 90 days or greater
            _ewlListStartDate = null;
            _ewlListEndDate = null;
            var now = DateTime.Today;

            if (waitTime == Keys.RmGridColumns.WaitTimeLessThan30)
            {
                _ewlListStartDate = now.AddDays(-30);
                _ewlListEndDate = now;
            }
            else if (waitTime == Keys.RmGridColumns.WaitTimeBetween30And60)
            {
                _ewlListStartDate = now.AddDays(-60);
                _ewlListEndDate = now.AddDays(-30);
            }
            else if (waitTime == Keys.RmGridColumns.WaitTimeBetween60And90)
            {
                _ewlListStartDate = now.AddDays(-90);
                _ewlListEndDate = now.AddDays(-60);
            }
            else if (waitTime == Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90)
            {
            }
            else
                _ewlListEndDate = now;

            if (Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded == false || Keys.BooleanFlags.RefreshWaitList == true || Keys.BooleanFlags.ClearButtonPressed)
            {
                Keys.BooleanFlags.RefreshWaitList = false;
                Keys.BooleanFlags.ClearButtonPressed = false;
                DoRefreshWaitList("REFRESH");

                if (waitTime == Keys.RmGridColumns.WaitTimeLessThan30)
                    newList = new List<AppointmentRequest>(
                            _unfilteredAdmittedPatients.Where(c => c.WaitTime.HasValue && c.WaitTime.Value < 30));
                else if (waitTime == Keys.RmGridColumns.WaitTimeBetween30And60)
                    newList = new List<AppointmentRequest>(
                            _unfilteredAdmittedPatients.Where(
                                c => c.WaitTime.HasValue && c.WaitTime.Value >= 30 && c.WaitTime.Value < 60));
                else if (waitTime == Keys.RmGridColumns.WaitTimeBetween60And90)
                    newList = new List<AppointmentRequest>(
                            _unfilteredAdmittedPatients.Where(
                                c => c.WaitTime.HasValue && c.WaitTime.Value >= 60 && c.WaitTime.Value < 90));
                else if (waitTime == Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90)
                    newList = new List<AppointmentRequest>(
                            _unfilteredAdmittedPatients.Where(c => c.WaitTime.HasValue && c.WaitTime.Value >= 90));
                else
                    newList = new List<AppointmentRequest>(_unfilteredAdmittedPatients);
            }
            else
            {
                if (waitTime == Keys.RmGridColumns.WaitTimeLessThan30)
                    newList = new List<AppointmentRequest>(
                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Where(c => c.WaitTime.HasValue && c.WaitTime.Value < 30));
                else if (waitTime == Keys.RmGridColumns.WaitTimeBetween30And60)
                    newList = new List<AppointmentRequest>(
                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Where(
                                    c => c.WaitTime.HasValue && c.WaitTime.Value >= 30 && c.WaitTime.Value < 60));
                else if (waitTime == Keys.RmGridColumns.WaitTimeBetween60And90)
                    newList = new List<AppointmentRequest>(
                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Where(
                                    c => c.WaitTime.HasValue && c.WaitTime.Value >= 60 && c.WaitTime.Value < 90));
                else if (waitTime == Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90)
                    newList = new List<AppointmentRequest>(
                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Where(c => c.WaitTime.HasValue && c.WaitTime.Value >= 90));
                else
                    newList = new List<AppointmentRequest>(Keys.AppointmentRequestWaitListTables.AllAppointmentRequests);
            }

            var rmGridFilteredForPatient = IsRmGridFilteredForPatient(Patient);
            AdmittedPatients.Clear();

            foreach (var request in newList)
            {
                if (Keys.BooleanFlags.ClearButtonPressed || Keys.BooleanFlags.RefreshWaitList == true || !rmGridFilteredForPatient)
                {
                    Patient = null;
                    Keys.BooleanFlags.ClearButtonPressed = false;
                    Keys.BooleanFlags.RefreshWaitList = false;
                }
                if (Patient == null)
                    AdmittedPatients.Add(request);
                else if (Patient.IEN == request.Patient.IEN)
                    AdmittedPatients.Add(request);
            }
        }

        private bool IsRmGridFilteredForPatient(Patient patient)
        {
            if (patient == null)
                return false;
            return Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.All(request => request.Patient.IEN == patient.IEN);
        }

        public void SelectTimeScale(int timeScaleIndex)
        {
            TimeScaleIndex = timeScaleIndex;
        }

        public void DisplayDefaultDivision(string divisionName)
        {
            var name = divisionName;

            if (divisionName.Contains("^"))
            {
                // DivisionIEN^DivisionName
                var parts = divisionName.Split('^');
                name = parts[1];
            }

            PaneTitle = "VistA Scheduling for Division: "
                + (string.IsNullOrEmpty(name) ? "UNASSIGNED" : name)
                + (Keys.LoggedInUserSDECViewOnlyKey ? " - VIEW ONLY" : "");
            if (Keys.SignonResponse[7] == "0")
            {
                EnvironmentType = " TEST ENVIRONMENT ";
                EnvironmentColor = Brushes.Black;
            }
            else
            {
                EnvironmentType = " PRODUCTION ENVIRONMENT ";
                EnvironmentColor = Brushes.Red;
            }
        }

        private string _environmentType;

        public string EnvironmentType
        {
            get { return _environmentType; }
            set
            {
                _environmentType = value;
                NotifyPropertyChanged("EnvironmentType");
            }
        }

        private Brush _environmentColor;

        public Brush EnvironmentColor
        {
            get { return _environmentColor; }
            set
            {
                _environmentColor = value;
                NotifyPropertyChanged("EnvironmentColor");
            }
        }

        private AppointmentRequest _selectedAppointmentRequest;

        public AppointmentRequest SelectedAppointmentRequest
        {
            get { return _selectedAppointmentRequest; }
            set
            {
                if (value != null && value != _selectedAppointmentRequest && value.ApptRequestIen > 0)
                    EventAggregator.GetEvent<LaunchPrerequisitesDialogEvent>().Publish(value);

                _selectedAppointmentRequest = value;
                NotifyPropertyChanged("SelectedAppointmentRequest");

                if (_selectedAppointmentRequest != null)
                {
                    SelectedPendingAppointment = null;
                }                
            }
        }

        public void PatientSelected(Patient selectedPatient)
        {
            // Make sure this patient record is complete
            if ((selectedPatient != null) && (!selectedPatient.GetRegARead && !selectedPatient.PTLookRSRead))
                DataAccessService.GetPatient(ref selectedPatient);

            Patient = selectedPatient;

            //if ((Patient != null) && Patient.LoadRMGrid)
            if ((Patient != null) && Patient.LoadRMGrid)
            {
                if ((Keys.SelectedPatientAppointmentRequest == null) && (SelectedAppointmentRequest != null))
                    if (SelectedAppointmentRequest.Patient.IEN == Patient.IEN)
                        Keys.SelectedPatientAppointmentRequest = SelectedAppointmentRequest;

                View.PatientChanged(Patient, null, string.Empty, string.Empty, string.Empty, true);
                Patient.LoadRMGrid = false;
            }
            else
            {                
                Keys.SelectedPatient = selectedPatient;

                if ((Patient != null) && !Patient.Preferences.Any())
                    Patient.Preferences.AddRange(DataAccessService.GetPatientPreferences(Patient.IEN));

                OnPropertyChanged("Patient");

                IsNewAppointmentEnabled = selectedPatient != null && SelectedAppointment != null;
                View.UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(Patient);
                View.UpdatePatientFlags(Patient);
                //                View.UpdateTabOrder();
            }
        }        

        public void ResourceSelected(SchdResource selectedResource)
        {
            Clinic = selectedResource.Clinic;
            if (Clinic != null)
            {
                Keys.SelectedClinic = Clinic;
                if (string.IsNullOrEmpty(Clinic.PROVIDER))
                {
                    Clinic.PROVIDER = "None";
                }
                NotifyPropertyChanged("Clinic");
            }
            Resource = selectedResource;
            var newGroupList = new List<SchdResource>();
            newGroupList.Insert(0, Resource);
            SelectedResourceList = newGroupList;
        }

        public void ResourceGroupSelected(SchdResourceGroup selectedResourceGroup)
        {
            var c = new RPMSClinic(DataAccessService)
            {
                PROVIDER = "Multiple"
            };

            Clinic = c;
            ResourceGroup = selectedResourceGroup;
            SelectedResourceList = ResourceGroup.Resources;
        }

        public void SelectPatient(Patient selectedPatient)
        {            
            EventAggregator.GetEvent<PatientSelectedEvent>().Publish(selectedPatient);
        }
        
        public void PassAppointments(SchdAppointment newAppointment)
        {
            SelectedAppointment = newAppointment;
        }

        public void SchedulerDisplay(SchdResource selectedResource)
        {
            Clinic = selectedResource.Clinic;
        }

        private void FocusPatientSearch(object T)
        {
            View.FocusPatientSearch();
        }

        private void RibbonBarHotKeysHandler(Key k)
        {
            View.RibbonBarHotKeysHandler(k);
        }

        private void PatientQueueViewHotKeyHandler(Key k)
        {
            PatientQueueView.HotKeyHandler(k);
        }

        private void CopyKeysPressed(object T)
        {
            CopyAppointmentCommand.Execute(null);
        }

        private void PasteKeysPressed(object T)
        {
            PasteAppointmentCommand.Execute(null);
        }

        private void IsPasteAvailability(string command)
        {
            _isPasteAvailability = command == "New Availability";
        }

        private void SetManagerUser(bool isManager)
        {
            IsManagerUser = isManager && !Keys.IsCallCenterUser;
        }

        public Visibility SystemTabVisibility
        {
            get { return SDECViewOnly && IsManagerUser ? Visibility.Visible : Visibility.Collapsed; }
        }

        public bool SDECViewOnly
        {
            get { return !Keys.LoggedInUserSDECViewOnlyKey; }
        }

        public bool IsEditAppointmentEnabled
        {
            get { return _isEditAppointmentEnabled; }
            set
            {
                _isEditAppointmentEnabled = value;
                NotifyPropertyChanged("IsEditAppointmentEnabled");
            }
        }

        public bool IsViewAppointmentEnabled
        {
            get { return _isViewAppointmentEnabled; }
            set
            {
                _isViewAppointmentEnabled = value;
                NotifyPropertyChanged("IsViewAppointmentEnabled");
            }
        }

        public bool IsNewAppointmentEnabled
        {
            get { return _isNewAppointmentEnabled; }
            set
            {
                _isNewAppointmentEnabled = value;
                NotifyPropertyChanged("IsNewAppointmentEnabled");
            }
        }

        public bool IsExistAppointmentEnabled
        {
            get { return _isExistAppointmentEnabled; }
            set
            {
                _isExistAppointmentEnabled = value;
                NotifyPropertyChanged("IsExistAppointmentEnabled");
            }
        }

        public bool IsNoShowAppointmentEnabled
        {
            get { return _isNoShowAppointmentEnabled; }
            set
            {
                _isNoShowAppointmentEnabled = value;
                NotifyPropertyChanged("IsNoShowAppointmentEnabled");
            }
        }

        public bool IsCancelledAppointmentEnabled
        {
            get { return _isCancelledAppointmentEnabled; }
            set
            {
                _isCancelledAppointmentEnabled = value;
                NotifyPropertyChanged("IsCancelledAppointmentEnabled");
            }
        }

        public bool IsCheckedInAppointmentEnabled
        {
            get { return _isCheckedInAppointmentEnabled; }
            set
            {
                _isCheckedInAppointmentEnabled = value;
                NotifyPropertyChanged("IsCheckedInAppointmentEnabled");
            }
        }

        public bool IsCheckedOutAppointmentEnabled
        {
            get { return _isCheckedOutAppointmentEnabled; }
            set
            {
                _isCheckedOutAppointmentEnabled = value;
                NotifyPropertyChanged("IsCheckedOutAppointmentEnabled");
            }
        }

        public bool IsNotNoShowAppointmentEnabled
        {
            get { return _isNotNoShowAppointmentEnabled; }
            set
            {
                _isNotNoShowAppointmentEnabled = value;
                NotifyPropertyChanged("IsNotNoShowAppointmentEnabled");
            }
        }

        public bool IsNotCancelledAppointmentEnabled
        {
            get { return _isNotCancelledAppointmentEnabled; }
            set
            {
                _isNotCancelledAppointmentEnabled = value;
                NotifyPropertyChanged("IsNotCancelledAppointmentEnabled");
            }
        }

        public bool IsNotCheckedInAppointmentEnabled
        {
            get { return _isNotCheckedInAppointmentEnabled; }
            set
            {
                _isNotCheckedInAppointmentEnabled = value;
                NotifyPropertyChanged("IsNotCheckedInAppointmentEnabled");
            }
        }

        public bool IsNotCheckedOutAppointmentEnabled
        {
            get { return _isNotCheckedOutAppointmentEnabled; }
            set
            {
                _isNotCheckedOutAppointmentEnabled = value;
                NotifyPropertyChanged("IsNotCheckedOutAppointmentEnabled");
            }
        }

        public bool IsDisplayWalkInEnabled { get; set; }

        public bool ArePendingAppointmentsVisible
        {
            get { return _arePendingAppointmentsVisible; }
            set
            {
                _arePendingAppointmentsVisible = value;
                NotifyPropertyChanged("ArePendingAppointmentsVisible");
            }
        }

        private Patient _patient;

        public Patient Patient
        {
            get { return _patient; }
            set
            {
                _patient = value;
                NotifyPropertyChanged("Patient");
                NewRequestCommand.RaiseCanExecuteChanged();
            }
        }

        public void UpdatePatient()
        {
            NotifyPropertyChanged("Patient");
        }

        public SchdAppointment SelectedAppointment { get; set; }

        public RPMSClinic Clinic
        {
            get { return _clinic; }
            set
            {
                _clinic = value;
                NotifyPropertyChanged("Clinic");
            }
        }

        public IScheduleView View { get; }

        public ScheduleView View2 { get; set; }

        public IPatientQueueView PatientQueueView { get; }

        public SchdResourceGroup ResourceGroup { get; set; }

        public SchdResource Resource { get; set; }

        public IList<SchdResource> SelectedResourceList { get; set; }

        public DelegateCommand<string> ExpandedEntryCommand { get; private set; }
        public void ExecuteExpandedEntryCommand(string command)
        {
            var selectedPendingAppointment = SelectedPendingAppointment;

            if (selectedPendingAppointment == null)
                return;

            Dictionary<string, string> patientIenAndAppointmentStartTimeDictionary = new Dictionary<string, string>();
            patientIenAndAppointmentStartTimeDictionary.Add("IEN", selectedPendingAppointment.Dfn);
            patientIenAndAppointmentStartTimeDictionary.Add("AppointmentStartTime", ExtensionMethods.ConvertToFilemanDateTime(selectedPendingAppointment.AppointmentDateTime));

            EventAggregator.GetEvent<LaunchExpandedEntryDialogEvent>().Publish(patientIenAndAppointmentStartTimeDictionary);
        }


        public DelegateCommand<string> NewRequestCommand { get; }

        public DelegateCommand<string> NewAppointmentCommand { get; private set; }

        public DelegateCommand<string> WalkInAppointmentCommand { get; private set; }

        public DelegateCommand<string> ViewAppointmentCommand { get; private set; }

        public DelegateCommand<string> EditAppointmentCommand { get; private set; }

        public DelegateCommand<string> CopyAppointmentCommand { get; }

        public DelegateCommand<string> PasteAppointmentCommand { get; }

        public DelegateCommand<string> SelectPatientCommand { get; private set; }

        //public DelegateCommand<string> ChangeServerCommand { get; private set; }
        public DelegateCommand<string> ChangeUserCommand { get; private set; }

        public DelegateCommand<string> ChangeDivisionCommand { get; private set; }

        public DelegateCommand<string> MarkAsNoShowCommand { get; private set; }

        public DelegateCommand<string> UndoANoShowCommand { get; private set; }

        public DelegateCommand<string> CheckInPatientCommand { get; private set; }

        public DelegateCommand<string> UndoCheckInPatientCommand { get; private set; }

        public DelegateCommand<string> CheckOutPatientCommand { get; private set; }

        public DelegateCommand<string> UndoCheckOutPatientCommand { get; private set; }

        public DelegateCommand<string> SelectResourceCommand { get; private set; }

        public DelegateCommand<string> AvailabilityResourceCommand { get; private set; }

        public DelegateCommand<string> FileVersionsCommand { get; private set; }

        public DelegateCommand<string> SchedulingManagementCommand { get; private set; }

        public DelegateCommand<string> ViewDayRangeCommand { get; private set; }

        public DelegateCommand<string> ViewTimeScaleCommand { get; private set; }

        public DelegateCommand<string> DisplayWalkInCommand { get; private set; }

        public DelegateCommand<string> HelpCommand { get; private set; }

        public DelegateCommand<string> ExitCommand { get; private set; }

        public DelegateCommand<string> CancelAppointmentCommand { get; private set; }

        public DelegateCommand<string> UndoCancelAppointmentCommand { get; private set; }

        public DelegateCommand<string> FindAppointmentCommand { get; private set; }

        public DelegateCommand<string> CopyClipboardCommand { get; private set; }

        public DelegateCommand<string> ChangeViewModeCommand { get; private set; }

        public DelegateCommand<string> PrintCurrentSchedulerCommand { get; private set; }

        public DelegateCommand<string> ReportPatientHSMergeCommand { get; private set; }

        public DelegateCommand<string> ReportPatientHistoryCommand { get; private set; }

        public DelegateCommand<string> ReportPatientAppointmentCommand { get; private set; }

        public DelegateCommand<string> ReportPatientLetterCommand { get; private set; }

        public DelegateCommand<string> ReportReminderLetterCommand { get; private set; }

        public DelegateCommand<string> ReportRebookLetterCommand { get; private set; }

        public DelegateCommand<string> ReportCancellationLetterCommand { get; private set; }

        public DelegateCommand<string> ReportClinicSchedulerCommand { get; private set; }

        public DelegateCommand<string> ReportClinicWalkInCommand { get; private set; }

        public DelegateCommand<string> LaunchBrowserCommand { get; private set; }

        public DelegateCommand<string> UpdateUserPrefIconCommand { get; private set; }

        public DelegateCommand<string> ProcessUserPreferencesCommand { get; set; }

        public void ExecuteCopyClipboardCommand(string command)
        {
        }

        public void ExecuteNewRequestCommand(string command)
        {
            if (Patient == null || string.IsNullOrEmpty(Patient.IEN))
                View.AlertUser("Please select a patient to create a new appointment request", "Warning");
            else
                View.OpenNewRequestForPatient(Patient);
        }

        public void ExecuteNewAppointmentCommand(string command)
        {
            if (SelectedAppointment == null)
                SelectedAppointment = new SchdAppointment();
            SelectedAppointment.PATIENTID = Patient.IEN;
            SelectedAppointment.PATIENTNAME = Patient.Name;
            SelectedAppointment.RESOURCENAME = Resource.RESOURCE_NAME;
            SelectedAppointment.SelectedResource = Resource;
            EventAggregator.GetEvent<AddNewAppointmentEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteWalkInAppointmentCommand(string title)
        {
            if (SelectedAppointment == null)
                SelectedAppointment = new SchdAppointment();
            SelectedAppointment.PATIENTID = Patient.IEN;
            SelectedAppointment.PATIENTNAME = Patient.Name;
            SelectedAppointment.RESOURCENAME = Resource.RESOURCE_NAME;
            SelectedAppointment.SelectedResource = Resource;
            EventAggregator.GetEvent<AddWalkInAppointmentEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteViewAppointmentCommand(string command)
        {
            EventAggregator.GetEvent<ViewAppointmentEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteEditAppointmentCommand(string command)
        {
            EventAggregator.GetEvent<EditAppointmentEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteCopyAppointmentCommand(string command)
        {
            EventAggregator.GetEvent<CopyAppointmentEvent>().Publish(SelectedAppointment);
        }

        public void ExecutePasteAppointmentCommand(string command)
        {
            if (_isPasteAvailability)
            {
                EventAggregator.GetEvent<PasteAvailabilityEvent>().Publish(SelectedAppointment);
            }
            else
            {
                EventAggregator.GetEvent<PasteAppointmentEvent>().Publish(SelectedAppointment);
            }
        }

        public void ExecuteSelectPatientCommand(string title)
        {
            EventAggregator.GetEvent<LaunchPatientSelectionDialogEvent>().Publish(title);
        }

        //public void ExecuteChangeServerCommand (string title)
        //{
        //    this.eventAggregator.GetEvent<LaunchChangeServerDialogEvent> ().Publish (title);
        //}

        public void ExecuteChangeUserCommand(string title)
        {
            if (View.ConfirmUser("Are you sure you want to change the login user?", "Change User"))
            {
                DataAccessService.ChangeUserServer();
            }
        }

        public void ExecuteChangeDivisionCommand(string title)
        {
            EventAggregator.GetEvent<LaunchChangeDivisionDialogEvent>().Publish(title);
        }

        public void ExecuteMarkAsNoShowCommand(string title)
        {
            EventAggregator.GetEvent<LaunchMarkAsNoShowDialogEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteUndoANoShowCommand(string command)
        {
            EventAggregator.GetEvent<UndoNoShowAppointmentEvent>().Publish(command);
        }

        public void ExecuteCheckInPatientCommand(string command)
        {
            EventAggregator.GetEvent<LaunchCheckInDialogEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteUndoCheckInPatientCommand(string command)
        {
            EventAggregator.GetEvent<UndoCheckInPatientEvent>().Publish(command);
        }

        public void ExecuteCheckOutPatientCommand(string command)
        {
            EventAggregator.GetEvent<LaunchCheckOutDialogEvent>().Publish(SelectedAppointment);
        }

        public void ExecuteUndoCheckOutPatientCommand(string command)
        {
            EventAggregator.GetEvent<UndoCheckOutPatientEvent>().Publish(command);
        }

        public void ExecuteCancelAppointmentCommand(string command)
        {
            EventAggregator.GetEvent<LaunchCancelApptDialogEvent>().Publish(new CancelApptDialogEventPayload(SelectedAppointment, null));
        }

        public void ExecuteUndoCancelAppointmentCommand(string command)
        {
            EventAggregator.GetEvent<UndoCancelAppointmentEvent>().Publish(command);
        }

        public void ExecuteFindAppointmentCommand(string command)
        {
            EventAggregator.GetEvent<LaunchFindApptDialogEvent>().Publish(SelectedResourceList);
        }

        public void ExecuteSelectResourceCommand(string title)
        {
            CompositePresentationEvent<SchdResource> newforwardingEvent = EventAggregator.GetEvent<SchedulerResourcesEvent>();
            EventAggregator.GetEvent<LaunchResourceSelectionDialogEvent>().Publish(newforwardingEvent);
        }

        public void ExecuteAvailabilityResourceCommand(string title)
        {
            CompositePresentationEvent<SchdResource> newforwardingEvent = EventAggregator.GetEvent<AvailabilityResourcesEvent>();
            EventAggregator.GetEvent<LaunchResourceAvailabilityDialogEvent>().Publish(newforwardingEvent);
        }

        public void ExecuteFileVersionsCommand(string title)
        {
            EventAggregator.GetEvent<LaunchFileVersionsEvent>().Publish(title);
        }

        public void ExecuteViewDayRangeCommand(string title)
        {
            EventAggregator.GetEvent<ViewDayRangeEvent>().Publish(title);
        }

        public void ExecuteViewTimeScaleCommand(string title)
        {
            EventAggregator.GetEvent<ViewTimeScaleEvent>().Publish(title);
        }

        public void ExecuteChangeViewModeCommand(string mode)
        {
            EventAggregator.GetEvent<ChangeViewModeEvent>().Publish(mode);
        }

        public void ExecuteDisplayWalkInCommand(string isDisplayWalkInEnabled)
        {
            EventAggregator.GetEvent<DisplayWalkInEvent>().Publish(isDisplayWalkInEnabled);
        }

        public void ExecuteSchedulingManagementCommand(string title)
        {
            EventAggregator.GetEvent<LaunchSchedulingManagementDialogEvent>().Publish(title);
        }

        private void ExecuteUserPrefsCommand(string title)
        {
            UserPreferencesCommand.Execute(title);
        }

        public void ExecuteHelpCommand(string title)
        {
            EventAggregator.GetEvent<HelpEvent>().Publish("Help");
        }

        public void ExecuteExitCommand(string title)
        {
            EventAggregator.GetEvent<ExitApplicationEvent>().Publish("Exiting");
            DataAccessService.Logout();
            DataAccessService.RemoveJasperReportDependenciesFromUsersFolder();
            Environment.Exit(1);
        }

        public void ExecutePrintCurrentSchedulerCommand(string cmd)
        {
            EventAggregator.GetEvent<PrintCurrentSchedulerEvent>().Publish(null);
        }

        private void ExecuteReportPatientHSMergeCommand(string cmd)
        {
            var args = new object[2];
            args[0] = "Patient HS Merge";
            args[1] = Patient.IEN;
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportPatientHistoryCommand(string cmd)
        {
            var args = new object[2];
            args[0] = "Patient History Report";
            args[1] = Patient.IEN;
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportPatientAppointmentCommand(string cmd)
        {
            var args = new object[2];
            args[0] = "Patient Appointment Letter";
            args[1] = Patient.IEN;
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportPatientLetterCommand(string cmd)
        {
            var args = new object[3];
            args[0] = "Patient Reminder Letter Formatted";
            args[1] = Resource.RESOURCEID;
            args[2] = "R";
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportReminderLetterCommand(string cmd)
        {
            var args = new object[3];
            args[0] = "Patient Reminder Letter Formatted";
            args[1] = Resource.RESOURCEID;
            args[2] = "R";
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportRebookLetterCommand(string cmd)
        {
            var args = new object[3];
            args[0] = "Patient No Show Letter Formatted";
            args[1] = Resource.RESOURCEID;
            args[2] = "N";
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportCancellationLetterCommand(string cmd)
        {
            var args = new object[3];
            args[0] = "Patient Cancellation Letter Formatted";
            args[1] = Resource.RESOURCEID;
            args[2] = "A";
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportClinicSchedulerCommand(string cmd)
        {
            var args = new object[5];
            args[0] = "Clinic Scheduler Report";
            args[1] = Resource.RESOURCEID + "|";
            args[2] = DateTime.Now.AddDays(-7).ToString("M/d/yyyy@HH:mm");
            args[3] = DateTime.Now.AddDays(7).ToString("M/d/yyyy@HH:mm");
            args[4] = "0";
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteReportClinicWalkInCommand(string cmd)
        {
            var args = new object[4];
            args[0] = "Clinic WalkIn Report";
            args[1] = Resource.RESOURCEID + "|";
            args[2] = DateTime.Now.AddDays(-7).ToString("M/d/yyyy@HH:mm");
            args[3] = DateTime.Now.AddDays(7).ToString("M/d/yyyy@HH:mm");
            EventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void ExecuteLaunchBrowserCommand(string url)
        {
            var requestManagementBrowser = new ReqMngmtBrowser();
            CLaunchWindow.LaunchWindow(requestManagementBrowser, "Request Management Browser", 700, 700, "", true);
        }

        private void ExecuteUpdateUserPrefIconCommand(string cmd)
        {
            EventAggregator.GetEvent<UpdateUserPrefIconEvent>().Publish(Keys.UserPreferences.AreDefined);
        }

        private void ExecuteProcessUserPreferencesCommand(string cmd)
        {
            EventAggregator.GetEvent<ProcessUserPreferencesEvent>().Publish(Keys.UserPrefList);
        }

        public bool CanLaunchBrowserCommand(string url)
        {
            return true;
        }

        public bool CanExecuteUpdateUserPrefIconCommand(string cmd)
        {
            return true;
        }

        public bool CanExecuteProcessUserPreferencesCommand(string cmd)
        {
            return true;
        }

        public bool CanExecuteReportLetterCommand(string cmd)
        {
            return Clinic != null;
        }

        public bool CanExecutePrintCurrentSchedulerCommand(string cmd)
        {
            return true;
        }

        public bool CanExecuteCopyClipboardCommand(string title)
        {
            return true;
        }

        public bool CanExecuteNewRequestCommand(string title)
        {
            return Patient != null && !string.IsNullOrEmpty(Patient.IEN) && !Patient.IsDead && !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteNewAppointmentCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteWalkInAppointmentCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteViewAppointmentCommand(string title)
        {
            return _isViewAppointmentEnabled;
        }

        public bool CanExecuteEditAppointmentCommand(string title)
        {
            return _isEditAppointmentEnabled && !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteCopyAppointmentCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecutePasteAppointmentCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteSelectPatientCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        //public bool CanExecuteChangeServerCommand (string title)
        //{
        //    return true;
        //}

        public bool CanExecuteChangeUserCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteChangeDivisionCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteMarkAsNoShowCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteUndoANoShowCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteCheckInPatientCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteUndoCheckInPatientCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteCheckOutPatientCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteUndoCheckOutPatientCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteCancelAppointmentCommand(string title)
        {
            return _isNotCancelledAppointmentEnabled && !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteUndoCancelAppointmentCommand(string title)
        {
            return _isCancelledAppointmentEnabled && !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteFindAppointmentCommand(string title)
        {
            //if (this.Clinic != null) {
            return !Keys.LoggedInUserSDECViewOnlyKey;
            //} else {
            //	return false;
            //}
        }

        public bool CanExecuteSelectResourceCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteAvailabilityResourceCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteFileVersionsCommand(string title)
        {
            return true;
        }
        public bool CanExecuteViewDayRangeCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteViewTimeScaleCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteChangeViewModeCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteDisplayWalkInCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteSchedulingManagementCommand(string title)
        {
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public bool CanExecuteHelpCommand(string title)
        {
            return true;
        }

        public bool CanExecuteExitCommand(string title)
        {
            return true;
        }

        private bool CanExecuteReportPatientCommand(string cmd)
        {
            if (Patient == null)
            {
                return false;
            }
            return !Keys.LoggedInUserSDECViewOnlyKey;
        }

        public void OnPropertyChanged(string propertyName)
        {
            NotifyPropertyChanged(propertyName);
        }

        public void EnableAppointmentButtons(string command)
        {
            switch (command)
            {
                case "Reset Buttons":
                case "New Availability":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = false;
                    IsEditAppointmentEnabled = false;
                    IsExistAppointmentEnabled = false;
                    IsNoShowAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = false;
                    IsNotCancelledAppointmentEnabled = false;
                    break;

                case "New Appointment":
                    IsNewAppointmentEnabled = Patient != null && SelectedAppointment != null;
                    IsEditAppointmentEnabled = false;
                    IsViewAppointmentEnabled = false;
                    IsExistAppointmentEnabled = false;
                    IsNoShowAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = false;
                    IsNotCancelledAppointmentEnabled = false;
                    break;

                case "EXIST Appointment":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = true;
                    IsEditAppointmentEnabled = true;
                    IsExistAppointmentEnabled = true;
                    IsNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = true;
                    IsNotCheckedInAppointmentEnabled = true;
                    IsNotCheckedOutAppointmentEnabled = false;
                    IsNotCancelledAppointmentEnabled = true;
                    break;

                case "NOSHOW Appointment":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = true;
                    IsEditAppointmentEnabled = true;
                    IsExistAppointmentEnabled = true;
                    IsNoShowAppointmentEnabled = true;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = false;
                    IsNotCancelledAppointmentEnabled = true;
                    break;

                case "CANCELLED Appointment":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = true;
                    IsEditAppointmentEnabled = false;
                    IsExistAppointmentEnabled = true;
                    IsNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = true;
                    IsNotNoShowAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = false;
                    IsNotCancelledAppointmentEnabled = false;
                    break;

                case "WALKIN Appointment":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = true;
                    IsEditAppointmentEnabled = true;
                    IsExistAppointmentEnabled = true;
                    IsNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = true;
                    IsNotCancelledAppointmentEnabled = false;
                    break;

                case "CHECKOUT Appointment":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = true;
                    IsEditAppointmentEnabled = false;
                    IsExistAppointmentEnabled = true;
                    IsNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = false;
                    IsCheckedOutAppointmentEnabled = true;
                    IsCancelledAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = false;
                    IsNotCancelledAppointmentEnabled = false;
                    break;

                case "CHECKIN Appointment":
                    IsNewAppointmentEnabled = false;
                    IsViewAppointmentEnabled = true;
                    IsEditAppointmentEnabled = true;
                    IsExistAppointmentEnabled = true;
                    IsNoShowAppointmentEnabled = false;
                    IsCheckedInAppointmentEnabled = true;
                    IsCheckedOutAppointmentEnabled = false;
                    IsCancelledAppointmentEnabled = false;
                    IsNotNoShowAppointmentEnabled = false;
                    IsNotCheckedInAppointmentEnabled = false;
                    IsNotCheckedOutAppointmentEnabled = true;
                    IsNotCancelledAppointmentEnabled = false;
                    break;
            }
        }

        public void SearchStringChanged(string newSearchString)
        {
            if (_patientList != null)
                _patientList.Clear();

            bool isNameSearch;
            string errorMessage;
            NameValue ccowPatient = null;
            Keys.HavePatientData = false;
            Keys.SendPatientData = true;

            if (newSearchString == " ") // Special entry to get the current latest patient from ORWPT TOP (CCOW)
            {
                ccowPatient = DataAccessService.GetCCOWCurrentPatient();

                if (ccowPatient != null)
                {
                    Keys.HavePatientData = true;
                    Keys.SendPatientData = false;

                    if (!string.IsNullOrEmpty(ccowPatient.Name))
                        newSearchString = ccowPatient.Name;

                    View2.PatientSearchComboBox.Text = newSearchString;
                }
                else
                {
                    View.AlertUser("Patient not found.", "CCOW Patient Search");
                    return;
                }
            }

            if (!ValidateSearchString(newSearchString, out errorMessage, out isNameSearch))
            {
                View.AlertUser(errorMessage, "Patient Search Criteria");
                View.FocusPatientSearch();
                return;
            }

            var tempPatient = new Patient(DataAccessService)
            {
                IsAsync = false
            };
            tempPatient.GetPatients(newSearchString, (s, args) =>
           {
               lock (args)
               {
                   var list = args.Result as IList<Patient>;
                   SearchFailed = list == null;
                   if (list != null && list.Count == Keys.MaxPatientsToReturn && !string.IsNullOrEmpty(list[list.Count - 1].LastSub))
                   {
                       if (isNameSearch)
                       {
                           View.AlertUser("The current search criteria returns too many records.\nPlease refine your search and try again.", "Patient Search Criteria");
                           View.FocusPatientSearch();
                           return;
                       }
                   }

                   PatientList = list;

                   if (list != null)
                       if (Keys.HavePatientData && (ccowPatient != null))
                       {
                           if (!string.IsNullOrEmpty(ccowPatient.Value))
                               PatientList = list.Where(p => (p.IEN == ccowPatient.Value)).ToList();
                           else if (!string.IsNullOrEmpty(ccowPatient.Name))
                               PatientList = list.Where(p => (p.Name == ccowPatient.Name)).ToList();
                       }
               }
           });
        }

        public bool SearchFailed
        {
            get { return _searchFailed; }
            set
            {
                _searchFailed = value;
                NotifyPropertyChanged("SearchFailed");
            }
        }

        private bool ValidateSearchString(string newSearchString, out string errorMessage, out bool isNameSearch)
        {
            isNameSearch = true;

            // Check length greater than minimum Search characters
            if (newSearchString == null || newSearchString.Length < Keys.PatientSearchCharMin)
            {
                errorMessage = string.Format("The Patient Search value must be at least {0} characters in length.", Keys.PatientSearchCharMin);
                return false;
            }

            // does the search string contain any numbers ?
            if (newSearchString.Any(char.IsDigit))
            {
                // Check for SSN, all numeric and either 4 or 9 digits
                if (newSearchString.All(char.IsDigit))
                {
                    if (newSearchString.Length != 4 && newSearchString.Length != 9)
                    {
                        errorMessage = "When searching for a patient by SSN, the entered value must be 4 (SSN last 4) or 9 digits in length.";
                        return false;
                    }
                }
                else
                {
                    // Validate First initial of last name\last 4 of SSN pattern match
                    var regex = new Regex(@"^[a-zA-z]\d{4}$");

                    if (!regex.IsMatch(newSearchString))
                    {
                        errorMessage = "When searching for a patient by last name initial and last 4 of the SSN, the entered value must have one character (a-z) and 4 (last 4) digits.";
                        return false;
                    }
                }
                isNameSearch = false;
            }
            errorMessage = string.Empty;
            return true;
        }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                NotifyPropertyChanged("PaneTitle");
            }
        }

        public int TimeScaleIndex
        {
            get { return _timeScaleIndex; }
            set
            {
                _timeScaleIndex = value;
                NotifyPropertyChanged("TimeScaleIndex");
            }
        }

        public IList<Patient> PatientList
        {
            get { return _patientList; }
            set
            {
                if (_patientList != value && value != null)
                {
                    _patientList = value;
                    NotifyPropertyChanged("PatientList");
                    View.SetPatientSearchComboBoxOpen(_patientList.Count > 0);
                }
            }
        }

        public bool IsManagerUser
        {
            get { return _isManagerUser; }
            set
            {
                if (_isManagerUser != value)
                {
                    _isManagerUser = value;
                    NotifyPropertyChanged("IsManagerUser");
                    NotifyPropertyChanged("SystemTabVisibility");
                }
            }
        }

        public string Title { get; set; }

        internal void BlurPatientSearch(RadComboBox sender, string patientName)
        {
            var elementWithFocus = Keyboard.FocusedElement as UIElement;
            if (elementWithFocus != null)
                elementWithFocus.MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));
        }

        public void RemoveAppointmentRequest(AppointmentRequest appointmentRequest, string selectedCause = "")
        {
            try
            {
                if (appointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl || Keys.Methods.IsApptOrVet(appointmentRequest.AppointmentRequestType))
                {
                    for (var i = 0; i < Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count; i++)
                    {
                        var dr = Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i];
                        if (appointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl)
                        {
                            if (dr.RequestManagmentIen == appointmentRequest.RequestManagmentIen)
                            {
                                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Remove(dr);
                                break;
                            }
                        }
                        else if (Keys.Methods.IsApptOrVet(appointmentRequest.AppointmentRequestType))
                        {
                            if (dr.ApptRequestIen == appointmentRequest.ApptRequestIen)
                            {
                                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Remove(dr);
                                break;
                            }
                        }
                    }
                    if (string.IsNullOrEmpty(selectedCause))
                        selectedCause = Keys.WaitListDispositions.RemovedScheduledAssigned;

                    if (appointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl)
                    {
                        var errorText = DataAccessService.WLClose(appointmentRequest.RequestManagmentIen, selectedCause);

                        if (!string.IsNullOrEmpty(errorText))
                        {
                            View.AlertUser(errorText, "Close Request Error");
                            return;
                        }

                        foreach (DataRow row in Keys.AppointmentRequestWaitListTables.EwlWaitList.Rows)
                            if (row["IEN"].ToString() == appointmentRequest.RequestManagmentIen)
                            {
                                row.Delete();
                                break;
                            }
                    }
                    else if (Keys.Methods.IsApptOrVet(appointmentRequest.AppointmentRequestType))
                    {
                        var errorText = DataAccessService.ARClose(appointmentRequest.ApptRequestIen.ToString(), selectedCause);
                        if (errorText.Contains("-2"))
                        {
                            MessageBox.Show("An error occured when attempting to discontinue the RTC Order in CPRS:  " +
                                Environment.NewLine +
                                errorText + Environment.NewLine +
                                "Please contact your system administrator!",
                                "CPRS Error", MessageBoxButton.OK, MessageBoxImage.Warning);

                            errorText = ""; 
                        }
                        if (!string.IsNullOrEmpty(errorText))
                            View.AlertUser(errorText, "Close Request Error");
                        else
                        {
                            foreach (DataRow row in Keys.AppointmentRequestWaitListTables.ApptWaitList.Rows)
                                if (row["IEN"].ToString() == appointmentRequest.ApptRequestIen.ToString())
                                {
                                    row.Delete();
                                    break;
                                }
                            if (!DataAccessService.UnlockAppointmentRequest("A|" + appointmentRequest.ApptRequestIen))
                            {
                                View.AlertUser("Unable to unlock appointment request " + appointmentRequest.ApptRequestIen, "Close Request Error");
                                Mouse.OverrideCursor = Cursors.Arrow;
                                return;
                            }
                            Keys.LastRecordUnlocked = appointmentRequest.ApptRequestIen.ToString();
                        }
                    }
                }
                else if ((appointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Consult) || (appointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Procedure))
                {
                    for (var i = 0; i < Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count; i++)
                    {
                        var dr = Keys.AppointmentRequestWaitListTables.AllAppointmentRequests[i];
                        if (dr.ConsultationRequestIen == appointmentRequest.ConsultationRequestIen)
                        {
                            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Remove(dr);
                            break;
                        }
                    }
                    foreach (DataRow row in Keys.AppointmentRequestWaitListTables.ConsultWaitList.Rows)
                        if (row["CONSULTIEN"].ToString() == appointmentRequest.ConsultationRequestIen.ToString())
                        {
                            row.Delete();
                            break;
                        }
                }


                _unfilteredAdmittedPatients.Remove(appointmentRequest);
                AdmittedPatients.Remove(appointmentRequest);
                Keys.WaitListQueryTotalRecords--;
                PatientQueueView.PatientQueueRadGridView.ToolTip = "Viewing records 1 thru " + Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count + " of " + Keys.WaitListQueryTotalRecords;

                EventAggregator.GetEvent<CloseSchedulesEvent>().Publish(appointmentRequest.ClinicIen);
                EventAggregator.GetEvent<UnselectClinicEvent>().Publish(string.Empty);
                NotifyPropertyChanged("AdmittedPatients");
                NotifyPropertyChanged("UnfilteredAdmittedPatients");
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "BlurPatientSearch", ex);
            }
        }

        private AppointmentRequest _apptRequest;
        private string _selectedCause;
        private bool _userPrefsAreSet;

        public void RemovePtCSchRequest(AppointmentRequest appointmentRequest, string selectedCause)
        {
            _apptRequest = appointmentRequest;
            _selectedCause = selectedCause;
            var comment = string.Empty;

            if (Keys.PromptUser("PtCSch Comment", "Comment:", ref comment, 0, 0, 80, "PtCSchCommentInput"))
            {
                RemoveAppointmentRequest(_apptRequest);
                var parms = new List<string>(new[]
                {
                    _apptRequest.PtCSchRequestIen.ToString(),
                    _selectedCause, comment
                });
                var adt = OVIDDataTableRepository.Instance;
                var result = adt.callOVIDRPC("SDEC RECDSET", parms);
            }
        }

        public void ExecuteSchedulerDisplayCommand(SchdResource resource)
        {
            if (SelectedAppointmentRequest != null)
                if (SelectedAppointmentRequest.MultipleAppointmentsRequired && SelectedAppointmentRequest.IsMRTCParent)
                {
                    if (DataAccessService.IsProhibitedClinic(SelectedAppointmentRequest.ClinName, SelectedAppointmentRequest.ClinicIen))
                        return;

                    EventAggregator.GetEvent<LaunchFindApptDialogEvent>().Publish(SelectedResourceList);
                    return;
                }

            // The resource != null test prevents APPT Requests for Service\Speciality from attemting to launch a Clinic Schedule
            if (resource != null)
                EventAggregator.GetEvent<ResourceSelectedFromRmGrid>().Publish(resource);
        }

        internal void ShowPatientDemoInfoDialog(string patientIEN)
        {
            var selectedPatient = patientIEN;
            var parms = new List<string> { selectedPatient };
            _adt = OVIDDataTableRepository.Instance;
            var result = _adt.callOVIDRPC("SDEC PTINQ", parms);
            var prompt = "";
            if (result.Columns.Count > 0)
            {
                foreach (DataRow row in result.Rows)
                {
                    prompt += row.ItemArray[0] + Environment.NewLine;
                }

                // Display the patients information in a scrollable dialog to the user.
                var patientDemographics = new PatientDemographics
                {
                    PatientAlertInfo = prompt
                };
                Mouse.OverrideCursor = Cursors.Arrow;
                CLaunchWindow.LaunchWindow(patientDemographics, "Patient Inquiry Detail ", 600, 500, "PatientDemographics", true, true);
            }
        }

        public void CallPatientSelectedEvent(Patient selectedPatient)
        {
            EventAggregator.GetEvent<PatientSelectedEvent>().Publish(selectedPatient);
        }

        public void GetSensitiveRecordInfo(ObservableCollection<AppointmentRequest> appointmentRequests, WorkCompletedMethod workCompletedMethod)
        {
            IsAsync = true;
            DoWorkAsync((s, args) =>
           {
               args.Result = DataAccessService.GetSensitiveRecordInfo(appointmentRequests);
           }, workCompletedMethod);
        }

        #region HPE OLD CONTACT CODE (COMMENTED OUT)
        //internal void ContactPatient( string waitListIen, string selectedDateTime, string selectedPhoneNumber, string selectedAction, string userName, bool isAppt = false )
        //{
        //    var parms = new List<string> {selectedDateTime + "~~" + userName + "~~~~" + selectedAction + "~~" + selectedPhoneNumber, waitListIen};
        //    _adt = OVIDDataTableRepository.Instance;
        //    DataTable rslt;
        //    if ( !isAppt )
        //        rslt = _adt.callOVIDRPC( "SDEC WLPCSET", parms );
        //    else
        //        rslt = _adt.callOVIDRPC( "SDEC ARPCSET", parms );
        //    foreach ( DataRow row in rslt.Rows )
        //    {
        //        var errorId = row["RETURNCODE"].ToString();
        //        var errorText = row["TEXT"].ToString();

        //        if ( errorId != "0" )
        //        {
        //            if ( !string.IsNullOrEmpty( errorText ) )
        //            {
        //                MessageBox.Show( errorText );
        //                return;
        //            }
        //        }
        //    }
        //    UpdatePatientContacts( waitListIen, isAppt );
        //}

        //private void UpdatePatientContacts( string wlIen, bool isAppt )
        //{
        //    var updateRequired = false;
        //    try
        //    {
        //        var parms = new List<string> {wlIen};
        //        List<AppointmentRequest> appts;
        //        if ( isAppt )
        //        {
        //            var table = _adt.callOVIDRPC( Keys.RpcCalls.GetAppointmentRequestFileRequests, parms );
        //            appts = DataAccessService.GetApptAppointmentRequests( table );
        //            if ( appts == null )
        //                return;
        //            var appt = appts[0];
        //            foreach ( var aq in AdmittedPatients.Where( aq => aq.ApptRequestIen == Convert.ToInt32( wlIen ) ) )
        //            {
        //                aq.PatientContacts = appt.PatientContacts;
        //                updateRequired = true;
        //                break;
        //            }
        //        }
        //        else
        //        {
        //            var table = _adt.callOVIDRPC( Keys.RpcCalls.WaitListGet, parms );
        //            appts = DataAccessService.GetEwlAppointmentRequests( table );
        //            if ( appts == null )
        //                return;
        //            var appt = appts[0];
        //            foreach ( var aq in AdmittedPatients.Where( aq => aq.RequestManagmentIen == wlIen ) )
        //            {
        //                aq.PatientContacts = appt.PatientContacts;
        //                updateRequired = true;
        //                break;
        //            }
        //        }
        //        if ( updateRequired )
        //            NotifyPropertyChanged( "AdmittedPatients" );
        //    }
        //    catch ( Exception ex )
        //    {
        //        MessageBox.Show( TraceLog.FormatExceptionMessage( GetType().Name, "UpdatePatientContacts", ex ) );
        //    }
        //}
        #endregion

        #region VAR Mobile Request Members

        public void VAR_ButtonViewEventHandler(UserControl varButton)
        {
            View.InjectVARButton(varButton);
            EventAggregator.GetEvent<VAR_PatientLookupEvent>().Subscribe(VAR_PatientLookupEventHandler, ThreadOption.UIThread, false);
        }

        private void VAR_PatientLookupEventHandler(string patientSSN)
        {
            if (string.IsNullOrEmpty(patientSSN))
                return;

            var pdDate = string.Empty;

            if (patientSSN.Contains('^'))
            {
                var parts = patientSSN.Split('^');
                patientSSN = parts[0];
                pdDate = parts[1];
            }

            IList<Patient> pList = DataAccessService.GetPatients(patientSSN, 1);

            if (pList != null && (pList.Count > 0))
            {
                PatientList = pList;
                View.SelectPatient(PatientList[0], true, pdDate);
            }
            else
            {
                View.AlertUser("Patient Not Found.", "Mobile Request Patient Search");
                ClearPatient();
                //PatientUnselect("VARPatientSearch");
            }
        }

        #endregion VAR Mobile Request Members

        public bool MoveJasperReportDependenciesToUsersFolder()
        {
            try
            {
                var srcPath = Directory.GetCurrentDirectory();
                var targetPath = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
                targetPath += "\\" + Keys.UsersTempFolder;
                if (!Directory.Exists(targetPath))
                    Directory.CreateDirectory(targetPath);
                foreach (var fileName in Keys.JasperReportFiles.JasperFiles)
                {
                    var srcFile = Path.Combine(srcPath, fileName);
                    var destFile = Path.Combine(targetPath, fileName);
                    if (File.Exists(srcFile) && !File.Exists(destFile))
                    {
                        File.Copy(srcFile, destFile, true);
                        var fileInfo = new FileInfo(destFile)
                        {
                            IsReadOnly = false
                        };
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(TraceLog.FormatExceptionMessage(GetType().Name, "MoveJasperReportDependenciesToUsersFolder", ex));
                //MessageBox.Show(ex.Message);
                return false;
            }
            return true;
        }

        private void CloseFindAppointmentEventHandler(string wIEN)
        {
            //Get All Records for Selected Patient....
            if (Patient == null)
                return;

            GetWaitListRecordsForSelectedPatient(Patient.IEN, string.Empty, true);
        }

        private void UpdateUserPrefIconEventHandler(bool haveDefaults)
        {
            UserPrefsAreSet = AreUserPrefsSet();
        }

        private void ProcessUserPreferencesEventHandler(List<string> userPreferences)
        {
            ProcessUserPreferences(userPreferences);
        }

        private void DisplayFileVersionsEventHandler(string title)
        {
            DisplayFileVersions(title);
        }

        public UserPreferences UserPreferences { get; set; }

        public bool UserPrefsAreSet
        {
            get { return _userPrefsAreSet; }
            set
            {
                _userPrefsAreSet = value;
                BorderColor = value ? Brushes.LimeGreen : Brushes.Transparent;
            }
        }
        private SolidColorBrush _borderColor = Brushes.Transparent;
        private bool _searchFailed;

        public SolidColorBrush BorderColor
        {
            get { return _borderColor; }
            set
            {
                _borderColor = value;
                NotifyPropertyChanged("BorderColor");
            }
        }

        public ICommand UserPreferencesCommand
        {
            get
            {
                return GetCommand(
                    "UserPreferencesCommand",
                    _ =>
                    {
                        try
                        {
                            var vm = new UserPreferencesVM(DataAccessService)  // PatientQueueView.PatientQueueRadGridView 
                            {
                                Title = "User Preferences",
                            };

                            var dlg = new UserPreferencesV
                            {
                                Context = vm
                            };
                            vm.View = dlg;
                            vm.CloseCallback = userPreferences =>
                            {
                                dlg.Close();

                                if (userPreferences != null)
                                {
                                    // Save the Settings
                                    if (vm.SaveAsDefaultView)
                                    {
                                        DataAccessService.SetUserPreference(Keys.LoggedInUserIen, userPreferences);
                                        UserPrefsAreSet = AreUserPrefsSet();
                                    }

                                    ProcessUserPreferences(userPreferences);

                                    // Get the data based on the user preferences.
                                    Keys.WaitListQueryLastRec = "";
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Clear();
                                    ClearPatient();
                                    DoRefreshWaitList("");
                                }
                            };
                            dlg.ShowDialog();
                        }
                        catch (Exception ex)
                        {
                            MessageBox.Show(ex.Message + "\n\n" + ex.StackTrace, "UserPreferencesCommand");
                        }
                    },
                    _ => true);
            }
        }

        public void ClearPatient()
        {
            Mouse.OverrideCursor = Cursors.Wait;

            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Clear();
            Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
            Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
            Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
            Keys.QueryFormCurrentFilter = string.Empty;
            Keys.WaitListQueryLastRec = null;
            Keys.SelectedPatient = null;
            Keys.PatientIen_SensitiveRecordCheck = null;
            EventAggregator.GetEvent<PatientUnselectEvent>().Publish("");
            try
            {
                Keys.BooleanFlags.ClearButtonPressed = true;
                IList<Patient> emptyList = new List<Patient>();
                PatientQueueView.IsRefreshOrClear = true;
                PatientQueueView.UpdateWaitTimeValue(Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90);
                EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("ClearPatient");
                EventAggregator.GetEvent<UnselectClinicEvent>().Publish("CLEAR");
                try
                {
                    PatientQueueView.PatientQueueRadGridView.SelectedItem = null;
                    Patient = null;
                    SelectedAppointmentRequest = null;
                    Keys.SelectedPatient = null;
                    View2.PatientSearchComboBox.Text = string.Empty;
                    PatientList = emptyList;
                    if (!Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
                        DoRefreshWaitList("");
                }
                finally
                {
                    Keys.BooleanFlags.ClearButtonPressed = false;
                    PatientQueueView.IsRefreshOrClear = false;
                }

                View2.PatientSearchComboBox.Focus();
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        public bool AreUserPrefsSet()
        {
            //            if (Keys.UserPreferences == null)
            //                return false;

            //            return Keys.UserPreferences.AreDefined;

            var col = 0;

            if (Keys.UserPrefList != null)
                foreach (var userPref in Keys.UserPrefList)
                {
                    if (!userPref.Contains(_defaultColumnOrder[col]))
                        return true;
                    if (FilterInfo(userPref).Length > 0)
                        return true;
                    if (SortOrder(userPref) > 0)
                        return true;
                    col++;
                }

            return false;
        }

        private readonly List<string> _defaultColumnOrder = Keys.RMGridDefaultColumnOrder.ToList();
        private RPMSClinic _clinic;
        private bool _arePendingAppointmentsVisible;

        private void ProcessUserPreferences(List<string> userPreferences)
        {
            if (!BuildUserPreferences(userPreferences))
                return;

            if (userPreferences.Count == 0)
                return;

            if (Keys.UserPreferences.RequestType != null)
                ReorderColumns();
        }

        private void DisplayFileVersions(string title)
        {
            GetAssemblies();
        }

        private void GetAssemblies()
        {
            List<string> refs = new List<string>();
            Queue<AssemblyName> pending = new Queue<AssemblyName>();
            pending.Enqueue(Assembly.GetEntryAssembly().GetName());
            while (pending.Count > 0)
            {
                AssemblyName an = pending.Dequeue();
                string s = an.ToString();
                if (refs.Contains(s))
                    continue; // done already
                refs.Add(s);
                try
                {
                    Assembly asm = Assembly.Load(an);
                    if (asm != null)
                    {
                        foreach (AssemblyName sub in asm.GetReferencedAssemblies())
                        {
                            pending.Enqueue(sub);
                        }
                        foreach (Type type in asm.GetTypes())
                        {
                            foreach (MethodInfo method in type.GetMethods(
                                BindingFlags.Static | BindingFlags.Public |
                                     BindingFlags.NonPublic))
                            {
                                DllImportAttribute attrib = (DllImportAttribute)
                                    Attribute.GetCustomAttribute(method,
                                        typeof(DllImportAttribute));
                                if (attrib != null && !refs.Contains(attrib.Value))
                                {
                                    refs.Add(attrib.Value);
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    Console.Error.WriteLine(ex.Message);
                }
            }
            refs.Sort();
            FileVersions = new ObservableCollection<FileVersion>();
            foreach (string name in refs)
            {
                if (name.Contains("ClinSchd") || name.Contains("VSE") || name.Contains("Medsphere") || name.Contains("SDEC") || name.Contains("OVID"))
                {
                    var fv = new FileVersion();
                    fv.File = name.Piece(",", 1);
                    fv.Version = name.Piece(",", 2).Piece("=", 2);
                }
            }
            //MessageBox.Show( files, "Component Files" );
        }

        public ObservableCollection<FileVersion> FileVersions { get; set; }

        public bool PendingAppointmentsRightClick { get; set; }

        private void ReorderColumns()
        {
            if (Keys.UserPreferences.RequestType != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.RequestType.ColumnName].DisplayIndex = Keys.UserPreferences.RequestType.Column;

            if (Keys.UserPreferences.WaitTime != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.WaitTime.ColumnName].DisplayIndex = Keys.UserPreferences.WaitTime.Column;

            if (Keys.UserPreferences.PatientName != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.PatientName.ColumnName].DisplayIndex = Keys.UserPreferences.PatientName.Column;

            if (Keys.UserPreferences.SCVisit != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.SCVisit.ColumnName].DisplayIndex = Keys.UserPreferences.SCVisit.Column;

            if (Keys.UserPreferences.ClinicService != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.ClinicService.ColumnName].DisplayIndex = Keys.UserPreferences.ClinicService.Column;

            if (Keys.UserPreferences.Comment != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.Comment.ColumnName].DisplayIndex = Keys.UserPreferences.Comment.Column;

            if (Keys.UserPreferences.RequestedBy != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.RequestedBy.ColumnName].DisplayIndex = Keys.UserPreferences.RequestedBy.Column;

            if (Keys.UserPreferences.Requestor != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.Requestor.ColumnName].DisplayIndex = Keys.UserPreferences.Requestor.Column;

            if (Keys.UserPreferences.PriorityGroup != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.PriorityGroup.ColumnName].DisplayIndex = Keys.UserPreferences.PriorityGroup.Column;

            if (Keys.UserPreferences.PreferredDate != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.PreferredDate.ColumnName].DisplayIndex = Keys.UserPreferences.PreferredDate.Column;

            if (Keys.UserPreferences.EnteredDate != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.EnteredDate.ColumnName].DisplayIndex = Keys.UserPreferences.EnteredDate.Column;

            if (Keys.UserPreferences.SSN != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.SSN.ColumnName].DisplayIndex = Keys.UserPreferences.SSN.Column;

            if (Keys.UserPreferences.Telephone != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.Telephone.ColumnName].DisplayIndex = Keys.UserPreferences.Telephone.Column;

            if (Keys.UserPreferences.MRTC != null)
                PatientQueueView.PatientQueueRadGridView.Columns[Keys.UserPreferences.MRTC.ColumnName].DisplayIndex = Keys.UserPreferences.MRTC.Column;
        }

        private const string DLIM = ";";

        public bool BuildUserPreferences(List<string> userPreferences)
        {
            FilterInfo requestType = new FilterInfo(0, _defaultColumnOrder[0], false, string.Empty, false, 0);
            FilterInfo waitTime = new FilterInfo(1, _defaultColumnOrder[1], false, string.Empty, false, 0);
            FilterInfo patientName = new FilterInfo(2, _defaultColumnOrder[2], false, string.Empty, false, 0);
            FilterInfo mrtc = new FilterInfo(3, _defaultColumnOrder[3], false, string.Empty, false, 0);
            FilterInfo ssn = new FilterInfo(4, _defaultColumnOrder[4], false, string.Empty, false, 0);
            FilterInfo scVisit = new FilterInfo(5, _defaultColumnOrder[5], false, string.Empty, false, 0);
            FilterInfo telephone = new FilterInfo(6, _defaultColumnOrder[6], false, string.Empty, false, 0);
            FilterInfo priorityGroup = new FilterInfo(7, _defaultColumnOrder[7], false, string.Empty, false, 0);
            FilterInfo preferredDate = new FilterInfo(8, _defaultColumnOrder[8], false, string.Empty, false, 0);
            FilterInfo enteredDate = new FilterInfo(9, _defaultColumnOrder[9], false, string.Empty, false, 0);
            FilterInfo requestor = new FilterInfo(10, _defaultColumnOrder[10], false, string.Empty, false, 0);
            FilterInfo requestedBy = new FilterInfo(11, _defaultColumnOrder[11], false, string.Empty, false, 0);
            FilterInfo clinicService = new FilterInfo(12, _defaultColumnOrder[12], false, string.Empty, false, 0);
            FilterInfo comment = new FilterInfo(13, _defaultColumnOrder[13], false, string.Empty, false, 0);
            bool areDefined = false;

            if (userPreferences?.Count == 0)
                return false;

            foreach (var userPref in userPreferences)
            {

                var columnNum = ColumnNumber(userPref); // Column #
                var columnName = ColumnName(userPref); // ColumnName
                var isFiltered = IsFiltered(userPref); // IsFiltered
                var filterInfo = FilterInfo(userPref); // Filter info
                var isServiceList = IsServiceList(userPref); // Is ServiceList no ClinicList
                var sortOrder = SortOrder(userPref); // Sort Order

                if (!areDefined)
                    areDefined = filterInfo != "" || sortOrder > 0;

                var col = new FilterInfo(columnNum, columnName, isFiltered, filterInfo, isServiceList, sortOrder);

                switch (col.ColumnName.ToUpper())
                {
                    case "0":
                    case "REQUEST TYPE":
                    case "REQUESTTYPE":
                        col.ColumnName = _defaultColumnOrder[0];
                        requestType = col;
                        break;
                    case "1":
                    case "WAIT TIME":
                    case "WAITTIME":
                        col.ColumnName = _defaultColumnOrder[1];
                        waitTime = col;
                        break;
                    case "2":
                    case "PT NAME":
                    case "PTNAME":
                        col.ColumnName = _defaultColumnOrder[2];
                        patientName = col;
                        break;
                    case "3":
                    case "MRTC":
                        col.ColumnName = _defaultColumnOrder[3];
                        mrtc = col;
                        break;
                    case "4":
                    case "SSN":
                        col.ColumnName = _defaultColumnOrder[4];
                        ssn = col;
                        break;
                    case "5":
                    case "SCVISIT":
                        col.ColumnName = _defaultColumnOrder[5];
                        scVisit = col;
                        break;
                    case "6":
                    case "TELEPHONE":
                        col.ColumnName = _defaultColumnOrder[6];
                        telephone = col;
                        break;
                    case "7":
                    case "PRIORITY":
                        col.ColumnName = _defaultColumnOrder[7];
                        priorityGroup = col;
                        break;
                    case "8":
                    case "CID / PREFERRED DATE":
                    case "CID/PREFERRED DATE":
                        col.ColumnName = _defaultColumnOrder[8];
                        preferredDate = col;
                        break;
                    case "9":
                    case "ENTERED / RR NO DATE":
                    case "ENTERED/RR NO DATE":
                        col.ColumnName = _defaultColumnOrder[9];
                        enteredDate = col;
                        break;
                    case "10":
                    case "REQUESTOR":
                        col.ColumnName = _defaultColumnOrder[10];
                        requestor = col;
                        break;
                    case "11":
                    case "REQUESTEDBY":
                    case "REQUESTED BY":
                        col.ColumnName = _defaultColumnOrder[11];
                        requestedBy = col;
                        break;
                    case "12":
                    case "CLINIC / SERVICE":
                    case "CLINIC/SERVICE":
                        col.ColumnName = _defaultColumnOrder[12];
                        clinicService = col;
                        break;
                    case "13":
                    case "COMMENT":
                        col.ColumnName = _defaultColumnOrder[13];
                        comment = col;
                        break;
                    default:
                        MessageBox.Show("Unknown Column Name " + col.ColumnName, "BuildUserPreferences");
                        break;
                }
            }

            Keys.UserPreferences = new UserPreferences(
                requestType,
                waitTime,
                patientName,
                ssn,
                mrtc,
                scVisit,
                telephone,
                priorityGroup,
                preferredDate,
                enteredDate,
                requestor,
                requestedBy,
                clinicService,
                comment)
            {
                AreDefined = areDefined
            };


            return true; // isReordered;
        }

        // Columnn    C ; col# ; columnName
        public int ColumnNumber(string line)
        {
            return line.Piece(":", 1).Piece(";", 2).Trim().ToIntDef(0);
        }

        public string ColumnName(string line)
        {
            return line.Piece(":", 1).Piece(";", 3).Trim();
        }

        // Filter    F ; 1/0 ; filterinfo ; isServiceList
        public bool IsFiltered(string line)
        {
            return line.Piece(":", 2).Piece(";", 2).Trim() == "1";
        }

        public string FilterInfo(string line)
        {
            return line.Piece(":", 2).Piece(";", 3).Trim();
        }

        public bool IsServiceList(string line)
        {
            return line.Piece(":", 2).Piece(";", 4).Trim() == "1";
        }

        // Sort Order   S ; 0,1,2 or 3
        public int SortOrder(string line)
        {
            return line.Piece(":", 3).Piece(";", 2).Trim().ToIntDef(0);
        }

        private void SchedulerViewModeChanged(string viewMode)
        {
            View.ChangeViewMode(viewMode);
        }
    }
}