using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.Reports.Reports;
using ClinSchd.Modules.Ribbon.Views;
using Telerik.Windows.Controls;
using ClinSchd.Infrastructure.Models;
using System.Windows.Input;
using System.Printing;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.GridView;
using KeyEventArgs = System.Windows.Input.KeyEventArgs;
using MessageBox = System.Windows.MessageBox;
using UserControl = System.Windows.Controls.UserControl;
using System.Windows.Media;
using ClinSchd.Infrastructure.Interfaces;
using HorizontalAlignment = System.Windows.HorizontalAlignment;
using Keys = ClinSchd.Infrastructure.Keys;
using ClinSchd.Modules.DataAccess.Views;

namespace ClinSchd.Modules.Ribbon.Schedule
{
    /// <summary>
    /// Interaction logic for ScheduleView.xaml
    /// </summary>
    public partial class ScheduleView : IScheduleView
    {
        private class tabControl
        {
            public Control TargetControl { get; private set; } = null;

            public bool TargetMustBeEnabled { get; private set; } = true;

            public bool TargetMustBeVisible { get; private set; } = true;

            public Control IsVisibileControl { get; private set; } = null;

            public Control HasItemsControl { get; private set; } = null;

            public tabControl(Control targetControl, Control isVisibileControl = null, Control hasItemsControl = null, bool targetMustBeEnabled = true, bool targetMustBeVisible = true)
            {
                TargetControl = targetControl;
                TargetMustBeEnabled = targetMustBeEnabled;
                TargetMustBeVisible = targetMustBeVisible;
                IsVisibileControl = isVisibileControl;
                HasItemsControl = hasItemsControl;
            }

            public bool CanTabTo
            {
                get
                {
                    if (TargetMustBeEnabled && !TargetControl.IsEnabled)
                        return false;

                    if (TargetMustBeVisible && (TargetControl.Visibility != Visibility.Visible))
                        return false;

                    if (IsVisibileControl != null)
                        if (IsVisibileControl.Visibility != Visibility.Visible)
                            return false;

                    if (HasItemsControl != null)
                    {
                        if (HasItemsControl is ListBox)
                            if (!((ListBox) HasItemsControl).HasItems)
                                return false;

                        if (HasItemsControl is ListView)
                            if (!((ListView) HasItemsControl).HasItems)
                                return false;

                        if (HasItemsControl is ComboBox)
                            if (!((ComboBox) HasItemsControl).HasItems)
                                return false;

                        if (HasItemsControl is RadComboBox)
                            if (!((RadComboBox)HasItemsControl).HasItems)
                                return false;
                    }

                    return true;
                }
            }
        }

        private Dictionary<string, Dictionary<int, tabControl>> _tabKeyController = new Dictionary<string, Dictionary<int, tabControl>>();

        private void AddControlToTabOrder(Control control, string ribbonBarTabName, Control isVisibileControl = null, Control hasItemsControl = null, bool targetMustBeEnabled = true, bool targetMustBeVisible = true)
        {
            // A RibbonBar Tab MUST be defined
            if (string.IsNullOrEmpty(ribbonBarTabName))
                return;

            // Control MUST exist
            if (control == null)
                return;

            // Control MUST be Focasable
            if (!control.Focusable)
                return;

            // Control MUST be a TabStop
            if (!control.IsTabStop)
                return;

            // Control MUST have a TabIndex > 0
            if (control.TabIndex < 0)
                return;

            // Create a new tabControl object
            tabControl newControl = new tabControl(control, isVisibileControl, hasItemsControl, targetMustBeEnabled, targetMustBeVisible);
            // this will be the working TabKeyController dictionary for the passed in ribbonBarTabName
            Dictionary<int, tabControl> ribbonTabKeyController = null;

            // Is there already a TabKeyController for the current RibbonBar Name ?
            if (_tabKeyController.Keys.Contains(ribbonBarTabName))
                // Yes, let's use that one
                ribbonTabKeyController = _tabKeyController[ribbonBarTabName];
            else
            {
                // No, create a new one and add it to the _tabKeyController
                ribbonTabKeyController = new Dictionary<int, tabControl>();
                _tabKeyController.Add(ribbonBarTabName, ribbonTabKeyController);
            }

            // Does the working controller have the current controller TabIndex ?
            if (ribbonTabKeyController.Keys.Contains(control.TabIndex))
                // Yes, so it needs to be updated.
                ribbonTabKeyController[control.TabIndex] = newControl;
            else
                // No, so add the controller to the current tab key dictionary
                ribbonTabKeyController.Add(control.TabIndex, newControl);
        }

        private void RemoveControlFromTabOrder(Control control, string ribbonBarTabName)
        {
            // A RibbonBar Tab MUST be defined
            if (string.IsNullOrEmpty(ribbonBarTabName))
                return;

            // Control MUST exist
            if (control == null)
                return;

            // Control MUST have a TabIndex > 0
            if (control.TabIndex < 0)
                return;

            // Get the tab controler for the passed in ribbonBar
            Dictionary<int, tabControl> ribbonTabKeyController = _tabKeyController[ribbonBarTabName];

            // If no TabKeyController was found we are done
            if (ribbonTabKeyController == null)
                return;

            // See if there is a control for the current Control TabIndex
            if (ribbonTabKeyController.Keys.Contains(control.TabIndex))
                // if so, remove it
                ribbonTabKeyController.Remove(control.TabIndex);
        }

        private void InitializeTabOrder()
        {
            AddControlToTabOrder(PatientSearchComboBox, "TasksTab");
            AddControlToTabOrder(BtnSearch, "TasksTab");
            AddControlToTabOrder(BtnClear, "TasksTab");
            AddControlToTabOrder(BtnNewReq, "TasksTab");
            AddControlToTabOrder(RadCombox3, "TasksTab");
            AddControlToTabOrder(LbFutureAppointmentsView, "TasksTab", null, LbFutureAppointmentsView);
            AddControlToTabOrder(LbSpecialNeeds, "TasksTab", null, LbSpecialNeeds);
            AddControlToTabOrder(PrintRequestGridBtn, "TasksTab");
            AddControlToTabOrder(ExportRequestGridBtn, "TasksTab");
            AddControlToTabOrder(RefreshRMGrid, "TasksTab");
            AddControlToTabOrder(QueryRMGrid, "TasksTab");
            AddControlToTabOrder(BtnUserPrefs, "TasksTab", RibGrpUserPrefs);
            AddControlToTabOrder(BtnSchedMgmt, "SystemTab");
            AddControlToTabOrder(BtnAvailability, "SystemTab");
            AddControlToTabOrder(BtnAuditActivity, "ReportsTab");
            AddControlToTabOrder(BtnClinics, "ReportsTab");
            AddControlToTabOrder(BtnMgmtReports, "ReportsTab");
        }

        public string PatientSearchString { get; set; }

        private IDataAccessService _dataAccessService;

        public ScheduleView( IDataAccessService dataAccessService )
        {
            try
            {
                InitializeComponent();
                _dataAccessService = dataAccessService;
            }
            catch ( Exception ex )
            {
                MessageBox.Show( ex.Message + "\n\n" + ex.StackTrace );
            }
        }

        public SchedulePresentationModel Model
        {
            get { return DataContext as SchedulePresentationModel; }
            set { DataContext = value; }
        }

        public void UpdatePatientSearchText(string patientName)
        {
            if (string.IsNullOrEmpty(patientName))
            {
                if(Model.Patient == null)
                {
                    LblCity.Content                       = string.Empty;
                    LblPatientType.Content                = string.Empty;
                    LblDOB.Content                        = string.Empty;
                    LblGender.Content                     = string.Empty;
                    LblPatientName.Content                = string.Empty;
                    LblPatientStreet.Content              = string.Empty;
                    LblServiceConnected.Content           = string.Empty;
                    LblServiceConnectedPercentage.Content = string.Empty;
                    LblState.Content                      = string.Empty;
                    LbFutureAppointmentsView.Items.Clear();
                    LbSpecialNeeds.Items.Clear();
                    UpdatePatientFlags(null);
                    PatientSearchComboBox.Text            = string.Empty;
                }
            }
            else
            {
                PatientSearchComboBox.Text            = patientName;
                LblPatientType.Content                = Model.SelectedAppointmentRequest.Patient.TypeName;
                LblDOB.Content                        = Model.SelectedAppointmentRequest.Patient.DOBFormatted;
                LblGender.Content                     = Model.SelectedAppointmentRequest.Patient.Gender;
                LblPatientName.Content                = Model.SelectedAppointmentRequest.Patient.Name;
                LblPatientStreet.Content              = Model.SelectedAppointmentRequest.Patient.PrimaryContactInfo.Street1;
                LblServiceConnected.Content           = Model.SelectedAppointmentRequest.Patient.ServiceConnection;
                LblServiceConnectedPercentage.Content = Model.SelectedAppointmentRequest.Patient.ServiceConnectionPercentage;
                LblCity.Content                       = Model.SelectedAppointmentRequest.Patient.PrimaryContactInfo.CityState;
                LblState.Content                      = Model.SelectedAppointmentRequest.Patient.Ward;
            }
        }

        public void UpdatePatientDemoInfoRibbon(Patient patient)
        {
            PatientSearchComboBox.Text            = patient.Name;
            LblPatientType.Content                = patient.TypeName;
            LblDOB.Content                        = patient.DOBFormatted;
            LblGender.Content                     = patient.Gender;
            LblPatientName.Content                = patient.Name;
            LblPatientStreet.Content              = patient.PrimaryContactInfo.Street1;
            LblCity.Content                       = patient.PrimaryContactInfo.CityState;
            LblServiceConnected.Content           = patient.ServiceConnection;
            LblState.Content                      = patient.Ward;
            LblServiceConnectedPercentage.Content = patient.ServiceConnectionPercentage;
        }

        public void UpdateSpecialNeedsRibbon(List<string> preferences)
        {
            LbSpecialNeeds.Items.Clear();
            if (preferences == null)
                return;
            foreach (var pref in preferences)
                LbSpecialNeeds.Items.Add(pref);
        }

        public void UpdateFutureAppointments(ObservableCollection<FuturePatientAppointment> futureAppointments)
        {
            LbFutureAppointmentsView.Items.Clear();

            if (futureAppointments == null)
                return;

            FuturePatientAppointment topAppt = null; //appt to scroll to
            var topApptIdx = -1;

            foreach (var appt in futureAppointments)
            {
                LbFutureAppointmentsView.Items.Add(appt);

                if (topAppt == null)
                {
                    topApptIdx++;
                    
                    if (appt.AppointmentDateTime >= DateTime.Today)
                        topAppt = appt;
                }
            }

            if (topApptIdx >= 0)
            {
                var pendApptListScrollView = FindScrollViewer(LbFutureAppointmentsView);

                if (pendApptListScrollView != null)					
                    pendApptListScrollView.ScrollToVerticalOffset(topApptIdx);
            }
        }

        private ScrollViewer FindScrollViewer(DependencyObject d)
        {
            if (d is ScrollViewer)
                return d as ScrollViewer;

            for (var i = 0; i < VisualTreeHelper.GetChildrenCount(d); i++) {
                var sv = FindScrollViewer(VisualTreeHelper.GetChild(d, i));
                if (sv != null)
                    return sv;
            }
            return null;
        }

        private void TimeScale_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            string timeScale;
            switch (((RadComboBox)sender).SelectedIndex)
            {
                case 1:
                    timeScale = "60";
                    break;
                case 3:
                    timeScale = "20";
                    break;
                case 4:
                    timeScale = "15";
                    break;
                case 6:
                    timeScale = "10";
                    break;
                case 7:
                    timeScale = "5";
                    break;
                default:
                    timeScale = "30";
                    break;
            }
            Model.ExecuteViewTimeScaleCommand(timeScale);
        }

        public void OpenNewRequestForPatient(Patient patient)
        {
            PatientChanged(patient);
        }

        public void OpenNewRequestForPatient(Patient patient, DateTime desiredDateTime, string resourceIEN, string resourceName, string requestType = Keys.AppointmentRequestTypes.Appt)
        {
            PatientChanged(patient, desiredDateTime, resourceIEN, resourceName, requestType);
        }

        public void PatientChanged(Patient patient, DateTime? desiredDateTime = null, string resourceIEN = "", string resourceName = "", string requestType = "", bool loadingFromSchedule = false)
        {
            try
            {
                bool requestedPatientRequests = false;
                IList<Patient> emptyList = new List<Patient>();

                if (Model.PatientList != null)
                    Model.PatientList.Clear();

                var das = _dataAccessService;

                if (!Model.ArePendingAppointmentsVisible)
                {
                    Model.View.FocusPatientSearch();
                    Model.ArePendingAppointmentsVisible = true;
                }

                if (patient != null)
                {
                    if (string.IsNullOrEmpty(patient.Name))
                    {
                        MessageBox.Show("Please select a Patient");
                        return;
                    }

                    Mouse.OverrideCursor = Cursors.Wait;
                    var patients = _dataAccessService.GetPatients("#" + patient.IEN, 1);

                    foreach (var pat in patients)
                    {
                        patient = pat;
                        break;
                    }

                    // Get missing Patient Information
                    patient = _dataAccessService.GetPatient(ref patient);

                    if (Model.Patient == null || Model.Patient.IEN != patient.IEN)
                    {
                        //Patient Record Checks.................................................
                        var prompts = das.GetPatientSelectionPrompts(patient.IEN, ref patient);
                        if (prompts == "0")
                        {
                            Model.PatientList = emptyList;
                            PatientSearchString = "";
                            PatientSearchComboBox.Text = "";
                            Mouse.OverrideCursor = Cursors.Arrow;
                            return;
                        }
                        UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                        //Model.EventAggregator.GetEvent<PatientSelectedEvent>().Publish(patient);
                    }

                    //////////////////////////////BEGIN UPDATE PATIENT RECORDS////////////////////////////////////////////
                    Model.PatientQueueView.UpdateWaitTimeValue(Keys.RmGridColumns.WaitTimeAll);
                    Model.Patient = patient;
                    Keys.SelectedPatient = patient;
                    Model.PatientList = emptyList;
                    Model.CallPatientSelectedEvent(patient);

                    if (!loadingFromSchedule)
                        Model.EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("PatientChanged");

                    UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                    //////////////////////END Update Patient Records...///////////////////////////////////////////

                    var appReqTypeViewNoneChecked = false;
                    var appReqTypeViewAppointmentRequestType = requestType;
                    
                    Model.AdmittedPatients.Clear();
                    bool showNewRequestDialog = false;

                    // Cannot show Request Type selection if loading from a schedule selection, in View Only mode, or the user does not have the SDWLMenu key
                    if (string.IsNullOrEmpty(appReqTypeViewAppointmentRequestType) && !loadingFromSchedule && !Keys.LoggedInUserSDECViewOnlyKey)
                    {
                        //Show the AppointmentRequest Type dialog first - to distinguish between EWL and PtCSch appointment request types..
                        Model.AppointmetRequestTypeViewNoneChecked = false;
                        var appReqTypeView = new AppointmentRequestTypeView
                        {
                            IsDead = patient.IsDead
                        };
                        Mouse.OverrideCursor = Cursors.Arrow;
                        CLaunchWindow.LaunchWindow(appReqTypeView, "Request Type", 140, 275);
                        Mouse.OverrideCursor = Cursors.Wait;

                        if (appReqTypeView.DialogResult == false)
                        {
                            UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                            Model.PatientList = emptyList;
                            Model.PatientList.Add(patient);
                            PatientSearchString = patient.Name;
                            PatientSearchComboBox.Text = patient.Name;
                        }
                        else
                            showNewRequestDialog = true;
                        
                        appReqTypeViewAppointmentRequestType = appReqTypeView.AppointmentRequestType;
                        appReqTypeViewNoneChecked = appReqTypeView.NoneChecked;
                    }

                    if (showNewRequestDialog)
                    {
                        Model.AppointmetRequestTypeViewNoneChecked = appReqTypeViewNoneChecked;
                        if (appReqTypeViewNoneChecked != true)
                        {
                            var isPtCSch = appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch;

                            var requestManagement = new RequestManagement(Model.DataAccessService, Model.EventAggregator)
                            {
                                AppointmentRequest =
                            {
                                Patient = patient
                            }
                            };
                            if (appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.Appt)
                            {
                                requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Appt;
                                requestManagement.AppointmentRequest.IsVeteran = _isVARRequest;
                            }
                            else if (appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.Ewl)
                                requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Ewl;
                            else if (appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch)
                                requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.PtCSch;
                            else if (appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.Veteran)
                                requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Veteran;
                            else if (appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.ReturnToClinic)
                                requestManagement.AppointmentRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.ReturnToClinic;

                            if (requestManagement.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch)
                            {
                                // requestManagement.AppointmentRequest.PtCSchProviders = das.GetPtCSchProviders();
                                requestManagement.AppointmentRequest.PtCSchApptTypes = das.GetPtCSchAppointmentTypes();
                            }

                            requestManagement.AppointmentRequest.Patient.ServiceRelated = requestManagement.AppointmentRequest.Patient.ServiceConnectionPercent >= 50;

                            //Get Patients SpecialNeeds/Preferences....................................................
                            GetPatientPreferences(requestManagement.AppointmentRequest.Patient);

                            if (desiredDateTime.HasValue)
                                requestManagement.AppointmentRequest.DesiredApptDate = desiredDateTime;

                            if (!string.IsNullOrEmpty(resourceName))
                            {
                                requestManagement.AppointmentRequest.ClinName = resourceName;
                                requestManagement.AppointmentRequest.RequestedByText = "PATIENT";
                                requestManagement.AppointmentRequest.EnrolleeStatus = "ESTABLISHED";
                            }

                            // Set VAR Request Values
                            if (appReqTypeViewAppointmentRequestType == Keys.AppointmentRequestTypes.Appt && _isVARRequest)
                            {
                                requestManagement.AppointmentRequest.RequestedByText = "PATIENT";

                                if (!string.IsNullOrEmpty(_varPDDate))
                                {
                                    var varPDDate = _varPDDate;
                                    var varPDTimeOfDay = string.Empty;

                                    if (_varPDDate.Contains(' '))
                                    {
                                        var parts = _varPDDate.Split(' ');
                                        varPDDate = parts[0];
                                        varPDTimeOfDay = parts[1];
                                    }

                                    if (!string.IsNullOrEmpty(varPDDate))
                                    {
                                        DateTime desiredDate;

                                        if (DateTime.TryParse(varPDDate, out desiredDate))
                                            requestManagement.AppointmentRequest.DesiredApptDate = desiredDate;
                                    }

                                    if (!string.IsNullOrEmpty(varPDTimeOfDay))
                                    {
                                        if (varPDTimeOfDay == "AM")
                                        {
                                            requestManagement.AppointmentRequest.Patient.SpecialNeedsPreferences.Am = true;
                                            requestManagement.AppointmentRequest.Patient.SpecialNeedsPreferences.Pm = false;
                                        }
                                        else if (varPDTimeOfDay == "PM")
                                        {
                                            requestManagement.AppointmentRequest.Patient.SpecialNeedsPreferences.Am = false;
                                            requestManagement.AppointmentRequest.Patient.SpecialNeedsPreferences.Pm = true;
                                        }
                                        else
                                        {
                                            requestManagement.AppointmentRequest.Patient.SpecialNeedsPreferences.Am = false;
                                            requestManagement.AppointmentRequest.Patient.SpecialNeedsPreferences.Pm = false;
                                        }
                                    }
                                }
                            }

                            Mouse.OverrideCursor = Cursors.Arrow;
                            var height = requestManagement.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch ? 575 : 625;
                            CLaunchWindow.LaunchWindow(requestManagement, "Appointment Request for " + requestManagement.AppointmentRequest.Patient.Name, height, 700);
                            System.Windows.Forms.Application.DoEvents();
                            Mouse.OverrideCursor = Cursors.Wait;

                            if (requestManagement.DialogResult == false)
                            {
                                UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                                Model.PatientList = emptyList;
                                Model.PatientList.Add(patient);
                                PatientSearchString = patient.Name;
                                PatientSearchComboBox.Text = patient.Name;
                            }
                            else
                            {
                                Keys.BooleanFlags.RefreshWaitList = false;
                                var newRequest = requestManagement.AppointmentRequest;
                                Model.GetWaitListRecordsForSelectedPatient(Keys.SelectedPatient.IEN, "", true, true, newRequest.AppointmentRequestType, string.Empty, newRequest);
                                requestedPatientRequests = true;
                                requestManagement.AppointmentRequest.DesiredApptDate = requestManagement.OriginalDesiredDate;
                                Keys.SelectedPatientAppointmentRequest = newRequest;
                            }
                        }

                        // This will update the current patient information in the event the patient data was edited during the new request process.
                        patient = _dataAccessService.GetPatient(ref patient);
                    }

                   if (!requestedPatientRequests)
                        Model.GetWaitListRecordsForSelectedPatient(patient.IEN, string.Empty, true);

                    Model.Patient = patient;
                    Keys.SelectedPatient = patient;
                    Model.PatientList = emptyList;
                    UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(patient);
                    Mouse.OverrideCursor = Cursors.Arrow;
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private void GetPatientPreferences(Patient patient)
        {
            var dasResult = _dataAccessService.GetPatientSpecialNeedsPreferences(patient.IEN);
            patient.SpecialNeedsPreferences = new SpecialNeedsPreferences();
            patient.Preferences.Clear();
            var remarks = new List<string>();

            foreach (DataRow dr in dasResult.Rows)
            {
                if (dasResult.Columns.Contains("RETURNCODE"))
                    continue;

                var remark                                            = dr["REMARKS"].ToString();
                patient.SpecialNeedsPreferences.Remarks               = remark;
                patient.SpecialNeedsPreferences.DateAdded             = dr["DATE_ADDED"].ToString();
                patient.SpecialNeedsPreferences.AddedByUserIen        = dr["ADDED_BY_USER_IEN"].ToString();
                patient.SpecialNeedsPreferences.AddedByUserName       = dr["ADDED_BY_USER_NAME"].ToString();
                patient.SpecialNeedsPreferences.DateInactivated       = dr["DATE_INACTIVATED"].ToString();
                patient.SpecialNeedsPreferences.InactivatedByUserIen  = dr["INACTIVATED_BY_USER_IEN"].ToString();
                patient.SpecialNeedsPreferences.InactivatedByUserName = dr["INACTIVATED_BY_USER_NAME"].ToString();

                if (!string.IsNullOrEmpty(remark))
                    if (!remarks.Contains(remark))
                        remarks.Add(remark);

                patient.Preferences.Add(dr["PREF_TEXT"].ToString());
                var prefText = dr["PREF_TEXT"].ToString();
                switch (prefText)
                {
                    case "ON STRETCHER":
                        patient.SpecialNeedsPreferences.OnStretcher = true;
                        break;
                    case "NEEDS ESCORT":
                        patient.SpecialNeedsPreferences.NeedsEscort = true;
                        break;
                    case "GENDER SPECIFIC PROVIDER":
                        patient.SpecialNeedsPreferences
                            .GenderSpecificProvider = true;
                        break;
                    case "HEARING IMPAIRED":
                        patient.SpecialNeedsPreferences.HearingImpaired =
                            true;
                        break;
                    case "SPECIAL MODE OF TRANSPORTATION":
                        patient.SpecialNeedsPreferences
                            .SpecialModeOfTransportation = true;
                        break;
                    case "LANGUAGE PREFERENCE":
                        patient.SpecialNeedsPreferences.LanguagePreference
                            = true;
                        break;
                    case "MORNING":
                        patient.SpecialNeedsPreferences.Am = true;
                        break;
                    case "AFTERNOON":
                        patient.SpecialNeedsPreferences.Pm = true;
                        break;
                    case "MONDAY":
                        patient.SpecialNeedsPreferences.Monday = true;
                        break;
                    case "TUESDAY":
                        patient.SpecialNeedsPreferences.Tuesday = true;
                        break;
                    case "WEDNESDAY":
                        patient.SpecialNeedsPreferences.Wednesday = true;
                        break;
                    case "THURSDAY":
                        patient.SpecialNeedsPreferences.Thursday = true;
                        break;
                    case "FRIDAY":
                        patient.SpecialNeedsPreferences.Friday = true;
                        break;
                    case "SATURDAY":
                        patient.SpecialNeedsPreferences.Saturday = true;
                        break;
                }
            }

            if (remarks.Count > 0)
            {
                patient.Preferences.Add("REMARKS:");

                foreach (var remark in remarks)
                    patient.Preferences.Add("  " + remark);
            }
        }

        private bool AreSameAppointments(AppointmentRequest one, AppointmentRequest two)
        {
            var result = false;

            if (one.IsApptOrEwl && two.IsApptOrEwl)
                result = one.RequestManagmentIen == two.RequestManagmentIen;
            else if (one.IsPtCSch && two.IsPtCSch)
                result = one.PtCSchRequestIen == two.PtCSchRequestIen;
            else if (!one.IsApptOrEwl && !one.IsPtCSch && !two.IsApptOrEwl && !two.IsPtCSch)
                result = one.ConsultationRequestIen == two.ConsultationRequestIen;

            return result;
        }

        public void UpdateFutureAppointmentsOnly(Patient patient)
        {
            //  If no patient passed in, use the currently selected patient
            if (patient == null)
                patient = Model.Patient;

            // If no patient is present then exit
            if (patient == null)
                return;

            UpdateSpecialNeedsRibbon(patient.Preferences);
            var futureAppts = _dataAccessService.GetFutureAppointments(patient.IEN, DateTime.Now.AddDays(-365));
            UpdateFutureAppointments(futureAppts);
        }

        private PatientPrompt PatientFlags = null;

        public void UpdatePatientFlags(Patient patient)
        {
            while (DpPatientFlags.Children.Count > 0)
            {
                RemoveControlFromTabOrder((Control)DpPatientFlags.Children[0], "TasksTab");
                DpPatientFlags.Children[0].KeyDown -= General_KeyDown;
                DpPatientFlags.Children.RemoveAt(0);
            }

            if (patient != null)
            {
                PatientFlags = PatientPrompt.ProcessFlags(patient, FontSize, true, false, true);

                if (PatientFlags != null)
                {
                    PatientFlags.Name = "PatientFlags";
                    DockPanel.SetDock(PatientFlags, Dock.Top);
                    DpPatientFlags.Children.Add(PatientFlags);
                    PatientFlags.KeyDown += General_KeyDown;
                    PatientFlags.Focusable = true;
                    PatientFlags.TabIndex = 108;
                    PatientFlags.IsTabStop = true;
                    AddControlToTabOrder(PatientFlags, "TasksTab", RibGrpPatientFlags);
                }
            }

            RibGrpPatientFlags.Visibility = DpPatientFlags.Children.Count > 0 ? Visibility.Visible : Visibility.Collapsed;
        }

        public void UpdateSpecialNeedsFutureAppointmentsAndDemoInfo(Patient patient)
        {
            GetPatientPreferences(patient);
            UpdateSpecialNeedsRibbon(patient.Preferences);
            var futureAppts = _dataAccessService.GetFutureAppointments(patient.IEN, DateTime.Now.AddDays(-365));
            UpdateFutureAppointments(futureAppts);
            UpdatePatientDemoInfoRibbon(patient);
        }

        private bool _bDialogResult;

        public bool ConfirmUser(string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }

        public void AlertUser(string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        public void ChangeViewMode(string viewMode)
        {
            string[] parts = viewMode.Split('^');

            if (parts[0] != "RibbonBar.Navigation")
                RadCombox3.Text = parts[1];
        }

        private void ViewMode_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var mode = "Week";

            if (((RadComboBox)sender).IsDropDownOpen)
            {
                switch (((RadComboBox)sender).SelectedIndex)
                {
                    case 0:
                        mode = "Day";
                        break;

                    case 1:
                        mode = "Week";
                        break;

                    case 2:
                        mode = "Month";
                        break;
                    case 3:
                        mode = "Timeline";
                        break;
                }

                Model.ExecuteChangeViewModeCommand("RibbonBar.Navigation^" + mode);
            }
        }

        private void WaitListPatients_SelectionChanged(object sender, SelectionChangeEventArgs e)
        {
            if (e.AddedItems.Count > 0)
            {
                Model.SelectPatient(e.AddedItems[0] as Patient);
            }
        }

        private void OnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
            e.Handled = true;
        }

        private void OnGridViewDropInfo(object sender, DragDropEventArgs e)
        {
        }

        private void OnGridViewDragQuery(object sender, DragDropQueryEventArgs e)
        {
            if (e.Options.Source is GridViewRow)
            {
                var gridView = sender as RadGridView;
                var draggedItems = e.Options.Payload as IEnumerable;
                if (gridView != null)
                {
                    e.QueryResult = gridView.SelectedItems.Count > 0;
                    e.Options.Payload = gridView.SelectedItems;
                }
                else
                {
                    e.QueryResult = true;
                    e.Handled = true;
                }
            }
        }

        private void OnGridViewDragInfo(object sender, DragDropEventArgs e)
        {
            if (e.Options.Source is GridViewRow)
            {
                if (e.Options.Status == DragStatus.DragInProgress)
                {
                    var radGridView = sender as RadGridView;
                    if (radGridView != null)
                    {
                        var cue2 = new ContentControl();
                        var content = radGridView.SelectedItems[0] as Patient;
                        if (content != null)
                            cue2.Content = content.Name;
                        else
                            cue2.Content = e.Options.Payload as IEnumerable;
                        e.Options.DragCue = cue2;
                    }
                    e.Handled = true;
                }
                else if (e.Options.Status == DragStatus.DragComplete)
                {
                    e.Handled = true;
                }
            }
        }

        private void ScheduleView_Loaded(object sender, RoutedEventArgs e)
        {
            //mnFindAppointment.IsEnabled = true;
            //BtnFindAppointment.IsEnabled = true;
            InitializeTabOrder();
            RadCombox3.Text = "Week";
        }

        public void ClearPatient_Click(object sender, RoutedEventArgs e)
        {
            Model.ClearPatient();
        }

        private void SelectPatient_DoubleClick(object sender, MouseButtonEventArgs e)
        {
        }

        private void SelectPatient_RightButtonDown(object sender, MouseButtonEventArgs e)
        {
            string szIENTest = "";
            if (Model.Patient != null)
                szIENTest = Model.Patient.IEN;

            if (szIENTest.Length > 0)
            {
                Model.EventAggregator.GetEvent<LaunchDemographicEditor>().Publish(string.Empty);
                Model.UpdatePatient();
            }
        }

        private IList<RMGridExportModel> ConvertRMGridToReportModel()
        {
            IList<RMGridExportModel> reportData = new List<RMGridExportModel>();
            
            foreach (var item in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
            {
                var req = item;
                reportData.Add(new RMGridExportModel(req));
            }

            return reportData;
        }

        private void AddItemToReportGrid(Grid grid, string itemValue, int row, int col, Brush backgroundBrush = null)
        {
            var tb = new TextBlock
            {
                HorizontalAlignment = HorizontalAlignment.Stretch,
                VerticalAlignment = VerticalAlignment.Center,
                Padding = new Thickness( 5, 5, 5, 5 ),
                Text = itemValue
            };

            if (backgroundBrush != null)
                tb.Background = backgroundBrush;

            grid.Children.Add(tb);
            Grid.SetRow(tb, row);
            Grid.SetColumn(tb, col);
        }

        private void PrintRequestGridBtn_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new PrintDialog();
            var radGridView = Model.PatientQueueView.PatientQueueRadGridView;
            var result = dialog.ShowDialog();
            if (result.Value)
            {
                Mouse.OverrideCursor = Cursors.Wait;

                try
                {
                    // Get the list of items to print
                    var reportData = ConvertRMGridToReportModel();

                    // Create Grid Panel
                    var reportGrid = new Grid();

                    // Create required auto-sized columns
                    for (var cIndex = 0; cIndex < radGridView.Columns.Count; cIndex++)
                    {
                        var coldef = new ColumnDefinition
                        {
                            Width = GridLength.Auto
                        };
                        reportGrid.ColumnDefinitions.Add(coldef);
                    }

                    // Create required auto-size rows
                    for (var rIndex = 0; rIndex <= reportData.Count; rIndex++)
                    {
                        var rowdef = new RowDefinition
                        {
                            Height = GridLength.Auto
                        };
                        reportGrid.RowDefinitions.Add(rowdef);
                    }

                    var rowIndex = 0;

                    Brush headerBrush = new LinearGradientBrush(Colors.CornflowerBlue, Colors.White, new Point(0, 0),
                        new Point(0, 1));

                    // Add Report Header
                    foreach (var item in reportData)
                    {
                        // Add a Header Row
                        AddItemToReportGrid(reportGrid, "REQUEST_TYPE", rowIndex, 0, headerBrush);
                        AddItemToReportGrid(reportGrid, "WAIT_TIME", rowIndex, 1, headerBrush);
                        AddItemToReportGrid(reportGrid, "PT_NAME", rowIndex, 2, headerBrush);
                        AddItemToReportGrid(reportGrid, "MRTC", rowIndex, 3, headerBrush);
                        AddItemToReportGrid(reportGrid, "SSN", rowIndex, 4, headerBrush);
                        AddItemToReportGrid(reportGrid, "SCVISIT", rowIndex, 5, headerBrush);
                        AddItemToReportGrid(reportGrid, "TELEPHONE", rowIndex, 6, headerBrush);
                        AddItemToReportGrid(reportGrid, "PRIORITY", rowIndex, 7, headerBrush);
                        AddItemToReportGrid(reportGrid, "CID/PREFERRED DATE", rowIndex, 8, headerBrush); // DESIRED_PtCSch_DATE
                        AddItemToReportGrid(reportGrid, "ENTERED_RR_NO_DATE", rowIndex, 9, headerBrush);
                        AddItemToReportGrid(reportGrid, "REQUESTED_BY", rowIndex, 10, headerBrush);
                        AddItemToReportGrid(reportGrid, "REQUESTOR", rowIndex, 11, headerBrush);
                        AddItemToReportGrid(reportGrid, "CLINIC_SERVICE", rowIndex, 12, headerBrush);
                        AddItemToReportGrid(reportGrid, "COMMENT", rowIndex, 13, headerBrush);
                    }

                    // Fill the Grid
                    foreach (var item in reportData)
                    {
                        // Add Data
                        rowIndex++;

                        Brush rowBrush = null;

                        if (rowIndex%2 == 0)
                            rowBrush = Model.PatientQueueView.PatientQueueRadGridView.AlternateRowBackground;

                        AddItemToReportGrid(reportGrid, item.REQUEST_TYPE, rowIndex, 0, rowBrush);
                        AddItemToReportGrid(reportGrid, item.WAIT_TIME.ToString(), rowIndex, 1, rowBrush);
                        AddItemToReportGrid(reportGrid, item.PT_NAME, rowIndex, 2, rowBrush);
                        AddItemToReportGrid(reportGrid, item.MRTC.ToString(), rowIndex, 3, rowBrush);
                        AddItemToReportGrid(reportGrid, item.SSN, rowIndex, 4, rowBrush);
                        AddItemToReportGrid(reportGrid, item.SCVisit.ToString(), rowIndex, 5, rowBrush);
                        AddItemToReportGrid(reportGrid, item.TELEPHONE, rowIndex, 6, rowBrush);
                        AddItemToReportGrid(reportGrid, item.PRIORITY, rowIndex, 7, rowBrush);
                        AddItemToReportGrid(reportGrid, item.DESIRED_PtCSch_DATE, rowIndex, 8, rowBrush);
                        AddItemToReportGrid(reportGrid, item.ENTERED_RR_NO_DATE, rowIndex, 9, rowBrush);
                        AddItemToReportGrid(reportGrid, item.REQUESTED_BY, rowIndex, 10, rowBrush);
                        AddItemToReportGrid(reportGrid, item.REQUESTOR, rowIndex, 11, rowBrush);
                        AddItemToReportGrid(reportGrid, item.CLINIC_SERVICE, rowIndex, 12, rowBrush);
                        AddItemToReportGrid(reportGrid, item.COMMENT, rowIndex, 13, rowBrush);
                    }

                    // Set page orientation
                    dialog.PrintTicket.PageOrientation = PageOrientation.Landscape;

                    // Size the Grid.
                    reportGrid.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
                    var sizeGrid = reportGrid.DesiredSize;
                    sizeGrid.Height += 50;
                    sizeGrid.Width += 50;

                    // Layout pass to size the reportGrid.
                    reportGrid.Arrange(new Rect(new Point(0, 0), sizeGrid)); //ptGrid, sizeGrid));

                    /***************  Print to Fit . . . Width only ************/

                    //get selected printer capabilities
                    var capabilities =
                        dialog.PrintQueue.GetPrintCapabilities(dialog.PrintTicket);

                    //get scale of the print wrt to screen of WPF visual -- Fit Width only
                    var scale = capabilities.PageImageableArea.ExtentWidth/reportGrid.ActualWidth;

                    //Transform the Visual to scale
                    reportGrid.LayoutTransform = new ScaleTransform(scale, scale);

                    //get the size of the printer page
                    var sz = new Size(capabilities.PageImageableArea.ExtentWidth,
                        capabilities.PageImageableArea.ExtentHeight);

                    //update the layout of the visual to the printer page size.
                    reportGrid.Measure(sz);
                    reportGrid.Arrange(
                        new Rect(
                            new Point(capabilities.PageImageableArea.OriginWidth,
                                capabilities.PageImageableArea.OriginHeight), sz));

                    /*******************************************************/

                    // Now print it.
                    dialog.PrintVisual(reportGrid, "Request Grid");
                }
                finally
                {
                    Mouse.OverrideCursor = Cursors.Arrow;
                }
            }
        }

        private void ExportRequestGridBtn_Click(object sender, RoutedEventArgs e)
        {
            var reportData = ConvertRMGridToReportModel();
            ExportReportToExcel.ExportReport("Request Grid", reportData);
        }

        private void RibbonBar_SelectedTabChanged(object sender, RadSelectionChangedEventArgs e)
        {
            var x = sender as RadRibbonView;

            if ( x != null && x.SelectedTab != null )
            {
                var tabIndex = x.SelectedTab.TabIndex;
                Model.EventAggregator.GetEvent<RibbonBarSelectedTabChangedEvent>().Publish( "RIBBON^" + tabIndex );
                ManageFocusDefaultsOnTabChange();
            }
        }

        private void RefreshRmGrid_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Mouse.OverrideCursor = Cursors.Wait;
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Clear();
                Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
                Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
                Keys.WaitListQueryLastRec = null;
                Keys.SelectedPatient = null;
                Keys.PatientIen_SensitiveRecordCheck = null;
                Model.EventAggregator.GetEvent<PatientUnselectEvent>().Publish("");
                try
                {
                    Keys.BooleanFlags.RefreshWaitList = true;
                    Model.Patient = null;
                    Keys.SelectedPatient = null;
                    Model.PatientQueueView.IsRefreshOrClear = true;
                    Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = false;
                    Model.PatientQueueView.UpdateWaitTimeValue(Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90);
                    Model.AdmittedPatients.Clear();  // MES - this resolves a chance for duplicate data if the WaitTime was changed.
                    Model.EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("RefreshRMGrid");
                    Model.EventAggregator.GetEvent<UnselectClinicEvent>().Publish("REFRESH");
                    Model.EventAggregator.GetEvent<ProcessUserPreferencesEvent>().Publish(Keys.UserPrefList);
                    try
                    {
                        if (!Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
                            Model.DoRefreshWaitList("");
                    }
                    finally
                    {
                        Keys.BooleanFlags.RefreshWaitList = false;
                        Model.PatientQueueView.IsRefreshOrClear = false;
                    }
                }
                finally
                {
                    Mouse.OverrideCursor = Cursors.Arrow;
                }
            }
            catch (Exception ex)
            {
                AlertUser(TraceLog.FormatExceptionMessage(GetType().Name, "RefreshRMGerid_Clock", ex), "RM Grid Refresh Error");
            }
        }

        private void RadRibbonBar1_Loaded(object sender, RoutedEventArgs e)
        {
            var ribbonBar = (RadRibbonView)sender;

            if (ribbonBar != null)
            {
                var selectedTabPresenter = ribbonBar.Template.FindName("SelectedTabPresenter", ribbonBar) as ContentPresenter;
                if ( selectedTabPresenter != null )
                    selectedTabPresenter.Height = ribbonBar.Height;
            }
        }

        private void btnPatientDemoInfo_Click(object sender, RoutedEventArgs e)
        {
            var szIENTest = "";
            if (Model.Patient != null)
                szIENTest = Model.Patient.IEN;
            else if ( Keys.SelectedPatient != null )
                szIENTest = Keys.SelectedPatient.IEN;

            if (szIENTest.Length > 0)
                Model.ShowPatientDemoInfoDialog(szIENTest);

        }

        private void AuditReport_Click(object sender, RoutedEventArgs e)
        {
            var auditRpt = new AuditReportView(Model.DataAccessService);
            CLaunchWindow.LaunchWindow(auditRpt, "Audit Activity", 600, 950, "", true);

        }

        private void ClinicsReport_Click(object sender, RoutedEventArgs e)
        {
            var clinicRpt = new ClinicReportView(Model.DataAccessService);
            CLaunchWindow.LaunchWindow(clinicRpt, "Clinic Activity", 600, 900, "", true);

        }

        private void DashboardReport_Click(object sender, RoutedEventArgs e)
        {
            if(Model.MoveJasperReportDependenciesToUsersFolder())
            {
                var view = new EncounterReportDashboardView(_dataAccessService);
                CLaunchWindow.LaunchWindow(view, "Resource Management Report Selection", 260, 600, "", false, true);
                //Model.RemoveJasperReportDependenciesFromUsersFolder();
            }
            
        }

        private void QueryRmGrid_Click(object sender, RoutedEventArgs e)
        {
            var waitListQuery = new WaitListQueryForm( _dataAccessService );
            CLaunchWindow.LaunchWindow(waitListQuery, "Request Query", 510, 1285);
            if (waitListQuery.SubmitClicked)
            {
                Model.Patient = null;
                Keys.SelectedPatient = null;
                Keys.SelectedPatient = null;
                UpdatePatientSearchText("");
                IList<Patient> emptyList = new List<Patient>();
                PatientSearchComboBox.Text = string.Empty;
                Model.PatientList = emptyList;
                Model.EventAggregator.GetEvent<CloseAllSchedulesEvent>().Publish("QueryRMGrid");
                Model.EventAggregator.GetEvent<UnselectClinicEvent>().Publish("REFRESH");
                Model.EventAggregator.GetEvent<PatientUnselectEvent>().Publish("");

                Model.PatientQueueView.UpdateWaitTimeValue(waitListQuery.WaitTimeSetting);
                if (Model.AdmittedPatients == null)
                    return;
                if (Model.AdmittedPatients.Count > 0)
                    Model.AdmittedPatients.Clear();
                LbFutureAppointmentsView.Items.Clear();
                LbSpecialNeeds.Items.Clear();
                foreach (var rec in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
                    Model.AdmittedPatients.Add(rec);
                Model.NotifyPropertyChanged("AdmittedPatients");
                Model.PatientQueueView.PatientQueueRadGridView.Rebind();
                Model.PatientQueueView.PatientQueueRadGridView.ToolTip = "Viewing records 1 thru " + Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count + " of " + Keys.WaitListQueryTotalRecords;
            }
        }

        #region VARMobileRequest Members

        private UserControl VARRequestButton = null;

        public void InjectVARButton(UserControl button)
        {
            button.Name = "VARRequestButton";
            button.KeyDown += General_KeyDown;
            RibGrpMobilePanel.Children.Add(button);
            button.HorizontalAlignment = HorizontalAlignment.Center;
            button.VerticalAlignment = VerticalAlignment.Top;
            button.KeyDown += General_KeyDown;
            button.Focusable = true;
            button.IsTabStop = true;
            button.TabIndex = 109;
            AddControlToTabOrder(button, "TasksTab", RibGrpMobileRequests);
            VARRequestButton = button;
            RibGrpMobileRequests.Visibility = Visibility.Visible;
        }

        private bool _isVARRequest;
        private string _varPDDate = string.Empty;

        public void SelectPatient(Patient patient, bool isVARRequest = false, string varPDDate = "")
        {
            _isVARRequest = true;
            _varPDDate = varPDDate;

            try
            {
                PatientSearchComboBox.SelectedItem = patient;
            }
            finally
            {
                _isVARRequest = false;
                _varPDDate = string.Empty;
            }
        }

        #endregion VARMobileRequest Members

        private void TasksTabHotKeys(Key k)
        {
            switch (k)
            {
                case Key.S:
                    Keyboard.Focus( PatientSearchComboBox );
                    break;

                case Key.A:
                    if (BtnNewReq.IsEnabled)
                        Keyboard.Focus(RibGrpActions);

                    break;

                case Key.D:
                    if (btnPatientDemoInfo.IsEnabled)
                        Keyboard.Focus(btnPatientDemoInfo);

                    break;

                case Key.P:
                    if (LbFutureAppointmentsView.HasItems)
                        Keyboard.Focus(LbFutureAppointmentsView);

                    break;

                case Key.F:
                    if (RibGrpPatientFlags.Visibility == Visibility.Visible)
                    {
                        DpPatientFlags.Children[0].Focus();
                        Keyboard.Focus(DpPatientFlags.Children[0]);
                    }

                    break;

                case Key.G:
                    if (RadCombox3.IsEnabled)
                        Keyboard.Focus(RibGrpArrangement);

                    break;

                case Key.H:
                    Keyboard.Focus(BtnSearch);
                    break;

                case Key.N:
                    if (LbSpecialNeeds.HasItems)
                        Keyboard.Focus(RibGrpSpecialNeeds);

                    break;

                case Key.M:
                    if ((RibGrpMobileRequests.Visibility == Visibility.Visible) && VARRequestButton.IsEnabled)
                        Keyboard.Focus(VARRequestButton);

                    break;

                case Key.T:
                    if (PrintRequestGridBtn.IsEnabled)
                        Keyboard.Focus(RibGrpTools);

                    break;

                case Key.U:
                    if ((RibGrpUserPrefs.Visibility == Visibility.Visible) &&  BtnUserPrefs.IsEnabled)
                        Keyboard.Focus(BtnUserPrefs);

                    break;
            }
        }

        private void TabsHotKeys(Key k)
        {
            switch (k)
            {
                case Key.T:
                    TasksTab.IsSelected = true;
                    break;
                case Key.Y:
                    SystemTab.IsSelected = true;
                    break;
                case Key.R:
                    ReportsTab.IsSelected = true;
                    break;
            }
        }

        private void SystemTabHotKeys(Key k)
        {
            switch (k)
            {
                case Key.M:
                    BtnSchedMgmt.Focus();
                    Keyboard.Focus(BtnSchedMgmt);
                    break;
                case Key.H:
                    BtnAvailability.Focus();
                    Keyboard.Focus(BtnAvailability);
                    break;
                case Key.L:
                    RibGrpSelClin.Focus();
                    Keyboard.Focus(RibGrpSelClin);
                    break;
                case Key.X:
                    Model.EventAggregator.GetEvent<CloseSchedulesEvent>().Publish("CLOSE_AVAILABILITY");
                    break;
            }
        }

        private void ReportsTabHotKeys(Key k)
        {
            switch (k)
            {
                case Key.H:
                    BtnClinics.Focus();
                    Keyboard.Focus(BtnClinics);
                    break;
                case Key.M:
                    BtnAuditActivity.Focus();
                    Keyboard.Focus(BtnAuditActivity);
                    break;
                case Key.R:
                    RibGrpReports.Focus();
                    Keyboard.Focus(RibGrpReports);
                    break;
            }
        }

        public void RibbonBarHotKeysHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
            {
                TabsHotKeys(k);
            }
            else if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                if (RadRibbonBar1.SelectedTab.Name.Equals("TasksTab"))
                {
                    TasksTabHotKeys(k);
                }
                else if (RadRibbonBar1.SelectedTab.Name.Equals("SystemTab"))
                {
                    SystemTabHotKeys(k);
                }
                else if (RadRibbonBar1.SelectedTab.Name.Equals("ReportsTab"))
                {
                    ReportsTabHotKeys(k);
                }
            }
        }

        public void DoHotKeys(object sender, KeyEventArgs e)
        {
                var key = e.Key == Key.System ? e.SystemKey : e.Key;
            //TraceLog.LogInfo(GetType().Name, "DoHotKeys", key.ToString());
                RibbonBarHotKeysHandler(key);            
        }

        private void SystemTab_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            RibbonBarHotKeysHandler(e.Key);
        }

        private void TasksTab_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            if ((Keyboard.Modifiers == ModifierKeys.Control) || (Keyboard.Modifiers == ModifierKeys.Alt))
                RibbonBarHotKeysHandler(e.Key);
        }

        private void ReportsTab_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            RibbonBarHotKeysHandler(e.Key);
        }

        private void ManageFocusDefaultsOnTabChange()
        {
            SetFocusOnFirstTabStop();
        }

        #region Patient Search members

        private bool _justRan = true;
        private string _veryLastSub = string.Empty;

        public void FocusPatientSearch()
        {
            if (Model.PatientList != null)
                Model.PatientList.Clear();

            Keyboard.Focus( PatientSearchComboBox );
        }

        private void patientSearchCombo_ScrollChanged(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty( PatientSearchComboBox.Text))
                return;
            if ( PatientSearchComboBox.Text.Length < Keys.PatientSearchCharMin)
                return;

            var allRecordsReturned = false;
            if (Model.PatientList != null && string.IsNullOrEmpty(Model.PatientLastSub) && Model.PatientList.Count > 0 /*&& !_bAllRecordsReturnedForSearch*/)
            {
                if (!_justRan)
                {
                    _justRan = !_justRan;
                    return;
                }
                var scrollViewer = e.OriginalSource as ScrollViewer;
                if (scrollViewer != null && scrollViewer.VerticalOffset == scrollViewer.ScrollableHeight)
                {
                    // save previous very last sub for use in testing
                    var veryLastSub = _veryLastSub;
                    /*foreach(var pat in Model.PatientList)
                        lastSub = pat.LastSub;*/

                    var lastPat = Model.PatientList[Model.PatientList.Count - 1];
                    var lastSub = lastPat.LastSub;

                    // save the current very last sub for future testing
                    _veryLastSub = lastSub;

                    // test using the saved previous last sub
                    if (veryLastSub == lastSub)
                        return;

                    var morePats = Model.DataAccessService.GetPatients( PatientSearchComboBox.Text, Keys.MaxPatientsToReturn, ref allRecordsReturned, lastSub);
                    if (morePats != null && morePats.Count > 0)
                        foreach (var morePat in morePats)
                            if (Model.PatientList.FirstOrDefault(p => p.IEN == morePat.IEN) == null)
                                Model.PatientList.Add(morePat);

                    PatientSearchComboBox.Items.Refresh();
                    _justRan = !_justRan;
                }
            }
        }

        private void patientSearchCombo_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e != null && e.AddedItems.Count > 0)
            {
                PatientChanged(e.AddedItems[0] as Patient);
                SetPatientSearchComboBoxOpen(false);
            }
            else
                PatientChanged(null);
        }

        private void patientSearchCombo_TextChanged(object sender, RoutedEventArgs e)
        {
            PatientSearchString = ((RadComboBox)sender).Text;

            if ( PatientSearchComboBox.IsDropDownOpen)
                SetPatientSearchComboBoxOpen(false);

            if ( Model != null && Model.PatientList != null && Model.PatientList.Count > 0 )
                Model.PatientList.Clear();
        }

        private void patientSearchButton_Click(object sender, RoutedEventArgs e)
        {
            Dispatcher.BeginInvoke((Action)(() => Model.SearchStringChanged(PatientSearchString)));
            FocusPatientSearch();
        }

        public void SetPatientSearchComboBoxOpen(bool openDropDown)
        {
            PatientSearchComboBox.IsDropDownOpen = openDropDown;
        }

        #endregion Patient Search members

        private void RibbonBar_DoubleClick(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
        }

        private void UserControl_Unloaded( object sender, RoutedEventArgs e )
        {
            Model.ExecuteExitCommand("Exit");
        }

        private void General_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Tab:
                    switch (Keyboard.Modifiers)
                    {
                        case ModifierKeys.Alt:
                            break;
                        case ModifierKeys.Control:
                            break;
                        case ModifierKeys.None:
                            e.Handled = TabNext(sender);
                            break;
                        case ModifierKeys.Shift:
                            e.Handled = TabPrev(sender);
                            break;
                        case ModifierKeys.Windows:
                            break;
                    }

                    break;
                case Key.Enter:
                    e.Handled = ButtonExecute(sender);
                    break;
            }
        }

        private bool ButtonExecute(object sender)
        {
            bool handled = true;

            if (sender == BtnSchedMgmt)
                patientSearchButton_Click(sender, new RoutedEventArgs());
            else if (sender == btnPatientDemoInfo)
                MessageBox.Show("Patient Demog Button");
            else
                handled = false;

            return handled;
        }

        private bool TabNext(object sender)
        {
            bool handled = false;

            if (!(sender is Control))
                return handled;

            if (RadRibbonBar1.SelectedTab == null)
                return handled;

            Dictionary<int, tabControl> tabKeyController = _tabKeyController[RadRibbonBar1.SelectedTab.Name];

            if (tabKeyController == null)
                return handled;

            Control control = (Control)sender;
            tabControl nextControl = null;
            List<int> indexes = tabKeyController.Keys.Where(k => (k > control.TabIndex)).ToList();

            if (indexes.Count == 0)
                nextControl = tabKeyController[tabKeyController.Keys.Min()];
            else
                nextControl = tabKeyController[indexes.Min()];

            if (nextControl.CanTabTo)
            {
#if DEBUG
                TraceLog.LogInfo(GetType().Name, "TabNext", $"tabbing to {nextControl.TargetControl.Name} on {RadRibbonBar1.SelectedTab.Name}.");
#endif
                Keyboard.Focus(nextControl.TargetControl);
                handled = true;
            }
            else
            {
#if DEBUG
                TraceLog.LogInfo(GetType().Name, "TabNext", $"{nextControl.TargetControl.Name} on {RadRibbonBar1.SelectedTab.Name} is disabled, moving on.");
#endif
                handled = TabNext(nextControl.TargetControl);
            }

            return handled;
        }

        private bool TabPrev(object sender)
        {
            bool handled = false;

            if (!(sender is Control))
                return handled;

            if (RadRibbonBar1.SelectedTab == null)
                return handled;

            Dictionary<int, tabControl> tabKeyController = _tabKeyController[RadRibbonBar1.SelectedTab.Name];

            if (tabKeyController == null)
                return handled;

            Control control = (Control)sender;
            tabControl nextControl = null;
            List<int> indexes = tabKeyController.Keys.Where(k => (k < control.TabIndex)).ToList();

            if (indexes.Count == 0)
                nextControl = tabKeyController[tabKeyController.Keys.Max()];
            else
                nextControl = tabKeyController[indexes.Max()];

            if (nextControl.CanTabTo)
            {
#if DEBUG
                TraceLog.LogInfo(GetType().Name, "TabPrev", $"tabbing to {nextControl.TargetControl.Name} on {RadRibbonBar1.SelectedTab.Name}.");
#endif
                Keyboard.Focus(nextControl.TargetControl);
                handled = true;
            }
            else
            {
#if DEBUG
                TraceLog.LogInfo(GetType().Name, "TabPrev", $"{nextControl.TargetControl.Name} on {RadRibbonBar1.SelectedTab.Name} is disabled, moving on.");
#endif
                handled = TabNext(nextControl.TargetControl);
            }

            return handled;
        }

        private void SetFocusOnFirstTabStop()
        {
            if (RadRibbonBar1.SelectedTab == null)
                return;

            var controller = _tabKeyController[RadRibbonBar1.SelectedTab.Name];

            if (controller == null)
                return;

            var lastControl = controller[controller.Keys.Max()];

            if (lastControl == null)
                return;

            TabNext(lastControl.TargetControl);
        }

        private void PendingAppointmentsRightClick_Click(object sender, MouseButtonEventArgs e)
        {
            Model.PendingAppointmentsRightClick = true;
        }
    }
}
