﻿using System;
using System.Collections.Generic;
using System.Windows.Input;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Ribbon.Schedule
{
    public class UserPreferencesVM : BaseViewModel
    {
        private string[] AppointmentRequestTypes =
        {
            "RECALL",
            "EWL",
            "CONSULT",
            "PROCEDURE",
            "APPT",
            "VETERAN",
            "MOBILE",
            "W2VA"
        };

        public Action<List<string>> CloseCallback { get; set; }

        public UserPreferencesVM()
        {
            BuildRequestTypesTable();
            BuildUserSettingsTable();
        }

        public List<UserSetting> Settings { get; set; } = new List<UserSetting>();

        private void BuildUserSettingsTable()
        {
            Settings = new List<UserSetting>
            {
                new UserSetting( "Request Type", RequestTypes, false, false ),
                new UserSetting( "Wait Time", WaitTime, false, false ),
                new UserSetting( "Patient Name", null, false, false ),
                new UserSetting( "MRTC", MRTC, false, false ),
                new UserSetting( "SSN", null, false, false ),
                new UserSetting( "SCV Visit", SCVVisit, false, false ),
                new UserSetting( "Telephone", null, false, false ),
                new UserSetting( "Priority Group", PriorityGroups, false, false ),
                new UserSetting( "CID/Preferred Date", PreferredDate, false, false ),
                new UserSetting( "Entered/RR No Date", EnteredRRNoDate, false, false ),
                new UserSetting( "Requestor", null, false, false ),
                new UserSetting( "Requested By", null, false, false ),
                new UserSetting( "Clinic/Specialty", Clinics, false, false ),
                new UserSetting( "Comment", null, false, false ),
            };
        }

        public List<DateTime> EnteredRRNoDate { get; set; } = new List<DateTime>();

        public List<DateTime> PreferredDate { get; set; } = new List<DateTime>();

        private void BuildRequestTypesTable()
        {
            RequestTypes.Clear();
            foreach ( var appointmentRequestType in AppointmentRequestTypes )
            {
                RequestTypes.Add( new RequestType {RequestName = appointmentRequestType} );
            }
        }

        public List<RequestType> RequestTypes { get; set; } = new List<RequestType>();

        public List<string> SCVVisit { get; set; } = new List<string> { "Yes", "No", "Both" };

        public List<string> MRTC { get; set; } = new List<string> {"Yes", "No", "Both"};

        public List<string> PriorityGroups { get; set; } = new List<string>();

        public List<Clinic> Clinics { get; set; } = new List<Clinic>();

        public int WaitTime { get; set; }

        //==========================================================================================
        public class UserSetting : BaseViewModel
        {
            public UserSetting( string name, object value, bool filter, bool sort )
            {
                Name       = name;
                Value      = value;
                Filter     = filter;
                Sort       = sort;

                SetNameValues( value );
            }

            public bool ShowFilter
            {
                get { return Value != null; }
            }

            public bool Sort { get; set; }

            public bool Filter { get; set; }

            private void SetNameValues( object value )
            {
                switch ( Name )
                {
                    case "Request Type" :
                        RequestTypes = (List<RequestType> )value;
                        break;
                    case "Wait Time" :
                        break;
                    case "Patient Name" :
                        break;
                    case "SSN" :
                        break;
                    case "MRTC" :
                        break;
                    case "Clinic/Spcialty" :
                        break;
                    case "Priority Group" :
                        break;
                    case "Comment" :
                        break;
                    case "Telephone" :
                        break;
                }
            }

            public string Name { get; private set; }

            public object Value { get; set; }

            public bool IsCheckBox { get; private set; }
            public List<RequestType> RequestTypes { get; private set; }

            public ICommand GetRequestTypesCommand
            {
                get
                {
                    return GetCommand(
                        "GetRequestTypesCommand",
                        param =>
                        {
                            if ( !Filter )
                                return;

                            var vm = new RequestTypesVM
                            {
                                Title = "Request Types",
                                RequestTypes = RequestTypes
                            };

                            var dlg = new RequestTypesV
                            {
                                Context = vm
                            };
                            vm.CloseCallback = result =>
                            {
                                dlg.Close();
                                if ( result != null )
                                {
                                }
                            };
                            dlg.ShowDialog();
                        },
                        _ => true );
                }
            }
        }
    }

}
