﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class PriorityGroupsVM : BaseViewModel
    {
        public Action<string> CloseCallback { get; set; }

        public List<PriorityGroup> PriorityGroups { get; set; }

        public bool AllSelected { get; set; }
        public string SelectedGroups
        {
            get
            {
                var list = "";
                foreach ( var priorityGroup in PriorityGroups )
                {
                    if ( priorityGroup.IsSelected )
                        list += priorityGroup.PriorityGroupName + ", ";
                }
                if ( list.EndsWith( ", " ) )
                    list = list.Substring( 0, list.Length-2 );
                return list;
            }
        }

        public ICommand SelectAllCommand
        {
            get
            {
                return GetCommand(
                    "SelectAllCommand",
                    param =>
                    {
                        foreach ( var priorityGroup in PriorityGroups )
                        {
                            priorityGroup.IsSelected = AllSelected;
                        }
                    },
                    _ => true );
            }
        }

        #region ICloseCommandProvider Members

        public bool IsClosing { get; private set; }

        public ICommand CloseCommand
        {
            get
            {
                return GetCommand(
                    "CloseCommand",
                    _ =>
                    {
                        IsClosing = true;
                        if ( CloseCallback != null )
                        {
                            CloseCallback( null );
                        }
                    },
                    _ => true
                    );
            }
        }

        #endregion ICloseCommandProvider Members

        #region ISaveCommandProvider Members

        public ICommand SaveCommand
        {
            get
            {
                return GetCommand(
                    "SaveCommand",
                    _ =>
                    {
                        if ( CloseCallback != null )
                        {
                            CloseCallback( SelectedGroups );
                        }
                    },
                    _ => PriorityGroups.Any( iss => iss.IsSelected ) );
            }
        }

        #endregion ISaveCommandProvider Members
    }
}
