﻿using System;
using System.Collections.Generic;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.DataAccess.Services;
using Keys = ClinSchd.Infrastructure.Keys;
using MessageBox = System.Windows.MessageBox;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class RequestManagementViewModel : BaseViewModel
    {
        public string RequestManagmentIen { get; set; }

        private Patient _patient;

        public Patient Patient
        {
            get { return _patient; }
            set
            {
                _patient = value;
                ChkSvcRelatedEnabled = _patient.ServiceConnectionBool;
            }
        }

        public string OriginationUser { get; set; }

        public string Institution { get; set; }

        public DateTime OriginationDate { get; set; }

        public int WaitTime2
        {
            get
            {
                var diff = DateTime.Today - OriginationDate;
                return diff.Days;
            }
        }

        public bool MultipleAppointmentsRequired { get; set; }

        public int NumberOfAppointmentsRequired { get; set; }

        public int AppointmentIntervalInDays { get; set; }

        public static int Errors { get; set; }

        public DataAccessService das = new DataAccessService();

        public string AppointmentType { get; set; }

        public string Specialty { get; set; }

        public List<string> Specialties { get; set; }

        public string SpecialtyIen { get; set; }

        public ServiceSpecialty ServiceSpecialty { get; set; }

        public IList<ServiceSpecialty> ServiceSpecialties
        {
            get { return das.GetServiceSpecialties(); }
        }

        public Clinic Clinic { get; set; }

        public IList<Clinic> EwlClinics
        {
            get { return das.GetEwlClinics(); }
        }

        private string _clinServiceSpecialtyMerge;

        public string ClinicServiceSpecialtyMerge
        {
            get
            {
                if ( !string.IsNullOrEmpty( _clinServiceSpecialtyMerge ) )
                    return _clinServiceSpecialtyMerge;
                return string.IsNullOrEmpty( ClinName ) ? ClinName : Specialty;
            }
            set { _clinServiceSpecialtyMerge = value; }
        }

        public string ClinName { get; set; }

        //public List<string> Clinics { get; set; }
        public string ClinicIen { get; set; }

        public string AddedToRequestManagmentQueueBy { get; set; }

        public string RequestManagmentComment { get; set; }

        public string RequestManagementCommentTrunc
        {
            get
            {
                if ( RequestManagmentComment.Length < 65 )
                    return RequestManagmentComment;
                return RequestManagmentComment.Substring( 0, 64 ) + "...";
            }
        }

        public string DesiredApptDateShortString
        {
            get { return _desiredApptDate != null ? _desiredApptDate.Value.Date.ToShortDateString() : ""; }
        }

        private DateTime? _desiredApptDate;

        public DateTime? DesiredApptDate
        {
            get
            {
                if ( _desiredApptDate != null )
                {
                    return _desiredApptDate.Value.Date;
                }
                return null;
            }
            set { _desiredApptDate = value; }
        }

        private readonly List<string> _desiredApptDates = new List<string>();

        public List<string> DesiredApptDates
        {
            get
            {
                for ( int i = 0; i < 60; i++ )
                {
                    try
                    {
                        _desiredApptDates.Add( DateTime.Today.AddDays( i ).ToShortDateString() );
                    }
                    catch ( Exception ex )
                    {
                        MessageBox.Show( ex.Message, "Desired Appointment Date Error" );
                        throw;
                    }
                }
                return _desiredApptDates;
            }
        }

        public int? WaitTime
        {
            get
            {
                if ( OriginatingDate == null )
                    return null;
                var diff = DateTime.Today - OriginatingDate;
                var waitTime = diff.Value.Days;
                return waitTime;
            }
        }

        //private readonly List<string> _waitListTypes = new List<string>(new[] { "PCMM TEAM ASSIGNMENT", "PCMM POSITION ASSIGNMENT", "SERVICE/SPECIALITY", "SPECIFIC CLINIC" });

        public List<string> WaitListTypes { get; } = new List<string>( new[]
        {
            Keys.RequestManagementKeys.ServiceSpecialty,
            Keys.RequestManagementKeys.SpecificClinic
        } );

        public string WaitListType { get; set; }

        public string OriginatingUser { get; set; }

        public IList<Provider> OriginatingUsers { get; set; }

        public string OriginatingUserName { get; set; }

        public DateTime? OriginatingDate { get; set; }

        public string OriginatingDateString
        {
            get { return OriginatingDate != null ? OriginatingDate.Value.Date.ToShortDateString() : ""; }
        }

        public bool ChkSvcRelatedEnabled { get; private set; }
    }
}
