﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class RequestTypesVM : BaseViewModel
    {
        public Action<string> CloseCallback { get; set; }

        public List<RequestType> RequestTypes { get; set; } = new List<RequestType>();

        public string SelectedTypes
        {
            get
            {
                var list = "";
                foreach ( var requestType in RequestTypes )
                {
                    if ( requestType.IsSelected )
                        list += requestType.RequestName + ", ";
                }
                if ( list.EndsWith( ", " ) )
                    list = list.Substring( 0, list.Length-2 );
                return list;
            }
        }

        public bool SelectAll { get; set; }
        public ICommand SelectAllCommand
        {
            get
            {
                return GetCommand(
                    "SelectAllCommand",
                    param =>
                    {
                        foreach ( var requestType in RequestTypes )
                        {
                            requestType.IsSelected = SelectAll;
                        }
                    },
                    _ => true );
            }
        }

        #region ICloseCommandProvider Members

        public bool IsClosing { get; private set; }

        public ICommand CloseCommand
        {
            get
            {
                return GetCommand(
                    "CloseCommand",
                    _ =>
                    {
                        IsClosing = true;
                        if ( CloseCallback != null )
                        {
                            CloseCallback( null );
                        }
                    },
                    _ => true
                    );
            }
        }

        #endregion ICloseCommandProvider Members

        #region ISaveCommandProvider Members

        public ICommand SaveCommand
        {
            get
            {
                return GetCommand(
                    "SaveCommand",
                    _ =>
                    {
                        if ( CloseCallback != null )
                        {
                            CloseCallback( SelectedTypes );
                        }
                    },
                    _ => RequestTypes.Any( iss => iss.IsSelected ) );
            }
        }

        #endregion ISaveCommandProvider Members
    }
}
