﻿using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Ribbon.Views;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class RowSetting : BaseViewModel
    {
        private bool _isFiltered;
        private string _settings;
        private string _sortOrder;

        public void LoadPriorities()
        {
            SortList.Add( "" );
            SortList.Add( "1" );
            SortList.Add( "2" );
            SortList.Add( "3" );
        }

        public RowSetting()
        {
        }
        public RowSetting( string columnName, object columnOptions, bool isFiltered, string settings, string sortOrder )
        {
            ColumnName    = columnName.Trim();
            ColumnOptions = columnOptions;
            IsFiltered    = isFiltered;
            FilterInfo      = settings.Trim();
            // might need to add IsServiceList here
            SortOrder     = sortOrder.Trim();

            SetColumnOptions( columnOptions );
            LoadPriorities();
        }

        public bool ShowFilterCB
        {
            get { return ColumnOptions != null; }
        }

        public List<string> SortList { get; set; } = new List<string>();

        public string SortOrder
        {
            get { return _sortOrder; }
            set
            {
                _sortOrder = value;
                NotifyPropertyChanged( "SortOrder" );
            }
        }

        public bool IsFiltered
        {
            get { return _isFiltered; }
            set
            {
                _isFiltered = value;
                NotifyPropertyChanged( "IsFiltered" );
            }
        }

        private void DisplayDialog()
        {
            switch ( ColumnName.ToUpper() )
            {
                case "REQUEST TYPE" :
                    GetRequestTypesCommand.Execute( ColumnName );
                    break;

                case "WAIT TIME" :
                    GetWaitTimeCommand.Execute( ColumnName );
                    break;

                case "PT NAME": // No Dialog
                    break;

                case "MRTC" :
                    //GetMRTCCommand.Execute( ColumnName );
                    break;

                case "SSN": // No Dialog
                    break;

                case "SCVISIT" :
                    GetSCVisitCommand.Execute( ColumnName );
                    break;

                case "TELEPHONE": // No Dialog
                    break;

                case "PRIORITY" :
                    GetPriorityGroupsCommand.Execute( ColumnName );
                    break;

                case "CID/PREFERRED DATE" :
                    GetCIDPreferredDatesCommand.Execute( ColumnName );
                    break;

                case "ENTERED/RR NO DATE" :
                    GetEnteredRRNoDatesCommand.Execute( ColumnName );
                    break;

                case "REQUESTOR": // No Dialog
                    break;

                case "REQUEST BY": // No Dialog
                    break;

                case "CLINIC/SERVICE" :
                    GetClinicsCommand.Execute( ColumnName );
                    break;

                case "COMMENT": // No Dialog
                    break;
            }
        }

        private void SetColumnOptions( object options )
        {
            switch ( ColumnName.ToUpper() )
            {
                case "REQUEST TYPE" :
                    RequestTypes = (List<RequestType>)options;
                    break;
                case "WAIT TIME" :
                    break;
                case "PT NAME" :
                    break;
                case "SSN" :
                    break;
                case "MRTC" :
                    //MRTCOptions = (List<string>)options;
                    break;
                case "SCVISIT" :
                    SCVisitOptions = (List<string>)options;
                    break;
                case "TELEPHONE" :
                    break;
                case "PRIORITY" :
                    PriorityGroups = (List<PriorityGroup>)options;
                    break;
                case "CID/PREFERRED DATE" :
                    break;
                case "ENTERED/RR NO DATE" :
                    break;
                case "REQUESTOR" :
                    break;
                case "REQUEST BY" :
                    break;
                case "CLINIC/SERVICE" :
                    break;
                case "COMMENT" :
                    break;
            }
        }

        public List<string> SCVisitOptions { get; set; }

        public List<string> MRTCOptions { get; set; }

        public string FilterInfo
        {
            get { return _settings; }
            set
            {
                _settings = value;
                NotifyPropertyChanged( "FilterInfo" );
            }
        }

        public string ColumnName { get; set; }

        public object ColumnOptions { get; set; }

        public List<RequestType> RequestTypes { get; set; }

        public List<PriorityGroup> PriorityGroups { get; set; }

        #region Commands

        public ICommand FilterCommand
        {
            get
            {
                return GetCommand(
                    "FilterCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }
                        DisplayDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetRequestTypesCommand
        {
            get
            {
                return GetCommand(
                    "GetRequestTypesCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new RequestTypesVM
                        {
                            Title = "Request Types",
                            RequestTypes = RequestTypes
                        };

                        var dlg = new RequestTypesV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetWaitTimeCommand
        {
            get
            {
                return GetCommand(
                    "GetWaitTimeCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new WaitTimeVM
                        {
                            Title = "Wait Time",
                        };

                        var dlg = new WaitTimeV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetPriorityGroupsCommand
        {
            get
            {
                return GetCommand(
                    "GetPriorityGroupsCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new PriorityGroupsVM
                        {
                            Title = "Priority Groups",
                            PriorityGroups = PriorityGroups
                        };

                        var dlg = new PriorityGroupsV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetMRTCCommand
        {
            get
            {
                return GetCommand(
                    "GetMRTCCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new YesNoBothVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new YesNoBothV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetSCVisitCommand
        {
            get
            {
                return GetCommand(
                    "GetSCVisitCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new YesNoBothVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new YesNoBothV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetCIDPreferredDatesCommand
        {
            get
            {
                return GetCommand(
                    "GetCIDPreferredDatesCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new DatesLogicVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new DatesLogicV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetEnteredRRNoDatesCommand
        {
            get
            {
                return GetCommand(
                    "GetEnteredRRNoDatesCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new DatesLogicVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new DatesLogicV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        public ICommand GetClinicsCommand
        {
            get
            {
                return GetCommand(
                    "GetClinicsCommand",
                    param =>
                    {
                        if ( !IsFiltered )
                        {
                            FilterInfo = "";
                            return;
                        }

                        var vm = new ClinicListVM
                        {
                            Title = (string)param,
                        };

                        var dlg = new ClinicListV
                        {
                            Context = vm
                        };
                        vm.CloseCallback = result =>
                        {
                            dlg.Close();
                            if ( result != null )
                            {
                                FilterInfo = result;
                            }
                            else
                            {
                                IsFiltered = false;
                            }
                        };
                        dlg.ShowDialog();
                    },
                    _ => true );
            }
        }

        #endregion Commands
    }
}
