﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class UserPreferencesVM : BaseViewModel
    {
        const string DLIM = ";" ;
        private IDataAccessService _dataAccessService;

        private readonly string[] _appointmentRequestTypes =
        {
            "PtCSch",
            "EWL",
            "CONSULT",
            "PROCEDURE",
            "APPT"//,
            //"VETERAN",
            //"MOBILE",
            //"W2VA"
        };

        private ObservableCollection<ColumnSetting> _columnSettings = new ObservableCollection<ColumnSetting>();

        public Action<List<string>> CloseCallback { get; set; }

        public ObservableCollection<ColumnSetting>ColumnSettings
        {
            get { return _columnSettings; }
            set
            {
                _columnSettings = value;
                NotifyPropertyChanged( "ColumnSettings" );
            }
        }

        public UserPreferencesVM(IDataAccessService dataAccessService)
        {
            _dataAccessService = dataAccessService;
            BuildRequestTypes(false);
            BuildPriorityGroups();

            if (( Keys.UserPrefList == null ) || ( Keys.UserPrefList.Count == 0 ))
                BuildColumnSettings();
            else
                LoadUserPreferences( Keys.UserPrefList );
        }

        public Window View { get; set; }

        public void LoadUserPreferences( List<string> userPrefList )
        {
            var colSettings = new ObservableCollection<ColumnSetting>();
            ColumnSetting clinicServiceCol = null;
            ColumnSetting requestTypeCol = null;

            foreach ( var userPreference in userPrefList )
            {
                //  name, options, isFiltered, settings, sortOrder
                var colSetting = new ColumnSetting( _dataAccessService )
                {
                    Name       = ColumnName( userPreference ),          // Column Name
                    IsFiltered = IsFiltered( userPreference ),          // Is active filter
                    FilterInfo = FilterInfo( userPreference ),          // Filter data
                    SortOrder  = SortOrder( userPreference ).ToString() // Sort Order
                };

                colSetting.FilterChangedEvent += HandleFilterChangedEvent;

                switch ( colSetting.Name.ToUpper() ) // Column Options (loaded below)
                {
                    case Keys.RMGridDefaultColumnOrder.REQUEST_TYPE:
                        colSetting.Choices = RequestTypes;
                        colSetting.RequestTypes = RequestTypes;
                        requestTypeCol = colSetting;
                        break;
                    case Keys.RMGridDefaultColumnOrder.WAIT_TIME:
                        colSetting.Choices = WaitTime;
                        break;
                    // artf20768 : 04/06/2017 : RonH : Removed SCVisit as a filter option
                    //case Keys.RMGridDefaultColumnOrder.SCVISIT:
                    //     colSetting.Choices = SCVisit;
                    //     break;
                    case Keys.RMGridDefaultColumnOrder.PRIORITY:
                        colSetting.Choices = PriorityGroups;
                        colSetting.PriorityGroups = PriorityGroups;
                        break;
                    case Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE:
                        colSetting.Choices = PreferredDates;
                        break;
                    case Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE:
                        colSetting.Choices = EnteredRRNoDates;
                        break;
                    case Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE:
                        colSetting.Choices = Clinics;
                        clinicServiceCol = colSetting;
                        break;
                }
                colSetting.LoadPriorities();

                if ((clinicServiceCol != null) && (requestTypeCol != null))
                {
                    BuildRequestTypes(clinicServiceCol.IsFiltered && !clinicServiceCol.IsServiceList);
                    requestTypeCol.Choices = RequestTypes;
                }

                colSettings.Add( colSetting );
            }
            ColumnSettings = colSettings;
        }

        public List<RequestType> RequestTypes { get; set; } = new List<RequestType>();

        private void BuildRequestTypes(bool forClinic)
        {
            RequestTypes.Clear();
            foreach ( var appointmentRequestType in _appointmentRequestTypes )
            {
                if (forClinic && ("CONSULT,PROCEDURE".Contains(appointmentRequestType)))
                    continue;

                RequestTypes.Add( new RequestType { RequestName = appointmentRequestType } );
            }
        }

        public List<PriorityGroup> PriorityGroups { get; set; } = new List<PriorityGroup>();

        private void BuildPriorityGroups()
        {
            PriorityGroups.Clear();
            PriorityGroups.Add( new PriorityGroup { PriorityGroupName = "null" } );
            for ( var i = 1; i <= 8; i++ )
            {
                PriorityGroups.Add( new PriorityGroup { PriorityGroupName = "Group " + i } );
            }
        }

        private void BuildColumnSettings()
        {
            ColumnSettings = new ObservableCollection<ColumnSetting>
            {
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.REQUEST_TYPE, RequestTypes, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.WAIT_TIME, WaitTime, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.PT_NAME, null, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.MRTC, null, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.SSN, null, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.SCVISIT, null, false, "", false, "" ),
                // artf20758: Removeing the SCVisit object from the constructor call insures the SCVisit cannot be filtered.
                //new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.SCVISIT, SCVisit, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.TELEPHONE, null, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.PRIORITY, PriorityGroups, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE, PreferredDates, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE, EnteredRRNoDates, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.REQUESTOR, null, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.REQUESTED_BY, null, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE, Clinics, false, "", false, "" ),
                new ColumnSetting( _dataAccessService, Keys.RMGridDefaultColumnOrder.COMMENT, null, false, "", false, "" ),
            };
        }

        public List<DateTime> EnteredRRNoDates { get; set; } = new List<DateTime>();

        public List<DateTime> PreferredDates { get; set; } = new List<DateTime>();

        //public List<string> SCVisit { get; set; } = new List<string> { "Yes", "No", "Both" };

        public List<string> MRTC { get; set; } = new List<string> {"Yes", "No", "Both"};

        public List<Clinic> Clinics { get; set; } = new List<Clinic>();

        public string WaitTime { get; set; } = "Wait Time";

        public bool SaveAsDefaultView { get; set; }

        public bool Reordered { get; set; }

        // Columnn    C ; col# ; columnName
        public int ColumnNumber( string line )
        {
            return line.Piece( ":", 1 ).Piece( ";", 2 ).Trim().ToIntDef( 0 );
        }

        public string ColumnName( string line )
        {
            return line.Piece( ":", 1 ).Piece( ";", 3 ).Trim();
        }

        // Filter    F ; 1/0 ; filterinfo ; isServiceList
        public bool IsFiltered( string line )
        {
            return line.Piece( ":", 2 ).Piece( ";", 2 ).Trim() == "1";
        }

        public string FilterInfo( string line )
        {
            return line.Piece( ":", 2 ).Piece( ";", 3 ).Trim();
        }

        public bool IsServiceList( string line )
        {
            return line.Piece( ":", 2 ).Piece( ";", 4 ).Trim() == "1";
        }

        // Sort Order   S ; 0,1,2 or 3
        public int SortOrder( string line )
        {
            return line.Piece( ":", 3 ).Piece( ";", 2 ).Trim().ToIntDef( 0 );
        }

        private void HandleFilterChangedEvent(ColumnSetting sender)
        {
            switch (sender.Name)
            {
                case Keys.RMGridDefaultColumnOrder.REQUEST_TYPE:
                    break;
                case Keys.RMGridDefaultColumnOrder.WAIT_TIME:
                    if (!string.IsNullOrEmpty(sender.FilterInfo))
                    {
                        ColumnSetting col = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE));

                        if (col != null)
                        {
                            col.SetFilterInfo(string.Empty);
                            col.IsFiltered = false;
                        }

                        col = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE));

                        if (col != null)
                        {
                            col.SetFilterInfo(string.Empty);
                            col.IsFiltered = false;
                        }
                    }

                    break;
                case Keys.RMGridDefaultColumnOrder.PT_NAME:
                    break;
                case Keys.RMGridDefaultColumnOrder.MRTC:
                    break;
                case Keys.RMGridDefaultColumnOrder.SSN:
                    break;
                case Keys.RMGridDefaultColumnOrder.SCVISIT:
                    break;
                case Keys.RMGridDefaultColumnOrder.TELEPHONE:
                    break;
                case Keys.RMGridDefaultColumnOrder.PRIORITY:
                    break;
                case Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE:
                    if (!string.IsNullOrEmpty(sender.FilterInfo))
                    {
                        ColumnSetting col = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.WAIT_TIME));

                        if (col != null)
                        {
                            col.SetFilterInfo(string.Empty);
                            col.IsFiltered = false;
                        }

                        col = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE));

                        if (col != null)
                        {
                            col.SetFilterInfo(string.Empty);
                            col.IsFiltered = false;
                        }
                    }

                    break;
                case Keys.RMGridDefaultColumnOrder.ENTERED_RR_NO_DATE:
                    if (!string.IsNullOrEmpty(sender.FilterInfo))
                    {
                        ColumnSetting col = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.WAIT_TIME));

                        if (col != null)
                        {
                            col.SetFilterInfo(string.Empty);
                            col.IsFiltered = false;
                        }

                        col = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.CID_PREFERRED_DATE));

                        if (col != null)
                        {
                            col.SetFilterInfo(string.Empty);
                            col.IsFiltered = false;
                        }
                    }

                    break;
                case Keys.RMGridDefaultColumnOrder.REQUESTOR:
                    break;
                case Keys.RMGridDefaultColumnOrder.REQUESTED_BY:
                    break;
                case Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE:
                    ColumnSetting clinicServiceCol = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE));
                    ColumnSetting requestTypeCol = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.REQUEST_TYPE));

                    BuildRequestTypes(clinicServiceCol.IsFiltered && !clinicServiceCol.IsServiceList);
                    requestTypeCol.Choices = RequestTypes;

                    if (clinicServiceCol.IsFiltered && !clinicServiceCol.IsServiceList)
                    {
                        if (!string.IsNullOrEmpty(requestTypeCol.FilterInfo))
                        {
                            string working = requestTypeCol.FilterInfo.Replace(", ", "^");
                            string[] types = working.Split('^');
                            List<string> newList = new List<string>();

                            foreach (string type in types)
                                if ((type != "CONSULT") && (type != "PROCEDURE"))
                                    newList.Add(type);

                            if (newList.Count == 0)
                            {
                                requestTypeCol.FilterInfo = string.Empty;
                                requestTypeCol.IsFiltered = false;
                            }
                            else
                            {
                                requestTypeCol.FilterInfo = string.Join(", ", newList.ToArray());
                                requestTypeCol.IsFiltered = true;
                            }

                            requestTypeCol.IsFiltered = !string.IsNullOrEmpty(requestTypeCol.FilterInfo);
                        }
                    }

                    break;
                case Keys.RMGridDefaultColumnOrder.COMMENT:
                    break;
            }
        }

        private bool ValidateQueryRules()
        {
            // Blue Group is in use ONLY when ALL controls in the grou are selected
            ColumnSetting reqType = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.REQUEST_TYPE));
            ColumnSetting clinics = _columnSettings.FirstOrDefault(c => (c.Name == Keys.RMGridDefaultColumnOrder.CLINIC_SERVICE));

            bool success = (
                    (reqType.IsFiltered && clinics.IsFiltered)
                ||
                    (!reqType.IsFiltered && !clinics.IsFiltered)
            );

            if (!success)
                Keys.AlertUser(View, "Incomplete search criteria selected.\nThe Clinic/Service and Request Type filters MUST be used together or not at all.", "User Preferences Validation");

            return success;
        }

        public ICommand ClearCommand
        {
            get
            {
                return GetCommand(
                    "ClearCommand",
                    _ =>
                    {
                        foreach ( var columnSetting in ColumnSettings )
                        {
                            columnSetting.IsFiltered = false;
                            columnSetting.FilterInfo = "";
                            columnSetting.SortOrder = "";
                        }
                        BuildColumnSettings();
                    },
                    _ => true
                    );
            }
        }

        #region ICloseCommandProvider Members

        public bool IsClosing { get; private set; }

        public ICommand CloseCommand
        {
            get
            {
                return GetCommand(
                    "CloseCommand",
                    _ =>
                    {
                        IsClosing = true;
                        if ( CloseCallback != null )
                        {
                            CloseCallback( null );
                        }
                    },
                    _ => true
                    );
            }
        }

        #endregion ICloseCommandProvider Members

        #region ISaveCommandProvider Members

        public ICommand SaveCommand
        {
            get
            {
                return GetCommand(
                    "SaveCommand",
                    _ =>
                    {
                        if ( CloseCallback != null )
                        {
                            if (!ValidateQueryRules())
                                return;

                            var userPrefs = new List<string>();
                            var columnNum = 0;
                            foreach ( var columnSetting in ColumnSettings )
                            {
                                var up = "C;" + columnNum + ";" + columnSetting.Name + "; : " +
                                         "F;" + ( columnSetting.IsFiltered ? "1" : "0" ) + ";" + columnSetting.FilterInfo + ";" + columnSetting.IsServiceList + "; : " + 
                                         "S;" + columnSetting.SortOrder + ";";
                                userPrefs.Add( up );

                                columnNum++;
                            }
                            CloseCallback( userPrefs );
                        }
                    },
                    _ => true );
            }
        }

        #endregion ISaveCommandProvider Members
    }
}
