﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.Ribbon.ViewModels
{
    public class WaitListQueryFormViewModel : AsyncValidatableModel
    {
        public bool ConsultUrgencyActivated { get; set; }
        public bool IsPtCSchOnly { get; set; }
        public List<ConsultUrgency> ConsultUrgencyValues
        {
            get { return DataAccessService.GetConsultUrgencyValues(); }
        }

        public List<string> ServiceRelatedValues = new List<string>{"YES", "NO", "BOTH"};
        public List<string> ScVisitValues = new List<string> { "YES", "NO", "BOTH" };
        public List<string> SortFields = new List<string> { "Default", "Patient Name", "Clinic", "Request Type", "Wait Time", "Priority Group", "Origination Date", "CID/Preferred Date", "Service Related", "SCVisit" }; 

        private ObservableCollection<AppointmentRequest> _requestPreviewGrid;
        public ObservableCollection<AppointmentRequest> RequestPreviewGrid
        {
            get { return _requestPreviewGrid; }
            set
            {
                _requestPreviewGrid = value;
                NotifyPropertyChanged("RequestPreviewGrid");
            }
        }

        private IList<string> _waitTimes; 
        public IList<string> WaitTimes
        {
            get
            {
                if ( _waitTimes != null )
                    return _waitTimes;

                _waitTimes = new List<string>
                {
                    Keys.RmGridColumns.WaitTimeNull,
                    Keys.RmGridColumns.WaitTimeAll,
                    Keys.RmGridColumns.WaitTimeLessThan30,
                    Keys.RmGridColumns.WaitTimeBetween30And60,
                    Keys.RmGridColumns.WaitTimeBetween60And90,
                    Keys.RmGridColumns.WaitTimeGreaterThanEqualTo90
                };
                return _waitTimes;
            } 
        } 

        public IDataAccessService DataAccessService;
        private IList<Patient> _patientList;
        public AppointmentRequest AppointmentRequest;

        private ObservableCollection<Patient> _selectedPatients;

        public ObservableCollection<Patient> SelectedPatients
        {
            get { return _selectedPatients; }
            set
            {
                if (_selectedPatients != value)
                {
                    _selectedPatients = value;
                    NotifyPropertyChanged("SelectedPatients");
                }
            }
        }

        public bool IsServiceList { get; set; }

        public WaitListQueryFormViewModel( IDataAccessService dataAccessService )
        {
            DataAccessService = dataAccessService;
            AppointmentRequest = new AppointmentRequest(DataAccessService);
            SelectedPatients = new ObservableCollection<Patient>();
            FilteredRequestTypes = new ObservableCollection<RequestType>();
            if(RequestPreviewGrid == null)
                RequestPreviewGrid = new ObservableCollection<AppointmentRequest>();
        }

        public ObservableCollection<ClinicServiceList> FilteredClinics { get; set; }

        public ObservableCollection<RequestType> FilteredRequestTypes { get; set; }

        public ObservableCollection<PriorityGroup> FilteredPriorityGroups { get; set; } 

        public IList<string> PatientListStr { get; set; }

        public IList<Patient> PatientList
        {
            get { return _patientList; }
            set
            {
                if ( _patientList != value )
                {
                    _patientList = value;
                    NotifyPropertyChanged( "PatientList" );
                }
            }
        }

        //public void SearchStringChanged(string newSearchString)
        //{
        //    if (_patientList != null)
        //        _patientList.Clear();

        //    if (newSearchString == null)
        //        return;

        //    if (newSearchString.Length < Keys.PatientSearchCharMin)
        //        return;

        //    var tempPatient = new Patient( DataAccessService )
        //    {
        //        IsAsync = false
        //    };
        //    Dispatcher.Invoke((Action)(() => tempPatient.GetPatients(newSearchString, (s, args) =>
        //    {
        //        lock (args)
        //        {
        //            PatientList = args.Result as IList<Patient>;
        //        }
        //        NotifyPropertyChanged("PatientList");
        //    })));

        //    //PatientList = DataAccessService.GetPatients(newSearchString, 50);
        //    //NotifyPropertyChanged("PatientList");


        //    /* tempPatient.GetPatients(newSearchString, (s, args) =>
        //     {
        //         lock (args)
        //         {
        //             this.PatientList = args.Result as IList<Patient>;
        //         }
        //         NotifyPropertyChanged("PatientList");
        //     });*/
        //}
    }
}
