﻿using System;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Ribbon.ViewModels;

namespace ClinSchd.Modules.Ribbon.Views
{
    /// <summary>
    /// Interaction logic for ClinicListV.xaml
    /// </summary>
    public partial class ClinicListV
    {
        public ClinicListVM ViewModel { get; set; }

        public ClinicListV(IDataAccessService dataAccessService, ClinicListVM viewModel)
        {
            ViewModel = viewModel;
            ViewModel.View = this;
            DataContext = ViewModel;
            InitializeComponent();
        }

        private void CLinicListView_Loaded( object sender, RoutedEventArgs e ) 
        {
            Keys.SetRenderingMode( this );

            SearchTextBox.Focus();
            Keyboard.Focus( SearchTextBox );
        }

        private void SearchTextBox_GotFocus( object sender, RoutedEventArgs e )  //
        {
            BtnOk.IsDefault = false;
            BtnFind.IsDefault = true;
        }

        private void SearchTextBox_LostFocus( object sender, RoutedEventArgs e )  //
        {
            BtnOk.IsDefault = true;
            BtnFind.IsDefault = false;
        }

        private void HotkeyHandler( Key k ) //
        {
            if ( Keyboard.Modifiers == ModifierKeys.Alt )
            {
                switch ( k )
                {
                    case Key.O :
                        Keyboard.Focus( BtnOk );
                        break;

                    case Key.C :
                        Keyboard.Focus( BtnCancel );
                        break;

                    case Key.F :
                        Keyboard.Focus( BtnFind );
                        break;
                }
            }
        }

        private void UserControl_PreviewKeyUp( object sender, KeyEventArgs e ) //
        {
            var key = e.Key == Key.System ? e.SystemKey : e.Key;
            HotkeyHandler( key );
        }

        private void ClinicListGridView_PreviewKeyDown( object sender, KeyEventArgs e ) //
        {
            if ( e.Key == Key.Tab && Keyboard.Modifiers == ModifierKeys.None )
            {
                e.Handled = true;
                Keyboard.Focus( BtnOk );
            }
            if ( Keyboard.Modifiers == ModifierKeys.None && e.Key == Key.Space )
            {
                foreach ( var clinList in ViewModel.ClinicServices )
                {
                    if ( ClinicListGridView.SelectedItem == clinList )
                    {
                        clinList.IsSelected = !clinList.IsSelected;
                    }

                }
            }
        }

        private void ClinicListGridView_DataLoaded( object sender, EventArgs e )
        {
            if ( ClinicListGridView.HasItems )
            {
                ClinicListGridView.SelectedItem = ClinicListGridView.Items[0];
                ClinicListGridView.ScrollIntoView( ClinicListGridView.SelectedItem );
            }
        }

        private void radServices_PreviewKeyDown( object sender, KeyEventArgs e )
        {
            if ( e.Key == Key.Tab && ClinicListGridView.HasItems )
            {
                e.Handled = true;
                ClinicListGridView.SelectedItem = ClinicListGridView.Items[0];
                ClinicListGridView.ScrollIntoView( ClinicListGridView.SelectedItem );
                ClinicListGridView.Focus();
            }
        }

        public void SetSearchTextFocus()
        {
            SearchTextBox.Focus();
            Keyboard.Focus(SearchTextBox);
        }
    }
}