﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Ribbon.ViewModels;

namespace ClinSchd.Modules.Ribbon.Views
{
    /// <summary>
    /// Interaction logic for PriorityGroupListView.xaml
    /// </summary>
    public partial class PriorityGroupListView
    {
        public PriorityGroupListViewModel ViewModel { get; set; }

        public IList<PriorityGroup> FilteredPriorityGroups;

        public PriorityGroupListView()
        {
            InitializeComponent();
            ViewModel = new PriorityGroupListViewModel();
            DataContext = ViewModel;
        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void CloseWindow()
        {
            var window = Window.GetWindow(this);

            if (window != null)
                window.Close();
        }

        private void ChkSelectUnselectAll_Click(object sender, RoutedEventArgs e)
        {
            if (ChkSelectUnselectAll.IsChecked == true)
                foreach (var priGroupList in ViewModel.PriorityGroupSelectionList)
                    priGroupList.IsSelected = true;
            else
                foreach (var priGroupList in ViewModel.PriorityGroupSelectionList)
                    priGroupList.IsSelected = false;

            ViewModel.OnPropertyChanged("PriorityGroupSelectionList");
        }

        private void PriorityGroupListView_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);

            foreach (var priGroupList in ViewModel.PriorityGroupSelectionList)
            {
                foreach (PriorityGroup pgl in FilteredPriorityGroups)
                {
                    if (pgl.PriorityGroupName == priGroupList.PriorityGroupName)
                    {
                        priGroupList.IsSelected = true;
                        break;
                    }
                }
            }

            ViewModel.OnPropertyChanged("PriorityGroupSelectionList");
        }

        private void HotkeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                switch (k)
                {
                    case Key.O:
                        Keyboard.Focus(BtnOk);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnCancel);
                        break;                    
                }
            }
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            Key key = (e.Key == Key.System ? e.SystemKey : e.Key);
            HotkeyHandler(key);
        }

        private void ChkSelectUnselectAll_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Tab && PriorityListViewGrid.HasItems)
            {
                e.Handled = true;
                PriorityListViewGrid.SelectedItem = PriorityListViewGrid.Items[0];
                PriorityListViewGrid.ScrollIntoView(PriorityListViewGrid.SelectedItem);
                PriorityListViewGrid.Focus();
            }
        }

        private void PriorityListViewGrid_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (PriorityListViewGrid.HasItems && (e.Key == Key.Space || e.Key == Key.Enter))
            {
                foreach (var priGroupList in ViewModel.PriorityGroupSelectionList)
                    if (priGroupList == PriorityListViewGrid.SelectedItem)
                        priGroupList.IsSelected = !priGroupList.IsSelected;
            }

            if (e.Key == Key.Tab && Keyboard.Modifiers==ModifierKeys.None)
            {
                e.Handled = true;
                Keyboard.Focus(BtnOk);
            }
            else if (e.Key == Key.Tab && Keyboard.Modifiers == ModifierKeys.Shift)
            {
                e.Handled = true;
                Keyboard.Focus(ChkSelectUnselectAll);
            }
            
        }

        private void RadButton_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Tab && Keyboard.Modifiers == ModifierKeys.None)
                Keyboard.Focus(ChkSelectUnselectAll);
        }
    }
}
