﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Ribbon.ViewModels;

namespace ClinSchd.Modules.Ribbon.Views
{
    /// <summary>
    /// Interaction logic for RequestTypeListView.xaml
    /// </summary>
    public partial class RequestTypeListView : UserControl
    {
        public RequestTypeListViewModel ViewModel { get; set; }
        public IList<RequestType> FilteredRequestTypes;
        public RequestTypeListView()
        {
            InitializeComponent();
            ViewModel = new RequestTypeListViewModel();
            DataContext = ViewModel;

        }

        private void OK_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }

        private void Cancel_Click(object sender, RoutedEventArgs e)
        {
            CloseWindow();
        }
        private void CloseWindow()
        {
            var window = Window.GetWindow(this);
            if (window != null)
                window.Close();
        }

        private void ChkSelectUnselectAll_Click(object sender, RoutedEventArgs e)
        {
            if (ChkSelectUnselectAll.IsChecked == true)
                foreach (var reqList in ViewModel.RequestSelectionList)
                    reqList.IsSelected = true;
            else
                foreach (var reqList in ViewModel.RequestSelectionList)
                    reqList.IsSelected = false;
            ViewModel.OnPropertyChanged("RequestSelectionList");
        }

        private void RequestTypeListView_Loaded(object sender, RoutedEventArgs e)
        {
            Infrastructure.Keys.SetRenderingMode(this);

            foreach (var reqTypeList in ViewModel.RequestSelectionList)
            {
                foreach (RequestType rt in FilteredRequestTypes)
                {
                    if (rt.RequestName == reqTypeList.RequestName)
                    {
                        reqTypeList.IsSelected = true;
                        break;
                    }
                }
            }
            ViewModel.OnPropertyChanged("RequestSelectionList");
            SetTabIndices();
            ChkSelectUnselectAll.Focus();
            Keyboard.Focus(ChkSelectUnselectAll);
        }

        private void HotkeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                switch (k)
                {
                    case Key.O:
                        Keyboard.Focus(BtnOk);
                        break;
                    case Key.C:
                        Keyboard.Focus(BtnCancel);
                        break;
                }
            }
        }

        private void UserControl_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            Key key = (e.Key == Key.System ? e.SystemKey : e.Key);
            HotkeyHandler(key);
        }

        private void RadButton_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Tab && Keyboard.Modifiers == ModifierKeys.None)
            {
                e.Handled = true;
                Keyboard.Focus(ChkSelectUnselectAll);
            }
        }

        private void SetTabIndices()
        {
            ChkSelectUnselectAll.TabIndex = 0;
            if (RequestTypeListViewGrid.HasItems)
            {                
               //RequesttypeListViewGrid.Focus();
            }
            
            BtnOk.TabIndex = 20;
            BtnCancel.TabIndex = 30;
        }

        private void RequestTypeListViewGrid_DataLoaded(object sender, EventArgs e)
        {
            if (RequestTypeListViewGrid.HasItems)
            {
                RequestTypeListViewGrid.SelectedItem = RequestTypeListViewGrid.Items[0];
                RequestTypeListViewGrid.ScrollIntoView(RequestTypeListViewGrid.SelectedItem);
            }
        }

        private void ChkSelectUnselectAll_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            
            if (e.Key==Key.Tab && RequestTypeListViewGrid.HasItems)
            {
                e.Handled = true;
                RequestTypeListViewGrid.SelectedItem = RequestTypeListViewGrid.Items[0];
                RequestTypeListViewGrid.ScrollIntoView(RequestTypeListViewGrid.SelectedItem);
                RequestTypeListViewGrid.Focus();
            }
        }

        private void RequestTypeListViewGrid_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Tab)
            {
                e.Handled = true;
                Keyboard.Focus(BtnOk);
            }
            else if(Keyboard.Modifiers==ModifierKeys.Shift && e.Key == Key.Tab)
            {
                e.Handled = true;
                Keyboard.Focus(ChkSelectUnselectAll);
            }

            if (RequestTypeListViewGrid.HasItems && (e.Key == Key.Space || e.Key == Key.Enter))
            {
                foreach (var reqList in ViewModel.RequestSelectionList)
                {
                    if (reqList == RequestTypeListViewGrid.SelectedItem)
                        reqList.IsSelected = !reqList.IsSelected;
                }
                    
            }
            
        }
    }
}
