using System.Collections.Generic;
using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.AddEditAvailability.Services;

namespace ClinSchd.Modules.Task.AddEditAvailability
{
    public class AddEditAvailabilityPresentationModel : IAddEditAvailabilityPresentationModel, INotifyPropertyChanged
    {
        private readonly ITaskAddEditAvailabilityService _taskAddEditAvailabilityService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private IList<NameValue> _accessGroupList;
        private IList<FindGroupedAccessTypes> _schdGroupedAccessTypes;
        private string _accessGroupIEN = "0";
        private ValidationMessage _validationMessage;
        private SchdAvailability _schdAvailability;
        private string _groupedAccessTypeIEN;

        public AddEditAvailabilityPresentationModel (IAddEditAvailabilityView view,
            ITaskAddEditAvailabilityService taskAddEditAvailabilityService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _taskAddEditAvailabilityService = taskAddEditAvailabilityService;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;
            ValidationMessage = new ValidationMessage ();
            SchdAvailability = new SchdAvailability ();
            InitialErrorMessage ();
            LoadAccessTypeGroups ();
            LoadGroupedAccessTypes ();

            AddEditAvailabilityCommand = new DelegateCommand<string> (ExecuteAddEditAvailabilityCommand, CanExecuteAddEditAvailabilityCommand);
        }

        public void InitialErrorMessage ()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }

        public void ExecuteAddEditAvailabilityCommand (string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;

            string sError = string.Empty;
            _schdAvailability.ACCESSTYPEID = GroupedAccessTypeIEN;
            if (_schdAvailability.SLOTS < 1)
            {
                _schdAvailability.SLOTS = 1;
            }

            if (_schdAvailability.APPOINTMENTID != null)
            {
                sError = _dataAccessService.DeleteAvailability (_schdAvailability);
            }

            if (sError == string.Empty)
            {
                sError = _dataAccessService.AddNewAvailability (SchdAvailability);
            } 

            if (sError != string.Empty)
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "Access Block";
                _validationMessage.Message = sError;
            } else {
                _eventAggregator.GetEvent<RefreshAvailability> ().Publish (command);
                _eventAggregator.GetEvent<RefreshCurrentScheduleEvent>().Publish(string.Empty);
            }
        }

        public bool CanExecuteAddEditAvailabilityCommand (string command)
        {
            return true;
        }

        public void LoadAccessTypeGroups ()
        {
            AccessGroupList = _dataAccessService.GetAccessGroups ();
            AccessGroupList.Insert (0, new NameValue ("<Show All Access Types>", "0"));
            AccessGroupIEN = AccessGroupList[0].Value;

            OnPropertyChanged ("AccessGroupList");
        }

        public void LoadGroupedAccessTypes ()
        {
            SchdGroupedAccessTypes = _dataAccessService.GetFindAccessTypeByGroupID (AccessGroupIEN);
            if (SchdGroupedAccessTypes.Count > 0)
            {
                if ( SchdAvailability.ACCESSTYPEID != null )
                    GroupedAccessTypeIEN = SchdAvailability.ACCESSTYPEID;
                else
                    GroupedAccessTypeIEN = SchdGroupedAccessTypes[0].ACCESS_TYPE_ID;
            }
        }

        public IAddEditAvailabilityView View { get; }
        public DelegateCommand<string> AddEditAvailabilityCommand { get; set; }

        public string GroupedAccessTypeIEN
        {
            get { return _groupedAccessTypeIEN; }
            set
            {
                _groupedAccessTypeIEN = value;
                OnPropertyChanged( "GroupedAccessTypeIEN" );
            }
        }

        public string AccessGroupIEN
        {
            get { return _accessGroupIEN; }
            set
            {
                if ( _accessGroupIEN != value )
                {
                    _accessGroupIEN = value;
                    OnPropertyChanged( "AccessGroupIEN" );
                }
            }
        }

        public IList<FindGroupedAccessTypes> SchdGroupedAccessTypes
        {
            get { return _schdGroupedAccessTypes; }
            set
            {
                if ( _schdGroupedAccessTypes != value )
                {
                    _schdGroupedAccessTypes = value;
                    OnPropertyChanged( "SchdGroupedAccessTypes" );
                }
            }
        }

        public IList<NameValue> AccessGroupList
        {
            get { return _accessGroupList; }
            set
            {
                if ( _accessGroupList != value )
                {
                    _accessGroupList = value;
                    OnPropertyChanged( "AccessGroupList" );
                }
            }
        }

        public SchdAvailability SchdAvailability
        {
            get { return _schdAvailability; }
            set
            {
                if ( _schdAvailability != value )
                {
                    _schdAvailability = value;
                    OnPropertyChanged( "SchdAvailability" );
                }
            }
        }

        public ValidationMessage ValidationMessage
        {
            get { return _validationMessage; }
            set
            {
                _validationMessage = value;
                OnPropertyChanged( "ValidationMessage" );
            }
        }

        public void OnClose ()
        {
            View.Close ();
        }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
