using System;
using System.Collections.Generic;
using System.Linq;
using System.Timers;
using System.Windows.Input;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.Task.Availability.Services;
using ClinSchd.Modules.Task.Scheduler;

namespace ClinSchd.Modules.Task.Availability.Controllers
{
    public class TaskAvailabilityController : ITaskAvailabilityController
    {
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private readonly IDataAccessService _dataAccessService;
        private List<IAvailabilityView> ChildPanes { get; set; }
        private readonly IRegionManager _regionManager;
        private readonly Timer _waitTimer = new Timer(500);

        public TaskAvailabilityController (
            IUnityContainer container,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator,
            IRegionManager regionManager)
        {			
            _container = container;
            _eventAggregator = eventAggregator;
            _dataAccessService = dataAccessService;
            _regionManager = regionManager;
            ChildPanes = new List<IAvailabilityView> ();
            _waitTimer.AutoReset = false;
            _waitTimer.Elapsed += waitTimer_Elapsed;
        }

        public void Run()
        {
            _container.RegisterType<IAvailabilityView, AvailabilityView> ();
            _container.RegisterType<IAvailabilityPresentationModel, AvailabilityPresentationModel> ();
            _container.RegisterType<ITaskAvailabilityService, TaskAvailabilityService> ();
            _eventAggregator.GetEvent<AvailabilityResourcesEvent> ().Subscribe (GetAvailibility, ThreadOption.UIThread, true);
			_eventAggregator.GetEvent<CloseSchedulesEvent> ().Subscribe (CloseSchedulesEvent, ThreadOption.UIThread, true);
        }

        private int _newTabIndex = 1;
        private int _currentTabIndex;

        private bool IsNewSchedule(string scheduleID)
        {
            if (ChildPanes.FirstOrDefault(s => (s.AvailabilityViewModel.ScheduleID == scheduleID)) != null)
                return false;

            CloseSchedules(scheduleID);
            return true;
        }

        public void GetAvailibility (SchdResource selectedResource)
        {
            if (selectedResource == null)
                return;

            if (IsNewSchedule("ResourceAvailability^" + selectedResource.RESOURCEID))
            {
                Mouse.OverrideCursor = Cursors.Wait;
                IAvailabilityView view = _container.Resolve<IAvailabilityView>();
                IAvailabilityPresentationModel presenter = view.AvailabilityViewModel;
                presenter.ScheduleID = "ResourceAvailability^" + selectedResource.RESOURCEID;
                ChildPanes.Add(view);
                presenter.View.TabIndex = _newTabIndex;
                _currentTabIndex = _newTabIndex;
                _newTabIndex++;
                _regionManager.Regions[RegionNames.SystemTabRegion].Add((RadPane)presenter.View);
                //presenter.View.Scheduler_EnableEvents(false);
                //presenter.View.IsDockedInRegion = true;
                presenter.Controller = this;
                presenter.SelectedResource = selectedResource;
                presenter.ResourceName = selectedResource.RESOURCE_NAME;
                presenter.PaneTitle = selectedResource.RESOURCE_NAME + " Availability";
                InitializeGroups(presenter, selectedResource, false);
                InitializePresenter(presenter, selectedResource.Clinic.DISPLAY_INCREMENTS);
            }
        }

        private void InitializePresenter(IAvailabilityPresentationModel presenter, int timeScaleIndex)
        {
            try
            {
                _eventAggregator.GetEvent<SelectTimeScaleEvent>().Publish(timeScaleIndex);
                TimeSpan _timeScale;

                switch (timeScaleIndex)
                {
                    case 1:
                        _timeScale = TimeSpan.FromMinutes(60);
                        presenter.SelectedResource.TIMESCALE = "60";
                        break;
                    case 3:
                        _timeScale = TimeSpan.FromMinutes(20);
                        presenter.SelectedResource.TIMESCALE = "20";
                        break;
                    case 4:
                        _timeScale = TimeSpan.FromMinutes(15);
                        presenter.SelectedResource.TIMESCALE = "15";
                        break;
                    case 6:
                        _timeScale = TimeSpan.FromMinutes(10);
                        presenter.SelectedResource.TIMESCALE = "10";
                        break;
                    case 12:
                        _timeScale = TimeSpan.FromMinutes(5);
                        presenter.SelectedResource.TIMESCALE = "5";
                        break;
                    default:
                        _timeScale = TimeSpan.FromMinutes(30);
                        presenter.SelectedResource.TIMESCALE = "30";
                        break;

                }

                ///*  Day View */
                //VSEDayViewDefinition dayView = new VSEDayViewDefinition();
                //dayView.TimeScale = (int)_timeScale.TotalMinutes;
                ////dayView.GroupHeaderDateStringFormat = "{0:dd dddd}";
                ////dayView.ShowAllDayArea = true;
                ////dayView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(((timeScaleIndex < 3) ? 15 : (int)_timeScale.TotalMinutes), 0, 0, 0, 0));
                ////dayView.TimerulerMinorTickStringFormat = "{0: :mm}";
                ////dayView.MinTimeRulerExtent = 17 * 24 * ((timeScaleIndex < 3) ? 4 : timeScaleIndex);
                ////dayView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(0, 1, 0, 0, 0));

                /* Week View */
                //VSEWeekViewDefinition weekView = new VSEWeekViewDefinition();
                WeekViewDefinition weekView = new WeekViewDefinition();
                //weekView.TimeScale = (int)_timeScale.TotalMinutes;
                //weekView.IsAvailabilityScheduler = true;
                weekView.GroupHeaderDateStringFormat = "{0:dd dddd}";
                weekView.ShowAllDayArea = true;
                weekView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(((timeScaleIndex < 3) ? 15 : (int)_timeScale.TotalMinutes), 0, 0, 0, 0));
                weekView.TimerulerMinorTickStringFormat = "{0: :mm}";
                weekView.MinTimeRulerExtent = 17 * 24 * ((timeScaleIndex < 3) ? 4 : timeScaleIndex);
                weekView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(0, 1, 0, 0, 0));
                weekView.MaxTimeRulerExtent = 15000;

                ///* Timeline View */
                //TimelineViewDefinition timelineView = new TimelineViewDefinition();
                //timelineView.TimerulerGroupStringFormat = "";
                //timelineView.TimerulerMajorTickStringFormat = "{0:ddd MM/dd/yyyy}";
                //timelineView.TimerulerMinorTickStringFormat = "";
                //timelineView.ApplyStartEndTimeToEachDay = true;
                //timelineView.GroupTickLength = new FixedTickProvider(new DateTimeInterval(0, 0, 1, 0, 0));
                //timelineView.MajorTickLength = new FixedTickProvider(new DateTimeInterval(0, 0, 1, 0, 0));
                //timelineView.MinorTickLength = new FixedTickProvider(new DateTimeInterval(0, 0, 1, 0, 0));
                ////timelineView.MinTimeRulerExtent = 17 * 24 * ((timeScaleIndex < 3) ? 4 : timeScaleIndex);
                //timelineView.MaxTimeRulerExtent = 5000;

                ///* Month View */
                //VSEMonthViewDefninition monthView = new VSEMonthViewDefninition();
                //// Changes the Top Daily Header
                /////monthView.GroupHeaderDateStringFormat = "{0:dddd}";
                //// Changes the Lower Daily Date Header
                ////monthView.TimerulerMajorTickStringFormat = "{0:d}";
                //// Changes the vertical date range header
                ////monthView.TimerulerGroupStringFormat = "{0:MM/dd}";


                presenter.View.Scheduler.ViewDefinitions.Clear();
                //presenter.View.Scheduler.ViewDefinitions.Add(dayView);
                presenter.View.Scheduler.ViewDefinitions.Add(weekView);
                //presenter.View.Scheduler.ViewDefinitions.Add(monthView);
                //presenter.View.Scheduler.ViewDefinitions.Add(timelineView);
                presenter.View.Scheduler.ActiveViewDefinition = weekView;

                presenter.View.Scheduler.MinTimeRulerExtent = 20 * 24 * timeScaleIndex;
                presenter.RefreshAvailabilityScheduler("INITIAL_LAUNCH");
            }
            finally
            {
                Mouse.OverrideCursor = Cursors.Arrow;
            }
        }

        private int InitializeGroups(IAvailabilityPresentationModel presenter, SchdResource selectedResource, bool createGroups = true)
        {
            int timeScaleIndex = 0;

            // Create Resource Types
            ResourceType resourceType = new ResourceType(Keys.AppointmentGroupNames.Appointment);
            timeScaleIndex = Math.Max(timeScaleIndex, selectedResource.Clinic.DISPLAY_INCREMENTS);
            resourceType.Resources.Add(new Resource(selectedResource.ScheduleTabCaption));
            presenter.ResourceTypesCollection.Add(resourceType);

            // Create grouping beyond date grouping the groups if needed
            if (createGroups)
            {
                ResourceGroupDescription groupDescription = new ResourceGroupDescription()
                {
                    ResourceType = Keys.AppointmentGroupNames.Appointment
                };

                presenter.GroupsDescriptionCollection.Insert(0, groupDescription);
            }

            return timeScaleIndex;
        }

        public void RemoveViewFromList (IAvailabilityView view)
        {
            if (ChildPanes.ToString().Contains("IAvailabilityView"))
                _eventAggregator.GetEvent<ClearClinicInfoEvent>().Publish("");
            ChildPanes.Remove (view);
        }

        private void CloseSchedulesEvent(string command)
        {
            if (command != "CLOSE_AVAILABILITY")
                return;

            IAvailabilityView view = null;

            if (ChildPanes.Count == 0)
                return;

            if (ChildPanes.Count == 1)
                view = ChildPanes[0];
            else
                view = ChildPanes.FirstOrDefault(i => i.IsSelected);

            if (view != null)
                CloseSchedules(view.Model.SelectedResource.Clinic.HOSPITAL_LOCATION_ID);
        }

        private void CloseSchedules (string clinicIEN)
        {
            var openPanes = new List<IAvailabilityView> ();
            foreach (var item in ChildPanes) {
                if (item.Model.SelectedResource.Clinic.HOSPITAL_LOCATION_ID == clinicIEN) {
                    openPanes.Add (item);
                    break;
                }
            }
            foreach (var item in openPanes) {
                RemoveViewFromList (item);
                item.Close ();
            }
        }

        private void waitTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _waitTimer.Stop();
            _eventAggregator.GetEvent<RibbonBarSelectedTabChangedEvent>().Publish("AVAILABILITY^" + _currentTabIndex);
        }

    }
}
