using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;

using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.EmptyScheduler;
using ClinSchd.Modules.Task.EmptyScheduler.Services;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Task.Group;

namespace ClinSchd.Modules.Task.EmptyScheduler.Controllers
{
    public class TaskEmptySchedulerController : ITaskEmptySchedulerController
    {
		private readonly IUnityContainer container;
		private readonly IEventAggregator eventAggregator;
		private readonly IRegionManager regionManager;
		private readonly Factory<SchdResource> resourceFactory;
		private readonly IDataAccessService dataAccessService;
		private List<IEmptySchedulerView> ChildPanes { get; set; }

		public TaskEmptySchedulerController(
			IUnityContainer container,
			IDataAccessService dataAccessService,
			IEventAggregator eventAggregator,			
			IRegionManager regionManager,
			Factory<SchdResource> resourceFactory)
        {			
			this.container = container;
            this.eventAggregator = eventAggregator;
			this.dataAccessService = dataAccessService;
			this.regionManager = regionManager;
			this.resourceFactory = resourceFactory;
			ChildPanes = new List<IEmptySchedulerView> ();
		}

		public void Run()
		{
			this.container.RegisterType<IEmptySchedulerView, EmptySchedulerView>();
			this.container.RegisterType<IEmptySchedulerPresentationModel, EmptySchedulerPresentationModel>();
			this.container.RegisterType<ITaskEmptySchedulerService, TaskEmptySchedulerService>();

			this.eventAggregator.GetEvent<SchedulerDisplayEmptyPaneEvent> ().Subscribe (EmptySchedulerDisplay, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayEvent>().Subscribe(SchedulerDisplay, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayGroupEvent>().Subscribe(SchedulerDisplayGroups, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<SchedulerDisplayForProvidersEvent>().Subscribe(SchedulerDisplayGroups, ThreadOption.UIThread, true);
        }

        public void EmptySchedulerDisplay (string message)
		{
			IEmptySchedulerPresentationModel presenter = this.container.Resolve<IEmptySchedulerPresentationModel> ();
			this.regionManager.Regions[RegionNames.TaskGroupRegion].Add (presenter.View);
			presenter.Controller = this;
			presenter.Message = message;
			presenter.SelectedResource = resourceFactory.Create ();
            presenter.PaneTitle = message.Replace("There are no Clinics associated with ", "");
            ChildPanes.Add (presenter.View);
		}

		public void RemoveViewFromList (IEmptySchedulerView View)
		{
            if (ChildPanes.Contains(View))
                ChildPanes.Remove(View);

            if (this.regionManager.Regions[RegionNames.TaskGroupRegion].Views.Contains(View))
                this.regionManager.Regions[RegionNames.TaskGroupRegion].Remove(View);
        }

        private void SchedulerDisplay(SchdResource selectedResource)
        {
            CloseSchedules();
        }

        private void SchedulerDisplayGroups(SchdResourceGroup selectedResourceGroup)
        {
            CloseSchedules();
        }

        private void CloseSchedules()
        {
            List<IEmptySchedulerView> openPanes = new List<IEmptySchedulerView>();
            foreach (IEmptySchedulerView item in ChildPanes)
                openPanes.Add(item);

            foreach (IEmptySchedulerView pane in openPanes)
                RemoveViewFromList(pane);
        }
    }
}
