using System.ComponentModel;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.Group;
using ClinSchd.Modules.Task.EmptyScheduler.Controllers;

namespace ClinSchd.Modules.Task.EmptyScheduler
{
    public class EmptySchedulerPresentationModel : IEmptySchedulerPresentationModel, ITaskPresentationModel, INotifyPropertyChanged
    {
        private readonly IRegionManager _regionManager;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private string _paneTitle = "";
        private bool _isSelectedTab;
        private SchdResource _selectedResource;
        private SchdResourceGroup _selectedResourceGroup;
        private bool _isEnableUpdateAppointment = !Keys.LoggedInUserSDECViewOnlyKey;

        public EmptySchedulerPresentationModel(
            IEmptySchedulerView view,
            IRegionManager regionManager,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _regionManager = regionManager;
            _dataAccessService = dataAccessService;
            _eventAggregator = eventAggregator;

            ResetUnselectedTabCommand = new DelegateCommand<string> (ExecuteEnableAppointmentButtonsCommand, CanExecuteEnableAppointmentButtonsCommand);
            TaskTabSelectedCommand = new DelegateCommand<object> (ExecuteEmptySchedulerTabSelectedCommand, CanExecuteEmptySchedulerTabSelectedCommand);
        }

        public string Message { get; set; }
        public DelegateCommand<string> ResetUnselectedTabCommand { get; }
        public DelegateCommand<object> TaskTabSelectedCommand { get; }

        public void ExecuteEmptySchedulerTabSelectedCommand (object resource)
        {
            SelectedResource = resource as SchdResource;
            _eventAggregator.GetEvent<ResourceSelectedEvent> ().Publish (SelectedResource);
            OnPropertyChanged ("Message");
        }

        public void ExecuteEnableAppointmentButtonsCommand (string command)
        {
        }

        public bool CanExecuteEmptySchedulerTabSelectedCommand (object resource)
        {
            return true;
        }

        public bool CanExecuteEnableAppointmentButtonsCommand (string command)
        {
            return true;
        }

        #region IEmptySchedulerPresentationModel Members

        public IEmptySchedulerView View { get; }

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        public bool IsEnableUpdateAppointment
        {
            get { return _isEnableUpdateAppointment; }
            set
            {
                if ( _isEnableUpdateAppointment != value )
                {
                    _isEnableUpdateAppointment = value;
                    OnPropertyChanged( "IsEnableUpdateAppointment" );
                }
            }
        }

        public bool IsSelectedTab
        {
            get { return _isSelectedTab; }
            set
            {
                _isSelectedTab = value;
                OnPropertyChanged( "IsSelectedTab" );
            }
        }

        public SchdResource SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                if ( _selectedResource != value && value != null )
                {
                    _selectedResource = value;
                    OnPropertyChanged( "SelectedResource" );
                }
            }
        }

        public SchdResourceGroup SelectedResourceGroup
        {
            get { return _selectedResourceGroup; }
            set
            {
                if ( _selectedResourceGroup != value && value != null )
                {
                    _selectedResourceGroup = value;
                    OnPropertyChanged( "SelectedResourceGroup" );
                }
            }
        }

        public ITaskEmptySchedulerController Controller { get; set; }
        public void ClearViewFromList ()
        {
            if (Controller != null) {
                Controller.RemoveViewFromList (View);
            }
        }
        #endregion

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }
    }
}
