﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Timers;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.Group;
using ClinSchd.Modules.Task.MultiScheduler;
using ClinSchd.Modules.Task.Scheduler;
using DevComponents.WpfSchedule;
using DevComponents.WpfSchedule.Model;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;
using Microsoft.Practices.Unity;
using Telerik.Windows.Controls;
using Appointment = DevComponents.WpfSchedule.Model.Appointment;

namespace ClinSchd.Modules.Task.ChildModules.MultiScheduler
{
    /// <summary>
    /// Interaction logic for ClinicGroupMultiView.xaml
    /// </summary>
    public partial class ClinicGroupMultiView : IClinicGroupMultiView, IDisposable
    {
        public IUnityContainer Container { get; set; }
        public IRegionManager RegionManager { get; set; }
        public IEventAggregator EventAggregator { get; set; }
        public IMultiSchedulerPresentationModel GroupPresentationModel { get; set; }

        public IDataAccessService DataAccessService { get; set; }
        public SchdResourceGroup ResourceGroup;
        public Dictionary<string, ISchedulerPresentationModel> Presenters = new Dictionary<string, ISchedulerPresentationModel>(); 
        CalendarModel _calendarModel;
        private bool _windowLoaded;
        CalendarHitTestResult _curSlot;
        //private bool _initialLaunch = true;
        public IClinicGroupMultiView View { get; private set; }
        private string _patientId = "";
        private AppointmentView _appointmentToAdd;
        private eAppointmentChangeType _appointmentEditAction;
        private SchdAppointment _preEditAppointment;
        private string _preEditOwnerKey;
        private DateTime _lastScheduleUpdate = DateTime.MinValue;
        private Timer _refreshTimer;        
        private RoutedEventHandler _onBeforeAppointmentDragHandler;
//        private RoutedEventHandler _appointmentChangingEventHandler;
        private RoutedEventHandler _appointmentChangedEvent;
        public string Name { get; set; }
        private bool _disposing;
        //private Guid _internalID = Guid.NewGuid();
        public bool _respondToEvents;

        public ClinicGroupMultiView(IEventAggregator eventAggregator)
        {
            InitializeComponent();
            _refreshTimer = new Timer(500);
            _refreshTimer.Elapsed += refreshTimer_Elapsed;
            _calendarModel = new CalendarModel();

            var startTime = new WorkTime(0, 0);
            var endTime  = new WorkTime(23, 59);

            foreach(var workDay in _calendarModel.WorkDays)
            {
                workDay.WorkStartTime = startTime;
                workDay.WorkEndTime = endTime;
            }

            // Assign model to the CalendarView so view has data to display
            calendarView1.CalendarModel = _calendarModel;

            if(Keys.ScheduleRequestType == "Provider")
                calendarView1.SelectedView = eCalendarView.Timeline;
            if(Keys.ScheduleRequestType == "ClinicGroups")
                calendarView1.SelectedView = eCalendarView.Day;
            
            DevComponents.WpfEditors.EditColors.ChangeColorScheme(DevComponents.WpfEditors.eEditorsVisualStyle.Office2010Blue);
            EventAggregator = eventAggregator;
            RespondToEvents(true);

            _onBeforeAppointmentDragHandler = OnBeforeAppointmentDrag;
            _appointmentChangedEvent = OnAppointmentChanged;

            calendarView1.AddHandler(AppointmentView.BeforeAppointmentDragEvent, _onBeforeAppointmentDragHandler);
            calendarView1.AddHandler(AppointmentView.AppointmentChangedEvent, _appointmentChangedEvent);
        }

        public void RespondToEvents(bool respond)
        {
            _respondToEvents = respond;

            if (respond)
            {
                EventAggregator.GetEvent<SchedulerViewDateEvent>().Subscribe(UpdateSchedulerViewDate, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<AppointmentAddedEvent>().Subscribe(AddAppointment, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<AppointmentCancelledEvent>().Subscribe(CancelAppointment, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<AppointmentCheckInEvent>().Subscribe(CheckInAppointment, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<AppointmentNoShowEvent>().Subscribe(MarkAsNoShow, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<AppointmentCheckOutEvent>().Subscribe(AppointmentCheckOut, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<UndoPatientCheckInSuccessEvent>().Subscribe(UndoCheckInPatientSuccess, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<UndoNoShowSuccessEvent>().Subscribe(UndoNoShowSuccess, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<UndoCheckOutPatientSuccessEvent>().Subscribe(UndoCheckOutPatientSuccess, ThreadOption.UIThread, false);
                EventAggregator.GetEvent<UndoCancelAppointmentSuccessEvent>().Subscribe(UndoCancelAppointmentSuccess, ThreadOption.UIThread, false);
            }
            else
            {
                EventAggregator.GetEvent<SchedulerViewDateEvent>().Unsubscribe(UpdateSchedulerViewDate);
                EventAggregator.GetEvent<AppointmentAddedEvent>().Unsubscribe(AddAppointment);
                EventAggregator.GetEvent<AppointmentCancelledEvent>().Unsubscribe(CancelAppointment);
                EventAggregator.GetEvent<AppointmentCheckInEvent>().Unsubscribe(CheckInAppointment);
                EventAggregator.GetEvent<AppointmentNoShowEvent>().Unsubscribe(MarkAsNoShow);
                EventAggregator.GetEvent<AppointmentCheckOutEvent>().Unsubscribe(AppointmentCheckOut);
                EventAggregator.GetEvent<UndoPatientCheckInSuccessEvent>().Unsubscribe(UndoCheckInPatientSuccess);
                EventAggregator.GetEvent<UndoNoShowSuccessEvent>().Unsubscribe(UndoNoShowSuccess);
                EventAggregator.GetEvent<UndoCheckOutPatientSuccessEvent>().Unsubscribe(UndoCheckOutPatientSuccess);
                EventAggregator.GetEvent<UndoCancelAppointmentSuccessEvent>().Unsubscribe(UndoCancelAppointmentSuccess);
            }
        }

        public void ShutDownPresenters()
        {
            // Since the Model\myModel presentation model is also in this list it will be shut down here as well
            foreach (var kvp in Presenters)
            {
                kvp.Value.RespondToEvents(false);
                kvp.Value.IsSelectedTab = false;
            }
        }

        private void OnAppointmentChanged(object sender, RoutedEventArgs e)
        {
            var args = (AppointmentChangedRoutedEventArgs)e;

            switch (args.ChangeType)
            {
                case eAppointmentChangeType.AppointmentResized:
                    args.Cancel = true;
                    break;
                case eAppointmentChangeType.OwnerChanged:
                case eAppointmentChangeType.AppointmentMoved:
                    _appointmentToAdd = (AppointmentView)args.OriginalSource;
                    _appointmentEditAction = args.ChangeType;
                    break;
            }
        }

/*        private void OnAppointmentChanging(object sender, RoutedEventArgs e)
        {
        }
//*/

        private void OnBeforeAppointmentDrag(object sender, RoutedEventArgs e)
        {
            // Needs to save original appointment
            var appointmentToCancel = (AppointmentView)e.OriginalSource;
            TraceLog.LogInfo(GetType().Name, "OnBeforeAppointmentDrag 1", appointmentToCancel.Appointment.OwnerKey + "  " + appointmentToCancel.StartTime.Value + "  " + appointmentToCancel.EndTime.Value);

            if (_preEditAppointment == null)
            {
                _preEditOwnerKey = appointmentToCancel.Appointment.OwnerKey;
                _preEditAppointment = SchdAppointmentFromAppointmentView(appointmentToCancel);
                TraceLog.LogInfo(GetType().Name, "OnBeforeAppointmentDrag 2", _preEditAppointment.RESOURCENAME + "  " + _preEditAppointment.START_TIME + "  " + _preEditAppointment.END_TIME);
                return;
            }

            e.Handled = true;
        }

        private void UndoCancelAppointmentSuccess(string appointmentId)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.NewAppointmentColor).Color.ToString();
                    break;
                }
        }

        private void UndoCheckOutPatientSuccess(string appointmentId)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    var schdAppt = FindSchdAppointment(appointmentId);

                    if (schdAppt == null)
                        appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.CheckedInColor).Color.ToString();
                    else
                    {
                        if (schdAppt.AppointmentStatus == AppointmentStatuses.WalkIn_CheckdIn)
                            appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.WalkInAppointmentColor).Color.ToString();
                        else
                            appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.CheckedInColor).Color.ToString();
                    }

                    break;
                }
        }

        private void UndoNoShowSuccess(string appointmentId)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.NewAppointmentColor).Color.ToString();
                    break;
                }
        }

        private void UndoCheckInPatientSuccess(string appointmentId)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    var schdAppt = FindSchdAppointment(appointmentId);

                    if (schdAppt == null)
                        appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.NewAppointmentColor).Color.ToString();
                    else
                    {
                        if (schdAppt.AppointmentStatus == AppointmentStatuses.WalkIn_NotCheckedIn)
                            appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.WalkInAppointmentColor).Color.ToString();
                        else
                            appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.NewAppointmentColor).Color.ToString();
                    }

                    break;
                }
        }

        private void AppointmentCheckOut(string appointmentId)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.CheckedOutColor).Color.ToString();
                    break;
                }
        }

        private void MarkAsNoShow(string appointmentId)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.NoShowColor).Color.ToString();
                    break;
                }
        }

        private void CheckInAppointment(string appointmentId)
        {
            if (_disposing)
                return;

            foreach (var appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    var schdAppt = FindSchdAppointment(appointmentId);

                    if (schdAppt == null)
                        appt.CategoryColor = ((SolidColorBrush) Keys.ColorCodes.CheckedInColor).Color.ToString();
                    else
                    {
                        if (schdAppt.AppointmentStatus == AppointmentStatuses.WalkIn_CheckdIn)
                            appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.WalkInAppointmentColor).Color.ToString();
                        else
                            appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.CheckedInColor).Color.ToString();
                    }

                    break;
                }
        }

        private void CancelAppointment(string payload)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            var appointmentId = payload;

            if (payload.Contains('^'))
            {
                var parts = payload.Split('^');
                appointmentId = parts[0];
            }

            var removedAppointments = 0;
            for (var index = _calendarModel.Appointments.Count - 1; index >= 0; index--)
            {
                var appt = _calendarModel.Appointments[index];
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    _calendarModel.Appointments.Remove(appt);
                    removedAppointments++;
                }
            }
            if (removedAppointments > 0)
                PopulateAccessBlocks();

            /*Original Implementation...................................
            foreach(Appointment appt in _calendarModel.Appointments)
                if (appt.Id == Convert.ToInt32(appointmentId))
                {
                    appt.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.CancelledAppointmentColor).Color.ToString();
                    break;
                }
             */
        }

        private void UpdateSchedulerViewDate(SchedulerViewDateEventArgs dayToView)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            if (!_iChangedTheDate && (dayToView.Sender != "MultiScheduler.ClinicGroupMultiView"))
            {
                if (_windowLoaded)
                {
                    DoRefreshScheduler();
                }
            }
        }

        private SchedulerPresentationModel myModel
        {
            get { return Model as SchedulerPresentationModel; }
            set { Model = value; }
        }

        public ITaskPresentationModel Model
        {
            get { return DataContext as SchedulerPresentationModel; }
            set { DataContext = value; }
        }

        public ViewModes ViewMode
        {
            get
            {
                switch (calendarView1.SelectedView)
                {
                    case eCalendarView.Day:
                        return ViewModes.Day;

                    case eCalendarView.Week:
                        return ViewModes.Week;

                    case eCalendarView.Month:
                        return ViewModes.Month;

                    case eCalendarView.Timeline:
                        return ViewModes.TimeLine;
                }

                return ViewModes.None;
            }
            private set
            {
                switch (value)
                {
                    case ViewModes.Day:
                        calendarView1.SelectedView = eCalendarView.Day;
                        break;

                    case ViewModes.Week:
                        calendarView1.SelectedView = eCalendarView.Week;
                        break;

                    case ViewModes.Month:
                        calendarView1.SelectedView = eCalendarView.Month;
                        break;

                    case ViewModes.TimeLine:
                        calendarView1.SelectedView = eCalendarView.Timeline;
                        break;

                    case ViewModes.None:
                        calendarView1.SelectedView = eCalendarView.None;
                        break;
                }
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (!_windowLoaded)
            {
                _windowLoaded = true;
                DoRefreshScheduler();
            }
        }

        public Guid LoadingID { get; set; }

        private void refreshTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _refreshTimer.Stop();
            Dispatcher.BeginInvoke((Action)DoRefreshScheduler);
        }

        private void DoRefreshScheduler()
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            var allSchedulesUpdated = true;
            var lastUpdate = DateTime.MinValue;

            foreach (var kvp in Presenters)
            {
                if (kvp.Value.LastUpdate > lastUpdate)
                    lastUpdate = kvp.Value.LastUpdate;

                allSchedulesUpdated = lastUpdate > _lastScheduleUpdate;

                if (!allSchedulesUpdated)
                    break;
            }

            if (allSchedulesUpdated)
            {
                _lastScheduleUpdate = lastUpdate;
                RefreshScheduler();
            }
            else
            {
                _refreshTimer.Start();
            }
        }

        private int _timeToBeginDisplay;

        private int _timeSlotDuration;

        private void ReadClinsetDisplayData(SchdResource resource)
        {
            var timeToBeginDisplay = 8;
            var timeSlotDuration = 60;

            if (resource.Clinic != null)
            {
                var timeToBeginDisplayText = resource.Clinic.HOUR_DISPLAY_BEGIN;
                var timeSlotDurationIndex = resource.Clinic.DISPLAY_INCREMENTS;

                if (Keys.Methods.IsNumeric(timeToBeginDisplayText))
                    timeToBeginDisplay = Convert.ToInt32(timeToBeginDisplayText);

                timeSlotDuration = 60 / timeSlotDurationIndex;
            }

            _timeToBeginDisplay = Math.Min(_timeToBeginDisplay, timeToBeginDisplay);
            _timeSlotDuration = Math.Min(_timeSlotDuration, timeSlotDuration);
        }

        public string ScheduleType { get; set; }

        private void RefreshScheduler()
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            if (ResourceGroup == null)
                return;

            calendarView1.SelectedViewChanged -= CalendarView1_OnSelectedViewChanged;
            calendarView1.DateSelectionChanged -= CalendarView1_OnDateSelectionChanged;
            calendarView1.UserDateChanged -= UserDateChanged_Event;
            _timeToBeginDisplay = 24;
            _timeSlotDuration = 90;

            try
            {
                _calendarModel.Appointments.Clear();
                calendarView1.DisplayedOwners.Clear();

                try
                {
                    foreach (var res in ResourceGroup.Resources)
                    {
                        ReadClinsetDisplayData(res);
                        string displayOwner;

                        if (res.IsProvidersSchedule)
                            displayOwner = res.GroupResourceName + "-" + res.RESOURCE_NAME;
                        else
                            displayOwner = res.RESOURCE_NAME;

                        if (ViewMode != myModel.ViewMode)
                            ViewMode = myModel.ViewMode;

                        switch (calendarView1.SelectedView)
                        {
                            case eCalendarView.Day:
                                calendarView1.DayViewDate = myModel.VisibleStartDate;
                                break;

                            case eCalendarView.Week:
                                calendarView1.WeekViewStartDate = myModel.VisibleStartDate;
                                calendarView1.WeekViewEndDate = myModel.VisibleEndDate;
                                break;

                            case eCalendarView.Month:
                                calendarView1.MonthViewStartDate = myModel.ActiveMonthStartDate;
                                calendarView1.MonthViewEndDate = myModel.ActiveMonthStartDate.AddMonths(1).AddDays(-1);
                                break;

                            case eCalendarView.Timeline:
                                calendarView1.TimelineViewStartDate = myModel.VisibleStartDate;
                                break;

                            case eCalendarView.None:
                                break;
                        }

                        if (!calendarView1.DisplayedOwners.Contains(displayOwner))
                            calendarView1.DisplayedOwners.Add(displayOwner);

                        var presenter = FindPresenter(displayOwner);
                        
                        if (presenter.AppointmentList != null)
                        {
                            foreach (var appt in presenter.AppointmentList)
                            {
                                _patientId = appt.PATIENTID;
                                var appoint = new Appointment
                                {
                                    Id                                   = Convert.ToInt32( appt.APPOINTMENTID ),
                                    Subject                              = appt.PATIENTNAME, /*+
                                              "  SSN: " + ( appt.Patient == null ? string.Empty : appt.Patient.SSN4 ) +
                                              "  DOB: " + ( appt.Patient == null ? string.Empty : appt.Patient.DOBMasked ), */
                                    StartTime                            = Convert.ToDateTime( appt.START_TIME ),
                                    EndTime                              = Convert.ToDateTime( appt.END_TIME ),
                                    CategoryColor                        = ( (SolidColorBrush)Keys.ColorCodes.CategoriesList[appt.CategoryColorName] ).Color.ToString(),
                                    Description                          =
//                                        appt.PATIENTNAME +
                                        "\nSSN: " + (appt.Patient == null ? string.Empty : appt.Patient.SSN4) +
                                        "\nDOB: " + (appt.Patient == null ? string.Empty : appt.Patient.DOBMasked) +
                                        "\n" + appt.NOTE,
                                    Locked                               = false,
                                    OwnerKey                             = displayOwner,
                                    StyleResourceName                    = new ComponentResourceKey( typeof( ClinicGroupMultiView ), "AppointmentTooltip" ),
                                    Tag                                  = _patientId
                                };
                                // Lock appointment so it cannot be modified by end-user through UI
                                // Assign the Custom Style to the views that are created for this Appointment
                                // Add appointment to the model
                                _calendarModel.Appointments.Add(appoint);

                                // Make appointment visible in current view
                                calendarView1.EnsureVisible(appoint);
                            }
                        }

                        // Special Instructions Indicator
                        if (!string.IsNullOrEmpty(res.Clinic.SpecialInstructions))
                        {
                            var siAppoint = new Appointment
                            {
                                Subject           = "Special Instructions",
                                StartTime         = myModel.VisibleStartDate,
                                EndTime           = myModel.VisibleEndDate.AddMinutes( 1 ),
                                CategoryColor     = ( (SolidColorBrush)Keys.ColorCodes.SpecialInstructionsColor ).Color.ToString(),
                                Tag               = res.Clinic.SpecialInstructions,
                                Locked            = true,
                                OwnerKey          = displayOwner,
                                StyleResourceName = new ComponentResourceKey( typeof( ClinicGroupMultiView ), "AppointmentTooltip" )
                            };
                            // Lock appointment so it cannot be modified by end-user through UI
                            // Assign the Custom Style to the views that are created for this Appointment
                            // Add appointment to the model
                            _calendarModel.Appointments.Add(siAppoint);
                            // Make appointment visible in current view
                            calendarView1.EnsureVisible(siAppoint);
                        }

                        // Variable Length Indicator
                        var vlAppoint = new Appointment
                        {
                            Subject = string.Format(
                                "{0}{1} min Appt{2}",
                                res.Clinic.VariableAppointmentLength ? "VL " : string.Empty,
                                res.Clinic.Duration,
                                string.IsNullOrEmpty( res.Clinic.OVERBOOK ) ? string.Empty : " | Max OB: " + res.Clinic.OVERBOOK
                            ),
                            StartTime = myModel.VisibleStartDate,
                            EndTime = myModel.VisibleEndDate.AddMinutes( 1 ),
                            CategoryColor = ( (SolidColorBrush)Keys.ColorCodes.VariableLengthApptColor ).Color.ToString(),
                            Locked = true,
                            OwnerKey = displayOwner,
                            StyleResourceName = new ComponentResourceKey( typeof( ClinicGroupMultiView ), "Appointment" )
                        };
                        // Lock appointment so it cannot be modified by end-user through UI
                        // Assign the Custom Style to the views that are created for this Appointment
                        // Add appointment to the model
                        _calendarModel.Appointments.Add(vlAppoint);
                        // Make appointment visible in current view
                        calendarView1.EnsureVisible(vlAppoint);

                        // Holiday indicator
                        var hStart = myModel.VisibleStartDate.Date;

                        do
                        {
                            var sHoliday = DataAccessService.IsHoliday(hStart);
                            var abbrevHoliday = sHoliday;

                            if (!string.IsNullOrEmpty(sHoliday))
                            {
                                // Using only first wod of holiday name
                                if (sHoliday.Contains(" "))
                                    abbrevHoliday = sHoliday.Split(' ')[0];

                                var holidayEvent = new Appointment
                                {
                                    Subject           = abbrevHoliday,
                                    Description       = sHoliday + " Holiday",
                                    StartTime         = hStart,
                                    EndTime           = hStart.AddDays( 1 ),
                                    CategoryColor     = ( (SolidColorBrush)Keys.ColorCodes.HolidayNotWorkingColor ).Color.ToString(),
                                    Locked            = true,
                                    OwnerKey          = displayOwner,
                                    StyleResourceName = new ComponentResourceKey( typeof( ClinicGroupMultiView ), "Appointment" ),
                                    
                                };
                                // Lock appointment so it cannot be modified by end-user through UI
                                // Assign the Custom Style to the views that are created for this Appointment
                                // Add appointment to the model
                                _calendarModel.Appointments.Add(holidayEvent);
                                // Make appointment visible in current view
                                calendarView1.EnsureVisible(holidayEvent);
                            }

                            hStart = hStart.AddDays(1);
                        } while (hStart <= myModel.VisibleEndDate);
                    }

                    calendarView1.TimeSlotDuration = _timeSlotDuration == 90 ? 30 : _timeSlotDuration;
                    calendarView1.ScrollToTime(_timeToBeginDisplay == 24 ? 8 : _timeToBeginDisplay, 0);
                }
                catch (Exception ex)
                {
                    TraceLog.LogException(GetType().Name, "RefresScheduler", ex);
                    //MessageBox.Show(TraceLog.FormatExceptionMessage(this.GetType().Name, "RefresScheduler", ex));
                }

                calendarView1.CalendarModel = _calendarModel;
                PopulateAccessBlocks();
            }
            finally
            {
                calendarView1.SelectedViewChanged += CalendarView1_OnSelectedViewChanged;
                calendarView1.DateSelectionChanged += CalendarView1_OnDateSelectionChanged;
                calendarView1.UserDateChanged += UserDateChanged_Event;
            }
        }

        private void PopulateAccessBlocks()
        {
            foreach(var res in ResourceGroup.Resources)
            {
                var appearances = new DaySlotAppearances();
                string displayOwner;
                if(res.IsProvidersSchedule)
                    displayOwner = res.GroupResourceName + "-" + res.RESOURCE_NAME;
                else
                    displayOwner = res.RESOURCE_NAME;

                var presenter = FindPresenter(displayOwner);
                IList<SchdAvailability> accessBlocks = null;

                if (presenter != null)
                    accessBlocks = presenter.AccessBlocks;

                for (var currDate = myModel.VisibleStartDate; currDate <= myModel.VisibleEndDate; currDate = currDate.AddMinutes(calendarView1.TimeSlotDuration))
                {
                    var start = currDate;
                    var end = currDate.AddMinutes(calendarView1.TimeSlotDuration);
                    var appearance = new DaySlotAppearance(
                        start.Hour,
                        start.Minute,
                        end.Hour,
                        end.Minute,
                        Keys.ColorCodes.UnavailableSlotColor,
                        new Pen(Brushes.DarkGray, 1d)
                    );

                    if (accessBlocks != null)
                    {
                        var accessBlock = accessBlocks.FirstOrDefault(ab => Keys.Methods.TimesOverlap(Convert.ToDateTime(ab.StartTime), Convert.ToDateTime(ab.EndTime), start, end));

                        if (accessBlock != null)
                        {
                            if (accessBlock.ACCESSTYPEID == Keys.AccessTypes.Available)
                            {
                                appearance.Background = Keys.ColorCodes.AvailableSlotColor;
                                appearance.Pen = new Pen(Brushes.Gray, 1d);
                            }
                            else if (accessBlock.ACCESSTYPEID == Keys.AccessTypes.Unavailable)
                            {
                                //  This uses the default set above
                            }
                            else if (accessBlock.ACCESSTYPEID == Keys.AccessTypes.Cancelled)
                            {
                                appearance.Background = Keys.ColorCodes.CancelledAvailabilitySlotColor;
                                appearance.Pen = new Pen(Keys.ColorCodes.HatchedBrushFontColor, 1d);
                            }
                        }

                        appearances.Add(currDate, appearance);
                    }
                }
                if (calendarView1.ViewDisplayCustomizations.OwnerDaySlotBackgrounds.Contains(displayOwner))
                    calendarView1.ViewDisplayCustomizations.OwnerDaySlotBackgrounds.Remove(displayOwner);

                calendarView1.ViewDisplayCustomizations.OwnerDaySlotBackgrounds.Add(displayOwner, appearances);
            }

        }

        private void ViewSizeCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            // Set fixed width for the single user view
            calendarView1.ViewSize = 400;
        }

        private void ViewSizeCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            // Clear fixed width setting
            calendarView1.ClearValue(CalendarView.ViewSizeProperty);
        }

        private void calendarView1_MouseDown(object sender, MouseButtonEventArgs e)
        {
            try
            {
                _curSlot = calendarView1.HitTest(e.GetPosition(calendarView1));

                if (e.ChangedButton == MouseButton.Left)
                    if (_curSlot.AppointmentView != null)
                        if (_curSlot.AppointmentView.IsAllDayEvent)
                            if ((_curSlot.AppointmentView.Subject == "Special Instructions") && (_curSlot.AppointmentView.Appointment.Tag != null))
                            {
                                var inst = (string)_curSlot.AppointmentView.Appointment.Tag;
                                Keys.AlertUser(this, inst, "Special Instructions");
                            }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "calendarViewe1_MouseDown", ex);
                Console.WriteLine(ex.Message);
            }
        }

        private void calendarView1_MouseUp(object sender, MouseButtonEventArgs e)
        {
            TraceLog.LogInfo(GetType().Name, "calendarView1_MouseUp 1", "Mouse Up");

            if (_preEditAppointment != null)
            {
                switch (_appointmentEditAction)
                {
                    case eAppointmentChangeType.AppointmentResized:
                        break;
                    case eAppointmentChangeType.OwnerChanged:
                        break;
                    case eAppointmentChangeType.AppointmentMoved:
                        var appointmentToAdd = _appointmentToAdd == null ? null : _appointmentToAdd.Appointment;

                        // All of these objects MUST exist
                        if ((_appointmentToAdd == null) || (appointmentToAdd == null))
                        {
                            // Cancel the drop operation and return the appointment to its original slot
                            CancelMove();
                            return;
                        }

                        // Figure out the presenter for the new Appointment
                        var addPresenter = FindPresenter(appointmentToAdd.OwnerKey);
                        var message = "Are you sure you want to move this appointment?\n\n{0}New appointment time: " + appointmentToAdd.StartTime;
                        var messageClinic = string.Empty;

                        if (addPresenter != null)
                        {
                            // Get the presenter's resource's over book authority for the current slot.
                            var overBookAuthority = OverBookAuthority(appointmentToAdd.StartTime, addPresenter.SelectedResource);

                            // This checks overbooks as well as availability and hides the context menu accordingly
                            if (HideContextMenu(addPresenter, appointmentToAdd.StartTime, overBookAuthority))
                            {
                                // Cancel the drop operation and return the appointment to its original slot
                                CancelMove();
                                return;
                            }

                            // Confirm user's choice to move the appointment
                            if (addPresenter.ResourceName != _preEditAppointment.RESOURCENAME)
                                messageClinic = "New Clinic: " + addPresenter.ResourceName + "\n";
                        }

                        if (!myModel.View.ConfirmUser(string.Format(message, messageClinic), "Move Appointment?"))
                        {
                            // Cancel the drop operation and return the appointment to its original slot
                            CancelMove();
                            return;
                        }

                        // Move the appointment
                        if (!MoveAppointment())
                            // Move failed: Cancel the drop operation and return the appointment to its original slot
                            CancelMove();

                        break;
                }
            }

            _preEditAppointment = null;
        }

        private void CancelMove()
        {
            if ((_appointmentToAdd == null) || (_preEditAppointment == null) || string.IsNullOrEmpty(_preEditOwnerKey))
                return;

            if (_appointmentToAdd.Appointment.OwnerKey != _preEditOwnerKey)
                _appointmentToAdd.Appointment.OwnerKey = _preEditOwnerKey;

            _appointmentToAdd.Appointment.MoveTo(Convert.ToDateTime(_preEditAppointment.START_TIME));
            _preEditAppointment = null;
        }

        private void AddAppointment(AppointmentData apptData)
        {
            if (!_respondToEvents)
                return;

            if (_disposing)
                return;

            // Create new appointment and add it to the model
            // Appointment will show up in view automatically
            var appointment = new Appointment
            {
                Id = Convert.ToInt32( apptData.AppointmentID )
            };

            if (!string.IsNullOrEmpty(apptData.Appointment.PATIENTNAME))
                appointment.Subject = apptData.Appointment.PATIENTNAME;
            else if (Keys.SelectedPatient != null)
                appointment.Subject = Keys.SelectedPatient.Name;

            appointment.StartTime     = Convert.ToDateTime(apptData.StartTime);
            appointment.EndTime       = Convert.ToDateTime(apptData.EndTime);
            appointment.TimeMarkedAs  = Appointment.TimerMarkerBusy;
            appointment.OwnerKey      = apptData.ResourceName; // _curSlot.OwnerKey;
            appointment.CategoryColor = ((SolidColorBrush)Keys.ColorCodes.NewAppointmentColor).Color.ToString();
            // Add appointment to the model
            _calendarModel.Appointments.Add(appointment);
            
            // Make appointment visible in current view
            calendarView1.EnsureVisible(appointment);
        }

        private void AddAppointment(string color)
        {
            if (_disposing)
                return;

            // Create new appointment and add it to the model
            // Appointment will show up in view automatically
            var appointment = new Appointment();
            if (Keys.SelectedPatient != null) appointment.Subject = Keys.SelectedPatient.Name;
            appointment.StartTime = _curSlot.DateTime;
            appointment.EndTime = appointment.StartTime.AddMinutes(30);
            if(!string.IsNullOrEmpty(color))
                appointment.CategoryColor = color;
            appointment.TimeMarkedAs = Appointment.TimerMarkerBusy;
            appointment.OwnerKey = _curSlot.OwnerKey;
            // Add appointment to the model
            _calendarModel.Appointments.Add(appointment);

            // Make appointment visible in current view
            calendarView1.EnsureVisible(appointment);
        }

        private void SchedulerContextMenu_ItemClick(object sender, Telerik.Windows.RadRoutedEventArgs e)
        {
           // var ispatientsel = myModel.IsAPatientSelected;
            var item = e.OriginalSource as RadMenuItem;
            var headerText = item.Header as string;
            if (Keys.SelectedPatient != null)
                myModel.SelectedPatient = Keys.SelectedPatient;
            else if (Keys.SelectedAppointmentRequest != null)
                myModel.SelectedPatient = Keys.SelectedAppointmentRequest.Patient;
            if (myModel.SelectedPatient != null)
                myModel.Patient = myModel.SelectedPatient;
            if(myModel.SelectedPatient == null && (headerText == "Add Appointment" || headerText == "Create Walk-in Appointment"))
            {
                MessageBox.Show("Please select a patient");
                return;
            }

            var duration = 30;
            
            var newAppt = new SchdAppointment();
            var presenter = FindPresenter(_curSlot.OwnerKey);

            if (presenter != null)
            {
                newAppt.RESOURCENAME = presenter.SelectedResource.RESOURCE_NAME;
                newAppt.SelectedResource = presenter.SelectedResource;
            }

            newAppt.ACCESSTYPEID = Keys.AccessTypes.Available;

            foreach (var res in ResourceGroup.Resources.Where(res => res.RESOURCE_NAME == newAppt.RESOURCENAME))
            {
                duration = res.Clinic.Duration;
                break;
            }
           
            newAppt.START_TIME = _curSlot.DateTime.ToString(CultureInfo.InvariantCulture);
            newAppt.END_TIME = _curSlot.DateTime.AddMinutes(duration).ToString(CultureInfo.InvariantCulture);
            if (myModel.SelectedPatient != null)
            {
                newAppt.PATIENTID = myModel.SelectedPatient.IEN;
                newAppt.PATIENTNAME = myModel.SelectedPatient.Name;
            }

            var selectedAppt = new AppointmentView();

            foreach (var appt in calendarView1.SelectedAppointments)
                selectedAppt = appt;

            if ("Add Appointment^Create Walk-in Appointment".Contains(headerText))
                if (newAppt.AppointmentRequest == null)
                {
                    if (Keys.SelectedAppointmentRequest == null)
                    {
                        MessageBox.Show("Please select an Appointment Request");
                        return;
                    }
                    newAppt.AppointmentRequest = Keys.SelectedAppointmentRequest;
                }
                

            switch (headerText)
            {
                case "Add Appointment":
                    Keys.IsAddAppointmentOrLeftClickOnScheduler = true;
                    myModel.LaunchAddNewAppointmentEvent(newAppt);
                    //AddAppointment(Appointment.CategoryDefault);
                    break;
                case "Create Walk-in Appointment":
                    //myModel.SchdAppointment.START_TIME = _curSlot.DateTime.ToString();
                    //myModel.SchdAppointment.END_TIME = _curSlot.DateTime.AddMinutes(30).ToString(CultureInfo.InvariantCulture);
                    //myModel.SchdAppointment.RESOURCENAME = myModel.ResourceName;
                    myModel.LaunchWalkInAppointmentEvent(newAppt);
                    //AddAppointment(Appointment.CategoryBlue);
                    //myModel.CreateWalkInAppointmentCommand.Execute("WalkIn Appointment");
                    break;
                case "Edit Appointment":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.EditAppointmentCommand.Execute("Edit Appointment");
                    break;
                case "View Appointment":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.ViewAppointmentCommand.Execute("View Appointment");
                    break;
                case "Mark as No Show":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.MarkAsNoShowCommand.Execute("Mark as No Show");
                    break;
                case "Undo NoShow":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.UndoANoShowCommand.Execute("Undo NoShow");
                    break;
                case "Check In Patient":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.CheckInPatientCommand.Execute("Check In Patient");
                    break;
                case "Undo Check In Patient":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.UndoCheckInPatientCommand.Execute("Undo Check In Patient");
                    break;
                case "Check Out Patient":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.CheckOutPatientCommand.Execute("Check Out Patient");
                    break;
                case "Undo Check Out Patient":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.UndoCheckOutPatientCommand.Execute("Undo Check Out Patient");
                    break;
                case "Cancel Appointment":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.CancelAppointmentCommand.Execute(null);
                    break;
                case "Undo Cancel Appointment":
                    LoadExistAppointmentInfo(selectedAppt);
                    myModel.UndoCancelAppointmentCommand.Execute("Undo Cancel Appointment");
                    break;
            }
        }

        private SchdAppointment SchdAppointmentFromAppointmentView(AppointmentView selectedAppointment)
        {
            SchdAppointment schdAppt = null;

            if (selectedAppointment != null)
            {
                schdAppt = myModel.SchdAppointmentFactory.Create();
                schdAppt.PATIENTNAME = selectedAppointment.Subject;
                schdAppt.START_TIME = selectedAppointment.StartTime.ToString();
                schdAppt.END_TIME = selectedAppointment.EndTime.ToString();
                schdAppt.APPOINTMENTID = selectedAppointment.Appointment.Id.ToString(CultureInfo.InvariantCulture);

                if (selectedAppointment.Appointment.Tag != null)
                {
                    var patientID = selectedAppointment.Appointment.Tag;
                    schdAppt.PATIENTID = patientID.ToString();
                }
                else if (Keys.SelectedPatient != null)
                    schdAppt.PATIENTID = Keys.SelectedPatient.IEN;

                var presenter = FindPresenter(selectedAppointment.Appointment.OwnerKey);

                if (presenter != null)
                    schdAppt.RESOURCENAME = presenter.SelectedResource.RESOURCE_NAME;

                foreach (IResource resource in selectedAppointment.Resources)
                {
                    if (resource.ResourceType == "APPOINTMENTID")
                    {
                        schdAppt.APPOINTMENTID = resource.ResourceName;
                    }
                    if (resource.ResourceType == "PATIENTID")
                    {
                        schdAppt.PATIENTID = resource.ResourceName;
                    }
                    if (resource.ResourceType == "PATIENTNAME")
                    {
                        schdAppt.PATIENTNAME = resource.ResourceName;
                    }
                    if (resource.ResourceType == "RESOURCENAME")
                    {
                        schdAppt.RESOURCENAME = resource.ResourceName;
                    }
                    if (resource.ResourceType == "ACCESSTYPEID")
                    {
                        schdAppt.ACCESSTYPEID = resource.ResourceName;
                    }
                    if (resource.ResourceType == "NOTE")
                    {
                        schdAppt.NOTE = resource.ResourceName;
                    }
                    if (resource.ResourceType == "HRN")
                    {
                        schdAppt.HRN = resource.ResourceName;
                    }
                    if (resource.ResourceType == "START_TIME")
                    {
                        schdAppt.START_TIME = resource.ResourceName;
                    }
                    if (resource.ResourceType == "END_TIME")
                    {
                        schdAppt.END_TIME = resource.ResourceName;
                    }
                    if (resource.ResourceType == "VPROVIDER")
                    {
                        schdAppt.VPROVIDER = resource.ResourceName;
                    }
                    if (resource.ResourceType == "MRTC_PARENT_IEN")
                    {
                        schdAppt.MRTCParentRequestIEN = resource.ResourceName;
                    }
                }
            }

            return schdAppt;
        }

        private void LoadExistAppointmentInfo(AppointmentView selectedAppointment)
        {
            if (selectedAppointment == null)
                return;

            myModel.SchdAppointment = SchdAppointmentFromAppointmentView(selectedAppointment);
        }

        public void Collapse()
        {

        }

        public void Expand()
        {

        }

        private void paneClinicGroupMultiView_Unloaded(object sender, RoutedEventArgs e)
        {
        }

        private bool _iChangedTheDate = false;
        private void UserDateChanged_Event(object sender, RoutedEventArgs e)
        {
            var cal = sender as CalendarView;
            EventAggregator.GetEvent<SchedulerViewDateEvent>().Publish(new SchedulerViewDateEventArgs("MultiScheduler.ClinicGroupMultiView", cal.DayViewDate.Date));

            if (_windowLoaded)
                DoRefreshScheduler();
        }

        private bool TimeIsInRange(TimeSpan start, TimeSpan end, DateTime target, bool inclusive = true)
        {
            if (inclusive)
                return (start <= target.TimeOfDay) && (target.TimeOfDay <= end);
            else
                return (start < target.TimeOfDay) && (target.TimeOfDay < end);
        }

        private bool OverBookAuthority(DateTime startTime, SchdResource resource)
        {
            //  Can the user Master Over Book ??
            var appt = new SchdAppointment
            {
                START_TIME = startTime.ToString(),
                RESOURCENAME = resource.RESOURCE_NAME,
                ResourceId = resource.RESOURCEID,
                SelectedResource = resource
            };

            return DataAccessService.OverBookAuthority(appt, resource, true);
        }

        private bool HideContextMenu(ISchedulerPresentationModel presenter, DateTime slotDateTime, bool overBookAuthority)
        {
            var result = true;

            // presenter selected
            if (presenter == null)
                return result;

            // Look for an AcceesBlock containing the time slot
            var block = presenter.AccessBlocks.FirstOrDefault(ab => (DateTime.Parse(ab.StartTime).CompareTo(slotDateTime) <= 0) && (slotDateTime.CompareTo(DateTime.Parse(ab.EndTime)) <= 0));

            // Found a block, check accessability based on Access Type ID
            if (block != null)
            {
                // Is Access ID is Cancled, return true to hide context menu
                if (block.ACCESSTYPEID == Keys.AccessTypes.Cancelled)
                    return true;

                // Determine, by Access Type IS, if the context menu should be hidden
                result = block.ACCESSTYPEID == Keys.AccessTypes.Unavailable;
            }

            // If hiding the Context Menu at this point check Overbook authority
            if (result)
                result = !overBookAuthority;

            return result;
        }

        private void ContextMenu_Open(object sender, RoutedEventArgs e)
        {
            var menu = (RadContextMenu)sender;

            var overBookAuthority = false;
            // Figure out which presenter
            var presenter = FindPresenter(_curSlot.OwnerKey);

            if (presenter != null)
                // Get the presenter's resource's over book authority for the current slot.
                overBookAuthority = OverBookAuthority(_curSlot.DateTime, presenter.SelectedResource);

            var menuVisibility = Visibility.Visible;

            if (calendarView1.SelectedAppointments.Count == 0)
            {
                if (_curSlot == null)
                    menuVisibility = Visibility.Collapsed;
                // Is the request a Master MRTC Request ??
                else if (presenter != null && presenter.CurrentAppointmentRequest != null && presenter.CurrentAppointmentRequest.IsMRTCParent)
                    menuVisibility = Visibility.Collapsed;
                else if (Keys.SelectedAppointmentRequest != null && Keys.SelectedAppointmentRequest.IsMRTCParent)
                    menuVisibility = Visibility.Collapsed;
                // This checks over books as well as availability and hides the context menu accordingly
                else if (HideContextMenu(presenter, _curSlot.DateTime, overBookAuthority))
                    menuVisibility = Visibility.Collapsed;

                menu.Visibility = menuVisibility;

                if (menuVisibility != Visibility.Visible)
                    return;

                // Sets Context Menu Item Visibility to separate Tome Slot vs Appointment Menus
                AddApptCm.Visibility             = Visibility.Visible;
                CreateWalkInApptCm.Visibility    = Visibility.Visible;
                ViewApptCm.Visibility            = Visibility.Collapsed;
                EditApptCm.Visibility            = Visibility.Collapsed;
                MarkAsNoShowCm.Visibility        = Visibility.Collapsed;
                CancelApptCm.Visibility          = Visibility.Collapsed;
                CheckInPatientCm.Visibility      = Visibility.Collapsed;
                CheckOutPatientCm.Visibility     = Visibility.Collapsed;
                UndoCancelCm.Visibility          = Visibility.Collapsed;
                UndoCheckInPatientCm.Visibility  = Visibility.Collapsed;
                UndoCheckOutPatientCm.Visibility = Visibility.Collapsed;
                UndoNoShowCm.Visibility          = Visibility.Collapsed;
                Sep_1.Visibility                 = Visibility.Collapsed;
                Sep_2.Visibility                 = Visibility.Collapsed;
                Sep_3.Visibility                 = Visibility.Collapsed;
                
                AddApptCm.IsEnabled = false;
                CreateWalkInApptCm.IsEnabled = false;
                var userCanAccessClinic = true;
                var resources = DataAccessService.GetResources(calendarView1.SelectedOwnerKey, true);                
                var prohibitedClinic = DataAccessService.IsProhibitedClinic(presenter.SelectedResource.ClinicName, presenter.SelectedResource.HOSPITAL_LOCATION_ID, true);
//                var prohibitedClinic = resources.Select(res => DataAccessService.IsProhibitedClinic(res.ClinicName, res.HOSPITAL_LOCATION_ID, true)).FirstOrDefault();

                if (prohibitedClinic)
                    userCanAccessClinic = DataAccessService.UserHasAccessToProhibitedClinic(presenter.SelectedResource.HOSPITAL_LOCATION_ID);

                AddApptCm.IsEnabled = userCanAccessClinic && Keys.SelectedAppointmentRequest != null;
                
                var bgSlots = calendarView1.ViewDisplayCustomizations.OwnerDaySlotBackgrounds[_curSlot.OwnerKey];

                if (bgSlots != null)
                {
                    var slots = bgSlots[_curSlot.DateTime];

                    if (slots != null)
                    {
                        var slot = slots.FirstOrDefault(s => TimeIsInRange(new TimeSpan(s.StartTime.Hour, s.StartTime.Minute, 0), new TimeSpan(s.EndTime.Hour, s.EndTime.Minute, 0), _curSlot.DateTime));

                        if ( slot != null && ( Equals( slot.Background, Keys.ColorCodes.AvailableSlotColor ) || overBookAuthority ) && Keys.SelectedAppointmentRequest != null )
                            if ( calendarView1.DateSelectionStart != null && calendarView1.DateSelectionStart.Value.ToShortDateString() == DateTime.Now.ToShortDateString() )
                                CreateWalkInApptCm.IsEnabled = AddApptCm.IsEnabled;
                    }
                }

                EditApptCm.IsEnabled = false;
                ViewApptCm.IsEnabled = false;
                UndoCancelCm.IsEnabled = false;
                UndoCheckInPatientCm.IsEnabled = false;
                UndoCheckOutPatientCm.IsEnabled = false;
                UndoNoShowCm.IsEnabled = false;
                MarkAsNoShowCm.IsEnabled = false;
                CheckInPatientCm.IsEnabled = false;
                CheckOutPatientCm.IsEnabled = false;
                CancelApptCm.IsEnabled = false;
            }
            else
            {
                // Hide the Appointments menu for all day events.
                if (calendarView1.SelectedAppointments[0].IsAllDayEvent)
                {
                    menu.Visibility = Visibility.Collapsed;
                    return;
                }
                menu.Visibility = Visibility.Visible;

                // Insure context menu is visible
                ((RadContextMenu)sender).Visibility = Visibility.Visible;

                // Sets Context Menu Item Visibility to separate Tome Slot vs Appointment Menus
                AddApptCm.Visibility             = Visibility.Collapsed;
                CreateWalkInApptCm.Visibility    = Visibility.Collapsed;
                ViewApptCm.Visibility            = Visibility.Visible;
                EditApptCm.Visibility            = Visibility.Visible;
                MarkAsNoShowCm.Visibility        = Visibility.Visible;
                CancelApptCm.Visibility          = Visibility.Visible;
                CheckInPatientCm.Visibility      = Visibility.Visible;
                CheckOutPatientCm.Visibility     = Visibility.Visible;
                UndoCancelCm.Visibility          = Visibility.Visible;
                UndoCheckInPatientCm.Visibility  = Visibility.Visible;
                UndoCheckOutPatientCm.Visibility = Visibility.Visible;
                UndoNoShowCm.Visibility          = Visibility.Visible;
                Sep_1.Visibility                 = Visibility.Visible;
                Sep_2.Visibility                 = Visibility.Visible;
                Sep_3.Visibility                 = Visibility.Visible;

                var schdAppointment = presenter.AppointmentList.FirstOrDefault(a => a.APPOINTMENTID == calendarView1.SelectedAppointments[0].Appointment.Id.ToString());

                AddApptCm.IsEnabled          = false;
                CreateWalkInApptCm.IsEnabled = false;
                EditApptCm.IsEnabled         = false; // true;
                ViewApptCm.IsEnabled         = true;

                MarkAsNoShowCm.IsEnabled        = schdAppointment.CanMarkAsNoShow;
                CancelApptCm.IsEnabled          = schdAppointment.CanCancel;
                CheckInPatientCm.IsEnabled      = schdAppointment.CanCheckIn;
                CheckOutPatientCm.IsEnabled     = schdAppointment.CanCheckOut;
                UndoCancelCm.IsEnabled          = schdAppointment.CanUndoCancel;
                UndoCheckInPatientCm.IsEnabled  = schdAppointment.CanUndoCheckIn;
                UndoCheckOutPatientCm.IsEnabled = schdAppointment.CanUndoCheckOut;
                UndoNoShowCm.IsEnabled          = schdAppointment.CanUndoMarkAsNoShow;
            }
        }

        private void CalendarView1_OnSelectedViewChanged(object sender, RoutedEventArgs e)
        {
            try
            {
                if (_windowLoaded)
                {
                    foreach (var kvp in Presenters)
                        kvp.Value.MultiCalendarDayViewDate = calendarView1.DayViewDate;

                    var calView = string.Empty;

                    switch (calendarView1.SelectedView)
                    {
                        case eCalendarView.Day:
                            calView = "Day";
                            break;

                        case eCalendarView.Week:
                            calView = "Week";
                            break;

                        case eCalendarView.Month:
                            calView = "Month";
                            break;

                        case eCalendarView.Timeline:
                            calView = "TimeLine";
                            break;

                        case eCalendarView.None:
                            calView = "None";
                            break;
                    }

                    EventAggregator.GetEvent<ChangeViewModeEvent>().Publish(calView);
                    DoRefreshScheduler();
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CalendarView1_OnSelectedViewChanged", ex);
                //MessageBox.Show(TraceLog.FormatExceptionMessage(this.GetType().Name, "CalendarView1_OnSelectedViewChanged", ex));
            }
        }

        private void CalendarView1_OnDateSelectionChanged(object sender, RoutedEventArgs e)
        {
            foreach (var kvp in Presenters)
                kvp.Value.MultiCalendarDayViewDate = calendarView1.DayViewDate;
        }

        private ISchedulerPresentationModel FindPresenter(string key)
        {
            if (string.IsNullOrEmpty(key))
                return null;

            ISchedulerPresentationModel presenter = null;
            string presenterKey;

            if (key.Contains("^"))
                presenterKey = key;
            else
                presenterKey = Presenters.Keys.FirstOrDefault(k => k.Replace("^", "-") == key);

            if (!string.IsNullOrEmpty(presenterKey))
                if (Presenters.Keys.Contains(presenterKey))
                    presenter = Presenters[presenterKey];

            return presenter;
        }

        private SchdAppointment FindSchdAppointment(string apptID)
        {
            SchdAppointment result = null;

            foreach (var presenter in Presenters.Values)
            {
                result = presenter.AppointmentList.FirstOrDefault(a => a.APPOINTMENTID == apptID);

                if (result != null)
                    break;
            }

            return result;
        }

        internal bool MoveAppointment()
        {
            try
            {
                var newAppointment = SchdAppointmentFromAppointmentView(_appointmentToAdd);
                var appointmentToCancel = _preEditAppointment;
                var addPresenter = FindPresenter(_appointmentToAdd.Appointment.OwnerKey);

                if (addPresenter != null)
                    newAppointment.SelectedResource = addPresenter.SelectedResource;

                var newAppointmentId = string.Empty;
                string errorMessage;

                var a = new AppointmentData
                {
                    AppointmentID = "0",
                    StartTime     = newAppointment.START_TIME,
                    EndTime       = newAppointment.END_TIME,
                    Duration      = ( _appointmentToAdd.EndTime.Value - _appointmentToAdd.StartTime.Value ).TotalMinutes.ToString(),
                    Notes         = newAppointment.NOTE,
                    ResourceName  = newAppointment.RESOURCENAME,
                    PatientId     = newAppointment.PATIENTID
                };
                var p = new PatientInformation
                {
                    IEN = newAppointment.PATIENTID,
                    Name = newAppointment.PATIENTNAME
                };

                List<string> errorMessages;
                // Check Holiday
                errorMessage = DataAccessService.IsHoliday(Convert.ToDateTime(a.StartTime));
                var canBookHoliday = false;

                if (!string.IsNullOrEmpty(errorMessage))
                {
                    if (!string.IsNullOrEmpty(newAppointment.RESOURCENAME))
                    {
                        if (newAppointment.SelectedResource != null)
                            canBookHoliday = newAppointment.SelectedResource.Clinic.ClinicCanBookOnHolidays;
                        else
                            canBookHoliday = DataAccessService.ClinicCanBookOnHoliday(null, newAppointment.RESOURCENAME);
                    }

                    if (!canBookHoliday)
                    {
                        var msg = _appointmentToAdd.StartTime.Value.ToShortDateString() + " is a holiday (" + errorMessage + ") and is not allowed to be scheduled.";
                        MessageBox.Show(msg);
                        return false;
                    }
                }

                bool isTelehealth;

                //check conflict appointment
                if (DataAccessService.CheckConflictAppointment("Move Appointment", a.ResourceName, p.IEN, p.Name, a.StartTime, a.EndTime, out isTelehealth))
                    return false;

                //check overbook authority
                var bOverbookAppointment = DataAccessService.OverBookAuthority(newAppointment);

                // refresh available slots
                Keys.IsAddAppointmentOrLeftClickOnScheduler = true;

                if (newAppointment.SelectedResource == null)
                    DataAccessService.RefreshAvailabilitySchedule(newAppointment.RESOURCENAME, Convert.ToDateTime(newAppointment.START_TIME), Convert.ToDateTime(newAppointment.END_TIME));
                else
                    DataAccessService.RefreshAvailabilitySchedule(newAppointment.SelectedResource, Convert.ToDateTime(newAppointment.START_TIME), Convert.ToDateTime(newAppointment.END_TIME));

                //Check No-shows
                errorMessage = string.Empty;

                if (Convert.ToDateTime(a.StartTime).Date > DateTime.Now.Date)
                    errorMessage = DataAccessService.NoShow(p.IEN, a.ResourceName, "AddNew");

                if (errorMessage != string.Empty)
                    myModel.View.AlertUser(errorMessage, "Patient No-Shows");

                //check slots available
                Keys.IsAddAppointmentOrLeftClickOnScheduler = true;
                errorMessage = DataAccessService.CheckSlotsAvailable(a.StartTime, a.EndTime, a.ResourceName);

                if (!string.IsNullOrEmpty(errorMessage) && !Keys.Methods.IsNumeric(errorMessage))
                {
                    if (bOverbookAppointment)
                    {
                        a.IsOverbook = Keys.ConfirmUser((DependencyObject)View, errorMessage, "Availability", true);

                        if (!a.IsOverbook)
                            return false;
                    }
                    else
                        return false;
                }

                var cancelPayload = new CancelApptDialogEventPayload(
                    appointmentToCancel,
                    success =>
                    {
                        if (success)
                        {
                            a.Appointment = newAppointment;
                            a.Appointment.PATIENTNAME = appointmentToCancel.PATIENTNAME;
                            a.Appointment.PATIENTID = appointmentToCancel.PATIENTID;
                            a.Appointment.DesiredAppointmentDate = appointmentToCancel.DesiredAppointmentDate;

                            if ( appointmentToCancel.AppointmentRequest == null && Keys.Methods.IsNumeric( appointmentToCancel.APPOINTMENTID ) )
                            {
                                var reqType = string.Empty;
                                appointmentToCancel.AppointmentRequest = DataAccessService.GetAppointmentRequestByAppointmentIen( appointmentToCancel.APPOINTMENTID, ref reqType );
                            }

                            if (isTelehealth)
                                // If this is part of a Telehealth pair then a conflict DOES exist, must add i minute to the starttime to appease VistA
                                a.StartTime = Convert.ToDateTime(a.StartTime).AddMinutes(1).ToString("MMM d,yyyy hh:mm");

                            if (appointmentToCancel.AppointmentRequest == null)
                                errorMessages = DataAccessService.AddNewAppointment(a, p, false, true);
                            else
                                errorMessages = DataAccessService.AddNewAppointment(a, p, false, appointmentToCancel.AppointmentRequest);

                            if (errorMessages != null && errorMessages.Count > 0)
                            {
                                newAppointmentId = errorMessages[0];
                                errorMessage = errorMessages[1].ToString();
                            }

                            if (!string.IsNullOrEmpty(newAppointmentId) && Keys.Methods.IsNumeric(newAppointmentId))
                                System.Threading.Thread.Sleep(200);

                            if ( string.IsNullOrEmpty( errorMessage ) && appointmentToCancel.AppointmentRequest != null )
                            {
                                var errorText = string.Empty;

                                if ( appointmentToCancel.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Ewl )
                                    errorText = DataAccessService.WLClose( appointmentToCancel.AppointmentRequest.RequestManagmentIen,
                                        Keys.WaitListDispositions.RemovedScheduledAssigned );
                                else if ( appointmentToCancel.AppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Appt )
                                    errorText = DataAccessService.ARClose( appointmentToCancel.AppointmentRequest.RequestManagmentIen,
                                        Keys.WaitListDispositions.RemovedScheduledAssigned );

                                if ( errorText != string.Empty )
                                    myModel.View.AlertUser(
                                               "Appointment Request could not be reclosed:" + Environment.NewLine +
                                               Environment.NewLine + errorText, "Error" );
                            }

                            // Add appointment to the model
                            AddAppointment(a);
                        }
                    },
                    false
                );

                // Ron TODO: Uncomment ??
                //myModel.EventAggregator.GetEvent<LaunchCancelApptDialogEvent>().Publish(cancelPayload);
                return true;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "MoveAppointment", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
                return false;
            }
        }

        #region IDisposable members

        protected void Dispose(bool itIsSafeToAlsoFreeManagedObjects)
        {
            try
            {
                _disposing = true;
                // Free unmanaged resources
                DataAccessService = null;

                //Free managed resources too, but only if I'm being called from Dispose
                //(If I'm being called from Finalize then the objects might not exist
                //anymore
                if (itIsSafeToAlsoFreeManagedObjects)
                {
                    RespondToEvents(false);
                    ShutDownPresenters();

                    if (_refreshTimer != null)
                    {
                        if (_refreshTimer.Enabled)
                            _refreshTimer.Stop();

                        _refreshTimer.Elapsed -= refreshTimer_Elapsed;
                        _refreshTimer.Dispose();
                        _refreshTimer = null;
                    }

                    Loaded -= Window_Loaded;
                    Unloaded -= paneClinicGroupMultiView_Unloaded;

                    calendarView1.RemoveHandler(AppointmentView.BeforeAppointmentDragEvent, _onBeforeAppointmentDragHandler);
//	                calendarView1.RemoveHandler(AppointmentView.AppointmentChangingEvent, _appointmentChangingEventHandler);
                    calendarView1.RemoveHandler(AppointmentView.AppointmentChangedEvent, _appointmentChangedEvent);
                    _onBeforeAppointmentDragHandler = null;
//	                _appointmentChangingEventHandler = null;
                    _appointmentChangedEvent = null;
                    calendarView1.MouseDown -= calendarView1_MouseDown;
                    calendarView1.SelectedViewChanged -= CalendarView1_OnSelectedViewChanged;
                    calendarView1.DateSelectionChanged -= CalendarView1_OnDateSelectionChanged;
                    calendarView1.UserDateChanged -= UserDateChanged_Event;
                    calendarView1.DisplayedOwners.Clear();
                    calendarView1.CalendarModel = null;
                    _calendarModel.Appointments.Clear();
                    _calendarModel = null;
//	                calendarView1 = null;

                    AppointmentContextMenu.ItemClick -= SchedulerContextMenu_ItemClick;
                    AppointmentContextMenu.Opened -= ContextMenu_Open;
                    AppointmentContextMenu = null;
                    ViewSizeCheckBox.Checked -= ViewSizeCheckBox_Checked;
                    ViewSizeCheckBox.Unchecked -= ViewSizeCheckBox_Unchecked;

                    Content = null;
                    Header = null;
                    ContextMenu = null;
                    DataContext = null;
                    // The model is also in the Presenters list and will be disposed there
                    Model = null;
                    ResourceGroup.Resources.Clear();
                    ResourceGroup.Resources = null;
                    ResourceGroup = null;
                    GroupPresentationModel = null;
                    View = null;

                    // Dispose of ISchedulerView & ISchedulerPresentationModel objects
                    foreach (var kvp in Presenters)
                    {
                        var m = kvp.Value;
                        m.View.Close();
                        m.Dispose();
                    }

                    Presenters.Clear();
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "Dispose", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }
    
        public void Dispose()
        {
            Dispose(true); //true: safe to free managed resources
            GC.SuppressFinalize(this); //Hey, GC: don't bother calling finalize later
        }

        ~ClinicGroupMultiView()
        {
            //we're being finalized (i.e. destroyed), call Dispose in case the user forgot to
            Dispose(false);
        }

        #endregion IDisposable members
    }
}
