using System;
using System.Collections.Generic;
using System.ComponentModel;

using System.Linq;
using System.Text;

using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using ClinSchd.Modules.Task.ChildModules.MultiScheduler;
using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.MultiScheduler;
using ClinSchd.Modules.Task.MultiScheduler.Services;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.Task.Scheduler;
using Microsoft.Practices.Composite.Presentation.Regions;
using ClinSchd.Modules.Task.Group;
using Telerik.Windows.Controls;

namespace ClinSchd.Modules.Task.MultiScheduler.Controllers
{
    public class TaskMultiSchedulerController : ITaskMultiSchedulerController
    {
		private readonly IUnityContainer container;
		private readonly IRegionManager regionManager;
        private readonly IEventAggregator eventAggregator;
		private readonly IDataAccessService dataAccessService;
        private List<IMultiSchedulerView> ChildPanes { get; set; }
//        private Dictionary<string, WeakReference> _references = new Dictionary<string, WeakReference>();

		public TaskMultiSchedulerController(
			IUnityContainer container,
			IDataAccessService dataAccessService,
			IEventAggregator eventAggregator,
			IRegionManager regionManager)
        {			
			this.container = container;
            this.eventAggregator = eventAggregator;
			this.dataAccessService = dataAccessService;
			this.regionManager = regionManager;
            ChildPanes = new List<IMultiSchedulerView>();
        }

		public void Run()
		{
			this.eventAggregator.GetEvent<SchedulerDisplayGroupEvent> ().Subscribe (SchedulerDisplayGroup, ThreadOption.UIThread, true);
		    this.eventAggregator.GetEvent<SchedulerDisplayForProvidersEvent>().Subscribe(SchedulerDisplayForProviders, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<CloseAllSchedulesEvent>().Subscribe(CloseAllSchedules, ThreadOption.UIThread, true);
        }

        public void SchedulerDisplayForProviders(SchdResourceGroup selectedResourceGroup)
        {
            if (selectedResourceGroup == null)
                return;

            bool paneAlreadyExists = false;
            foreach (IMultiSchedulerView item in ChildPanes)
            {
                if (item.Model.SelectedResourceGroup.Name == selectedResourceGroup.Name)
                {
                    item.Model.IsSelectedTab = true;
                    paneAlreadyExists = true;
                    break;
                }
            }
            if (!paneAlreadyExists)
            {
                IMultiSchedulerPresentationModel groupPresentationModel = this.container.Resolve<IMultiSchedulerPresentationModel>();
                RegionManager.SetRegionManager(groupPresentationModel.View.MultiSchedulerGroupControl, this.regionManager);
                groupPresentationModel.Controller = this;
                groupPresentationModel.PaneTitle = selectedResourceGroup.Name;
                groupPresentationModel.SelectedResourceGroup = selectedResourceGroup;
                this.regionManager.Regions[RegionNames.TaskGroupRegion].Add(groupPresentationModel.View);
                var clinicGroupView = new ClinicGroupMultiView(eventAggregator)
                {
                    ScheduleType = "Providers",
                    LoadingID = selectedResourceGroup.LoadingID,
                    ResourceGroup = new SchdResourceGroup(dataAccessService)
                };
                clinicGroupView.ResourceGroup.Resources = new List<SchdResource>();
                clinicGroupView.Name = selectedResourceGroup.Name;
                clinicGroupView.DataAccessService = dataAccessService;
                //ensure no duplicate resource EXISTS()........bj:7-28_2015
               // selectedResourceGroup.Resources = (IList<SchdResource>) selectedResourceGroup.Resources.Distinct();
                var resourceNames = "";
                foreach (SchdResource selectedResource in selectedResourceGroup.Resources)
                {
                    if (selectedResource == null)
                        continue;

                    ISchedulerPresentationModel presenter = this.container.Resolve<ISchedulerPresentationModel>();
                    presenter.IsPartofGoupSchedule = true;
//                    presenter.View.TimeSlotViewerEnableEvents(false);
                    presenter.SelectedResource = selectedResource;
//                    presenter.ResourceName = selectedResource.RESOURCE_NAME;
                    if (resourceNames.ToUpper().Contains(presenter.ResourceName.ToUpper()))
                        continue;
                    resourceNames += presenter.ResourceName + ", ";
                    presenter.PaneTitle = selectedResource.RESOURCE_NAME;
                    presenter.IsSelectedTab = true;
                    clinicGroupView.ResourceGroup.Name = selectedResource.RESOURCE_NAME;
                    clinicGroupView.ResourceGroup.IEN = selectedResource.ResourceTypeIEN;
                    presenter.ViewModeChanged("Day");

                    if (clinicGroupView.Model == null)
                        clinicGroupView.Model = (ITaskPresentationModel)presenter;

                    clinicGroupView.ResourceGroup.Resources.Add(selectedResource);
                    clinicGroupView.Presenters.Add(presenter.SelectedResource.RESOURCE_NAME, presenter);
                }

                if (clinicGroupView.Presenters.Count > 0)
                {
                    groupPresentationModel.View.DockSchedule(clinicGroupView);
                    ChildPanes.Add(groupPresentationModel.View);
                }
            }
        }

		public void SchedulerDisplayGroup (SchdResourceGroup selectedResourceGroup)
		{
            if (selectedResourceGroup == null)
                return;

            bool paneAlreadyExists = false;

            foreach (IMultiSchedulerView item in ChildPanes)
            {
				if (item.Model.SelectedResourceGroup.Name == selectedResourceGroup.Name) {
					item.Model.IsSelectedTab = true;
					paneAlreadyExists = true;
					break;
				}
			}

			if (!paneAlreadyExists)
            {
                IMultiSchedulerPresentationModel groupPresentationModel = this.container.Resolve<IMultiSchedulerPresentationModel>();
                RegionManager.SetRegionManager(groupPresentationModel.View.MultiSchedulerGroupControl, this.regionManager);
				groupPresentationModel.Controller = this;
				groupPresentationModel.PaneTitle = selectedResourceGroup.Name;
				groupPresentationModel.SelectedResourceGroup = selectedResourceGroup;
				this.regionManager.Regions[RegionNames.TaskGroupRegion].Add (groupPresentationModel.View);
                var clinicGroupView = new ClinicGroupMultiView(eventAggregator)
                {
                    ScheduleType = "Clinic Group",
                    LoadingID = selectedResourceGroup.LoadingID,
                    ResourceGroup = new SchdResourceGroup(dataAccessService)
                };
                clinicGroupView.Name = selectedResourceGroup.Name;
                clinicGroupView.ResourceGroup.Resources = new List<SchdResource>();
                clinicGroupView.DataAccessService = dataAccessService;
                
                foreach (SchdResource selectedResource in selectedResourceGroup.Resources)
                {
                    if (selectedResource == null)
                        continue;

			        var providerPiece = selectedResource.ResourceType.Split('|');

                    if (providerPiece[0].ToUpper() == "P")
                    {
                        var providerClinics = dataAccessService.GetClinicsByProvider(selectedResource.ResourceTypeIEN);
                        foreach (var clin in providerClinics)
                        {
                            clin.IsProvidersSchedule = true;
                            clin.GroupResourceName = selectedResource.RESOURCE_NAME;
                            clin.ResourceTypeIEN = selectedResource.ResourceTypeIEN;
                            clinicGroupView.ResourceGroup.Resources.Add(clin);

                            ISchedulerPresentationModel clinPresenter = this.container.Resolve<ISchedulerPresentationModel>();
                            clinPresenter.IsPartofGoupSchedule = true;
                            clinPresenter.SelectedResource = clin;
//                            clinPresenter.ResourceName = clin.RESOURCE_NAME;
                            clinPresenter.PaneTitle = clin.RESOURCE_NAME;
                            clinPresenter.IsSelectedTab = true;
                            clinPresenter.ViewModeChanged("Day");

                            if (clinicGroupView.Model == null)
                                clinicGroupView.Model = (ITaskPresentationModel)clinPresenter;

                            clinicGroupView.Presenters.Add(selectedResource.RESOURCE_NAME + "^" + clinPresenter.SelectedResource.RESOURCE_NAME, clinPresenter);
                        }
                    }
                    else
                    {
                        clinicGroupView.ResourceGroup.Name = selectedResource.RESOURCE_NAME;
                        clinicGroupView.ResourceGroup.IEN = selectedResource.ResourceTypeIEN;
                        clinicGroupView.ResourceGroup.Resources.Add(selectedResource);
                        ISchedulerPresentationModel resPresenter = this.container.Resolve<ISchedulerPresentationModel>();
                        resPresenter.SelectedResource = selectedResource;

                        if (clinicGroupView.Model == null)
                            clinicGroupView.Model = (ITaskPresentationModel)resPresenter;

//                        resPresenter.ResourceName = selectedResource.RESOURCE_NAME;
                        resPresenter.PaneTitle = selectedResource.RESOURCE_NAME;
                        resPresenter.IsSelectedTab = true;
                        resPresenter.ViewModeChanged("Day");
                        clinicGroupView.Presenters.Add(resPresenter.SelectedResource.RESOURCE_NAME, resPresenter);
                    }
				}

                if (clinicGroupView.Presenters.Count > 0)
                {
                    groupPresentationModel.View.DockSchedule(clinicGroupView);
                    ChildPanes.Add(groupPresentationModel.View);
                }
            }
		}

		public void RemoveViewFromList (IMultiSchedulerView View)
		{
		    try
		    {
		        if (ChildPanes.Contains(View))
		            ChildPanes.Remove(View);
		    }
            catch (Exception ex)
            {
                TraceLog.LogException(this.GetType().Name, "RemoveViewFromList", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }

        private void CloseAllSchedules(string arg)
        {
            try
            {
                List<IMultiSchedulerView> openPanes = new List<IMultiSchedulerView>();
                openPanes.AddRange(ChildPanes);

                foreach (IMultiSchedulerView pane in openPanes)
                {
                    RemoveViewFromList(pane);
                    RegionManager.SetRegionManager(pane.MultiSchedulerGroupControl, this.regionManager);
                    this.regionManager.Regions[RegionNames.TaskGroupRegion].Remove(pane);
                    pane.Dispose();
                }

                openPanes.Clear();
                openPanes = null;
                GC.Collect(3);
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }
    }
}
