using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Collections;
using System.Globalization;
using System.Linq;
using System.Xml.Linq;
using System.Data;

using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Regions;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Events;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Scheduler;
using ClinSchd.Modules.Task.Group;
using ClinSchd.Modules.Task.Scheduler.Controllers;
using Microsoft.Practices.Composite.Presentation.Regions;
using ClinSchd.Modules.Task.MultiScheduler.Controllers;

namespace ClinSchd.Modules.Task.MultiScheduler
{
	public class MultiSchedulerOutlookPresentationModel : IMultiSchedulerOutlookPresentationModel, ITaskPresentationModel, INotifyPropertyChanged
    {
		private readonly IRegionManager regionManager;
		private readonly IDataAccessService dataAccessService;
		private readonly IEventAggregator eventAggregator;
		public event PropertyChangedEventHandler PropertyChanged;
		private string _paneTitle = "";
		private int _dayRange = 1;
		private TimeSpan _timeScale = TimeSpan.FromMinutes(10);
		private bool _isSelectedTab = false;
		private SchdResource selectedResource;
		private SchdResourceGroup selectedResourceGroup;
		private IList<SchdAppointment> appointmentList;
		private DateTime visibleStartDate;
		private DateTime visibleEndDate;
		private bool _isDisplayWalkInEnabled = true;
		private bool _isEnableUpdateAppointment = true;
		private string _resourceName;
		private int _timeSlotToSkip = 15;
	    public AppointmentRequest AppointmentRequest { get; set; }
	    public Patient SelectedPatient { get; set; }
        public MultiSchedulerOutlookPresentationModel(
			IMultiSchedulerOutlookView view,
			IRegionManager regionManager,
			IDataAccessService dataAccessService,
			IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
			this.regionManager = regionManager;
			this.dataAccessService = dataAccessService;
			this.eventAggregator = eventAggregator;

			LoadAppointmentCategories ();
			foreach (KeyValuePair<string, Category> pair in AppointmentCategories) {
				View.Scheduler.Categories.Add (pair.Value);
			}

			CreateAppointmentCommand = new DelegateCommand<string> (ExecuteCreateAppointmentCommand, CanExecuteCreateAppointmentCommand);
			CreateWalkInAppointmentCommand = new DelegateCommand<string> (ExecuteCreateWalkInAppointmentCommand, CanExecuteCreateWalkInAppointmentCommand);
			EditAppointmentCommand = new DelegateCommand<string> (ExecuteEditAppointmentCommand, CanExecuteEditAppointmentCommand);
			ViewAppointmentCommand = new DelegateCommand<string> (ExecuteViewAppointmentCommand, CanExecuteViewAppointmentCommand);
			EnableAppointmentButtonsCommand = new DelegateCommand<string> (ExecuteEnableAppointmentButtonsCommand, CanExecuteEnableAppointmentButtonsCommand);
			ResetUnselectedTabCommand = new DelegateCommand<string> (ExecuteEnableAppointmentButtonsCommand, CanExecuteEnableAppointmentButtonsCommand);
			TaskTabSelectedCommand = new DelegateCommand<object> (ExecuteSchedulerTabSelectedCommand, CanExecuteSchedulerTabSelectedCommand);
			MarkAsNoShowCommand = new DelegateCommand<string> (ExecuteMarkAsNoShowCommand, CanExecuteMarkAsNoShowCommand);
			UndoANoShowCommand = new DelegateCommand<string> (ExecuteUndoANoShowCommand, CanExecuteUndoANoShowCommand);
			CheckInPatientCommand = new DelegateCommand<string> (ExecuteCheckInPatientCommand, CanExecuteCheckInPatientCommand);
			UndoCheckInPatientCommand = new DelegateCommand<string> (ExecuteUndoCheckInPatientCommand, CanExecuteUndoCheckInPatientCommand);
			CheckOutPatientCommand = new DelegateCommand<string> (ExecuteCheckOutPatientCommand, CanExecuteCheckOutPatientCommand);
			UndoCheckOutPatientCommand = new DelegateCommand<string> (ExecuteUndoCheckOutPatientCommand, CanExecuteUndoCheckOutPatientCommand);
			CancelAppointmentCommand = new DelegateCommand<Action<bool>> (ExecuteCancelAppointmentCommand, CanExecuteCancelAppointmentCommand);
			UndoCancelAppointmentCommand = new DelegateCommand<string> (ExecuteUndoCancelAppointmentCommand, CanExecuteUndoCancelAppointmentCommand);
			CopyAppointmentCommand = new DelegateCommand<string> (ExecuteCopyAppointmentCommand, CanExecuteCopyAppointmentCommand);
			PasteAppointmentCommand = new DelegateCommand<string> (ExecutePasteAppointmentCommand, CanExecutePasteAppointmentCommand);

			this.eventAggregator.GetEvent<ViewDayRangeEvent> ().Subscribe (DayRangeChanged, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<ViewTimeScaleEvent> ().Subscribe (TimeScaleChanged, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<ChangeViewModeEvent> ().Subscribe (ViewModeChanged, ThreadOption.UIThread, false);
			this.eventAggregator.GetEvent<DisplayWalkInEvent> ().Subscribe (DisplayWalkInChanged, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<RefreshScheduler> ().Subscribe (RefreshAppoinmentScheduler, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<UndoNoShowAppointmentEvent> ().Subscribe (UndoNoShowAppointment, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<UndoCheckInPatientEvent> ().Subscribe (UndoCheckInPatient, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<UndoCheckOutPatientEvent> ().Subscribe (UndoCheckOutPatient, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<UndoCancelAppointmentEvent> ().Subscribe (UndoCancelAppointment, ThreadOption.UIThread, true);
			this.eventAggregator.GetEvent<PatientSelectedEvent> ().Subscribe (PatientSelected, ThreadOption.UIThread, false);
            this.eventAggregator.GetEvent<PatientUnselectEvent>().Subscribe(PatientUnselect, ThreadOption.UIThread, false);
			this.eventAggregator.GetEvent<SchedulerViewDateEvent> ().Subscribe (SchedulerDateChanged, ThreadOption.UIThread, false);
			this.eventAggregator.GetEvent<CopyAppointmentEvent> ().Subscribe (CopyAppointment, ThreadOption.UIThread, false);
			this.eventAggregator.GetEvent<PasteAppointmentEvent> ().Subscribe (PasteAppointment, ThreadOption.UIThread, false);
			this.eventAggregator.GetEvent<PrintCurrentSchedulerEvent> ().Subscribe (PrintCurrentScheduler, ThreadOption.UIThread, false);
            this.eventAggregator.GetEvent<AppointmentRequestSelectedEvent>().Subscribe(AppointmentRequestSelected, ThreadOption.UIThread, false);
		}

		public Patient Patient { get; set; }

	    private bool _isAPatientSelected;

        private void PatientUnselect(string s)
        {
            Patient = null;
            _isAPatientSelected = false;
        }
		public bool IsAPatientSelected 
        { 
            get { return _isAPatientSelected; }
            set { _isAPatientSelected = value; }
        }

		public void SelectPatient (Patient selectedPatient)
		{
			patientSelectedByMe = true;
			this.eventAggregator.GetEvent<PatientSelectedEvent> ().Publish (selectedPatient);
		    SelectedPatient = selectedPatient;
		}

	    private void AppointmentRequestSelected(AppointmentRequest appointmentRequest)
	    {
	        AppointmentRequest = appointmentRequest;
	    }

		private Dictionary<string, Category> appointmentCategories = new Dictionary<string, Category> ();

		[Dependency]
		public Factory<Patient> PatientFactory { get; set; }

		[Dependency]
		public Factory<SchdAppointment> SchdAppointmentFactory { get; set; }

		public DelegateCommand<string> CreateAppointmentCommand { get; private set; }
		public DelegateCommand<string> CreateWalkInAppointmentCommand { get; private set; }
		public DelegateCommand<string> EditAppointmentCommand { get; private set; }
		public DelegateCommand<string> ViewAppointmentCommand { get; private set; }
		public DelegateCommand<string> EnableAppointmentButtonsCommand { get; private set; }
		public DelegateCommand<string> ResetUnselectedTabCommand { get; private set;}
		public DelegateCommand<object> TaskTabSelectedCommand { get; private set; }
		public DelegateCommand<string> MarkAsNoShowCommand { get; private set; }
		public DelegateCommand<string> UndoANoShowCommand { get; private set; }
		public DelegateCommand<string> CheckInPatientCommand { get; private set; }
		public DelegateCommand<string> UndoCheckInPatientCommand { get; private set; }
		public DelegateCommand<string> CheckOutPatientCommand { get; private set; }
		public DelegateCommand<string> UndoCheckOutPatientCommand { get; private set; }
        public DelegateCommand<Action<bool>> CancelAppointmentCommand { get; private set; }
		public DelegateCommand<string> UndoCancelAppointmentCommand { get; private set; }
		public DelegateCommand<string> CopyAppointmentCommand { get; private set; }
		public DelegateCommand<string> PasteAppointmentCommand { get; private set; }

		private void LoadAppointmentCategories ()
		{
            foreach (var item in Keys.ColorCodes.CategoriesList)
                appointmentCategories.Add(item.Key, new Category(item.Key, item.Value));
		}

		public void LoadAppointments ()
		{
		    AppointmentList = SelectedResource.GetAppointments(visibleStartDate, visibleEndDate);
            View.Scheduler.AppointmentsSource = new SchdAppointmentSource(this.AppointmentList, this.AppointmentCategories, this.IsEnableUpdateAppointment,dataAccessService).Appointments;

			/*this.SelectedResource.GetVisibleAppointments (this.VisibleStartDate, this.VisibleEndDate,
				(s, args) => {
					this.AppointmentList = args.Result as List<SchdAppointment>;
					this.View.Scheduler.AppointmentsSource = new SchdAppointmentSource (this.AppointmentList, this.AppointmentCategories, this.IsEnableUpdateAppointment).Appointments;
				});
			*/
		}

		private bool shouldRefreshScheduler = true;
		public void RefreshAppoinmentScheduler (string command)
		{
			if (shouldRefreshScheduler) {

			    if (Keys.Methods.IsDate(Keys.RequestKeys.CurrentSelectedDesiredDate))
			    {
			        if (Convert.ToDateTime(Keys.RequestKeys.CurrentSelectedDesiredDate) <= DateTime.Today)
			        {
			            this.visibleStartDate = DateTime.Today;
			            this.visibleEndDate = DateTime.Today.AddDays(7);
			        }
			        else
			        {
                        this.VisibleStartDate = Convert.ToDateTime(Keys.RequestKeys.CurrentSelectedDesiredDate).AddDays(-3);
                        this.VisibleEndDate = Convert.ToDateTime(Keys.RequestKeys.CurrentSelectedDesiredDate).AddDays(3);
			        }
			    }
			    else
			    {
                    this.VisibleStartDate = this.View.Scheduler.VisibleRangeStart;
                    this.VisibleEndDate = this.View.Scheduler.VisibleRangeEnd;
			    }

                if (AppointmentRequest != null)
                    Keys.RequestKeys.CurrentSelectedDesiredDate = AppointmentRequest.DesiredApptDateShortString;
				this.dataAccessService.RefreshAvailabilitySchedule (SelectedResource.RESOURCE_NAME, this.VisibleStartDate.AddDays (-1), this.VisibleEndDate.AddDays (1));

				((ClinSchd.Modules.Task.Scheduler.TimeSlotTemplateSelector)View.Scheduler.TimeSlotTemplateSelector).startTime = this.VisibleStartDate;
                ((ClinSchd.Modules.Task.Scheduler.TimeSlotTemplateSelector)View.Scheduler.TimeSlotTemplateSelector).endTime = this.VisibleEndDate;
                ((ClinSchd.Modules.Task.Scheduler.TimeSlotTemplateSelector)View.Scheduler.TimeSlotTemplateSelector).Refresh();

                AppointmentList = SelectedResource.GetAppointments(visibleStartDate, visibleEndDate);
                View.Scheduler.AppointmentsSource = new SchdAppointmentSource(this.AppointmentList, this.AppointmentCategories, this.IsEnableUpdateAppointment, dataAccessService).Appointments;
                SchedulerViewMode currentMode = View.Scheduler.ViewMode;
                View.Scheduler.ViewMode = SchedulerViewMode.Month;
                View.Scheduler.ViewMode = currentMode;
                if (View.Scheduler.VisibleRangeStart != this.VisibleStartDate)
                {
                    shouldRefreshScheduler = false;
                    View.Scheduler.FirstDayOfWeek = this.VisibleStartDate.DayOfWeek;
                    View.Scheduler.SetFirstVisibleDate(this.VisibleStartDate);
                }
                /*
				this.SelectedResource.GetVisibleAppointments (this.VisibleStartDate, this.VisibleEndDate,
					(s, args) => {
						this.AppointmentList = args.Result as List<SchdAppointment>;
						this.View.Scheduler.AppointmentsSource = new SchdAppointmentSource (this.AppointmentList, this.AppointmentCategories, this.IsEnableUpdateAppointment).Appointments;
						SchedulerViewMode currentMode = View.Scheduler.ViewMode;
						View.Scheduler.ViewMode = SchedulerViewMode.Month;
						View.Scheduler.ViewMode = currentMode;
						if (View.Scheduler.VisibleRangeStart != this.VisibleStartDate)
						{
							shouldRefreshScheduler = false;
							View.Scheduler.FirstDayOfWeek = this.VisibleStartDate.DayOfWeek;
							View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
						}
					});*/
			} else {
				shouldRefreshScheduler = true;
			}
		}

		private void PrintCurrentScheduler (object T)
		{
			if (this.IsSelectedTab) {
				PrintDialog dialog = new PrintDialog ();
				if (dialog.ShowDialog () == true) { dialog.PrintVisual (View.Scheduler, "Schedule"); }
			}
		}

		public void DisplayWalkInChanged (string isDisplayWalkInEnabled)
		{
			if (IsSelectedTab) {
				this.VisibleStartDate = this.View.Scheduler.VisibleRangeStart;
				this.VisibleEndDate = this.View.Scheduler.VisibleRangeEnd;
				this._isDisplayWalkInEnabled = isDisplayWalkInEnabled == "true" ? true : false;

			    AppointmentList = SelectedResource.GetAppointments(visibleStartDate, visibleEndDate);
                View.Scheduler.AppointmentsSource = new SchdAppointmentSource(this.AppointmentList, this.AppointmentCategories, this.IsEnableUpdateAppointment, dataAccessService).Appointments;
                /*
				this.SelectedResource.GetVisibleAppointments (this.VisibleStartDate, this.VisibleEndDate,
					(s, args) => {
						this.AppointmentList = args.Result as List<SchdAppointment>;
						this.View.Scheduler.AppointmentsSource = new SchdAppointmentSource (this.AppointmentList, this.AppointmentCategories, this.IsEnableUpdateAppointment).Appointments;
					});*/
			}
		}

		public void CopyAppointment (SchdAppointment copyAppt)
		{
			this.CopySchdAppointment = copyAppt;
		}

		public void PasteAppointment (SchdAppointment pasteAppt)
		{
			if (this.IsSelectedTab && this.CopySchdAppointment != null) {
				AppointmentData a = new AppointmentData ();
				PatientInformation p = new PatientInformation ();
				a.AppointmentID = "0";
				a.StartTime = pasteAppt.START_TIME.ToString ();
				a.Duration = (Convert.ToDateTime (this.CopySchdAppointment.END_TIME.ToString ()) - Convert.ToDateTime (this.CopySchdAppointment.START_TIME.ToString ())).TotalMinutes.ToString ();
				a.EndTime = (Convert.ToDateTime (pasteAppt.START_TIME.ToString ()).AddMinutes (int.Parse (a.Duration))).ToString ();
				a.Notes = this.CopySchdAppointment.NOTE;
				a.ResourceName = this.CopySchdAppointment.RESOURCENAME;

				p.IEN = this.CopySchdAppointment.PATIENTID;
				p.Name = this.CopySchdAppointment.PATIENTNAME;

				string errorMessage = string.Empty;
				List<string> errorMessages = new List<string> ();
				bool bOverbookAppointment = false;
				bool isConflictAppointment = false;

				errorMessage = this.dataAccessService.IsHoliday (Convert.ToDateTime (pasteAppt.START_TIME));
                var canBookHoliday = false;
                if (AppointmentRequest != null && !string.IsNullOrEmpty(errorMessage))
                {
                    if (!string.IsNullOrEmpty(AppointmentRequest.ClinicIen))
                        canBookHoliday = dataAccessService.ClinicCanBookOnHoliday(AppointmentRequest.ClinicResourceId, null);
                    if (!canBookHoliday)
                    {
                        var msg = Convert.ToDateTime(pasteAppt.START_TIME).ToShortDateString() + " is a holiday (" + errorMessage + ") and is not allowed to be scheduled.";
                        MessageBox.Show(msg);
                        return;
                    }
                }
				//if (errorMessage == string.Empty || (View.ConfirmUser (Convert.ToDateTime (pasteAppt.START_TIME).ToShortDateString () + " is a holiday (" + errorMessage + ").  Are you sure you want to make this appointment?", "Paste Appointment"))) 
                //{
					//check conflict appointment
                    string conflictingClinic;
                    isConflictAppointment = this.dataAccessService.CheckConflictAppointment(pasteAppt.RESOURCENAME, p.IEN, pasteAppt.START_TIME, pasteAppt.END_TIME, out conflictingClinic);

					if (isConflictAppointment) {
                        View.AlertUser("Patient " + p.Name + " already had appt at " + pasteAppt.START_TIME + " in clinic " + conflictingClinic, "Paste Appointment");
						return;
					}

					//check overbook authority
					bOverbookAppointment = this.dataAccessService.OverBookAuthority (pasteAppt);

                    //Check No Shows
                    errorMessage = string.Empty;
                    if (Convert.ToDateTime(pasteAppt.START_TIME).Date > DateTime.Now.Date)
                        errorMessage = this.dataAccessService.NoShow(p.IEN, pasteAppt.RESOURCENAME, "AddNew");
                    if (errorMessage != string.Empty)
                        View.AlertUser(errorMessage, "Patient No-Shows");
					//check slots available
					errorMessage = this.dataAccessService.CheckSlotsAvailable (pasteAppt.START_TIME, pasteAppt.END_TIME, pasteAppt.RESOURCENAME);

					if (errorMessage != string.Empty) {
						if (bOverbookAppointment) {
							if (View.ConfirmUser (errorMessage, "Paste Appointment")) {
                                errorMessages = this.dataAccessService.AddNewAppointment (a, p, false);
							}
						} else {
							View.AlertUser ("You do not have permission to overbook this appointment.", "New Appointment");
							return;
						}
					} else {
						errorMessages = this.dataAccessService.AddNewAppointment (a, p, false);
					}
				//}
				if (errorMessages.Count > 0) {
					errorMessage = errorMessages[1];
				}
				if (errorMessage != string.Empty) {
					this.View.AlertUser (errorMessage, "Copy/Paste Appointment");
				}
				this.eventAggregator.GetEvent<RefreshScheduler> ().Publish ("Edit Appointment");
			}
		}

		public void UndoCancelAppointment (string apptID)
		{
			string errorMessage = string.Empty;
			errorMessage = this.dataAccessService.UndoCancelAppointment (this.SchdAppointment.APPOINTMENTID);

			if (errorMessage != string.Empty) {
				this.View.AlertUser (errorMessage, "Undo Cancel Appointment");
			} else {
				RefreshAppoinmentScheduler (string.Empty);
			}
		}

		public void UndoNoShowAppointment (string apptID)
		{
			string errorMessage = string.Empty;
			errorMessage = this.dataAccessService.AppointmentNoShow (this.SchdAppointment.APPOINTMENTID, false);

			if (errorMessage != string.Empty) {
				this.View.AlertUser (errorMessage, "Undo NoShow Appointment");
			} else {
				RefreshAppoinmentScheduler (string.Empty);
			}
		}

		public void UndoCheckInPatient (string apptID)
		{
			string errorMessage = string.Empty;
			errorMessage = this.dataAccessService.UndoCheckIn (this.SchdAppointment.APPOINTMENTID);

			if (errorMessage != string.Empty) {
				this.View.AlertUser (errorMessage, "Undo CheckIn Patient");
			} else {
				RefreshAppoinmentScheduler (string.Empty);
			}
		}

		public void UndoCheckOutPatient (string apptID)
		{
			string errorMessage = string.Empty;
			errorMessage = this.dataAccessService.UndoCheckOut (this.SchdAppointment.APPOINTMENTID);

			if (errorMessage != string.Empty) {
				this.View.AlertUser (errorMessage, "Undo CheckOut Patient");
			} else {
				RefreshAppoinmentScheduler (string.Empty);
			}
		}

		private void SchedulerDateChanged (SchedulerViewDateEventArgs newDate)
		{
			if (IsSelectedTab) {
				//((TimeSlotTemplateSelector)View.Scheduler.TimeSlotTemplateSelector).startTime = newDate.AddDays (1);
				//((TimeSlotTemplateSelector)View.Scheduler.TimeSlotTemplateSelector).endTime = newDate.AddDays (View.Scheduler.WeekViewDefinition.VisibleDays + 1);
				//((TimeSlotTemplateSelector)View.Scheduler.TimeSlotTemplateSelector).Refresh ();
				this.View.Scheduler.FirstDayOfWeek = newDate.NewDate.DayOfWeek;
				this.View.Scheduler.SetFirstVisibleDate (newDate.NewDate);
			}
		}

		public void DayRangeChanged (string dayRange)
		{
			_dayRange = int.Parse (dayRange);

			if (IsSelectedTab) {
				this.View.Scheduler.WeekViewDefinition.VisibleDays = _dayRange;
			}
		}

		public void ViewModeChanged (string viewMode)
		{
			if (IsSelectedTab) {
				switch (viewMode) {
				case "Day":
					this.View.Scheduler.ViewMode = SchedulerViewMode.Day;
					this.View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
					break;
				case "Month":
					this.View.Scheduler.ViewMode = SchedulerViewMode.Month;
					this.View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
					break;
				default:
					this.View.Scheduler.ViewMode = SchedulerViewMode.Week;
					this.View.Scheduler.SetFirstVisibleDate (this.VisibleStartDate);
					break;
				}
			}
		}

		public void TimeScaleChanged (string timeScale)
		{
			switch (timeScale) {
			case "10":
				_timeScale = System.TimeSpan.FromMinutes (10);
				_timeSlotToSkip = 46;
				break;
			case "15":
				_timeScale = System.TimeSpan.FromMinutes (15);
				_timeSlotToSkip = 30;
				break;
			case "20":
				_timeScale = System.TimeSpan.FromMinutes (20);
				_timeSlotToSkip = 23;
				break;
			case "30":
				_timeScale = System.TimeSpan.FromMinutes (30);
				_timeSlotToSkip = 15;
				break;
                case "60":
			        _timeScale = System.TimeSpan.FromMinutes(60);
			        _timeSlotToSkip = 30;
			        break;
			default:
				_timeScale = System.TimeSpan.FromMinutes (30);
				_timeSlotToSkip = 15;
				break;
			}
			if (IsSelectedTab) {
				this.View.Scheduler.WeekViewDefinition.TimeSlotLength = _timeScale;
				this.View.Scheduler.DayViewDefinition.TimeSlotLength = _timeScale;
			}
		}

		public void ExecuteCopyAppointmentCommand (string command)
		{
			this.eventAggregator.GetEvent<CopyAppointmentEvent> ().Publish (this.SchdAppointment);
		}

		public bool CanExecuteCopyAppointmentCommand (string command)
		{
			return true;
		}

		public void ExecutePasteAppointmentCommand (string command)
		{
			this.eventAggregator.GetEvent<IsPasteAvailabilityEvent> ().Publish (command);
		}

		public bool CanExecutePasteAppointmentCommand (string command)
		{
			return true;
		}

		public void ExecuteSchedulerTabSelectedCommand (object resource)
		{
			SelectedResource = resource as SchdResource;
			this.eventAggregator.GetEvent<ResourceSelectedEvent> ().Publish (SelectedResource);

			int selectedMin = 0;
			int selectedIndex = 3;
			if (IsSelectedTab) {
				switch (this.View.Scheduler.ViewMode)
				{
					case SchedulerViewMode.Week:
					    selectedMin = this.View.Scheduler.WeekViewDefinition.TimeSlotLength.Minutes;
                        if (SelectedResource != null && selectedMin == 0)
                            selectedMin = SelectedResource.Clinic.Duration;
					    break;
					case SchedulerViewMode.Day:
					selectedMin = this.View.Scheduler.DayViewDefinition.TimeSlotLength.Minutes;
                    if (SelectedResource != null && selectedMin == 0)
                        selectedMin = SelectedResource.Clinic.Duration;
					break;
					default:
					selectedMin = 30;
					break;
				}

				switch (selectedMin) {
					case 10:
					    selectedIndex = 0;
					    break;
					case 15:
					    selectedIndex = 1;
					    break;
					case 20:
					    selectedIndex = 2;
					    break;
					case 30:
					    selectedIndex = 3;
					    break;
                    case 60:
				        selectedIndex = 4;
				        break;
				}
				eventAggregator.GetEvent<SelectTimeScaleEvent> ().Publish (selectedIndex);
			}
		}

		public void ExecuteEnableAppointmentButtonsCommand (string command)
		{
			this.eventAggregator.GetEvent<PassAppointmentsEvent> ().Publish (this.SchdAppointment);

			string errorMessage = string.Empty;
			if (command == "New Appointment") {
				//check access block
				errorMessage = this.dataAccessService.CheckSlotsAvailable (this.SchdAppointment.START_TIME, this.SchdAppointment.END_TIME, this.SchdAppointment.RESOURCENAME);

				if (errorMessage != string.Empty && errorMessage == "IsPreventAccess") {
					IsAPatientSelected = false;
					OnPropertyChanged ("IsAPatientSelected");
					command = "Reset Buttons";
				} else {
					if (Patient != null) {
						IsAPatientSelected = true;
						OnPropertyChanged ("IsAPatientSelected");
					} else {
						IsAPatientSelected = false;
						OnPropertyChanged ("IsAPatientSelected");
					}
				}
			}

			this.eventAggregator.GetEvent<EnableAppointmentButtonsEvent> ().Publish (command);

		}

		private bool patientSelectedByMe;
		public void PatientSelected (Patient selectedPatient)
		{
			if (selectedPatient != null)
			{
			    Keys.SelectedPatient = selectedPatient;
				if (!patientSelectedByMe) {
					Patient = selectedPatient;
					OnPropertyChanged ("Patient");
				} else {
					patientSelectedByMe = false;
				}
				IsAPatientSelected = true;
				OnPropertyChanged ("IsAPatientSelected");
			} else {
				IsAPatientSelected = false;
				OnPropertyChanged ("IsAPatientSelected");
			}
		}

		public void ExecuteEditAppointmentCommand (string command)
		{
			this.eventAggregator.GetEvent<EditAppointmentEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteViewAppointmentCommand (string command)
		{
			this.eventAggregator.GetEvent<ViewAppointmentEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteCreateAppointmentCommand (string title)
		{
            if(Patient == null)
            {
                MessageBox.Show("Please select a patient");
                return;
            }
			this.SchdAppointment.PATIENTID = this.Patient.IEN;
			this.SchdAppointment.PATIENTNAME = this.Patient.Name;
			this.eventAggregator.GetEvent<AddNewAppointmentEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteCreateWalkInAppointmentCommand (string title)
		{
			this.SchdAppointment.PATIENTID = this.Patient.IEN;
			this.SchdAppointment.PATIENTNAME = this.Patient.Name;
			this.eventAggregator.GetEvent<AddWalkInAppointmentEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteMarkAsNoShowCommand (string title)
		{
			this.eventAggregator.GetEvent<LaunchMarkAsNoShowDialogEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteUndoANoShowCommand (string title)
		{
			UndoNoShowAppointment (this.SchdAppointment.APPOINTMENTID);
		}

		public void ExecuteCheckInPatientCommand (string command)
		{
			this.eventAggregator.GetEvent<LaunchCheckInDialogEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteUndoCheckInPatientCommand (string command)
		{
			UndoCheckInPatient (this.SchdAppointment.APPOINTMENTID);
		}

		public void ExecuteCheckOutPatientCommand (string command)
		{
			this.eventAggregator.GetEvent<LaunchCheckOutDialogEvent> ().Publish (this.SchdAppointment);
		}

		public void ExecuteUndoCheckOutPatientCommand (string command)
		{
			UndoCheckOutPatient (this.SchdAppointment.APPOINTMENTID);
		}

		public void ExecuteCancelAppointmentCommand (Action<bool> successCallback)
		{
            if (String.IsNullOrEmpty(this.SchdAppointment.MRTCParentRequestIEN))
                this.eventAggregator.GetEvent<LaunchCancelApptDialogEvent>().Publish(new CancelApptDialogEventPayload(this.SchdAppointment, successCallback));
            else
                this.eventAggregator.GetEvent<LaunchMRTCCancelDialog>().Publish(new LaunchMRTCCancelDialogArgs(null, this.SchdAppointment, null, null));
		}

		public void ExecuteUndoCancelAppointmentCommand (string command)
		{
			UndoCancelAppointment (this.SchdAppointment.APPOINTMENTID);
		}

		public bool CanExecuteSchedulerTabSelectedCommand (object resource)
		{
			return true;
		}

		public bool CanExecuteEnableAppointmentButtonsCommand (string command)
		{
			return true;
		}

		public bool CanExecuteViewAppointmentCommand (string title)
		{
			return true;
		}

		public bool CanExecuteEditAppointmentCommand (string title)
		{
			return true;
		}

		public bool CanExecuteCreateAppointmentCommand (string title)
		{
			return true;
		}

		public bool CanExecuteCreateWalkInAppointmentCommand (string title)
		{
			return true;
		}

		public bool CanExecuteMarkAsNoShowCommand (string title)
		{
			return true;
		}

		public bool CanExecuteUndoANoShowCommand (string title)
		{
			return true;
		}

		public bool CanExecuteCheckInPatientCommand (string title)
		{
			return true;
		}

		public bool CanExecuteUndoCheckInPatientCommand (string title)
		{
			return true;
		}

		public bool CanExecuteCheckOutPatientCommand (string title)
		{
			return true;
		}

		public bool CanExecuteUndoCheckOutPatientCommand (string title)
		{
			return true;
		}

		public bool CanExecuteCancelAppointmentCommand (Action<bool> callback)
		{
			return true;
		}

		public bool CanExecuteUndoCancelAppointmentCommand (string title)
		{
			return true;
		}

		#region ISchedulerPresentationModel Members

		public IMultiSchedulerOutlookView View { get; private set; }

		public int TimeSlotToSkip
		{
			get { return _timeSlotToSkip; }
			set { _timeSlotToSkip = value; }
		}

		public bool IsDisplayWalkInEnabled
		{
			get { return _isDisplayWalkInEnabled; }
			set { _isDisplayWalkInEnabled = value; }
		}

		public bool IsEnableUpdateAppointment
		{
			get
			{
                if (Keys.SelectedAppointmentRequest == null)
                    _isEnableUpdateAppointment = false;
				return _isEnableUpdateAppointment;
			}
			set
			{
				if (this._isEnableUpdateAppointment != value) {
					this._isEnableUpdateAppointment = value;
					this.OnPropertyChanged ("IsEnableUpdateAppointment");
				}
			}
		}

		public Dictionary<string, Category> AppointmentCategories
		{
			get
			{
				return appointmentCategories;
			}
		}

		public IList<SchdAppointment> AppointmentList
		{
			get
			{
				return appointmentList;
			}
			set
			{
				if (this.appointmentList != value) {
					this.appointmentList = value;
					//this.OnPropertyChanged ("AppointmentList");
				}
			}
		}

		public SchdAppointment SchdAppointment { get; set; }
		public SchdAppointment CopySchdAppointment { get; set; }

		public DateTime VisibleStartDate
		{
			get
			{
				return visibleStartDate;
			}
			set
			{
				visibleStartDate = value;
			}
		}

		public DateTime VisibleEndDate
		{
			get
			{
				return visibleEndDate;
			}
			set
			{
				visibleEndDate = value;
			}
		}

		public string PaneTitle
		{
			get
			{
				return _paneTitle;
			}
			set
			{
				_paneTitle = value;
			}
		}

		public string ResourceName
		{
			get
			{
				return _resourceName;
			}
			set
			{
				_resourceName = value;
			}
		}

		public int DayRange
		{
			get
			{
				return _dayRange;
			}
			set
			{
				_dayRange = value;
			}
		}

		public TimeSpan TimeScale
		{
			get
			{
				return _timeScale;
			}
			set
			{
				_timeScale = value;
			}
		}

		public bool IsSelectedTab
		{
			get
			{
				return _isSelectedTab;
			}
			set
			{
				_isSelectedTab = value;
				OnPropertyChanged ("IsSelectedTab");
			}
		}

		public SchdResource SelectedResource
		{
			get
			{
				return selectedResource;
			}
			set
			{
				if (this.selectedResource != value && value != null) {
					this.selectedResource = value;
				}

				//check user's authority for updating the appointments
				if (!this.dataAccessService.IsEnableUpdateAppointment (this.selectedResource.RESOURCEID)) {
					this.View.Scheduler.IsReadOnly = true;
					this.View.Model.IsEnableUpdateAppointment = false;
				}
			}
		}

		public SchdResourceGroup SelectedResourceGroup
		{
			get
			{
				return selectedResourceGroup;
			}
			set
			{
				if (this.selectedResourceGroup != value && value != null) {
					this.selectedResourceGroup = value;
				}
			}
		}

		public ITaskMultiSchedulerController Controller { get; set; }

		public void ClearViewFromList ()
		{
			if (Controller != null) {
				Controller.RemoveViewFromList (View as IMultiSchedulerView);
			}
		}
		#endregion

		public void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

		internal void ResizeAppointment (IAppointment newAppointment)
		{
			string errorMessage = string.Empty;
			string apptnote = string.Empty;
			string apptid = string.Empty;
			string resourceid = string.Empty;
			string resoourcename = string.Empty;
			AppointmentData a = new AppointmentData ();
			foreach (Resource item in newAppointment.Resources) {
				switch (item.ResourceType) {
				case "NOTE":
					apptnote = item.ResourceName;
					break;
				case "RESOURCENAME":
					resoourcename = item.ResourceName;
					break;
				case "APPOINTMENTID":
					apptid = item.ResourceName;
					break;
				default:
					break;
				}
			}
			a.Notes = apptnote;
			a.Duration = (newAppointment.End - newAppointment.Start).TotalMinutes.ToString ();
			a.AppointmentID = apptid;

            var list = this.dataAccessService.GetResources(resoourcename, false);
            var schdRes = list.FirstOrDefault(r => (r.RESOURCE_NAME == resoourcename));
            bool variableLengthAppointments = false;

            if (schdRes != null)
                variableLengthAppointments = schdRes.VariableLengthAppointments;

            if (!variableLengthAppointments)
            {
                View.AlertUser("Appointment could not be modified:" + Environment.NewLine + Environment.NewLine +
                "Variable Length Appointments are not enabled for this clinic in VistA Scheduling Management.", "Error");
			}
            else
            {
				errorMessage = this.dataAccessService.EditAppointment (a);

				if (errorMessage != string.Empty) {
					View.AlertUser ("Appointment could not be modified:" + Environment.NewLine + Environment.NewLine + errorMessage, "Error");
				}				
			}
			this.eventAggregator.GetEvent<RefreshScheduler> ().Publish ("Edit Appointment");
		}

		internal bool MoveAppointment (IAppointment newAppointment)
		{			
			string errorMessage = string.Empty;
			string apptid = string.Empty;
			string apptnote = string.Empty;
			string apptresourcename = string.Empty;
			string patientien = string.Empty;
			string patientname = string.Empty;
			foreach (Resource item in newAppointment.Resources) {
				switch (item.ResourceType) {
				case "APPOINTMENTID":
					apptid = item.ResourceName;
					break;
				case "NOTE":
					apptnote = item.ResourceName;
					break;
				case "RESOURCENAME":
					apptresourcename = item.ResourceName;
					break;
				case "PATIENTID":
					patientien = item.ResourceName;
					break;
				case "PATIENTNAME":
					patientname = item.ResourceName;
					break;
				default:
					break;
				}
			}
			AppointmentData a = new AppointmentData ();
			PatientInformation p = new PatientInformation ();
			SchdAppointment Appt = new SchdAppointment ();
			a.AppointmentID = "0";
			a.StartTime = newAppointment.Start.ToString();
			a.EndTime = newAppointment.End.ToString();
			a.Duration = (newAppointment.End - newAppointment.Start).TotalMinutes.ToString ();
			a.Notes = apptnote;
			a.ResourceName = apptresourcename;
			Appt.RESOURCENAME = a.ResourceName;
			Appt.START_TIME = a.StartTime;
			Appt.END_TIME = a.EndTime;

			p.IEN = patientien;
			p.Name = patientname;


			bool bOverbookAppointment = false;
			bool isConflictAppointment = false;
			List<string> errorMessages = new List<string> ();
			errorMessage = this.dataAccessService.IsHoliday (Convert.ToDateTime (a.StartTime));
            var canBookHoliday = false;
            if (!string.IsNullOrEmpty(errorMessage))
            {
                if (!string.IsNullOrEmpty(Appt.RESOURCENAME))
                    canBookHoliday = dataAccessService.ClinicCanBookOnHoliday(null, Appt.RESOURCENAME);
                if (!canBookHoliday)
                {
                    var msg = newAppointment.Start.ToShortDateString() + " is a holiday (" + errorMessage + ") and is not allowed to be scheduled.";
                    MessageBox.Show(msg);
                    return false;
                }
            }
            
            //if (errorMessage == string.Empty || (View.ConfirmUser(Convert.ToDateTime(a.StartTime).ToShortDateString() + " is a holiday (" + errorMessage + ").  Are you sure you want to make this appointment?", "Paste Appointment")))
            //{
                //check conflict appointment
                string conflictingClinic;
                isConflictAppointment = this.dataAccessService.CheckConflictAppointment(a.ResourceName, p.IEN, a.StartTime, a.EndTime, out conflictingClinic);

                if (isConflictAppointment)
                {
                    View.AlertUser("Patient " + p.Name + " has an appt scheduled in " + conflictingClinic + " at " + a.StartTime, "Move Appointment");
                    return false;
                }

                //check overbook authority
                bOverbookAppointment = this.dataAccessService.OverBookAuthority(Appt);
                //Check No Shows
                errorMessage = string.Empty;
                if (Convert.ToDateTime(a.StartTime).Date > DateTime.Now.Date)
                    errorMessage = this.dataAccessService.NoShow(p.IEN, a.ResourceName, "AddNew");
                if (errorMessage != string.Empty)
                    View.AlertUser(errorMessage, "Patient No-Shows");
                //check slots available
                errorMessage = this.dataAccessService.CheckSlotsAvailable(a.StartTime, a.EndTime, a.ResourceName);

                CancelAppointment cancelAppointment = new CancelAppointment();
                cancelAppointment.ApptID = apptid;
                cancelAppointment.CancelledByClinic = true;
                cancelAppointment.PatientDesiredDate = Convert.ToDateTime(a.StartTime);
                cancelAppointment.Notes = "Moved By Clinic";
                if (errorMessage == string.Empty || bOverbookAppointment)
                {
                    cancelAppointment.ReasonIEN = this.dataAccessService.GetCancellationReasons("CLINIC")[0].Value;

                    errorMessage = this.dataAccessService.CancelAppointment(cancelAppointment);
                    if (errorMessage == string.Empty)
                    {
                        errorMessages = this.dataAccessService.AddNewAppointment(a, p, false);
                        errorMessage = errorMessages[1].ToString();
                    }
                }
                else
                {
                    View.AlertUser("You do not have permission to overbook this appointment.", "Move Appointment");
                    return false;
                }

            //}
            //else
           // {
            //    return false;
            //}

			if (errorMessage != string.Empty) {
				View.AlertUser ("Appointment could not be modified:" + Environment.NewLine + Environment.NewLine + errorMessage, "Error");
				return false;
			}
			this.eventAggregator.GetEvent<RefreshScheduler> ().Publish ("Edit Appointment");
			return true;
		}

        public void LaunchAddNewAppointmentEvent(SchdAppointment schdAppointment)
        {
            eventAggregator.GetEvent<AddNewAppointmentEvent>().Publish(schdAppointment);
        }
        public void LaunchWalkInAppointmentEvent(SchdAppointment schdAppointment)
        {
            eventAggregator.GetEvent<AddWalkInAppointmentEvent>().Publish(this.SchdAppointment);
        }
	}
}
