using System;
using System.ComponentModel;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.MultiScheduler.Controllers;
using ClinSchd.Modules.Task.Group;
using Microsoft.Practices.Unity;
using ClinSchd.Infrastructure;
using Microsoft.Practices.Composite.Presentation.Commands;
using Microsoft.Practices.Composite.Events;

namespace ClinSchd.Modules.Task.MultiScheduler
{
    public class MultiSchedulerPresentationModel : IMultiSchedulerPresentationModel, ITaskPresentationModel, INotifyPropertyChanged, IDisposable
    {
        private readonly ITaskService _taskService;
        private readonly IEventAggregator _eventAggregator;
        public event PropertyChangedEventHandler PropertyChanged;
        private SchdResourceGroup _selectedResourceGroup;
        private SchdResource _selectedResource;
        private bool _isEnableUpdateAppointment = true;

        public MultiSchedulerPresentationModel (IMultiSchedulerView view, ITaskService taskService, IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _taskService = taskService;
            _eventAggregator = eventAggregator;
            ResetUnselectedTabCommand = new DelegateCommand<string> (ExecuteEnableAppointmentButtonsCommand, CanExecuteEnableAppointmentButtonsCommand);
            TaskTabSelectedCommand = new DelegateCommand<object> (ExecuteSchedulerTabSelectedCommand, CanExecuteSchedulerTabSelectedCommand);
        }

        public void ExecuteSchedulerTabSelectedCommand (object resourceGroup)
        {
            SelectedResourceGroup = resourceGroup as SchdResourceGroup;
            _eventAggregator.GetEvent<ResourceGroupSelectedEvent> ().Publish (SelectedResourceGroup);
        }

        public void ExecuteEnableAppointmentButtonsCommand (string command)
        {
            
        }

        public bool CanExecuteSchedulerTabSelectedCommand (object resource)
        {
            return true;
        }

        public bool CanExecuteEnableAppointmentButtonsCommand (string command)
        {
            return true;
        }

        public IMultiSchedulerView View { get; private set; }

        public ITaskMultiSchedulerController Controller { get; set; }

        public void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }

        #region ITaskPresentationModel Members

        public bool IsSelectedTab { get; set; }

        public bool IsEnableUpdateAppointment
        {
            get { return _isEnableUpdateAppointment; }
            set
            {
                if ( _isEnableUpdateAppointment != value )
                {
                    _isEnableUpdateAppointment = value;
                    OnPropertyChanged( "IsEnableUpdateAppointment" );
                }
            }
        }

        public SchdResourceGroup SelectedResourceGroup
        {
            get { return _selectedResourceGroup; }
            set
            {
                if ( _selectedResourceGroup != value && value != null )
                {
                    _selectedResourceGroup = value;
                    OnPropertyChanged( "SelectedResourceGroup" );
                }
            }
        }

        public SchdResource SelectedResource
        {
            get { return _selectedResource; }
            set
            {
                if ( _selectedResource != value && value != null )
                {
                    _selectedResource = value;
                    OnPropertyChanged( "SelectedResource" );
                }
            }
        }

        private string _paneTitle;

        public string PaneTitle
        {
            get { return _paneTitle; }
            set
            {
                _paneTitle = value;
                OnPropertyChanged( "PaneTitle" );
            }
        }

        [Dependency]
        public Factory<SchdAppointment> SchdAppointmentFactory { get; set; }

        public DelegateCommand<object> TaskTabSelectedCommand { get; private set; }

        public DelegateCommand<string> ResetUnselectedTabCommand { get; private set; }

        public void ClearViewFromList ()
        {
            Controller.RemoveViewFromList (View);
        }

        #endregion

        #region IDisposable members
    
        protected void Dispose(bool itIsSafeToAlsoFreeManagedObjects)
        {
            try
            {
                // Free unmanaged resources
                
                //Free managed resources too, but only if I'm being called from Dispose
                //(If I'm being called from Finalize then the objects might not exist
                //anymore
                if (itIsSafeToAlsoFreeManagedObjects)  
                {
                        SchdAppointmentFactory = null;
                        ResetUnselectedTabCommand = null;
                        TaskTabSelectedCommand = null;
                        _selectedResource = null;
                        Controller = null;
                        OnPropertyChanged("SelectedResource");
                        _selectedResourceGroup = null;
                        OnPropertyChanged("SelectedResourceGroup");

                        if (View != null)
                            View = null;
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "Dispose", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }
    
        public void Dispose()
        {
            Dispose(true); //true: safe to free managed resources
            GC.SuppressFinalize(this); //Hey, GC: don't bother calling finalize later
        }

        ~MultiSchedulerPresentationModel()
        {
            //we're being finalized (i.e. destroyed), call Dispose in case the user forgot to
            Dispose(false);
        }
    
        #endregion IDisposable members
    }
}
