using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.Task.Group;
using ClinSchd.Modules.Task.Scheduler;
using Telerik.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls.ScheduleView;

namespace ClinSchd.Modules.Task.ChildModules.Scheduler.Scheduler
{
    /// <summary>
    /// Interaction logic for SchedulerView.xaml
    /// </summary>
    public partial class SchedulerView : ISchedulerView, IDisposable
    {
        #region private fields

        private bool _bDialogResult;
        private int _schedulerDisableEventsCounter;

        #endregion private fields

        #region constructor

        public SchedulerView(ISchedulerPresentationModel viewModel)
        {
            SchedulerViewModel = viewModel;
            SchedulerViewModel.View = this;
            InitializeComponent();
            ((VSESlotSelectionBehavior)Scheduler.SlotSelectionBehavior).SlotsCollection = viewModel.SlotsCollection;
            DragDropManager.SchedulerViewModel = viewModel;
            CommandManager.AddPreviewExecutedHandler(scheduler, ExecutedEventHandler);
        }

        #endregion constructo

        #region ITaskView members

        public ITaskPresentationModel Model
        {
            get { return DataContext as SchedulerPresentationModel; }
            set { DataContext = value; }
        }

        #endregion ITaskView members

        #region ISchedulerView members

        public void AlertUser(string message, string caption)
        {
            Keys.AlertUser(this, message, caption);
        }

        public bool ConfirmUser(string message, string caption)
        {
            _bDialogResult = Keys.ConfirmUser(this, message, caption);
            return _bDialogResult;
        }

        public void Close()
        {
            Scheduler_EnableEvents(false);
            SchedulerViewModel.RespondToEvents(false);
            SchedulerViewModel.IsSelectedTab = false;
        }

        public void Collapse()
        {
            IsHidden = true;
        }

        public void Expand()
        {
            IsHidden = false;
        }

        public void CollapseTimeSlotViewerExpander(bool expand)
        {
            TimeSlotViewerExpander.IsExpanded = expand;
        }

        public void HotKeyHandler(Key k)
        {
            if (Keyboard.Modifiers == ModifierKeys.Control)
            {

            }
        }

        public bool IsDockedInRegion { get; set; }

        public RadScheduleView Scheduler
        {
            get { return scheduler; }
            set { scheduler = value; }
        }

        public void Scheduler_EnableEvents(bool enable)
        {
            if (enable)
            {
                if (_schedulerDisableEventsCounter > 0)
                    _schedulerDisableEventsCounter--;
            }
            else
            {
                _schedulerDisableEventsCounter++;
            }
        }

        public ISchedulerPresentationModel SchedulerViewModel
        {
            get { return DataContext as SchedulerPresentationModel; }
            set { DataContext = value; }
        }

        public ViewDefinitionBase ViewDefinitionByType(Type type)
        {
            return scheduler.ViewDefinitions.FirstOrDefault(vd => vd.GetType() == type);
        } 

        #endregion ISchedulerView members

        #region Command Handlers

        private void ExecutedEventHandler(object a, ExecutedRoutedEventArgs e)
        {

            if (e.Command == RadScheduleViewCommands.DeleteAppointment)
            {
                //var app = ((Occurrence)e.Parameter).Appointment;
                if (SchedulerViewModel.ProhibitedClinic && !SchedulerViewModel.PrivilegedUser)
                {
                    e.Handled = true;
                }
            }

        }

        #endregion Command Handlers

        #region Scheduler Event Handlers

        private void scheduler_AppointmentCreating(object sender, AppointmentCreatingEventArgs e)
        {
            e.Cancel = true;
        }

        private void scheduler_AppointmentEditing(object sender, AppointmentEditingEventArgs e)
        {
            e.Cancel = true;
        }

        private void scheduler_AppointmentDeleting(object sender, AppointmentDeletingEventArgs e)
        {
            e.Cancel = true;
        }

        private void scheduler_ShowDialog(object sender, ShowDialogEventArgs e)
        {
            e.Cancel = true;
        }

        private void SchedulerContextMenu_OnOpened(object sender, RoutedEventArgs e)
        {
            SchedulerViewModel.IsContextMenuOpen = true;
        }

        private void SchedulerContextMenu_OnClosed(object sender, RoutedEventArgs e)
        {
            SchedulerViewModel.IsContextMenuOpen = false;
        }

        private void TimeSlotViewer_ContextMenu_OnOpening(object sender, RadRoutedEventArgs e)
        {
            SchedulerViewModel.IsContextMenuOpen = true;
            GridViewRow row = this.TimeSlotViewer_ContextMenu.GetClickedElement<GridViewRow>();
            this.TimeSlotViewer.SelectedItem = row?.DataContext;
        }

        private void HorizontalTimelineMajorItemTemplate_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;
        }

        #endregion Scheduler Event Handlers

        #region public members

        public void ScrollSchedulerToOffset()
        {
            try
            {
                Scheduler.ScrollTimeRuler(TimeSpan.FromHours(SchedulerViewModel.FirstVisibleHour), true);
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "ScrollSchedulerToOffset", ex);
            }
        }

        #endregion public members

        #region private members

        #endregion private members

        #region IDisposable members

        protected void Dispose(bool itIsSafeToAlsoFreeManagedObjects)
        {
            try
            {
                // Free unmanaged resources

                //Free managed resources too, but only if I'm being called from Dispose
                //(If I'm being called from Finalize then the objects might not exist
                //anymore
                if (itIsSafeToAlsoFreeManagedObjects)
                {
                    Scheduler = null;
                    Content = null;
                    Header = null;
                    ContextMenu = null;
                    DataContext = null;

                    if (IsDockedInRegion)
                        RemoveFromParent();

                    if (Model != null)
                    {
                        Model.ClearViewFromList();
                        Model = null;
                    }
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "Dispose", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
        }
    
        public void Dispose()
        {
            Dispose(true); //true: safe to free managed resources
            GC.SuppressFinalize(this); //Hey, GC: don't bother calling finalize later
        }

        ~SchedulerView()
        {
            //we're being finalized (i.e. destroyed), call Dispose in case the user forgot to
            Dispose(false);
        }

        #endregion IDisposable members
    }
}