﻿using System;
using System.Globalization;

using Telerik.Windows.Controls;
using Telerik.Windows.Controls.ScheduleView;

namespace ClinSchd.Modules.Task.Scheduler
{
    internal class VSEWeekViewDefinition : WeekViewDefinition
    {
        private int _timeScale;
        private DateTime? _currentVisibleStartDate = null;
        private DateTime? _currentVisibleEndDate = null;

        public bool IsAvailabilityScheduler { get; set; } = false;
        public int TimeScale
        {
            get { return _timeScale; }
            set
            {
                if (_timeScale != value)
                {
                    _timeScale = value;
                    int timeScaleIndex = 60 / _timeScale;
                    MinorTickLength = new FixedTickProvider(new DateTimeInterval(timeScaleIndex < 3 ? 15 : _timeScale, 0, 0, 0, 0));
                    MinTimeRulerExtent = 17 * 24 * (timeScaleIndex < 3 ? 4 : timeScaleIndex);
                    OnPropertyChanged("MinorTickLength");
                    OnPropertyChanged("MinTimeRulerExtent");
                }
            }
        }

        public VSEWeekViewDefinition()
        {
            LargeChangeInterval = new DateTimeInterval(7, 0);
            GroupHeaderDateStringFormat = "{0:dd dddd}";
            TimerulerMinorTickStringFormat = "{0: :mm}";
            MajorTickLength = new FixedTickProvider(new DateTimeInterval(0, 1, 0, 0, 0));
            base.MaxTimeRulerExtent = 5000;
            ShowAllDayArea = true;
            TimeScale = 30;
        }

        #region protected override methods

        protected override DateTime GetVisibleRangeStart(DateTime currentDate, CultureInfo culture, DayOfWeek? firstDayOfWeek)
        {
            if (IsAvailabilityScheduler)
                return base.GetVisibleRangeStart(currentDate, culture, firstDayOfWeek);

            DateTime today = DateTime.Today;
            DateTime returnDate;

            // if we have not been here before, or current date is outside the current visible range, recalculate start date
            if ((!_currentVisibleStartDate.HasValue && !_currentVisibleEndDate.HasValue) || ((currentDate.CompareTo(_currentVisibleStartDate.Value) < 0) || (currentDate.CompareTo(_currentVisibleEndDate.Value) > 0)))
            {
                if (currentDate.CompareTo(today) <= 0)
                    returnDate = currentDate;
                else
                {
                    TimeSpan delta = currentDate - today;
                    double daysToSubtract = Math.Min(3, delta.TotalDays);
                    returnDate = currentDate.AddDays(-1 * daysToSubtract);
                }

                _currentVisibleStartDate = returnDate;
                _currentVisibleEndDate = returnDate.AddDays(6);
            }
            else
                returnDate = _currentVisibleStartDate.Value;

            return returnDate;
        }

        #endregion protected override methods
    }
}
