using System;
using System.ComponentModel;
using System.Windows.Media;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Modules.Task.Controllers;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using Microsoft.Practices.Composite.Presentation.Events;

namespace ClinSchd.Modules.Task.Group
{
    public class GroupPresentationModel : IGroupPresentationModel, INotifyPropertyChanged
    {
        private readonly ITaskService _taskService;
        public event PropertyChangedEventHandler PropertyChanged;
        private readonly IEventAggregator _eventAggregator;

        public GroupPresentationModel(IGroupView view, ITaskService taskService, IEventAggregator eventAggregator)
        {
            System.Windows.Threading.Dispatcher.CurrentDispatcher.BeginInvoke (new Action (() => { HideLegend (); }), System.Windows.Threading.DispatcherPriority.ApplicationIdle);
            View = view;
            View.Model = this;
            _taskService = taskService;
            _eventAggregator = eventAggregator;
            _eventAggregator.GetEvent<FocusSchedulerEvent> ().Subscribe (FocusScheduler, ThreadOption.UIThread, false);
        }

        public IGroupView View { get; }

        public ITaskController Controller { get; set; }

        private void OnPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if ( handler != null )
                handler( this, new PropertyChangedEventArgs( propertyName ) );
        }

        private void FocusScheduler (object T)
        {
            View.FocusSchedulerGroup ();
        }

        public void HideLegend ()
        {
            LegendVisibility = System.Windows.Visibility.Collapsed;
        }

        public void ShowLegend ()
        {
            LegendVisibility = System.Windows.Visibility.Visible;
        }

        private System.Windows.Visibility _legendVisibility;

        public System.Windows.Visibility LegendVisibility
        {
            get { return _legendVisibility; }
            set
            {
                _legendVisibility = value;
                OnPropertyChanged( "LegendVisibility" );
            }
        }

        public Brush NewAppointmentColor { get { return Keys.ColorCodes.NewAppointmentColor; } }
        public Brush WalkInColor { get { return Keys.ColorCodes.WalkInAppointmentColor; } }
        public Brush NoShowColor { get { return Keys.ColorCodes.NoShowColor; } }
        public Brush CheckInColor { get { return Keys.ColorCodes.CheckedInColor; } }
        public Brush CheckOutColor { get { return Keys.ColorCodes.CheckedOutColor; } }
        public Brush AvailableColor { get { return Keys.ColorCodes.AvailableSlotColor; } }
        public Brush UnavailableColor { get { return Keys.ColorCodes.NonWorkingSlotColor; } }
        public Brush OverbookColor { get { return Keys.ColorCodes.UnavailableSlotColor; } }
    }
}
