using System.Windows;
using Microsoft.Practices.Composite.Events;
using ClinSchd.Infrastructure;
using ClinSchd.Modules.UserLogin.Services;
using ClinSchd.Modules.UserLogin.UserLogin;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Presentation.Events;

namespace ClinSchd.Modules.UserLogin.Controllers
{
    public class UserLoginController : IUserLoginController
    {
        private readonly IUnityContainer _container;
        private readonly IUserLoginService _userLoginService;
        private readonly IEventAggregator _eventAggregator;

        public UserLoginController (IUnityContainer container,
            IUserLoginService userLoginService,
            IEventAggregator eventAggregator)
        {
            _container = container;
            _userLoginService = userLoginService;
            _eventAggregator = eventAggregator;
        }

        public void Run ()
        {
            _eventAggregator.GetEvent<LaunchUserLoginDialogEvent> ().Subscribe (LaunchUserLoginDialog, ThreadOption.UIThread, true);

//            _eventAggregator.GetEvent<LaunchUserLoginDialogEvent>().Publish("User Login");
        }

        public void LaunchUserLoginDialog (string title)
        {
            IUserLoginPresentationModel model = _container.Resolve<IUserLoginPresentationModel> ();
            
            if (model.ValidationMessage.IsValid)
            {
                model.ValidationMessage.IsValid = false;

                _userLoginService.ShowDialog (model.View, model, () => model.OnClose ());

                if (!model.ValidationMessage.IsValid)
                    Application.Current.Shutdown();
                else
                    _eventAggregator.GetEvent<FocusPatientSearchEvent>().Publish(null);
            }
            else
            {
                model.View.AlertUser (model.ValidationMessage.Message, model.ValidationMessage.Title);
                model.ValidationMessage.IsValid = true;
                model.ValidationMessage.Title = string.Empty;
                model.ValidationMessage.Message = string.Empty;
            }
        }
    }
}
