using System;
using System.ComponentModel;
using System.Collections.Generic;


using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Commands;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Modules.UserLogin.Services;
using ClinSchd.Infrastructure;

namespace ClinSchd.Modules.UserLogin.UserLogin
{
    public class UserLoginPresentationModel : IUserLoginPresentationModel, INotifyPropertyChanged
    {
        private readonly IUserLoginService _userLoginService;
        private readonly IDataAccessService _dataAccessService;
        private readonly IEventAggregator _eventAggregator;
        private ValidationMessage _validationMessage;
        //private IList<Divisions> _divisions;

        public UserLoginPresentationModel(
            IUserLoginView view,
            IUserLoginService userLoginService,
            IDataAccessService dataAccessService,
            IEventAggregator eventAggregator)
        {
            View = view;
            View.Model = this;
            _eventAggregator = eventAggregator;
            _userLoginService = userLoginService;
            _dataAccessService = dataAccessService;
            ValidationMessage = new ValidationMessage();
            InitialErrorMessage();

            UserLoginCommand = new DelegateCommand<string>(ExecuteUserLoginCommand, CanExecuteUserLoginCommand);
        }

        public void InitialErrorMessage()
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
        }


        public void ExecuteUserLoginCommand (string command)
        {
            _validationMessage.IsValid = true;
            _validationMessage.Title = string.Empty;
            _validationMessage.Message = string.Empty;
            string szErrorMsg;

            var returnValue = _dataAccessService.LoginUser(AccessCode, VerifyCode, out szErrorMsg);

            if (returnValue) 
            {
                string errorMessage;
                if (!_dataAccessService.LoadUserInfo(out errorMessage))
                {
                    _validationMessage.IsValid = false;
                    _validationMessage.Title = "User Access Rights";
                    _validationMessage.Message = errorMessage;
                    return;
                }

                _dataAccessService.LoadGlobalRecordsets ();
                _dataAccessService.StartRefreshTimer();

                var cmdArgs = new List<string>(Environment.GetCommandLineArgs());

                if (cmdArgs.Contains("/CCOW"))
                    Keys.CCOW = new CCOWBase("VistAScheduler");
            }
            else 
            {
                _validationMessage.IsValid = false;
                _validationMessage.Title = "User Login";
                _validationMessage.Message = szErrorMsg;
            }
        }

        public bool CanExecuteUserLoginCommand(string command)
        {
            return true;
        }

        public void OnClose()
        {
            View.Close();
        }

        #region IUserLoginPresentationModel Members

        public DelegateCommand<string> UserLoginCommand { get; private set; }

        public IUserLoginView View { get; private set; }

        public string AccessCode { get; set; }
        public string VerifyCode { get; set; }
        
        public ValidationMessage ValidationMessage
        {
            get
            {
                return _validationMessage;
            }
            set
            {
                _validationMessage = value;
            }
        }
        #endregion

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler Handler = PropertyChanged;
            if (Handler != null) Handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
