using System.Windows;
using System.Windows.Input;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;
using Microsoft.Practices.Unity;

using ClinSchd.Infrastructure;
using ClinSchd.Modules.VAR.Events;
using ClinSchd.Modules.VAR.Models;
using ClinSchd.Modules.VAR.ProcessRequest;
using ClinSchd.Modules.VAR.Services;
using ClinSchd.Modules.VAR.VAR;

namespace ClinSchd.Modules.VAR.Controllers
{
	public class VARController : IVARController
    {
        private IUnityContainer container = null;
        private IVARService varService = null;
        private IEventAggregator eventAggregator = null;

        public VARController (IUnityContainer container, IVARService varService, IEventAggregator eventAggregator)
		{
			this.container = container;
			this.varService = varService;
			this.eventAggregator = eventAggregator;
        }

		public void Run ()
		{
            this.eventAggregator.GetEvent<VAR_LaunchRequestDialog>().Subscribe(LaunchRequestDialog, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<VAR_LaunchProcessDialog>().Subscribe(LaunchProcessDialog, ThreadOption.UIThread, true);
            this.eventAggregator.GetEvent<VAR_LaunchRequestDetailsDialog>().Subscribe(LaunchDetailsRequestDialog, ThreadOption.UIThread, true);
        }

        public void LaunchRequestDialog (string command)
        {
            if (command == "LAUNCH")
            {
                IVARPresentationModel Model = container.Resolve<IVARPresentationModel>();
                Model.View.Owner = Keys.MainWindow;

                // Positioning Calculations
                // We want this to open so the top\right corner is just inside the main window top\right corner
                Window child = (Window)Model.View;
                Rect screenRect = Infrastructure.Keys.CurrentScreenDimensions();
                Rect mainWindowRect = (
                    (Keys.MainWindow.WindowState == WindowState.Maximized) ? screenRect :
                    new Rect(Keys.MainWindow.Left, Keys.MainWindow.Top, Keys.MainWindow.Width, Keys.MainWindow.Height)
                );
                double childTop = screenRect.Top + mainWindowRect.Top + 25;
                double childLeft = mainWindowRect.Right - (child.Width + 10);
                Rect childWindowRect = new Rect(childLeft, childTop, child.Width, child.Height);

                if (!screenRect.Contains(childWindowRect))
                {
                    if (childWindowRect.Top <= screenRect.Top)
                        childWindowRect.Offset(0.0, childWindowRect.Top - (screenRect.Top + 25));

                    if (childWindowRect.Right >= screenRect.Right)
                        childWindowRect.Offset((screenRect.Right - 10) - childWindowRect.Right, 0.0);
                }

                child.Top = childWindowRect.Top;
                child.Left = childWindowRect.Left;

                if (Model.ValidationMessage.IsValid)
                {
                    this.varService.ShowDialog(
                    Model.View,
                    Model, () => Model.OnClose());
                }
                else
                {
                    Model.View.AlertUser(Model.ValidationMessage.Message, Model.ValidationMessage.Title);
                    Model.ValidationMessage.IsValid = true;
                    Model.ValidationMessage.Title = string.Empty;
                    Model.ValidationMessage.Message = string.Empty;
                }
            }
        }

        public void LaunchProcessDialog(IProcessRequestModel processRequest)
        {
            IProcessRequestPresentationModel Model = container.Resolve<IProcessRequestPresentationModel>();
            Model.ProcessRequestModel = processRequest;
            Window w = (Window)(Model.View);
            w.Owner = (Window)processRequest.Owner;

            if (Model.ValidationMessage.IsValid)
            {
                this.varService.ShowDialog(
                Model.View,
                Model, () => Model.OnClose());
            }
            else
            {
                Model.View.AlertUser(Model.ValidationMessage.Message, Model.ValidationMessage.Title);
                Model.ValidationMessage.IsValid = true;
                Model.ValidationMessage.Title = string.Empty;
                Model.ValidationMessage.Message = string.Empty;
            }
        }

        public void LaunchDetailsRequestDialog(IAppointmentRequestItemPresentationModel model)
        {
            IVARDetailView view = container.Resolve<IVARDetailView>();
            view.Owner = (Window)model.Parent;

            // Positioning Calculations
            // We want the details window to open at the parent top\right
            Window child = ((Window)view);
            Rect screenRect = Infrastructure.Keys.CurrentScreenDimensions();
            Rect parentWindowRect = (
                (view.Owner.WindowState == WindowState.Maximized) ? screenRect :
                new Rect(view.Owner.Left, view.Owner.Top, view.Owner.Width, view.Owner.Height)
            );

            double childTop = parentWindowRect.Top;
            double childLeft = screenRect.Left + (parentWindowRect.Right - child.Width);
            Rect childWindowRect = new Rect(childLeft, childTop, child.Width, child.Height);

            if (!screenRect.Contains(childWindowRect))
            {
                if (childWindowRect.Top <= screenRect.Top)
                    childWindowRect.Offset(0.0, childWindowRect.Top - (screenRect.Top + 25));

                if (childWindowRect.Right >= screenRect.Right)
                    childWindowRect.Offset((screenRect.Right - 10) - childWindowRect.Right, 0.0);
            }

            child.Top = childWindowRect.Top;
            child.Left = childWindowRect.Left;

            if (model.ValidationMessage.IsValid)
            {
                this.varService.ShowDialog(
                view,
                model, () => model.OnClose());
            }
            else
            {
                model.View.AlertUser(model.ValidationMessage.Message, model.ValidationMessage.Title);
                model.ValidationMessage.IsValid = true;
                model.ValidationMessage.Title = string.Empty;
                model.ValidationMessage.Message = string.Empty;
            }
        }
    }
}
