﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using ClinSchd.Infrastructure.Models;

namespace ClinSchd.Modules.VAR.Models
{
    public class VARRequestModel
    {
        #region constants

        private const string SORTABLE_DATE_FORMAT = "yyyy/MM/dd";

        #endregion constants

        #region constructor

        public VARRequestModel(string source)
        {
            FromString(source);
        }

        #endregion constructor

        #region public members

        public string ClinicName { get; private set; }

        #region DateEntered

        private DateTime? _dateEntered;
        public DateTime? DateEntered
        {
            get { return _dateEntered; }
            set { _dateEntered = value; }
        }

        public string DateEnteredShortString
        {
            get { return (DateEntered.HasValue ? DateEntered.Value.ToShortDateString() : ""); }
            set { DateEntered = StringToDateTime(value); }
        }

        public string DateEnteredShortStringSortable
        {
            get
            {
                return _dateEntered != null ? _dateEntered.Value.Date.ToString(SORTABLE_DATE_FORMAT) : "";
            }
        }

        #endregion DateEntered

        #region DesiredApptDate

        private DateTime? _desiredApptDate;
        public DateTime? DesiredApptDate
        {
            get
            {
                if (_desiredApptDate != null)
                    return _desiredApptDate.Value.Date;
                return null;
            }
            set
            {
                _desiredApptDate = value;
            }
        }

        public string DesiredApptDateShortString
        {
            get { return _desiredApptDate != null ? _desiredApptDate.Value.Date.ToShortDateString() : ""; }
            set { DesiredApptDate = StringToDateTime(value); }
        }

        public string DesiredApptDateShortStringSortable
        {
            get
            {
                return _desiredApptDate != null ? _desiredApptDate.Value.Date.ToString(SORTABLE_DATE_FORMAT) : "";
            }
        }

        #endregion DesiredApptDate

        public Patient Patient { get; private set; }

        public string PatientAddress
        {
            get
            {
                StringBuilder sb = new StringBuilder();

                if (Patient != null)
                {
                    if (!String.IsNullOrEmpty(Patient.Address))
                        sb.AppendLine(Patient.Address);

                    if (!String.IsNullOrEmpty(Patient.Address2))
                        sb.AppendLine(Patient.Address2);

                    if (!String.IsNullOrEmpty(Patient.Address3))
                        sb.AppendLine(Patient.Address3);

                    string line = (String.IsNullOrEmpty(Patient.City) ? String.Empty : Patient.City);
                    line += (String.IsNullOrEmpty(line) ? Patient.State : (String.IsNullOrEmpty(Patient.State) ? String.Empty : ", " + Patient.State));
                    string country = (Patient.Country == null ? String.Empty : Patient.Country.Abbreviation);
                    line += (String.IsNullOrEmpty(line) ? country : (String.IsNullOrEmpty(country) ? String.Empty : " " + country));
                    line += (String.IsNullOrEmpty(line) ? Patient.ZipCode : (String.IsNullOrEmpty(Patient.ZipCode) ? String.Empty : ", " + Patient.ZipCode));

                    if (!String.IsNullOrEmpty(line.Trim()))
                        sb.AppendLine(line);
                }

                return sb.ToString();
            }
        }

        public string PatientDOBShortStringSortable
        {
            get
            {
                return Patient.DOBDate.HasValue ? Patient.DOBDate.Value.Date.ToString(SORTABLE_DATE_FORMAT) : "";
            }
        }

        public string RequestIEN { get; private set; }

        public int? WaitTime
        {
            get
            {
                if (DateEntered == null)
                    return null;
                var diff = DateTime.Today - DateEntered;
                var waitTime = diff.Value.Days;
                return waitTime;
            }
        }

        #endregion public members

        #region data conversion methods

        public void FromString(string source)
        {
            string[] fields = source.Split('^');
            RequestIEN = fields[0].ToString();
            this.Patient = new Patient(null)
            {
                Name = fields[1].ToString(),
                SSN = fields[2].ToString(),
                DOB = fields[3].ToString(),
                IEN = fields[4].ToString(),
                Gender = fields[5].ToString(),
                Address = fields[6].ToString(),
                Address2 = fields[7].ToString(),
                Address3 = fields[8].ToString(),
                City = fields[9].ToString(),
                State = fields[10].ToString(),
                Country = new Country() { Name = fields[11].ToString() },
                ZipCode = fields[12].ToString(),
                PhoneResidence = fields[13].ToString()
            };

            DesiredApptDateShortString = fields[14].ToString();
            DateEnteredShortString = fields[15].ToString();
            ClinicName = fields[16].ToString();
        }

        public string ToString()
        {
            List<string> results = new List<string>();
            results.Add(RequestIEN);
            results.Add(Patient.Name);
            results.Add(Patient.SSN);
            results.Add(Patient.DOB);
            results.Add(Patient.Address);
            results.Add(Patient.Address2);
            results.Add(Patient.Address3);
            results.Add(Patient.City);
            results.Add(Patient.State);
            results.Add(Patient.Country.Name);
            results.Add(Patient.ZipCode);
            results.Add(Patient.PhoneResidence);
            results.Add(DesiredApptDateShortString);
            results.Add(DateEnteredShortString);
            results.Add(ClinicName);
            return String.Join("^", results.ToArray());
        }

        #endregion data conversion methods

        #region private methods

        private DateTime? StringToDateTime(string source)
        {
            DateTime? result = null;

            if (!String.IsNullOrEmpty(source))
                result = Convert.ToDateTime(source.Replace('@', ' '));

            return result;
        }

        #endregion private methods
    }
}
