using System;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media.Animation;
using ClinSchd.Infrastructure;
using Telerik.Windows.Controls;

using ClinSchd.Infrastructure.Models;

using System.Diagnostics;

namespace ClinSchd.Modules.VAR.ProcessRequest
{
    /// <summary>
	/// Interaction logic for UserLoginView.xaml
    /// </summary>
    public partial class ProcessRequestView : Window, IProcessRequestView
    {
        public ProcessRequestView()
        {
            InitializeComponent();

            _resourceSearchTimer = new Timer(Keys.SearchTimerInterval);
            _resourceSearchTimer.Elapsed += new ElapsedEventHandler(resourceSearchTimer_Elapsed);
        }

		public IProcessRequestPresentationModel Model
        {
            get
            {
                return this.DataContext as IProcessRequestPresentationModel;
            }

            set
            {
                this.DataContext = value;
            }
        }

		private bool bDialogResult = false;
		public bool ConfirmUser (string message, string caption)
		{
            bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return bDialogResult;
        }

		public void AlertUser (string message, string caption)
		{
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.S:
                        Keyboard.Focus(SubmitButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }

        #region ResourceComboBox search members

        private Timer _resourceSearchTimer;
        public string _resourceSearchString { get; set; }

        void resourceSearchTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _resourceSearchTimer.Stop();
            Dispatcher.BeginInvoke((Action)(() => Model.ResourceSearchStringChanged(_resourceSearchString)));
        }

        private void ResourceComboBox_KeyUp(object sender, System.Windows.Input.KeyEventArgs e)
        {
            _resourceSearchTimer.Stop();

            if (Keys.Methods.ComboBoxStartDataLookupKey(e.Key))
                _resourceSearchTimer.Start();
        }

        private void ResourceComboBox_OnGotFocus(object sender, RoutedEventArgs e)
        {
            ResourceComboBox.Text = String.Empty;
        }

        private void ResourceComboBox_OnLostFocus(object sender, RoutedEventArgs e)
        {
            //SetResourceComboBoxOpen(false);
        }

        private void ResourceComboBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            ComboBox cbx = (ComboBox)sender;
            _resourceSearchString = cbx.Text;
        }

        private void ResourceComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
        }

        public void SetResourceComboBoxOpen(bool openDropDown)
        {
            ResourceComboBox.IsDropDownOpen = openDropDown;
            //  The following code insures the CombBox text is not selected and the cursor is positioned for additional typing (ComboBox ONLY)
            TextBox tbx = (System.Windows.Controls.TextBox)ResourceComboBox.Template.FindName("PART_EditableTextBox", ResourceComboBox);
            tbx.SelectionStart = ((tbx.Text.Length == 0) ? 0 : tbx.Text.Length);
            tbx.SelectionLength = 0;
        }

        private void ResourceComboBox_ScrollChanged(object sender, ScrollChangedEventArgs e)
        {
            var scrollViewer = e.OriginalSource as ScrollViewer;

            if ((scrollViewer != null) && (scrollViewer.VerticalOffset > 0) && (scrollViewer.VerticalOffset >= (scrollViewer.ScrollableHeight - Keys.ClinicResourceSearchGetNextWithin)))
                Dispatcher.BeginInvoke((Action)(() => Model.ResourceScrollChanged(scrollViewer)));
        }

        #endregion ResourceComboBox search members

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));
        }
    }
}
