using System;
using System.Collections.Generic;
using System.Globalization;
using System.Collections;
using System.Linq;
using System.Xml.Linq;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Unity;

using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Modules.VAR.Events;
using ClinSchd.Modules.VAR.Models;
using ClinSchd.Modules.VAR.ProcessRequest;
using ClinSchd.Modules.VAR.VAR;
using ClinSchd.Modules.VAR.VARButton;

namespace ClinSchd.Modules.VAR.Services
{
	public class VARService : IVARService
    {
        private readonly IVARWebServiceAPIService _webServiceAPI;
        private IEventAggregator _eventAggregator;
        private IUnityContainer _container;

        public VARService(IEventAggregator eventAggregator, IUnityContainer container, IVARWebServiceAPIService WebServiceAPI)
        {
            _eventAggregator = eventAggregator;
            _container = container;
            this._eventAggregator.GetEvent<VAR_IsVARUserEvent>().Subscribe(VARSetUserSecurity);
            _webServiceAPI = WebServiceAPI;
        }

        private void VARSetUserSecurity(bool isVARRequestUser)
        {
            if (isVARRequestUser)
            {
                IVARButtonPresentationModel model = _container.Resolve<IVARButtonPresentationModel>();
                model.IsVARRequestUser = isVARRequestUser;
                var view = model.View;
                view.DataContext = model;
                _eventAggregator.GetEvent<VAR_ButtonViewEvent>().Publish((UserControl)view);
                var vse_Information = _container.Resolve<IVSE_Information>();
                vse_Information.VSE_USER = "vse_user";
                vse_Information.VSE_USER_ID = "AI";
                vse_Information.VistAUserIEN = Keys.LoggedInUserIen;
            }
        }

		#region IVAR Members

		public void ShowDialog<VARPresentationModel>(IVARView view, VARPresentationModel viewModel, Action onDialogClose)
		{
            Window w = (Window)view;
            w.Owner = Infrastructure.Keys.MainWindow;
			view.DataContext = viewModel;
           
			if (onDialogClose != null)
			{
				view.Closed += (sender, e) => onDialogClose();
			}
			view.Show();
		}

        public void ShowDialog<ProcessRequestPresentationModel>(IProcessRequestView view, ProcessRequestPresentationModel viewModel, Action onDialogClose)
        {
            Window w = (Window)view;
            view.DataContext = viewModel;

            if (onDialogClose != null)
            {
                view.Closed += (sender, e) => onDialogClose();
            }
            view.ShowDialog();
            _eventAggregator.GetEvent<VAR_Refresh>().Publish("REFRESH");
        }

        public void ShowDialog<AppointmentRequestItemPresentationModel>(IVARDetailView view, AppointmentRequestItemPresentationModel viewModel, Action onDialogClose)
        {
            //Window w = (Window)view;
            view.Model = (IAppointmentRequestItemPresentationModel)viewModel;

            if (onDialogClose != null)
            {
                view.Closed += (sender, e) => onDialogClose();
            }
            view.Show();
        }


        #endregion IVAR Members
    }
}
