﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using ClinSchd.Infrastructure;
using ClinSchd.Modules.VAR.Models;

using System.ServiceModel;
using System.Security.Cryptography.X509Certificates;
using VSE_VAR_Service;

namespace ClinSchd.Modules.VAR.Services
{
    internal class WebServiceCallingException : Exception { }

    internal class WebServiceReturnedException : Exception { }

    internal class VARWebServiceAPIService : IVARWebServiceAPIService
    {
        #region enums

        private enum webServiceMethods
        {
            GetAppointmentRequestCount,
            GetAppointmentRequestList,
            GetAppointmentRequest,
            GetDetailCodeList,
            BookAppointment,
            NotBookAppointment,
            UnlockAppointment
        }

        #endregion enums

        #region static members

        #endregion static members

        #region private fields

        private IVSE_Information _vseInformation = null;
        private IList<DETAIL_CODE> _detailCodeList = new List<DETAIL_CODE>();
        private Exception _serviceException = null;


        #endregion private fields

        #region constructor

        public VARWebServiceAPIService(IVSE_Information vseInformation)
        {
            _vseInformation = vseInformation;
        }

        #endregion constructor

        #region public IVARWebServiceAPIService members

        public bool BookAppointment(APPOINTMENT_REQUEST request)
        {            
            const string methodName = "BookAppointment";
            LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.BookAppointment" };
            logItem.MethodParams.Add(new paramItem() { Name = "request", Value = request, IsObject = true });

            BOOKED_APPOINTMENT apptInfo = new BOOKED_APPOINTMENT()
            {
                APPOINTMENT_REQUEST_ID = request.APPOINTMENT_REQUEST_ID,
                BOOKED_APPT_DATETIME = request.BOOKED_APPT_DATETIME.Value,
                APPOINTMENT_DATE = request.APPOINTMENT_DATE,
                APPOINTMENT_TIME = request.APPOINTMENT_TIME,
                APPOINTMENT_TYPE = request.APPOINTMENT_TYPE,
                MESSAGE = request.NewMessage,
                PROVIDER_ID = request.PROVIDER_ID,
                PROVIDER_NAME = request.PROVIDER_NAME,
                DetailCode = request.DetailCode,
                STATUS = request.STATUS,
                USER_ID = _vseInformation.VSE_USER_ID
            };

            logItem.APIParams.Add(new paramItem() { Name = "BOOKED_APPOINTMENT", Value = apptInfo, IsObject = true });
            STANDARD_RETURN returnResults = (STANDARD_RETURN)ExecuteWebServiceCall(webServiceMethods.BookAppointment, apptInfo);

            if (_serviceException == null)
            {
                if (returnResults != null)
                    logItem.Results = new paramItem() {Name = methodName + " Return", Value = returnResults, IsObject = true};

                CreateLogEntry(logItem);

                if (returnResults != null)
                {
                    ProcessReturnedErrorMessages(returnResults.ErrorMessages, methodName);
                    return returnResults.Success;
                }
                else
                    return false;
            }
            else
            {
                string[] errMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "BookAppointment", _serviceException).Split(new string[] {Environment.NewLine}, StringSplitOptions.None);
                STANDARD_RETURN errReturn = new STANDARD_RETURN(errMessages.ToList());
                errReturn.Success = false;
                _serviceException = null;
                return false;
            }
        }

        public int GetAppointmentRequestCount(out string errorMessages)
        {
            const string methodName = "GetAppointmentRequestCount";
            int result = 0;
            errorMessages = String.Empty;
            LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.GetAppointmentRequestCount" };
            logItem.APIParams.Add(new paramItem() { Name = "FACILITY_CODE", Value = _vseInformation.FACILITY_CODE });
            APPOINTMENT_REQUEST_COUNT_RETURN returnResults = (APPOINTMENT_REQUEST_COUNT_RETURN)ExecuteWebServiceCall(webServiceMethods.GetAppointmentRequestCount, null);

            if (_serviceException == null)
            {
                if (returnResults != null)
                    logItem.Results = new paramItem() {Name = methodName + " Return", Value = returnResults, IsObject = true};

                CreateLogEntry(logItem);

                if (returnResults != null)
                {
                    if (returnResults.ErrorMessages.Count() > 0)
                        errorMessages = String.Join("\n", returnResults.ErrorMessages);

                    if (returnResults.Success)
                        result = returnResults.Count;
                }
            }
            else
            {
                errorMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "GetAppointmentRequestCount", _serviceException);
                _serviceException = null;
            }

            return result;
        }

        public IList<APPOINTMENT_REQUEST_LIST_ITEM> GetAppointmentRequestList()
        {
            IList<APPOINTMENT_REQUEST_LIST_ITEM> result = new List<APPOINTMENT_REQUEST_LIST_ITEM>();
            const string methodName = "GetAppointmentRequestList";
            LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.GetAppointmentRequestList" };
            logItem.APIParams.Add(new paramItem() { Name = "FACILITY_CODE", Value = _vseInformation.FACILITY_CODE });
            APPOINTMENT_REQUEST_LIST_RETURN returnResults = (APPOINTMENT_REQUEST_LIST_RETURN)ExecuteWebServiceCall(webServiceMethods.GetAppointmentRequestList, null);

            if (_serviceException == null)
            {
                if (returnResults != null)
                    logItem.Results = new paramItem() { Name = methodName + " Return", Value = returnResults, IsObject = true };

                CreateLogEntry(logItem);

                if (returnResults != null)
                {
                    ProcessReturnedErrorMessages(returnResults.ErrorMessages, methodName);

                    if (returnResults.Success)
                        result = returnResults.Appointment_Request_Item_List.ToList();
                }
            }
            else
            {
                string[] errMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "GetAppointmentRequestList", _serviceException).Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                ProcessReturnedErrorMessages(errMessages, methodName);
                _serviceException = null;
            }

            return result;
        }

        public APPOINTMENT_REQUEST GetAppointmentRequest(string appointment_request_id, out bool isLocked)
        {
            isLocked = false;
            const string methodName = "GetAppointmentRequest";
            LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.GetAppointmentRequest" };
            logItem.MethodParams.Add(new paramItem() { Name = "appointment_request_id", Value = appointment_request_id });

            APPT_REQ_INPROCESS requestInfo = new APPT_REQ_INPROCESS()
            {
                APPT_REQ_ID = appointment_request_id,
                USER_ID = _vseInformation.VSE_USER_ID + "-" + _vseInformation.VistAUserIEN,
                FIRST_NAME = _vseInformation.VSE_USER,
                LAST_NAME = _vseInformation.VSE_USER
            };

            logItem.APIParams.Add(new paramItem() { Name = "APPT_REQ_INPROCESS", Value = requestInfo, IsObject = true });
            APPOINTMENT_REQUEST_RETURN returnResults = (APPOINTMENT_REQUEST_RETURN)ExecuteWebServiceCall(webServiceMethods.GetAppointmentRequest, requestInfo);

            if (_serviceException == null)
            {
                // Do logging
                if (returnResults != null)
                    logItem.Results = new paramItem() {Name = methodName + " Return", Value = returnResults, IsObject = true};

                CreateLogEntry(logItem);

                // Handle return results
                if (returnResults != null)
                {
                    // API call failed
                    if (!returnResults.Success)
                    {
                        // Was the failure because the selected record is locked by another user ???
                        if (returnResults.ErrorMessages.Count() == 1)
                            if (returnResults.ErrorMessages[0] == "LOCKED")
                            {
                                isLocked = true;
                                return null;
                            }

                        // Not a locked record, some other error, throw the error
                        ProcessReturnedErrorMessages(returnResults.ErrorMessages, methodName);
                    }

                    return returnResults.Appointment_Request;
                }
                else
                    return null;
            }
            else
            {
                string[] errMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "GetAppointmentRequest", _serviceException).Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                ProcessReturnedErrorMessages(errMessages, methodName);
                _serviceException = null;
                return null;
            }
        }

        public IList<DETAIL_CODE> GetDetailCodeList()
        {
            if (_detailCodeList.Count == 0)
            {
                const string methodName = "GetDetailCodeList";
                LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.GetDetailCodeList" };
                DETAIL_CODE_LIST_RETURN returnResults = (DETAIL_CODE_LIST_RETURN)ExecuteWebServiceCall(webServiceMethods.GetDetailCodeList, null);

                if (_serviceException == null)
                {
                    if (returnResults != null)
                        logItem.Results = new paramItem() {Name = methodName + " Return", Value = returnResults, IsObject = true};

                    CreateLogEntry(logItem);

                    if (returnResults != null)
                    {
                        ProcessReturnedErrorMessages(returnResults.ErrorMessages, methodName);

                        if (returnResults.Success)
                            _detailCodeList = returnResults.DetailCodeList;
                    }
                }
                else
                {
                    string[] errMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "GetDetailCodeList", _serviceException).Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                    ProcessReturnedErrorMessages(errMessages, methodName);
                    _serviceException = null;
                }
            }

            return _detailCodeList;
        }

        public bool NotBookAppointment(APPOINTMENT_REQUEST request)
        {
            const string methodName = "NotBookAppointment";
            LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.NotBookAppointment" };
            logItem.MethodParams.Add(new paramItem() { Name = "request", Value = request });

            NOT_BOOKED_APPOINTMENT apptInfo = new NOT_BOOKED_APPOINTMENT()
            {
                APPOINTMENT_REQUEST_ID = request.APPOINTMENT_REQUEST_ID,
                MESSAGE = request.NewMessage,
                USER_ID = _vseInformation.VSE_USER_ID,
                DetailCode = request.DetailCode,
                STATUS = request.STATUS
            };

            logItem.APIParams.Add(new paramItem() { Name = "NOT_BOOKED_APPOINTMENT", Value = apptInfo, IsObject = true });
            STANDARD_RETURN returnResults = (STANDARD_RETURN)ExecuteWebServiceCall(webServiceMethods.NotBookAppointment, apptInfo);

            if (_serviceException == null)
            {
                if (returnResults != null)
                    logItem.Results = new paramItem() {Name = methodName + " Return", Value = returnResults, IsObject = true};

                CreateLogEntry(logItem);

                if (returnResults != null)
                {
                    ProcessReturnedErrorMessages(returnResults.ErrorMessages, methodName);
                    return returnResults.Success;
                }
                else
                    return false;
            }
            else
            {
                string[] errMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "NotBookAppointment", _serviceException).Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                ProcessReturnedErrorMessages(errMessages, methodName);
                _serviceException = null;
                return false;
            }
        }

        public bool UnlockAppointmentRequest(string appointment_request_id)
        {
            const string methodName = "UnlockAppointmentRequest";
            string parameter = appointment_request_id + "^" + _vseInformation.VSE_USER_ID + "-" + _vseInformation.VistAUserIEN;
            LogClass logItem = new LogClass() { MethodName = methodName, APIName = "VSE_VAR_Service.UnlockAppointmentRequest" };
            logItem.MethodParams.Add(new paramItem() { Name = "appointment_request_id", Value = parameter });
            logItem.APIParams.Add(new paramItem() { Name = "appointment_request_id", Value = parameter });
            STANDARD_RETURN returnResults = (STANDARD_RETURN)ExecuteWebServiceCall(webServiceMethods.UnlockAppointment, parameter);

            if (_serviceException == null)
            {
                if (returnResults != null)
                    logItem.Results = new paramItem() {Name = methodName + " Return", Value = returnResults, IsObject = true};

                CreateLogEntry(logItem);

                if (returnResults != null)
                {
                    ProcessReturnedErrorMessages(returnResults.ErrorMessages, methodName);
                    return returnResults.Success;
                }
                else
                    return false;
            }
            else
            {
                string[] errMessages = TraceLog.FormatExceptionMessage(this.GetType().Name, "UnlockAppointmentRequest", _serviceException).Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                ProcessReturnedErrorMessages(errMessages, methodName);
                _serviceException = null;
                return false;
            }
        }

        #endregion public IVARWebServiceAPIService members

        #region private methods

        private object ExecuteWebServiceCall(webServiceMethods method, object parm)
        {
            object result = null;
            string methodName = method.ToString();
            _serviceException = null;

            using (ChannelFactory<IVSE_VAR_Service> serviceProxy = new ChannelFactory<IVSE_VAR_Service>("VSE_VAR_ServiceEndpoint"))
            {
                try
                {
                    serviceProxy.Credentials.ClientCertificate.Certificate = new X509Certificate2(Environment.CurrentDirectory + "\\var_service.vse.cc.med.va.gov.pfx", "vse_rocks");
                    serviceProxy.Open();
                    IVSE_VAR_Service service = serviceProxy.CreateChannel();

                    switch(method)
                    {
                        case webServiceMethods.GetAppointmentRequestCount:
                            result = service.GetAppointmentRequestCount(_vseInformation.FACILITY_CODE);
                            break;
                        case webServiceMethods.GetAppointmentRequestList:
                            result = service.GetAppointmentRequestList(_vseInformation.FACILITY_CODE);
                            break;
                        case webServiceMethods.GetAppointmentRequest:
                            result = service.GetAppointmentRequest((APPT_REQ_INPROCESS)parm);
                            break;
                        case webServiceMethods.GetDetailCodeList:
                            result = service.GetDetailCodeList();
                            break;
                        case webServiceMethods.BookAppointment:
                            result = service.BookAppointment((BOOKED_APPOINTMENT)parm);
                            break;
                        case webServiceMethods.NotBookAppointment:
                            result = service.NotBookedAppointment((NOT_BOOKED_APPOINTMENT)parm);
                            break;
                        case webServiceMethods.UnlockAppointment:
                            result = service.UnlockAppointmentRequest((string)parm);
                            break;
                    }

                    serviceProxy.Close();
                }
                catch (EndpointNotFoundException e)
                {
//                    Exception exDisplay = new Exception(
                    _serviceException = new Exception(
                        "Scheduler was unable to find an endpoint listening for the VSE_VAR_Service at the configured address.\n" +
                        "This is a configuration issue.  Please notify your supervisor.\n\n" +
                        "Review the Trace Log for further details."
                    );
                    Exception exLog = new Exception("Scheduler was unable to find an endpoint listening for the VSE_VAR_Service at the configured address.", e);
                    TraceLog.LogException(this.GetType().Name, methodName + "\\ExecuteWebServiceCall", exLog);
//                    Keys.AlertUserError(Keys.MainWindow, exDisplay, "VAR Web Service Not Found Error");
                    serviceProxy.Abort();
                }
                catch (CommunicationException e)
                {
//                    Exception exDisplay = new Exception(
                    _serviceException = new Exception(
                        "There was a Communication problem accessing the VSE_VAR_Service.\n" +
                        "Wait a minute or so and try the operation again.\n" + 
                        "If the problem persists notify your supervisor.\n\n" +
                        "Review the Trace Log for details of this issue."
                    );
                    Exception exLog = new Exception("There was a Communication problem accessing the VSE_VAR_Service.", e);
                    TraceLog.LogException(this.GetType().Name, methodName + "\\ExecuteWebServiceCall", exLog);
//                    Keys.AlertUserError(Keys.MainWindow, exDisplay, "VAR Web Service Communications Error");
                    serviceProxy.Abort();
                }
                catch (TimeoutException e)
                {
//                    Exception exDisplay = new Exception(
                    _serviceException = new Exception(
                        "The attempt to communicate with the VSE_VAR_Service timed out.\n" +
                        "Wait a minute or so and try the operation again.\n" + 
                        "If the problem persists notify your supervisor.\n\n" +
                        "Review the Trace Log for details of this issue."
                    );
                    Exception exLog = new Exception("The attempt to communicate with the VSE_VAR_Service timed out.", e);
                    TraceLog.LogException(this.GetType().Name, methodName + "\\ExecuteWebServiceCall", exLog);
//                    Keys.AlertUserError(Keys.MainWindow, exDisplay, "VAR Web Service Time Out Error");
                    serviceProxy.Abort();
                }
                catch (Exception e)
                {
//                    Exception exDisplay = new Exception(
                    _serviceException = new Exception(
                        "The was a problem communicating with the VSE_VAR_Service.\n" +
                        "Wait a minute or so and try the operation again.\n" + 
                        "If the problem persists notify your supervisor.\n\n" +
                        "Review the Trace Log for details of this issue."
                    );
                    Exception exLog = new Exception("The was a problem communicating with the VSE_VAR_Service.", e);
                    TraceLog.LogException(this.GetType().Name, methodName + "\\ExecuteWebServiceCall", exLog);
//                    Keys.AlertUserError(Keys.MainWindow, exDisplay, "VAR Web Service Error");
                    serviceProxy.Abort();
                }
            }

            return result;
        }

        private void ProcessReturnedErrorMessages(string[] errorMessages, string methodName)
        {
            if (errorMessages.Count() == 0)
                return;

            Exception innerException = new Exception(String.Join("\n", errorMessages));
            Exception exception = new Exception(String.Format("The VSE_VAR_Service returned error messages when called from the {0} method.", methodName));
            TraceLog.LogException(this.GetType().Name, methodName, exception);
            Keys.AlertUserError(Keys.MainWindow, exception, "VSE_VAR_Service Errors");
        }

        #endregion private methods

        #region TraceLog members

        private class paramItem
        {
            public string Name = "";
            public bool IsObject = false;
            public bool IsObjectAList = false;
            public string ValueAsString
            {
                get
                {
                    if (Value == null)
                        return "";

                    if (Value.GetType() == typeof(DateTime))
                        return ((DateTime)Value).ToString(Keys.Methods.ISO8601DateFormat);

                    return Value.ToString();
                }
            }
            // If Value is a list assign it as an array
            public object Value = null;
        }

        private class LogClass
        {
            public string MethodName = "";
            public List<paramItem> MethodParams = new List<paramItem>();
            public string APIName = "";
            public List<paramItem> APIParams = new List<paramItem>();
            public paramItem Results = new paramItem();
        }

        private static List<string> ConvertObjectListToStringList(object[] objList, bool returnNames = false)
        {
            List<string> results = new List<string>();

            if (objList != null)
            {
                if (objList.Count() != 0)
                {
                    if (returnNames)
                        results.Add(ConvertObjectToLogString(objList[0], true));

                    foreach (object obj in objList)
                        results.Add(ConvertObjectToLogString(obj));
                }
                results.Add("Empty");
            }
            else
                results.Add("null");

            return results;
        }

        private static List<string> ConvertObjetToStringList(object obj, bool returnNames = false)
        {
            List<string> results = new List<string>();

            if (obj != null)
            {
                foreach (var prop in obj.GetType().GetProperties())
                {
                    string line = "";
                    if (returnNames)
                        line = prop.Name + "=";

                    try
                    {
                        if ((prop.PropertyType == typeof(String[])) || (prop.PropertyType == typeof(System.String[])))
                        {
                            string[] list = (string[])prop.GetValue(obj, null);
                            string subLine = String.Join("|", list);
                            line += subLine;
                        }
                        else if (prop.PropertyType == typeof(int[]))
                        {
                            int[] list = (int[])prop.GetValue(obj, null);
                            string subLine = String.Join("|", list);
                            line += subLine;
                        }
                        else if (prop.PropertyType == typeof(DateTime[]))
                        {
                            DateTime[] list = (DateTime[])prop.GetValue(obj, null);
                            List<string> sList = new List<string>();

                            foreach (DateTime dt in list)
                                sList.Add(dt.ToString(Keys.Methods.ISO8601DateFormat));

                            string subLine = String.Join("|", sList.ToArray());
                            line += subLine;
                        }
                        else if (prop.PropertyType == typeof(Decimal[]))
                        {
                            Decimal[] list = (Decimal[])prop.GetValue(obj, null);
                            string subLine = String.Join("|", list);
                            line += subLine;
                        }
                        else if (prop.PropertyType.IsArray)
                        {
                            object[] items = (object[])prop.GetValue(obj, null);
                            List<string> subItems = new List<string>();

                            foreach (object item in items)
                            {
                                string subLine = String.Empty;
                                List<string> subProps = ConvertObjetToStringList(item, true);

                                if (subProps.Count > 0)
                                    subLine = String.Join("~~", subProps);

                                subItems.Add(subLine);
                            }

                            line += (String.IsNullOrEmpty(line) ? "" : "^") + String.Join("|", subItems.ToArray());
                        }
                        else
                            line += prop.GetValue(obj, null);
                    }
                    catch
                    {
                        line += "Unable to determine value";
                    }

                    results.Add(line);
                }
            }
            else
                results.Add("null");

            return results;
        }

        private static string ConvertObjectToLogString(object obj, bool returnNames = false)
        {
            string line = "";

            if (obj != null)
            {
                var props = obj.GetType().GetProperties();
                for (int index = 0; index < props.Length; index++)
                {
                    var prop = obj.GetType().GetProperties()[index];

                    if (returnNames)
                        line += (String.IsNullOrEmpty(line) ? "" : "^") + prop.Name;
                    else
                    {
                        try
                        {
                            if ((prop.PropertyType == typeof(String[])) || (prop.PropertyType == typeof(System.String[])))
                            {
                                string[] list = (string[])prop.GetValue(obj, null);
                                string subLine = String.Empty;
                                
                                if (list.Count() > 0)
                                    subLine = String.Join("|", list);

                                line += (String.IsNullOrEmpty(line) ? "" : "^") + subLine;
                            }
                            else if (prop.PropertyType == typeof(int[]))
                            {
                                int[] list = (int[])prop.GetValue(obj, null);
                                string subLine = String.Join("|", list);
                                line += (String.IsNullOrEmpty(line) ? "" : "^") + subLine;
                            }
                            else if (prop.PropertyType == typeof(DateTime))
                            {
                                DateTime val = (DateTime)prop.GetValue(obj, null);
                                line += (String.IsNullOrEmpty(line) ? "" : "^") + ((val == null) ? "Unknown" : ((DateTime)val).ToString(Keys.Methods.ISO8601DateFormat));
                            }
                            else if (prop.PropertyType == typeof(DateTime[]))
                            {
                                DateTime[] list = (DateTime[])prop.GetValue(obj, null);
                                List<string> sList = new List<string>();

                                foreach (DateTime dt in list)
                                    sList.Add(dt.ToString(Keys.Methods.ISO8601DateFormat));

                                string subLine = String.Join("|", sList.ToArray());
                                line += (String.IsNullOrEmpty(line) ? "" : "^") + subLine;
                            }
                            else if (prop.PropertyType == typeof(Decimal[]))
                            {
                                Decimal[] list = (Decimal[])prop.GetValue(obj, null);
                                string subLine = String.Join("|", list);
                                line += (String.IsNullOrEmpty(line) ? "" : "^") + subLine;
                            }
                            else if (prop.PropertyType.IsArray)
                            {
                                object[] items = (object[])prop.GetValue(obj, null);
                                List<string> subItems = new List<string>();

                                foreach (object item in items)
                                {
                                    string subLine = String.Empty;
                                    List<string> subProps = ConvertObjetToStringList(item, true);

                                    if (subProps.Count > 0)
                                        subLine = String.Join("~~", subProps);

                                    subItems.Add(subLine);
                                }

                                line += (String.IsNullOrEmpty(line) ? "" : "^") + String.Join("|", subItems.ToArray());
                            }
                            else
                                line += (String.IsNullOrEmpty(line) ? "" : "^") + prop.GetValue(obj, null);
                        }
                        catch
                        {
                            line += "Unable to determine value";
                        }
                    }
                }
            }
            else
                line = "null";

            return line;
        }

        private static void CreateLogEntry(LogClass logItems)
        {
            StringBuilder log = new StringBuilder();
            string line = logItems.MethodName + "(";
            List<string> list = new List<string>();

            foreach (paramItem item in logItems.MethodParams)
            {
                if (item.IsObject)
                    list.Add(item.Name);
                else
                {
                    string val = item.ValueAsString;
                    list.Add(String.IsNullOrEmpty(val) ? "empty" : val);
                }
            }

            line += String.Join(", ", list.ToArray()) + ")";
            log.AppendLine(line);

            foreach (paramItem item in logItems.MethodParams)
                if (item.IsObject)
                {
                    line = "  " + item.Name;

                    try
                    {
                        if (item.Value == null)
                            log.AppendLine(line += "    null");
                        else if (item.IsObjectAList)
                        {
                            log.AppendLine(line);
                            List<string> oList = ConvertObjectListToStringList((object[])item.Value, true);

                            foreach (string o in oList)
                                log.AppendLine(o);
                        }
                        else
                        {
                            line += "    " + ConvertObjectToLogString(item.Value, true);
                            line += "    " + ConvertObjectToLogString(item.Value);
                        }
                    }
                    catch
                    {
                        log.AppendLine(line += "    Unable to render the object as a string of property values.");
                    }
                }

            log.AppendLine("");
            log.AppendLine(new String('-', 80));

            log.AppendLine(logItems.APIName);
            int itemNo = 0;

            foreach (paramItem item in logItems.APIParams)
                if (item.IsObject)
                {
                    log.AppendLine("  #" + (itemNo++).ToString() + ": " + item.Name);

                    try
                    {
                        if (item.Value == null)
                            log.AppendLine("    null");
                        else if (item.IsObjectAList)
                        {
                            List<string> oList = ConvertObjectListToStringList((object[])item.Value, true);

                            foreach (string o in oList)
                                log.AppendLine("    " + o);
                        }
                        else
                        {
                            List<string> iList = ConvertObjetToStringList(item.Value, true);

                            foreach (string i in iList)
                                log.AppendLine("    " + i);
                        }
                    }
                    catch
                    {
                        log.AppendLine("    Unable to render the object as a string of property values.");
                    }
                }
                else
                    log.AppendLine("  #" + (itemNo++).ToString() + ": " + item.ValueAsString);


            log.AppendLine("");
            log.AppendLine(new String('-', 80));


            if (logItems.Results.IsObject)
            {
                try
                {
                    if (logItems.Results.IsObjectAList)
                    {
                        List<string> oList = ConvertObjectListToStringList((object[])logItems.Results.Value, true);

                        foreach (string o in oList)
                            log.AppendLine(o);
                    }
                    else
                    {
                        log.AppendLine(ConvertObjectToLogString(logItems.Results.Value, true));
                        log.AppendLine(ConvertObjectToLogString(logItems.Results.Value));
                    }
                }
                catch
                {
                    log.AppendLine("    Unable to render the object as a string of property values.");
                }
            }
            else
                log.AppendLine((string)logItems.Results.ValueAsString);

            TraceLog.LogInfo("VARWebServiceAPIService", logItems.MethodName, log.ToString());

        }

        #endregion TraceLog members
    }
}
