using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;

namespace ClinSchd.Modules.VAR.VAR
{
    /// <summary>
	/// Interaction logic for UserLoginView.xaml
    /// </summary>
	public partial class VARView :  Window, IVARView
    {
		public VARView ()
        {
            InitializeComponent();
        }

		public VARPresentationModel Model
        {
            get
            {
				return this.DataContext as VARPresentationModel;
            }

            set
            {
                this.DataContext = value;
            }
        }

		private bool bDialogResult = false;
		public bool ConfirmUser (string message, string caption)
		{
            bDialogResult = Infrastructure.Keys.ConfirmUser(this, message, caption);
            return bDialogResult;
        }

		public void AlertUser (string message, string caption)
		{
            Infrastructure.Keys.AlertUser(this, message, caption);
        }

        public void ClearSortFilerDescriptors()
        {
            MobileRequests.SortDescriptors.Clear();
            MobileRequests.FilterDescriptors.Clear();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
            MobileRequests.SelectedItem = null;
            Model.SelectedMobileRequest = null;
            MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));
        }

        public void ScrollSelectedRequestIntoView()
        {
            if (Model.SelectedMobileRequest != null)
                MobileRequests.ScrollIntoView(Model.SelectedMobileRequest);
        }

        private void HotKeyHandler(object sender, KeyEventArgs e)
        {
            if (Keyboard.Modifiers == ModifierKeys.Alt)
            {
                Key key = e.Key == Key.System ? e.SystemKey : e.Key;
                switch (key)
                {
                    case Key.P:
                        Keyboard.Focus(ProcessButton);
                        break;
                    case Key.C:
                        Keyboard.Focus(CancelButton);
                        break;
                }
            }
        }

        public void SetFocus()
        {
            if (Visibility == Visibility.Visible)
            {
                Focus();
                Keyboard.Focus(this);
            }
        }
    }
}
