﻿using Microsoft.Practices.Composite.Modularity;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.Presentation.Events;

using ClinSchd.Modules.VAR.Controllers;
using ClinSchd.Modules.VAR.Models;
using ClinSchd.Modules.VAR.ProcessRequest;
using ClinSchd.Modules.VAR.Services;
using ClinSchd.Modules.VAR.VAR;
using ClinSchd.Modules.VAR.VARButton;

namespace ClinSchd.Modules.VAR
{
    public class VARModule : IModule
    {
        private readonly IUnityContainer container;
        private readonly IEventAggregator eventAggregator;
        private IVARController controller;

        public VARModule (IUnityContainer container, IEventAggregator eventAggregator)
        {
            this.container = container;
            this.eventAggregator = eventAggregator;
        }

        // Summary:
        //     Notifies the module that it has be initialized.
        public void Initialize()
        {
            RegisterViewsAndServices();
            controller = container.Resolve<IVARController>();
            controller.Run();
        }

        protected void RegisterViewsAndServices()
        {
            container.RegisterType<IVSE_Information, VSE_Information>(new ContainerControlledLifetimeManager());
            container.RegisterType<IAppointmentRequestItemPresentationModel, AppointmentRequestItemPresentationModel>();
            container.RegisterType<IVARDetailView, VARDetailView>();
            container.RegisterType<IProcessRequestModel, ProcessRequestModel>();
            container.RegisterType<IProcessRequestPresentationModel, ProcessRequestPresentationModel>();
            container.RegisterType<IProcessRequestView, ProcessRequestView>();
            container.RegisterType<IVARController, VARController>();
            container.RegisterType<IVARButtonView, VARButtonView>();
            container.RegisterType<IVARButtonPresentationModel, VARButtonPresentationModel>();
            container.RegisterType<IVARView, VARView>();
            container.RegisterType<IVARPresentationModel, VARPresentationModel>();
            container.RegisterType<IVARService, VARService>();
            container.RegisterType<IVARWebServiceAPIService, VARWebServiceAPIService>();
        }
    }
}
