using System;
using System.Diagnostics;
using System.Windows;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using Microsoft.Practices.Composite.Events;
using Microsoft.Practices.Composite.UnityExtensions;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;
using Microsoft.Practices.Unity;
using Application = System.Windows.Application;
using MessageBox = System.Windows.MessageBox;

namespace ClinSchd
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        internal static IUnityContainer Container { get; set; }

        protected override void OnStartup(StartupEventArgs e)
        {

            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(assembly.Location);
            Keys.ApplicationVersion = "[v" + fvi.FileVersion + "]";

            var traceSession = Guid.NewGuid().ToString();
            traceSession = traceSession.Replace("-", "");
            TraceLog.UseSoftwareRendering = true;
            TraceLog.DumpFileLocation = "C:\\ProgramData\\VA VistA Scheduler\\VSETraceLog.xml";
            Keys.TraceSessionId = traceSession;
            
            TraceLog.LogInfo("SESSION ID", Keys.TraceSessionId, "");
            
            base.OnStartup(e);

 
#if (DEBUG)
            RunInDebugMode();
#else
            RunInReleaseMode();
#endif
            ShutdownMode = ShutdownMode.OnMainWindowClose;
        }

        private static void RunInDebugMode()
        {
            UnityBootstrapper bootstrapper = new ClinSchdBootstrapper();
            bootstrapper.Run();
            Container = bootstrapper.Container;
        }

        private static void RunInReleaseMode()
        {
            AppDomain.CurrentDomain.UnhandledException += AppDomainUnhandledException;
            try
            {
                UnityBootstrapper bootstrapper = new ClinSchdBootstrapper();
                bootstrapper.Run();
                Container = bootstrapper.Container;
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        private static void AppDomainUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            HandleException(e.ExceptionObject as Exception);
        }

        private static void HandleException(Exception ex)
        {
            if (ex == null)
                return;

            TraceLog.LogException("App", "HandleException", ex);
            TraceLog.SaveLogToFile("C:\\ProgramData\\VA VistA Scheduler\\VSETraceLog.xml");
            ExceptionPolicy.HandleException(ex, "Default Policy");
            MessageBox.Show(ClinSchd.Properties.Resources.UnhandledException);
            Environment.Exit(1);
        }

        private void Application_Exit(object sender, ExitEventArgs e)
        {
            IDataAccessService das = Container.Resolve<IDataAccessService>();
            das.Logout();
            das.RemoveJasperReportDependenciesFromUsersFolder();
            Current.Shutdown();
        }
    }
}