﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using Microsoft.Win32;

namespace ClinSchd.InactivityTimeout
{
    /// <summary>
    /// Interaction logic for CountDownControl.xaml
    /// </summary>
    public partial class CountDownControl : UserControl, IDisposable
    {
        #region Events

        /// <summary>
        /// Occurs when when the Timeout has been exceeded
        /// </summary>
        public event EventHandler OnTimeOut;

        #endregion Events

        #region Fields

        private DispatcherTimer _timer;
        private EventHandler _timerEventHandler;
        private MediaPlayer _mediaPlayer = null;

        private int _innerMaximum;
        private int _innerProgress;
        private int _middleMaximum;
        private int _middleProgress;
        private int _outerMaximum;
        private int _outerProgress;
        private int _remainingSeconds;

        #endregion Fields

        #region Constructor

        public CountDownControl()
        {
            InitializeComponent();

            if (DesignerProperties.GetIsInDesignMode(this))
            {
                InnerRingArc.StartAngle = 0;
                InnerRingArc.EndAngle = 270;
                MiddleRingArc.StartAngle = 0;
                MiddleRingArc.EndAngle = 180;
                OuterRingArc.StartAngle = 0;
                OuterRingArc.EndAngle = 90;
                CountTextBox.Text = "xx";
                UnitsTextBox.Text = "seconds";
            }
            else
            {
                InnerRingArc.StartAngle = 0;
                InnerRingArc.EndAngle = 0;
                MiddleRingArc.StartAngle = 0;
                MiddleRingArc.EndAngle = 0;
                OuterRingArc.StartAngle = 0;
                OuterRingArc.EndAngle = 0;
                CountTextBox.Text = "0";
                UnitsTextBox.Text = "seconds";
            }
        }

        #endregion Constructor

        #region Progress Bar Properties

        /// <summary>
        /// ProgressBarBorder: A Brush object defning the border on either side of the progress bar rings
        /// </summary>
        public Brush ProgressBarBorder
        {
            get { return OuterRingEllipse1.Stroke; }
            set
            {
                OuterRingEllipse1.Stroke = value;
                OuterRingEllipse2.Stroke = value;
                MiddleRingEllipse1.Stroke = value;
                MiddleRingEllipse2.Stroke = value;
                InnerRingEllipse1.Stroke = value;
                InnerRingEllipse2.Stroke = value;
            }
        }

        /// <summary>
        /// ProgressBarFill: A Brush object describing the fill for the area between the progress bar rings
        /// </summary>
        public Brush ProgressBarFill
        {
            get { return OuterRingEllipse1.Fill; }
            set
            {
                OuterRingEllipse1.Fill = value;
                MiddleRingEllipse1.Fill = value;
                InnerRingEllipse1.Fill = value;
            }
        }

        /// <summary>
        /// ProgressBarStroke:  A Brush object describing the progress bar ring color
        /// </summary>
        public Brush ProgressBarStroke
        {
            get { return OuterRingArc.Stroke; }
            set
            {
                OuterRingArc.Stroke = value;
                MiddleRingArc.Stroke = value;
                InnerRingArc.Stroke = value;
            }
        }

        /// <summary>
        /// Fill: A Brush object describing the color of the area inside the inner progress bar ring
        /// </summary>
        public Brush Fill
        {
            get { return OuterRingEllipse2.Fill; }
            set
            {
                OuterRingEllipse2.Fill = value;
                MiddleRingEllipse2.Fill = value;
                InnerRingEllipse2.Fill = value;
            }
        }

        /// <summary>
        /// Direction: A value from the SweepDirection enumeration indicating the direction the progress bar will fill, Clockwise, Counterclockwise
        /// </summary>
        public SweepDirection Direction
        {
            get { return OuterRingArc.Direction; }
            set
            {
                OuterRingArc.Direction = value;
                MiddleRingArc.Direction = value;
                InnerRingArc.Direction = value;
            }
        }

        /// <summary>
        /// OriginRotationDegress: A double value indicating the angle at which the progrss bar "starts"
        /// </summary>
        public Double OriginRotationDegrees
        {
            get { return OuterRingArc.OriginRotationDegrees; }
            set
            {
                OuterRingArc.OriginRotationDegrees = value;
                MiddleRingArc.OriginRotationDegrees = value;
                InnerRingArc.OriginRotationDegrees = value;
            }
        }

        #endregion Progress Bar Properties

        #region Status Text Properties

        /// <summary>
        /// ShowStatusText: A boolean value indicating if the control should display the status text in the center of the inner progress bar ring
        /// </summary>
        public bool ShowTextStatus
        {
            get { return (TextDisplayBorder.Visibility == Visibility.Visible); }
            set
            {
                if (value)
                    TextDisplayBorder.Visibility = Visibility.Visible;
                else
                    TextDisplayBorder.Visibility = Visibility.Hidden;
            }
        }

        /// <summary>
        /// ShowUnits: A boolean value indicating if the Units portion of the status text should be displayed in the center of the inner progress bar ring
        /// </summary>
        public bool ShowUnits
        {
            get { return (UnitsTextBox.Visibility == Visibility.Visible); }
            set
            {
                if (value)
                    UnitsTextBox.Visibility = Visibility.Visible;
                else
                    UnitsTextBox.Visibility = Visibility.Collapsed;
            }
        }
        
        /// <summary>
        /// StatusTextBorder: A Brush object describing the color of the Status text border in the center of the inner progress bar ring
        /// </summary>
        public Brush StatusTextBorder
        {
            get { return TextDisplayBorder.BorderBrush; }
            set { TextDisplayBorder.BorderBrush = value; }
        }

        /// <summary>
        /// StatusTextForeground: ABrush object describing the color of the Text Status text in the center of the inner progress bar ring
        /// </summary>
        public Brush StatusTextForeground
        {
            get { return TextDisplayBorder.BorderBrush; }
            set
            {
                TextDisplayBorder.BorderBrush = value;
                UnitsTextBox.Foreground = value;
                CountTextBox.Foreground = value;
            }
        }

        /// <summary>
        /// StatusTextBackground: A Brush object describing the background of the Status Text area in the center of the inner progress bar ring
        /// </summary>
        public Brush StatusTextBackground
        {
            get { return TextDisplayBorder.Background; }
            set { TextDisplayBorder.Background = value; }
        }

        #endregion Status Text Properties

        #region Run Time Properties

        /// <summary>
        /// CountDownInterval: A TimeSpan object describing the Time Out Interval, usually in seconds
        /// </summary>
        public TimeSpan CountDownInterval { get; set; }

        /// <summary>
        /// IsRunning: A Boolean value indicating the CountDown control is counting down
        /// </summary>
        public bool IsRunning
        {
            get
            {
                if (_timer == null)
                    return false;

                return _timer.IsEnabled;
            }
        }

        #endregion Run Time Properties

        #region Public Methods

        /// <summary>
        /// Start(TimeSpan countDownInterval): This method starts the Countdown using the countdown interval in the countDownInterval parameter
        ///     Parameters:
        ///         TimeSpan countDownInterval: A TimeSpan object describing the Time Out Interval, usually in seconds
        /// </summary>
        /// <param name="countDownInterval"></param>
        public void Start(TimeSpan countDownInterval)
        {
            CountDownInterval = countDownInterval;
            Start();
        }

        /// <summary>
        /// Start(): this method starts the Countdown using the countdown interval set in the CountDownInterval property
        /// </summary>
        /// <param name="countDownInterval"></param>
        public void Start()
        {
            if (CountDownInterval == null)
                throw new Exception("Cannot start the CountDownControl, a CountDownInterval has not been defined.  Please set the CountDownInterval property.");

            if (_timer == null)
            {
                _timerEventHandler = new EventHandler(_timer_Tick);
                // Create Timer with 10ms Intervals
                _timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(10) };
                _timer.Tick += _timerEventHandler;
            }

            _remainingSeconds = CountDownInterval.Seconds;
            UpdateStatusText();

            _innerMaximum = 100;
            _innerProgress = _innerMaximum;
            _middleMaximum = ((int)CountDownInterval.TotalMilliseconds / 10) / 4;
            _middleProgress = _middleMaximum;
            _outerMaximum = (int)CountDownInterval.TotalMilliseconds / 10;
            _outerProgress = _outerMaximum;

            SetProgress(InnerRingArc, _innerProgress, 0, _innerMaximum);
            SetProgress(MiddleRingArc, _middleProgress, 0, _middleMaximum);
            SetProgress(OuterRingArc, _outerProgress, 0, _outerMaximum);

            if (System.IO.File.Exists("TimeOutSoundEffect.mp3"))
            {
                _mediaPlayer = new MediaPlayer();
                _mediaPlayer.Open(new Uri(AppDomain.CurrentDomain.BaseDirectory + "TimeOutSoundEffect.mp3"));
                _mediaPlayer.MediaEnded +=new EventHandler(_mediaPlayer_MediaEnded);
                _mediaPlayer.Play();
            }

            if (!_timer.IsEnabled)
                _timer.Start();
        }

        /// <summary>
        /// Stop: This method stops the count down
        /// </summary>
        public void Stop()
        {
            if (_mediaPlayer != null)
            {
                _mediaPlayer.Stop();
                _mediaPlayer = null;
            }

            if (_timer != null)
            {
                if (_timer.IsEnabled)
                    _timer.Stop();

                _timer.Tick -= _timerEventHandler;
                _timerEventHandler = null;
                _timer = null;
            }
        }

        /// <summary>
        /// Reset: This method resets the CounDown by stopping, then starting the count down
        /// </summary>
        public void Reset()
        {
            Stop();
            Start();
        }

        /// <summary>
        /// Reset: This method resets the CounDown by stopping, then starting the count down with the countDownInterval parameter
        ///     Parameters:
        ///         TimeSpan countDownInterval: A TimeSpan object describing the Time Out Interval, usually in seconds
        /// </summary>
        public void Reset(TimeSpan countDownInterval)
        {
            Stop();
            Start(countDownInterval);
        }

        #endregion Public Methods

        #region Event Handlers

        private void _mediaPlayer_MediaEnded(object sender, EventArgs e)
        {
            _mediaPlayer.Play();
        }

        private void _timer_Tick(object sender, EventArgs e)
        {
            _innerProgress--;

            if (_innerProgress == 0)
            {
                _innerProgress = _innerMaximum;
                _remainingSeconds--;
                UpdateStatusText();
            }

            if (_remainingSeconds == 0)
            {
                Stop();
                SetProgress(InnerRingArc, 0, 0, _innerMaximum);
                SetProgress(MiddleRingArc, 0, 0, _middleMaximum);
                SetProgress(OuterRingArc, 0, 0, _outerMaximum);
                SendTimeOutEvent();
                return;
            }

            _middleProgress--;

            if (_middleProgress == 0)
                _middleProgress = _middleMaximum;

            _outerProgress--;

            SetProgress(InnerRingArc, _innerProgress, 0, _innerMaximum);
            SetProgress(MiddleRingArc, _middleProgress, 0, _middleMaximum);
            SetProgress(OuterRingArc, _outerProgress, 0, _outerMaximum);
        }

        #endregion Event Handlers

        #region Private Methods

        private void SendTimeOutEvent()
        {
            var temp = OnTimeOut;

            if (temp != null)
            {
                temp(this, EventArgs.Empty);
            }
        }

        private void SetProgress(Arc arc, double progress, double minimum, double maximum)
        {
            var startAngle = 359.999 * (minimum / (maximum - minimum));
            var endAngle = 359.999 * (progress / (maximum - minimum));
            arc.StartAngle = startAngle;
            arc.EndAngle = endAngle;
        }

        private void UpdateStatusText()
        {
            CountTextBox.Text = _remainingSeconds.ToString();
            UnitsTextBox.Text = ((_remainingSeconds == 1) ? "second" : "seconds");
        }

        #endregion Private Methods

        #region IDisposable members

        public void Dispose()
        {
            Stop();
        }

        #endregion IDisposable members
    }
}
