using System;
using System.Windows;
using Telerik.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interop;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Events;

namespace ClinSchd
{
    /// <summary>
    /// Interaction logic for Shell.xaml
    /// </summary>
    public partial class Shell : Window, IShellView
    {
        private GlobalHotKeyManager _hotKeyManager;

        public Shell()
        {
            Keys.MainWindow = this;
            StyleManager.ApplicationTheme = new Office_BlueTheme();
            InitializeComponent();
            Loaded += (s, e) => {
                TraceLog.AttachToSystemMenu(Handle, false);
                _hotKeyManager = new GlobalHotKeyManager(this);
                _hotKeyManager.RegisterGlobalHotKey("DemogEdit", 0x50, GlobalHotKeyManager.MOD_CONTROL, DemogEditAction);
            };
        }

        public readonly Action<object> DemogEditAction = o => { ((Shell)o).Presenter.EventAggregator.GetEvent<LaunchDemographicEditor>().Publish(string.Empty); };

        public ShellPresenter Presenter { get; set; }

        public Window Instance { get { return this; } }

        public void ShowView()
        {
            Show();
        }

        private void Grid_KeyDown (object sender, KeyEventArgs e)
        {
            Presenter.KeyPressed (e);
        }

        /// <summary>
        /// This is the Win32 Interop Handle for this Window
        /// </summary>
        public IntPtr Handle
        {
            get { return new WindowInteropHelper(this).Handle; }
        }

        public void SetSelectedTab(string command)
        {
            string[] parts = command.Split('^');
            int tabIndex = Convert.ToInt32(parts[1]);

            if (tabIndex < 0)
                return;
            
            switch (parts[0])
            {
                case "RIBBON":

                    foreach (RadPane item in MainPaneGroup.Items)
                        item.IsSelected = item.TabIndex == tabIndex;
                    break;

                case "AVAILABILITY":
                    foreach (RadPane item in SystemGroupPane.Items)
                        item.IsSelected = item.TabIndex == tabIndex;
                    break;
            }
        }

        private void SystemDummyRadPane_HeaderMouseButtonEvent(object sender, MouseButtonEventArgs e)
        {
            Presenter.ExecuteAvailabilityResourceCommand("Scheduled Visit");
            e.Handled = true;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            Keys.SetRenderingMode(this);
        }

        private void Window_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            Presenter.KeyPressed(e);
        }

        private void Window_Closed(object sender, EventArgs e)
        {
            _hotKeyManager.UnregisterAllHotKeys();
            OnClosed(e);
        }

        private void Window_Closing( object sender, System.ComponentModel.CancelEventArgs e )
        {
            e.Cancel = true;   // Don't close but send the code to an appropriate close
            Presenter.TimedOutEventHandler( sender, EventArgs.Empty );
        }

        private void Shell_OnContentRendered(object sender, EventArgs e)
        {
            WindowState = WindowState.Maximized;
        }

        private void Window_Activated(object sender, EventArgs e)
        {
            Presenter.ViewActivated();
        }
    }
}
