using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Input;
using ClinSchd.Infrastructure;
using ClinSchd.Infrastructure.Interfaces;
using ClinSchd.Infrastructure.Models;
using ClinSchd.Infrastructure.Events;
using ClinSchd.Infrastructure.Converters;
using System.Data;
using ClinSchd.Modules.DataAccess.Views;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Composite.Events;
using System.Timers;
using Cursors = System.Windows.Input.Cursors;
using Keys = ClinSchd.Infrastructure.Keys;
using MessageBox = System.Windows.MessageBox;
using Timer = System.Timers.Timer;
using System.Threading.Tasks;
using ClinSchd.Infrastructure.Factories;

namespace ClinSchd.Modules.DataAccess.Services
{
    public class DataAccessService : AsyncValidatableModel, IDataAccessService
    {
        private readonly IList<Patient> _emptyList = new List<Patient>();
        private readonly IList<SchdResource> _resourceEmptyList = new List<SchdResource>();
        private readonly IUnityContainer _container;
        private readonly IEventAggregator _eventAggregator;
        private string _currentUser = string.Empty;
        private string _currentUserName = string.Empty;
        private DataSet _mDsGlobal = new DataSet();
        private bool _mBSchedManager;
        private bool _mLoadGlobalRecordsets = true;
        private readonly OVIDDataTableRepository _adt;
        private const bool IS_LOGIN_DIALOG_SHOW = false;
        private readonly Dictionary<string, IList<SchdAvailability>> _mGlobalAvailabilities;
        readonly Timer _refreshTimer;
        private const int TIMER_INTERVAL = 1000;

        const string ACCESS_RIGHTS_ERROR =
                "The user does not have access rights to this application." +
                "\r\nContact the system administrator for further information.";


        /* MSC/DKA 9/26/14 Added Logout() method to facilitate clean exit from RPC Broker connection */
        public void Logout()
        {
            _adt.UserInfoLogout();
        }
        public void StartRefreshTimer()
        {
            _refreshTimer.Start();
        }
        public DataAccessService()
        {
            TraceLog.LogInfo("DataAccessService", "Construct", "Loading DataAccessService");
            OVIDDataTableRepository.Container = new System.Windows.Controls.Grid();
            _adt = OVIDDataTableRepository.Instance;
        }

        private int _initializeCounter = 0;
        private int _initializeAction = 0;
        private bool _initializeCompmlete = false;
        public DataAccessService(IUnityContainer container)
        {
            TraceLog.LogInfo("DataAccessService", "Construct", "Loading DataAccessService");
            _container = container;
            OVIDDataTableRepository.Container = new System.Windows.Controls.Grid();
            _adt = OVIDDataTableRepository.Instance;
            _mGlobalAvailabilities = new Dictionary<string, IList<SchdAvailability>>();
            var newModel = new PassthroughAsyncModel();
            newModel.InvokeAsync((s, args) => LoadGlobalRecordsets(), null);
            _refreshTimer = new Timer(TIMER_INTERVAL);
            _refreshTimer.Enabled = false;
            _refreshTimer.Elapsed += refreshTimer_Elapsed;

            // This mess of code is designed to try and prevent an exception on load the SOMETIMES happens
            //  It is HOPED that this is a timing issue and that a short wait after an exception, then retry will prevent
            // and allow the app to finish loading
            if (_initializeAction < 3)
            {
                while (!_initializeCompmlete && (_initializeCounter < 3))
                {
                    try
                    {
                        if (_initializeAction == 0)
                        {
                            _eventAggregator = _container.Resolve<IEventAggregator>();
                            _initializeAction++;
                        }

                        if (_initializeAction == 1)
                        {
                            _eventAggregator.GetEvent<ManagementItemAddedEvent>().Subscribe(ResetSchedulerTimer);
                            _initializeAction++;
                        }

                        if (_initializeAction == 2)
                        {
                            _eventAggregator.GetEvent<LoadGlobalRecordsetsEvent>().Subscribe(ResetSchedulerTimer);
                            _initializeAction++;
                        }

                        _initializeCompmlete = true;
                    }
                    catch (Exception ex)
                    {
                        if (_initializeCounter == 3)
                            throw;

                        _initializeCounter++;
                        System.Threading.Thread.Sleep(250);
                    }
                }
            }
        }

        public void refreshTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _eventAggregator.GetEvent<RefreshScheduler>().Publish(null);
            _eventAggregator.GetEvent<RefreshAvailability>().Publish(null);

            if (Keys.BooleanFlags.RefreshWaitList != false)
                _eventAggregator.GetEvent<LoadGlobalRecordsetsEvent>().Publish(null);

            ResetSchedulerTimer("");
        }

        private void ResetSchedulerTimer(string args)
        {
            _refreshTimer.Stop();
            _refreshTimer.Interval = 1800000;
            _refreshTimer.Start();
        }

        // Safely extract a string from a DataRow
        private string GetCellString(DataRow row, string name)
        {
            if (row.Table.Columns.Contains(name))
                return row[name].ToString();
            return string.Empty;
        }

        public DataSet GlobalDataSet
        {
            get { return _mDsGlobal; }
            set { _mDsGlobal = value; }
        }

        public bool SchedManager
        {
            get { return _mBSchedManager; }
            set { _mBSchedManager = value; }
        }

        public Divisions CurrentDivision { get; set; }

        public string CurrentDivisionIEN
        {
            get
            {
                if (CurrentDivision == null)
                    return string.Empty;

                return CurrentDivision.IEN;
            }
        }

        public bool IsLoginDialogShow
        {
            get { return IS_LOGIN_DIALOG_SHOW; }
        }

        public List<DataTable> GetAuditReportData(DateTime? startDate, DateTime? endDate, string scheduler = "", string lastSub = "")
        {
            List<DataTable> result = new List<DataTable>();
            result.Add(GetAuditData(startDate, endDate, scheduler, lastSub));
            result.Add(GetCancelAuditData(startDate, endDate, scheduler));
            return result;
        }

        #region IDataAccess Members

        public string GetVistAPatientAppointmentStatus(string patientIen, DateTime appointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC APPT STATUS",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });
        }


        #region EXPANDED ENTRY CALLS
        public string[] GetExpandedEntryInfomation(string patientIen, DateTime appointmentDateTime)
        {
            var demo = _adt.CallVistaRpcString("SDEC EP DEMOGRAPHICS",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var eventLog = _adt.CallVistaRpcString("SDEC EP EVENT LOG",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var waitTime = _adt.CallVistaRpcString("SDEC EP WAIT TIME",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var patientInfo = _adt.CallVistaRpcString("SDEC EP PT INFO",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var checkoutClassifications = _adt.CallVistaRpcString("SDEC EP CLASSIFICATION",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var checkoutProviders = _adt.CallVistaRpcString("SDEC EP PROVIDER",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var checkoutDiagnosis = _adt.CallVistaRpcString("SDEC EP DIAGNOSIS",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var checkoutStopCodes = _adt.CallVistaRpcString("SDEC EP STOP CODES",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            var checkoutProcedures = _adt.CallVistaRpcString("SDEC EP CPT",
                        new List<string> { patientIen, ExtensionMethods.ConvertToFilemanDateTime(appointmentDateTime) });

            return new string[] { demo, eventLog, waitTime, patientInfo, checkoutClassifications, checkoutProviders, checkoutDiagnosis, checkoutStopCodes, checkoutProcedures };
        }
        public string GetExpandedEntryDemographics(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP DEMOGRAPHICS",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedEntryEventLog(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP EVENT LOG",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedEntryWaitTime(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP WAIT TIME",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedPatientInfo(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP PT INFO",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedCheckoutClassifications(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP CLASSIFICATION",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedCheckoutProviders(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP PROVIDER",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedCheckoutDiagnosis(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP DIAGNOSIS",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedCheckoutStopCodes(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP STOP CODE",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        public string GetExpandedCheckoutProcedures(string patientIen, string fileManAppointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC EP CPT",
                        new List<string> { patientIen, fileManAppointmentDateTime });
        }
        #endregion

        #region PATIENT CONTACT CALLS
        //GETS
        public string GetPatientContact(List<string> patientContactRequestParams)
        {
            //PARAMS=PATIENT IEN, CLINIC IEN, DESIRED DATE, APPOINTMENT REQUEST TYPE (single letter), SERVICE IEN
            return _adt.CallVistaRpcString("SDEC CONTACT DISPLAY", patientContactRequestParams);
        }
        public string GetContactAttempts(List<string> patientContactRequestParams)
        {
            // PARAMS = PATIENT IEN, CLINIC IEN, DESIRED DATE, APPOINTMENT REQUEST TYPE (single letter), SERVICE IEN
            return _adt.CallVistaRpcString("SDEC CONTACT MULTI-DISPLAY", patientContactRequestParams);
        }
        public string GetClinicStopCodes(List<string> clinicStopCodeParam)
        {
            //PARAMS=Clinic IEN
            return _adt.CallVistaRpcString("SDEC CONTACT STOP CODE", clinicStopCodeParam);
        }

        //UPDATES
        public void UpdateContactAttempt(List<string> updatePatientContactRequestParams)
        {
            _adt.CallVistaRpcString("SDEC CONTACT UPDATE", updatePatientContactRequestParams);
        }
        public void ResetPatientContactMainSequence(List<string> patientContactReadRequestParams)
        {
            _adt.CallVistaRpcString("SDEC CONTACT SEQUENCE", patientContactReadRequestParams);
        }

        //INSERTS
        public void CreateNewSdecContact(List<string> createPatientContactRequestParams)
        {
            _adt.CallVistaRpcString("SDEC CONTACT NEW", createPatientContactRequestParams);
        }


        #endregion

        #region BENEFIT/ELIGIBILITY CALLS
        public string GetPatientEligibilities(string patientIen)
        {
            return _adt.CallVistaRpcString("SDEC ELIGIBILITY", new List<string> { patientIen });
        }

        public string GetEligibility(string dfn, string clinicIen, string appointmentDateTime)
        {
            return _adt.CallVistaRpcString("SDEC ELIG RETURN", new List<string> { dfn, clinicIen, appointmentDateTime });
        }
        #endregion

        #region RTC CALLS
        public string GetPrerequisites(string requestIen)
        {
            return _adt.CallVistaRpcString("SDEC GETPRER", new List<string> { requestIen });
        }
        #endregion


        public bool LoadUserInfo(out string errorMessage)
        {
            var results = false;
            errorMessage = string.Empty;

            var parm1 = new List<string> { CurrentUser };
            if (_adt != null)
            {
                var dtUser = _adt.callOVIDRPC("SDEC SUSRINFO", parm1, "SchedulerUser", _mDsGlobal);

                // Infrastructure.Keys.LoggedInUser = dtUser.Rows
                switch (dtUser.Rows.Count)
                {
                    case 0:
                        errorMessage = ACCESS_RIGHTS_ERROR;
                        break;
                    case 1:
                        var rUser = dtUser.Rows[0];

                        var oUser = rUser["MANAGER"];
                        var oMenu = rUser["MENU"];
                        var oRequest = rUser["SDECRMIC"];
                        var oSDWLMenu = rUser["SDWLMENU"];
                        var oSDECViewOnly = rUser["SDECVW"];

                        if ((oMenu.ToString() == "NO") && (oUser.ToString() == "NO") && (oSDECViewOnly.ToString() == "NO"))
                            errorMessage = ACCESS_RIGHTS_ERROR;
                        else
                        {

                            _currentUserName = rUser["USER_NAME"].ToString();
                            Keys.LoggedInUser = _currentUserName;
                            Keys.LoggedInUserRegularOverbook = Keys.Methods.StrToBool(rUser["SDOB"].ToString());
                            Keys.LoggedInUserMasterOverbook = Keys.Methods.StrToBool(rUser["SDMOB"].ToString());
                            Keys.LoggedInUserSDWLMenuKey = Keys.Methods.StrToBool(oSDWLMenu.ToString());
                            Keys.LoggedInUserSDECViewOnlyKey = Keys.Methods.StrToBool(oSDECViewOnly.ToString());

                            var sUser = oUser.ToString();
                            _mBSchedManager = sUser == "YES";

                            if (_eventAggregator != null)
                            {
                                _eventAggregator.GetEvent<IsManagerUserEvent>().Publish(_mBSchedManager);
                                _eventAggregator.GetEvent<VAR_IsVARUserEvent>().Publish(Keys.Methods.StrToBool(oRequest.ToString()) && Keys.VARWebServiceConfigured);
                            }

                            var parm2 = new List<string>();
                            var dtOther = _adt.CallVistaRpcString("ORWU USERINFO", parm2);
                            var parts = dtOther.Split('^');

                            if (parts.Length > 9 && _eventAggregator != null)
                                //_eventAggregator.GetEvent<InitializeIdleTimeTimeOutEvent>().Publish( string.Format( "INITIALIZE^120^60", parts[7], parts[8] ) );
                                _eventAggregator.GetEvent<InitializeIdleTimeTimeOutEvent>().Publish(string.Format("INITIALIZE^{0}^{1}", parts[7], parts[8]));

                            results = true;
                        }

                        break;
                    default:
                        errorMessage = "Multiple User Information records were returned. Contact your system administrator regarding this error.";
                        break;
                }
            }
            else
                errorMessage = "Unable to retrieve User Information, a data connection does not exist.";

            Keys.UserPrefList = GetUserPreferences(Keys.LoggedInUserIen);

            if ((Keys.UserPrefList != null) && (Keys.UserPrefList.Count > 0))
            {
                //Trigger precessor
                if (_eventAggregator != null)
                {
                    _eventAggregator.GetEvent<ProcessUserPreferencesEvent>().Publish(Keys.UserPrefList);
                    _eventAggregator.GetEvent<UpdateUserPrefIconEvent>().Publish(Keys.UserPrefList.Count > 0);
                }
            }
            return results;
        }

        public bool LoadGlobalRecordsets()
        {
            if (_adt == null)
                return false;

            TraceLog.LogInfo("DataAccessService", "LoadGlobalRecordsets", "Loading GlobalRecordsets");
            if (OVIDDataTableRepository.LoginDialogShow)
            {
                _eventAggregator.GetEvent<LaunchUserLoginDialogEvent>().Publish("User Login");
                return false;
            }

            //Schedule User Info
            var parm1 = new List<string> { CurrentUser };
            if (_adt != null)
            {
                if (_mDsGlobal.Tables["SchedulerUser"] == null)
                {
                    string errorMessage;
                    _mLoadGlobalRecordsets = LoadUserInfo(out errorMessage);
                }
            }

            //Get all divisions
            var getFac = _adt.callOVIDRPC("SDEC GETFAC", parm1, "AllDivisions", _mDsGlobal);
            foreach (DataRow dr in getFac.Rows)
            {
                Keys.DisplayAmieCandPPrompts = Keys.Methods.StrToBool(dr["DIALOGUE"].ToString());
                Keys.DefaultFacilityFontSize = dr["DEF_FONT"].ToString();
                if (Keys.DefaultFacilityFontSize == "")
                    Keys.DefaultFacilityFontSize = "13"; // Default is set by VA
                break;
            }

            //Get current division
            CurrentDivision = GetDefaultDivision();

            if (_eventAggregator != null)
            {
                var payload = "^";
                if (CurrentDivision != null)
                    payload = CurrentDivision.IEN + "^" + CurrentDivision.NAME;

                _eventAggregator.GetEvent<DisplayDefaultDivisionEvent>().Publish(payload);
            }

            //Get Access Types
            _adt.callOVIDRPC("SDEC ACCTYPE", null, "AccessTypes", _mDsGlobal);

            //Build Primary Key for AccessTypes table
            var dsa = new DataColumn[1];
            var dtsa = _mDsGlobal.Tables["AccessTypes"];
            dsa[0] = dtsa.Columns["SDEC_ACCESS_TYPE_IEN"];
            _mDsGlobal.Tables["AccessTypes"].PrimaryKey = dsa;

            //Get Grouped Access Types
            _adt.callOVIDRPC("SDEC ACCGPTYG", null, "GroupedAccessType", _mDsGlobal);

            //Get Schedule Users
            _adt.callOVIDRPC("SDEC SCHUSR", null, "ScheduleUser", _mDsGlobal);

            //Build Primary Key for ScheduleUser table
            var dsr = new DataColumn[1];
            var dtsr = _mDsGlobal.Tables["ScheduleUser"];
            dsr[0] = dtsr.Columns["USERID"];
            _mDsGlobal.Tables["ScheduleUser"].PrimaryKey = dsr;

            /*_adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, null, "ResourceUser", _mDsGlobal);
            //Build Primary Key for ResourceUser table
            var drr = new DataColumn[1];
            var dtrr = _mDsGlobal.Tables["ResourceUser"];
            drr[0] = dtrr.Columns["SDEC_RESOURCE_USER_IEN"];
            _mDsGlobal.Tables["ResourceUser"].PrimaryKey = drr;*/
            ////////////////
            var accessTypes = GetAccessTypes();
            foreach (var accessType in accessTypes)
            {
                switch (accessType.NAME.ToUpper())
                {
                    case "AVAILABLE":
                        Keys.AccessTypes.Available = accessType.BSDX_ACCESS_TYPE_IEN;
                        break;
                    case "UNAVAILABLE":
                        Keys.AccessTypes.Unavailable = accessType.BSDX_ACCESS_TYPE_IEN;
                        break;
                    case "CANCELED":
                        Keys.AccessTypes.Cancelled = accessType.BSDX_ACCESS_TYPE_IEN;
                        break;
                }
            }
            return _mLoadGlobalRecordsets;
        }

        public DataTable GetClinicSetupParameters(string hospitalLocationID, string hospitalLocation, string sdNoLet = "")
        {
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.SdecClinSet);

            if (Keys.SelectedAppointmentRequest != null && Keys.SelectedAppointmentRequest.AppointmentRequestType != Keys.AppointmentRequestTypes.PtCSch)
            {
                if (string.IsNullOrEmpty(sdNoLet))
                {
                    if (!string.IsNullOrEmpty(hospitalLocationID))
                    {
                        var key = Keys.Tables.SdecClinSet.Keys.FirstOrDefault(k => k.IEN == hospitalLocationID);
                        if (key != null && Keys.Tables.SdecClinSet[key].Table.Rows.Count > 0)
                            return Keys.Tables.SdecClinSet[key].Table;
                    }

                    if (!string.IsNullOrEmpty(hospitalLocation))
                    {
                        var key = Keys.Tables.SdecClinSet.Keys.FirstOrDefault(k => k.Name == hospitalLocation);
                        if (key != null && Keys.Tables.SdecClinSet[key].Table.Rows.Count > 0)
                            return Keys.Tables.SdecClinSet[key].Table;
                    }
                }
            }

            var parms = new List<string>
            {
                "1",
                hospitalLocationID,
                hospitalLocation
            };
            if (!string.IsNullOrEmpty(sdNoLet))
                parms.Add(sdNoLet);
            else
            {
                if (Keys.SelectedAppointmentRequest != null && Keys.SelectedAppointmentRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.PtCSch)
                    parms.Add("1");
            }

            var dt = _adt.callOVIDRPC(Keys.RpcCalls.ClinSet, parms);

            if (dt.Rows.Count > 0)
            {
                if (string.IsNullOrEmpty(hospitalLocationID))
                    hospitalLocationID = dt.Rows[0]["HOSPITAL_LOCATION_ID"].ToString();

                hospitalLocation = dt.Rows[0]["HOSPITAL_LOCATION"].ToString();
            }

            Keys.Tables.SdecClinSet.Add(new DataTableKey(hospitalLocationID, hospitalLocation), new DataTableCache(dt));
            return dt;
        }

        public DataTable SearchClinicSetupParameters(string searchString)
        {
            return GetClinicSetupParameters(null, searchString);
        }

        public DataTable GetClinicStop(string searchString)
        {
            DataTable dt = null;
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.ClinicStopData);
            var key = Keys.Tables.ClinicStopData.Keys.FirstOrDefault(k => k.IEN == searchString);

            if (key != null && Keys.Tables.ClinicStopData[key].Table.Rows.Count > 0)
                dt = Keys.Tables.ClinicStopData[key].Table;

            if (dt == null)
            {
                var parm1 = new List<string> { searchString };
                dt = _adt.callOVIDRPC("SDEC CLINSTOP", parm1);
                Keys.Tables.ClinicStopData.Add(new DataTableKey(searchString, ""), new DataTableCache(dt, 60));
            }

            return dt;
        }

        public List<ZipLinkInfo> GetLinkedCityState(string zipcode)
        {
            var list = new List<ZipLinkInfo>();

            if (zipcode.Length == 5 || zipcode.Length == 9)
            {
                var parm1 = new List<string> { zipcode };
                var linkedInfo = _adt.callOVIDRPC(Keys.RpcCalls.GetLinkedCityState, parm1);
                if (linkedInfo.Rows.Count == 0)
                {
                    MessageBox.Show(ACCESS_RIGHTS_ERROR, "Get linked City/State");
                    return list;
                }

                foreach (DataRow row in linkedInfo.Rows)
                {
                    var zipInfo = new ZipLinkInfo
                    {
                        City = row["CITYNAME"].ToString(),
                        CityAbbreviation = row["CITY_ABB"].ToString(),
                        CityKey = row["CITY_KEY"].ToString(),
                        PreferredCityKey = row["P_CITY_KEY"].ToString(),
                        State = row["STATENAME"].ToString(),
                        StateIEN = row["STATE_IEN"].ToString(),
                        County = row["COUNTYNAME"].ToString(),
                        CountyIEN = row["COUNTY_IEN"].ToString(),
                        FIPSCode = row["FIPS"].ToString(),
                        ZipCode = row["ZIP"].ToString(),
                        UniqueKey = row["UID"].ToString()
                    };

                    list.Add(zipInfo);
                }
            }
            return list;
        }

        public void SetCurrentPatient(Patient selectedPatient)
        {
            var parm1 = new List<string>
            {
                "DPT(",
                selectedPatient.IEN  // Patient DFN
            };
            _adt.CallVistaRpc("SDEC SPACEBAR", parm1);
        }

        public NameValue GetCCOWCurrentPatient()
        {
            NameValue result = null;
            var parm1 = new List<string>();
            var data = _adt.CallVistaRpc(Keys.PatientPromptRpcs.GetCurrentPatient, parm1);

            if (data != null)
            {
                string dataItem = GetArrayOfArrayItem(data, 0, 0);

                if (!string.IsNullOrEmpty(dataItem))
                    result = new NameValue(dataItem.Piece("^", 2), dataItem.Piece("^", 1));
            }

            return result;
        }

        public DataTable GetHospitalLocations(string hospitalLocaltionID)
        {
            // Remove any stale records from the Keys.Tables.HospLocCache
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.HospLocCache);

            if (string.IsNullOrEmpty(hospitalLocaltionID))
                return null;

            if (!Keys.Tables.HospLocCache.ContainsKey(hospitalLocaltionID))
            {
                var parms = new List<string> { hospitalLocaltionID };

                var table = _adt.callOVIDRPC(Keys.RpcCalls.HospitalLocations, parms);
                Keys.Tables.HospLocCache.Add(hospitalLocaltionID, new DataTableCache(table, 30));
            }

            return Keys.Tables.HospLocCache[hospitalLocaltionID].Table;
        }

        public List<ScheduleUser> GetSchedulers()
        {
            //Get Schedule Users
            var dtScheduleUsers = _adt.callOVIDRPC("SDEC SCHUSR", null, "ScheduleUser", _mDsGlobal);
            var schedUsers = new List<ScheduleUser>();
            foreach (DataRow row in dtScheduleUsers.Rows)
            {
                var schedUser = new ScheduleUser
                { Ien = GetCellString(row, "USERID"), Name = GetCellString(row, "USERNAME") };
                schedUsers.Add(schedUser);
            }

            return schedUsers;
        }

        public void LoadWaitList(WorkCompletedMethod workCompletedMethod)
        {
            //var x = 0;
            //var y = x;

            DoWorkAsync((s, args) =>
            {
                args.Result = LoadAllWaitListRecords();
            }, workCompletedMethod);
        }

        public object LoadAllWaitListRecords()
        {
            return null;
        }

        public void RefreshAllAppointmentRequests()
        {
            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Clear();
            var ewlRequests = GetEwlAppointmentRequests();
            foreach (var eReq in ewlRequests)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(eReq);
            var apptRequests = GetApptAppointmentRequests();
            foreach (var aReq in apptRequests)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(aReq);
            var parms1 = new List<string>();
            Keys.AppointmentRequestWaitListTables.PtCSchWaitList = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, parms1);
            Keys.AppointmentRequestWaitListTables.ConsultWaitList = _adt.callOVIDRPC(Keys.RpcCalls.GetConsultAppointmentRequests, parms1);
            var consultRequests = GetConsultAppointmentRequests();
            foreach (var cReq in consultRequests)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
            var ptCSchRequests = GetPtCSchAppointmentRequests();
            foreach (var rReq in ptCSchRequests)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(rReq);
            Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = true;
        }

        public bool GetWaitListRecordsForSelectedPatient(string patientIen, string lastRec = "", bool isNewQuery = false, bool byDescendingIen = false, string requestType = "", string newlyOpenedRequestIen = "", bool isInThread = false, List<string> messages = null)
        {
            if (string.IsNullOrEmpty(patientIen))
            {
                if (messages == null)
                    Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "Can not retrieve records for patient.  Selected Patient IEN is null.")));
                else
                    messages.Add("Can not retrieve records for patient.  Selected Patient IEN is null.");

                return false;
            }
            /*  Request Filters...................
                AppointmentRequestType - These can be EWL, PtCSch, Consult, new Appt Request file
                Patient.Name
                Patient.ServiceRelated
                Patient.PriorityGroup
                DesiredAppointmentDateByType
                OriginatingDateString
                ClinicServiceSpecialtyMerge
            */

            if (Keys.AppointmentRequestWaitListTables.EwlWaitList == null)
                Keys.AppointmentRequestWaitListTables.EwlWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.PtCSchWaitList == null)
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.ConsultWaitList == null)
                Keys.AppointmentRequestWaitListTables.ConsultWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.ApptWaitList == null)
                Keys.AppointmentRequestWaitListTables.ApptWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests == null)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests = new ObservableCollection<AppointmentRequest>();

            if (isNewQuery)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Clear();
            Keys.WaitListQueryType = Keys.WaitListQueryTypes.SelectPatient;


            Keys.WaitListCurrentFilter = "WaitTime^All Days|PATIENTNAME^" + patientIen;
            /*if (byDescendingIen && !string.IsNullOrEmpty(requestType))
            {
                if (requestType == Keys.AppointmentRequestTypes.Veteran)
                    requestType = Keys.AppointmentRequestTypes.Appt;

                Keys.WaitListCurrentFilter += "|REQUESTTYPE^" + requestType.Substring(0, 1).ToUpper();
                if (requestType == Keys.AppointmentRequestTypes.Appt)
                    Keys.WaitListCurrentFilter += "~E~R~C";
                else if(requestType == Keys.AppointmentRequestTypes.PtCSch)
                    Keys.WaitListCurrentFilter += "~A~E~C";
                else if(requestType == Keys.AppointmentRequestTypes.Ewl)
                    Keys.WaitListCurrentFilter += "~A~R~C";
            }*/

            var parms = new List<string>
                {
                    Keys.MaxWaitListRecordsToReturn,
                    lastRec,
                    Keys.WaitListCurrentFilter
                };
            if (byDescendingIen)
            {
                if (requestType == Keys.AppointmentRequestTypes.Veteran
                    || requestType == Keys.AppointmentRequestTypes.ReturnToClinic)
                    requestType = Keys.AppointmentRequestTypes.Appt;
                if (requestType == Keys.AppointmentRequestTypes.Appt)
                    parms.Add("ATOPD");
                else if (requestType == Keys.AppointmentRequestTypes.PtCSch)
                    parms.Add("RTOPD");
                else if (requestType == Keys.AppointmentRequestTypes.Ewl)
                    parms.Add("ETOPD");
                else
                    parms.Add(null);
                if (!string.IsNullOrEmpty(newlyOpenedRequestIen))
                {
                    if (requestType == Keys.AppointmentRequestTypes.Appt)
                        parms.Add("A" + newlyOpenedRequestIen);
                    else if (requestType == Keys.AppointmentRequestTypes.Ewl)
                        parms.Add("E" + newlyOpenedRequestIen);
                }


            }
            var rows = _adt.callOVIDRPC(Keys.RpcCalls.ServerQuery, parms);
            if (rows.Rows.Count == 0)
            {
                Keys.WaitListQueryLastRec = null;
                if (!string.IsNullOrEmpty(lastRec))
                {
                    if (messages == null)
                        Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "All records have been displayed.")));
                    else
                        messages.Add("All records have been displayed.");
                }
                return true;
            }
            var ctr = 0;
            try
            {
                foreach (DataRow row in rows.Rows)
                {
                    if (!isInThread)
                        System.Windows.Forms.Application.DoEvents();

                    ctr++;
                    if (ctr == 1)
                    {
                        if (Keys.Methods.IsNumeric(row["TOTAL"].ToString()))
                            Keys.WaitListQueryTotalRecords = Convert.ToInt32(row["TOTAL"]);
                        else
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "Missing Total Records from Wait List Query")));
                            else
                                messages.Add("Missing Total Records from Wait List Query");

                            return false;
                        }
                    }
                    var id = row["IEN"];
                    var type = row["TYPE"].ToString();
                    var key = row["KEY"].ToString();
                    if (ctr == rows.Rows.Count)
                        Keys.WaitListQueryLastRec = key;
                    if (type == "E")
                    {
                        Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
                        var param = new List<string> { id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "WLGET returning too many records.")));
                            else
                                messages.Add("WLGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.EwlWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.EwlWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                            //{
                            Keys.AppointmentRequestWaitListTables.EwlWaitList.Rows.Add(rec.ItemArray);

                        var ewlRequests = GetEwlAppointmentRequests();
                        foreach (var eReq in ewlRequests)
                        {
                            if (!string.IsNullOrEmpty(eReq.RequestManagmentIen))
                                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(eReq);
                            else
                                Keys.WaitListQueryTotalRecords--;
                        }
                        //}
                    }
                    else if (type == "R")
                    {
                        Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
                        var param = new List<string> { null, null, null, null, null, id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "RECGET returning too many records.")));
                            else
                                messages.Add("RECGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                            //{
                            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows.Add(rec.ItemArray);

                        var ptCSchRequests = GetPtCSchAppointmentRequests();
                        foreach (var rReq in ptCSchRequests)
                        {
                            if (rReq.PtCSchRequestIen != 0)
                                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(rReq);
                            else
                                Keys.WaitListQueryTotalRecords--;
                        }
                        //}
                    }
                    else if (type == "C")
                    {
                        Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
                        var param = new List<string> { null, null, null, null, id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetConsultAppointmentRequests, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "REQGET returning too many records.")));
                            else
                                messages.Add("REQGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.ConsultWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                            //{
                            Keys.AppointmentRequestWaitListTables.ConsultWaitList.Rows.Add(rec.ItemArray);

                        var consultRequests = GetConsultAppointmentRequests();
                        foreach (var cReq in consultRequests)
                        {
                            if (cReq.ConsultationRequestIen != 0)
                                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
                            else
                                Keys.WaitListQueryTotalRecords--;
                        }
                        //}
                    }
                    else if (type == "A")
                    {
                        Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
                        var param = new List<string> { id.ToString() };
                        //param.Add("8");
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, param);

                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "ARGET returning too many records.")));
                            else
                                messages.Add("ARGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.ApptWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.ApptWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                            //{
                            Keys.AppointmentRequestWaitListTables.ApptWaitList.Rows.Add(rec.ItemArray);

                        var apptRequests = GetApptAppointmentRequests();
                        foreach (var cReq in apptRequests)
                        {
                            if (cReq.ApptRequestIen != 0)
                                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
                            else
                                Keys.WaitListQueryTotalRecords--;
                        }
                        //}
                    }

                    if (!isInThread)
                        System.Windows.Forms.Application.DoEvents();
                }
                Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = true;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "GetWaitListRecordsForSelectedPatient", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
                return false;
            }
            return true;
        }

        public bool GetWaitListRecords(string lastRec, List<string> messages = null)
        {
            /*  Request Filters...................
                AppointmentRequestType - These can be EWL, PtCSch, Consult, new Appt Request file
                Patient.Name
                Patient.ServiceRelated
                Patient.PriorityGroup
                DesiredAppointmentDateByType
                OriginatingDateString
                ClinicServiceSpecialtyMerge
            */

            if (Keys.AppointmentRequestWaitListTables.EwlWaitList == null)
                Keys.AppointmentRequestWaitListTables.EwlWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.PtCSchWaitList == null)
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.ConsultWaitList == null)
                Keys.AppointmentRequestWaitListTables.ConsultWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.ApptWaitList == null)
                Keys.AppointmentRequestWaitListTables.ApptWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
            if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests == null)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests = new ObservableCollection<AppointmentRequest>();


            Keys.WaitListCurrentFilter = "";
            Keys.WaitListQueryType = Keys.WaitListQueryTypes.InitialPull;

            var parms = new List<string> { lastRec, Keys.MaxWaitListRecordsToReturn, "R" };
            //Reset Calendar View..
            _eventAggregator.GetEvent<ResetCalendarEvent>().Publish(DateTime.Today);

            var rows = _adt.callOVIDRPC(Keys.RpcCalls.InitialWaitListPullRpc, parms);
            if (rows.Rows.Count == 0)
            {
                Keys.WaitListQueryLastRec = null;
                string message;
                if (string.IsNullOrEmpty(lastRec))
                    message = "No records match the current query.";
                else
                    message = "All records have been displayed.";

                if (messages == null)
                    Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, message)));
                else
                    messages.Add(message);

                Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = true;
                return true;
            }

            if (string.IsNullOrEmpty(lastRec) && Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Count == 0)
            {
                var parm = new List<string>();
                var totalRecs = _adt.CallVistaRpcString(Keys.RpcCalls.InitialWaitListPullTotalCount, parm);
                if (Keys.Methods.IsNumeric(totalRecs))
                    Keys.WaitListQueryTotalRecords = Convert.ToInt32(totalRecs);
                else
                {
                    if (messages == null)
                        Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "Unable to retrieve the total number of records for the query.")));
                    else
                        messages.Add("Unable to retrieve the total number of records for the query.");
                }
            }

            var ctr = 0;
            try
            {
                foreach (DataRow row in rows.Rows)
                {
                    System.Windows.Forms.Application.DoEvents();
                    ctr++;
                    var id = row["IEN"];
                    var type = row["TYPE"].ToString();
                    var key = row["KEY"].ToString();

                    if (ctr == rows.Rows.Count)
                        Keys.WaitListQueryLastRec = key;

                    if (type == "E")
                    {
                        Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
                        var param = new List<string> { id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "WLGET returning too many records.")));
                            else
                                messages.Add("WLGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.EwlWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.EwlWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.EwlWaitList.Rows.Add(rec.ItemArray);
                            var ewlRequests = GetEwlAppointmentRequests();
                            foreach (var eReq in ewlRequests)
                            {
                                if (!string.IsNullOrEmpty(eReq.RequestManagmentIen))
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(eReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                    else if (type == "R")
                    {
                        Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
                        var param = new List<string> { null, null, null, null, null, id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "RECGET returning too many records.")));
                            else
                                messages.Add("RECGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows.Add(rec.ItemArray);
                            var ptCSchRequests = GetPtCSchAppointmentRequests();
                            foreach (var rReq in ptCSchRequests)
                            {
                                if (rReq.PtCSchRequestIen != 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(rReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                    else if (type == "C")
                    {
                        Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
                        var param = new List<string> { null, null, null, null, id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetConsultAppointmentRequests, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "REQGET returning too many records.")));
                            else
                                messages.Add("REQGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.ConsultWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.ConsultWaitList.Rows.Add(rec.ItemArray);
                            var consultRequests = GetConsultAppointmentRequests();
                            foreach (var cReq in consultRequests)
                            {
                                if (cReq.ConsultationRequestIen != 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                    else if (type == "A")
                    {
                        Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
                        var param = new List<string> { id.ToString() };
                        //param.Add("8");
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, param);

                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "ARGET returning too many records.")));
                            else
                                messages.Add("ARGET returning too many records.");
                        }
                        if (Keys.AppointmentRequestWaitListTables.ApptWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.ApptWaitList.Columns.Add(dc.ColumnName, dc.DataType);
                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.ApptWaitList.Rows.Add(rec.ItemArray);
                            var apptRequests = GetApptAppointmentRequests();
                            foreach (var cReq in apptRequests)
                            {
                                if (cReq.ApptRequestIen != 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                }
                Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = true;
            }
            catch (Exception ex)
            {
                if (messages == null)
                    Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, ex.Message, "Get Wait List Error")));
                else
                    messages.Add("Get Wait List Error: " + ex.Message);

                return false;
            }
            return true;
        }

        public bool GetFilteredWaitListRecords(string filters, string lastRec = "", bool isNewQuery = false, string sort = null, List<string> messages = null)
        {
            /*  Request Filters...................
                AppointmentRequestType - These can be EWL, PtCSch, Consult, new Appt Request file
                Patient.Name
                Patient.ServiceRelated
                Patient.PriorityGroup
                DesiredAppointmentDateByType
                OriginatingDateString
                ClinicServiceSpecialtyMerge
            */

            if (Keys.AppointmentRequestWaitListTables.EwlWaitList == null)
                Keys.AppointmentRequestWaitListTables.EwlWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();

            if (Keys.AppointmentRequestWaitListTables.PtCSchWaitList == null)
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();

            if (Keys.AppointmentRequestWaitListTables.ConsultWaitList == null)
                Keys.AppointmentRequestWaitListTables.ConsultWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();

            if (Keys.AppointmentRequestWaitListTables.ApptWaitList == null)
                Keys.AppointmentRequestWaitListTables.ApptWaitList = new DataTable();
            else
                Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();

            if (Keys.AppointmentRequestWaitListTables.AllAppointmentRequests == null)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests = new ObservableCollection<AppointmentRequest>();

            if (isNewQuery)
                Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Clear();

            Keys.WaitListQueryType = Keys.WaitListQueryTypes.ServerQuery;

            Keys.WaitListCurrentFilter = filters;
            var parms = new List<string>
            {
                Keys.MaxWaitListRecordsToReturn,
                lastRec,
                filters,
                sort

            };
            //Reset Calendar View..
            _eventAggregator.GetEvent<ResetCalendarEvent>().Publish(DateTime.Today);

            //WAIT TIME DEFAULT APPOINTMENTS/REQUESTS
            if (!parms[2].ToUpper().Contains("WAITTIME"))
                parms[2] = "WAITTIME^ALL DAYS|" + parms[2];

            var rows = _adt.callOVIDRPC(Keys.RpcCalls.ServerQuery, parms);
            if (rows.Rows.Count == 0)
            {
                Keys.WaitListQueryLastRec = null;
                string message;
                if (string.IsNullOrEmpty(lastRec))
                    message = "No records match the current query.";
                else
                    message = "All records have been displayed.";

                if (messages == null)
                    Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, message)));
                else
                    messages.Add(message);

                Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = true;
                return true;
            }

            var ctr = 0;
            try
            {
                foreach (DataRow row in rows.Rows)
                {
                    System.Windows.Forms.Application.DoEvents();
                    ctr++;
                    if (ctr == 1)
                    {
                        if (Keys.Methods.IsNumeric(row["TOTAL"].ToString()))
                            Keys.WaitListQueryTotalRecords = Convert.ToInt32(row["TOTAL"]);
                        else
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "Missing Total Records from Wait List Query")));
                            else
                                messages.Add("Missing Total Records from Wait List Query");

                            return false;
                        }
                    }
                    var id = row["IEN"];
                    var type = row["TYPE"].ToString();
                    var key = row["KEY"].ToString();
                    if (ctr == rows.Rows.Count)
                        Keys.WaitListQueryLastRec = key;

                    if (type == "E")
                    {
                        Keys.AppointmentRequestWaitListTables.EwlWaitList.Clear();
                        var param = new List<string> { id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "WLGET returning too many records.")));
                            else
                                messages.Add("WLGET returning too many records.");
                        }

                        if (Keys.AppointmentRequestWaitListTables.EwlWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.EwlWaitList.Columns.Add(dc.ColumnName, dc.DataType);

                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.EwlWaitList.Rows.Add(rec.ItemArray);
                            var ewlRequests = GetEwlAppointmentRequests();
                            foreach (var eReq in ewlRequests)
                            {
                                if (!string.IsNullOrEmpty(eReq.RequestManagmentIen))
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(eReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                    else if (type == "R")
                    {
                        Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Clear();
                        var param = new List<string> { null, null, null, null, null, id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "RECGET returning too many records.")));
                            else
                                messages.Add("RECGET returning too many records.");
                        }

                        if (Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Columns.Add(dc.ColumnName, dc.DataType);

                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Rows.Add(rec.ItemArray);
                            var ptCSchRequests = GetPtCSchAppointmentRequests();
                            foreach (var rReq in ptCSchRequests)
                            {
                                if (rReq.PtCSchRequestIen != 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(rReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                    else if (type == "C")
                    {
                        Keys.AppointmentRequestWaitListTables.ConsultWaitList.Clear();
                        var param = new List<string> { null, null, null, null, id.ToString() };
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetConsultAppointmentRequests, param);
                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "REQGET returning too many records.")));
                            else
                                messages.Add("REQGET returning too many records.");
                        }

                        if (Keys.AppointmentRequestWaitListTables.ConsultWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.ConsultWaitList.Columns.Add(dc.ColumnName, dc.DataType);

                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.ConsultWaitList.Rows.Add(rec.ItemArray);
                            var consultRequests = GetConsultAppointmentRequests();
                            foreach (var cReq in consultRequests)
                            {
                                if (cReq.ConsultationRequestIen != 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                    else if (type == "A")
                    {
                        Keys.AppointmentRequestWaitListTables.ApptWaitList.Clear();
                        var param = new List<string> { id.ToString() };
                        //param.Add("8");
                        var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, param);

                        if (newRec.Rows.Count > 1)
                        {
                            if (messages == null)
                                Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, "ARGET returning too many records.")));
                            else
                                messages.Add("ARGET returning too many records.");
                        }

                        if (Keys.AppointmentRequestWaitListTables.ApptWaitList.Columns.Count == 0)
                            foreach (DataColumn dc in newRec.Columns)
                                Keys.AppointmentRequestWaitListTables.ApptWaitList.Columns.Add(dc.ColumnName, dc.DataType);

                        foreach (DataRow rec in newRec.Rows)
                        {
                            Keys.AppointmentRequestWaitListTables.ApptWaitList.Rows.Add(rec.ItemArray);
                            var apptRequests = GetApptAppointmentRequests();
                            foreach (var cReq in apptRequests)
                            {
                                if (cReq.ApptRequestIen != 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(cReq);
                                else
                                    Keys.WaitListQueryTotalRecords--;
                            }
                        }
                    }
                }
                Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded = true;
            }
            catch (Exception ex)
            {
                if (messages == null)
                    Keys.MainWindow.Dispatcher.Invoke((Action)(() => MessageBox.Show(Keys.MainWindow, ex.Message, "Filtered Wait List Error")));
                else
                    messages.Add("Filtered Wait List Error: " + ex.Message);

                return false;
            }
            return true;
        }

        public void GetFilteredData(string lastRec, bool isNewQuery, bool isInThread = false, List<string> messages = null)
        {
            Cursor oldCursor = null;

            try
            {
                Keys.WaitListQueryTotalRecords = 0;
                var filters = "";

                if (Keys.UserPreferences.WaitTime.IsFiltered)
                {
                    var waitTime = "";
                    if (string.IsNullOrEmpty(Keys.UserPreferences.WaitTime.Filters[0]))
                        waitTime = Keys.RmGridColumns.WaitTimeNull;
                    else
                    {
                        if (Keys.UserPreferences.WaitTime.Filters[0].ToUpper().Contains("ALL DAYS"))
                            waitTime = "ALL DAYS";
                        else if (Keys.UserPreferences.WaitTime.Filters[0].Contains("<30"))
                            waitTime = "<30";
                        else if (Keys.UserPreferences.WaitTime.Filters[0].Contains("30-60"))
                            waitTime = "30-60";
                        else if (Keys.UserPreferences.WaitTime.Filters[0].Contains("60-90"))
                            waitTime = "60-90";
                        else if (Keys.UserPreferences.WaitTime.Filters[0].Contains(">=90"))
                            waitTime = ">=90";
                    }

                    if (!string.IsNullOrEmpty(waitTime))
                        filters = "WAITTIME^" + waitTime + "|";
                }
                //else
                //{
                //    filters = "WAITTIME^" + ">=90|";
                //}


                //if ( ViewModel.SelectedPatients.Count > 0 )
                //{
                //    filters += "PATIENTNAME^";
                //    foreach ( Patient patient in ViewModel.SelectedPatients )
                //        filters += patient.IEN + "~";
                //    filters = filters.TrimEnd( '~' );
                //    filters += "|";
                //}

                if (Keys.UserPreferences.RequestType.IsFiltered)
                {
                    if (Keys.UserPreferences.RequestType.Filters.Count > 0 && Keys.UserPreferences.RequestType.Filters.Count < 6)
                    {
                        filters += "REQUESTTYPE^";
                        foreach (var filter in Keys.UserPreferences.RequestType.Filters)
                            if (filter.ToUpper() == "PTCSCH")
                                filters += "R~";
                            else
                                filters += filter.Substring(0, 1) + "~";
                        filters = filters.TrimEnd('~');
                        filters += "|";
                    }
                }

                if (Keys.UserPreferences.PriorityGroup.IsFiltered)
                {
                    if (Keys.UserPreferences.PriorityGroup.Filters.Count > 0)
                    {
                        filters += "PRIORITYGROUPS^";
                        foreach (var filter in Keys.UserPreferences.PriorityGroup.Filters)
                            filters += filter + "~";
                        filters = filters.TrimEnd('~');
                        filters += "|";
                    }
                }


                if (Keys.UserPreferences.SCVisit.IsFiltered)
                {
                    filters += "SERVICERELATED^" + Keys.UserPreferences.SCVisit.Filters[0].ToUpper();
                    filters += "|";
                    filters += "SCVISIT^" + Keys.UserPreferences.SCVisit.Filters[0].ToUpper();
                    filters += "|";
                }

                if (Keys.UserPreferences.ClinicService.IsFiltered)
                {
                    if (Keys.UserPreferences.ClinicService.Filters.Count > 0)
                    {
                        if (Keys.UserPreferences.ClinicService.IsServiceList)
                            filters += "SERVICES^";
                        else
                            filters += "CLINICS^";
                        foreach (var clin in Keys.UserPreferences.ClinicService.Filters)
                        {
                            string ien = clin;

                            if (clin.Contains("="))
                                ien = clin.Piece("=", 2);

                            filters += ien + "~";
                        }
                        filters = filters.TrimEnd('~');
                        filters += "|";
                    }
                }

                if (Keys.UserPreferences.EnteredDate.IsFiltered)
                {
                    if (!string.IsNullOrEmpty(Keys.UserPreferences.EnteredDate.Filters[1]))
                    {
                        filters += "ORIGINATIONDATERANGE^" + Keys.UserPreferences.EnteredDate.Filters[1];
                        filters += "~";
                    }
                    if (!string.IsNullOrEmpty(Keys.UserPreferences.EnteredDate.Filters[4]))
                    {
                        filters += Keys.UserPreferences.EnteredDate.Filters[4];
                        filters += "|";
                    }
                }

                if (Keys.UserPreferences.PreferredDate.IsFiltered)
                {
                    if (!string.IsNullOrEmpty(Keys.UserPreferences.PreferredDate.Filters[1]))
                    {
                        filters += "DESIREDDATERANGE^" + Keys.UserPreferences.PreferredDate.Filters[1];
                        filters += "~";
                    }
                    if (!string.IsNullOrEmpty(Keys.UserPreferences.PreferredDate.Filters[4]))
                    {
                        filters += Keys.UserPreferences.PreferredDate.Filters[4];
                        filters += "|";
                    }
                }

                //if ( !string.IsNullOrEmpty( ConsultUrgency.Text ) )
                //{
                //    filters += "URGENCY^" + ConsultUrgency.SelectedValue;
                //    filters += "|";
                //}

                // Find the Sort order
                var sortOrder = "";
                for (var i = 1; i <= 3; i++)
                {
                    if (Keys.UserPreferences.RequestType.SortOrder == i)
                        sortOrder += "REQUESTTYPE|";
                    if (Keys.UserPreferences.WaitTime.SortOrder == i)
                        sortOrder += "WAITTIME|";
                    if (Keys.UserPreferences.PatientName.SortOrder == i)
                        sortOrder += "PATIENTNAME|";
                    if (Keys.UserPreferences.SCVisit.SortOrder == i)
                        sortOrder += "SCVISIT|";
                    if (Keys.UserPreferences.PriorityGroup.SortOrder == i)
                        sortOrder += "PRIORITYGROUPS|";
                    if (Keys.UserPreferences.EnteredDate.SortOrder == i)
                        sortOrder += "ORIGINATIONDATE|";
                    if (Keys.UserPreferences.PreferredDate.SortOrder == i)
                        sortOrder += "DESIREDDATE|";
                    if (Keys.UserPreferences.ClinicService.SortOrder == i)
                        sortOrder += "CLINICS|";
                    if (Keys.UserPreferences.SSN.SortOrder == i)
                        sortOrder += "SSN|";
                    if (Keys.UserPreferences.Telephone.SortOrder == i)
                        sortOrder += "TELEPHONE|";
                    if (Keys.UserPreferences.Requestor.SortOrder == i)
                        sortOrder += "REQUESTOR|";
                    if (Keys.UserPreferences.RequestedBy.SortOrder == i)
                        sortOrder += "REQUESTEDBY|";
                    if (Keys.UserPreferences.Comment.SortOrder == i)
                        sortOrder += "COMMENT|";
                    //if ( Keys.UserPreferences.MRTC.SortOrder == i )
                    //    sortOrder += "MRTC|";
                }

                var sort = "";
                if (sortOrder.Length > 0)
                {
                    //filters += "SORT^" + sortOrder;
                    sort += sortOrder;
                    sort = sort.TrimEnd('|');
                }
                else
                    sort = "SORT^DEFAULT";

                filters = filters.TrimEnd('|');
                GetFilteredWaitListRecords(filters, lastRec, isNewQuery, sort, messages);

                //IsAsync = false;
                //DoWorkAsync(
                //    ( s, args ) =>
                //    {
                //        args.Result = GetFilteredWaitListRecords( filters, lastRec, true, sort );
                //    },
                //    ( s, args ) =>
                //    {
                //        lock ( args )
                //        {
                //            Mouse.OverrideCursor = Cursors.Arrow;
                //            if ( !(bool)args.Result )
                //                return;
                //        }
                //    } );
            }
            catch (Exception ex)
            {
                Exception newException = new Exception("There was a problem reading the Filter Data. see the trace Log for details.\n", ex);
                TraceLog.LogException(GetType().Name, "GetFilterData", newException);

                if (isInThread)
                    throw newException;
                else
                    Keys.AlertUserError(null, newException, "GetFilterData Error");
            }
        }

        public List<AppointmentRequest> GetEwlAppointmentRequests(bool isInThread = false)
        {
            return GetEwlAppointmentRequests(Keys.AppointmentRequestWaitListTables.EwlWaitList.Copy(), isInThread);
        }

        private readonly object _getEwlAppointmentRequestsLock = new object();

        public List<AppointmentRequest> GetEwlAppointmentRequests(DataTable dt, bool isInThread = false)
        {
            if (dt == null)
                return new List<AppointmentRequest>();

            lock (_getEwlAppointmentRequestsLock)
            {
                var patientEwlRequests = new List<AppointmentRequest>();
                var waitList = dt;
                try
                {
                    foreach (DataRow wt in waitList.Rows)
                    {
                        var newRequest = new AppointmentRequest(this)
                        {
                            Patient = new Patient(this)
                            {
                                IEN = wt["DFN"].ToString()
                            }
                        };
                        var sensitive = wt["SENSITIVE"].ToString();
                        if (string.IsNullOrEmpty(sensitive) || sensitive == "0")
                            newRequest.Patient.IsSensitiveRecord = false;
                        else
                            newRequest.Patient.IsSensitiveRecord = true;

                        //newRequest.Patient.IsSensitiveRecord = sensitive == "1"; //This does not work in VA environments... logic above should work for all environments.  bj_2_17_15;
                        newRequest.RequestManagmentIen = wt["IEN"].ToString();
                        newRequest.Patient.Name = wt["NAME"].ToString();
                        newRequest.Patient.HRN = wt["HRN"].ToString();
                        newRequest.Patient.SSN = wt["SSN"].ToString();
                        newRequest.Patient.DOB = wt["DOB"].ToString();
                        newRequest.Patient.Gender = wt["GENDER"].ToString();
                        newRequest.Patient.OriginatingUser = wt["USERIEN"].ToString();
                        //"User, Demo"; // row["USER"].ToString();
                        newRequest.Patient.OriginatingUserName = wt["USERNAME"].ToString();
                        var origDate = wt["ORIGDT"].ToString();
                        newRequest.OriginatingDate = ConvertOrigDate(origDate);
                        newRequest.Patient.InstitutionIen = wt["INSTIEN"].ToString();
                        newRequest.Patient.Institution = wt["INSTNAME"].ToString();
                        newRequest.WaitListType = wt["TYPE"].ToString();
                        newRequest.Priority = wt["PRIO"].ToString();

                        if (!String.IsNullOrEmpty(wt["REQBY"].ToString()))
                            newRequest.RequestedBy = newRequest.RequestedByValues.FirstOrDefault(rb => (rb.Value == wt["REQBY"].ToString()));

                        newRequest.Patient.Provider = new Provider
                        {
                            IEN = wt["PROVIEN"].ToString(),
                            Name = wt["PROVNAME"].ToString()
                        };
                        newRequest.Provider = newRequest.Patient.Provider;
                        newRequest.EnrolleeStatus = wt["EESTAT"].ToString();
                        newRequest.ClinName = wt["CLINNAME"].ToString();
                        newRequest.Specialty = wt["SRVSPNAME"].ToString();
                        newRequest.ClinicIen = wt["CLINIEN"].ToString();
                        newRequest.SpecialtyIen = wt["SRVSPIEN"].ToString();
                        newRequest.RequestManagmentComment = wt["COMM"].ToString();
                        var desiredApptDate = wt["DAPTDT"].ToString();
                        newRequest.DesiredApptDate = ConvertOrigDate(desiredApptDate);
                        if (string.IsNullOrEmpty(newRequest.ClinName))
                            newRequest.ClinicServiceSpecialtyMerge = newRequest.Specialty;
                        else if (string.IsNullOrEmpty(newRequest.Specialty))
                            newRequest.ClinicServiceSpecialtyMerge = newRequest.ClinName;
                        newRequest.Patient.PrimaryContactInfo.Phone = wt["PTELEPHONE"].ToString();
                        newRequest.Patient.PrimaryContactInfo.PhoneResidence = wt["PTELEPHONE"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Country = newRequest.Patient.Countries.FirstOrDefault(c => c.Ien == wt["PCOUNTRY"].ToString());
                        newRequest.EnrollmentPriority = wt["ENROLLMENT_PRIORITY"].ToString();
                        newRequest.AppointmentScheduledDate = wt["APPT_SCHED_DATE"].ToString();

                        var multipleAppointmentsRequired = wt["MULTIPLE APPOINTMENT RTC"];
                        newRequest.MultipleAppointmentsRequired = multipleAppointmentsRequired.ToString() ==
                                                                  "YES";
                        var multApptNumber = wt["MULT APPT NUMBER"];
                        newRequest.NumberOfAppointmentsRequired = string.IsNullOrEmpty(multApptNumber.ToString())
                            ? 0
                            : Convert.ToInt32(wt["MULT APPT NUMBER"]);

                        var apptIntervalIndays = wt["MULT APPT RTC INTERVAL"];
                        newRequest.AppointmentIntervalInDays =
                            string.IsNullOrEmpty(apptIntervalIndays.ToString())
                                ? 0
                                : Convert.ToInt32(wt["MULT APPT RTC INTERVAL"]);
                        newRequest.AppointmentRequestType = "EWL";
                        newRequest.Patient.PriorityGroup = wt["PRIGRP"].ToString();
                        newRequest.Patient.EligibilityIen = wt["ELIGIEN"].ToString();
                        newRequest.Patient.EligibilityName = wt["ELIGNAME"].ToString();
                        newRequest.Patient.ServiceConnection = wt["SVCCONN"].ToString();
                        newRequest.Patient.ServiceConnectionPercentage = wt["SVCCONNP"].ToString();
                        newRequest.Patient.ServiceRelated = newRequest.Patient.ServiceConnectionPercent >= 50;
                        newRequest.Patient.SCVisit = wt["WLSVCCON"].ToString();
                        newRequest.Patient.TypeIen = wt["TYPEIEN"].ToString();
                        newRequest.Patient.TypeName = wt["TYPENAME"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street1 = wt["PADDRES1"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street2 = wt["PADDRES2"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street3 = wt["PADDRES3"].ToString();
                        newRequest.Patient.PrimaryContactInfo.City = wt["PCITY"].ToString();
                        newRequest.Patient.PrimaryContactInfo.ZipPlus4 = wt["PZIP4"].ToString();
                        var apptTypeIen = wt["APPTYPE"].ToString();
                        if (Keys.Methods.IsNumeric(apptTypeIen))
                        {
                            foreach (var apptType in AppointmentTypes(newRequest.Patient.IEN).Where(apptType => apptType.Ien == apptTypeIen))
                            {
                                newRequest.AppointmentType = apptType.Name;
                                break;
                            }
                        }

                        var dateentered = wt["DATE"].ToString();
                        if (!string.IsNullOrEmpty(dateentered))
                        {
                            dateentered = dateentered.Replace('@', ' ');
                            newRequest.DateEntered = Convert.ToDateTime(dateentered);
                        }

                        newRequest.Patient.GAF.Text = wt["GAF"].ToString();
                        var prohibitedClin = wt["PRHBLOC"].ToString();
                        if (Keys.Methods.IsNumeric(prohibitedClin))
                            newRequest.IsProhibitedClinic = Convert.ToInt32(wt["PRHBLOC"]) == 1;
                        else
                            newRequest.IsProhibitedClinic = false;

                        #region BJ'S PATIENT CONTACT REPLACEMENT CODE
                        //newRequest.PatientContacts = getPatientContact(newRequest.Patient.IEN, 
                        //                                                newRequest.ClinicIen, 
                        //                                                ExtensionMethods.ConvertToFilemanDateTime(
                        //                                                    Convert.ToDateTime(newRequest.DesiredApptDate)),
                        //                                                newRequest.AppointmentRequestType); 
                        #endregion

                        #region HPE OLD PATIENT CONTACT LOGIC (COMMENTED OUT)
                        //var pContact = wt["PCONTACT"].ToString();
                        //if (!string.IsNullOrEmpty(pContact))
                        //{
                        //    newRequest.PatientContacts = new List<PatientContact>();
                        //    var records = pContact.Split(new[] {"::"}, StringSplitOptions.None);
                        //    foreach (var record in records)
                        //    {
                        //        var entries = record.Split(new[] {"~~"}, StringSplitOptions.None);
                        //        var patientContact = new PatientContact
                        //        {
                        //            DateEntered = entries[0],
                        //            PCEnteredByUserIen = entries[1],
                        //            PCEnteredByUserName = entries[2],
                        //            Action = entries[3].ToUpper(),
                        //            PatientPhone = entries[4]
                        //        };
                        //        newRequest.PatientContacts.Add(patientContact);
                        //    }
                        //}
                        #endregion

                        patientEwlRequests.Add(newRequest);
                    }
                }
                catch (Exception ex)
                {
                    if (isInThread)
                        throw;
                    else
                        MessageBox.Show(ex.Message, "Get EWL Requests Error");
                }

                return patientEwlRequests;
            }
        }

        public IList<AppointmentType> AppointmentTypes(string patientIEN)
        {
            return GetAppointmentTypes(patientIEN);
        }

        public AppointmentRequest GetEwlApptRequest(string requestManagementIen)
        {
            try
            {
                var parm = new List<string> { requestManagementIen };
                var waitList = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, parm);
                if (waitList == null)
                    return new AppointmentRequest(this);

                var wt = waitList.Rows[0];

                var newRequest = new AppointmentRequest(this)
                {
                    Patient = new Patient(this)
                    {
                        IEN = wt["DFN"].ToString(),
                        Name = wt["NAME"].ToString(),
                        HRN = wt["HRN"].ToString(),
                        SSN = wt["SSN"].ToString(),
                        DOB = wt["DOB"].ToString(),
                        Gender = wt["GENDER"].ToString(),
                        OriginatingUser = wt["USERIEN"].ToString(),
                        OriginatingUserName = wt["USERNAME"].ToString(),
                        InstitutionIen = wt["INSTIEN"].ToString(),
                        Institution = wt["INSTNAME"].ToString(),
                        PriorityGroup = wt["PRIGRP"].ToString(),
                        EligibilityIen = wt["ELIGIEN"].ToString(),
                        EligibilityName = wt["ELIGNAME"].ToString(),
                        ServiceConnection = wt["SVCCONN"].ToString(),
                        ServiceConnectionPercentage = wt["SVCCONNP"].ToString(),
                        SCVisit = wt["WLSVCCON"].ToString(),
                        TypeIen = wt["TYPEIEN"].ToString(),
                        TypeName = wt["TYPENAME"].ToString(),
                        Provider = new Provider
                        {
                            IEN = wt["PROVIEN"].ToString(),
                            Name = wt["PROVNAME"].ToString()
                        }
                    },
                    RequestManagmentIen = wt["IEN"].ToString(),
                    WaitListType = wt["TYPE"].ToString(),
                    Priority = wt["PRIO"].ToString(),
                    EnrolleeStatus = wt["EESTAT"].ToString(),
                    ClinName = wt["CLINNAME"].ToString(),
                    Specialty = wt["SRVSPNAME"].ToString(),
                    ClinicIen = wt["CLINIEN"].ToString(),
                    SpecialtyIen = wt["SRVSPIEN"].ToString(),
                    RequestManagmentComment = wt["COMM"].ToString(),
                    AppointmentRequestType = "EWL",
                    EnrollmentPriority = wt["ENROLLMENT_PRIORITY"].ToString(),
                    AppointmentScheduledDate = wt["APPT_SCHED_DATE"].ToString()
                };

                if (!String.IsNullOrEmpty(wt["REQBY"].ToString()))
                    newRequest.RequestedBy = newRequest.RequestedByValues.FirstOrDefault(rb => (rb.Value == wt["REQBY"].ToString()));

                newRequest.Provider = newRequest.Patient.Provider;
                var desiredApptDate = wt["DAPTDT"].ToString();
                newRequest.DesiredApptDate = ConvertOrigDate(desiredApptDate);

                if (string.IsNullOrEmpty(newRequest.ClinName))
                    newRequest.ClinicServiceSpecialtyMerge = newRequest.Specialty;
                else if (string.IsNullOrEmpty(newRequest.Specialty))
                    newRequest.ClinicServiceSpecialtyMerge = newRequest.ClinName;

                var multipleAppointmentsRequired = wt["MULTIPLE APPOINTMENT RTC"];
                newRequest.MultipleAppointmentsRequired = multipleAppointmentsRequired.ToString() == "YES";

                var multApptNumber = wt["MULT APPT NUMBER"];
                newRequest.NumberOfAppointmentsRequired = string.IsNullOrEmpty(multApptNumber.ToString())
                    ? 0
                    : Convert.ToInt32(wt["MULT APPT NUMBER"]);

                var apptIntervalIndays = wt["MULT APPT RTC INTERVAL"];

                newRequest.AppointmentIntervalInDays = string.IsNullOrEmpty(apptIntervalIndays.ToString())
                    ? 0
                    : Convert.ToInt32(wt["MULT APPT RTC INTERVAL"]);

                newRequest.Patient.ServiceRelated = newRequest.Patient.ServiceConnectionPercent >= 50;

                newRequest.Patient.PrimaryContactInfo.Phone = wt["PTELEPHONE"].ToString();
                newRequest.Patient.PrimaryContactInfo.PhoneResidence = wt["PTELEPHONE"].ToString();
                newRequest.Patient.PrimaryContactInfo.Country = newRequest.Patient.Countries.FirstOrDefault(c => c.Ien == wt["PCOUNTRY"].ToString());
                newRequest.Patient.PrimaryContactInfo.Street1 = wt["PADDRES1"].ToString();
                newRequest.Patient.PrimaryContactInfo.Street2 = wt["PADDRES2"].ToString();
                newRequest.Patient.PrimaryContactInfo.Street3 = wt["PADDRES3"].ToString();
                newRequest.Patient.PrimaryContactInfo.City = wt["PCITY"].ToString();
                newRequest.Patient.PrimaryContactInfo.ZipPlus4 = wt["PZIP4"].ToString();

                var origDate = wt["ORIGDT"].ToString();
                newRequest.OriginatingDate = ConvertOrigDate(origDate);

                var sensitive = wt["SENSITIVE"].ToString();
                if (string.IsNullOrEmpty(sensitive) || sensitive == "0")
                    newRequest.Patient.IsSensitiveRecord = false;
                else
                    newRequest.Patient.IsSensitiveRecord = true;

                //newRequest.Patient.IsSensitiveRecord = sensitive == "1"; //This does not work in VA environments... logic above should work for all environments.  bj_2_17_15;

                var apptTypeIen = wt["APPTYPE"].ToString();
                if (Keys.Methods.IsNumeric(apptTypeIen))
                {
                    foreach (var apptType in AppointmentTypes(newRequest.Patient.IEN).Where(apptType => apptType.Ien == apptTypeIen))
                    {
                        newRequest.AppointmentType = apptType.Name;
                        break;
                    }
                }

                var dateEntered = wt["DATE"].ToString();
                if (!string.IsNullOrEmpty(dateEntered))
                {
                    dateEntered = dateEntered.Replace('@', ' ');
                    newRequest.DateEntered = Convert.ToDateTime(dateEntered);
                }

                newRequest.Patient.GAF.Text = wt["GAF"].ToString();

                var prohibitedClin = wt["PRHBLOC"].ToString();
                if (Keys.Methods.IsNumeric(prohibitedClin))
                    newRequest.IsProhibitedClinic = Convert.ToInt32(wt["PRHBLOC"]) == 1;
                else
                    newRequest.IsProhibitedClinic = false;

                //var pContact = wt["PCONTACT"].ToString();
                //if ( !string.IsNullOrEmpty( pContact ) )
                //{
                //    newRequest.PatientContacts = new List<PatientContact>();
                //    var records = pContact.Split( new[] {"::"}, StringSplitOptions.None );
                //    foreach ( var record in records )
                //    {
                //        var entries = record.Split( new[] {"~~"}, StringSplitOptions.None );
                //        var patientContact = new PatientContact
                //        {
                //            DateEntered = entries[0],
                //            PCEnteredByUserIen = entries[1],
                //            PCEnteredByUserName = entries[2],
                //            Action = entries[3].ToUpper(),
                //            PatientPhone = entries[4]
                //        };
                //        newRequest.PatientContacts.Add( patientContact );
                //    }
                //}
                return newRequest;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Get EWL Requests Error");
            }
            return new AppointmentRequest(this);
        }

        public IList<AppointmentRequest> GetPtCSchAppointmentRequests(bool isInThread = false, List<string> messages = null)
        {
            return GetPtCSchAppointmentRequests(Keys.AppointmentRequestWaitListTables.PtCSchWaitList.Copy(), isInThread, messages);
        }

        private readonly object _getPtCSchAppointmentRequests = new object();

        public IList<AppointmentRequest> GetPtCSchAppointmentRequests(DataTable dt, bool isInThread = false, List<string> messages = null)
        {
            if (dt == null)
                return new List<AppointmentRequest>();

            lock (_getPtCSchAppointmentRequests)
            {
                IList<AppointmentRequest> ptCSchList = new List<AppointmentRequest>();
                var apptAppointmentRequests = dt;
                var errorCount = 0;

                foreach (DataRow wt in apptAppointmentRequests.Rows)
                {
                    try
                    {
                        var newRequest = new AppointmentRequest(this)
                        {
                            PtCSchRequestIen = Convert.ToInt32(wt["IEN"]),
                            Patient = new Patient(this)
                            {
                                IEN = wt["DFN"].ToString(),
                                Name = wt["NAME"].ToString(),
                                HRN = wt["HRN"].ToString(),
                                SSN = wt["SSN"].ToString(),
                                DOB = wt["DOB"].ToString(),
                                Gender = wt["GENDER"].ToString(),
                                InstitutionIen = wt["INSTIEN"].ToString(),
                                Institution = wt["INSTNAME"].ToString(),

                                Provider = new Provider
                                {
                                    IEN = wt["RRPROVIEN"].ToString(),
                                    Name = wt["PROVNAME"].ToString()
                                },

                                OriginatingUser = wt["USERIEN"].ToString(),
                                OriginatingUserName = wt["USERNAME"].ToString(),
                                PriorityGroup = wt["PRIGRP"].ToString(),
                                EligibilityIen = wt["ELIGIEN"].ToString(),
                                EligibilityName = wt["ELIGNAME"].ToString(),
                                ServiceConnection = wt["SVCCONN"].ToString(),
                                ServiceConnectionPercentage = wt["SVCCONNP"].ToString(),
                                TypeIen = wt["TYPEIEN"].ToString(),
                                TypeName = wt["TYPENAME"].ToString(),
                            },

                            RequestManagmentIen = wt["IEN"].ToString(),
                            Accession = wt["ACCESION"].ToString(),
                            Fasting = wt["FASTING"].ToString(),
                            ClinicIen = wt["CLINIEN"].ToString(),
                            ClinName = wt["CLINNAME"].ToString(),
                            AppointmentLength = wt["APPTLEN"].ToString(),
                            PtCSchDateReminder = wt["DAPTDT"].ToString(),
                            PtCSchSecondPrintDate = wt["DATE2"].ToString(),
                            AppointmentRequestType = Keys.AppointmentRequestTypes.PtCSch,
                            RequestedByText = "PROVIDER",
                            RequestManagmentComment = wt["COMM"].ToString(),
                            PtCSchComment = wt["COMM"].ToString(),
                            OriginatingDateString = wt["DATE3"].ToString()
                        };

                        newRequest.RequestedBy = newRequest.RequestedByValues.FirstOrDefault(rb => (rb.Value == "1"));
                        var ptCSchDate = wt["DATE"].ToString();
                        newRequest.PtCSchDate = string.IsNullOrEmpty(ptCSchDate) ? (DateTime?)null : Convert.ToDateTime(ptCSchDate);

                        var ptCSchDatePerPatient = wt["DATE1"].ToString();
                        newRequest.PtCSchDatePerPatient = string.IsNullOrEmpty(ptCSchDatePerPatient) ? (DateTime?)null : Convert.ToDateTime(ptCSchDatePerPatient);

                        newRequest.SelectedPtCSchAppointmentType = newRequest.PtCSchApptTypes.FirstOrDefault(t => t.Ien == wt["RRAPPTYP"].ToString());
                        newRequest.Patient.PrimaryContactInfo.Phone = wt["PTPHONE"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street1 = wt["PADDRES1"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street2 = wt["PADDRES2"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street3 = wt["PADDRES3"].ToString();
                        newRequest.Patient.PrimaryContactInfo.City = wt["PCITY"].ToString();
                        newRequest.Patient.PrimaryContactInfo.ZipPlus4 = wt["PZIP4"].ToString();
                        newRequest.Patient.GAF.Text = wt["GAF"].ToString();

                        newRequest.DesiredApptDate = Convert.ToDateTime(newRequest.PtCSchDate);

                        var sensitive = wt["SENSITIVE"].ToString();
                        if (string.IsNullOrEmpty(sensitive) || sensitive == "0")
                            newRequest.Patient.IsSensitiveRecord = false;
                        else
                            newRequest.Patient.IsSensitiveRecord = true;

                        newRequest.Patient.ServiceRelated = newRequest.Patient.ServiceConnectionPercent >= 50;

                        if (newRequest.Patient.ServiceConnection == "YES" && newRequest.Patient.ServiceConnectionPercent >= 50)
                            newRequest.Patient.SCVisit = "YES";
                        else
                            newRequest.Patient.SCVisit = "NO";

                        newRequest.RequestedByProviderMerge = newRequest.Patient.Provider.Name;

                        if (!string.IsNullOrEmpty(newRequest.OriginatingDateString))
                        {
                            var sDate = newRequest.OriginatingDateString.Split('@')[0];
                            newRequest.OriginatingDate = Convert.ToDateTime(sDate);
                            // Convert.ToDateTime(newRequest.OriginatingDateString);
                        }
                        if (newRequest.RequestManagmentComment.Length > 0)
                        {
                            if (newRequest.AppointmentLength.Length > 0)
                                newRequest.RequestManagmentComment += ", " + newRequest.AppointmentLength;
                            if (newRequest.PtCSchDateReminder.Length > 0)
                                newRequest.RequestManagmentComment += ", " + newRequest.PtCSchDateReminder;
                        }
                        else
                        {
                            newRequest.RequestManagmentComment = newRequest.AppointmentLength;
                            if (newRequest.RequestManagmentComment.Length > 0)
                                newRequest.RequestManagmentComment += ", " + newRequest.PtCSchDateReminder;
                            else
                                newRequest.RequestManagmentComment = newRequest.PtCSchDateReminder;
                        }
                        newRequest.Patient.PrimaryContactInfo.Country = newRequest.Patient.Countries.FirstOrDefault(c => c.Ien == wt["PCOUNTRY"].ToString());

                        var dateentered = wt["DATE"].ToString();
                        if (!string.IsNullOrEmpty(dateentered))
                        {
                            dateentered = dateentered.Replace('@', ' ');
                            newRequest.DateEntered = Convert.ToDateTime(dateentered);
                        }
                        var prohibitedClin = wt["PRHBLOC"].ToString();
                        if (Keys.Methods.IsNumeric(prohibitedClin))
                            newRequest.IsProhibitedClinic = Convert.ToInt32(wt["PRHBLOC"]) == 1;
                        else
                            newRequest.IsProhibitedClinic = false;
                        if (string.IsNullOrEmpty(newRequest.EnrollmentPriority))
                            newRequest.EnrollmentPriority = newRequest.Patient.PriorityGroup;
                        ptCSchList.Add(newRequest);
                    }
                    catch (Exception ex)
                    {
                        TraceLog.LogException(GetType().Name, "GetPtCSchAppointmentRequests", ex);
                        errorCount++;
                    }
                }

                if (errorCount > 0)
                {
                    if (isInThread)
                        messages.Add("There were errors processing the PtCSch Request List.\nSee the Trace Log for details.");
                    else
                        Keys.AlertUserError(null, new Exception("There were errors processing the PtCSch Request List.\nSee the Trace Log for details."), "PtCSch Request Processing Errors");
                }

                return ptCSchList;
            }
        }

        public AppointmentRequest GetPtCSchApptRequest(string ptCSchRequestIen)
        {
            try
            {
                var param = new List<string> { null, null, null, null, null, ptCSchRequestIen };
                var ptCSchAppointmentRequests = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, param);

                if (ptCSchAppointmentRequests.Rows.Count == 0)
                    return null;

                if (ptCSchAppointmentRequests.Rows.Count > 1)
                {
                    MessageBox.Show("RECGET returning too many records.");
                    return new AppointmentRequest(this);
                }

                var list = GetPtCSchAppointmentRequests(ptCSchAppointmentRequests);
                return list[0];
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "GetPtCSchApptRequest", ex);
            }
            return new AppointmentRequest(this);
        }

        public DataTable GetPatientSpecialNeedsPreferences(string patientIen)
        {
            if (Keys.Tables.PatientPrefs == null || patientIen != Keys.Tables.PatientPrefsIEN || DateTime.Now > Keys.Tables.PatientPrefsLoaded.AddMinutes(15))
            {
                var parms = new List<string> { patientIen, "0" };
                Keys.Tables.PatientPrefs = _adt.callOVIDRPC(Keys.RpcCalls.SpecialNeedsRpcs.SpecialNeedsPreferencesGet, parms);
                Keys.Tables.PatientPrefsIEN = patientIen;
                Keys.Tables.PatientPrefsLoaded = DateTime.Now;
            }

            return Keys.Tables.PatientPrefs;
        }

        public void GetPatientSpecialNeedsPreferences(string patientIEN, ref SpecialNeedsPreferences prefs)
        {
            var table = GetPatientSpecialNeedsPreferences(patientIEN);

            if (prefs == null)
                prefs = new SpecialNeedsPreferences();

            if (table.Rows.Count == 0)
                return;

            if (table.Columns.Contains("RETURNCODE"))
            {
                var row = table.Rows[0];
                Keys.AlertUserError(null, new Exception("Error reading patient special needs/prederences:\n" + row["TEXT"]), "Special Needs/Preferences Error");
                return;
            }

            foreach (DataRow dr in table.Rows)
                prefs.SetPreference(dr["PREF_TEXT"].ToString(), dr["REMARKS"].ToString());
        }

        public bool SetPatientSpecialNeedsPreferences(string patientIEN, SpecialNeedsPreferences prefs)
        {
            var parms = new List<string>
            {
                patientIEN,
                prefs.ToPREF_TEXT(),
                prefs.RemarksParam
            };

            try
            {
                var rslt = _adt.callOVIDRPC(Keys.RpcCalls.SpecialNeedsRpcs.SpecialNeedsPreferencesSet, parms);
                Keys.Tables.PatientPrefs = null;
                return true;
            }
            catch (Exception ex)
            {
                Keys.AlertUserError(null, ex, "Error saving Special Needs and Preferences");
            }
            return false;
        }

        public List<string> GetUserPreferences(string userIen)
        {
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.UserPreferencesCache);

            string prefsString;

            if (Keys.Tables.UserPreferencesCache.ContainsKey(userIen))
                prefsString = Keys.Tables.UserPreferencesCache[userIen].SingleValue;
            else
            {
                var parms = new List<string> { userIen };
                prefsString = _adt.CallVistaRpcString(Keys.RpcCalls.UserPreferencesGet, parms);

                if (prefsString.Contains("-1") && prefsString.Contains("NO USER PREFERENCE SET"))
                    prefsString = string.Empty;

                if (!string.IsNullOrEmpty(prefsString))
                    Keys.Tables.UserPreferencesCache.Add(userIen, new DataTableCache(prefsString, 60));
            }

            if (string.IsNullOrEmpty(prefsString))
            {
                return new List<string>()
                {
                    "C; 0; CLINIC/SERVICE; : F; 0; ; FALSE; : S; 0;",
                    "C; 1; REQUEST TYPE; : F; 0; ; FALSE; : S; 0;",
                    "C; 2; WAIT TIME; : F; 0; ; FALSE; : S; 0;",
                    "C; 3; PT NAME; : F; 0; ; FALSE; : S; 0;",
                    "C; 4; MRTC; : F; 0; ; FALSE; : S; 0;",
                    "C; 5; SSN; : F; 0; ; FALSE; : S; 0;",
                    "C; 6; SCVISIT; : F; 0; ; FALSE; : S; 0;",
                    "C; 7; TELEPHONE; : F; 0; ; FALSE; : S; 0;",
                    "C; 8; PRIORITY; :F; 0; ; FALSE; : S; 0;",
                    "C; 9; CID/PREFERRED DATE; : F; 0; ; FALSE; : S; 0;",
                    "C; 10; ENTERED/RR NO DATE; : F; 0; ; FALSE; : S; 0;",
                    "C; 11; REQUESTOR; : F; 0; ; FALSE; : S; 0;",
                    "C; 12; REQUESTED BY; : F; 0; ; FALSE; : S; 0;",
                    "C; 13; COMMENT; : F; 0; ; FALSE; : S; 0;"
                };
            }
            else
                return prefsString.Split('|').ToList();
        }

        public bool SetUserPreference(string userIen, List<string> prefs)
        {
            // Build the list of User Preferences to send
            List<string> rpcArraySource = new List<string>();

            // Build on long string 
            string up = string.Join("|", prefs.ToArray());

            //foreach ( var userPreference in prefs ) // 13 elements of the Columns of the RM Grid
            //    up += userPreference + "|";

            //up = up.TrimEnd( '|' );

            // divide the user preference ( up ) string into strings of a length of 75
            List<string> subStrings = Keys.Methods.SplitByLength(up, 75).ToList();
            // Add the User IEN to the front of the array
            subStrings.Insert(0, userIen);

            try
            {
                var rslt = _adt.CallVistaRpcString(Keys.RpcCalls.UserPreferencesSet, subStrings);

                if (Keys.Tables.UserPreferencesCache.ContainsKey(userIen))
                    Keys.Tables.UserPreferencesCache.Remove(userIen);

                Keys.UserPrefList = prefs;
                return true;
            }
            catch (Exception ex)
            {
                Keys.AlertUserError(null, ex, "Error setting User Preference");
            }
            return false;
        }

        private readonly object _getConsultAppointmentRequestsLock = new object();

        public AppointmentRequest GetConsultApptRequestByIen(string recIen)
        {
            var param = new List<string> { null, null, null, null, recIen.ToString() };
            var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetConsultAppointmentRequests, param);

            if (newRec == null)
                return null;

            if (newRec.Rows.Count == 0)
                return null;

            List<AppointmentRequest> list = GetConsultAppointmentRequests(false, newRec);
            return list[0];
        }

        public List<AppointmentRequest> GetConsultAppointmentRequests(bool isInThread = false, DataTable dt = null)
        {
            DataTable consultAppointmentRequests = null;

            if (dt == null)
                consultAppointmentRequests = Keys.AppointmentRequestWaitListTables.ConsultWaitList.Copy();
            else
                consultAppointmentRequests = dt;

            lock (_getConsultAppointmentRequestsLock)
            {
                var patientConsultRequests = new List<AppointmentRequest>();

                try
                {
                    foreach (DataRow wt in consultAppointmentRequests.Rows)
                    {
                        var newRequest = new AppointmentRequest(this)
                        {
                            Patient = new Patient(this),
                            ConsultationRequestIen = Convert.ToInt32(wt["CONSULTIEN"])
                        };
                        newRequest.Patient.IEN = wt["DFN"].ToString();
                        var sensitive = wt["SENSITIVE"].ToString();

                        if (string.IsNullOrEmpty(sensitive) || sensitive == "0")
                            newRequest.Patient.IsSensitiveRecord = false;
                        else
                            newRequest.Patient.IsSensitiveRecord = true;

                        newRequest.Patient.Name = wt["NAME"].ToString();
                        newRequest.Patient.HRN = wt["HRN"].ToString();
                        newRequest.Patient.SSN = wt["SSN"].ToString();
                        newRequest.Patient.DOB = wt["DOB"].ToString();
                        newRequest.Patient.Gender = wt["GENDER"].ToString();
                        newRequest.ClinicIen = wt["CLINIEN"].ToString();
                        newRequest.ClinName = wt["TO_SERVICE"].ToString();
                        newRequest.Patient.OriginatingUser = wt["USERIEN"].ToString(); //"User, Demo"; // row["USER"].ToString();
                        newRequest.Patient.OriginatingUserName = wt["USERNAME"].ToString();
                        // artf20229: Changed to piece 8 (DATE_OF_REQUEST) per Fay S by Ron H
                        var origDate = wt["DATE_OF_REQUEST"].ToString();

                        if (string.IsNullOrEmpty(origDate))
                            origDate = wt["ORIGDT"].ToString();

                        newRequest.OriginatingDate = ConvertOrigDate(origDate);
                        //newRequest.RequestManagmentComment = wt["COMM"].ToString();
                        newRequest.Patient.InstitutionIen = wt["INSTIEN"].ToString();
                        newRequest.Patient.Institution = wt["INSTNAME"].ToString();
                        newRequest.Patient.Provider = new Provider
                        {
                            IEN = wt["PROVIEN"].ToString(),
                            Name = wt["PROVNAME"].ToString()
                        };

                        newRequest.RequestedByText = newRequest.Patient.Provider.Name;
                        newRequest.RequestedBy = newRequest.RequestedByValues.FirstOrDefault(rb => (rb.Value == "1"));
                        newRequest.RequestedByProviderMerge = wt["CLINNAME"].ToString();
                        newRequest.Provider = newRequest.Patient.Provider;
                        newRequest.AppointmentRequestType = wt["REQUEST_TYPE"].ToString().ToUpper();
                        var urgency = wt["PRIO"].ToString();

                        if (!Keys.Methods.IsNumeric(urgency) || urgency.Length != 7)
                        {
                            var urgArray = urgency.Split('-');

                            if (urgArray.Length > 1)
                            {
                                newRequest.ConsultUrgency = urgArray[1].ToString(CultureInfo.InvariantCulture).Trim();
                                var serviceRenderedAs = wt["SERVICE_RENDERED_AS"].ToString().ToUpper();
                                newRequest.ConsultUrgency += " - " + serviceRenderedAs;
                            }
                            else
                                newRequest.ConsultUrgency = urgency;
                        }
                        else
                        {
                            // artf20229: DesiredApptDate to come from RPC return piece 8, "DATE" ALWAYS per Fay S 11/17/2016 - Ron H
                            //newRequest.DesiredApptDate = ConvertFileManDateToDateTime(urgency);
                            newRequest.ConsultUrgency = newRequest.DesiredApptDate.HasValue ? newRequest.DesiredApptDate.Value.ToShortDateString() : string.Empty;
                        }

                        // artf20229: DesiredApptDate to come from RPC return piece 49 (EARLIEST).
                        //            if piece 49 not defined then piece 8, "DATE_OF_REQUEST" if NULL use "ORIGDT"
                        //            per Fay S 11/17/2016 - Ron H
                        var requestDate = wt["EARLIEST"].ToString();

                        if (string.IsNullOrEmpty(requestDate))
                            requestDate = wt["DATE_OF_REQUEST"].ToString();

                        if (string.IsNullOrEmpty(requestDate))
                            newRequest.DesiredApptDate = newRequest.OriginatingDate;
                        else
                        {
                            requestDate = requestDate.Replace('@', ' ');
                            newRequest.DesiredApptDate = ConvertFileManDateToDateTime(requestDate);
                        }

                        newRequest.Priority = urgency;
                        //newRequest.Patient.PriorityGroup = GetPriorityGroup(newRequest.Patient.Name);
                        newRequest.Patient.PrimaryContactInfo.Phone = wt["PTPHONE"].ToString();
                        newRequest.Patient.PriorityGroup = wt["PRIGRP"].ToString();
                        newRequest.Patient.EligibilityIen = wt["ELIGIEN"].ToString();
                        newRequest.Patient.EligibilityName = wt["ELIGNAME"].ToString();
                        newRequest.Patient.ServiceConnection = wt["SVCCONN"].ToString();
                        newRequest.Patient.ServiceConnectionPercentage = wt["SVCCONNP"].ToString();

                        newRequest.Patient.ServiceRelated = newRequest.Patient.ServiceConnectionPercent >= 50;

                        if (newRequest.Patient.ServiceConnection == "YES" && newRequest.Patient.ServiceConnectionPercent >= 50)
                            newRequest.Patient.SCVisit = "YES";
                        else
                            newRequest.Patient.SCVisit = "NO";

                        newRequest.Patient.TypeIen = wt["TYPEIEN"].ToString();
                        newRequest.Patient.TypeName = wt["TYPENAME"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street1 = wt["PADDRES1"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street2 = wt["PADDRES2"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Street3 = wt["PADDRES3"].ToString();
                        newRequest.Patient.PrimaryContactInfo.City = wt["PCITY"].ToString();
                        newRequest.Patient.PrimaryContactInfo.ZipPlus4 = wt["PZIP4"].ToString();
                        newRequest.Patient.PrimaryContactInfo.Country = newRequest.Patient.Countries.FirstOrDefault(c => c.Ien == wt["PCOUNTRY"].ToString());
                        newRequest.Patient.GAF.Text = wt["GAF"].ToString();
                        var prohibitedClin = wt["PRHBLOC"].ToString();
                        if (Keys.Methods.IsNumeric(prohibitedClin))
                            newRequest.IsProhibitedClinic = Convert.ToInt32(wt["PRHBLOC"]) == 1;
                        else
                            newRequest.IsProhibitedClinic = false;
                        if (string.IsNullOrEmpty(newRequest.EnrollmentPriority))
                            newRequest.EnrollmentPriority = newRequest.Patient.PriorityGroup;
                        newRequest.StopCodes = new List<int>();
                        var stopCodes = wt["STOP"].ToString().Split('|');
                        foreach (var code in stopCodes)
                            if (Keys.Methods.IsNumeric(code))
                                newRequest.StopCodes.Add(Convert.ToInt32(code));
                        patientConsultRequests.Add(newRequest);
                    }
                }
                catch (Exception ex)
                {
                    if (isInThread)
                        throw;
                    else
                        MessageBox.Show(ex.Message, "Get Consult Requests Error");
                }

                return patientConsultRequests;
            }
        }

        private DateTime ConvertFileManDateToDateTime(string urgency)
        {
            return Keys.Methods.ConvertFileManDateToDateTime(urgency);
        }

        public AppointmentRequest GetApptRequestByRequestIEN(string requestIEN)
        {
            var parms = new List<string> { requestIEN };
            var table = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, parms);

            if (table == null)
                return null;

            var apptRequests = GetApptAppointmentRequests(table);

            return apptRequests.Count == 0 ? null : apptRequests[0];
        }

        private readonly object _getApptAppointmentRequestsLock = new object();

        public List<AppointmentRequest> GetApptAppointmentRequests()
        {
            return GetApptAppointmentRequests(Keys.AppointmentRequestWaitListTables.ApptWaitList.Copy());
        }

        public List<AppointmentRequest> GetApptAppointmentRequests(DataTable dt)
        {
            if (dt == null)
                return new List<AppointmentRequest>();

            lock (_getApptAppointmentRequestsLock)
            {
                var patientApptRequests = new List<AppointmentRequest>();
                var apptAppointmentRequests = dt;

                try
                {
                    foreach (DataRow wt in apptAppointmentRequests.Rows)
                    {
                        var apptRequest = new AppointmentRequest(this)
                        {
                            Patient = new Patient(this)
                            {
                                IEN = wt["DFN"].ToString(),
                                Name = wt["NAME"].ToString(),
                                HRN = wt["HRN"].ToString(),
                                SSN = wt["SSN"].ToString(),
                                DOB = wt["DOB"].ToString(),
                                Gender = wt["GENDER"].ToString(),
                                OriginatingUser = wt["USERIEN"].ToString(),
                                OriginatingUserName = wt["USERNAME"].ToString(),
                                InstitutionIen = wt["INSTIEN"].ToString(),
                                Institution = wt["INSTNAME"].ToString(),

                                Provider = new Provider
                                {
                                    IEN = wt["PROVIEN"].ToString(),
                                    Name = wt["PROVNAME"].ToString()
                                },
                                PrimaryContactInfo =
                                {
                                    Phone = wt["PTPHONE"].ToString(),
                                    Street1 = wt["PADDRES1"].ToString(),
                                    Street2 = wt["PADDRES2"].ToString(),
                                    Street3 = wt["PADDRES3"].ToString(),
                                    City = wt["PCITY"].ToString(),
                                    ZipPlus4 = wt["PZIP4"].ToString(),
                                },

                                PriorityGroup = wt["PRIGRP"].ToString(),
                                EligibilityIen = wt["ELIGIEN"].ToString(),
                                EligibilityName = wt["ELIGNAME"].ToString(),
                                ServiceConnection = wt["SVCCONN"].ToString(),
                                ServiceConnectionPercentage = wt["SVCCONNP"].ToString(),
                                SCVisit = wt["WLSVCCON"].ToString(),
                                TypeIen = wt["TYPEIEN"].ToString(),
                                TypeName = wt["TYPENAME"].ToString(),
                                GAF = { Text = wt["GAF"].ToString() }
                            },

                            RequestManagmentIen = wt["IEN"].ToString(),
                            ClinicIen = wt["CLINIEN"].ToString(),
                            ClinName = wt["CLINNAME"].ToString(),
                            RequestManagmentComment = wt["COMM"].ToString(),
                            RequestedByProviderMerge = wt["CLINNAME"].ToString(),
                            AppointmentRequestType = wt["TYPE"].ToString().ToUpper(),
                            ApptRequestIen = Convert.ToInt32(wt["IEN"].ToString()),
                            Priority = wt["PRIO"].ToString(),
                            DispositionDate = wt["ARDISPD"].ToString(),
                            DispositionUserID = wt["ARDISPU"].ToString(),
                            DispositionUserName = wt["ARDISPUN"].ToString(),
                            EnrollmentPriority = wt["ENROLLMENT_PRIORITY"].ToString(),
                            ClinicStopIen = wt["STOPIEN"].ToString(),
                            ClinicStopName = wt["STOPNAME"].ToString(),
                            EnrolleeStatus = wt["EESTAT"].ToString(),
                            MRTCParentRequestIEN = wt["SDPARENT"].ToString(),
                        };

                        if (!String.IsNullOrEmpty(wt["REQBY"].ToString()))
                            apptRequest.RequestedBy = apptRequest.RequestedByValues.FirstOrDefault(rb => (rb.Value == wt["REQBY"].ToString()));

                        apptRequest.Provider = apptRequest.Patient.Provider;
                        apptRequest.Patient.PrimaryContactInfo.Country = apptRequest.Patient.Countries.FirstOrDefault(c => c.Ien == wt["PCOUNTRY"].ToString());

                        var origDate = wt["ORIGDT"].ToString();
                        apptRequest.OriginatingDate = ConvertOrigDate(origDate);

                        var sensitive = wt["SENSITIVE"].ToString();
                        apptRequest.Patient.IsSensitiveRecord = !string.IsNullOrEmpty(sensitive) && sensitive != "0";

                        var desiredApptDate = wt["DAPTDT"].ToString();
                        apptRequest.DesiredApptDate = ConvertOrigDate(desiredApptDate);

                        if (apptRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Mobile)
                            apptRequest.AppointmentRequestType = Keys.AppointmentRequestTypes.Veteran;

                        apptRequest.IsVeteran = apptRequest.AppointmentRequestType == Keys.AppointmentRequestTypes.Veteran;

                        apptRequest.Patient.ServiceRelated = apptRequest.Patient.ServiceConnectionPercent >= 50;

                        var apptTypeIen = wt["APPTYPE"].ToString();

                        if (Keys.Methods.IsNumeric(apptTypeIen))
                        {
                            foreach (var apptType in AppointmentTypes(apptRequest.Patient.IEN).Where(apptType => apptType.Ien == apptTypeIen))
                            {
                                apptRequest.AppointmentType = apptType.Name;
                                break;
                            }
                        }

                        var multipleAppointmentsRequired = wt["MULTIPLE APPOINTMENT RTC"].ToString();
                        apptRequest.MultipleAppointmentsRequired = Keys.Methods.StrToBool(multipleAppointmentsRequired);

                        var multApptNumber = wt["MULT APPT NUMBER"].ToString();
                        apptRequest.NumberOfAppointmentsRequired = string.IsNullOrEmpty(multApptNumber) ? 0 : Convert.ToInt32(wt["MULT APPT NUMBER"]);

                        var apptIntervalIndays = wt["MULT APPT RTC INTERVAL"].ToString();
                        apptRequest.AppointmentIntervalInDays = string.IsNullOrEmpty(apptIntervalIndays) ? 0 : Convert.ToInt32(wt["MULT APPT RTC INTERVAL"]);

                        var mrtcApptCount = wt["MRTCCOUNT"].ToString();
                        apptRequest.MrtcAppointmentCount = Keys.Methods.IsNumeric(mrtcApptCount) ? Convert.ToInt32(mrtcApptCount) : 0;

                        var prohibitedClin = wt["PRHBLOC"].ToString();
                        if (Keys.Methods.IsNumeric(prohibitedClin))
                            apptRequest.IsProhibitedClinic = Convert.ToInt32(wt["PRHBLOC"]) == 1;
                        else
                            apptRequest.IsProhibitedClinic = false;


                        #region BJ'S PATIENT CONTACT REPLACEMENT CODE
                        //apptRequest.PatientContacts = getPatientContact(apptRequest.Patient.IEN,
                        //                                                apptRequest.ClinicIen,
                        //                                                ExtensionMethods.ConvertToFilemanDateTime(
                        //                                                    Convert.ToDateTime(apptRequest.DesiredApptDate)),
                        //                                                    apptRequest.AppointmentRequestType);
                        #endregion

                        #region HPE OLD PATIENT CONTACT LOGIC (COMMENTED OUT)
                        //var pContact = wt["PCONTACT"].ToString();
                        //if (!string.IsNullOrEmpty(pContact))
                        //{
                        //    apptRequest.PatientContacts = new List<PatientContact>();
                        //    var records = pContact.Split(new[] {"::"}, StringSplitOptions.None);
                        //    foreach (var record in records)
                        //    {
                        //        var entries = record.Split(new[] {"~~"}, StringSplitOptions.None);
                        //        var patientContact = new PatientContact
                        //        {
                        //            DateEntered = entries[0],
                        //            PCEnteredByUserIen = entries[1],
                        //            PCEnteredByUserName = entries[2],
                        //            Action = entries[3].ToUpper(),
                        //            PatientPhone = entries[4]
                        //        };
                        //        apptRequest.PatientContacts.Add(patientContact);
                        //    }
                        //}
                        #endregion

                        var pApptPtrs = wt["APPTPTRS"].ToString();
                        apptRequest.MRTCAppointmentIENs.Clear();

                        if (!string.IsNullOrEmpty(pApptPtrs))
                        {
                            var apptList = pApptPtrs.Split('|');

                            foreach (var ien in apptList)
                                if (!string.IsNullOrEmpty(ien))
                                    apptRequest.MRTCAppointmentIENs.Add(ien);
                        }

                        var pChildren = wt["CHILDREN"].ToString();
                        apptRequest.MRTCAppointmentRequestIENs.Clear();

                        if (!string.IsNullOrEmpty(pChildren))
                        {
                            var childList = pChildren.Split('|');

                            foreach (var ien in childList)
                                if (!string.IsNullOrEmpty(ien))
                                    apptRequest.MRTCAppointmentRequestIENs.Add(ien);
                        }

                        patientApptRequests.Add(apptRequest);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(TraceLog.FormatExceptionMessage(GetType().Name, "GetAppointmentRequests", ex) /*.Message*/, "Get Appointment Request(s) Error");
                }
                return patientApptRequests;
            }
        }

        private IList<Divisions> _allDivisions;
        public IList<Divisions> GetDivisions()
        {
            var table = _mDsGlobal.Tables["AllDivisions"];

            var tempDivisions = new List<Divisions>();
            foreach (DataRow row in table.Rows)
            {
                var nameValue = new Divisions
                {
                    NAME = GetCellString(row, "DIV_NAME"),
                    IEN = GetCellString(row, "DIV_IEN"),
                    IsDefault = GetCellString(row, "DEFAULT") == "YES"
                };
                tempDivisions.Add(nameValue);
            }
            tempDivisions.Sort((x, y) => string.Compare(x.NAME, y.NAME, StringComparison.OrdinalIgnoreCase));
            _allDivisions = tempDivisions;
            return _allDivisions;
        }

        public bool SetDivision(string divisionId)
        {
            var parms = new List<string> { CurrentUser, divisionId };

            var dtDivision = _adt.callOVIDRPC("SDEC SETFAC", parms);

            var returnValue = false;
            if (dtDivision.Rows.Count == 1)
            {
                var r = dtDivision.Rows[0];
                returnValue = r["ERROR_ID"].ToString() == "1";
            }

            return returnValue;
        }

        public Divisions GetCurrentDivision()
        {
            //Get Current Division
            var parms = new List<string> { CurrentUser };
            var dtDivision = _adt.callOVIDRPC("SDEC CURFACG", parms, "CurrentDivision", _mDsGlobal);
            var divisionId = string.Empty;

            if (dtDivision.Rows.Count == 1)
                divisionId = dtDivision.Rows[0]["CURRENT_DIV"].ToString();

            return GetDivisions().FirstOrDefault(d => d.IEN == divisionId);
        }

        public Divisions GetDefaultDivision()
        {
            var divisionList = GetDivisions();
            var division = divisionList.FirstOrDefault(d => d.IsDefault);

            if (division == null && divisionList.Count == 1)
                division = divisionList[0];

            return division ?? GetCurrentDivision();
        }

        public bool OverBookAuthority(SchdAppointment a, SchdResource res = null, bool silent = false, bool isMRTC = false)
        {
            var startTime = Convert.ToDateTime(a.START_TIME);
            var maxOverBooks = 0;
            DateTime? clinicStartTime = null;
            var schdRes = res;

            // If the resource is null get the resource
            if (schdRes == null)
            {
                schdRes = GetSdecResource(a.ResourceId);

                if (schdRes == null)
                    schdRes = GetSdecResourceByName(a.RESOURCENAME);
            }

            if (schdRes != null)
            {
                var overbooks = schdRes.Clinic.OVERBOOK;

                if (!string.IsNullOrEmpty(overbooks) && Keys.Methods.IsNumeric(overbooks))
                    maxOverBooks = Convert.ToInt32(overbooks);

                var clinicStartTimeStr = schdRes.Clinic.HOUR_DISPLAY_BEGIN;

                if (!string.IsNullOrEmpty(clinicStartTimeStr) && Keys.Methods.IsNumeric(clinicStartTimeStr))
                    clinicStartTime = new DateTime(startTime.Year, startTime.Month, startTime.Day, Convert.ToInt32(clinicStartTimeStr), 0, 0, 0, 0);
            }

            var sHospLoc = schdRes.HOSPITAL_LOCATION_ID;
            var sResourceId = schdRes.RESOURCEID;

            //SDEC APPSLOTS...
            var slotCount = 0;
            var isWhiteCell = false;
            // There are times when Keys.AppointmentSlots has not been assigned, or does NOT contain the slots encompassing the target slot
            // Get Keys.AppointmentSlots
            var cSlots = ReturnApptSlots(schdRes, startTime.Date, startTime.Date.AddDays(1).AddMinutes(-1));

            // Check if date has availability
            if (cSlots.Slots.Count(s => s.AccessType == Keys.AccessTypes.Available) == 0)
            {
                if (!silent)
                    Keys.AlertUser(Keys.MainWindow, "You can not book an appointment in the selected time slot,\nthe selected day has no availability.", "New Appointment");

                return false;
            }
            var slot = cSlots.Slots.FirstOrDefault(s => startTime.CompareTo(s.StartTime) == 0);

            if (slot != null)
            {
                slotCount = slot.AvailabilityCount;
                isWhiteCell = true;
            }

            // At this point, if SlotCount > 0 we do not need to go further as we have an active slot with availability
            if (slotCount > 0)
                return true;

            // Check that the target slot start time is greater than the day's first availability
            var dateHasAvailability = false;
            // Get the first slot where AccessType != Unavaiable for day of the target time slot startTime
            var availabilityStart = AvailabilityStartTime(cSlots);

            // If a first available slot was found check the target time slot startTime to insure it is between the clinic start time and midnight
            if (availabilityStart.HasValue)
            {
                var availStart = clinicStartTime.HasValue ? clinicStartTime.Value : availabilityStart.Value;
                var availMidnight = availStart.AddDays(1).Date;
                //                DateTime availMidnight = startTime.AddDays(1).Date;
                dateHasAvailability = (startTime >= availStart) && (startTime < availMidnight);
            }

            // No availability or target time slot start is before first available then cannot book
            if (!dateHasAvailability)
            {
                if (!silent)
                    Keys.AlertUser(Keys.MainWindow, "You can not overbook an appointment in the selected time slot\nthe selected time is prior to the first time slot of the selected day.", "New Appointment");

                return false;
            }

            // Can the Clinic Overbook ?
            return ClinicOverbook(sHospLoc, Convert.ToDateTime(a.START_TIME), sResourceId, silent, isMRTC);
            //var bClinicOverbook = ClinicOverbook(sHospLoc, Convert.ToDateTime(a.START_TIME), sResourceId);

            //// Master Overbook can always overbook
            //if (Keys.LoggedInUserMasterOverbook)
            //    return true;
            //// User has Regular Overbook key
            //if (Keys.LoggedInUserRegularOverbook)
            //{
            //    if (bClinicOverbook && maxOverBooks > 0)
            //        return true;
            //    // Clinic cannot overbook
            //    if (isWhiteCell)
            //        if (!silent)
            //            Keys.AlertUser(Keys.MainWindow, "You do not have permission to overbook this appointment.", "Overbook");
            //    return false;
            //}
            //if(isWhiteCell)
            //    if (!silent)
            //        Keys.AlertUser(Keys.MainWindow, "You do not have permission to overbook this appointment.", "Overbook");
            //return false;
        }

        public string NoShow(string sPatientIen, string sResourceName, string sResourceID, string sAppointmentType)
        {
            SchdResource schdRes = GetSdecResource(sResourceID);

            if (schdRes == null)
            {
                MessageBox.Show("Unable to find resource: " + sResourceName);
                return "";
            }
            var sHospLoc = schdRes.HOSPITAL_LOCATION_ID;
            var parms = new List<string> { sPatientIen, sHospLoc };
            var table = _adt.callOVIDRPC("SDEC NOSHOPAT", parms);

            var sResult = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                var noShows = r["TOTAL_NO_SHOWS"].ToString();
                if (Keys.Methods.IsNumeric(noShows) && Convert.ToInt32(noShows) > 2)
                    sResult = "Patient has had " + r["TOTAL_NO_SHOWS"] + " no show appointments\nfor this clinic in the last 12 months.";
            }
            ClearPatientFutureAppointmentsCache(sPatientIen);
            return sResult;
        }

        public bool ClinicOverbook(string sClinicIen, DateTime dStartTime, string sResource, bool silent = false, bool isMRTC = false)
        {
            var parms = new List<string> {
                sClinicIen,
                dStartTime.ToString("M/d/yyyy@HH:mm"),
                isMRTC ? "1" : "0",
                Keys.LoggedInUserIen
            };

            var table = _adt.callOVIDRPC("SDEC57 OBM", parms);

            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                string returnContinue = r["CONTINUE"].ToString();
                string returnMessage = r["MESSAGE"].ToString();
                string returnPrompt = r["PROMPT"].ToString();
                string returnDefault = r["DEFAULT"].ToString();

                switch (returnContinue)
                {
                    case "-1":
                        throw new RPCException(string.Format("Error checking overbook authority:\n{0}", returnMessage));
                    case "0":
                        if (!silent)
                            Keys.AlertUser(null, returnMessage, "Overbook");

                        return false;
                    case "1":
                        return true;
                    case "2":
                        if (silent)
                            return true;
                        else
                            return Keys.ConfirmUser(null, returnPrompt, "Overbook", true);
                }
            }

            return true;
        }

        public Keys.OverbookingStatus ClinicOverbook(SchdAppointment appointment)
        {
            if (appointment.SelectedResource == null)
            {
                if (string.IsNullOrEmpty(appointment.ResourceId))
                {
                    if (!string.IsNullOrEmpty(appointment.RESOURCENAME))
                        appointment.SelectedResource = GetSdecResourceByName(appointment.RESOURCENAME);
                }
                else
                    appointment.SelectedResource = GetSdecResource(appointment.ResourceId);
            }

            if (appointment.SelectedResource == null)
            {
                Keys.AlertUser(null, "No Clinic selected.\nPlease select a clinic and try again.", "Clinic Overbook");
                return Keys.OverbookingStatus.Cannot;
            }

            var parms = new List<string> {
                appointment.SelectedResource.Clinic.ClinicIen,
                Convert.ToDateTime(appointment.START_TIME).ToString("M/d/yyyy@HH:mm"),
                "0", //String.IsNullOrEmpty(appointment.MRTCParentRequestIEN) ? "0" : "1",
                Keys.LoggedInUserIen
            };

            var table = _adt.callOVIDRPC("SDEC57 OBM", parms);

            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                string returnContinue = r["CONTINUE"].ToString();
                string returnMessage = r["MESSAGE"].ToString();
                string returnPrompt = r["PROMPT"].ToString();
                string returnDefault = r["DEFAULT"].ToString();

                switch (returnContinue)
                {
                    case "-1":
                        Exception ex = new RPCException(string.Format("Error checking overbook authority:\n{0}", returnMessage));
                        TraceLog.LogException(this.GetType().Name, "ClinicOverbook", ex);
                        Keys.AlertUserError(null, ex, "Clinic Overbook");
                        return Keys.OverbookingStatus.Cannot;
                    case "0":
                        Keys.AlertUser(null, returnMessage, "Overbook");
                        return Keys.OverbookingStatus.Cannot;
                    case "1":
                        return Keys.OverbookingStatus.NotRequired;
                    case "2":
                        if (Keys.ConfirmUser(null, returnPrompt, "Overbook", true))
                            return Keys.OverbookingStatus.Required;
                        else
                            return Keys.OverbookingStatus.Cannot;
                }
            }

            return Keys.OverbookingStatus.NotRequired;
        }

        public string IsHoliday(DateTime startDate)
        {
            try
            {
                if (Keys.Holidays == null)
                {
                    Keys.Holidays = new List<CHoliday>();
                    var parms = new List<string> { DateTime.Today.AddDays(-365).ToString("M/d/yyyy@HH:mm") };
                    //Get holidays
                    using (var dvHolidays = new DataView(_adt.callOVIDRPC("SDEC HOLIDAY", parms, "Holidays", _mDsGlobal)))
                    {
                        foreach (DataRowView rec in dvHolidays)
                        {
                            if (!Keys.Methods.IsDate(rec["SDEC_HOLIDAY_DATE"]))
                            {
                                TraceLog.LogException(GetType().Name, "IsHoliday", new Exception("Invalid SDEC_HOLIDAY_DATE in VistA"));
                                continue;
                            }
                            var newHoliday = new CHoliday
                            {
                                Name = rec["HOLIDAY_NAME"].ToString(),
                                Date = Convert.ToDateTime(rec["SDEC_HOLIDAY_DATE"]).Date
                            };
                            Keys.Holidays.Add(newHoliday);
                        }
                    }
                }

                var holiday = Keys.Holidays.FirstOrDefault(h => h.Date.Date.CompareTo(startDate.Date) == 0);

                if (holiday == null)
                    return string.Empty;

                return holiday.Name;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "IsHoliday", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
            return string.Empty;
        }

        public bool ClinicCanBookOnHoliday(string clincIen, string clinic)
        {
            var clinSetTable = GetClinicSetupParameters(clincIen, clinic);

            if (clinSetTable.Rows.Count == 0)
                return false;

            return Keys.Methods.StrToBool(clinSetTable.Rows[0]["HOLIDAYS"].ToString());
        }

        public string DeleteAvailability(string apptID)
        {
            var parms = new List<string> { apptID };

            var table = _adt.callOVIDRPC("SDEC AVDEL", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["ERRORID"].ToString() != "-1")
                {
                    sErrorId = r["ERRORID"].ToString();
                }
            }
            return sErrorId;
        }

        public string DeleteAvailability(SchdAvailability a)
        {
            var parms = new List<string> { a.APPOINTMENTID };

            var table = _adt.callOVIDRPC("SDEC AVDEL", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["ERRORID"].ToString() != "-1")
                {
                    sErrorId = r["ERRORID"].ToString();
                }
            }

            if (sErrorId == string.Empty)
            {
                GetAvailabilities(a.RESOURCENAME, a.StartTime, a.EndTime);
            }
            return sErrorId;
        }

        public string AppointmentNoShow(string apptID, bool bNoShow)
        {
            var parms = new List<string> { apptID, bNoShow ? "1" : "0" };
            var table = _adt.callOVIDRPC("SDEC NOSHOW", parms);

            var sErrorText = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["ERRORID"].ToString() != "1")
                {
                    sErrorText = r["ERRORTEXT"].ToString();
                }
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(string.Empty, apptID);

            return sErrorText;
        }

        public string UndoCancelAppointment(string apptID)
        {
            var parms = new List<string> { apptID };
            var table = _adt.callOVIDRPC("SDEC UNDOCANA", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                sErrorId = r["ERRORID"].ToString() == "0" ? string.Empty : r["ERRORID"].ToString();
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(string.Empty, apptID);
            return sErrorId;
        }

        public bool IsCompAndPenAppt(string apptId)
        {
            var parms = new List<string> { apptId };

            var rslts = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentType, parms);
            foreach (DataRow r in rslts.Rows)
            {
                var apptTypeIen = r["APPTTYPE_IEN"].ToString();
                //var apptTypeName = r["APPTTYPE_NAME"].ToString();
                return apptTypeIen == "1";
            }
            return false;
        }

        public string CancelAppointment(CancelAppointment cancelAppointmentData, SchdAppointment schdAppt = null)
        {
            if (IsCompAndPenAppt(cancelAppointmentData.ApptID))
            {
                var parms1 = new List<string>();
                if (schdAppt != null)
                {
                    parms1.Add(schdAppt.PATIENTID);
                    parms1.Add(schdAppt.AppointmentRequest.ClinicIen);
                    var dt = Convert.ToDateTime(schdAppt.START_TIME);
                    var stTime = ConvertDateTimeToExternalMumpsFormat(dt);
                    parms1.Add(stTime);
                    var canRslt = _adt.callOVIDRPC(Keys.RpcCalls.SdecCapCan, parms1);
                    foreach (DataRow r in canRslt.Rows)
                    {
                        var reqIen = r["REQIEN"].ToString();
                        var ien = r["IEN"].ToString();
                        if (Keys.Methods.IsNumeric(reqIen) && Convert.ToInt32(reqIen) < 0)
                            if (ien.Contains("updated"))
                                MessageBox.Show("The associated 2507 request link has been updated");
                            else
                                MessageBox.Show(ien);
                    }
                }
            }

            var parms = new List<string>
            {
                cancelAppointmentData.ApptID,
                cancelAppointmentData.CancelledByClinic ? "C" : "PC",
                cancelAppointmentData.ReasonIEN,
                cancelAppointmentData.Notes
            };

            var sErrorId = string.Empty;

            try
            {
                var table = _adt.callOVIDRPC("SDEC APPDEL", parms);

                if (table.Rows.Count == 1)
                {
                    var r = table.Rows[0];
                    sErrorId = r["ERRORID"].ToString() == "0" ? string.Empty : r["ERRORID"].ToString();
                }

                if (string.IsNullOrEmpty(sErrorId))
                {
                    // Clear the Future Appointments Cache
                    ClearPatientFutureAppointmentsCache(cancelAppointmentData.PatientID);

                    //TODO: Need to get the Request ID for the cancelAppointmentData.ApptID...
                    var parms2 = new List<string> { cancelAppointmentData.ApptID };
                    var rslt = _adt.callOVIDRPC(Keys.RpcCalls.GetRequestId, parms2);
                    foreach (DataRow dr in rslt.Rows)
                    {
                        var typeIen = dr["SDAPTYP"].ToString();
                        var pieces = typeIen.Split('|');
                    }

                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CancelAppointment", ex);
                sErrorId = "Unable to cancel the selected appointment, please review the Trace Log for details.";
            }

            return sErrorId;
        }

        public string UndoCheckOut(string apptID)
        {
            var parms = new List<string> { apptID };
            var table = _adt.callOVIDRPC("SDEC CANCKOUT", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                sErrorId = r["ERRORID"].ToString() == "0" ? string.Empty : r["ERRORID"].ToString();
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(string.Empty, apptID);
            return sErrorId;
        }

        public string CheckOutPatient(CheckOutPatient checkOutData)
        {
            var parms = new List<string>
            {
                checkOutData.PatientID,
                Convert.ToDateTime( checkOutData.AppointmentStartTime ).ConvertToExternalDateTimeFormat( true, true ),
                Convert.ToDateTime( checkOutData.CheckOutDateTime ).ConvertToExternalDateTimeFormat( true, true ),
                checkOutData.ApptID,
                checkOutData.ProviderIEN
            };
            var table = _adt.callOVIDRPC("SDEC CHECKOUT", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                sErrorId = r["ERRORID"].ToString() == "0" ? string.Empty : r["ERRORID"].ToString();
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(checkOutData.PatientID);
            return sErrorId;
        }

        public string UndoCheckIn(string apptID)
        {
            var parms = new List<string> { apptID, "@" };
            var table = _adt.callOVIDRPC("SDEC CHECKIN", parms);
            var sErrorMessage = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                var sErrorId = r["ERRORID"].ToString();
                if (r["Message"].ToString() != string.Empty)
                {
                    if (sErrorId == "0")
                    {
                        sErrorMessage = "Undo Patient Check In Successful.\n\n";
                    }
                    sErrorMessage += "However, the following error(s) occurred while Undo Checking In the patient:\n" + r["MESSAGE"];
                }
                else
                {
                    sErrorMessage = r["MESSAGE"].ToString();
                }
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(string.Empty, apptID);
            return sErrorMessage;
        }

        public string CheckInPatient(CheckInPatient checkInData)
        {
            var sErrorMessage = string.Empty;
            if (checkInData == null)
            {

                sErrorMessage = "CheckIn data failed to populate.  Patient will not be checked in.";
                return sErrorMessage;
            }

            var parms = new List<string>
                {
                    checkInData.ApptID,
                    Convert.ToDateTime(checkInData.CheckInDateTime).ConvertToExternalDateTimeFormat(true, true),
                    checkInData.ClinicStopIEN,
                    checkInData.ProviderIEN,
                    checkInData.PrintRouteSlip.ToString(),
                    checkInData.HealthSummaryLetterIEN,
                    checkInData.PrintWellnessHandout.ToString(),
                    checkInData.PCCClinicIEN ?? "",
                    checkInData.PCCFormIEN ?? "",
                    checkInData.PCCOutGuide.ToString(),
                    checkInData.MakeChartRequest.ToString()
                };
            var table = _adt.callOVIDRPC("SDEC CHECKIN", parms);


            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                var sErrorId = r["ERRORID"].ToString();
                if (r["Message"].ToString() != string.Empty)
                {
                    if (sErrorId == "0")
                    {
                        sErrorMessage = "Patient Check In Successful.\n\n";
                    }
                    sErrorMessage += "However, the following error(s) occurred while Checking In the patient:\n" + r["MESSAGE"];
                }
                else
                {
                    sErrorMessage = r["MESSAGE"].ToString();
                }
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(checkInData.PatientIEN);
            return sErrorMessage;
        }

        public bool CheckConflictAppointment(
            string command,
            string sResourceName,
            string sPatientIEN,
            string sPatientName,
            string sStartDate,
            string sEndDate,
            out bool isTelehealth,
            string excludeAppointmentID = "",
            bool silent = false,
            bool exactMatchOnly = false
                )
        {
            isTelehealth = false;
            DateTime startDate = Convert.ToDateTime(sStartDate);
            DateTime endDate = Convert.ToDateTime(sEndDate);
            // Checks in the future for 365 days.  
            IList<FuturePatientAppointment> list = ReadFutureAppointments(sPatientIEN, startDate.Date, endDate.Date.AddYears(1));
            var sameDayList = new List<string>();
            var conflictingList = new List<string>();

            // Check for a clinic conflict based on the data coming back from the GetFutureAppointments call
            //  Filter out all cancelled appointments
            foreach (FuturePatientAppointment row in list.Where(fa => (!fa.IsCancelled)))
            {
                // Skip when the conflicting appointment is to be excluded
                if (!string.IsNullOrEmpty(excludeAppointmentID))
                    if (row.AppointmentID == excludeAppointmentID)
                        continue;

                int dApptLength = 30;

                var clinic = GetClinicByID(row.ClinicIen);

                if (clinic != null)
                    dApptLength = clinic.AppointmentLength;

                DateTime dEndDate = row.AppointmentDateTime.AddMinutes(dApptLength);
                string message = $"    at {row.AppointmentDateTime.ToString()} in {row.Clinic}";

                if (exactMatchOnly)
                {
                    if (Convert.ToDateTime(startDate).CompareTo(row.AppointmentDateTime) == 0)
                        conflictingList.Add(message);
                }
                else
                {
                    // Checks if there is an appt time/date conflict with another clinic
                    if (Keys.Methods.TimesOverlap(startDate, endDate, row.AppointmentDateTime, dEndDate))
                        conflictingList.Add(message);
                    else if (startDate.Date.CompareTo(row.AppointmentDateTime.Date) == 0)
                        sameDayList.Add(message);
                }

                // Commented out 08/26, Ron H per Fay Sruble - returns true to prevent further booking operations
                //if (silent)
                //    return true;
                //else
                //{
                //     If appt is NOT a telehealth appt then there is a confilct, return true, else return false to allow booking to continue
                //    isTelehealth = Keys.ConfirmUser(
                //        Keys.MainWindow,
                //        "Patient " + sPatientName + " already has an appointment scheduled at " + sStartDate + " in clinic " + resourceName + "." +
                //        "\n\nIs this appointment part of a Telehealth or Chart Check appointment pair?",
                //        command,
                //        true
                //        );

                //    return !isTelehealth;
                //}
            }

            if (!silent)
            {
                if ((conflictingList.Count > 0) || (sameDayList.Count > 0))
                {
                    string message = "Patient " + sPatientName + " has other appointments scheduled";

                    if (conflictingList.Count > 0)
                    {
                        message += "\n  that conflicts with the selected appointment time:\n" + string.Join("\n", conflictingList.ToArray());

                        if (sameDayList.Count > 0)
                            message += "\n";
                    }

                    if (sameDayList.Count > 0)
                        message += "\n  the same day as the selected appointment time:\n" + string.Join("\n", sameDayList.ToArray());


                    Keys.AlertUser(Keys.MainWindow, message, command + " Conflicts");
                }
            }

            return (conflictingList.Count > 0);
        }

        public IList<ConflictAppointments> GetConflictAppointment(string sResourceName, string sPatientIEN, string sStartDate, string sEndDate, bool isNewAppointment, string excludeAppointmentID = "")
        {
            var tempAppointments = new List<ConflictAppointments>();
            DateTime start = Convert.ToDateTime(sStartDate);
            DateTime end = Convert.ToDateTime(sEndDate);
            int duration = (int)(end - start).TotalMinutes;
            IList<FuturePatientAppointment> list = ReadFutureAppointments(sPatientIEN, start.Date, end.Date.AddYears(1));

            foreach (FuturePatientAppointment row in list)
            {
                if (!string.IsNullOrEmpty(excludeAppointmentID))
                    if (row.AppointmentID == excludeAppointmentID)
                        continue;

                DateTime dStartDate = Keys.Methods.ConvertFileManExternalTimeFormatToDateTime(row.AppointmentDate);
                DateTime dEndDate = dStartDate.AddMinutes(duration);

                if (Keys.Methods.TimesOverlap(Convert.ToDateTime(sStartDate), Convert.ToDateTime(sEndDate), dStartDate, dEndDate))
                {
                    var ca = new ConflictAppointments();
                    ca.ResourceName = row.Clinic;
                    var patientId = sPatientIEN;
                    var newPatient = GetPatientInfo(patientId);
                    ca.Patient = newPatient.Name;
                    ca.AppointmentID = row.AppointmentID;
                    ca.StartTime = dStartDate.ToString();
                    ca.EndTime = dEndDate.ToString();
                    tempAppointments.Add(ca);
                }
            }

            return tempAppointments;
        }

        private IList<NameValue> _scheduleUsers;
        public IList<NameValue> GetScheduleUsers()
        {
            if (_scheduleUsers == null || _scheduleUsers.Count == 0)
            {
                var table = _mDsGlobal.Tables["ScheduleUser"];

                var tempSchedUsers = new List<NameValue>();
                foreach (DataRow row in table.Rows)
                {
                    var nameValue = new NameValue { Value = GetCellString(row, "USERID"), Name = GetCellString(row, "USERNAME") };
                    tempSchedUsers.Add(nameValue);
                }
                tempSchedUsers.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));

                _scheduleUsers = tempSchedUsers;
            }
            return _scheduleUsers;
        }

        public IList<NameValue> GetResourceGroupList()
        {
            var tempResourceGroup = new List<NameValue>();

            if (Keys.Tables.ResourceGroupListDataLoaded.AddMinutes(30) < DateTime.Now || Keys.Tables.ResourceGroupListData == null)
            {
                var parm1 = new List<string> { CurrentUser };
                Keys.Tables.ResourceGroupListData = _adt.callOVIDRPC("SDEC RESGRPUS", parm1);
                Keys.Tables.ResourceGroupListDataLoaded = DateTime.Now;
            }

            if (Keys.Tables.ResourceGroupListData != null)
            {
                foreach (DataRow row in Keys.Tables.ResourceGroupListData.Rows)
                {
                    var nameValue = new NameValue { Value = GetCellString(row, "RESOURCE_GROUPID"), Name = GetCellString(row, "RESOURCE_GROUP") };
                    tempResourceGroup.Add(nameValue);
                }
            }

            tempResourceGroup.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
            var resourceGroups = tempResourceGroup;
            return resourceGroups;
        }

        private IList<SchdGroupedAccessTypes> _groupedAccessTypes;
        public IList<SchdGroupedAccessTypes> GetGroupedAccessTypes()
        {
            var table = _mDsGlobal.Tables["GroupedAccessType"];
            var tempGroupedAccessType = new List<SchdGroupedAccessTypes>();
            foreach (DataRow row in table.Rows)
            {
                var at = new SchdGroupedAccessTypes
                {
                    ACCESS_GROUP_TYPEID = GetCellString(row, "ACCESS_GROUP_TYPEID"),
                    ACCESS_GROUP = GetCellString(row, "ACCESS_GROUP"),
                    ACCESS_GROUP_ID = GetCellString(row, "ACCESS_GROUP_ID"),
                    ACCESS_TYPE = GetCellString(row, "ACCESS_TYPE"),
                    ACCESS_TYPE_ID = GetCellString(row, "ACCESS_TYPE_ID")
                };
                tempGroupedAccessType.Add(at);
            }
            tempGroupedAccessType.Sort((x, y) => string.Compare(x.ACCESS_TYPE, y.ACCESS_TYPE, StringComparison.OrdinalIgnoreCase));
            _groupedAccessTypes = tempGroupedAccessType;
            return _groupedAccessTypes;
        }

        private IList<SchdGroupedAccessTypes> _groupedAccessType;
        public IList<SchdGroupedAccessTypes> GetAccessTypeByGroupID(string groupID)
        {
            var table = _mDsGlobal.Tables["GroupedAccessType"] ??
                        _adt.callOVIDRPC("SDEC ACCGPTYG", null, "GroupedAccessType", _mDsGlobal);

            var trmpAccessType = new List<SchdGroupedAccessTypes>();
            foreach (DataRow row in table.Rows)
            {
                var at = new SchdGroupedAccessTypes();
                if (GetCellString(row, "ACCESS_GROUP_ID") == groupID || groupID == "0")
                {
                    at.ACCESS_GROUP_TYPEID = GetCellString(row, "ACCESS_GROUP_TYPEID");
                    at.ACCESS_GROUP = GetCellString(row, "ACCESS_GROUP");
                    at.ACCESS_GROUP_ID = GetCellString(row, "ACCESS_GROUP_ID");
                    at.ACCESS_TYPE = GetCellString(row, "ACCESS_TYPE");
                    at.ACCESS_TYPE_ID = GetCellString(row, "ACCESS_TYPE_ID");
                    trmpAccessType.Add(at);
                }
            }
            trmpAccessType.Sort((x, y) => string.Compare(x.ACCESS_TYPE, y.ACCESS_TYPE, StringComparison.OrdinalIgnoreCase));
            _groupedAccessType = trmpAccessType;
            return _groupedAccessType;
        }

        private IList<FindGroupedAccessTypes> _findGroupedAccessType;
        public IList<FindGroupedAccessTypes> GetFindAccessTypeByGroupID(string groupID)
        {
            var table = _mDsGlobal.Tables["GroupedAccessType"];
            if (table == null)
                table = _adt.callOVIDRPC("SDEC ACCGPTYG", null, "GroupedAccessType", _mDsGlobal);
            var tableAll = _mDsGlobal.Tables["AccessTypes"] ?? _adt.callOVIDRPC("SDEC ACCTYPE", null, "AccessTypes", _mDsGlobal);

            var trmpAccessType = new List<FindGroupedAccessTypes>();
            if (groupID == "0")
            {
                foreach (DataRow row in tableAll.Rows)
                {
                    var atAll = new FindGroupedAccessTypes
                    {
                        ACCESS_TYPE = GetCellString(row, "NAME"),
                        ACCESS_TYPE_ID = GetCellString(row, "SDEC_ACCESS_TYPE_IEN"),
                        IsChecked = false
                    };
                    trmpAccessType.Add(atAll);
                }
            }
            else
            {
                foreach (DataRow row in table.Rows)
                {
                    var at = new FindGroupedAccessTypes();

                    if (GetCellString(row, "ACCESS_GROUP_ID") == groupID)
                    {
                        at.ACCESS_TYPE = GetCellString(row, "ACCESS_TYPE");
                        at.ACCESS_TYPE_ID = GetCellString(row, "ACCESS_TYPE_ID");
                        at.IsChecked = false;
                        trmpAccessType.Add(at);
                    }
                }
            }
            trmpAccessType.Sort((x, y) => string.Compare(x.ACCESS_TYPE, y.ACCESS_TYPE, StringComparison.OrdinalIgnoreCase));
            _findGroupedAccessType = trmpAccessType;
            return _findGroupedAccessType;
        }

        public IList<SchdGroupedResources> GetResourcesByGroupID(string groupId)
        {
            var table = GetResourceGroupsData();

            var trmpResource = new List<SchdGroupedResources>();
            foreach (DataRow row in table.Rows)
            {
                var r = new SchdGroupedResources();

                if (GetCellString(row, "RESOURCE_GROUPID") == groupId)
                {
                    r.RESOURCE_GROUP = GetCellString(row, "RESOURCE_GROUP");
                    r.RESOURCE_GROUP_ITEMID = GetCellString(row, "RESOURCE_GROUP_ITEMID");
                    r.RESOURCE_GROUPID = GetCellString(row, "RESOURCE_GROUPID");
                    r.RESOURCE_ID = GetCellString(row, "RESOURCEID");
                    r.RESOURCE_NAME = GetCellString(row, "RESOURCE_NAME");
                    trmpResource.Add(r);
                }
            }
            trmpResource.Sort((x, y) => string.Compare(x.RESOURCE_NAME, y.RESOURCE_NAME, StringComparison.OrdinalIgnoreCase));
            var groupedResource = trmpResource;
            return groupedResource;
        }

        public IList<SchdResource> GetSchdResourcesByGroupID(string groupId)
        {
            var table = GetResourceGroupsData();

            var trmpResource = new List<SchdResource>();
            foreach (DataRow row in table.Rows)
            {
                if (GetCellString(row, "RESOURCE_GROUPID") == groupId)
                {
                    var resID = GetCellString(row, "RESOURCEID");
                    var schdRes = GetSdecResource(resID);
                    trmpResource.Add(schdRes);
                }
            }

            trmpResource.Sort((x, y) => string.Compare(x.RESOURCE_NAME, y.RESOURCE_NAME, StringComparison.OrdinalIgnoreCase));
            var groupedResource = trmpResource;
            return groupedResource;
        }

        private IList<SchdGroupedAccessTypes> _rgroupedAccessType;
        public IList<SchdGroupedAccessTypes> RemoveGroupedAccessTypeByID(string groupID, string groupedAccessTypeID)
        {
            var parms = new List<string> { groupID, groupedAccessTypeID };
            _adt.callOVIDRPC("SDEC DELAGI", parms);

            _mDsGlobal.Tables["GroupedAccessType"].Clear();
            //Get Grouped Access Types
            _adt.callOVIDRPC("SDEC ACCGPTYG", null, "GroupedAccessType", _mDsGlobal);
            _rgroupedAccessType = GetAccessTypeByGroupID(groupID);
            return _rgroupedAccessType;
        }

        private IList<SchdGroupedAccessTypes> _agroupedAccessType;
        public IList<SchdGroupedAccessTypes> AddAccessTypeToGroupByID(string groupID, string groupedAccessTypeID)
        {
            var parms = new List<string> { groupID, groupedAccessTypeID };
            _adt.callOVIDRPC("SDEC ADDAGI", parms);
            _mDsGlobal.Tables["GroupedAccessType"].Clear();
            //Get Grouped Access Types
            _adt.callOVIDRPC("SDEC ACCGPTYG", null, "GroupedAccessType", _mDsGlobal);
            _agroupedAccessType = GetAccessTypeByGroupID(groupID);
            return _agroupedAccessType;
        }

        public IList<NameValue> RemoveAccessTypeGroup(string groupID)
        {
            var parms = new List<string> { groupID };
            _adt.callOVIDRPC("SDEC DELAG", parms);

            //_mDsGlobal.Tables["AccessGroups"].Clear ();
            //Get Resource Groups
            //_adt.callOVIDRPC ("SDEC ACCGROUP", null, "AccessGroups", _mDsGlobal);
            _rGroups = GetAccessGroups();
            return _rGroups;
        }

        private IList<SchdGroupedResources> _rgroupedChinic;
        public IList<SchdGroupedResources> RemoveGroupedClinicByID(string groupID, string groupedClinicID)
        {
            var parms = new List<string> { groupID, groupedClinicID };
            var table = _adt.callOVIDRPC(Keys.RpcCalls.RemoveClnicGroup, parms);

            // Forces next Resource Group Data access to read the database
            Keys.Tables.ResourceGroupData = null;
            _rgroupedChinic = GetResourcesByGroupID(groupID);
            return _rgroupedChinic;
        }

        private IList<NameValue> _rGroups;
        public IList<NameValue> RemoveResourceGroup(string groupName)
        {
            var parms = new List<string> { groupName };
            _adt.callOVIDRPC("SDEC DELRESGP", parms);

            // Forces next Resource Group access to read the database
            Keys.Tables.ResourceGroupListData = null;
            Keys.Tables.ResourceGroupData = null;
            _rGroups = GetResourceGroupList();
            return _rGroups;
        }

        private IList<SchdGroupedResources> _agroupedChinic;
        public IList<SchdGroupedResources> AddClinicToGroupByID(string groupID, string groupedClinicID)
        {
            var parms = new List<string> { groupID, groupedClinicID };
            _adt.callOVIDRPC("SDEC ADDRGI", parms);

            // Forces next Resource Group Data access to read the database
            Keys.Tables.ResourceGroupData = null;
            _agroupedChinic = GetResourcesByGroupID(groupID);
            return _agroupedChinic;
        }

        private IList<NameValue> _accessGroups;
        public IList<NameValue> GetAccessGroups()
        {
            //var table = _mDsGlobal.Tables["AccessGroups"];
            var parm1 = new List<string>();
            var table = _adt.callOVIDRPC("SDEC ACCGROUP", parm1);
            var tempAccessGroups = new List<NameValue>();
            foreach (DataRow row in table.Rows)
            {
                var nameValue = new NameValue
                {
                    Value = GetCellString(row, "SDEC_ACCESS_GROUP_IEN"),
                    Name = GetCellString(row, "NAME")
                };
                tempAccessGroups.Add(nameValue);
            }
            tempAccessGroups.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
            _accessGroups = tempAccessGroups;
            return _accessGroups;
        }

        private IList<ResourceUser> _resourceUsers;
        public IList<ResourceUser> GetResourceUsers()
        {

            var parms = new List<string>();
            var table = _adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, parms);
            var tempResourceUsers = new List<ResourceUser>();
            foreach (DataRow row in table.Rows)
            {
                var rUser = new ResourceUser
                {
                    BSDX_RESOURCE_USER_IEN = GetCellString(row, "SDEC_RESOURCE_USER_IEN"),
                    RESOURCE_ID = GetCellString(row, "RESOURCE_ID"),
                    RESOURCE_NAME = GetCellString(row, "RESOURCE_NAME"),
                    OVERBOOK = GetCellString(row, "OVERBOOK"),
                    MODIFY_SCHEDULE = GetCellString(row, "MODIFY_SCHEDULE"),
                    MODIFY_APPTS = GetCellString(row, "MODIFY_APPTS"),
                    USERNAME = GetCellString(row, "USERNAME"),
                    USER_ID = GetCellString(row, "USER_ID"),
                    MASTEROVERBOOK = GetCellString(row, "MASTEROVERBOOK")
                };
                tempResourceUsers.Add(rUser);
            }
            tempResourceUsers.Sort((x, y) => string.Compare(x.RESOURCE_NAME, y.RESOURCE_NAME, StringComparison.OrdinalIgnoreCase));
            _resourceUsers = tempResourceUsers;
            return _resourceUsers;
        }

        private IList<ResourceUser> _rresourceUser;
        public IList<ResourceUser> RemoveResourceUserByID(string sdecResourceUserIen, string resourceID)
        {
            //var parms = new List<string> { sdecResourceUserIen };
            //_adt.callOVIDRPC ("SDEC DELRU", parms);
            //_mDsGlobal.Tables["ResourceUser"].Clear ();
            //var parm1 = new List<string> {resourceID};
            //_adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, null, "ResourceUser", _mDsGlobal);

            var parms = new List<string> { resourceID };
            var associatedUsers = _adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, parms);
            var newAssociationList = "";
            foreach (DataRow au in associatedUsers.Rows)
            {
                var resourceId = au["RESOURCE_ID"].ToString();
                var resUserIen = au["SDEC_RESOURCE_USER_IEN"].ToString();
                var userId = au["USER_ID"].ToString();
                if (resUserIen != sdecResourceUserIen)
                    newAssociationList += userId + ",";
            }
            newAssociationList = newAssociationList.TrimEnd(',');

            var parm = new List<string> { GetHospitalLocationIdByResourceIen(resourceID), newAssociationList };
            var rslt = _adt.callOVIDRPC(Keys.RpcCalls.UpdatePrivilegedUsers, parm);
            _rresourceUser = GetResourceUsersByResourceID(resourceID);
            return _rresourceUser;
        }

        public bool UpdatePrivilegedUsers(List<string> parms)
        {
            try
            {
                var rslt = _adt.callOVIDRPC(Keys.RpcCalls.UpdatePrivilegedUsers, parms);
                if (rslt.Rows.Count > 0)
                {
                    foreach (DataRow row in rslt.Rows)
                    {
                        var issue = row["ERRORTEXT"].ToString();
                        if (issue == "SUCCESSFUL")
                            return true;
                        MessageBox.Show(issue);
                        return false;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Update Privileged User Error");
                return false;
            }
            return false;
        }

        private IList<ResourceUser> _resourceUser;
        public IList<ResourceUser> GetResourceUsersByResourceID(string resourceID)
        {
            var parms = new List<string> { resourceID };
            var table = _adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, parms);

            var trmpUser = new List<ResourceUser>();
            foreach (DataRow row in table.Rows)
            {
                var r = new ResourceUser();

                if (GetCellString(row, "RESOURCE_ID") == resourceID)
                {
                    r.BSDX_RESOURCE_USER_IEN = GetCellString(row, "SDEC_RESOURCE_USER_IEN");
                    r.RESOURCE_ID = GetCellString(row, "RESOURCE_ID");
                    r.RESOURCE_NAME = GetCellString(row, "RESOURCE_NAME");
                    r.OVERBOOK = GetCellString(row, "OVERBOOK");
                    r.MODIFY_SCHEDULE = GetCellString(row, "MODIFY_SCHEDULE");
                    r.MODIFY_APPTS = GetCellString(row, "MODIFY_APPTS");
                    r.USERNAME = GetCellString(row, "USERNAME");
                    r.USER_ID = GetCellString(row, "USER_ID");
                    r.MASTEROVERBOOK = GetCellString(row, "MASTEROVERBOOK");
                    trmpUser.Add(r);
                }
            }
            trmpUser.Sort((x, y) => string.Compare(x.USERNAME, y.USERNAME, StringComparison.OrdinalIgnoreCase));
            _resourceUser = trmpUser;
            return _resourceUser;
        }

        public bool IsEnableUpdateAppointment(string sResourceId)
        {
            var isEnableUpdateAppointment = false;
            var users = GetResourceUsersByResourceID(sResourceId);
            foreach (var u in users)
            {
                if (u.USER_ID == CurrentUser)
                {
                    isEnableUpdateAppointment = u.MODIFY_APPTS == "YES";
                }
            }
            return isEnableUpdateAppointment; //This is obsolete.... with definition of Prohibited Clinics..

        }

        public bool IsEnableUpdateAvailability(string sResourceID)
        {
            var isEnableUpdateAvailability = false;
            var users = GetResourceUsersByResourceID(sResourceID);
            foreach (var u in users.Where(u => u.USER_ID == CurrentUser))
                isEnableUpdateAvailability = u.MODIFY_SCHEDULE == "YES";
            return isEnableUpdateAvailability;
        }

        public SchdAccessType GetAccessTypeByID(string accessTypeID)
        {
            var table = _mDsGlobal.Tables["AccessTypes"] ?? _adt.callOVIDRPC("SDEC ACCTYPE", null, "AccessTypes", _mDsGlobal);
            //Build Primary Key for AccessType table
            var accessTypeDc = new DataColumn[1];
            accessTypeDc[0] = table.Columns["SDEC_ACCESS_TYPE_IEN"];
            table.PrimaryKey = accessTypeDc;
            var row = table.Rows.Find(accessTypeID);
            var newAccessType = _container.Resolve<Factory<SchdAccessType>>().Create();
            if (row != null)
            {
                newAccessType.BSDX_ACCESS_TYPE_IEN = row["SDEC_ACCESS_TYPE_IEN"].ToString();
                newAccessType.NAME = row["NAME"].ToString();
                newAccessType.BLUE = row["BLUE"].ToString();
                newAccessType.DEPARTMENT_NAME = row["DEPARTMENT_NAME"].ToString();
                newAccessType.INACTIVE = row["INACTIVE"].ToString();
                newAccessType.DISPLAY_COLOR = row["DISPLAY_COLOR"].ToString();
                newAccessType.GREEN = row["GREEN"].ToString();
                newAccessType.PREVENT_ACCESS = row["PREVENT_ACCESS"].ToString();
                newAccessType.RED = row["RED"].ToString();
            }
            return newAccessType;
        }

        private IList<SchdAccessType> _accessTypes;
        public IList<SchdAccessType> GetAccessTypes()
        {

            var table = _mDsGlobal.Tables["AccessTypes"] ?? _adt.callOVIDRPC("SDEC ACCTYPE", null, "AccessTypes", _mDsGlobal);
            var tempAccessTypes = new List<SchdAccessType>();
            foreach (DataRow row in table.Rows)
            {
                var accessTypeValue = new SchdAccessType
                {
                    BSDX_ACCESS_TYPE_IEN = GetCellString(row, "SDEC_ACCESS_TYPE_IEN"),
                    NAME = GetCellString(row, "NAME"),
                    BLUE = GetCellString(row, "BLUE"),
                    DEPARTMENT_NAME = GetCellString(row, "DEPARTMENT_NAME"),
                    DISPLAY_COLOR = GetCellString(row, "DISPLAY_COLOR"),
                    GREEN = GetCellString(row, "GREEN"),
                    INACTIVE = GetCellString(row, "INACTIVE"),
                    PREVENT_ACCESS = GetCellString(row, "PREVENT_ACCESS"),
                    RED = GetCellString(row, "RED")
                };
                tempAccessTypes.Add(accessTypeValue);
            }
            tempAccessTypes.Sort((x, y) => string.Compare(x.NAME, y.NAME, StringComparison.OrdinalIgnoreCase));
            _accessTypes = tempAccessTypes;
            return _accessTypes;
        }

        private IList<NameValue> _clinicCancellationReasons;
        private IList<NameValue> _patientCancellationReasons;
        public IList<NameValue> GetCancellationReasons(string type)
        {
            if (type == "CLINIC" && _clinicCancellationReasons != null)
            {
                return _clinicCancellationReasons;
            }
            if (type == "PATIENT" && _patientCancellationReasons != null)
            {
                return _patientCancellationReasons;
            }

            var table = _adt.callOVIDRPC("SDEC CANREAS", null, "CancellationReasons", _mDsGlobal);

            var tempReasons = new List<NameValue>();
            foreach (DataRow row in table.Rows)
            {
                var nameValue = new NameValue
                {
                    Name = GetCellString(row, "NAME"),
                    Value = GetCellString(row, "CANCELLATION_REASONS_IEN")
                };
                var rowType = GetCellString(row, "TYPE");
                if (rowType == type || rowType == "BOTH")
                    tempReasons.Add(nameValue);
            }
            tempReasons.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
            if (type == "CLINIC")
                _clinicCancellationReasons = tempReasons;
            else if (type == "PATIENT")
                _patientCancellationReasons = tempReasons;
            return tempReasons;
        }

        private IList<NameValue> _summaryReport;
        public IList<NameValue> GetSummaryReport()
        {
            if (_summaryReport == null || _summaryReport.Count == 0)
            {
                var table = _adt.callOVIDRPC("SDEC PWH", null, "PWHTypes", _mDsGlobal);

                var tempSummaryReport = new List<NameValue>();
                foreach (DataRow row in table.Rows)
                {
                    var nameValue = new NameValue
                    {
                        Name = GetCellString(row, "PWH_TYPE_NAME"),
                        Value = GetCellString(row, "PWH_TYPE_IEN")
                    };
                    tempSummaryReport.Add(nameValue);
                }
                _summaryReport = tempSummaryReport;
            }
            return _summaryReport;
        }

        private IList<Provider> _providers;
        public IList<Provider> GetProviders()
        {
            if (_providers == null || _providers.Count == 0)
            {
                var parm1 = new List<string> { null, null, null, null, "0" };
                var table = _adt.callOVIDRPC(Keys.RpcCalls.GetProviders, parm1);

                var tempProviders = new List<Provider>();
                foreach (DataRow row in table.Rows)
                {
                    var newProvider = _container == null ? new Provider() : _container.Resolve<Factory<Provider>>().Create();
                    newProvider.Name = GetCellString(row, "NAME");
                    newProvider.IEN = GetCellString(row, "NEW_PERSON_IEN");
                    tempProviders.Add(newProvider);
                }
                tempProviders.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
                _providers = tempProviders;
            }
            return _providers;
        }

        public IList<Provider> GetProviders(string searchString)
        {
            var parm1 = new List<string>();

            if (string.IsNullOrEmpty(searchString))
                return GetProviders();

            parm1.Add(null);
            parm1.Add(searchString);
            parm1.Add(Keys.ProviderSearchMaxRecords.ToString());
            parm1.Add(null);
            parm1.Add("0");
            var table = _adt.callOVIDRPC(Keys.RpcCalls.GetProviders, parm1);
            var tempProviders = new List<Provider>();

            foreach (DataRow row in table.Rows)
            {
                var newProvider = _container == null ? new Provider() : _container.Resolve<Factory<Provider>>().Create();
                newProvider.Name = GetCellString(row, "NAME");
                newProvider.IEN = GetCellString(row, "NEW_PERSON_IEN");
                tempProviders.Add(newProvider);
            }

            tempProviders.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
            return tempProviders;
        }

        public DataTable GetWaitingList(string resourceIEN)
        {
            var parms = new List<string> { resourceIEN };
            return _adt.callOVIDRPC("SDEC WAITLIST", parms);
        }

        public string AddEditAccessType(SchdAccessType accessType)
        {
            var parms = new List<string>();
            var parm = accessType.BSDX_ACCESS_TYPE_IEN ?? "0";
            parm = parm + "|" + accessType.NAME + "|" + accessType.INACTIVE + "|" + accessType.DISPLAY_COLOR + "|" + accessType.RED + " | " + accessType.GREEN + " | " + accessType.BLUE + "|" + accessType.PREVENT_ACCESS;
            parms.Add(parm);
            var table = _adt.callOVIDRPC("SDEC ADDACCTY", parms);
            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                sErrorId = r["ERRORTEXT"].ToString() != "-1" ? r["ERRORTEXT"].ToString() : string.Empty;
            }
            if (sErrorId == string.Empty)
                _adt.callOVIDRPC("SDEC ACCTYPE", null, "AccessTypes", _mDsGlobal);
            return sErrorId;
        }

        public string AddEditResourceUser(ResourceUser resourceUser, bool isInitialLoadOfPorhibitedClinics = false)
        {
            var sErrorId = string.Empty;
            var parm1 = new List<string>();
            var resourceUsersForClinic = new List<string>();
            parm1.Add(resourceUser.RESOURCE_ID);
            var resUsers = _adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, parm1);

            foreach (DataRow dr in resUsers.Rows)
            {
                var resourceId = dr["RESOURCE_ID"].ToString();
                var userId = dr["USER_ID"].ToString();
                if (resourceId == resourceUser.RESOURCE_ID)
                    resourceUsersForClinic.Add(userId);
            }
            resourceUsersForClinic.Add(resourceUser.USER_ID);
            var parm2 = new List<string> { GetHospitalLocationIdByResourceIen(resourceUser.RESOURCE_ID) };
            var parameter2 = "";
            foreach (var ien in resourceUsersForClinic)
                parameter2 += ien + ",";
            parameter2 = parameter2.TrimEnd(',');

            var items = parameter2.Split(',');
            parameter2 = "";
            var uniqueItems = new HashSet<string>(items);
            foreach (var s in uniqueItems)
                parameter2 += s + ",";
            parameter2 = parameter2.TrimEnd(',');
            parm2.Add(parameter2);
            var rslt = _adt.callOVIDRPC(Keys.RpcCalls.UpdatePrivilegedUsers, parm2);

            return sErrorId;
        }

        public string AddEditAccessTypeGroup(string groupID, string groupName)
        {
            var parms = new List<string>();
            var parm = groupID == string.Empty ? "0" : groupID;
            parm = parm + "|" + groupName;
            parms.Add(parm);
            var table = _adt.callOVIDRPC("SDEC ADDACCG", parms);
            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                sErrorId = r["ERRORTEXT"].ToString();
            }
            //if (sErrorId == string.Empty)
            //_adt.callOVIDRPC ("SDEC ACCGROUP", null, "AccessGroups", _mDsGlobal);
            return sErrorId;
        }

        public string AddEditClinicGroup(string groupID, string groupName)
        {
            var parms = new List<string>();
            var parm = groupID == string.Empty ? "0" : groupID;
            parm = parm + "|" + groupName;
            parms.Add(parm);
            var table = _adt.callOVIDRPC("SDEC ADDRG", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                sErrorId = r["ERRORTEXT"].ToString();
            }

            if (sErrorId == string.Empty)
            {
                // Forces next Resource Group access to read the database
                Keys.Tables.ResourceGroupData = null;
                Keys.Tables.ResourceGroupListData = null;
            }

            return sErrorId;
        }

        public string AddEditClinic(SchdResource schdResource)
        {
            var parms = new List<string>();
            var parm = schdResource.RESOURCEID + "|" +
                       schdResource.Clinic.HOSPITAL_LOCATION + "|" +
                       /* NOT USED */ "|" +
                       schdResource.Clinic.HOSPITAL_LOCATION_ID + "|" +
                       schdResource.TIMESCALE + "|" +
                       (string.IsNullOrEmpty(schdResource.LETTER_TEXT) ? "" : schdResource.LETTER_TEXT).TrimEnd(Environment.NewLine.ToCharArray()) + "|" +
                       (string.IsNullOrEmpty(schdResource.NO_SHOW_LETTER) ? "" : schdResource.NO_SHOW_LETTER).TrimEnd(Environment.NewLine.ToCharArray()) + "|" +
                       (string.IsNullOrEmpty(schdResource.CLINIC_CANCELLATION_LETTER) ? "" : schdResource.CLINIC_CANCELLATION_LETTER).TrimEnd(Environment.NewLine.ToCharArray()) + "|" +
                       "|" +
                       "|" +
                       (string.IsNullOrEmpty(schdResource.InactiveDate) ? "" : schdResource.InactiveDate) + "|" +
                       (string.IsNullOrEmpty(schdResource.InactiveUserIEN) ? "" : schdResource.InactiveUserIEN) + "|" +
                       (string.IsNullOrEmpty(schdResource.ReactivatedDate) ? "" : schdResource.ReactivatedDate) + "|" +
                       (string.IsNullOrEmpty(schdResource.ReactivatedUserIEN) ? "" : schdResource.ReactivatedUserIEN) + "|" +
                       (string.IsNullOrEmpty(schdResource.ResourceTypeCode) ? "H" : schdResource.ResourceTypeCode) + "|" +
                       (string.IsNullOrEmpty(schdResource.ResourceTypeIEN) ? schdResource.Clinic.HOSPITAL_LOCATION_ID : schdResource.ResourceTypeIEN);

            parms.Add(parm);
            var table = _adt.callOVIDRPC("SDEC ADDRES", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["RESOURCEID"].ToString() == "0")
                    sErrorId = r["ERRORTEXT"].ToString();
                //else
                //LoadResources();
            }
            else
                sErrorId = "Unable to " + (schdResource.RESOURCEID == "0" ? "add new" : "update") + " Resource.";
            return sErrorId;
        }

        public string DeleteClinic(string clinicID)
        {
            var parms = new List<string> { clinicID + "|@" };
            var table = _adt.callOVIDRPC("SDEC ADDRES", parms);

            var sErrorId = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["ERRORTEXT"].ToString() != string.Empty)
                    sErrorId = r["ERRORTEXT"].ToString();
            }

            return sErrorId;
        }

        public string AddNewAvailability(SchdAvailability a)
        {
            var resourceAvailabilityList = _mGlobalAvailabilities[a.RESOURCENAME];

            if (string.IsNullOrEmpty(a.ACCESSTYPEID))
                return "Null ACCESSTYPEID is invalid.\nThis functionality is not fully implemented.";
            a.ACCESSTYPENAME = AccessNameFromID(int.Parse(a.ACCESSTYPEID));
            a.IsPreventAccess = IsPreventAccessFromID(int.Parse(a.ACCESSTYPEID));
            if (!a.IsPreventAccess)
            {
                foreach (SchdAvailability resourceAvailability in resourceAvailabilityList)
                {
                    var sStart = Convert.ToDateTime(resourceAvailability.StartTime);
                    var sEnd = Convert.ToDateTime(resourceAvailability.EndTime);
                    if (Keys.Methods.TimesOverlap(Convert.ToDateTime(a.StartTime), Convert.ToDateTime(a.EndTime), sStart, sEnd) &&
                        !IsPreventAccessFromID(int.Parse(resourceAvailability.ACCESSTYPEID.ToString(CultureInfo.InvariantCulture))))
                        return "Access blocks may not overlap.";
                }
            }

            var sErrorId = string.Empty;
            var parms = new List<string>
            {
                Convert.ToDateTime( a.StartTime ).ToString( "M-d-yyyy@HH:mm" ),
                Convert.ToDateTime( a.EndTime ).ToString( "M-d-yyyy@HH:mm" ),
                a.ACCESSTYPEID,
                a.RESOURCENAME,
                a.SLOTS.ToString(),
                a.Note
            };
            var table = _adt.callOVIDRPC("SDEC AVADD", parms);

            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["ERRORID"].ToString() != "-1")
                    sErrorId = r["ERRORID"].ToString();
            }
            return sErrorId;
        }

        private string AccessNameFromID(int nAccessTypeID)
        {
            var sAccessTypeName = string.Empty;
            var dvTypes = new DataView(GlobalDataSet.Tables["AccessTypes"])
            {
                Sort = "SDEC_ACCESS_TYPE_IEN ASC"
            };
            var nRow = dvTypes.Find(nAccessTypeID);
            if (nRow >= 0)
            {
                var drv = dvTypes[nRow];
                sAccessTypeName = drv["NAME"].ToString();
            }
            return sAccessTypeName;
        }

        private bool IsPreventAccessFromID(int nAccessTypeID)
        {
            var bIsPreventAccess = false;
            var dvTypes = new DataView(GlobalDataSet.Tables["AccessTypes"])
            {
                Sort = "SDEC_ACCESS_TYPE_IEN ASC"
            };
            var nRow = dvTypes.Find(nAccessTypeID);
            if (nRow >= 0)
            {
                var drv = dvTypes[nRow];
                var sIsPreventAccess = drv["PREVENT_ACCESS"].ToString();
                bIsPreventAccess = sIsPreventAccess == "YES";
            }
            return bIsPreventAccess;
        }

        public bool CanAddNewAppointmentPreCheck(string command, SchdAppointment appointment, out bool overbookAppointment, out bool isTelehealth, string conflictExcludeAppointmentID = "", bool allowConflicts = true)
        {
            overbookAppointment = false;
            isTelehealth = false;
            DateTime startTime = Convert.ToDateTime(appointment.START_TIME);

            if (appointment.SelectedResource != null)
            {
                //  Check if attempting to book on a holiday
                string errorMessage = IsHoliday(startTime.Date);

                if (!string.IsNullOrEmpty(errorMessage))
                {
                    if (!appointment.SelectedResource.Clinic.ClinicCanBookOnHolidays)
                    {
                        string msg = startTime.ToShortDateString() + " is a holiday (" + errorMessage + ") and is not allowed to be scheduled.";
                        Keys.WarnUser(null, msg, "Holiday Booking");
                        return false;
                    }
                }
            }

            // Check for conflicts, warning only, proceed even if conflicts exist
            if (CheckConflictAppointment(command, appointment.RESOURCENAME, appointment.PATIENTID, appointment.PATIENTNAME, appointment.START_TIME, appointment.END_TIME, out isTelehealth, conflictExcludeAppointmentID))
                if (!allowConflicts)
                    return false;

            //Check No Shows
            if (Convert.ToDateTime(appointment.START_TIME).Date > DateTime.Now.Date)
            {
                string warningMessage = NoShow(appointment.PATIENTID, appointment.RESOURCENAME, appointment.ResourceId, command);

                if (warningMessage != string.Empty)
                    Keys.WarnUser(null, warningMessage, "Patient No-Shows");
            }

            // Overbook check
            switch (ClinicOverbook(appointment))
            {
                case Infrastructure.Keys.OverbookingStatus.Cannot:
                    return false;
                case Infrastructure.Keys.OverbookingStatus.NotRequired:
                    break;
                case Infrastructure.Keys.OverbookingStatus.Required:
                    overbookAppointment = true;
                    break;
            }

            return true;
        }

        public List<string> AddNewAppointment(AppointmentData a, PatientInformation p, bool bMakeChartRequest, bool isMoveAppt = false)
        {
            var resId = a.ResourceName.ToUpper();
            var res = GetSdecResourceByName(a.ResourceName);

            if (res != null)
                resId = res.RESOURCEID;

            var parms = new List<string>();
            //  1. Start Date\Time
            parms.Add(Convert.ToDateTime(a.StartTime).ConvertToExternalDateTimeFormat(true, true));
            //  2. End Date\Time
            parms.Add(Convert.ToDateTime(a.EndTime).ConvertToExternalDateTimeFormat(true, true));
            //  3. Patient IEN
            parms.Add(p.IEN);
            //  4. Resource Name \ ID
            parms.Add(resId);
            //  5. Duration in Minutes
            parms.Add(a.Duration);
            //  6. Note (Max 150 characters)
            parms.Add(a.Notes);
            //  7. "WALKIN" for WalkIn Appt or ## = Access Type ID for Rebooking
            parms.Add(a.AppointmentID);
            //  8. WALKIN Routing Slip Flag
            parms.Add(bMakeChartRequest ? "1" : "0");
            //  9. MRTC Flag
            parms.Add(null);

            // 10. Desired Date of Appointment
            if (a.Appointment != null)
            {
                var desiredDate = a.Appointment.DesiredAppointmentDate;
                parms.Add(desiredDate.HasValue ? desiredDate.Value.ConvertToExternalDateTimeFormat(true, true) : "");
            }
            else
                parms.Add(null);

            // 11. Requested By ("PROVIDER"\"PATIENT")
            parms.Add(a.Appointment.AppointmentRequest.RequestedByText);
            // 12. LAB Date/Time
            parms.Add(null);
            // 13. Provider IEN, pointer to nEW PERSON file
            parms.Add(null);
            // 14. External ID
            parms.Add(null);

            // 15. Request Type
            if ((string.IsNullOrEmpty(a.RequestType) || string.IsNullOrEmpty(a.AppointmentRequestIen)) && !string.IsNullOrEmpty(a.AppointmentID))
            {
                var parms2 = new List<string> { a.AppointmentID };
                var rslt = _adt.callOVIDRPC(Keys.RpcCalls.GetRequestId, parms2);
                foreach (DataRow dr in rslt.Rows)
                {
                    var typeIen = dr["SDAPTYP"].ToString();
                    var pieces = typeIen.Split('|');
                    a.RequestType = pieces[0];
                    a.AppointmentRequestIen = pieces[1];
                }
            }

            if (!string.IsNullOrEmpty(a.RequestType) && !string.IsNullOrEmpty(a.AppointmentRequestIen))
            {
                switch (a.RequestType)
                {
                    case "E":
                        parms.Add("E|" + a.AppointmentRequestIen);
                        break;
                    case "V":
                    case "A":
                        parms.Add("A|" + a.AppointmentRequestIen);
                        break;
                    case "R":
                        parms.Add("R|" + a.AppointmentRequestIen);
                        break;
                    case "P":
                    case "C":
                        parms.Add("C|" + a.AppointmentRequestIen);
                        break;
                }
            }
            else
                parms.Add(null);

            // 16. Service Connected Priority (YES\NO)
            parms.Add((a.IsServiceConnected ? "YES" : "NO"));
            // 17. Service Connected Percentage (Numeric)
            parms.Add(null);    //parms.Add(a.ServicePercentage);
            // 18. HOSP_LOC_ID \ Clinic IEN
            // The apptRequest.ClinicIen is the Clinic assigned to the Request.  It MAY NOT be the clinic to which the appointment is made.
            //  Se we get the Clinic ID \ HOSPITAL_LOCATION_ID from the resource object retrieved at the start of this method
            string hospLocID = null;

            if (a.Appointment != null && a.Appointment.AppointmentRequest != null)
                hospLocID = a.Appointment.AppointmentRequest.ClinicIen;

            parms.Add(hospLocID); // apptRequest.ClinicIen);
            // 19. EKG Date/Time
            parms.Add(null);
            // 20. XRay Date/Time
            parms.Add(null);
            // 21. Appointment Type IEN
            parms.Add(a.AppointmentTypeIen);
            // 22. Patient Status ("NEW", "ESTABLISHED")
            string patientStatus = null;

            if (a.Appointment != null && a.Appointment.AppointmentRequest != null)
                patientStatus = a.Appointment.AppointmentRequest.EnrolleeStatus;

            parms.Add(patientStatus);
            // 23.  OVB (Optional) overbook flag
            parms.Add(a.IsOverbook ? "1" : "0");
            // 24. MRTC Parent Request IEN
            string mrtcParentRequestIEN = null;
            // 25. Eligibility IEN of appointment
            parms.Add(a.EligibilityIen);

            if (a.Appointment != null && a.Appointment.AppointmentRequest != null)
                mrtcParentRequestIEN = a.Appointment.AppointmentRequest.MRTCParentRequestIEN;

            parms.Add(mrtcParentRequestIEN);    // MRTC Parent Request IEN
            var returnData = new List<string>();
            DataTable table;

            if (AddNewAppointment(parms, out table))
            {
                if (table.Rows.Count == 1)
                {
                    var r = table.Rows[0];
                    returnData.Add(r["APPOINTMENTID"].ToString());
                    returnData.Add(r["ERRORID"].ToString());
                }

                if (!isMoveAppt && !string.IsNullOrEmpty(a.AppointmentRequestIen) && !string.IsNullOrEmpty(a.RequestType) && Keys.Methods.IsNumeric(returnData[0]))
                {
                    // Request Type A: Appt, E:EWL, V:Veteran
                    if (a.RequestType == "A" || a.RequestType == "E" || a.RequestType == "V")
                    {
                        _eventAggregator.GetEvent<RemoveRequestEvent>().Publish(a.AppointmentRequestIen);
                    }
                }

                // Clear the Future Appointments Cache
                ClearPatientFutureAppointmentsCache(p.IEN);
            }
            else
                returnData.Add("Unable to overbook the appointment");

            return returnData;
        }

        public List<string> AddNewAppointment(AppointmentData a, PatientInformation p, bool bMakeChartRequest, AppointmentRequest apptRequest)
        {
            var resId = a.ResourceName.ToUpper();
            var res = GetSdecResourceByName(a.ResourceName);

            if (res != null)
                resId = res.RESOURCEID;

            var isConsult = false;
            var newCalc = false;
            Keys.MrtcCloseRequestIen = "";
            var parms = new List<string>
            {
                Convert.ToDateTime( a.StartTime ).ConvertToExternalDateTimeFormat( true, true ),            //  1. Start Date\Time
                Convert.ToDateTime( a.EndTime ).ConvertToExternalDateTimeFormat( true, true ),            //  2. End Date\Time
                p.IEN,   //  3. Patient IEN
                resId,    //  4. Resource Name
                a.Duration,    //  5. Duration in Minutes
                a.Notes,    //  6. Note (Max 150 characters)
                a.AppointmentID,   //  7. "WALKIN" for WalkIn Appt or ## = Access Type ID for Rebooking
                bMakeChartRequest ? "1" : "0",   //  8. WALKIN Routing Slip Flag
                apptRequest.MultipleAppointmentsRequired ? "True" : "False"   //  9. MRTC Flag
            };

            // 10. Desired Date of Appointment
            parms.Add(apptRequest.DesiredApptDate != null ? apptRequest.DesiredApptDate.Value.ToShortDateString() : null);
            // 11. Requested By
            parms.Add(apptRequest.RequestedByText);
            // 12. LAB Date/Time
            parms.Add(null);
            // 13. Provider IEN
            if (apptRequest.Provider != null)
                parms.Add(apptRequest.Provider.IEN);
            else
                parms.Add(null);
            // 14. External ID
            parms.Add(null);
            // 15. Appointment Request Type
            switch (apptRequest.AppointmentRequestType)
            {
                case Keys.AppointmentRequestTypes.Ewl:
                    parms.Add("E|" + apptRequest.RequestManagmentIen);
                    break;
                case Keys.AppointmentRequestTypes.Appt:
                case Keys.AppointmentRequestTypes.Veteran:
                case Keys.AppointmentRequestTypes.WelcomeToVa:
                case Keys.AppointmentRequestTypes.ReturnToClinic:
                    parms.Add("A|" + apptRequest.ApptRequestIen);
                    break;
                case Keys.AppointmentRequestTypes.PtCSch:
                    parms.Add("R|" + apptRequest.PtCSchRequestIen);
                    break;
                case Keys.AppointmentRequestTypes.Procedure:
                case Keys.AppointmentRequestTypes.Consult:
                    parms.Add("C|" + apptRequest.ConsultationRequestIen);
                    isConsult = true;
                    break;
            }
            // 16. Service Connected Priority (YES\NO)
            parms.Add((a.IsServiceConnected ? "YES" : "NO"));
            // 17. Service Connected Percentage (Numeric)
            parms.Add(null);    //parms.Add(a.ServicePercentage);
            // 18. HOSP_LOC_ID \ Clinic IEN
            // The apptRequest.ClinicIen is the Clinic assigned to the Request.  It MAY NOT be the clinic to which the appointment is made.
            //  Se we get the Clinic ID \ HOSPITAL_LOCATION_ID from the resource object retrieved at the start of this method
            parms.Add(res != null ? res.HOSPITAL_LOCATION_ID : null); // apptRequest.ClinicIen);
            // 19. EKG Date/Time
            parms.Add(null);
            // 20. XRay Date/Time
            parms.Add(null);
            // 21. Appointment Type IEN
            parms.Add(a.AppointmentTypeIen);
            // 22. Patient Status ("NEW", "ESTABLISHED")
            parms.Add(apptRequest.EnrolleeStatus);
            // 23.  OVB (Optional) overbook flag
            parms.Add(apptRequest.MultipleAppointmentsRequired ? "1" : (a.IsOverbook ? "1" : "0"));
            // 24. MRTC Parent Request IEN
            parms.Add(string.IsNullOrEmpty(apptRequest.MRTCParentRequestIEN) ? string.Empty : apptRequest.MRTCParentRequestIEN);    // MRTC Parent Request IEN
            // 25. Eligibility IEN of appointment
            parms.Add(a.EligibilityIen);


            var returnData = new List<string>();
            DataTable table;

            if (!AddNewAppointment(parms, out table))
            {
                returnData.Add("0");
                returnData.Add("OVERBOOK FAILED");
                return returnData;
            }

            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                returnData.Add(r["APPOINTMENTID"].ToString());
                returnData.Add(r["ERRORID"].ToString());
            }

            if (a.AppointmentTypeIen == "1") //Comp and Pen appt
            {
                string errorCode;
                var compAndPenRequestList = new List<CompAndPensionRequest>();
                var parameters = new List<string>();
                var hospId = "";
                parameters.Add(p.IEN);
                var tbl = GetClinicSetupParameters(null, a.ResourceName);
                if (tbl == null)
                {
                    MessageBox.Show("Invalid clinic: " + a.ResourceName);
                    return null;
                }
                foreach (DataRow dr in tbl.Rows)
                {
                    hospId = dr["HOSPITAL_LOCATION_ID"].ToString();
                    parameters.Add(hospId);
                }
                var stTime = ConvertDateTimeToExternalMumpsFormat(Convert.ToDateTime(a.StartTime));
                parameters.Add(stTime);
                parameters.Add("O");
                parameters.Add("1");
                var rslt = _adt.callOVIDRPC(Keys.RpcCalls.SdecCapGet, parameters);
                foreach (DataRow dr in rslt.Rows)
                {
                    var compAndPenRequest = new CompAndPensionRequest
                    {
                        ReqIen = dr["REQIEN"].ToString(),
                        Ien = dr["IEN"].ToString(),
                        Name = dr["NAME"].ToString(),
                        Date = dr["DATE"].ToString(),
                        Exists = Keys.Methods.StrToBool(dr["EXISTS"].ToString()),
                        AppointmentLinksString = dr["APPT_LINKS"].ToString(),
                    };
                    if (compAndPenRequest.Exists)
                    {
                        var apptLinks = compAndPenRequest.AppointmentLinksString.Split('|');
                        compAndPenRequest.AppointmentLinks = new List<CompAndPensionRequestAppointmentLink>();
                        foreach (var link in apptLinks)
                        {
                            var cpApptLink = new CompAndPensionRequestAppointmentLink();
                            var pieces = link.Split('~');
                            cpApptLink.LinkId = pieces[0];
                            cpApptLink.InitialAppointment = pieces[1];
                            cpApptLink.ClockStopAppointment = pieces[2];
                            cpApptLink.CurrentAppointment = pieces[3];
                            cpApptLink.ClinicName = pieces[4];
                            compAndPenRequest.AppointmentLinks.Add(cpApptLink);
                        }
                    }
                    compAndPenRequestList.Add(compAndPenRequest);
                }

                if (compAndPenRequestList.Count == 0)
                    MessageBox.Show("You have made a C&P appointment for a patient who has no pending 2507 request!");
                else if (compAndPenRequestList.Count == 1 && !compAndPenRequestList[0].Exists)
                {
                    var prms = new List<string> { compAndPenRequestList[0].ReqIen, null, null, hospId, stTime };
                    var setRslt = _adt.callOVIDRPC(Keys.RpcCalls.SdecCapSet, prms);
                    foreach (DataRow dr in setRslt.Rows)
                    {
                        errorCode = dr["CODE"].ToString();
                        var errorMessage = dr["MESSAGE"].ToString();
                        if (Convert.ToInt32(errorCode) < 0)
                            MessageBox.Show(errorMessage);
                        else
                        {
                            var msg = "AMIE C&P EXAM TRACKING record updated.";
                            MessageBox.Show(msg);
                        }
                        break;
                    }
                }
                else
                {
                    var compAndPensionRequestsView = new CompAndPensionRequestView
                    {
                        CompAndPensionRequests = compAndPenRequestList
                    };
                    CLaunchWindow.LaunchWindow(compAndPensionRequestsView, "Comp & Pension Requests", 350, 550, "CompAndPensionrequests", true);
                    if (compAndPensionRequestsView.SelectedCompAndPensionRequest == null)
                    {
                        //NO SET
                        MessageBox.Show(
                            "You have not selected a 2507 request to link the C&P appointment to. The appointment should be linked with the AMIE/C&P Appointment Link Management Option to ensure proper processing time calculation for this 2507 in the event of a veteran cancellation.");
                    }
                    else
                    {
                        if (!compAndPensionRequestsView.SelectedCompAndPensionRequest.Exists)
                        {
                            var prms = new List<string> { compAndPensionRequestsView.SelectedCompAndPensionRequest.ReqIen, null, null, hospId, stTime };
                            var setRslt = _adt.callOVIDRPC(Keys.RpcCalls.SdecCapSet, prms);
                            foreach (DataRow dr in setRslt.Rows)
                            {
                                errorCode = dr["CODE"].ToString();
                                var errorMessage = dr["MESSAGE"].ToString();
                                if (Convert.ToInt32(errorCode) < 0)
                                    MessageBox.Show(errorMessage);
                                else
                                {
                                    var msg = "AMIE C&P EXAM TRACKING record updated.";
                                    MessageBox.Show(msg);
                                }
                                break;
                            }
                        }
                        else if (Keys.DisplayAmieCandPPrompts)
                        {
                            //Display Appointment Links... in selected Request..
                            var mbRslt = MessageBox.Show("This 2507 already has appointments. Is this appointment due to a cancellation?", "Comp & Pension Request",
                                MessageBoxButton.YesNo);
                            if (mbRslt == MessageBoxResult.No)
                            {
                                var prms2 = new List<string> { compAndPensionRequestsView.SelectedCompAndPensionRequest.ReqIen, null, "0", hospId, stTime };
                                //set to null so that new appointment link is created per ST_1_7_2016
                                var sRslt = _adt.callOVIDRPC(Keys.RpcCalls.SdecCapSet, prms2);
                                foreach (DataRow dr in sRslt.Rows)
                                {
                                    errorCode = dr["CODE"].ToString();
                                    var errorMessage = dr["MESSAGE"].ToString();
                                    if (Convert.ToInt32(errorCode) < 0)
                                        MessageBox.Show(errorMessage);
                                    else
                                        MessageBox.Show(errorMessage);
                                    break;
                                }
                            }
                            else if (mbRslt == MessageBoxResult.Yes)
                            {
                                //MessageBox.Show(compAndPensionRequestsView.SelectedCompAndPensionRequest.AppointmentLinksString);
                                var apptLinkView = new CompAndPenRequestAppointmentLinksView
                                {
                                    CmpAndPensRequest = compAndPensionRequestsView.SelectedCompAndPensionRequest
                                };
                                apptLinkView.ApptLinks = apptLinkView.CmpAndPensRequest.AppointmentLinks;
                                CLaunchWindow.LaunchWindow(apptLinkView, "Comp & Pension Request Appointment Links", 350, 550, "CompAndPensionRequestApptLinks", true);
                                if (apptLinkView.SelectedCpReqApptLink != null)
                                {
                                    var mbRslt2 = MessageBox.Show("Is this appointment due to a veteran requested cancellation or 'No Show'", "C&P Appt Links", MessageBoxButton.YesNo);
                                    var prms3 = new List<string> { compAndPensionRequestsView.SelectedCompAndPensionRequest.ReqIen };
                                    if (compAndPensionRequestsView.SelectedCompAndPensionRequest.Exists)
                                    {
                                        var selectedApptLink = apptLinkView.SelectedCpReqApptLink.LinkId + "~" +
                                                                apptLinkView.SelectedCpReqApptLink.InitialAppointment +
                                                                "~" +
                                                                apptLinkView.SelectedCpReqApptLink.ClockStopAppointment +
                                                                "~" +
                                                                apptLinkView.SelectedCpReqApptLink.CurrentAppointment +
                                                                "~" + apptLinkView.SelectedCpReqApptLink.ClinicName;
                                        prms3.Add(selectedApptLink);
                                    }
                                    else
                                        prms3.Add(null);
                                    prms3.Add("1");
                                    prms3.Add(hospId);
                                    prms3.Add(stTime);
                                    var stRslt = _adt.callOVIDRPC(Keys.RpcCalls.SdecCapSet, prms3);
                                    foreach (DataRow dr in stRslt.Rows)
                                    {
                                        errorCode = dr["CODE"].ToString();
                                        var errorMessage = dr["MESSAGE"].ToString();
                                        if (Convert.ToInt32(errorCode) < 0)
                                            MessageBox.Show(errorMessage);
                                        else
                                        {
                                            string msg;
                                            if (mbRslt2 == MessageBoxResult.Yes)
                                                msg = "AMIE C&P EXAM TRACKING record updated.";
                                            else
                                            {
                                                var dtStr = compAndPensionRequestsView.SelectedCompAndPensionRequest.Date.Split('@');
                                                var compAndPenReqDate = dtStr.Length > 1 ? dtStr[0] : compAndPensionRequestsView.SelectedCompAndPensionRequest.Date;
                                                var curApptInfo = apptLinkView.SelectedCpReqApptLink.CurrentAppointment.Split('@');
                                                var curAppt = curApptInfo.Length > 1 ? curApptInfo[0] : apptLinkView.SelectedCpReqApptLink.CurrentAppointment;
                                                msg = "Adjusting C&P appointment link for 2507 request dated " + compAndPenReqDate + ". Remember to cancel the appointment for " + curAppt + ".";

                                            }
                                            MessageBox.Show(msg);
                                        }
                                        break;
                                    }
                                }
                                else
                                {
                                    MessageBox.Show("You have not selected the linked appointment being rescheduled. You may need to adjust the link to the appointment with the AMIE link management option to ensure proper processing time calculation for this 2507.");
                                }
                            }
                        }
                    }
                }
            }

            if (string.IsNullOrEmpty(returnData[1]))
                a.AppointmentID = returnData[0];

            // Deal with MRTC Issues
            if (apptRequest != null)
                // Is MRTC
                if (apptRequest.MultipleAppointmentsRequired && apptRequest.IsMRTCChild)
                {
                    // Update the Appointment Request with the booked appointment ID
                    SetMRTCChildRequest(apptRequest, apptRequest.ApptRequestIen.ToString(), a.AppointmentID);
                }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(p.IEN);

            return returnData;
        }

        private bool AddNewAppointment(List<string> parms, out DataTable table)
        {
            bool result = true;
            bool continueProcessing = false;

            do
            {
                continueProcessing = false;
                table = _adt.callOVIDRPC(Keys.RpcCalls.AppAdd, parms);
                var returnData = new List<string>();
                if (table.Rows.Count == 1)
                {
                    var r = table.Rows[0];
                    string appointmentID = r["APPOINTMENTID"].ToString();
                    string errorID = r["ERRORID"].ToString();

                    if ((appointmentID == "-2") || (appointmentID == "0"))
                    {
                        if (errorID.StartsWith("OBM"))
                        {
                            string[] parts = errorID.Split('|');
                            string obmErrorID = parts[0].Substring(3);

                            switch (obmErrorID)
                            {
                                case "-1":
                                    // An error occurred
                                    throw new Exception("There was a problem adding the new appointment.\n" + parts[1]);
                                case "0":
                                    // Overbook is required and the user cannot overbook
                                    Keys.AlertUser(null, parts[1], "Overbook");
                                    result = false;
                                    break;
                                case "1":
                                    // Overbook is not required
                                    break;
                                case "2":
                                    // Overbook is required, the user CAN overbook, ask the user if that is what is desired
                                    if (Keys.ConfirmUser(null, parts[2], "Overbook"))
                                    {
                                        // The user said yes, set the overbook flag and reprocess
                                        // parms piece 23 is the overbook flag, so we set param[22] in the 0 based list
                                        parms[22] = "1";
                                        continueProcessing = true;
                                    }

                                    else
                                        // The user said NO, set result to cannot and return
                                        result = false;

                                    break;
                            }
                        }
                    }
                }
            } while (continueProcessing);

            return result;
        }

        public int ClosestToZero(List<double> haystack)
        {
            var closest = haystack[0];
            foreach (double item in haystack)
            {
                if (Math.Abs(item - 0) < Math.Abs(closest - 0)) closest = item;
            }
            var j = 0;
            foreach (var ts in haystack)
            {
                if (ts == closest)
                    return j;
                j++;
            }
            return 0;
        }

        private DateTime? AvailabilityStartTime(SlotsAvailable slotsAvailable, DateTime? startTime = null)
        {
            CSlot firstSlot;

            if (startTime.HasValue)
                firstSlot = slotsAvailable.Slots.FirstOrDefault(s => s.AccessType != Keys.AccessTypes.Unavailable && s.Date.Value.CompareTo(startTime.Value.Date) == 0);
            else
                firstSlot = slotsAvailable.Slots.FirstOrDefault(s => s.AccessType != Keys.AccessTypes.Unavailable);

            if (firstSlot == null)
                return null;
            return firstSlot.StartTime;
        }

        private DateTime? AvailabilityStartTime(SchdResource resource, DateTime sStartTime)
        {
            DateTime? result = null;

            try
            {
                if (resource != null)
                {
                    var availability = GetAppSlots(resource, sStartTime, sStartTime);
                    result = AvailabilityStartTime(availability);
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "AvailabilityStartTime(Resource, StartTime)", ex);
            }

            return result;
        }

        public string CheckSlotsAvailable(string startTime, string endTime, string resourceName)
        {
            var res = GetSdecResourceByName(resourceName);
            return CheckSlotsAvailable(res, startTime, endTime);
        }

        public string CheckSlotsAvailable(SchdResource resource, string startTime, string endTime)
        {
            var sErrorID = string.Empty;
            string sAccessType;
            string sAvailabilityMessage;
            var dStartTime = Convert.ToDateTime(startTime);
            var dEndTime = Convert.ToDateTime(endTime);
            //Check available time slots
            var mNSlots = SlotsAvailable(dStartTime, dEndTime, resource.RESOURCE_NAME, out sAccessType, out sAvailabilityMessage);

            //adjust for time_scale availability time mismatches...
            //var availabilityEnds = new DateTime();
            var availabilityBegins = AvailabilityStartTime(resource, dStartTime);
            var availabiltyStarts = availabilityBegins.HasValue ? availabilityBegins.Value : DateTime.MinValue;
            var timeScaleInMinutes = Convert.ToInt32(resource.TIMESCALE);
            var apptLen = resource.Clinic.Duration;

            if (!Keys.IsAddAppointmentOrLeftClickOnScheduler && availabilityBegins.HasValue)
            {
                Keys.IsAddAppointmentOrLeftClickOnScheduler = false;
                if (availabiltyStarts.Minute == 5 || availabiltyStarts.Minute == 15 || availabiltyStarts.Minute == 25 || availabiltyStarts.Minute == 35 || availabiltyStarts.Minute == 45 || availabiltyStarts.Minute == 5)
                {
                    if (apptLen == 10)
                        return "5";
                }
                switch (availabiltyStarts.Minute)
                {
                    case 50:
                    case 10:
                        return "10";
                    case 45:
                    case 15:
                        return "15";
                    case 20:
                    case 40:
                        return "20";
                }
                return timeScaleInMinutes.ToString();
            }
            Keys.IsAddAppointmentOrLeftClickOnScheduler = false;

            if (mNSlots < 1)
            {
                if (sAvailabilityMessage == "IsPreventAccess")
                    sErrorID = "IsPreventAccess";
                else
                    sErrorID = "There are no slots available at the selected time.  Do you want to overbook this appointment?";
            }
            return sErrorID;
        }

        public string CreateNewAppointment(AppointmentData a, PatientInformation p, bool bMakeChartRequest)
        {
            var sErrorID = string.Empty;
            return sErrorID;
        }

        public DataTable CreateAssignedTypeSchedule(string sResourceName, DateTime startTime, DateTime endTime)
        {
            try
            {
                var parms = new List<string> { startTime.ToString("M-d-yyyy"), endTime.ToString("M-d-yyyy"), sResourceName };
                var rs = _adt.callOVIDRPC("SDEC TPBLKOV", parms);

                if (rs.Rows.Count == 0)
                    return rs;

                var rsCopy = new DataTable("rsCopy");
                var dCol = new DataColumn
                {
                    DataType = Type.GetType("System.DateTime"),
                    ColumnName = "StartTime",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                rsCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.DateTime"),
                    ColumnName = "EndTime",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                rsCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.Int32"),
                    ColumnName = "AppointmentTypeID",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                rsCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.Int32"),
                    ColumnName = "AvailabilityID",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                rsCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.String"),
                    ColumnName = "ResourceName",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                rsCopy.Columns.Add(dCol);

                DateTime dLastEnd;
                DateTime dStart;
                DateTime dEnd;

                var rNew = rs.Rows[rs.Rows.Count - 1];
                if (!string.IsNullOrEmpty(rNew["EndTime"].ToString()))
                    dLastEnd = Convert.ToDateTime(rNew["EndTime"].ToString());
                else
                    dLastEnd = DateTime.Now;
                rNew = rs.Rows[0];
                if (!string.IsNullOrEmpty(rNew["StartTime"].ToString()))
                    dStart = Convert.ToDateTime(rNew["StartTime"].ToString());
                else
                    dStart = DateTime.Now;

                const long UNSPECIFIED_TYPE = 10;
                const long UNSPECIFIED_ID = 0;

                //if first block in vgetrows starts later than StartTime,
                // then pad with a new block

                if (dStart > startTime)
                {
                    dEnd = dStart;
                    rNew = rsCopy.NewRow();

                    rNew["StartTime"] = startTime;
                    rNew["EndTime"] = dEnd;
                    rNew["AppointmentTypeID"] = UNSPECIFIED_TYPE;
                    rNew["AvailabilityID"] = UNSPECIFIED_ID;
                    rNew["ResourceName"] = sResourceName;
                    rsCopy.Rows.Add(rNew);
                }

                //if first block start time is < StartTime then trim
                if (dStart < startTime)
                {
                    rNew = rs.Rows[0];
                    rNew["StartTime"] = startTime;
                    dStart = startTime;
                }

                dEnd = dStart;
                foreach (DataRow rEach in rs.Rows)
                {
                    dStart = Convert.ToDateTime(rEach["StartTime"].ToString());
                    if (dStart > dEnd)
                    {
                        rNew = rsCopy.NewRow();

                        rNew["StartTime"] = dEnd;
                        rNew["EndTime"] = dStart;
                        rNew["AppointmentTypeID"] = 0;
                        rNew["AvailabilityID"] = UNSPECIFIED_ID;
                        rNew["ResourceName"] = sResourceName;
                        rsCopy.Rows.Add(rNew);
                    }

                    dEnd = Convert.ToDateTime(rEach["EndTime"].ToString());

                    if (dEnd > endTime)
                        dEnd = endTime;
                    var nAppointmentTypeID = int.Parse(rEach["AppointmentTypeID"].ToString());
                    var nAvailabilityID = int.Parse(rEach["AvailabilityID"].ToString());

                    rNew = rsCopy.NewRow();

                    rNew["StartTime"] = dStart;
                    rNew["EndTime"] = dEnd;
                    rNew["AppointmentTypeID"] = nAppointmentTypeID;
                    rNew["AvailabilityID"] = nAvailabilityID;
                    rNew["ResourceName"] = sResourceName;
                    rsCopy.Rows.Add(rNew);
                }

                //Pad the end if necessary
                if (dLastEnd < endTime)
                {
                    rNew = rsCopy.NewRow();

                    rNew["StartTime"] = dLastEnd;
                    rNew["EndTime"] = endTime;
                    rNew["AppointmentTypeID"] = UNSPECIFIED_TYPE;
                    rNew["AvailabilityID"] = UNSPECIFIED_ID;
                    rNew["ResourceName"] = sResourceName;
                    rsCopy.Rows.Add(rNew);
                }
                return rsCopy;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CreateAssignedTypeSchedule", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
            return null;
        }

        public void RefreshAvailabilitySchedule(string resourcesName, DateTime startDate, DateTime endDate, string src = "0")
        {
            RefreshAvailabilitySchedule(GetSdecResourceByName(resourcesName), startDate, endDate, src);
        }

        public void RefreshAvailabilitySchedule(SchdResource resource, DateTime startDate, DateTime endDate, string src = "0")
        {
            _avArray.Clear();

            var nApptTypeID = 0;

            //Refresh Availability schedules
            var rAvailabilitySchedule = CreateAssignedSlotSchedule(resource, startDate.ToString(), endDate.ToString(), "", src);

            //Refresh Type Schedule
            var rTypeSchedule = CreateAssignedTypeSchedule(resource.RESOURCE_NAME, startDate, endDate);

            foreach (DataRow rTemp in rAvailabilitySchedule.Rows)
            {
                //get StartTime, EndTime and Slots 
                var dStart = Convert.ToDateTime(rTemp["START_TIME"].ToString());
                var dEnd = Convert.ToDateTime(rTemp["END_TIME"].ToString());
                var sSlots = rTemp["SLOTS"].ToString();
                int nSlots = Convert.ToInt16(sSlots);

                var sResourceList = rTemp["RESOURCE"].ToString();
                var sAccessRuleList = rTemp["ACCESS_TYPE"].ToString();

                var sPreventAccessTypeID = "";
                if (sAccessRuleList.Contains("^"))
                {
                    sPreventAccessTypeID = sAccessRuleList.Substring(0, 1);
                }

                var sNote = rTemp["NOTE"].ToString();

                if (nSlots < -1000 || sAccessRuleList == "")
                {
                    nApptTypeID = 0;
                }
                else
                {
                    foreach (DataRow rType in rTypeSchedule.Rows)
                    {

                        var dTypeStart = Convert.ToDateTime(rType["StartTime"].ToString());
                        var dTypeEnd = Convert.ToDateTime(rType["EndTime"].ToString());
                        //if start & end times overlap, then
                        var sTypeResource = rType["ResourceName"].ToString();
                        if (dTypeStart.DayOfYear == dStart.DayOfYear && sResourceList == sTypeResource)
                        {
                            if (Keys.Methods.TimesOverlap(dStart, dEnd, dTypeStart, dTypeEnd))
                            {
                                var sTemp = sPreventAccessTypeID != "" ? sPreventAccessTypeID : rType["AppointmentTypeID"].ToString();
                                nApptTypeID = Convert.ToInt16(sTemp);
                                break;
                            }
                        }
                    }//end foreach datarow rType
                }
                AddAvailability(dStart, dEnd, nApptTypeID, nSlots, false, sResourceList, sAccessRuleList, sNote);
            }//end foreach datarow rTemp
        }

        public int AddAvailability(DateTime startTime, DateTime endTime, int nType, int nSlots, bool updateView, string sResourceList, string sAccessRuleList, string sNote)
        {
            //adds it to the object array
            //Returns the index in the array

            var pNewAv = new SchdAvailability();
            pNewAv.Create(startTime.ToString(), endTime.ToString(), nType.ToString(), nSlots, sResourceList, sAccessRuleList);
            pNewAv.Note = sNote;

            //Look up the color and type name using the AppointmentTypes datatable
            var dtType = _mDsGlobal.Tables["AccessTypes"];
            var dRow = dtType.Rows.Find(nType.ToString());
            if (dRow != null)
            {
                var sColor = dRow["DISPLAY_COLOR"].ToString();
                pNewAv.DisplayColor = sColor;
                var sTemp = dRow["RED"].ToString();
                sTemp = sTemp == "" ? "0" : sTemp;
                int nRed = Convert.ToInt16(sTemp);
                pNewAv.Red = nRed;
                sTemp = dRow["GREEN"].ToString();
                sTemp = sTemp == "" ? "0" : sTemp;
                int nGreen = Convert.ToInt16(sTemp);
                pNewAv.Green = nGreen;
                sTemp = dRow["BLUE"].ToString();
                sTemp = sTemp == "" ? "0" : sTemp;
                int nBlue = Convert.ToInt16(sTemp);
                pNewAv.Blue = nBlue;

                var sName = dRow["NAME"].ToString();
                pNewAv.ACCESSTYPENAME = sName;

                var bIsPreventAccess = Keys.Methods.StrToBool(dRow["PREVENT_ACCESS"].ToString());
                pNewAv.IsPreventAccess = bIsPreventAccess;
            }

            var nIndex = _avArray.Add(pNewAv);
            if (updateView)
            {
                //this.UpdateAllViews ();
            }
            return nIndex;
        }

        private readonly ArrayList _avArray = new ArrayList();

        public ArrayList AvailabilityArray
        {
            get { return _avArray; }
        }

        public int SlotsAvailable(DateTime dSelStart, DateTime dSelEnd, string sResource, out string sAccessType, out string sAvailabilityMessage)
        {
            sAccessType = "";
            sAvailabilityMessage = "";
            var nAvailableSlots = 999;
            var i = 0;
            //loop thru m_pAvArray
            //Compare the start time and end time of eachblock
            while (i < _avArray.Count)
            {
                var pAv = (SchdAvailability)_avArray[i];
                var dStart = DateTime.Parse(pAv.StartTime);
                var dEnd = DateTime.Parse(pAv.EndTime);
                if (sResource == pAv.RESOURCENAME && (dSelStart.Date == dStart.Date || dSelStart.Date == dEnd.Date))
                {
                    if (Keys.Methods.TimesOverlap(dStart, dEnd, dSelStart, dSelEnd))
                    {
                        if (pAv.IsPreventAccess)
                        {
                            nAvailableSlots = -1;
                            i = _avArray.Count;
                            sAvailabilityMessage = "IsPreventAccess";
                        }
                        else
                        {
                            var nSlots = pAv.SLOTS;
                            if (nSlots < 1)
                            {
                                nAvailableSlots = 0;
                                break;
                            }
                            if (nSlots < nAvailableSlots)
                            {
                                nAvailableSlots = nSlots;
                                sAccessType = pAv.ACCESSTYPENAME;
                                sAvailabilityMessage = pAv.Note;
                            }
                        }
                    }
                }
                i++;
            }
            if (nAvailableSlots == 999)
            {
                nAvailableSlots = 0;
            }
            return nAvailableSlots;
        }

        public string EditAppointment(AppointmentData a)
        {
            var parms = new List<string> { a.AppointmentID, a.Notes, a.Duration };
            var table = _adt.callOVIDRPC("SDEC EDITAPPT", parms);

            var sErrorID = string.Empty;
            if (table.Rows.Count == 1)
            {
                var r = table.Rows[0];
                if (r["ERRORID"].ToString() != "-1")
                    sErrorID = r["ERRORID"].ToString();
            }

            // Clear the Future Appointments Cache
            ClearPatientFutureAppointmentsCache(a.PatientId);
            return sErrorID;
        }

        public IList<SchdAppointment> GetAppointments(SchdResource resource, string startDate, string endDate)
        {
            var parms = new List<string>
            {
                resource.RESOURCEID,
                startDate,
                endDate
            };
            var table = _adt.callOVIDRPC("SDEC CRSCHED", parms);

            IList<SchdAppointment> appointmentList = new List<SchdAppointment>();
            foreach (DataRow row in table.Rows)
            {
                var appointment = _container.Resolve<SchdAppointment>();

                appointment.ACCESSTYPEID = row["ACCESSTYPEID"].ToString();
                appointment.APPOINTMENTID = row["APPOINTMENTID"].ToString();
                appointment.AUXTIME = row["AUXTIME"].ToString();
                appointment.CANCELLED = row["CANCELLED"].ToString();
                appointment.CHECKIN = row["CHECKIN"].ToString();
                appointment.CHECKOUT = row["CHECKOUT"].ToString();
                appointment.END_TIME = row["END_TIME"].ToString();
                //newAppointment.END_TIME    = Convert.ToDateTime(newAppointment.END_TIME).AddHours(-3).ToString();
                appointment.HRN = row["HRN"].ToString();
                appointment.NOSHOW = row["NOSHOW"].ToString();
                appointment.NOTE = row["NOTE"].ToString();
                appointment.PATIENTID = row["PATIENTID"].ToString();
                appointment.Patient = new Patient(this)
                {
                    IEN = appointment.PATIENTID
                };
                appointment.Patient.LoadPatientInfoByIEN();
                appointment.PATIENTNAME = row["PATIENTNAME"].ToString();
                appointment.RESOURCENAME = row["RESOURCENAME"].ToString();
                appointment.START_TIME = row["START_TIME"].ToString();
                //newAppointment.START_TIME         = Convert.ToDateTime(newAppointment.START_TIME).AddHours(-3).ToString();
                appointment.VPROVIDER = row["VPROVIDER"].ToString();
                appointment.WALKIN = row["WALKIN"].ToString();
                appointment.IsDead = row["DIEDON"].ToString() != "0";
                appointment.MRTCParentRequestIEN = row["SDPARENT"].ToString();

                var appReqType = row["APPTREQTYPE"].ToString();

                if (!string.IsNullOrEmpty(appReqType))
                {
                    var apptReqTypeParts = appReqType.Split('|');
                    appointment.APPTREQTYPE_REQTYPE = apptReqTypeParts.Any() ? apptReqTypeParts[0] : string.Empty;
                    appointment.APPTREQTYPE_REQIEN = apptReqTypeParts.Length > 1 ? apptReqTypeParts[1] : string.Empty;
                }

                // Insure End Time != Start Time
                var startTime = Convert.ToDateTime(appointment.START_TIME);
                DateTime endTime;

                if (!DateTime.TryParse(appointment.END_TIME, out endTime))
                {
                    endTime = startTime.AddMinutes(resource.Clinic.Duration);
                    appointment.END_TIME = endTime.ToString("MMM dd, yyyy hh:mm");
                }

                if (startTime.CompareTo(endTime) == 0)
                    appointment.END_TIME = startTime.AddMinutes(resource.Clinic.Duration).ToString("MMM dd, yyyy hh:mm");

                // Do not display cancelled Appointments by Ron H per Fay 10/02/2015
                if (appointment.CANCELLED == "0")
                    appointmentList.Add(appointment);
            }
            return appointmentList;
        }

        public SchdAppointment GetAppointmentByAppointmentID(string appointmentID)
        {
            SchdAppointment newAppointment = null;

            var parms = new List<string>
            {
                // MAXREC = (optional) Max records returned default to all
                string.Empty,
                // LASTSUB = (optional) last subscripts from previous call
                string.Empty,
                // SDBEG = (optional) Begin Date range in external format
                string.Empty,
                // SDEND = (optional) End Date range in external format
                string.Empty,
                // NOTEFLG = (optional) 1=do NOT return NOTE text, 0=return NOTE text
                "0",
                // SDRES = (optional) Resource ID pointer to SDEC RESOURCE file
                string.Empty,
                // DFN = (optional) pointer to PATIENT file 2
                string.Empty,
                // SDID = (optional) external ID (free-text)
                string.Empty,
                // APPOINTMENTID
                appointmentID
            };
            var table = _adt.callOVIDRPC(Keys.RpcCalls.AppSDGet, parms);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                newAppointment = _container.Resolve<SchdAppointment>();
                newAppointment.APPOINTMENTID = row["IEN"].ToString();

                if (newAppointment.APPOINTMENTID != appointmentID)
                    return null;

                newAppointment.ACCESSTYPEID = row["SDEC_ACCESS_TYPE_IEN"].ToString();
                // Not present in SDEC APPSDGET return
                // newAppointment.AUXTIME = row["AUXTIME"].ToString();
                newAppointment.CANCELLED = string.IsNullOrEmpty(row["DATE6"].ToString()) ? "0" : "1";
                newAppointment.CHECKIN = row["CHECKIN_TIME"].ToString().Replace("@", " ");
                newAppointment.CHECKOUT = row["CHECKOUT"].ToString().Replace("@", " ");
                DateTime tempDate;

                if (Keys.Methods.TryParseDateTime(row["DATE2"].ToString(), out tempDate))
                    newAppointment.END_TIME = tempDate.ToString(); //"MM/dd/yyyy HH:mm");
                // Not present in SDEC APPSDGET return
                // newAppointment.HRN       = row["HRN"].ToString();
                newAppointment.NOSHOW = Keys.Methods.StrToBool(row["NOSHOW"].ToString()) ? "1" : "0";
                newAppointment.NOTE = row["NOTE"].ToString();
                newAppointment.PATIENTID = row["DFN"].ToString();
                newAppointment.PATIENTNAME = row["NAME"].ToString();
                newAppointment.RESOURCENAME = row["RESOURCE_NAME"].ToString();

                if (Keys.Methods.TryParseDateTime(row["DATE1"].ToString(), out tempDate))
                    newAppointment.START_TIME = tempDate.ToString(); //"MM/dd/yyyy HH:mm");

                newAppointment.VPROVIDER = row["V_PROVIDER_NAME"].ToString();
                newAppointment.WALKIN = Keys.Methods.StrToBool(row["WALKIN"].ToString()) ? "1" : "0";
                // Not present in SDEC APPSDGET return
                // newAppointment.IsDead = row["DIEDON"].ToString() != "0";
                // newAppointment.MRTCParentRequestIEN = row["SDPARENT"].ToString();

                if (!string.IsNullOrEmpty(row["DAPTDT"].ToString()))
                    newAppointment.DesiredAppointmentDate = Convert.ToDateTime(row["DAPTDT"].ToString());

                var appReqType = row["SDAPTYP"].ToString();

                if (!string.IsNullOrEmpty(appReqType))
                {
                    var apptReqTypeParts = appReqType.Split('|');
                    newAppointment.APPTREQTYPE_REQTYPE = apptReqTypeParts.Any() ? apptReqTypeParts[0] : string.Empty;
                    newAppointment.APPTREQTYPE_REQIEN = apptReqTypeParts.Length > 1 ? apptReqTypeParts[1] : string.Empty;
                }
                if (table.Columns.Contains("APPTTYPE_IEN"))
                    newAppointment.APPTTYPE_IEN = row["APPTTYPE_IEN"].ToString();

                if (table.Columns.Contains("APPTTYPE_NAME"))
                    newAppointment.APPTTYPE_NAME = row["APPTTYPE_NAME"].ToString();


            }
            return newAppointment;
        }

        public SchdAppointment GetAppointment(SchdResource resource, string startDate, string endDate, string appointmentId)
        {
            var parms = new List<string> { resource.RESOURCEID, startDate, endDate };
            var table = _adt.callOVIDRPC("SDEC CRSCHED", parms);

            var appointment = new SchdAppointment();

            foreach (DataRow row in table.Rows)
            {
                var newAppointment = _container.Resolve<SchdAppointment>();

                newAppointment.APPOINTMENTID = row["APPOINTMENTID"].ToString();
                if (newAppointment.APPOINTMENTID != appointmentId)
                    continue;

                newAppointment.ACCESSTYPEID = row["ACCESSTYPEID"].ToString();
                newAppointment.AUXTIME = row["AUXTIME"].ToString();
                newAppointment.CANCELLED = row["CANCELLED"].ToString();
                newAppointment.CHECKIN = row["CHECKIN"].ToString();
                newAppointment.CHECKOUT = row["CHECKOUT"].ToString();
                newAppointment.END_TIME = row["END_TIME"].ToString();
                //newAppointment.END_TIME           = Convert.ToDateTime(newAppointment.END_TIME).AddHours(-3).ToString();
                newAppointment.HRN = row["HRN"].ToString();
                newAppointment.NOSHOW = row["NOSHOW"].ToString();
                newAppointment.NOTE = row["NOTE"].ToString();
                newAppointment.PATIENTID = row["PATIENTID"].ToString();
                newAppointment.PATIENTNAME = row["PATIENTNAME"].ToString();
                newAppointment.RESOURCENAME = row["RESOURCENAME"].ToString();
                newAppointment.START_TIME = row["START_TIME"].ToString();
                //newAppointment.START_TIME         = Convert.ToDateTime(newAppointment.START_TIME).AddHours(-3).ToString();
                newAppointment.VPROVIDER = row["VPROVIDER"].ToString();
                newAppointment.WALKIN = row["WALKIN"].ToString();
                newAppointment.IsDead = row["DIEDON"].ToString() != "0";
                newAppointment.MRTCParentRequestIEN = row["SDPARENT"].ToString();

                var appReqType = row["APPTREQTYPE"].ToString();

                if (!string.IsNullOrEmpty(appReqType))
                {
                    var apptReqTypeParts = appReqType.Split('|');
                    newAppointment.APPTREQTYPE_REQTYPE = apptReqTypeParts.Any() ? apptReqTypeParts[0] : string.Empty;
                    newAppointment.APPTREQTYPE_REQIEN = apptReqTypeParts.Length > 1 ? apptReqTypeParts[1] : string.Empty;
                }

                // Insure End Time != Start Time
                var startTime = Convert.ToDateTime(newAppointment.START_TIME);
                var endTime = Convert.ToDateTime(newAppointment.END_TIME);

                if (startTime.CompareTo(endTime) == 0)
                    newAppointment.END_TIME = startTime.AddMinutes(resource.Clinic.Duration).ToString("MMM dd, yyyy hh:mm");

                appointment = newAppointment;
                break;
                //AppointmentList.Add(newAppointment);
            }
            return appointment;
        }

        private IList<SchdAvailability> GetAvailabilities(string resourceName, string startDate, string endDate)
        {
            var resource = GetSdecResource(resourceName);
            return GetAvailabilities(resource, startDate, endDate);
        }

        public IList<SchdAvailability> GetAvailabilities(SchdResource resource, string startDate, string endDate)
        {
            IList<SchdAvailability> availabilityList = new List<SchdAvailability>();

            if (resource == null)
                return availabilityList;

            var src = "0";
            if (Keys.IsAvailabilitySearch)
            {
                src = Keys.Availability;
                Keys.IsAvailabilitySearch = false;
            }

            var rAppointmentSchedule = CreateAssignedSlotSchedule(resource, startDate, endDate, "", src);

            if (rAppointmentSchedule == null)
                return null;

            foreach (DataRow r in rAppointmentSchedule.Rows)
            {
                var pAppointment = new SchdAvailability();
                pAppointment.Create((DateTime)r["START_TIME"], (DateTime)r["END_TIME"], r["ACCESS_TYPE"].ToString(), (int)r["SLOTS"], r["SLOT_COUNT"].ToString(), resource);
                pAppointment.OverbookedCount = (int)r["OVERBOOKED_COUNT"];
                var tAccess = GetAccessTypeByID(r["ACCESS_TYPE"].ToString());
                pAppointment.ACCESSTYPENAME = tAccess.NAME;
                pAppointment.Red = int.Parse(tAccess.RED ?? "0");
                pAppointment.Green = int.Parse(tAccess.GREEN ?? "0");
                pAppointment.Blue = int.Parse(tAccess.BLUE ?? "0");
                pAppointment.IsPreventAccess = tAccess.PREVENT_ACCESS == "YES";
                pAppointment.APPOINTMENTID = r["AVAILABILITYID"].ToString();
                availabilityList.Add(pAppointment);
            }

            if (!_mGlobalAvailabilities.ContainsKey(resource.RESOURCE_NAME))
                _mGlobalAvailabilities.Add(resource.RESOURCE_NAME, availabilityList);
            else
                _mGlobalAvailabilities[resource.RESOURCE_NAME] = availabilityList;

            return availabilityList;
        }

        private string AdjustAvailabilityStartTimeBasedOnClinicsTimeScale(string startTime, int timeScale, string sResourceName, bool useRounding = false)
        {
            //get timescale..
            if (!Keys.Methods.IsDate(startTime))
            {
                MessageBox.Show("Invalid start time for Availability: " + startTime + " Clinic: " + sResourceName);
                return string.Empty;
            }
            var dStartTime = Convert.ToDateTime(startTime);
            var hour = dStartTime.Hour;
            var minute = dStartTime.Minute;

            if (timeScale != minute && minute != 0)
            {
                if (timeScale > minute && minute == 10)
                    timeScale = 0;


                if (useRounding)
                    dStartTime = Keys.Methods.DateTime_Round(dStartTime, TimeSpan.FromMinutes(timeScale));
                else
                    dStartTime = dStartTime.AddMinutes(-timeScale);

                return dStartTime.ToString(CultureInfo.InvariantCulture);
            }
            return startTime;
        }

        private string AdjustAvailabilityEndTimeBasedOnClinicsTimeScale(string endTime, int timeScale, string sResourceName, bool useRounding = false)
        {
            //get timescale..
            if (!Keys.Methods.IsDate(endTime))
            {
                MessageBox.Show("Invalid end time for Availability: " + endTime + " Clinic: " + sResourceName);
                return string.Empty;
            }
            var dEndTime = Convert.ToDateTime(endTime);
            var hour = dEndTime.Hour;
            var minute = dEndTime.Minute;
            if (timeScale == minute || minute == 0)
                return endTime;

            if (useRounding)
                dEndTime = Keys.Methods.DateTime_Round(dEndTime, TimeSpan.FromMinutes(timeScale), false);
            else
                dEndTime = dEndTime.AddMinutes(-timeScale);

            return dEndTime.ToString(CultureInfo.InvariantCulture);
        }

        private int GetTotalMinutes(DateTime dDate)
        {
            return dDate.Hour * 60 + dDate.Minute;
        }

        public PatientInformation GetPatientInfo(string patientIEN)
        {
            var newPatient = new PatientInformation();
            var table = GetPatientRecord(patientIEN);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                newPatient.IEN = row["IEN"].ToString();
                var sensitiveInfo = row["SENSITIVE"].ToString();
                if (string.IsNullOrEmpty(sensitiveInfo) || sensitiveInfo == "0")
                    newPatient.IsSensitiveRecord = false;
                else
                    newPatient.IsSensitiveRecord = true;

                newPatient.Name = row["Name"].ToString();
                newPatient.HRN = row["HRN"].ToString();
                newPatient.SSN = row["SSN"].ToString();
                newPatient.DOB = row["DOB"].ToString();
                newPatient.Street = row["STREET"].ToString();
                newPatient.City = row["CITY"].ToString();
                newPatient.OfficePhone = row["OFCPHONE"].ToString();
                newPatient.State = row["STATE"].ToString();
                newPatient.Zip = row["ZIP"].ToString();
                newPatient.HomePhone = row["HOMEPHONE"].ToString();
                newPatient.ServiceConnection = row["SVCCONN"].ToString();
                newPatient.ServiceConnectionPercentage = row["SVCCONNP"].ToString();

                newPatient.GAF.Text = row["GAF"].ToString();
            }

            var patSvcInfo = GetPatientServiceInfo(newPatient.IEN);

            if (patSvcInfo == null)
                return newPatient;

            newPatient.ServiceConnection = patSvcInfo.ServiceConnection;
            newPatient.ServiceConnectionPercentage = patSvcInfo.ServiceConnectionPercentage;
            return newPatient;
        }

        public void GetPatientPhoneNumbers(ref Patient patient)
        {
            var table = GetPatientRecord(patient.IEN);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                patient.PrimaryContactInfo.City = row["CITY"].ToString();
                patient.PrimaryContactInfo.State = GetState(row["STATE"].ToString());
                patient.PrimaryContactInfo.PhoneWork = row["OFCPHONE"].ToString();
                patient.PrimaryContactInfo.PhoneResidence = row["HOMEPHONE"].ToString();
            }
        }


        public void GetPatientInfoForPrompt(string patientIen, ref string dob, ref string ssn)
        {
            var table = GetPatientRecord(patientIen);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                ssn = row["SSN"].ToString();
                dob = row["DOB"].ToString();
            }
        }

        private RPMSClinic GetRPMSClinicFromDataRow(DataRow row, RPMSClinic oldClinic = null)
        {
            RPMSClinic clinic;

            if (oldClinic != null)
                clinic = oldClinic;
            else
                clinic = new RPMSClinic(this);

            //var clinic = new RPMSClinic( this )
            //{
            clinic.ClinicIen = row["HOSPITAL_LOCATION_ID"].ToString();
            clinic.ClinicName = row["HOSPITAL_LOCATION"].ToString();
            clinic.CreateVisit = row["CREATE_VISIT"].ToString();
            clinic.VisitServiceCategory = row["VISIT_SERVICE_CATEGORY"].ToString();
            clinic.SpecialInstructions = row["SPECIAL"].ToString().Trim();
            clinic.Duration = !string.IsNullOrEmpty(row["APPTLEN"].ToString()) ? Convert.ToInt32(row["APPTLEN"].ToString()) : 30;
            clinic.MaxOverBooksRaw = row["MAX_OVERBOOKS"].ToString();
            clinic.DefaultAppointmentTypeId = row["SDECDAT"].ToString();
            clinic.DefaultAppointmentTypeName = row["SDECDATN"].ToString();
            clinic.VariableAppointmentLength = row["VAPPTLEN"].ToString() == "V";
            clinic.SLOTS = row["SLOTS"].ToString();
            clinic.PROVUSERPRESENT_BOOL = row["PRIVUSERPRESENT_BOOL"].ToString();
            clinic.PROTECTED = row["PROTECTED"].ToString();
            clinic.HOUR_DISPLAY_BEGIN = row["HOUR_DISPLAY_BEGIN"].ToString();
            clinic.MultipleClinicCodes = Keys.Methods.StrToBool(row["MULTIPLE_CLINIC_CODES_USED?"].ToString());
            clinic.VisitProviderRequired = Keys.Methods.StrToBool(row["VISIT_PROVIDER_REQUIRED"].ToString());
            clinic.GeneratePccPlusForms = Keys.Methods.StrToBool(row["GENERATE_PCCPLUS_FORMS?"].ToString());
            //};
            // Below removed after clinic.AppointmentLength has been tied to clinic.Duration
            //clinic.AppointmentLength = !string.IsNullOrEmpty(row["APPTLEN"].ToString()) ? Convert.ToInt32(row["APPTLEN"].ToString()) : 30;

            if (Keys.Methods.IsNumeric(row["DISPLAY_INCREMENTS"].ToString()))
                clinic.DISPLAY_INCREMENTS = Convert.ToInt32(row["DISPLAY_INCREMENTS"].ToString());
            else
                clinic.DISPLAY_INCREMENTS = 2;

            clinic.HOLIDAYS = row["HOLIDAYS"].ToString();
            clinic.CLINIC_STOP = row["CLINIC_STOP"].ToString();
            clinic.ABBR = row["ABBR"].ToString();
            clinic.MoreRecords = (row["MORE"].ToString() == "1");
            clinic.IsCompleteRecord = true;

            return clinic;
        }

        private Clinic GetClinicFromDataRow(DataRow row)
        {
            return GetRPMSClinicFromDataRow(row);
        }

        public IList<Clinic> GetPtCSchClinics(string clinicID, string clinicName)
        {
            var table = GetClinicSetupParameters(clinicID, clinicName);
            var clinics = new List<Clinic>();
            if (table.Columns.Count > 1)
            {
                foreach (DataRow row in table.Rows)
                {
                    var clinic = GetClinicFromDataRow(row);
                    clinics.Add(clinic);
                }
            }
            return clinics;
        }

        public bool ClinicExists(string sClinicName)
        {
            var table = GetClinicSetupParameters("", sClinicName);
            return table != null && table.Rows.Count > 0;
        }

        public bool ServiceExists(string serviceName)
        {
            var dt = GetClinicStop(serviceName);
            if (dt != null)
                foreach (DataRow dr in dt.Rows)
                    if (dr["Name"].ToString().ToUpper() == serviceName.ToUpper())
                        return true;
            return false;
        }

        public IList<Clinic> SearchPtCSchClinics(string searchString, string sdNoLet = "")
        {
            var parms = new List<string> { "1", "", searchString };
            if (!string.IsNullOrEmpty(sdNoLet))
                parms.Add(sdNoLet);
            else
                parms.Add("0");
            parms.Add(Keys.ClinicResourceSearchMaxRecords.ToString());
            var table = _adt.callOVIDRPC(Keys.RpcCalls.ClinSet, parms);
            var clinics = new List<Clinic>();
            if (table.Columns.Count > 1)
            {
                foreach (DataRow row in table.Rows)
                {
                    var clinic = GetClinicFromDataRow(row);
                    clinics.Add(clinic);
                }
            }
            return clinics;
        }

        private DataTable GetClinAll(string searchString)
        {
            var parms = new List<string>()
            {
                Keys.ClinicResourceSearchMaxRecords.ToString(),
                searchString
            };
            Keys.Tables.ClinAllData = _adt.callOVIDRPC("SDEC CLINALL", parms);
            return Keys.Tables.ClinAllData;
        }

        public IList<NameValue> GetEwlClinics(string searchString)
        {
            var table = GetClinAll(searchString);
            var clinics = new List<NameValue>();

            if (table.Columns.Count > 1)
            {
                foreach (DataRow row in table.Rows)
                {
                    string altName = row["CLINIC_NAME"].ToString();
                    string abbr = row["ABBR"].ToString();

                    if (!string.IsNullOrEmpty(abbr))
                    {
                        int index = altName.IndexOf(abbr, StringComparison.Ordinal);
                        altName = ((index < 0) ? altName : altName.Remove(index, abbr.Length).Trim());
                    }

                    var clinic = new NameValue()
                    {
                        AlternateValue = row["CLINIC_IEN"].ToString(),
                        AlternateName = altName,
                        Name = row["CLINIC_NAME"].ToString(),
                        Value = row["HOSPITAL_LOCATION_ID"].ToString(),
                        MoreRecords = (row["MORE"].ToString() == "1")
                    };
                    clinics.Add(clinic);
                }
            }
            return clinics;
        }

        public string GetEwlClinicIen(string clinicName)
        {
            GetClinAll(clinicName);

            foreach (DataRow clinic in Keys.Tables.ClinAllData.Rows)
                if (clinic["CLINIC_NAME"].ToString().ToUpper() == clinicName.ToUpper())
                    return clinic["CLINIC_IEN"].ToString();

            return string.Empty;
        }

        public string GetHospitalLocationIdByResourceIen(string resourceIen)
        {
            var schdRes = GetSdecResource(resourceIen);

            return schdRes == null ? string.Empty : schdRes.HOSPITAL_LOCATION_ID;
        }

        public List<AppointmentType> GetAppointmentTypes(string patientIen)
        {
            if (string.IsNullOrEmpty(patientIen))
                return new List<AppointmentType>();

            // Remove any stale records from the Keys.Tables.FutureApptsTableCache
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.ApptTypesCache);
            DataTable table;

            if (Keys.Tables.ApptTypesCache.ContainsKey(patientIen))
                table = Keys.Tables.ApptTypesCache[patientIen].Table;
            else
            {
                var parms = new List<string> { patientIen };
                table = _adt.callOVIDRPC(Keys.RpcCalls.AppointmentTypes, parms);

                if (!Keys.Tables.ApptTypesCache.ContainsKey(patientIen))
                    Keys.Tables.ApptTypesCache.Add(patientIen, new DataTableCache(table, 60));
            }

            List<AppointmentType> appointmentTypes = new List<AppointmentType>();

            if (table.Columns.Count > 1)
            {
                foreach (DataRow row in table.Rows)
                {
                    var apptType = new AppointmentType
                    {
                        Ien = row["APPTTYPE_IEN"].ToString(),
                        Name = row["APPTTYPE_NAME"].ToString()
                    };

                    appointmentTypes.Add(apptType);
                }
            }

            return appointmentTypes;
        }

        public IList<ServiceSpecialty> GetServiceSpecialties()
        {
            if (Keys.Tables.ServiceSpecialtypData == null || Keys.Tables.ServiceSpecialtyDataLoaded.AddHours(1) < DateTime.Now)
            {
                var parms = new List<string>();
                Keys.Tables.ServiceSpecialtypData = _adt.callOVIDRPC("SDEC SVSPALL", parms);
            }

            var svcSpecialties = new List<ServiceSpecialty>();
            if (Keys.Tables.ServiceSpecialtypData.Columns.Count > 1)
            {
                foreach (DataRow row in Keys.Tables.ServiceSpecialtypData.Rows)
                {
                    var svcSpecialty = new ServiceSpecialty
                    {
                        Ien = row["SERVICESPECIALTY_IEN"].ToString(),
                        Name = row["SERVICESPECIALTY_NAME"].ToString()
                    };

                    if (!svcSpecialty.Name.Contains("ALCOHOL AND SUBSTANCE") && !svcSpecialty.Name.Contains("AUDIOLOGY"))
                        svcSpecialties.Add(svcSpecialty);
                }
            }

            return svcSpecialties;
        }

        public IList<Patient> GetPatients(string searchString, int maxRows)
        {
            var allRecordsLoaded = false;
            return GetPatients(searchString, maxRows, ref allRecordsLoaded, string.Empty);
        }

        public IList<Patient> GetPatients(string searchString, int maxRows, ref bool allRecordsDisplayed, string lastSub = "")
        {
            if (string.IsNullOrEmpty(searchString) || !Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
            {
                if (!Keys.AppointmentRequestWaitListTables.AllAppointmentsLoaded)
                    MessageBox.Show("Patient Data still loading... please try again.");
                return _emptyList;
            }

            string tempSsearchString = searchString.Replace("-", "");

            var patientList = new List<Patient>();
            try
            {
                if (searchString.Length == 10)
                    if (searchString.Substring(9, 1) == "P")
                        searchString = searchString.Substring(0, 9);
                var isNumeric = Keys.Methods.IsNumeric(tempSsearchString);
                var firstCharLastNameLast4Ssn = false;
                var last4SearchString = "";
                var firstCharSearchString = "";
                if (searchString.Length == 5)
                {
                    last4SearchString = searchString.Substring(1, 4);
                    firstCharSearchString = searchString.Substring(0, 1);
                    if (Keys.Methods.IsNumeric(last4SearchString) && !Keys.Methods.IsNumeric(firstCharSearchString))
                        firstCharLastNameLast4Ssn = true;
                }

                var parms = new List<string> { searchString, maxRows.ToString(CultureInfo.InvariantCulture), lastSub };
                var table = _adt.callOVIDRPC(Keys.RpcCalls.PatientLookUpRs, parms);
                if (table.Rows.Count == 0)
                {
                    allRecordsDisplayed = true;
                    return null;
                }

                var hasErrors = false;

                foreach (DataRow row in table.Rows)
                {
                    bool hasError;
                    var newPatient = PTLOOKRS_ToPatientRecord(row, out hasError);
                    hasErrors = hasErrors || hasError;

                    foreach (var item in Keys.Tables.FutureApptsTableCache.Where(c => (c.Key.Contains(newPatient.IEN + "^"))).ToList())
                        Keys.Tables.FutureApptsTableCache.Remove(item.Key);

                    if (firstCharLastNameLast4Ssn)
                    {
                        var firstCharName = newPatient.Name.Substring(0, 1);
                        var last4Ssn = newPatient.SSN4;
                        if (last4SearchString == last4Ssn && firstCharName.ToUpper() == firstCharSearchString.ToUpper())
                        {
                            patientList.Add(newPatient);
                            continue;
                        }
                    }

                    if (isNumeric)  //AKA SSN Search
                    {
                        if ((newPatient.SSNRaw.Replace("-", "") == tempSsearchString) || newPatient.SSNRaw.StartsWith(tempSsearchString) || newPatient.SSNRaw.EndsWith(tempSsearchString)) // using Raw SSN, should never contain SENSITIVE || newPatient.SSN.ToUpper().Contains("SENSITIVE"))
                            patientList.Add(newPatient);
                    }
                    else
                        patientList.Add(newPatient);
                }

                if (hasErrors)
                    throw new Exception("There were problems processing some or all Patient Lookup Records.\nSee the Trace Log for details.");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Get Patient(s) Error");
            }
            return patientList;
        }

        private Patient PTLOOKRS_ToPatientRecord(DataRow row, out bool hasError)
        {
            hasError = false;
            var newPatient = _container != null ? _container.Resolve<Factory<Patient>>().Create() : new Patient(this);
            var pName = newPatient.Name = row["NAME"].ToString();

            try
            {
                //  1
                newPatient.Name = row["NAME"].ToString();
                //  2
                newPatient.HRN = row["HRN"].ToString();
                //  3
                newPatient.SSN = row["SSN"].ToString();
                //  4
                newPatient.DOB = row["DOB"].ToString();
                //  5
                newPatient.IEN = row["IEN"].ToString();
                //  6
                newPatient.Gender = row["GENDER"].ToString();
                //  7
                newPatient.InstitutionIen = row["INSTIEN"].ToString();
                //  8
                newPatient.Institution = row["INSTNAME"].ToString();
                //  9
                newPatient.OriginatingUser = row["USERIEN"].ToString(); //"User, Demo"; // row["USER"].ToString();
                // 10
                newPatient.OriginatingUserName = row["USERNAME"].ToString();
                // 11
                newPatient.PriorityGroup = row["PRIGRP"].ToString();
                // 12
                newPatient.EligibilityIen = row["ELIGIEN"].ToString();
                // 13
                newPatient.EligibilityName = row["ELIGNAME"].ToString();
                // 14
                newPatient.ServiceConnection = row["SVCCONN"].ToString();
                // 15
                newPatient.ServiceConnectionPercentage = row["SVCCONNP"].ToString();
                // 16
                newPatient.TypeIen = row["TYPEIEN"].ToString();
                // 17
                newPatient.TypeName = row["TYPENAME"].ToString();
                // 18
                newPatient.PrimaryContactInfo.Street1 = row["PADDRES1"].ToString();
                // 19
                newPatient.PrimaryContactInfo.Street2 = row["PADDRES2"].ToString();
                // 20
                newPatient.PrimaryContactInfo.Street3 = row["PADDRES3"].ToString();
                // 21
                newPatient.PrimaryContactInfo.City = row["PCITY"].ToString();
                // 22
                newPatient.PrimaryContactInfo.State = GetState(row["PSTATE"].ToString());
                // 23
                newPatient.PrimaryContactInfo.Country = newPatient.Countries.FirstOrDefault(c => c.Ien == row["PCOUNTRY"].ToString() || c.Abbreviation == row["PCOUNTRY"].ToString());
                // 24
                newPatient.PrimaryContactInfo.ZipPlus4 = row["PZIP4"].ToString();
                // 25
                newPatient.GAF.Text = row["GAF"].ToString();
                // 26
                newPatient.SensitiveRecord = new PatientSensitiveRecord(row["SENSITIVE"].ToString());
                newPatient.IsSensitiveRecord = newPatient.SensitiveRecord.IsSensitiveRecord;
                // 27 & 28
                newPatient.SetRaces(row["PRACE"].ToString(), row["PRACEN"].ToString());
                // 29 & 29
                newPatient.SetEthnicity(row["PETH"].ToString(), row["PETHN"].ToString());
                // 30
                newPatient.Ethnicity.Name = row["PETHN"].ToString();
                // 31
                newPatient.LastSub = row["LASTSUB"].ToString();
                // 32 - Setting BadAddressType will also set IsBadAddress
                newPatient.BadAddressType = row["BADADD"].ToString();

                //if (!string.IsNullOrEmpty(newPatient.BadAddressType))
                //    newPatient.IsBadAddress = true;
                // 33
                newPatient.PrimaryContactInfo.PhoneResidence = row["HPHONE"].ToString();
                // 34
                newPatient.PrimaryContactInfo.PhoneWork = row["OPHONE"].ToString();
                // 35
                // NOK & KNAME return the same information
                // 36
                newPatient.PrimaryNextOfKin.Name = row["KNAME"].ToString();
                // 37
                newPatient.PrimaryNextOfKin.Relationship = row["KREL"].ToString();
                // 38
                newPatient.PrimaryNextOfKin.ContactInfo.Phone = row["KPHONE"].ToString();
                // 39
                newPatient.PrimaryNextOfKin.ContactInfo.Street1 = row["KSTREET"].ToString();
                // 40
                newPatient.PrimaryNextOfKin.ContactInfo.Street2 = row["KSTREET2"].ToString();
                // 41
                newPatient.PrimaryNextOfKin.ContactInfo.Street3 = row["KSTREET3"].ToString();
                // 42
                newPatient.PrimaryNextOfKin.ContactInfo.City = row["KCITY"].ToString();
                // 43
                newPatient.PrimaryNextOfKin.ContactInfo.State = GetState(row["KSTATE"].ToString());
                // 44
                newPatient.PrimaryNextOfKin.ContactInfo.ZipPlus4 = row["KZIP"].ToString();
                // 45
                // NOK2 & K2NAME return the same information
                // 46
                newPatient.SecondaryNextOfKin.Name = row["K2NAME"].ToString();
                // 47
                newPatient.SecondaryNextOfKin.Relationship = row["K2REL"].ToString();
                // 48
                newPatient.SecondaryNextOfKin.ContactInfo.Phone = row["K2PHONE"].ToString();
                // 49
                newPatient.SecondaryNextOfKin.ContactInfo.Street1 = row["K2STREET"].ToString();
                // 50
                newPatient.SecondaryNextOfKin.ContactInfo.Street2 = row["K2STREET2"].ToString();
                // 51
                newPatient.SecondaryNextOfKin.ContactInfo.Street3 = row["K2STREET3"].ToString();
                // 52
                newPatient.SecondaryNextOfKin.ContactInfo.City = row["K2CITY"].ToString();
                // 53
                newPatient.SecondaryNextOfKin.ContactInfo.State = GetState(row["K2STATE"].ToString());
                // 54
                newPatient.SecondaryNextOfKin.ContactInfo.ZipPlus4 = row["K2ZIP"].ToString();
                // 55
                newPatient.PrimaryContactInfo.County = row["PCOUNTY"].ToString();
                // 56
                newPatient.MaritalStatus = row["PMARITAL"].ToString();
                // 57
                newPatient.ReligiousPreference = row["PRELIGION"].ToString();
                // 58
                newPatient.TemporaryContactInfo.IsActive = Keys.Methods.StrToBool(row["PTACTIVE"].ToString());
                // 59
                newPatient.TemporaryContactInfo.Street1 = row["PTADDRESS1"].ToString();
                // 60
                newPatient.TemporaryContactInfo.Street2 = row["PTADDRESS2"].ToString();
                // 61
                newPatient.TemporaryContactInfo.Street3 = row["PTADDRESS3"].ToString();
                // 62
                newPatient.TemporaryContactInfo.City = row["PTCITY"].ToString();
                // 63
                newPatient.TemporaryContactInfo.State = GetState(row["PTSTATE"].ToString());
                // 64 - NOT USED
                //newPatient.TemporaryContactInfo.ZipCode = row["PTZIP"].ToString();
                // 65
                newPatient.TemporaryContactInfo.ZipPlus4 = row["PTZIP+4"].ToString();
                // 66
                newPatient.TemporaryContactInfo.Country = GetCountry(row["PTCOUNTRY"].ToString());
                // 67
                newPatient.TemporaryContactInfo.County = row["PTCOUNTY"].ToString();
                // 68
                newPatient.TemporaryContactInfo.Phone = row["PTPHONE"].ToString();
                // 69
                DateTime dt;

                if (Keys.Methods.TryParseDateTime(row["PTSTART"].ToString(), out dt))
                    newPatient.TemporaryContactInfo.Start = dt;
                // 70
                if (Keys.Methods.TryParseDateTime(row["PTEND"].ToString(), out dt))
                    newPatient.TemporaryContactInfo.End = dt;
                // 71
                newPatient.PrimaryContactInfo.PhoneCell = row["PCELL"].ToString();
                // 72
                newPatient.PrimaryContactInfo.PhonePager = row["PPAGER"].ToString();
                // 73
                newPatient.PrimaryContactInfo.EmailAddress = row["PEMAIL"].ToString();
                // 74
                newPatient.FugitiveFelonFlag = Keys.Methods.StrToBool(row["PF_FFF"].ToString());
                // 75
                newPatient.CatastrophicDisabilityFlag = Keys.Methods.StrToBool(row["PF_VCD"].ToString());
                // 76
                newPatient.PatientFlags.Clear();
                var flags = row["PFNATIONAL"].ToString();

                if (!string.IsNullOrEmpty(flags.Trim()))
                {
                    var flagParts = flags.Split('|');

                    foreach (var flag in flagParts)
                        newPatient.PatientFlags.Add(new PatientFlag(PatientFlagType.National, flag));
                }
                // 77
                flags = row["PFLOCAL"].ToString();

                if (!string.IsNullOrEmpty(flags.Trim()))
                {
                    var flagParts = flags.Split('|');

                    foreach (var flag in flagParts)
                        newPatient.PatientFlags.Add(new PatientFlag(PatientFlagType.Local, flag));
                }
                // 78
                newPatient.EnrollmentSubGroup = row["SUBGRP"].ToString();
                // 79
                newPatient.Category8Flag = Keys.Methods.StrToBool(row["CAT8G"].ToString());
                // 80 - Similar Patients

                // Inidicate the patient record was read from SDEC PTLOOKRS
                newPatient.PTLookRSRead = true;
            }
            catch (Exception ex)
            {
                hasError = true;
                var newEx = new Exception(string.Format("Error creating {0}'s patient record from the lookup record.", pName), ex);
                TraceLog.LogException(GetType().Name, "PTLOOKRS_ToPatientRecord", newEx);
            }

            return newPatient;
        }

        private DataTable GetPatientRecord(string patientIen)
        {
            if (string.IsNullOrEmpty(patientIen))
                return null;
            // Remove any stale records from the Keys.Tables.PatientTableCache
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.PatientTableCache);

            if (!Keys.Tables.PatientTableCache.ContainsKey(patientIen))
            {
                var parms = new List<string> { patientIen };
                var table = _adt.callOVIDRPC(Keys.RpcCalls.GetPatientRegA, parms);
                Keys.Tables.PatientTableCache.Add(patientIen, new DataTableCache(table));
            }

            return Keys.Tables.PatientTableCache[patientIen].Table;
        }

        public Patient GetPatient(ref Patient newPatient)
        {
            var table = GetPatientRecord(newPatient.IEN);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                //  1
                newPatient.IEN = row["IEN"].ToString();
                //  2
                //newPatient.Address = row["STREET"].ToString();
                newPatient.PrimaryContactInfo.Street1 = row["STREET"].ToString();
                //  3
                //newPatient.CityState = row["CITY/STATE"].ToString();
                newPatient.PrimaryContactInfo.CityState = row["CITY/STATE"].ToString();
                var cityAndState = ParseCityState(newPatient.PrimaryContactInfo.CityState);
                //newPatient.City = cityAndState[0];
                newPatient.PrimaryContactInfo.City = cityAndState[0];
                //newPatient.State = GetState(cityAndState[1]);
                newPatient.PrimaryContactInfo.State = GetState(cityAndState[1]);
                //  4
                newPatient.Ward = row["WARD:"].ToString();
                //  5
                //newPatient.ZipCode = row["ZIP"].ToString();
                newPatient.PrimaryContactInfo.ZipPlus4 = row["ZIP"].ToString();
                //  6
                newPatient.Name = row["Name"].ToString();
                //  7
                newPatient.DOB = row["DOB"].ToString();
                //  8
                newPatient.SSN = row["SSN"].ToString();
                //  9
                newPatient.HRN = row["HRN"].ToString();
                // 10
                newPatient.PrimaryContactInfo.PhoneResidence = row["HOMEPHONE"].ToString();
                // 11
                newPatient.PrimaryContactInfo.PhoneWork = row["OFCPHONE"].ToString();
                // 12
                newPatient.PrimaryContactInfo.PhoneMessage = row["MSGPHONE"].ToString();
                // 13
                newPatient.PrimaryNextOfKin.Name = row["NOK NAME"].ToString();
                // 14
                newPatient.PrimaryNextOfKin.Relationship = row["RELATIONSHIP"].ToString();
                // 15
                newPatient.PrimaryNextOfKin.ContactInfo.Phone = row["PHONE"].ToString();
                // 16
                newPatient.PrimaryNextOfKin.ContactInfo.Street1 = row["STREET"].ToString();
                // 17
                if (!string.IsNullOrEmpty(row["CITY"].ToString()))
                    newPatient.PrimaryNextOfKin.ContactInfo.City = row["CITY"].ToString();
                // 18
                if (!string.IsNullOrEmpty(row["STATE"].ToString()))
                    newPatient.PrimaryNextOfKin.ContactInfo.State = GetState(row["STATE"].ToString());
                // 19
                newPatient.PrimaryNextOfKin.ContactInfo.ZipPlus4 = row["NOK_ZIP"].ToString();
                // 20 - Date Reviewed (NOT USED)
                // 21
                newPatient.MedicareNo = row["Medicare#"].ToString();
                // 22
                newPatient.Suffix = row["Suffix"].ToString();
                // 23
                newPatient.RegistrationComments = row["RegistrationComments"].ToString();
                // 24
                newPatient.GAF.Text = row["GAF"].ToString();
                // 25 & 26
                newPatient.SetRaces(row["PRACE"].ToString(), row["PRACEN"].ToString());
                // 27 & 28
                newPatient.SetEthnicity(row["PETH"].ToString(), row["PETHN"].ToString());
                // 29
                //newPatient.Country = newPatient.Countries.FirstOrDefault(c => ((c.Ien == row["PCOUNTRY"].ToString()) || (c.Abbreviation == row["PCOUNTRY"].ToString())));
                newPatient.PrimaryContactInfo.Country = newPatient.Countries.FirstOrDefault(c => c.Ien == row["PCOUNTRY"].ToString() || c.Abbreviation == row["PCOUNTRY"].ToString());
                newPatient.PrimaryContactInfo.County = row["PCOUNTY"].ToString();
                // 30
                var fullGender = row["GENDER"].ToString();
                newPatient.Gender = fullGender.Substring(0, 1);
                // 31 - SENSITIVE - Sensiitive Record flags
                newPatient.SensitiveRecord = new PatientSensitiveRecord(row["SENSITIVE"].ToString());
                newPatient.IsSensitiveRecord = newPatient.SensitiveRecord.IsSensitiveRecord;
                // 32 - SVCCONN - Service Connected
                newPatient.ServiceConnection = row["SVCCONN"].ToString();
                // 33 - SVCCONNP - Service Connection Percentage
                newPatient.ServiceConnectionPercentage = row["SVCCONNP"].ToString();
                // 34 - BADADD - Setting BadAddressType will also set IsBadAddress
                newPatient.BadAddressType = row["BADADD"].ToString();

                //if (!string.IsNullOrEmpty(newPatient.BadAddressType))
                //    newPatient.IsBadAddress = true;
                // 35
                newPatient.PrimaryContactInfo.Street2 = row["PADDRES2"].ToString();
                // 36
                newPatient.PrimaryContactInfo.Street3 = row["PADDRES3"].ToString();
                // 37
                newPatient.PrimaryContactInfo.County = row["PCOUNTY"].ToString();
                // 38
                newPatient.PrimaryContactInfo.PhoneCell = row["PCELL"].ToString();
                // 39
                newPatient.PrimaryContactInfo.EmailAddress = row["PEMAIL"].ToString();
                // 40
                newPatient.MaritalStatus = row["PMARITAL"].ToString();
                // 41
                newPatient.ReligiousPreference = row["PRELIGION"].ToString();
                // 42
                newPatient.TemporaryContactInfo.Street1 = row["PTADDRESS1"].ToString();
                // 43
                newPatient.TemporaryContactInfo.Street2 = row["PTADDRESS2"].ToString();
                // 44
                newPatient.TemporaryContactInfo.Street3 = row["PTADDRESS3"].ToString();
                // 45
                newPatient.TemporaryContactInfo.City = row["PTCITY"].ToString();
                // 46
                newPatient.TemporaryContactInfo.State = GetState(row["PTSTATE"].ToString());
                // 47 NOT USED
                //newPatient.TemporaryContactInfo.ZipPlus4 = row["PTZIP"].ToString();
                // 48
                newPatient.TemporaryContactInfo.ZipPlus4 = row["PTZIP+4"].ToString();
                // 49
                newPatient.TemporaryContactInfo.Country = GetCountry(row["PTCOUNTRY"].ToString());
                // 50
                newPatient.TemporaryContactInfo.County = row["PTCOUNTY"].ToString();
                // 51
                DateTime dt;

                if (Keys.Methods.TryParseDateTime(row["PTSTART"].ToString(), out dt))
                    newPatient.TemporaryContactInfo.Start = dt;
                // 52
                if (Keys.Methods.TryParseDateTime(row["PTEND"].ToString(), out dt))
                    newPatient.TemporaryContactInfo.End = dt;
                if (newPatient.TemporaryContactInfo.Start.HasValue && newPatient.TemporaryContactInfo.End.HasValue)
                    // Setting Temporary Address Active based on Start\Stop Dates compared to today
                    newPatient.TemporaryContactInfo.IsActive = newPatient.TemporaryContactInfo.Start.Value.Date < DateTime.Now.Date && DateTime.Now.Date < newPatient.TemporaryContactInfo.End.Value.Date;
                else
                    newPatient.TemporaryContactInfo.IsActive = false;
                // 53
                newPatient.PrimaryNextOfKin.ContactInfo.Street2 = row["KSTREET2"].ToString();
                // 54
                newPatient.PrimaryNextOfKin.ContactInfo.Street3 = row["KSTREET3"].ToString();
                // 55 - Same as K2NAME, so not using
                //newPatient.SecondaryNextOfKin.IEN = row["NOK2"].ToString();
                // 56
                newPatient.SecondaryNextOfKin.Name = row["K2NAME"].ToString();
                // 57
                newPatient.SecondaryNextOfKin.Relationship = row["K2REL"].ToString();
                // 58
                newPatient.SecondaryNextOfKin.ContactInfo.Phone = row["K2PHONE"].ToString();
                // 59
                newPatient.SecondaryNextOfKin.ContactInfo.Street1 = row["K2STREET"].ToString();
                // 60
                newPatient.SecondaryNextOfKin.ContactInfo.Street2 = row["K2STREET2"].ToString();
                // 61
                newPatient.SecondaryNextOfKin.ContactInfo.Street3 = row["K2STREET3"].ToString();
                // 62
                newPatient.SecondaryNextOfKin.ContactInfo.City = row["K2CITY"].ToString();
                // 63
                newPatient.SecondaryNextOfKin.ContactInfo.State = GetState(row["K2STATE"].ToString());
                // 64
                newPatient.SecondaryNextOfKin.ContactInfo.ZipPlus4 = row["K2ZIP"].ToString();
                // 65
                newPatient.FugitiveFelonFlag = Keys.Methods.StrToBool(row["PF_FFF"].ToString());
                // 66
                newPatient.CatastrophicDisabilityFlag = Keys.Methods.StrToBool(row["PF_VCD"].ToString());
                // 67
                newPatient.PatientFlags.Clear();
                var flags = row["PFNATIONAL"].ToString();

                if (!string.IsNullOrEmpty(flags.Trim()))
                {
                    var flagParts = flags.Split('|');

                    foreach (var flag in flagParts)
                        newPatient.PatientFlags.Add(new PatientFlag(PatientFlagType.National, flag));
                }
                // 68
                flags = row["PFLOCAL"].ToString();

                if (!string.IsNullOrEmpty(flags.Trim()))
                {
                    var flagParts = flags.Split('|');

                    foreach (var flag in flagParts)
                        newPatient.PatientFlags.Add(new PatientFlag(PatientFlagType.Local, flag));
                }
                // 69
                newPatient.EnrollmentSubGroup = row["SUBGRP"].ToString();
                // 70
                newPatient.Category8Flag = Keys.Methods.StrToBool(row["CAT8G"].ToString());
                // 71 - Similar Patients

                // Indicate the Patient Record was read from SDEC GETREGA
                newPatient.GetRegARead = true;
            }
            return newPatient;
        }

        private string[] ParseCityState(string cityState)
        {
            if (cityState != null)
            {
                var lastComma = cityState.LastIndexOf(',');
                if (lastComma > -1)
                {
                    return new[]
                    {
                        cityState.Substring( 0, lastComma ),
                        cityState.Substring( lastComma + 1 )
                    };
                }
            }
            return new[] { cityState ?? "", "" };
        }

        public State GetPatientState(string patientIen)
        {
            var table = GetPatientRecord(patientIen);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                return GetState(row["STATE"].ToString());
            }

            return null;
        }

        public string GetPatientCityState(string patientIen)
        {
            var table = GetPatientRecord(patientIen);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                var citystate = row["CITY/STATE"].ToString();
                return citystate;
            }

            return "";
        }

        public string GetPatientWard(string patientIen)
        {
            var table = GetPatientRecord(patientIen);

            if (table.Rows.Count > 0)
            {
                var row = table.Rows[0];
                return row["WARD:"].ToString();
            }

            return "";
        }

        public string ConvertDateTimeToExternalMumpsFormat(DateTime dt)
        {
            var millenium = "3"; //yes... i'm hardcoding this... for Y3K...
            var yr = dt.Year.ToString();
            yr = yr.Substring(2, 2);
            var mnth = "0" + dt.Month;
            if (mnth.Length > 2)
                mnth = dt.Month.ToString();
            var day = "0" + dt.Day;
            if (day.Length > 2)
                day = dt.Day.ToString();
            var hour = "0" + dt.Hour;
            if (hour.Length > 2)
                hour = dt.Hour.ToString();
            var minute = "0" + dt.Minute;
            if (minute.Length > 2)
                minute = dt.Minute.ToString();
            return millenium + yr + mnth + day + "@" + hour + minute;
        }

        public string ConvertToExternalMumpsFormat(DateTime dt)
        {
            var millenium = "3"; //yes... i'm hardcoding this... for Y3K...
            var yr = dt.Year.ToString();
            yr = yr.Substring(2, 2);
            var mnth = "0" + dt.Month;
            if (mnth.Length > 2)
                mnth = dt.Month.ToString();
            var day = "0" + dt.Day;
            if (day.Length > 2)
                day = dt.Day.ToString();
            return millenium + yr + mnth + day;
        }

        public DateTime? ConvertOrigDate(string origDate)
        {
            if (string.IsNullOrEmpty(origDate))
                return null;
            var millenium = origDate.Substring(0, 1);
            millenium = (17 + Convert.ToInt32(millenium)).ToString();
            var year = millenium + origDate.Substring(1, 2);
            var month = origDate.Substring(3, 2);
            var day = origDate.Substring(5, 2);
            origDate = month + "/" + day + "/" + year;
            return Convert.ToDateTime(origDate).Date;
        }

        public RPMSClinic GetClinicByID(string clinicIEN)
        {
            var clinicTable = GetClinicSetupParameters(clinicIEN, null);

            if (clinicTable.Rows.Count == 0)
                return null;

            return GetRPMSClinicFromDataRow(clinicTable.Rows[0]);
        }

        public void RefreshClinic(RPMSClinic clinic)
        {
            var clinicTable = GetClinicSetupParameters(clinic.ClinicIen, null);

            if (clinicTable.Rows.Count == 0)
                return;

            GetRPMSClinicFromDataRow(clinicTable.Rows[0], clinic);
        }

        public AppointmentType GetDefaultClinicAppointmentType(string clinicIEN, string clinicName, string sdNoLet = "")
        {
            var table = GetClinicSetupParameters(clinicIEN, clinicName, sdNoLet);

            if (table.Rows.Count > 0)
                return new AppointmentType()
                {
                    Ien = table.Rows[0]["SDECDAT"].ToString(),
                    Name = table.Rows[0]["SDECDATN"].ToString()
                };

            return null;
        }

        private SchdResource ResourceFromDataRow(DataRow row)
        {
            var newResource = _container == null ? new SchdResource(this) : _container.Resolve<Factory<SchdResource>>().Create();

            newResource.RESOURCEID = row["RESOURCEID"].ToString();
            newResource.RESOURCE_NAME = row["RESOURCE_NAME"].ToString().ToUpper();
            newResource.INACTIVE = row["INACTIVE"].ToString();
            newResource.TIMESCALE = row["TIMESCALE"].ToString();
            newResource.HOSPITAL_LOCATION_ID = row["HOSPITAL_LOCATION_ID"].ToString();
            newResource.LETTER_TEXT = row["LETTER_TEXT"].ToString();
            newResource.NO_SHOW_LETTER = row["NO_SHOW_LETTER"].ToString();
            newResource.CLINIC_CANCELLATION_LETTER = row["CLINIC_CANCELLATION_LETTER"].ToString();
            newResource.VIEW = row["VIEW"].ToString();
            newResource.OVERBOOK = row["OVERBOOK"].ToString();
            newResource.MODIFY_SCHEDULE = row["MODIFY_SCHEDULE"].ToString();
            newResource.MODIFY_APPOINTMENTS = row["MODIFY_APPOINTMENTS"].ToString();
            newResource.ResourceType = row["RESOURCETYPE"].ToString();
            newResource.Date = row["DATE"].ToString();
            newResource.UserIEN = row["USERIEN"].ToString();
            newResource.UserName = row["USERNAME"].ToString();
            newResource.InactiveDate = row["DATE1"].ToString();
            newResource.InactiveUserIEN = row["USERIEN1"].ToString();
            newResource.InactiveUserName = row["USERNAME1"].ToString();
            newResource.ReactivatedDate = row["DATE2"].ToString();
            newResource.ReactivatedUserIEN = row["USERIEN2"].ToString();
            newResource.ReactivatedUserName = row["USERNAME2"].ToString();
            newResource.ClinicName = row["CLINNAME"].ToString();
            newResource.IsProhibitedClinic = Keys.Methods.StrToBool(row["PRHBLOC"].ToString());
            newResource.LastSub = row["LASTSUB"].ToString();
            return newResource;
        }

        private IList<SchdResource> ResourcesListFromDataTable(DataTable table, bool includeInactive)
        {
            if (table == null)
                return null;
            if (table.Columns.Count == 0)
                return null;

            var resourceList = new List<SchdResource>();
            foreach (DataRow row in table.Rows)
            {
                var res = ResourceFromDataRow(row);
                var isInactive = res.INACTIVE == "YES";

                if (!isInactive || includeInactive)
                {
                    resourceList.Add(res);
                }
            }
            resourceList.Sort((x, y) => string.Compare(x.RESOURCE_NAME, y.RESOURCE_NAME, StringComparison.OrdinalIgnoreCase));
            return resourceList;

        }

        /*        public IList<SchdResource> GetAllResources(bool includeInactive)
                {
                    return this.ResourcesListFromDataTable(GetResources(), includeInactive);
                }
        */

        public IList<SchdResource> GetResources(string searchString, bool includeInactive)
        {
            var parms = new List<string> { CurrentUser, includeInactive ? "0" : "1", "", Keys.ClinicResourceSearchMaxRecords.ToString(), null, null, searchString };
            var resourceList = new List<SchdResource>();
            try
            {
                var dt = _adt.callOVIDRPC(Keys.RpcCalls.SdecResource, parms);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        var res = ResourceFromDataRow(row);
                        resourceList.Add(res);
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetResources", e);
                Console.WriteLine(e.Message);
            }
            return resourceList;
        }


        /*        public IList<SchdResource> GetResourcesByType(string type, string searchString, bool includeInactive)
                {
                    if (searchString == null)
                        return _resourceEmptyList;
                    else
                        return GetResourcesByType(type, includeInactive).Where(res => ((res.RESOURCE_NAME.Length >= searchString.Length) && (res.RESOURCE_NAME.Substring(0, searchString.Length).ToUpper() == searchString.ToUpper()))).ToList();
                }
        */

        /*        public IList<SchdResource> GetResourcesByType(string type, bool includeInactive)
                {

                    if(type.ToUpper() != "H" && type.ToUpper() != "P" && type.ToUpper() != "A")
                    {
                        MessageBox.Show("Invalid resource type specified.  Please contact your System Administrator.");
                        return null;
                    }

                    DataTable table;

                    if (type == "H")
                        table = GetHospitalResources();
                    else  
                        table = GetResources();

                    return ResourcesListFromDataTable(table, includeInactive);
                }
        */

        public string GetResourceIdByClinicIdAndName(string clinicId, string clinName, bool includeInactive, bool silent = true)
        {
            if (string.IsNullOrEmpty(clinicId) || string.IsNullOrEmpty(clinName))
                return string.Empty;

            var list = GetSdecHResources(clinName);
            var res = list.FirstOrDefault(r => r.HOSPITAL_LOCATION_ID == clinicId);

            if (res != null)
                return res.RESOURCEID;

            if (!silent)
                MessageBox.Show(string.Format("{0} is an inactive clinic, please select another clinic to schedule this appointment.", clinName), "Inactive Clinic", MessageBoxButton.OK, MessageBoxImage.Warning);

            return string.Empty;
        }

        public bool GetVariableAppointmentsFlag(string hospLocID)
        {
            var result = false;
            var parms = new List<string> { hospLocID };
            var varTable = _adt.callOVIDRPC("SDEC CVARAPPT", parms);

            if (varTable != null && varTable.Rows.Count > 0)
            {
                try
                {
                    if (varTable.Columns[0].ColumnName == "VAR_APPT_FLAG")
                        result = varTable.Rows[0][0].ToString() == "1";
                    else if (varTable.Columns[0].ColumnName == "VAR_APPT_FLAG")
                        throw new Exception(varTable.Rows[0][0].ToString());
                    else
                        throw new Exception("Calling the Clinic Variable Length Flag RPC returned unexpected data.\nReview the Trace Log for details.");
                }
                catch (Exception ex)
                {
                    Keys.AlertUserError(this, ex, "Clinic Variable Length Flag Error");
                }
            }

            return result;
        }

        public List<Clinic> GetClinicsWithMatchingStopCode(string clinicID, string searchString = "")
        {
            /****************************
            searchString = STOP - (optional)Clinic Stop partial name lookup into the CLINIC STOP
                   file(#40.7)
                   OR Clinic Stop id pointer to the CLINIC STOP file (#40.7)
                   OR "A"999 Amis Reporting Stop Code
            clinicID = SC   - (optional) Clinic ID pointer to HOSPITAL LOCATION file(#44)
            ****************************/
            List<string> parms = new List<string> { searchString, clinicID };
            var table = _adt.callOVIDRPC(Keys.RpcCalls.ClinicListByStopCode, parms);
            List<Clinic> clinics = new List<Clinic>();

            if (table.Columns.Count > 0)
            {
                foreach (DataRow row in table.Rows)
                {
                    RPMSClinic newClinic = new RPMSClinic(this)
                    {
                        IsCompleteRecord = false,
                        CLINIC_STOP = row["CLINSTOP"].ToString(),
                        ClinicIen = row["CLINIEN"].ToString(),
                        ClinicName = row["CLINNAME"].ToString(),
                    };

                    clinics.Add(newClinic);
                }
            }

            return clinics;
        }

        public IList<SchdResource> GetClinicsByProvider(string providerIEN)
        {
            if (providerIEN == null)
                return _resourceEmptyList;

            var parms = new List<string> { providerIEN };
            var table = _adt.callOVIDRPC(Keys.RpcCalls.ClinicByProvider, parms);

            var resourceList = new List<SchdResource>();

            if (table.Columns.Count > 1)
            {
                foreach (DataRow row in table.Rows)
                {
                    var newResourceList = GetSdecHResources(row["CLINIC_NAME"].ToString());
                    //ResourceList.Add(schdRes);

                    foreach (var item in newResourceList)
                    {
                        if (item.HOSPITAL_LOCATION_ID == row["CLINIC_IEN"].ToString())
                        {
                            resourceList.Add(item);
                        }
                    }
                }
            }
            return resourceList;
        }

        public IList<Provider> GetProvidersByClinic(string clinicIEN)
        {
            // Remove any stale records from the Keys.Tables.ProvClinCache
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.ProvClinCache);
            var providerList = new List<Provider>();

            if (clinicIEN == null)
                return providerList;

            if (!Keys.Tables.ProvClinCache.ContainsKey(clinicIEN))
            {
                var parms = new List<string> { clinicIEN };
                var table = _adt.callOVIDRPC(Keys.RpcCalls.ProviderByClinics, parms);
                Keys.Tables.ProvClinCache.Add(clinicIEN, new DataTableCache(table, 30));
            }

            foreach (DataRow row in Keys.Tables.ProvClinCache[clinicIEN].Table.Rows)
            {
                var newProvider = _container.Resolve<Factory<Provider>>().Create();
                newProvider.IEN = row["PROVIDER_IEN"].ToString();
                newProvider.Name = row["PROVIDER_NAME"].ToString();
                newProvider.IsDefaultProvider = row["DEFAULT"].ToString() == "YES";

                providerList.Add(newProvider);
            }
            //TODO: call ADDRES..
            return providerList;
        }

        private DataTable GetResourceGroupsData()
        {
            if (Keys.Tables.ResourceGroupDataLoaded.AddMinutes(30) < DateTime.Now || Keys.Tables.ResourceGroupData == null)
            {
                var parms = new List<string> { CurrentUser };
                Keys.Tables.ResourceGroupData = _adt.callOVIDRPC("SDEC RESGPUSR", parms);
                Keys.Tables.ResourceGroupDataLoaded = DateTime.Now;
            }
            return Keys.Tables.ResourceGroupData;
        }

        public IList<SchdResourceGroup> GetResourceGroups()
        {
            var resourceGroupList = new List<SchdResourceGroup>();
            var table = GetResourceGroupsData();

            if (table != null)
            {
                foreach (DataRow row in table.Rows)
                {
                    var newResourceGroup = _container.Resolve<Factory<SchdResourceGroup>>().Create();
                    var resourceGroupAlreadyAdded = false;
                    foreach (var item in resourceGroupList)
                    {
                        if (item.IEN == row["RESOURCE_GROUPID"].ToString())
                        {
                            newResourceGroup = item;
                            resourceGroupAlreadyAdded = true;
                            break;
                        }
                    }
                    if (!resourceGroupAlreadyAdded)
                    {
                        newResourceGroup.Name = row["RESOURCE_GROUP"].ToString();
                        newResourceGroup.IEN = row["RESOURCE_GROUPID"].ToString();
                        resourceGroupList.Add(newResourceGroup);
                    }
                }
            }
            return resourceGroupList;
        }

        public int GetSelectedType(string sResourceName, DateTime startDate, DateTime endDate)
        {
            var rsOut = CreateAssignedSlotSchedule(sResourceName, startDate.ToString(), endDate.ToString(), "", "");
            if (rsOut.Rows.Count == 0)
                return 0;

            var nAccessTypeID = 0;
            for (var i = 0; i < rsOut.Rows.Count; i++)
            {
                var r = rsOut.Rows[i];
                var dStart = DateTime.Parse(r["START_TIME"].ToString());
                var dEnd = DateTime.Parse(r["END_TIME"].ToString());

                if (Keys.Methods.TimesOverlap(dStart, dEnd, startDate, endDate))
                {
                    nAccessTypeID = int.Parse(r["ACCESS_TYPE"].ToString());
                    break;
                }
            }
            return nAccessTypeID;
        }

        public string AutoRebook(AutoRebookData autoRebookData)
        {
            var aRebook = autoRebookData.RebookAppointment;

            //get accessTypeID
            var autoRebookAccessID = GetSelectedType(aRebook.RESOURCENAME, Convert.ToDateTime(aRebook.START_TIME), Convert.ToDateTime(aRebook.END_TIME));

            var dResult = new DateTime(); //StartTime of access block to autorebook into
            var sSearchInfo = "";

            //Find the StartTime of first availability block of this type for this clinic
            //between nMinimumDays and nMaximumDays
            var sAVStart = Convert.ToDateTime(autoRebookData.RebookAppointment.START_TIME).AddDays(autoRebookData.MinimumDays).ToString("M/d/yyyy@H:mm");
            var dtAVEnd = Convert.ToDateTime(autoRebookData.RebookAppointment.START_TIME).AddDays(autoRebookData.MinimumDays + autoRebookData.MaximumDays);
            var sAVEnd = dtAVEnd.ToString("M/d/yyyy@H:mm");

            //Increment start day to search a week (or so) at a time
            //30 is a test increment.  Need to test different values for performance
            //int nIncrement = (autoRebookData.MaximumDays < 30) ? autoRebookData.MaximumDays : 30;
            var nIncrement = autoRebookData.MaximumDays;

            //nCount and nCountEnd are the 'moving' counters 
            //that I add to start and end to get the bracket
            //At the beginning of the DO loop, nCount and nCountEnd are already set
            var bFinished = false;
            var bFound = false;

            var dStart = Convert.ToDateTime(autoRebookData.RebookAppointment.START_TIME).AddDays(autoRebookData.MinimumDays);
            var dEnd = dStart.AddDays(nIncrement);

            do
            {
                var parms = new List<string> { dStart.ToString("M/d/yyyy@H:mm"), autoRebookData.RebookAppointment.RESOURCENAME, autoRebookAccessID.ToString() };
                var table = _adt.callOVIDRPC("SDEC REBKNEXT", parms);

                if (table.Rows.Count > 0)
                {
                    var drNextBlockRow = table.Rows[0];
                    var oNextBlock = drNextBlockRow["NEXTBLOCK"];
                    if (oNextBlock.ToString() == string.Empty)
                        break;
                    var dNextBlock = Convert.ToDateTime(drNextBlockRow["NEXTBLOCK"].ToString());
                    if (dNextBlock > dtAVEnd)
                        break;

                    dStart = dNextBlock;
                    dEnd = dStart.AddDays(nIncrement);
                    if (dEnd > dtAVEnd)
                        dEnd = dtAVEnd;
                }

                var dtResult = CreateAssignedSlotSchedule(autoRebookData.RebookAppointment.RESOURCENAME, dStart.ToString(), dEnd.ToString(), "", sSearchInfo);

                //Loop thru dtResult looking for a slot having the required availability type.
                //If found, set bFinished = true;	
                foreach (DataRow dr in dtResult.Rows)
                {
                    var sSlots = dr["SLOTS"].ToString();
                    if (sSlots == "")
                        sSlots = "0";
                    int nSlots = Convert.ToInt16(sSlots);
                    if (nSlots > 0)
                    {
                        var nAccessTypeID = 0;
                        if (dr["ACCESS_TYPE"].ToString() != "")
                            nAccessTypeID = Convert.ToInt16(dr["ACCESS_TYPE"].ToString());
                        if (!autoRebookData.IsAccessTypeChecked && (nAccessTypeID > 0) && !IsPreventAccessFromID(nAccessTypeID)) //Match on any non-zero type
                        {
                            bFinished = true;
                            bFound = true;
                            dResult = (DateTime)dr["START_TIME"];
                            break;
                        }
                        if (nAccessTypeID == autoRebookAccessID && !IsPreventAccessFromID(nAccessTypeID))
                        {
                            bFinished = true;
                            bFound = true;
                            dResult = (DateTime)dr["START_TIME"];
                            break;
                        }
                    }
                }
                dStart = dEnd.AddDays(1);
                dEnd = dStart.AddDays(nIncrement);
                if (dEnd > dtAVEnd)
                    dEnd = dtAVEnd;
            } while (bFinished == false);

            string error;
            if (!bFound)
                return "Unable to rebook this patient: " + aRebook.PATIENTNAME;

            var a = new AppointmentData
            {
                AppointmentID = "0",
                Duration = (Convert.ToDateTime(aRebook.END_TIME) - Convert.ToDateTime(aRebook.START_TIME)).TotalMinutes.ToString(),
                StartTime = dResult.ToString(),
                Notes = aRebook.NOTE,
                ResourceName = aRebook.RESOURCENAME
            };
            a.EndTime = dResult.AddMinutes(int.Parse(a.Duration)).ToString();
            var p = new PatientInformation
            {
                IEN = aRebook.PATIENTID
            };
            var errorMessages = AddNewAppointment(a, p, false);
            SetAutoRebook(aRebook.APPOINTMENTID, dResult);
            DisplayAutoRebookLetter(errorMessages[0]);
            error = "A new appointment for patient " + p.Name + " has been booked on " + a.StartTime;
            return error;
        }

        private void DisplayAutoRebookLetter(string apptID)
        {
            var args = new object[2];
            args[0] = "Patient Auto Rebook Letter";
            args[1] = apptID + "|";
            _eventAggregator.GetEvent<DisplayReportEvent>().Publish(args);
        }

        private void SetAutoRebook(string apptID, DateTime dtRebookedTo)
        {
            var parms = new List<string> { apptID, dtRebookedTo.ToString("M/d/yyyy@HH:mm") };
            var table = _adt.callOVIDRPC("SDEC SETRBOOK", parms);
        }

        private IList<FindAppointmentResult> _findAppointments;
        public IList<FindAppointmentResult> FindAppointments(FindAppointmentData searchData)
        {
            var tempSearchResult = new List<FindAppointmentResult>();

            try
            {
                var holdSlots = Keys.AppointmentSlots;

                try
                {
                    // iterate all resources and accumulate the slots
                    foreach (var res in searchData.Resources)
                    {
                        GetApptSlots(res, searchData.StartDate, searchData.EndDate);
                        SlotsAvailable appSlots = Keys.AppointmentSlots;

                        foreach (var slot in appSlots.Slots)
                        {
                            //  Slot is UNAVAILABLE
                            if (slot.AccessType != Keys.AccessTypes.Available)
                                continue;

                            // Does slot meet AM\PM\Both filter
                            if (searchData.AMPM == "AM")
                            {
                                if (!slot.IsAMSlot)
                                    continue;
                            }
                            else if (searchData.AMPM == "PM")
                            {
                                if (!slot.IsPMSlot)
                                    continue;
                            }

                            // Does slot meet the day filter
                            if (!string.IsNullOrEmpty(searchData.DAYOFWEEK) && !searchData.DAYOFWEEK.Contains(slot.DayOfWeekCode))
                                continue;

                            // Does the slot have availability ??
                            if (!slot.HasSlotsAvailable)
                                // if not, Can the slot be booked ??
                                if (!ClinicOverbook(res.Clinic.HOSPITAL_LOCATION_ID, slot.StartTime, res.RESOURCEID, true))
                                    continue;

                            // If we get here the slot SHOULD be able to be booked, save it
                            var fr = new FindAppointmentResult();
                            fr.StartTime = slot.StartTime.ToString();
                            fr.EndTime = slot.EndTime.ToString();
                            fr.Slots = slot.SlotCount;
                            fr.AvailabilityCount = slot.AvailabilityCount;
                            fr.ResourceName = appSlots.Name;
                            fr.AccessType = "AVAILABLE";
                            tempSearchResult.Add(fr);
                        }
                    }
                }
                finally
                {
                    Keys.AppointmentSlots = holdSlots;
                }

                _findAppointments = tempSearchResult;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "FindAppointments", ex);
                throw new Exception("Error in DataAccessService.FindAppointments", ex);
                //                MessageBox.Show(ex.Message, "Find Appointments Error");
            }

            return _findAppointments;
        }

        private DataTable CreateAssignedSlotSchedule(string sResourceName, string sStartTime, string sEndTime, string sAccessTypes, string sSearchInfo)
        {
            try
            {
                var resource = GetSdecResourceByName(sResourceName);

                if (resource != null)
                    return CreateAssignedSlotSchedule(resource, sStartTime, sEndTime, sAccessTypes, sSearchInfo);
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CreateAssignedSlotSchedule(resourceName, . . . ", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }

            return null;
        }

        private DataTable CreateAssignedSlotSchedule(SchdResource resource, string sStartTime, string sEndTime, string sAccessTypes, string sSearchInfo)
        {
            var availabilityFound = false;
            var isHoliday = false;
            var cannotBookHolidays = !resource.Clinic.ClinicCanBookOnHolidays; ;

            try
            {
                GetApptSlots(resource, sStartTime, sEndTime);
                var dtTemp = new DataTable();
                dtTemp.Columns.Add("START_TIME", Type.GetType("System.DateTime"));
                dtTemp.Columns.Add("END_TIME", Type.GetType("System.DateTime"));
                dtTemp.Columns.Add("SLOTS", Type.GetType("System.Int32"));
                dtTemp.Columns.Add("OVERBOOKED_COUNT", Type.GetType("System.Int32"));
                dtTemp.Columns.Add("SLOT_COUNT", Type.GetType("System.String"));
                dtTemp.Columns.Add("RESOURCE", Type.GetType("System.String"));
                dtTemp.Columns.Add("ACCESS_TYPE", Type.GetType("System.String"));
                dtTemp.Columns.Add("NOTE", Type.GetType("System.String"));
                dtTemp.Columns.Add("AVAILABILITYID", Type.GetType("System.Int32"));
                var prevDay = DateTime.MinValue;
                var availabilityIndex = 1;

                var slotsAvailable = Keys.AppointmentSlots;

                foreach (var slot in slotsAvailable.Slots)
                {
                    var rNew = dtTemp.NewRow();
                    var curDay = slot.StartTime.Date;

                    if (prevDay.CompareTo(curDay) >= 0)
                    {
                        prevDay = curDay;
                        // Reset day based variables
                        isHoliday = !string.IsNullOrEmpty(IsHoliday(curDay));
                    }

                    availabilityFound = availabilityFound || (slot.AccessType == Keys.AccessTypes.Available);

                    rNew["START_TIME"] = slot.StartTime;
                    rNew["END_TIME"] = slot.EndTime;
                    rNew["RESOURCE"] = resource.RESOURCE_NAME;
                    rNew["OVERBOOKED_COUNT"] = slot.OverbookedCount;

                    if (isHoliday && cannotBookHolidays)
                    {
                        rNew["ACCESS_TYPE"] = "2";
                        rNew["SLOTS"] = "0";
                        rNew["SLOT_COUNT"] = "";
                    }
                    else
                    {
                        rNew["ACCESS_TYPE"] = slot.AccessType;
                        rNew["SLOTS"] = slot.AvailabilityCount;
                        rNew["SLOT_COUNT"] = slot.SlotCount;
                    }

                    //rNew["NOTE"] = r[dtRet.Columns["NOTE"]];
                    rNew["AVAILABILITYID"] = availabilityIndex++;
                    dtTemp.Rows.Add(rNew);
                }

                if (availabilityFound && _eventAggregator != null)
                    _eventAggregator.GetEvent<CollapseTimeSlotViewerEvent>().Publish(true);
                return dtTemp;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CreateAssignedSlotSchedule(resource, . . .", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }
            return null;
        }

        public bool CheckIfAvailabilityEverDefined(SchdResource resource, bool isSilent = false)
        {
            try
            {
                if (resource == null || string.IsNullOrEmpty(resource.HOSPITAL_LOCATION_ID))
                    return false;

                var parms = new List<string> { resource.HOSPITAL_LOCATION_ID };

                var dtRslt = _adt.callOVIDRPC(Keys.RpcCalls.AvailabilityEverDefined, parms);

                if (dtRslt != null && dtRslt.Rows.Count > 0)
                {
                    var dr = dtRslt.Rows[0];
                    var rslt = dr["AVAILABILITY"].ToString();

                    if (rslt.ToUpper() == "NO")
                    {
                        if (!isSilent)
                        {
                            if (_eventAggregator != null)
                                _eventAggregator.GetEvent<CollapseTimeSlotViewerEvent>().Publish(false);
                            var msg = "Clinic " + resource.RESOURCE_NAME + ", has no availability defined";
                            Dispatcher.BeginInvoke(new Action(() => Keys.AlertUser(this, msg, "VS Availability")));
                        }
                        return false;
                    }
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CheckIfAvailabilityEverDefined", ex);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }

            return true;
        }

        public DataTable CreateAppointmentSlotSchedule(string sResourceName, string startTime, string endTime)
        {
            var parms1 = new List<string> { Convert.ToDateTime(startTime).ToString("M-d-yyyy"), Convert.ToDateTime(endTime).ToString("M-d-yyyy"), sResourceName };
            var dtAppt = _adt.callOVIDRPC(Keys.RpcCalls.ConflictingApptsForClinic, parms1);

            var dtCopy = new DataTable("dtCopy");
            if (dtAppt.Rows.Count > 0)
            {
                //Create CDateTimeArray & load records from rsOut
                var nRC = dtAppt.Rows.Count;
                var cdtArray = new ArrayList { Capacity = nRC * 2 };
                int icount;

                foreach (DataRow r in dtAppt.Rows)
                {
                    DateTime v;
                    if (string.IsNullOrEmpty(r[dtAppt.Columns["START_TIME"]].ToString()))
                        continue;

                    v = Convert.ToDateTime(r[dtAppt.Columns["START_TIME"]]);
                    cdtArray.Add(v);
                    v = Convert.ToDateTime(r[dtAppt.Columns["END_TIME"]]);
                    cdtArray.Add(v);
                }
                cdtArray.Sort();

                //Create a CTimeBlockArray and load it from rsOut
                var ctbAppointments = new ArrayList(nRC);
                SchdAvailability cTB;
                foreach (DataRow r in dtAppt.Rows)
                {
                    cTB = new SchdAvailability
                    {
                        StartTime = r[dtAppt.Columns["START_TIME"]].ToString(),
                        EndTime = r[dtAppt.Columns["END_TIME"]].ToString()
                    };
                    ctbAppointments.Add(cTB);
                }

                //Create a TimeBlock Array from the data in the DateTime array
                var ctbApptSchedule = new ArrayList();
                var csStartTime = Convert.ToDateTime(startTime);
                var csEndTime = Convert.ToDateTime(endTime);

                if (csStartTime == csEndTime)
                {
                    csEndTime = csEndTime.AddDays(1).AddMinutes(-1);
                }
                ScheduleFromArray(cdtArray, csStartTime, csEndTime, ref ctbApptSchedule);

                //Find number of TimeBlocks in ctbApptSchedule that
                //overlap the TimeBlocks in ctbAppointments
                var ctbApptSchedule2 = new ArrayList();
                for (icount = 0; icount < ctbApptSchedule.Count; icount++)
                {
                    cTB = (SchdAvailability)ctbApptSchedule[icount];
                    var nSlots = BlocksOverlap(cTB, ctbAppointments, false);
                    var cTB2 = new SchdAvailability();
                    cTB2.Create(cTB.StartTime, cTB.EndTime, nSlots);
                    ctbApptSchedule2.Add(cTB2);
                }

                ConsolidateBlocks(ctbApptSchedule2);

                var dCol = new DataColumn
                {
                    DataType = Type.GetType("System.DateTime"),
                    ColumnName = "START_TIME",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                dtCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.DateTime"),
                    ColumnName = "END_TIME",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                dtCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.Int32"),
                    ColumnName = "SLOTS",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                dtCopy.Columns.Add(dCol);

                dCol = new DataColumn
                {
                    DataType = Type.GetType("System.String"),
                    ColumnName = "RESOURCE",
                    ReadOnly = true,
                    AllowDBNull = false,
                    Unique = false
                };
                dtCopy.Columns.Add(dCol);

                foreach (SchdAvailability apptSchedule in ctbApptSchedule2)
                {
                    var newRow = dtCopy.NewRow();

                    newRow["START_TIME"] = apptSchedule.StartTime;
                    newRow["END_TIME"] = apptSchedule.EndTime;
                    newRow["SLOTS"] = apptSchedule.SLOTS;
                    newRow["RESOURCE"] = sResourceName;
                    dtCopy.Rows.Add(newRow);
                }
            }
            return dtCopy;
        }

        public DataTable UnionBlocks(DataTable rs1, DataTable rs2)
        {
            DataTable rsCopy;
            DataRow dr;
            if (rs1.Columns.Count >= rs2.Columns.Count)
            {
                rsCopy = rs1.Copy();
                foreach (DataRow dr2 in rs2.Rows)
                {
                    dr = rsCopy.NewRow();
                    dr.ItemArray = dr2.ItemArray;
                    rsCopy.Rows.Add(dr);
                }
            }
            else
            {
                rsCopy = rs2.Copy();
                foreach (DataRow dr2 in rs1.Rows)
                {
                    dr = rsCopy.NewRow();
                    dr.ItemArray = dr2.ItemArray;
                    rsCopy.Rows.Add(dr);
                }
            }
            return rsCopy;
        }

        public DataTable SubtractSlotsRS2(DataTable rsBlocks1, DataTable rsBlocks2, string sResource)
        {
            if (rsBlocks2.Rows.Count == 0 && rsBlocks1.Rows.Count == 0)
                return rsBlocks1;

            //Create an array of the start and end times of blocks2
            var cdtArray = new ArrayList(2 * (rsBlocks1.Rows.Count + rsBlocks2.Rows.Count));

            foreach (DataRow r in rsBlocks1.Rows)
            {
                cdtArray.Add(r[rsBlocks1.Columns["START_TIME"]]);
                cdtArray.Add(r[rsBlocks1.Columns["END_TIME"]]);
            }

            foreach (DataRow r in rsBlocks2.Rows)
            {
                cdtArray.Add(r[rsBlocks2.Columns["START_TIME"]]);
                cdtArray.Add(r[rsBlocks2.Columns["END_TIME"]]);
            }

            cdtArray.Sort();

            var ctbReturn = new ArrayList();
            var cDate = new DateTime();
            ScheduleFromArray(cdtArray, cDate, cDate, ref ctbReturn);

            //Set up return table
            var rsCopy = CreateCopyTable();

            try
            {
                foreach (SchdAvailability schedule in ctbReturn)
                {
                    long nSlots = SlotsInBlock(schedule, rsBlocks1) - SlotsInBlock(schedule, rsBlocks2);
                    var sResourceList = "";
                    var sAccessRuleList = "";
                    var sNote = "";

                    if (nSlots > 0)
                    {
                        var bRet = ResourceRulesInBlock(schedule, rsBlocks1, ref sResourceList, ref sAccessRuleList, ref sNote);
                    }
                    var newRow = rsCopy.NewRow();

                    newRow["START_TIME"] = schedule.StartTime;
                    newRow["END_TIME"] = schedule.EndTime;
                    newRow["SLOTS"] = nSlots;
                    newRow["RESOURCE"] = sResource;
                    newRow["ACCESS_TYPE"] = sAccessRuleList;
                    newRow["NOTE"] = sNote;
                    rsCopy.Rows.Add(newRow);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Subtract Slots Error");
            }
            return rsCopy;
        }

        public bool ResourceRulesInBlock(SchdAvailability rTimeBlock, DataTable rsBlock, ref string sResourceList, ref string sAccessRuleList, ref string sNote)
        {
            if (rsBlock.Rows.Count == 0)
                return true;

            var dStart1 = DateTime.Parse(rTimeBlock.StartTime);
            var dEnd1 = DateTime.Parse(rTimeBlock.EndTime);

            var dtTypes = _mDsGlobal.Tables["AccessTypes"];
            var dvTypes = dtTypes.DefaultView;
            dvTypes.Sort = "SDEC_ACCESS_TYPE_IEN ASC";

            foreach (DataRow r in rsBlock.Rows)
            {
                var dStart2 = Convert.ToDateTime(r[rsBlock.Columns["START_TIME"]]);
                var dEnd2 = Convert.ToDateTime(r[rsBlock.Columns["END_TIME"]]);

                if (Keys.Methods.TimesOverlap(dStart1, dEnd1, dStart2, dEnd2))
                {
                    var sResource = r[rsBlock.Columns["RESOURCE"]].ToString();
                    if (sResource == "NULL")
                        sResource = "";
                    if (sResource != "")
                    {
                        if (sResourceList == "")
                            sResourceList += sResource;
                        else
                            sResourceList += "^" + sResource;
                    }
                    var sAccessRule = r[rsBlock.Columns["ACCESS_TYPE"]].ToString();
                    if (sAccessRule == "0")
                        sAccessRule = "";
                    if (sAccessRule != "")
                    {
                        if (sAccessRuleList == "")
                        {
                            sAccessRuleList += sAccessRule;
                        }
                        else
                        {
                            var nRow = dvTypes.Find(sAccessRule);
                            if (nRow < 0)
                                continue;
                            var drv = dvTypes[nRow];
                            var sIsPreventAccess = drv["PREVENT_ACCESS"].ToString();

                            if (sIsPreventAccess == "YES")
                            {
                                sAccessRule += "^" + sAccessRuleList;
                                sAccessRuleList = sAccessRule;
                            }
                            else
                            {
                                sAccessRuleList += "^" + sAccessRule;
                            }
                        }
                    }
                    sNote = r[rsBlock.Columns["NOTE"]].ToString();
                }
            }
            return true;
        }//End ResourceRulesInBlock

        public DataTable CreateCopyTable()
        {
            var dtCopy = new DataTable("dtCopy");
            var dCol = new DataColumn
            {
                DataType = Type.GetType("System.DateTime"),
                ColumnName = "START_TIME",
                ReadOnly = true,
                AllowDBNull = false,
                Unique = false
            };
            dtCopy.Columns.Add(dCol);

            dCol = new DataColumn
            {
                DataType = Type.GetType("System.DateTime"),
                ColumnName = "END_TIME",
                ReadOnly = true,
                AllowDBNull = false,
                Unique = false
            };
            dtCopy.Columns.Add(dCol);

            dCol = new DataColumn
            {
                DataType = Type.GetType("System.Int32"),
                ColumnName = "SLOTS",
                ReadOnly = true,
                AllowDBNull = false,
                Unique = false
            };
            dtCopy.Columns.Add(dCol);

            dCol = new DataColumn
            {
                DataType = Type.GetType("System.String"),
                ColumnName = "RESOURCE",
                ReadOnly = true,
                AllowDBNull = true,
                Unique = false
            };
            dtCopy.Columns.Add(dCol);

            dCol = new DataColumn
            {
                DataType = Type.GetType("System.String"),
                ColumnName = "ACCESS_TYPE",
                ReadOnly = true,
                AllowDBNull = true,
                Unique = false
            };
            dtCopy.Columns.Add(dCol);

            dCol = new DataColumn
            {
                DataType = Type.GetType("System.String"),
                ColumnName = "NOTE",
                ReadOnly = true,
                AllowDBNull = true,
                Unique = false
            };
            dtCopy.Columns.Add(dCol);

            return dtCopy;
        }

        public int SlotsInBlock(SchdAvailability rTimeBlock, DataTable rsBlock)
        {
            var nSlots = 0;

            if (rsBlock.Rows.Count == 0)
                return nSlots;

            var dStart1 = Convert.ToDateTime(rTimeBlock.StartTime);
            var dEnd1 = Convert.ToDateTime(rTimeBlock.EndTime);

            foreach (DataRow r in rsBlock.Rows)
            {
                var dStart2 = Convert.ToDateTime(r[rsBlock.Columns["START_TIME"]]);
                var dEnd2 = Convert.ToDateTime(r[rsBlock.Columns["END_TIME"]]);

                if (Keys.Methods.TimesOverlap(dStart1, dEnd1, dStart2, dEnd2))
                {
                    var sSlots = r[rsBlock.Columns["SLOTS"]].ToString();
                    nSlots = Convert.ToInt16(sSlots);
                    break;
                }
            }
            return nSlots;
        }

        public void ConsolidateBlocks(ArrayList rTBArray)
        {
            bool bDirty;
            if (rTBArray.Count < 2)
                return;
            do
            {
                bDirty = false;
                int j;
                for (j = 0; j < rTBArray.Count - 1; j++)
                {
                    var cBlockA = (SchdAvailability)rTBArray[j];
                    var cBlockB = (SchdAvailability)rTBArray[j + 1];
                    if (cBlockA.EndTime == cBlockB.StartTime
                        && cBlockA.SLOTS == cBlockB.SLOTS
                        && cBlockA.RESOURCENAME == cBlockB.RESOURCENAME
                        && cBlockA.RESOURCENAME == cBlockB.RESOURCENAME
                        )
                    {
                        var cTemp1 = new SchdAvailability
                        {
                            StartTime = cBlockA.StartTime,
                            EndTime = cBlockB.EndTime,
                            SLOTS = cBlockA.SLOTS,
                            AccessRuleList = cBlockA.AccessRuleList,
                            RESOURCENAME = cBlockA.RESOURCENAME
                        };
                        rTBArray.Insert(j, cTemp1);
                        rTBArray.RemoveRange(j + 1, 2);
                        bDirty = true;
                        break;
                    }
                }
            }
            while (bDirty && rTBArray.Count != 1);
        }

        public int BlocksOverlap(SchdAvailability rBlock, ArrayList rTBArray, bool bCountSlots)
        {
            var nCount = 0;

            var dStart1 = DateTime.Parse(rBlock.StartTime);
            var dEnd1 = DateTime.Parse(rBlock.EndTime);

            foreach (SchdAvailability cBlock in rTBArray)
            {
                var dStart2 = DateTime.Parse(cBlock.StartTime);
                var dEnd2 = DateTime.Parse(cBlock.EndTime);
                var nSlots = cBlock.SLOTS;
                if (Keys.Methods.TimesOverlap(dStart1, dEnd1, dStart2, dEnd2))
                {
                    if (bCountSlots)
                        nCount += nSlots;
                    else
                        nCount++;
                }
            }
            return nCount;
        }

        public void ScheduleFromArray(ArrayList cdtArray, DateTime dStartTime, DateTime dEndTime, ref ArrayList rTBArray)
        {
            int j;
            SchdAvailability cTB;

            if (cdtArray.Count == 0)
                return;

            //If StartTime passed in, then adjust for it
            if (dStartTime.Ticks > 0)
            {
                if ((DateTime)cdtArray[0] > dStartTime)
                {
                    cTB = new SchdAvailability();
                    cTB.Create(dStartTime.ToString(), ((DateTime)cdtArray[0]).ToString(), 0);
                    rTBArray.Add(cTB);
                }
                if ((DateTime)cdtArray[0] < dStartTime)
                {
                    for (j = 0; j < cdtArray.Count; j++)
                    {
                        if ((DateTime)cdtArray[j] < dStartTime)
                            cdtArray[j] = dStartTime;
                    }
                }
            }

            //Trim the end if necessary
            if (dEndTime.Ticks > 0)
            {
                for (j = 0; j < cdtArray.Count; j++)
                {
                    if ((DateTime)cdtArray[j] > dEndTime)
                        cdtArray[j] = dEndTime;
                }
            }

            //build the schedule in rTBArray
            var dTemp = new DateTime();
            for (j = 0; j < cdtArray.Count - 1; j++) //TODO: why minus 1?
            {
                if ((DateTime)cdtArray[j] == dTemp)
                    continue;

                var dStart = (DateTime)cdtArray[j];
                dTemp = dStart;
                int k;
                for (k = j + 1; k < cdtArray.Count; k++)
                {
                    var dEnd = new DateTime();
                    if ((DateTime)cdtArray[k] != dStart)
                    {
                        dEnd = (DateTime)cdtArray[k];
                    }
                    if (dEnd.Ticks > 0)
                    {
                        cTB = new SchdAvailability();
                        cTB.Create(dStart.ToString(), dEnd.ToString(), 0);
                        rTBArray.Add(cTB);
                        break;
                    }
                }
            }

        }

        public void CancelAccessBlocks(string sResourceID, string sResourceName, string sStart, string sEnd)
        {
            var parms1 = new List<string> { sResourceID, Convert.ToDateTime(sStart).ToString("M-d-yyyy"), Convert.ToDateTime(sEnd).ToString("M-d-yyyy") };
            var dtAppt = _adt.callOVIDRPC("SDEC AVDELDT", parms1);

            GetAvailabilities(sResourceName, sStart, sEnd);
        }

        public string GetReportText(string reportRPC, List<List<string>> parms)
        {
            if (string.IsNullOrEmpty(reportRPC))
                return "Report not defined";

            DataTable table;
            if (parms.Count == 1)
            {
                table = _adt.callOVIDRPC(reportRPC, parms[0]);
            }
            else
            {
                var ls = new List<string>();
                foreach (var p in parms)
                {
                    var s = "";
                    foreach (var i in p)
                        s += (string.IsNullOrEmpty(s) ? "" : "^") + i;
                    ls.Add(s);
                }
                table = _adt.callOVIDRPC(reportRPC, ls);
            }

            var reportText = string.Empty;
            if (table.Columns.Count > 0)
            {
                foreach (DataRow row in table.Rows)
                {
                    reportText += row[0] + Environment.NewLine;
                }
            }

            if (reportText == string.Empty)
                return "Empty report returned.";

            return reportText;
        }

        public IList<ServerPrinter> GetDynamicServerPrinters(string searchString, string lastSub = "")
        {
            var parms = new List<string> { "A", "25", lastSub, searchString };
            var table = _adt.callOVIDRPC("SDECDEV DEV", parms);

            var tempserverPrinters = new List<ServerPrinter>();
            foreach (DataRow row in table.Rows)
            {
                var serverPrinter = new ServerPrinter
                {

                    Ien = GetCellString(row, "DIEN"),
                    Name = GetCellString(row, "DNAME"),
                    Location = GetCellString(row, "LOCT"),
                    LastSub = GetCellString(row, "LASTSUB"),
                };
                tempserverPrinters.Add(serverPrinter);
            }
            return tempserverPrinters;
        }


        private IList<NameValue> _serverPrinters;
        public IList<NameValue> GetServerPrinters()
        {
            if (_serverPrinters != null && _serverPrinters.Count != 0)
                return _serverPrinters;

            var parms = new List<string>();
            var table = _adt.callOVIDRPC("SDECDEV DEVICE", parms);

            var tempserverPrinters = new List<NameValue>();
            foreach (DataRow row in table.Rows)
            {
                var nameValue = new NameValue
                {
                    Name = GetCellString(row, "PRINTER_NAME"),
                    Value = GetCellString(row, "PRINTER_IEN")
                };
                tempserverPrinters.Add(nameValue);
            }
            _serverPrinters = tempserverPrinters;
            return _serverPrinters;
        }

        public void ServerPrinterPrint(string appointmenrID, string letterType, string deviceIEN)
        {
            try
            {
                List<string> parms = new List<string>
                {
                    appointmenrID,
                    letterType,
                    deviceIEN
                };
                var table = _adt.callOVIDRPC("SDECDEV PRINT", parms);

                if (table.Rows.Count > 0)
                {
                    var row = table.Rows[0];

                    if (row["CODE"].ToString() == "-1")
                        Keys.WarnUser(null, row["MESSAGE"].ToString(), "Server Printer Print Warning");
                }
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "ServerPrinterPrint", ex);
                Keys.AlertUserError(null, ex, "Server Printer Print Error");
            }
        }

        public string CurrentUser
        {
            get
            {
                if (string.IsNullOrEmpty(_currentUser))
                {
                    _currentUser = _adt.UserID;
                    Keys.LoggedInUserIen = _currentUser;
                }
                return _currentUser;
            }
        }

        public void ChangeUserServer()
        {
            _currentUser = string.Empty;
            _adt.UserInfoLogout();
            LoadGlobalRecordsets();
        }

        public bool LoginUser(string accessCode, string verifyCode, out string szErrorMsg)
        {
            return _adt.LoginUser(accessCode, verifyCode, out szErrorMsg);
        }

        public bool IsPossibleDuplicatePatient(string patientIen)
        {
            var parms = new List<string> { patientIen };
            var table = _adt.callOVIDRPC("BEHOPTCX CHKDUP", parms);
            return true;
        }

        public List<string> GetPatientPreferences(string dfn)
        {
            var dtPreferences = GetPatientSpecialNeedsPreferences(dfn);

            var prefsList = new List<string>();
            foreach (var row in dtPreferences.Rows)
            {
                prefsList.Add(row.ToString());
            }
            return prefsList;
        }

        private void ClearPatientFutureAppointmentsCache(string patientIen, string appointmentId = "")
        {
            // remove stale data first
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.FutureApptsTableCache);

            // Some places from which this celar method is called do not have the Patient ID, but has an appointment ID, so we find the patient
            if (string.IsNullOrEmpty(patientIen) && !string.IsNullOrEmpty(appointmentId))
            {
                foreach (KeyValuePair<string, DataTableCache> kvp in Keys.Tables.FutureApptsTableCache)
                {
                    foreach (DataRow row in kvp.Value.Table.Rows)
                        if (row["IEN"].ToString() == appointmentId)
                        {
                            patientIen = row["DFN"].ToString();
                            break;
                        }
                }
            }

            if (string.IsNullOrEmpty(patientIen))
                return;

            List<string> keys = Keys.Tables.FutureApptsTableCache.Where(kvp => (kvp.Key.StartsWith(patientIen))).Select(x => x.Key).ToList();

            foreach (string key in keys)
                Keys.Tables.FutureApptsTableCache.Remove(key);
        }

        private IList<FuturePatientAppointment> ReadFutureAppointments(string patientIen, DateTime start, DateTime end)
        {
            IList<FuturePatientAppointment> futurePatientAppointments = new List<FuturePatientAppointment>();
            string startText = start.ToShortDateString();
            string endText = end.ToShortDateString();
            var cacheKey = $"{patientIen}^{startText}^{endText}";
            var parms = new List<string> { patientIen, startText, endText, "0" };

            try
            {
                // Remove any stale records from the Keys.Tables.FutureApptsTableCache
                Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.FutureApptsTableCache);

                DataTable table;

                if (Keys.Tables.FutureApptsTableCache.ContainsKey(cacheKey))
                    table = Keys.Tables.FutureApptsTableCache[cacheKey].Table;
                else
                {
                    table = _adt.callOVIDRPC(Keys.RpcCalls.GetFutureAppointments, parms);
                    Keys.Tables.FutureApptsTableCache.Add(cacheKey, new DataTableCache(table, 5));
                }

                foreach (DataRow row in table.Rows)
                {
                    var futureAppointment = new FuturePatientAppointment
                    {
                        Dfn = row["DFN"].ToString(),
                        Clinic = row["CLINIC_NAME"].ToString(),
                        ClinicIen = row["CLINIC_IEN"].ToString(),
                        AppointmentID = row["IEN"].ToString(),
                        AppointmentDate = row["APPT_DATE"].ToString(),
                        Status = row["STATUS"].ToString(),
                        AncillaryText = row["ANCTXT"].ToString(),
                        ConsultID = row["CONS"].ToString()
                    };

                    futureAppointment.AppointmentDateTime = Keys.Methods.ConvertFileManExternalTimeFormatToDateTime(futureAppointment.AppointmentDate);
                    futureAppointment.IsCancelled = futureAppointment.Status == "CANCELLED BY CLINIC" || futureAppointment.Status == "CANCELLED BY PATIENT";
                    futurePatientAppointments.Add(futureAppointment);
                }
            }
            catch (Exception ex)
            {
                Keys.AlertUserError(null, ex, "Future Appointments Error");
            }

            return futurePatientAppointments;
        }

        public ObservableCollection<FuturePatientAppointment> GetFutureAppointments(string patientIen)
        {
            return GetFutureAppointments(patientIen, DateTime.Now);
        }

        public ObservableCollection<FuturePatientAppointment> GetFutureAppointments(string patientIen, DateTime start)
        {
            IList<FuturePatientAppointment> list = ReadFutureAppointments(patientIen, start, start.AddDays(365 * 2));
            ObservableCollection<FuturePatientAppointment> collection = new ObservableCollection<FuturePatientAppointment>();

            foreach (FuturePatientAppointment item in list)
                collection.Add(item);

            return collection;
        }

        public Patient GetPatientServiceInfo(string patientIen)
        {
            //DataTable table = Keys.AppointmentRequestWaitListTables.EwlWaitList; // _mDsGlobal.Tables[Keys.RpcCalls.WaitListTable];

            foreach (var req in Keys.AppointmentRequestWaitListTables.AllAppointmentRequests)
            {
                if (req.Patient.IEN == patientIen)
                {
                    var patsy = new Patient(this)
                    {
                        ServiceConnection = req.Patient.ServiceConnection,
                        ServiceConnectionPercentage = req.Patient.ServiceConnectionPercentage
                    };
                    return patsy;
                }
            }
            return null;
        }

        public List<PtCSchAppointmentType> GetPtCSchAppointmentTypes()
        {
            if (Keys.Tables.PtCSchApptTypesData == null || Keys.Tables.PtCSchApptTypesDataLoaded.AddHours(1) > DateTime.Now)
            {
                var parms = new List<string>();
                Keys.Tables.PtCSchApptTypesData = _adt.callOVIDRPC(Keys.RpcCalls.PtCSchApptTypes, parms);
            }

            var prefsList = new List<PtCSchAppointmentType>();
            foreach (DataRow row in Keys.Tables.PtCSchApptTypesData.Rows)
            {
                var ptCSchApptType = new PtCSchAppointmentType
                {
                    Name = row["RRAPPTYPN"].ToString(),
                    Ien = row["RRAPPTYP"].ToString()
                };
                prefsList.Add(ptCSchApptType);
            }
            return prefsList;
        }

        public IList<Provider> GetPtCSchProviders(string searchString)
        {
            var parms = new List<string> { "0" };

            if (!string.IsNullOrEmpty(searchString))
            {
                parms.Add(searchString);
                parms.Add("25");
            }

            var dtPreferences = _adt.callOVIDRPC(Keys.RpcCalls.PtCSchProviders, parms);
            var ptCSchProviderList = new List<Provider>();
            foreach (DataRow row in dtPreferences.Rows)
            {
                var provider = new Provider
                {
                    Name = GetCellString(row, "PROVNAME"),
                    IEN = GetCellString(row, "RRPROVIEN")
                };
                ptCSchProviderList.Add(provider);
            }
            return ptCSchProviderList;
        }

        private DataTable GetAuditData(DateTime? startDate, DateTime? endDate, string scheduler = "", string lastSub = "")
        {
            var parms = new List<string>();
            var dtPreferences = new DataTable();
            //parms.Add(Keys.RpcCalls.AuditReportRpc);
            //parms.Add("1");
            parms.Add(startDate == null ? null : startDate.Value.ToShortDateString());
            parms.Add(endDate == null ? null : endDate.Value.ToShortDateString());

            if (!string.IsNullOrEmpty(scheduler))
                parms.Add(scheduler);
            if (!string.IsNullOrEmpty(lastSub))
                parms.Add(lastSub);
            try
            {
                dtPreferences = _adt.callOVIDRPC(Keys.RpcCalls.AuditReportRpc, parms);
                //var result = _adt.CallVistaRpc(Keys.DeferredRpcCalls.DeferredRpc, parms);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Audit Data Error");
            }
            return dtPreferences;
        }

        public DataTable GetClinicData(int pageSize, ref string lastSub, ref string searchStr)
        {
            var parms = new List<string>();
            var dtPreferences = new DataTable();
            parms.Add(pageSize.ToString(CultureInfo.InvariantCulture));
            parms.Add(lastSub);
            parms.Add(searchStr);
            //parms.Add(string.IsNullOrEmpty(scheduler) ? null : scheduler);
            try
            {
                dtPreferences = _adt.callOVIDRPC(Keys.RpcCalls.ClinicReportRpc, parms);
                if (dtPreferences != null)
                    foreach (DataRow dr in dtPreferences.Rows)
                        lastSub = dr["LastSub"].ToString();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Clinic Data Error");
            }
            return dtPreferences;
        }

        private DataTable GetCancelAuditData(DateTime? startDate, DateTime? endDate, string scheduler = "")
        {
            var parms = new List<string>
            {
                startDate == null ? null : startDate.Value.ToShortDateString(),
                endDate == null ? null : endDate.Value.ToShortDateString()
            };

            if (!string.IsNullOrEmpty(scheduler))
                parms.Add(scheduler);

            var dtPreferences = new DataTable();
            try
            {
                dtPreferences = _adt.callOVIDRPC(Keys.RpcCalls.AuditReportCancelRpc, parms);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Cancel Audit Data Error");
            }
            return dtPreferences;
        }

        public bool? GetClinicStopInfo(string patientIen, string hospitalLocationId)
        {
            var parms = new List<string> { patientIen, hospitalLocationId };
            //parms.Add(string.IsNullOrEmpty(scheduler) ? null : scheduler);
            try
            {
                var dtClinicStopInfo = _adt.callOVIDRPC(Keys.RpcCalls.ClinicStop, parms);
                foreach (DataRow row in dtClinicStopInfo.Rows)
                {
                    var returnCode = row["RETURNCODE"].ToString();
                    var text = row["TEXT"].ToString();
                    if (returnCode == "0" && text == "NO")
                        return true;
                    return false;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Clinic Stop Error");
            }

            return null;
        }

        public bool? GetServiceStopInfo(string patientIen, string serviceInfo)
        {
            var parms = new List<string> { patientIen, serviceInfo };  // Service Name if EWL  Service ID if not

            try
            {
                var dtServiceStopInfo = _adt.callOVIDRPC(Keys.RpcCalls.ServiceStop, parms);
                foreach (DataRow row in dtServiceStopInfo.Rows)
                {
                    var returnCode = row["RETURNCODE"].ToString();
                    var text = row["TEXT"].ToString();
                    if (returnCode == "0" && text == "NO")
                        return true;
                    return false;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Service Stop Error");
            }

            return null;
        }

        public List<Ethnicity> GetEthnicities()
        {
            var ethcnicities = new List<Ethnicity>();

            try
            {
                // Remove any stale records from the Keys.Tables.LookupTablesCache
                Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);
                DataTable dtClinicStopInfo;

                if (Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.GetEthnicities))
                    dtClinicStopInfo = Keys.Tables.LookupTablesCache[Keys.RpcCalls.GetEthnicities].Table;
                else
                {
                    var parms = new List<string>();
                    dtClinicStopInfo = _adt.callOVIDRPC(Keys.RpcCalls.GetEthnicities, parms);
                    Keys.Tables.LookupTablesCache.Add(Keys.RpcCalls.GetEthnicities, new DataTableCache(dtClinicStopInfo, 60));
                }

                foreach (DataRow row in dtClinicStopInfo.Rows)
                {
                    var ethncicity = new Ethnicity
                    {
                        Name = row["ETHNICITYNAME"].ToString(),
                        Ien = row["ETHNICITY"].ToString(),
                        Abbreviation = row["ETHNICITYABBR"].ToString()
                    };
                    ethcnicities.Add(ethncicity);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ethnicities List Error");
            }
            return ethcnicities;
        }

        public bool IsDeceased(string patientIen)
        {
            var parms = new List<string> { patientIen };
            var result = _adt.CallVistaRpcString(Keys.PatientPromptRpcs.DiedOn, parms); // adt.CallVistaRpc(Keys.PatientPromptRpcs.DiedOn, parms, true);

            return !string.IsNullOrEmpty(result) && result != "0";
        }

        public string IsOwnRecord(string patientIen)
        {
            NameValue result = GetPatientSensistiveInfo(patientIen);
            return result.Name == "3" ? result.Value : string.Empty;
        }

        public string GetPatientSelectionPrompts(string patientIEN, ref Patient patient, bool checkForSimilarPatients = true, bool checkForSensitiveRecord = true)
        {
            if (patientIEN == Keys.PatientIen_SensitiveRecordCheck)
                return string.Empty;

            Keys.PatientIen_SensitiveRecordCheck = string.Empty;
            var parms = new List<string> { patientIEN };
            var prompts = string.Empty;

            try
            {
                string[][] rpcReturn;

                if (checkForSimilarPatients)
                {
                    rpcReturn = _adt.CallVistaRpc(Keys.PatientPromptRpcs.SimilarRecord, parms);
                    if (rpcReturn.Length > 0 && !ProcessSimilarPatientPrompt(rpcReturn, ref patient))
                        return "0";
                }

                parms[0] = patient.IEN;

                if (checkForSensitiveRecord)
                {
                    NameValue sensitiveInfo = GetPatientSensistiveInfo(patient.IEN);

                    switch (sensitiveInfo.Name)
                    {
                        case "-1":
                            // Error or problem
                            Keys.AlertUserError(null, new Exception("Unable to review Patient Sensitive information.\nRequired variable not defined."), patient.Name + " - Sensitive Information Error");
                            return "0";
                        case "0":
                            //  Need to do nothing
                            break;
                        case "1":
                            // Display any returned message
                            DisplayPatientSensitiveInformation(patient, sensitiveInfo, patient.Name + " - Sensitive Record Information");
                            break;
                        case "2":
                            // Display any returned message and ask the user about continuing
                            if (!string.IsNullOrEmpty(sensitiveInfo.Value))
                            {
                                if (DisplayPatientSensitiveInformation(patient, sensitiveInfo, patient.Name + " - Sensitive Record Warning"))
                                {
                                    // Yes, Call the sensitive record bullitin RPC
                                    List<string> p = new List<string>() { patient.IEN }; //string.IsNullOrEmpty(patient.DFN) ? patient.IEN : patient.DFN, string.Empty, "3" };

                                    if (_adt.CallVistaRpcString(Keys.PatientPromptRpcs.SensitiveBulletin, p) != "1")
                                    {
                                        DisplayPatientSensitiveInformation(patient, new NameValue("4", "Unable to create access notification"), patient.Name + " - Access Denied");
                                        return "0";
                                    }
                                }
                                else
                                    // No, return access denied
                                    return "0";
                            }

                            break;
                        case "3":
                        case "4":
                            // Display any message and return access denied
                            if (!string.IsNullOrEmpty(sensitiveInfo.Value))
                                DisplayPatientSensitiveInformation(patient, sensitiveInfo, patient.Name + " - Access Denied");

                            return "0";
                    }

                    rpcReturn = _adt.CallVistaRpc(Keys.PatientPromptRpcs.HasFlag, parms);

                    if (!ProcessFlags(rpcReturn, patient))
                        return "0";
                }

                var rpcStringReturn = _adt.CallVistaRpcString(Keys.PatientPromptRpcs.DiedOn, parms);
                if (!string.IsNullOrEmpty(rpcStringReturn) && rpcStringReturn != "0")
                {
                    patient.IsDead = true;
                    if (!ProcessPromptsDiedOn(rpcStringReturn))
                    {
                        Mouse.OverrideCursor = Cursors.Arrow;
                        return "0";
                    }
                }
            }
            catch (Exception ex)
            {
                prompts = ex.Message;
            }

            if (string.IsNullOrEmpty(prompts))
                Keys.PatientIen_SensitiveRecordCheck = patientIEN;

            return prompts;
        }

        private bool DisplayPatientSensitiveInformation(Patient patient, NameValue sensitiveInfo, string title)
        {
            PatientPrompt patientAlerts = PatientPrompt.ProcessSensitiveInformation(patient, sensitiveInfo);

            if (patientAlerts == null)
                return true;

            Cursor oldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Arrow;

            try
            {
                System.Windows.Media.Brush b = new System.Windows.Media.SolidColorBrush(System.Windows.Media.Color.FromRgb(212, 212, 212));
                string[] lines = sensitiveInfo.Value.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
                var alertHeight = 100 + lines.Length * 14;

                CLaunchWindow.LaunchWindow(patientAlerts, title, alertHeight, 565, "PatientSensitiveInfoPrompts", false, true, false, true, b);
            }
            finally
            {
                Mouse.OverrideCursor = oldCursor;
            }

            switch (sensitiveInfo.Name)
            {
                case "-1":
                    // Error or problem - Should NEVER get here
                    return false;
                case "0":
                case "1":
                    return true;
                case "2":
                    return patientAlerts.BDialogResult;
                case "3":
                case "4":
                    return false;
            }

            return patientAlerts.BDialogResult;
        }

        private bool ProcessFlags(string[][] rpcRslt, Patient patient)
        {
            // Make sure this patient record is complete
            if ((patient != null) && (!patient.GetRegARead && !patient.PTLookRSRead))
                GetPatient(ref patient);

            if (rpcRslt.Length == 0 && patient.PatientFlags.Count == 0)
                return true;

            if (rpcRslt.Length > 0)
            {
                // Retrieve flag details
                var parms = new List<string> { patient.IEN, null };
                var flags = new List<string>();

                foreach (var row in rpcRslt)
                    if (row != null)
                        foreach (var pc in row)
                            flags.Add(pc);

                foreach (var flag in flags)
                {
                    // Split the HASFLG results
                    var flagParts = flag.Split('^');
                    // Retrieve the Flag information
                    parms[1] = flagParts[0];
                    var flagRslt = _adt.CallVistaRpc(Keys.PatientPromptRpcs.GetFlag, parms);
                    // Convert the results to a List<string>
                    var flagDetails = new List<string>();
                    // Assume Assignment Status is Active
                    var detailAssignmentStatus = true;

                    foreach (var row in flagRslt)
                        if (row != null)
                            for (var idx = 0; idx < row.Length; idx++)
                            {
                                var pc = row[idx];

                                // See if the flag details contain an Assignment Status
                                if (pc.Contains("Assignment Status:"))
                                    detailAssignmentStatus = pc.Contains("Active");

                                flagDetails.Add(pc);
                            }

                    // See if the patient already has a flag with the same name
                    var patientFlag = patient.PatientFlags.FirstOrDefault(f => f.FlagName == flagParts[1]);

                    if (patientFlag == null)
                    {
                        patientFlag = new PatientFlag
                        {
                            FlagName = flagParts[1],
                            // Assume Flag Status is Active
                            FlagStatus = true,
                            AssignmentStatus = detailAssignmentStatus
                        };

                        patientFlag.SetFlagType(flagParts[2]);
                        patient.PatientFlags.Add(patientFlag);
                    }

                    // Add the details to the patient flag
                    patientFlag.FlagDetails.Clear();
                    ((List<string>)patientFlag.FlagDetails).AddRange(flagDetails);
                }
            }

            var patientAlerts = PatientPrompt.ProcessFlags(patient);

            if (patientAlerts == null)
                return true;

            Cursor oldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Arrow;

            try
            {
                System.Windows.Media.Brush b = new System.Windows.Media.SolidColorBrush(System.Windows.Media.Color.FromRgb(212, 212, 212));
                var alertHeight = 500;

                if (patient.PatientFlags.Count == 0)
                {
                    alertHeight = 50;
                    alertHeight += (patient.FugitiveFelonFlag ? 50 : 0);
                    alertHeight += (patient.CatastrophicDisabilityFlag ? 50 : 0);
                    alertHeight += (patient.Category8Flag ? 50 : 0);
                }

                CLaunchWindow.LaunchWindow(patientAlerts, "Patient Record (" + patient.PatientFlags.Count + ") Flags", alertHeight, 500, "PatientPrompts", false, true, false, true, b);
            }
            finally
            {
                Mouse.OverrideCursor = oldCursor;
            }

            return patientAlerts.BDialogResult;
        }

        private bool ProcessSimilarPatientPrompt(string[][] rpcRslt, ref Patient patient)
        {
            var prompt = "";
            foreach (var row in rpcRslt)
                if (row != null)
                    foreach (var pc in row)
                        prompt += pc + Environment.NewLine;

            Mouse.OverrideCursor = Cursors.Arrow;
            if (prompt == "0" + Environment.NewLine)
                return true;

            var similarPatients = new List<SimilarPatient>();
            var lines = prompt.Split(new[] { Environment.NewLine }, StringSplitOptions.None);
            if (lines.Any())
            {
                foreach (var line in lines)
                {
                    var pcs = line.Split('^');
                    if (!pcs.Any() || pcs.Length < 2)
                        continue;

                    foreach (var pc in pcs)
                    {
                        switch (pc)
                        {
                            case "0":
                                break;
                            case "1":
                                var dob = "";
                                var ssn = "";
                                var sensitive = IsSensitive(pcs[1]);
                                GetPatientInfoForPrompt(pcs[1], ref dob, ref ssn);

                                var similarPatient = new SimilarPatient
                                {
                                    Ien = pcs[1],
                                    IsSensitive = sensitive,
                                    Dob = dob,
                                    Ssn = ssn,
                                    Name = pcs[2]
                                };

                                similarPatients.Add(similarPatient);
                                break;
                        }
                    }
                }
            }
            var similarPatientsPrompt = new SimilarPatientsPrompt(this) { SimilarPatientList = similarPatients, SelectedPatient = patient };
            CLaunchWindow.LaunchWindow(similarPatientsPrompt, "Similar Patients", 250, 360, "SimilarPatientPrompts", true);
            if (similarPatientsPrompt.SelectedPatient != null)
                patient = similarPatientsPrompt.SelectedPatient;

            return similarPatientsPrompt.BDialogResult;
        }

        public bool IsSensitive(string patientIen)
        {
            var parms = new List<string> { patientIen };
            var rpcRslt = GetPatientSensistiveInfo(patientIen);
            return (rpcRslt.Name != "0");
        }

        private bool ProcessPromptsDiedOn(string rpcRslt)
        {
            if (rpcRslt == "0" + Environment.NewLine || string.IsNullOrEmpty(rpcRslt))
                return true;

            var prompt = "This patient died ";
            prompt += ShortDateTimeConverter.ConvertMumpsDate(rpcRslt);
            prompt += Environment.NewLine + "Do you wish to continue?";
            var patientAlerts = new PatientPrompt { PatientAlertInfo = prompt };
            Cursor oldCursor = Mouse.OverrideCursor;
            Mouse.OverrideCursor = Cursors.Arrow;

            try
            {
                CLaunchWindow.LaunchWindow(patientAlerts, "Deceased Patient", 125, 300, "PatientPrompts", true);
            }
            finally
            {
                Mouse.OverrideCursor = oldCursor;
            }

            return patientAlerts.BDialogResult;
        }

        public List<Clinic> GetAssociatedClinicsForServiceSpeciatly(string svcSpecialtyName)
        {
            var parms = new List<string> { svcSpecialtyName };
            var associatedClinics = new List<Clinic>();
            //parms.Add(string.IsNullOrEmpty(scheduler) ? null : scheduler);
            try
            {
                var dtClinicStopInfo = _adt.callOVIDRPC(Keys.RpcCalls.AssociatedClinicsForSvcSpecialty, parms);
                foreach (DataRow row in dtClinicStopInfo.Rows)
                {
                    var clinique = GetClinicFromDataRow(row);
                    var resID = GetResourceIdByClinicIdAndName(clinique.ClinicIen, clinique.ClinicName, false);

                    if (!string.IsNullOrEmpty(resID))
                        associatedClinics.Add(clinique);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Associated Clinics Error");
            }
            return associatedClinics;
        }

        public Ethnicity GetEthnicity(string patientIen)
        {
            var table = GetPatientRecord(patientIen);

            if (table.Rows.Count <= 0)
                return null;

            var row = table.Rows[0];
            var patientEthnicity = new Ethnicity
            {
                Name = row["PETHN"].ToString(),
                Ien = row["PETH"].ToString()
            };
            return patientEthnicity;
        }

        public Race GetRace(string patientIen)
        {
            var table = GetPatientRecord(patientIen);

            if (table.Rows.Count <= 0)
                return null;

            var row = table.Rows[0];
            var patientRace = new Race
            {
                Name = row["PRACEN"].ToString(),
                Ien = row["PRACE"].ToString()
            };
            return patientRace;
        }


        public bool SetPatientDemographics(Patient patient)
        {
            /*var paramList = new List<string>();
            var dataTable = adt.callOVIDRPC(Keys.RpcCalls.CollectionMethod, paramList);
            foreach(DataRow dr in dataTable.Rows)
            {
                var collectionMethod = dr["ROW"].ToString();
            }*/
            var raceIENs = new List<string>();
            var defaultRace = patient.RaceList.FirstOrDefault(r => r.IsSelected && r.IsDefault);

            if (defaultRace == null)
                defaultRace = patient.RaceList.FirstOrDefault(r => r.IsSelected);

            if (defaultRace != null)
                raceIENs.Add(defaultRace.Ien + ";;1");

            foreach (var r in patient.RaceList)
            {
                var addIEN = true;

                if (defaultRace != null)
                    addIEN = r.Ien != defaultRace.Ien;

                if (addIEN)
                {
                    if (r.IsSelected)
                        raceIENs.Add(r.Ien + ";;1");
                    else if (r.LastIsSelected != r.IsSelected)
                        raceIENs.Add("@" + r.Ien);
                }
            }

            var ethnicity = patient.Ethnicity.Ien + ";;1";  //Method of Collection = "1" - SELF INDICATION
            var maritalStatus = string.IsNullOrEmpty(patient.MaritalStatus) ?
                null :
                GetMaritalStatusList().FirstOrDefault(ms => ms.name == patient.MaritalStatus || ms.ien == patient.MaritalStatus || ms.code == patient.MaritalStatus || ms.abbreviation == patient.MaritalStatus);
            var religion = string.IsNullOrEmpty(patient.ReligiousPreference) ?
                null :
                GetReligionList().FirstOrDefault(r => r.Name == patient.ReligiousPreference || r.Ien == patient.ReligiousPreference || r.Code == patient.ReligiousPreference || r.Abbreviation == patient.ReligiousPreference);

            var parms = new List<string>
            {
                // INP(1) = DFN - (required) pointer to PATIENT file 
                patient.IEN,

                // INP(2) = RACE - pointer to the RACE file 10 separated by |
                //             Each pipe piece contains the following pieces
                //    (Race1;;Method1 | Race2;;Method2):
                //         RACE INFORMATION -pointer to the RACE file 10  Use @ to delete 
                //         METHOD OF COLLECTION - pointer to the RACE AND ETHNICITY COLLECTION METHOD
                //                file 10.3 "2;;1|@12|13;;1" will add/ update 2 & 13 and delete 12
                string.Join("|", raceIENs.ToArray()),

                // INP(3) = ETHNICITY  Note that the ETHNICITY INFORMATION multiple only allows 1 entry.
                //              input contains the following ;; pieces:
                //     1.ETHNICITY INFORMATION - pointer to the ETHNICITY file 10.2 @ will delete
                //     2.METHOD OF COLLECTION -pointer to the ETHNICITY COLLECTION METHOD file 10.3
                //    @4;;1 or just @ will delete; 5;;1 will add 5;
                ethnicity,

                // INP(4) = ADDRESS1 - Street Address (Line 1) Free text  3-30 characters
                //                     @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("Street1"),

                // INP(5) = ADDRESS2 - Street Address (Line 2) Free text  3-30 characters
                //                     @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("Street2"),

                // INP(6) = ADDRESS3 - Street Address (Line 3) Free text  3-30 characters
                //                     @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("Street3"),

                // INP(7) = ZIP CODE - Zip+4 Free text of 5 or 9 digits This populates fields .116 and .1112
                patient.PrimaryContactInfo.GetRPCValue("ZipPlus4").Replace("-", ""),

                // INP(8) = CITY - Free text 2-15 characters 
                patient.PrimaryContactInfo.GetRPCValue("City"),

                // INP(9) = STATE - pointer to STATE file 5 
                patient.PrimaryContactInfo.GetRPCValue("State.Ien"),
                //patient.PrimaryContactInfo.State == null ? string.Empty : patient.PrimaryContactInfo.State.Ien,

                // INP(10) = COUNTRY - pointer to COUNTRY CODE file 779.004 
                patient.PrimaryContactInfo.GetRPCValue("Country.Ien"),
                //patient.PrimaryContactInfo.Country == null ? string.Empty : patient.PrimaryContactInfo.Country.Ien,

                // INP(11) = BAD ADDRESS INDICATOR - Valid Values:
                //              UNDELIVERABLE
                //              HOMELESS
                //              OTHER
                //              ADDRESS NOT FOUND
                //      @ will delete the existing entry
                patient.IsBadAddress ?  patient.BadAddressType : "@",

                // INP(12) = PHONE (RESIDENCE) - free text 4-20 characters  @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("PhoneResidence"),

                // INP(13) = PHONE (WORK) - free text 4-20 characters  @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("PhoneWork"),

                // INP(14) = COUNTY name - name of a county from the COUNTY multiple in the STATE file (#5) 
                patient.PrimaryContactInfo.GetRPCValue("County"),

                // INP(15) = PHONE NUMBER [CELLULAR] 4-20 characters  @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("PhoneCell"),

                // INP(16) = Patient Email Address - 3-50 characters containing 1 @     @ will delete the existing entry
                patient.PrimaryContactInfo.GetRPCValue("EmailAddress"),

                // INP(17) = Marital Status - pointer to MARITAL STATUS file (#11) 
                maritalStatus == null ? string.Empty : maritalStatus.ien,

                // INP(18) = Religious Preference - pointer to RELIGION file (#13) 
                religion == null ? string.Empty : religion.Ien,

                // INP(19) = Patient Temporary Address Active? (.12105) Y:YES N:NO 
                //           Temporary Address data can only be edited if INP(19) is
                //           'Y' or the 'TEMPORARY ADDRESS ACTIVE?' field is already
                //           defined as 'Y'.
                patient.TemporaryContactInfo.IsActive ? "Y" : "N"
            };

            // INP( 20 ) = Patient Temporary Address Line 1( .1211 )  @ will delete the existing entry
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("Street1"));

            // INP(21) = Patient Temporary Address Line 2 (.1212)   @ will delete the existing entry
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("Street2"));

            // INP(22) = Patient Temporary Address Line 3 (.1213)  @ will delete the existing entry
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("Street3"));

            // INP(23) = Patient Temporary City (.1214) 
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("City"));

            // INP(24) = Patient Temporary State (.1215) 
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("State.Ien"));
            //if (patient.TemporaryContactInfo.State == null)
            //    parms.Add(null);
            //else
            //    parms.Add(patient.TemporaryContactInfo.State.Ien);

            // INP(25) = Patient Temporary Zip (.1216) [[***Dont use  INP( 26 ) populates both***]] 
            parms.Add(string.Empty);

            // INP(26) = Patient Temporary Zip+4 (.12112)  Populates fields .1216 and .12112
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("ZipPlus4").Replace("-", ""));

            // INP(27) = Patient Temporary Country (.1223) 
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("Country.Ien"));
            //if (patient.TemporaryContactInfo.Country == null)
            //    parms.Add(null);
            //else
            //    parms.Add(patient.TemporaryContactInfo.Country.Ien);

            // INP(28) = Patient Temporary County (.12111) 
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("County"));

            // INP(29) = Patient Temporary Phone (.1219)  ( also referred to as Phone #2)
            //                @ will delete the existing entry
            parms.Add(patient.TemporaryContactInfo.GetRPCValue("Phone"));

            // INP(30) = Patient Temporary Address Start Date (.1217) in external format (no time)
            //                @ will delete the existing entry
            parms.Add(patient.TemporaryContactInfo.StartRPCValue);
            // INP(31) = Patient Temporary Address End Date (.1218) in external format (no time)
            //                @ will delete the existing entry
            parms.Add(patient.TemporaryContactInfo.EndRPCValue);
            // INP(32) = PAGER NUMBER 4-20 numeric characters.
            parms.Add(patient.PrimaryContactInfo.GetRPCValue("PhonePager"));

            var table = _adt.callOVIDRPC(Keys.RpcCalls.PatientSet, parms);
            if (table.Rows.Count <= 0)
                return true;

            var row = table.Rows[0];
            var returnCode = row["RETURNCODE"].ToString();
            if (returnCode == "0")
            {
                // Insure the patient record is reread after a successful save
                Keys.Tables.PatientTableCache.Remove(patient.IEN);
                return true;
            }
            MessageBox.Show(row["TEXT"].ToString(), "Save Patient Error");
            return false;
        }

        private State GetState(string s)
        {
            if (string.IsNullOrEmpty(s))
                return null;

            State state = null;
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);

            if (!Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.GetStates))
            {
                GetStates();
            }
            var table = Keys.Tables.LookupTablesCache[Keys.RpcCalls.GetStates].Table;
            var results = table.Select(string.Format("NAME='{0}' OR ABBR='{0}'", s));

            if (results.Length > 0)
            {
                var dr = results[0];
                state = new State
                {
                    Ien = dr["IEN"].ToString(),
                    Name = dr["NAME"].ToString(),
                    Abbreviation = dr["ABBR"].ToString(),
                    StateCode = dr["STCODE"].ToString()
                };
            }
            return state;
        }

        public IList<State> GetStates()
        {
            IList<State> states = new List<State>();

            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);

            if (Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.GetStates))
            {
                var table = Keys.Tables.LookupTablesCache[Keys.RpcCalls.GetStates].Table;
                foreach (DataRow dr in table.Rows)
                {
                    var state = new State
                    {
                        Ien = dr["IEN"].ToString(),
                        Name = dr["NAME"].ToString(),
                        Abbreviation = dr["ABBR"].ToString(),
                        StateCode = dr["STCODE"].ToString()
                    };
                    states.Add(state);
                }
            }
            else
            {
                var table = new DataTable();
                table.Columns.Add("IEN", typeof(string));
                table.Columns.Add("NAME", typeof(string));
                table.Columns.Add("ABBR", typeof(string));
                table.Columns.Add("STCODE", typeof(string));

                var rpcResult = _adt.CallVistaRpc(Keys.RpcCalls.GetStates, null);
                if (rpcResult != null && rpcResult.Length > 0)
                {
                    var data = rpcResult[0];
                    //first row is a count
                    var bdr = table.NewRow();  // Need a blank row to reset the state if the Zip is incorrect.

                    bdr["IEN"] = "";
                    bdr["NAME"] = "";
                    bdr["ABBR"] = "";
                    bdr["STCODE"] = "";
                    table.Rows.Add(bdr);
                    for (var i = 1; i < data.Length; i++)
                    {
                        var pieces = data[i].Split('^');
                        var state = new State
                        {
                            Ien = pieces[0],
                            Name = pieces[1],
                            Abbreviation = pieces[2],
                            StateCode = pieces[3]
                        };
                        states.Add(state);
                        var dr = table.NewRow();

                        dr["IEN"] = state.Ien;
                        dr["NAME"] = state.Name;
                        dr["ABBR"] = state.Abbreviation;
                        dr["STCODE"] = state.StateCode;

                        table.Rows.Add(dr);
                    }
                }
                Keys.Tables.LookupTablesCache.Add(Keys.RpcCalls.GetStates, new DataTableCache(table, 60));
            }

            return states;
        }

        private Country GetCountry(string c)
        {
            if (string.IsNullOrEmpty(c))
                return null;

            return GetCountries().FirstOrDefault(i => i.Ien == c || i.Name == c || i.Abbreviation == c);
        }

        public IList<Country> GetCountries()
        {
            var countries = new List<Country>();

            // Remove any stale records from the Keys.Tables.LookupTablesCache
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);
            DataTable table;

            if (Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.GetCountries))
                table = Keys.Tables.LookupTablesCache[Keys.RpcCalls.GetCountries].Table;
            else
            {
                var parms = new List<string>();
                table = _adt.callOVIDRPC(Keys.RpcCalls.GetCountries, parms);
                Keys.Tables.LookupTablesCache.Add(Keys.RpcCalls.GetCountries, new DataTableCache(table, 60));
            }

            if (table.Rows.Count > 0)
            {
                foreach (DataRow dr in table.Rows)
                {
                    var country = new Country
                    {
                        Ien = dr["COUNTRY"].ToString(),
                        Name = dr["COUNTRYNAME"].ToString(),
                        Abbreviation = dr["COUNTRYABBR"].ToString()
                    };
                    countries.Add(country);
                }
            }
            return countries;
        }
        public ObservableCollection<AppointmentRequest> GetSensitiveRecordInfo(ObservableCollection<AppointmentRequest> appointmentRequests)
        {
            var apptRequests = new ObservableCollection<AppointmentRequest>();
            foreach (var apRequest in appointmentRequests.ToList())
            {
                apRequest.Patient.IsSensitiveRecord = IsSensitive(apRequest.Patient.IEN);
                apptRequests.Add(apRequest);
            }
            return apptRequests;
        }

        public IList<Race> GetRaces()
        {
            var races = new List<Race>();

            try
            {
                // Remove any stale records from the Keys.Tables.LookupTablesCache
                Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);
                DataTable dt;

                if (Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.GetRaces))
                    dt = Keys.Tables.LookupTablesCache[Keys.RpcCalls.GetRaces].Table;
                else
                {
                    var parms = new List<string>();
                    dt = _adt.callOVIDRPC(Keys.RpcCalls.GetRaces, parms);
                    Keys.Tables.LookupTablesCache.Add(Keys.RpcCalls.GetRaces, new DataTableCache(dt, 60));
                }

                foreach (DataRow row in dt.Rows)
                {
                    var race = new Race
                    {
                        Ien = row["RACEIEN"].ToString(),
                        Name = row["RACENAME"].ToString(),
                        Abbreviation = row["RACEABBR"].ToString()
                    };
                    races.Add(race);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Race List Error");
            }
            return races;
        }

        public void GetEncounterDataFilterData(WorkCompletedMethod workCompletedMethod)
        {
            DoWorkAsync((s, args) =>
            {
                try
                {
                    args.Result = GetEncounterDataFilterData();
                }
                catch (Exception ex)
                {
                    args.Result = ex;
                }
            }, workCompletedMethod);
        }

        public string GetEncounterDataFilterData()
        {
            var parms = new List<string>();
            return _adt.CallVistaRpcString("SD VSE FILTER RPC", parms);
        }

        public void GetEncounterDataReportData(string reportType, string dateAgregate, string dateRange, WorkCompletedMethod workCompletedMethod)
        {
            DoWorkAsync((s, args) =>
            {
                try
                {
                    args.Result = GetEncounterDataReportData(reportType, dateAgregate, dateRange);
                }
                catch (Exception ex)
                {
                    args.Result = ex;
                }
                //System.Threading.Thread.Sleep(1000 * 60);
            }, workCompletedMethod);
        }

        public string GetEncounterDataReportData(string reportType, string dateAgregate, string dateRange)
        {
            var parms = new List<string> { reportType, dateAgregate, dateRange };
            return _adt.CallVistaRpcString("SD VSE REPORT RPC", parms);
        }

        public AppointmentRequest GetAppointmentRequestByAppointmentIen(string appointmentId, ref string reqType)
        {
            var parms = new List<string> { appointmentId, null, null };
            var result = _adt.callOVIDRPC(Keys.RpcCalls.WlOpen, parms);
            string errorId;
            var wlIen = "";
            foreach (DataRow rslt in result.Rows)
            {
                errorId = rslt["ERRORID"].ToString();
                if (errorId == "0")
                {
                    wlIen = rslt["ERRTEXT"].ToString();
                    break;
                }
            }
            if (!string.IsNullOrEmpty(wlIen))
            {
                parms.Clear();
                parms.Add(wlIen);
                var table = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, parms);
                foreach (DataRow wlRec in table.Rows)
                {
                    var ien = wlRec["IEN"].ToString();
                    if (ien != wlIen)
                        continue;
                    Keys.AppointmentRequestWaitListTables.EwlWaitList.ImportRow(wlRec);
                    var ewlRequests = GetEwlAppointmentRequests(table);
                    reqType = Keys.AppointmentRequestTypes.Ewl;

                    if (ewlRequests.Count > 0)
                        return ewlRequests[0]; //.FirstOrDefault();
                }
            }
            else
            {
                var arIen = "";
                parms.Clear();
                parms.Add(appointmentId);
                parms.Add(null);
                parms.Add(null);
                var arReq = _adt.callOVIDRPC(Keys.RpcCalls.ArOpen, parms);
                foreach (DataRow rslt in arReq.Rows)
                {
                    errorId = rslt["ERRORID"].ToString();
                    if (errorId == "0")
                        arIen = rslt["ERRTEXT"].ToString();
                }
                if (!string.IsNullOrEmpty(arIen))
                {
                    parms.Clear();
                    parms.Add(arIen);
                    var table = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, parms);
                    foreach (DataRow arRec in table.Rows)
                    {
                        var ien = arRec["IEN"].ToString();
                        if (ien != arIen)
                            continue;
                        Keys.AppointmentRequestWaitListTables.ApptWaitList.ImportRow(arRec);
                        var apptRequests = GetApptAppointmentRequests(table);
                        reqType = Keys.AppointmentRequestTypes.Appt;

                        if (apptRequests.Count > 0)
                            return apptRequests[0]; //.FirstOrDefault();
                    }
                }
            }
            return null;
        }

        public AppointmentRequest GetAppointmentRequestByRequestIenAndType(string requestIen, string requestType)
        {
            if (string.IsNullOrEmpty(requestIen))
                return null;

            string reqType = requestType;

            if (reqType.Length == 1)
                reqType = Keys.AppointmentRequestTypeFromFirstLetter(reqType);

            if (string.IsNullOrEmpty(reqType))
                return null;

            switch (reqType)
            {
                case Keys.AppointmentRequestTypes.PtCSch:
                    return GetPtCSchApptRequest(requestIen);
                case Keys.AppointmentRequestTypes.Ewl:
                    return GetEwlApptRequest(requestIen);
                case Keys.AppointmentRequestTypes.Consult:
                case Keys.AppointmentRequestTypes.Procedure:
                    return GetConsultApptRequestByIen(requestIen);
                case Keys.AppointmentRequestTypes.Appt:
                case Keys.AppointmentRequestTypes.Veteran:
                case Keys.AppointmentRequestTypes.Mobile:
                case Keys.AppointmentRequestTypes.WelcomeToVa:
                case Keys.AppointmentRequestTypes.ReturnToClinic:
                    return GetApptRequestByRequestIEN(requestIen);
            }

            return null;
        }


        //TODO: RESET SDEC CONTACT FILE SEQUENCE START
        public AppointmentRequest ReopenAppointmentRequestByRequestIen(string requestId, ref string reqType)
        {
            var parms = new List<string> { null, requestId, null };
            var result = _adt.callOVIDRPC(Keys.RpcCalls.WlOpen, parms);
            string errorId;
            var wlIen = "";
            foreach (DataRow rslt in result.Rows)
            {
                errorId = rslt["ERRORID"].ToString();
                if (errorId == "0")
                {
                    wlIen = rslt["ERRTEXT"].ToString();
                    break;
                }
            }
            if (!string.IsNullOrEmpty(wlIen))
            {
                parms.Clear();
                parms.Add(wlIen);
                var table = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, parms);
                foreach (DataRow wlRec in table.Rows)
                {
                    var ien = wlRec["IEN"].ToString();
                    if (ien != wlIen)
                        continue;
                    Keys.AppointmentRequestWaitListTables.EwlWaitList.ImportRow(wlRec);
                    var ewlRequests = GetEwlAppointmentRequests(table);
                    reqType = Keys.AppointmentRequestTypes.Ewl;

                    if (ewlRequests.Count > 0)
                        return ewlRequests[0]; //.FirstOrDefault();
                }
            }
            else
            {
                var arIen = "";
                parms.Clear();
                parms.Add(null);
                parms.Add(requestId);
                parms.Add(null);
                var arReq = _adt.callOVIDRPC(Keys.RpcCalls.ArOpen, parms);
                foreach (DataRow rslt in arReq.Rows)
                {
                    errorId = rslt["ERRORID"].ToString();
                    if (errorId == "0")
                        arIen = rslt["ERRTEXT"].ToString();
                }
                if (!string.IsNullOrEmpty(arIen))
                {
                    parms.Clear();
                    parms.Add(arIen);
                    var table = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, parms);
                    foreach (DataRow arRec in table.Rows)
                    {
                        var ien = arRec["IEN"].ToString();
                        if (ien != arIen)
                            continue;
                        Keys.AppointmentRequestWaitListTables.ApptWaitList.ImportRow(arRec);
                        var apptRequests = GetApptAppointmentRequests(table);
                        reqType = Keys.AppointmentRequestTypes.Appt;

                        if (apptRequests.Count > 0)
                            return apptRequests[0]; //.FirstOrDefault();
                    }
                }
            }
            return null;
        }

        //TODO: RESET SDEC CONTACT FILE SEQUENCE START
        public string ReopenWaitListRecordAfterCancelAppointment(string reqTypeAbbrev, string reqIen, string appointmentId, string name, string patIen, string desiredDate, bool desiredDateUpdated = false, bool addToWaitList = true)
        {
            try
            {
                List<string> parms = new List<string>();
                DataTable result = null;
                string errorId = string.Empty;
                string wlIen = string.Empty;
                var arIen = string.Empty;

                switch (Keys.AppointmentRequestTypeFromFirstLetter(reqTypeAbbrev))
                {
                    case Keys.AppointmentRequestTypes.PtCSch:
                        // PtCSch Requests are not removed from the grid after an appointment booking
                        // so do NOT need to be re-added to the grid on cancel.
                        // Leaving this here and cpmmented out in case the rules change :)
                        //if (addToWaitList)
                        //{
                        //    parms.Add(null);
                        //    parms.Add(null);
                        //    parms.Add(null);
                        //    parms.Add(null);
                        //    parms.Add(null);
                        //    parms.Add(reqIen);
                        //    var newRec = _adt.callOVIDRPC(Keys.RpcCalls.GetPtCSchAppointments, parms);

                        //    if ((newRec != null) && (newRec.Rows.Count > 0))
                        //    {
                        //        Keys.AppointmentRequestWaitListTables.PtCSchWaitList.ImportRow(newRec.Rows[0]);
                        //        IList<AppointmentRequest> recList = GetPtCSchAppointmentRequests(newRec);

                        //        if ((recList != null) && (recList.Count > 0))
                        //            Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(recList[0]);
                        //    }
                        //}

                        return reqIen;
                    case Keys.AppointmentRequestTypes.Ewl:
                        parms.Add(appointmentId);
                        parms.Add(reqIen);
                        parms.Add(desiredDate);
                        result = _adt.callOVIDRPC(Keys.RpcCalls.WlOpen, parms);

                        foreach (DataRow rslt in result.Rows)
                        {
                            errorId = rslt["ERRORID"].ToString();
                            if (errorId == "0")
                            {
                                wlIen = rslt["ERRTEXT"].ToString();
                                break;
                            }
                        }

                        if (!string.IsNullOrEmpty(wlIen) && addToWaitList)
                        {
                            var parm = new List<string> { wlIen };
                            var waitList = _adt.callOVIDRPC(Keys.RpcCalls.WaitListGet, parm);

                            if ((waitList != null) && (waitList.Rows.Count > 0))
                            {
                                var wt = waitList.Rows[0];
                                List<AppointmentRequest> list = GetEwlAppointmentRequests(waitList);
                                Keys.AppointmentRequestWaitListTables.EwlWaitList.ImportRow(wt);

                                if (list.Count > 0)
                                    Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(list[0]);
                            }
                        }

                        return wlIen;
                    case Keys.AppointmentRequestTypes.Consult:
                    case Keys.AppointmentRequestTypes.Procedure:
                        if (addToWaitList)
                        {
                            parms.Add(null);
                            parms.Add(null);
                            parms.Add(null);
                            parms.Add(null);
                            parms.Add(reqIen);
                            DataTable newReq = _adt.callOVIDRPC(Keys.RpcCalls.GetConsultAppointmentRequests, parms);

                            if ((newReq != null) && (newReq.Rows.Count > 0))
                            {
                                Keys.AppointmentRequestWaitListTables.ConsultWaitList.ImportRow(newReq.Rows[0]);
                                List<AppointmentRequest> recList = GetConsultAppointmentRequests();

                                if ((recList != null) && (recList.Count > 0))
                                {
                                    AppointmentRequest rec = recList.FirstOrDefault(r => (r.ConsultationRequestIen.ToString() == reqIen));

                                    if (rec != null)
                                        Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(rec);
                                }
                            }
                        }

                        return reqIen;
                    case Keys.AppointmentRequestTypes.Appt:
                    case Keys.AppointmentRequestTypes.Veteran:
                    case Keys.AppointmentRequestTypes.Mobile:
                    case Keys.AppointmentRequestTypes.WelcomeToVa:
                    case Keys.AppointmentRequestTypes.ReturnToClinic:
                        parms.Add(appointmentId);
                        parms.Add(reqIen);
                        parms.Add(null);
                        DataTable arReq = _adt.callOVIDRPC(Keys.RpcCalls.ArOpen, parms);

                        foreach (DataRow rslt in arReq.Rows)
                        {
                            errorId = rslt["ERRORID"].ToString();
                            if (errorId == "0")
                                arIen = rslt["ERRTEXT"].ToString();
                        }

                        if (!string.IsNullOrEmpty(arIen))
                        {
                            if (desiredDateUpdated)
                            {
                                //call arset with the updated desired date...
                                parms.Clear();
                                //  1. Wait List IEN
                                parms.Add(arIen);
                                DataTable arRec = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, parms);
                                List<AppointmentRequest> request = GetApptAppointmentRequests(arRec);
                                parms.Clear();
                                //  1. Wait List IEN
                                parms.Add(arIen);
                                //  2. DFN
                                parms.Add(patIen);
                                //  3. Origination Date
                                parms.Add(request[0].OriginatingDateString);
                                //  4. Institution Name
                                parms.Add(request[0].Patient.Institution);
                                //  5. Request Type
                                parms.Add(request[0].AppointmentRequestType);
                                //  6. Request Specific Clinic Name
                                parms.Add(request[0].ClinicServiceSpecialtyMerge);
                                //  7. Originating User
                                parms.Add(request[0].OriginatingUser);
                                //  8. Priority
                                parms.Add(request[0].PriorityText);
                                //  9. Requested By (PATIENT \ PROVIDER)
                                parms.Add(request[0].RequestedByText);
                                // 10. Privider Name, if Requested By Provider
                                if (request[0].Provider != null)
                                    parms.Add(request[0].Provider.Name);
                                else
                                    parms.Add(null);
                                var dtDesiredDate = desiredDate.Split(Convert.ToChar(" "))[0];
                                // 11. Desired Date \ CID
                                parms.Add(dtDesiredDate);
                                // 12. Comment
                                parms.Add(request[0].PtCSchComment);
                                // 13. Enrollment Priority
                                parms.Add(request[0].EnrollmentPriority);
                                //MRTC vars..
                                // 14. MRTC Required
                                parms.Add(request[0].MultipleAppointmentsRequired ? "YES" : "NO");

                                if (request[0].MultipleAppointmentsRequired)
                                {
                                    // 15. MRTC Appt Interval
                                    parms.Add(request[0].AppointmentIntervalInDays.ToString());
                                    // 16. MRTC Appt Number
                                    parms.Add(request[0].NumberOfAppointmentsRequired.ToString());
                                }
                                else
                                {
                                    // 15. MRTC Appt Interval
                                    parms.Add(null);
                                    // 16. MRTC Appt Number
                                    parms.Add(null);
                                }
                                // 17. Patient Contacts
                                #region BJ'S PATIENT CONTACT REPLACEMENT CODE
                                parms.Add("");
                                //TODO: NEED TO REMVOE THE "Patient Contacts" parms FROM THE Keys.RpcCalls.AppointmentRequestSet
                                #endregion

                                #region HPE ORIGINAL PATIENT CONTACT CODE (COMMENTED OUT) 
                                //var ptContact = "";
                                ////if (request[0].PatientContacts != null)
                                ////{
                                ////    foreach (var contact in request[0].PatientContacts)
                                ////    {
                                ////        if (!string.IsNullOrEmpty(ptContact))
                                ////            ptContact += "::";
                                ////        ptContact += contact.DateEntered + "~~" + contact.PCEnteredByUserIen + "~~" + contact.Action + "~~" + contact.PatientPhone;
                                ////    }
                                ////}
                                //parms.Add(ptContact);
                                #endregion

                                // 18. Service Connected
                                parms.Add(request[0].Patient.ServiceConnectionBool ? "YES" : "NO");
                                // 19. Service Connection Percentage
                                parms.Add(request[0].Patient.ServiceConnectionPercentage);
                                // 20. MRTC Calculated preferred dates, separated by pipe "|" (Optional)
                                parms.Add(null);
                                // 21.
                                parms.Add(request[0].ClinicStopName);
                                // 22. Appointment Type, pointer to Appointment Type file (Optional)
                                var apptTypeID = AppointmentTypes(request[0].Patient.IEN).FirstOrDefault(a => a.Name.ToUpper() == request[0].AppointmentType.ToUpper());
                                parms.Add(apptTypeID.Ien);
                                // 23. Patient Status (Optinal)
                                parms.Add(request[0].EnrolleeStatus);
                                // 24. (Optional - For Master MRTC Requests ONLY) Multiple pointers to Appointment File separated by "|"
                                if (request[0].IsMRTCParent && request[0].MRTCAppointmentRequestIENs.Count > 0)
                                {
                                    var ienList = string.Join("|", request[0].MRTCAppointmentRequestIENs.ToArray());
                                    parms.Add(ienList);
                                }
                                else
                                    parms.Add(null);
                                // 25. (Optional - MRTC Child Request ONLY) MRTC Master Request IEN
                                if (request[0].IsMRTCChild && !string.IsNullOrEmpty(request[0].MRTCParentRequestIEN))
                                    parms.Add(request[0].MRTCParentRequestIEN);
                                else
                                    parms.Add(null);

                                var dt = _adt.callOVIDRPC(Keys.RpcCalls.AppointmentRequestSet, parms);
                                if (dt != null && dt.Rows.Count == 1)
                                {
                                    var r = dt.Rows[0];
                                    if (r["ERRORTEXT"].ToString() != string.Empty)
                                    {
                                        var sError = r["ERRORTEXT"].ToString();
                                        MessageBox.Show("Unable to Update Appointment Request.  " + sError);
                                    }
                                }
                            }

                            if (addToWaitList)
                            {
                                parms.Clear();
                                //  1. Wait List IEN
                                parms.Add(arIen);
                                arReq = _adt.callOVIDRPC(Keys.RpcCalls.GetAppointmentRequestFileRequests, parms);

                                if ((arReq != null) && (arReq.Rows.Count > 0))
                                {
                                    Keys.AppointmentRequestWaitListTables.ApptWaitList.ImportRow(arReq.Rows[0]);
                                    List<AppointmentRequest> requestList = GetApptAppointmentRequests(arReq);

                                    if ((requestList != null) && (requestList.Count > 0))
                                        Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.Add(requestList[0]);
                                }
                            }
                        }

                        return arIen;
                }

                return string.Empty;
            }
            catch (Exception ex)
            {
                TraceLog.LogInfo(ex.Source, ex.GetType().ToString(), ex.Message);
                //MessageBox.Show(ex.Source + ": " + ex.Message);
            }

            return string.Empty;
        }

        public IList<NonDisplayPatientRequest> GetNonDisplayPatientRequests(string patientIen = "", string clinicIen = "")
        {
            //_BJ_8_13_15: This report has been deprecated
            /* if(string.IsNullOrEmpty(patientIen) && string.IsNullOrEmpty(clinicIen))
             {
                 MessageBox.Show("Please select a patient or clinic to view requests.");
                 return null;
             }
             * */
            var returnList = new List<NonDisplayPatientRequest>();

            //var parms = new List<string> {patientIen, clinicIen};

            /*
            if(string.IsNullOrEmpty(patientIen))
                parms.Add(null);
            else
                parms.Add(patientIen);
            if(string.IsNullOrEmpty(clinicIen))
                parms.Add(null);
            else
                parms.Add(clinicIen);
            */
            var ctr = 0;
            var results = new DataTable(); // _adt.callOVIDRPC(Keys.RpcCalls.NonDisplayPatientRequests, parms);
            foreach (DataRow dr in results.Rows)
            {

                var newRequest = new NonDisplayPatientRequest
                {
                    PatientIen = dr["DFN"].ToString(),
                    PatientName = dr["PATIENTNAME"].ToString(),
                    WlSpecificClinic = dr["WLCLNAME"].ToString(),
                    OriginatingUserIen = dr["USERIEN"].ToString(),
                    OriginatingUserName = dr["USERNAME"].ToString(),
                    DateTimeEntered = dr["DATE1"].ToString(),
                    DesiredDate = dr["DAPTDT"].ToString(),
                    OriginationDate = dr["ORIGDT"].ToString(),
                    CurrentStatus = dr["STATUS"].ToString()
                };
                returnList.Add(newRequest);
                ctr++;
                //if (ctr > 100)
                //  break;
            }
            return returnList;
        }

        public IList<DisabledClinic> GetDisabledClinics()
        {
            //_BJ_8_13_15: This report has been deprecated
            var disabledClinicRecs = new DataTable(); // _adt.callOVIDRPC(Keys.RpcCalls.DisabledClinics, parms);
            var disabledClinics = new List<DisabledClinic>();

            foreach (DataRow dr in disabledClinicRecs.Rows)
            {
                var dc = new DisabledClinic
                {
                    Name = dr["CLINIC_NAME"].ToString(),
                    Id = dr["CLINIC_IEN"].ToString()
                };
                disabledClinics.Add(dc);
            }

            var blankClinic = new DisabledClinic { Name = "" };
            disabledClinics.Insert(0, blankClinic);
            return disabledClinics;
        }

        public bool UserHasAccessToProhibitedClinic(string hospLocId)
        {
            var parms = new List<string> { hospLocId, "1" };
            //to get both Active and Inactive users.. //11_20_2015_BJ
            var dt = _adt.callOVIDRPC(Keys.RpcCalls.ProhibitedUsers, parms);
            foreach (DataRow dr in dt.Rows)
            {
                var prohibitedUser = new ProhibitedUser
                {
                    Name = dr["NEWPERSONNAME"].ToString(),
                    Ien = dr["NEWPERSONID"].ToString()
                };
                if (prohibitedUser.Ien == CurrentUser)
                    return true;
            }
            return false;
        }

        //        public IList<ProhibitedClinic> GetProhibitedClincs()
        //        {
        //            return GetSdecProhibitedClinics();
        //            var prohibitedClinics = new List<ProhibitedClinic>();

        //            foreach(DataRow rec in Keys.Tables.HospitalResourceTable.Rows)
        //            {
        //                var prhBloc = rec["PRHBLOC"].ToString();
        //                if (prhBloc == "1")
        //               {
        //                    var pClinic = new ProhibitedClinic
        //                        {
        //                            Name = rec["RESOURCE_NAME"].ToString(),
        //                            Ien = rec["RESOURCEID"].ToString(),
        //                            HospitalLocationId = rec["HOSPITAL_LOCATION_ID"].ToString()
        //                        };

        //                    prohibitedClinics.Add(pClinic);
        /*var privUsers = rec["PRIVLOC"].ToString();
        if (pClinic.ProhibitedUsers == null)
            pClinic.ProhibitedUsers = new List<ProhibitedUser>();
        if (privUsers == "1")
        {
            var parms = new List<string>();
            parms.Add(pClinic.HospitalLocationId);
            var dt = _adt.callOVIDRPC(Keys.RpcCalls.ProhibitedUsers, parms);
            foreach(DataRow dr in dt.Rows)
            {
                var prohibitedUser = new ProhibitedUser();
                prohibitedUser.Name = dr["NEWPERSONNAME"].ToString();
                prohibitedUser.Ien = dr["NEWPERSONID"].ToString();
                pClinic.ProhibitedUsers.Add(prohibitedUser);
            }
        }*/
        //                }
        //            }
        //            return prohibitedClinics.OrderBy(c => c.Name).ToList();
        //        }


        /*private DataTable GetProhibitedClincsDataTable()
        {
            var prohitedClinicsDt = Keys.Tables.HospitalResourceTable;

            for (int i = prohitedClinicsDt.Rows.Count - 1; i >= 0; i--)
            {
                var prohibitClinicRow = prohitedClinicsDt.Rows[i];
                var prhBloc = prohibitClinicRow["PRHBLOC"].ToString();
                if (prhBloc != "1")
                    prohibitClinicRow.Delete();
            }
            return prohitedClinicsDt;
        }

        public IList<SchdResource> GetProhibitedResourcesByType()
        {
            DataTable table;
            table = GetProhibitedClincsDataTable();
            return ResourcesListFromDataTable(table, false);
        }
        */
        /*private void UpdateProhibitedClinicsAndUsers()
        {
            var prohibitedClinics = Keys.Tables.ProhibitedClinicsAndUsers; // GetProhibitedClincs();
            //Get Resource Users
            var tmpResourceDataTable = _mDsGlobal.Tables["ResourceUser"];
            var resourceUserIds = new List<ResourceUserIds>();
            foreach (DataRow resUser in tmpResourceDataTable.Rows)
            {
                var bFound = false;
                var resUserClinicIen = resUser["RESOURCE_ID"].ToString();
                var resUserUserIen = resUser["USER_ID"].ToString();
                var sdecResourceUserIen = resUser["SDEC_RESOURCE_USER_IEN"].ToString();
                var userName = resUser["USERNAME"].ToString();
                var resName = resUser["RESOURCE_NAME"].ToString();
                foreach(var clinic in prohibitedClinics)
                {
                    var clinicId = clinic.Ien;
                    foreach(var user in clinic.ProhibitedUsers)
                    {
                        var userId = user.Ien;
                        if(resUserClinicIen == clinicId && resUserUserIen == userId)
                        {
                            bFound = true;
                            break;
                        }
                    }
                }
                if (!bFound)
                {
                    var resourceUserId = new ResourceUserIds
                        {ResourceUserIen = sdecResourceUserIen, ClinicIen = resUserClinicIen};
                    resourceUserIds.Add(resourceUserId);
                    //var lst = RemoveResourceUserByID(sdecResourceUserIen, resUserClinicIen);
                }
            }
            foreach(ResourceUserIds resIds in resourceUserIds)
            {
                var lst = RemoveResourceUserByID(resIds.ResourceUserIen, resIds.ClinicIen);
            }

            _mDsGlobal.Tables["ResourceUser"].Clear();
            //Get Resource Users
            _adt.callOVIDRPC(Keys.RpcCalls.GetResourceUsers, null, "ResourceUser", _mDsGlobal);

            
            var recordFound = false;
            foreach(var clinic in prohibitedClinics)
            {
                var clinicId = clinic.Ien;
                foreach(var user in clinic.ProhibitedUsers)
                {
                    recordFound = false;
                    var userId = user.Ien;
                    foreach(DataRow resUser in _mDsGlobal.Tables["ResourceUser"].Rows)
                    {
                        var resourceUserIen = resUser["SDEC_RESOURCE_USER_IEN"].ToString();
                        var resUserClinicIen = resUser["RESOURCE_ID"].ToString();
                        var resUserClinicName = resUser["RESOURCE_NAME"].ToString();
                        var resUserUserIen = resUser["USER_ID"].ToString();
                        var resUserName = resUser["USERNAME"].ToString();
                        if(resUserClinicIen == clinicId && resUserUserIen == userId)
                        {
                            //Update....
                            recordFound = true;
                            var ru = new ResourceUser
                                {
                                    BSDX_RESOURCE_USER_IEN = resourceUserIen,
                                    RESOURCE_ID = resUserClinicIen,
                                    RESOURCE_NAME = resUserClinicName,
                                    USER_ID = resUserUserIen,
                                    USERNAME = resUserName,
                                    MODIFY_APPTS = "YES",
                                    MODIFY_SCHEDULE = "YES"
                                };
                            AddEditResourceUser(ru, true);
                            break;
                        }


                    }
                    if(!recordFound)
                    {
                        var ru = new ResourceUser
                        {
                            BSDX_RESOURCE_USER_IEN = null,
                            RESOURCE_ID = clinic.Ien,
                            RESOURCE_NAME = clinic.Name,
                            USER_ID = user.Ien,
                            USERNAME = user.Name,
                            MODIFY_APPTS = "YES",
                            MODIFY_SCHEDULE = "YES"
                        };
                        AddEditResourceUser(ru, true); 
                    }
                }
            }

        }*/
        public bool IsProhibitedClinic(string clinicName, string clinicIen, bool bSilent = false)
        {
            var isProhibited = false;
            var bPrivilegedUserFound = false;
            var prohibitedClinics = GetSdecProhibitedClinics(clinicName, null);
            var prohibitedClinic = prohibitedClinics.FirstOrDefault(pc => pc.HospitalLocationId == clinicIen);
            // Clinic is prohibited if found in the returned prohibited clinics list
            var selectedClinicIsProhibited = prohibitedClinic != null;

            if (selectedClinicIsProhibited)
                // If the current user is found in ProhibitedUsers then the user is allowed
                bPrivilegedUserFound = UserHasAccessToProhibitedClinic(prohibitedClinic.HospitalLocationId);

            if (selectedClinicIsProhibited && !bPrivilegedUserFound)
            {
                isProhibited = true;
                if (!bSilent)
                {
                    MessageBox.Show(
                        string.Format("Access to {0} is prohibited!\nOnly users designated as a Privileged User for this clinic may access this clinic.", clinicName),
                        "Prohibited Clinic",
                        MessageBoxButton.OK,
                        MessageBoxImage.Warning
                    );
                }
            }
            return isProhibited;
        }

        public List<ConsultUrgency> GetConsultUrgencyValues()
        {
            var retUrgencies = new List<ConsultUrgency> { new ConsultUrgency { Ien = "", Name = "", Synonym = "" } };

            var parms = new List<string>();
            var urgencies = _adt.callOVIDRPC(Keys.RpcCalls.ConsultUrgency, parms);
            foreach (DataRow row in urgencies.Rows)
            {
                var urg = new ConsultUrgency
                {
                    Name = row["NAME"].ToString(),
                    Ien = row["IEN"].ToString(),
                    Synonym = row["SYNONYM"].ToString()
                };
                retUrgencies.Add(urg);
            }
            return retUrgencies;
        }

        public string WLClose(string requestManagmentIen, string selectedCause)
        {
            var result = string.Empty;
            var parms = new List<string> { requestManagmentIen, selectedCause };
            var rslt = _adt.callOVIDRPC("SDEC WLCLOSE", parms);

            foreach (DataRow row in rslt.Rows)
            {
                var errorId = row["ERRORID"].ToString();
                var errorText = row["ERRORTEXT"].ToString();

                if (errorId != "0" && !string.IsNullOrEmpty(errorText))
                {
                    result = errorId + ": " + errorText;
                    break;
                }
            }
            return result;
        }

        public string ARClose(string apptRequestIen, string selectedCause)
        {
            var parms = new List<string> { apptRequestIen, selectedCause, Keys.LoggedInUserIen, DateTime.Now.ToString("M/d/yyyy") };
            var rslt = _adt.callOVIDRPC("SDEC ARCLOSE", parms);

            if (rslt.Rows.Count == 0)
                return string.Empty;

            var result = string.Empty;
            foreach (DataRow row in rslt.Rows)
            {
                var errorId = row["ERRORID"].ToString();
                var errorText = row["ERRORTEXT"].ToString();

                if (!string.IsNullOrEmpty(errorText))
                {
                    result = errorId + ": " + errorText;
                    break;
                }
            }
            return result;
        }

        public void RemoveJasperReportDependenciesFromUsersFolder()
        {
            try
            {
                var targetPath = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);
                targetPath += "\\" + Keys.UsersTempFolder;
                var usersTempFolder = new DirectoryInfo(targetPath);
                if (Directory.Exists(targetPath))
                {
                    foreach (var file in usersTempFolder.GetFiles())
                        file.Delete();
                    foreach (var dir in usersTempFolder.GetDirectories())
                        dir.Delete(true);
                    Directory.Delete(targetPath);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Remove File Error");
            }
        }

        public int CheckDeferredRpcStatus(string rpcHandle)
        {
            var parms = new List<string> { rpcHandle };
            var result = _adt.CallVistaRpc(Keys.DeferredRpcCalls.DeferredRpcStatus, parms);
            var rslt = GetArrayOfArrayItem(result, 0, 0); // gets result[0][0], if it exists

            if (!string.IsNullOrEmpty(rslt))
            {
                var intRslt = rslt.Split('^')[0];

                if (Keys.Methods.IsNumeric(intRslt))
                    return Convert.ToInt32(intRslt);
            }

            Keys.AlertUserError(null, new Exception("Invalid return from RPC " + Keys.DeferredRpcCalls.DeferredRpcStatus), "Deferred RPC Error");
            return 0;
        }

        private string GetArrayOfArrayItem(string[][] array, int outerIndex, int innerIndex, string defaultValue = "")
        {
            string result = defaultValue;

            if (array.Length > outerIndex)
                if (array[outerIndex].Length > innerIndex)
                    result = array[outerIndex][innerIndex];

            return result;
        }

        public string[][] GetDeferredRpcData(string rpcHandle)
        {
            var parms = new List<string> { rpcHandle };

            var result = _adt.callOVIDRPC(Keys.DeferredRpcCalls.DeferredRpcGetData, parms);

            return null;

        }

        public void ClearDeferredRpcData(string rpcHandle)
        {
            var parms = new List<string> { rpcHandle };
            var rslt = _adt.CallVistaRpc(Keys.DeferredRpcCalls.DeferredRpcClearData, parms);
        }

        public bool UnlockAppointmentRequest(string request)
        {
            var parms = new List<string> { request, "0" };

            var lockInfo = _adt.callOVIDRPC(Keys.RpcCalls.UnLockAppointmentRequest, parms);
            foreach (DataRow dr in lockInfo.Rows)
            {
                var code = dr["CODE"].ToString();
                return code == "1";
            }
            return false;
        }

        public bool LockAppointmentRequest(AppointmentRequest ar, ref string lockedRecord, ref string msg)
        {
            var parms = new List<string>();
            var requestType = ar.AppointmentRequestType;

            var requestTypeId = "";
            switch (requestType)
            {
                case Keys.AppointmentRequestTypes.Ewl:
                    requestTypeId = "E|" + ar.RequestManagmentIen;
                    break;
                case Keys.AppointmentRequestTypes.Appt:
                case Keys.AppointmentRequestTypes.Veteran:
                case Keys.AppointmentRequestTypes.WelcomeToVa:
                case Keys.AppointmentRequestTypes.ReturnToClinic:
                    requestTypeId = "A|" + ar.ApptRequestIen;
                    break;
                case Keys.AppointmentRequestTypes.PtCSch:
                    requestTypeId = "R|" + ar.PtCSchRequestIen;
                    break;
                case Keys.AppointmentRequestTypes.Procedure:
                case Keys.AppointmentRequestTypes.Consult:
                    requestTypeId = "C|" + ar.ConsultationRequestIen;
                    break;
            }

            lockedRecord = requestTypeId;
            parms.Add(requestTypeId);
            var lockInfo = _adt.callOVIDRPC(Keys.RpcCalls.LockAppointmentRequest, parms);
            foreach (DataRow dr in lockInfo.Rows)
            {
                var code = dr["CODE"].ToString();
                if (code == "1")
                    return true;

                msg = dr["MESSAGE"].ToString();
                return false;
            }
            return false;
        }

        /*        private SchdResource ResourceFromDataRowHospLoc(DataRow row)
                {
                    var list = GetSdecHResources(row["HOSPITAL_LOCATION"].ToString().ToUpper());

                    if (list.Count == 0)
                        return null;

                    var newResource = list.FirstOrDefault(r => (r.HOSPITAL_LOCATION_ID == row["HOSPITAL_LOCATION_ID"].ToString()));

                    if (newResource == null)
                        return null;

                    newResource.DefaultProvider = row["DEFAULT_PROVIDER"].ToString();
                    newResource.StopCodeNumber = row["STOP_CODE_NUMBER"].ToString();
                    newResource.InactiveDate = row["INACTIVATE_DATE"].ToString();
                    newResource.ReactivatedDate = row["REACTIVATE_DATE"].ToString();
                    return newResource;
                }
         //*/

        public IList<SchdResource> GetSdecHResources(string resourceName)
        {
            var parms = new List<string> { CurrentUser, "1", "H", Keys.ClinicResourceSearchMaxRecords.ToString(), null, null, resourceName };

            var resourceList = new List<SchdResource>();
            try
            {
                var dt = GetSdecResources(null, resourceName, parms);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        var res = ResourceFromDataRow(row);
                        resourceList.Add(res);
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetSdecResources", e);
                Console.WriteLine(e.Message);
            }
            return resourceList;
        }

        public IList<SchdResource> SearchSdecHResources(string searchString, string lastSub = null)
        {
            var parms = new List<string> { CurrentUser, "1", "H", Keys.ClinicResourceSearchMaxRecords.ToString(), string.IsNullOrEmpty(lastSub) ? null : lastSub, null, searchString };

            var resourceList = new List<SchdResource>();
            try
            {
                var dt = _adt.callOVIDRPC(Keys.RpcCalls.SdecResource, parms);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        var res = ResourceFromDataRow(row);

                        if (res != null)
                            resourceList.Add(res);
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "SearchSdecHResources", e);
                Console.WriteLine(e.Message);
            }
            return resourceList;
        }

        private DataTable GetSdecResources(string resourceIEN, string resourceName, List<string> parms)
        {
            if (Keys.SdecResourceParms != null && !parms.Where((t, i) => t != Keys.SdecResourceParms[i]).Any())
                return Keys.SdecResourceData;

            Keys.SdecResourceParms = parms;
            Keys.SdecResourceData = _adt.callOVIDRPC(Keys.RpcCalls.SdecResource, parms);
            return Keys.SdecResourceData;
        }

        public SchdResource GetSdecResource(string resourceId)
        {
            if (string.IsNullOrEmpty(resourceId))
                return null;

            try
            {
                var parms = new List<string> { CurrentUser, "1", null, null, null, resourceId, null };

                var dt = GetSdecResources(resourceId, null, parms);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        var newResource = ResourceFromDataRow(row);
                        return newResource;
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetSdecResource", e);
                Console.WriteLine(e.Message);
                //throw;
            }
            return null;
        }

        public IList<Provider> GetSdecProviders(string providerName)
        {
            if (string.IsNullOrEmpty(providerName))
                return null;

            var parms = new List<string> { CurrentUser, "1", "P", null, null, null, providerName };

            IList<Provider> newProviders = new List<Provider>();
            try
            {
                var dt = GetSdecResources(null, providerName, parms);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        if (Keys.Methods.StrToBool(row["PROVCLIN"].ToString()))
                        {
                            var newProv = _container == null ? new Provider() : _container.Resolve<Factory<Provider>>().Create();

                            var resTypePieces = row["RESOURCETYPE"].ToString();
                            var resType = resTypePieces.Split('|');
                            newProv.IEN = resType[1];
                            newProv.Name = row["RESOURCE_NAME"].ToString();
                            newProv.HasResources = true;
                            newProviders.Add(newProv);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetSdecProviders", e);
                Console.WriteLine(e.Message);
            }
            return newProviders;
        }

        public IList<Provider> SearchSdecProviders(string searchStr, string lastSub = null)
        {
            var parms = new List<string> { CurrentUser, "1", "P", Keys.ProviderSearchMaxRecords.ToString(), string.IsNullOrEmpty(lastSub) ? null : lastSub, null, searchStr };

            IList<Provider> newProviders = new List<Provider>();
            try
            {
                var dt = _adt.callOVIDRPC(Keys.RpcCalls.SdecResource, parms);
                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        if (Keys.Methods.StrToBool(row["PROVCLIN"].ToString()))
                        {
                            var newProv = _container == null ? new Provider() : _container.Resolve<Factory<Provider>>().Create();

                            var resTypePieces = row["RESOURCETYPE"].ToString();
                            var resType = resTypePieces.Split('|');
                            newProv.IEN = resType[1];
                            newProv.Name = row["RESOURCE_NAME"].ToString();
                            newProv.HasResources = true;
                            newProv.LastSub = row["LASTSUB"].ToString();
                            newProviders.Add(newProv);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "SearchSdecProviders", e);
                //Console.WriteLine(e.Message);
            }
            return newProviders;
        }

        public SchdResource GetSdecResourceByName(string resourceName)
        {
            if (string.IsNullOrEmpty(resourceName))
                return null;

            try
            {
                var parms = new List<string> { CurrentUser, "1", null, null, null, null, resourceName };
                var dt = GetSdecResources(null, resourceName, parms);

                if (dt == null || dt.Rows.Count == 0)
                {
                    // This will happen when searching resource with abbreviation prefixed names, so we will remove the 1st word and try again
                    if (resourceName.Contains(" "))
                    {
                        string shortName = resourceName.Substring(resourceName.IndexOf(" ") + 1).Trim();
                        parms = new List<string> { CurrentUser, "1", null, null, null, null, shortName };
                        dt = GetSdecResources(null, shortName, parms);
                    }
                }

                if (dt != null && dt.Rows.Count > 0)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        var newResource = ResourceFromDataRow(row);
                        return newResource;
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetSdecResourcedByName", e);
                Console.WriteLine(e.Message);
            }
            return null;
        }

        public IList<ProhibitedClinic> GetSdecProhibitedClinics(string clinicName, string clinicIEN)
        {
            DataTable dt = null;
            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.ProhibitedClinicCache);

            if (!string.IsNullOrEmpty(clinicIEN))
            {
                var key = Keys.Tables.ProhibitedClinicCache.Keys.FirstOrDefault(k => k.IEN == clinicIEN);
                if (key != null && Keys.Tables.ProhibitedClinicCache[key].Table.Rows.Count > 0)
                    dt = Keys.Tables.ProhibitedClinicCache[key].Table;
            }

            if (!string.IsNullOrEmpty(clinicName))
            {
                var key = Keys.Tables.ProhibitedClinicCache.Keys.FirstOrDefault(k => k.Name == clinicName);
                if (key != null && Keys.Tables.ProhibitedClinicCache[key].Table.Rows.Count > 0)
                    dt = Keys.Tables.ProhibitedClinicCache[key].Table;
            }

            if (dt == null)
            {
                var parms = new List<string> { CurrentUser, "1", "PH", null, null, clinicIEN, clinicName };
                dt = GetSdecResources(clinicName, null, parms);
                Keys.Tables.ProhibitedClinicCache.Add(new DataTableKey(clinicIEN, clinicName), new DataTableCache(dt, 60));
            }

            var prhClinics = new List<ProhibitedClinic>();

            if (dt != null && dt.Rows.Count > 0)
            {
                foreach (DataRow rec in dt.Rows)
                {
                    var prhClinic = new ProhibitedClinic
                    {
                        Name = rec["RESOURCE_NAME"].ToString(),
                        Ien = rec["RESOURCEID"].ToString(),
                        HospitalLocationId = rec["HOSPITAL_LOCATION_ID"].ToString(),
                        ResourceType = rec["RESOURCETYPE"].ToString()
                    };
                    prhClinics.Add(prhClinic);
                }
            }
            return prhClinics;
        }

        public IList<ProhibitedClinic> SearchSdecProhibitedClinics(string searchString)
        {
            var parms = new List<string> { CurrentUser, "1", "PH", Keys.ClinicResourceSearchMaxRecords.ToString(), null, null, searchString };

            var dt = _adt.callOVIDRPC(Keys.RpcCalls.SdecResource, parms);
            var prhClinics = new List<ProhibitedClinic>();

            if (dt != null && dt.Rows.Count > 0)
            {
                foreach (DataRow rec in dt.Rows)
                {
                    var prhClinic = new ProhibitedClinic
                    {
                        Name = rec["RESOURCE_NAME"].ToString(),
                        Ien = rec["RESOURCEID"].ToString(),
                        HospitalLocationId = rec["HOSPITAL_LOCATION_ID"].ToString(),
                        ResourceType = rec["RESOURCETYPE"].ToString(),
                        LastSub = rec["LASTSUB"].ToString()
                    };
                    prhClinics.Add(prhClinic);
                }
            }
            return prhClinics;
        }

        public PatientDisabilitiesModel GetPatientDisabilities(string patientDFN)
        {
            var parms = new List<string> { patientDFN };
            var dt = _adt.callOVIDRPC(Keys.RpcCalls.GetPatientDisabilities, parms);

            if (dt.Rows.Count > 0)
                return new PatientDisabilitiesModel(dt.Rows[0]);
            return null;
        }

        private SlotsAvailable ReturnApptSlots(SchdResource res, DateTime startTime, DateTime endTime)
        {
            // Appointment Slots not null
            if (Keys.AppointmentSlots != null && Keys.AppointmentSlots.ResourceId == res.RESOURCEID && Keys.AppointmentSlots.Slots.Count > 0)
                if (Keys.AppointmentSlots.Slots[0].StartTime.Date.CompareTo(startTime.Date) <= 0 && Keys.AppointmentSlots.Slots[Keys.AppointmentSlots.Slots.Count - 1].EndTime.Date.CompareTo(endTime.Date) >= 0)
                {
                    var cSlots = new SlotsAvailable
                    {
                        ClinicIen = Keys.AppointmentSlots.ClinicIen,
                        Name = Keys.AppointmentSlots.Name,
                        ResourceId = Keys.AppointmentSlots.ResourceId
                    };

                    // Pull out only the slots that are within the start\end times and return them.
                    cSlots.Slots = Keys.AppointmentSlots.Slots.Where(s => (s.StartTime.CompareTo(startTime.Date) >= 0) && s.EndTime.CompareTo(endTime) <= 0).ToList();

                    return cSlots; // return the slots
                }

            // call for the required slots
            return GetAppSlots(res, startTime.Date, endTime.Date.AddDays(1).AddMinutes(-1));
        }

        private SlotsAvailable GetAppSlots(SchdResource res, DateTime startDate, DateTime endDate)
        {
            if (res == null)
                return new SlotsAvailable();

            var tmpSlots = new SlotsAvailable
            {
                ClinicIen = res.HOSPITAL_LOCATION_ID,
                ResourceId = res.RESOURCEID,
                Name = res.RESOURCE_NAME,
                Slots = new List<CSlot>()
            };

            try
            {
                var parms = new List<string>
                {
                    string.IsNullOrEmpty( res.RESOURCEID ) ? res.RESOURCE_NAME : res.RESOURCEID,
                    startDate.ToShortDateString(),
                    endDate.ToShortDateString()
                };

                var dt = _adt.callOVIDRPC(Keys.RpcCalls.AppointmentSlotsAvailable, parms);

                if (dt != null)
                {
                    foreach (DataRow dr in dt.Rows)
                    {
                        var theSlot = new CSlot();
                        var date = ConvertOrigDate(dr["DATE"].ToString());

                        if (!Keys.Methods.IsDate(date))
                            continue;

                        theSlot.Date = date;
                        theSlot.SlotCount = dr["AVAILABILITY"].ToString();
                        var startTime = dr["START_TIME"].ToString();

                        var hour = startTime.Substring(0, 2);
                        date = date.Value.AddHours(Convert.ToInt32(hour));

                        var min = startTime.Substring(2, 2);
                        date = date.Value.AddMinutes(Convert.ToInt32(min));
                        theSlot.StartTime = Convert.ToDateTime(date);

                        var endTime = dr["END_TIME"].ToString();
                        hour = endTime.Substring(0, 2);
                        date = theSlot.Date.Value.AddHours(Convert.ToInt32(hour));
                        min = endTime.Substring(2, 2);
                        date = date.Value.AddMinutes(Convert.ToInt32(min));
                        theSlot.EndTime = Convert.ToDateTime(date);

                        // It is possible for slots to be created where start time is later than end time, we are ignoring these anomolies
                        if (theSlot.StartTime.CompareTo(theSlot.EndTime) <= 0)
                            tmpSlots.Slots.Add(theSlot);
                    }
                }
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetAppSlots", e);
#if DEBUG
                Keys.AlertUserError(this, e, "App Slots Error");
#endif
            }

            return tmpSlots;
        }

        private void GetApptSlots(SchdResource res, string startDate, string endDate)
        {
            try
            {
                Keys.AppointmentSlots = GetAppSlots(res, Convert.ToDateTime(startDate), Convert.ToDateTime(endDate));
            }
            catch (Exception e)
            {
                TraceLog.LogException(GetType().Name, "GetApptSlots", e);
#if DEBUG
                Keys.AlertUserError(this, e, "App Slots Error");
#endif
            }
        }

        public bool SetMRTCChildRequest(AppointmentRequest request, string requestIEN = "", string appointmentIEN = "")
        {
            try
            {
                var parms = new List<string>
                {
                    //  1. Request IEN (Optional)
                    requestIEN,
                    //  2. Patient IEN
                    request.Patient.IEN,
                    //  3. Originating Date
                    request.OriginatingDate.HasValue ? request.OriginatingDate.Value.ToString( "M/d/yyyy@hh:mm" ) : DateTime.Today.ToString(),
                    //  4. Institution Name
                    request.Patient.Institution,
                    //  5. Request Type
                    request.AppointmentRequestType,
                    //  6. Request Specific Clinc, Name from file 44
                    request.ClinicIen,
                    //  7. Originating User Name
                    request.OriginatingUser,
                    //  8. Priority
                    request.PriorityText,
                    //  9. Requested By
                    request.RequestedByText,
                    // 10. Requesting Provider Name, if Provider Request
                    request.RequestedByText == "PATIENT" ? string.Empty : request.RequestedByProviderMerge,
                    // 11. Desired Date
                    request.DesiredApptDate.HasValue ? request.DesiredApptDate.Value.ToShortDateString() : DateTime.Today.ToShortDateString(),
                    // 12. Comment
                    request.RequestManagmentComment,
                    // 13. Enrollment Priority
                    request.EnrollmentPriority,
                    // 14. MRTC ?
                    request.MultipleAppointmentsRequired ? "YES" : "NO",
                    // 15. MRTC Appointment Interval - For child MRTC record this is the "index" of the child request
                    request.MultipleAppointmentsRequired ? request.AppointmentIntervalInDays.ToString() : null,
                    // 16. MRTC MRTC Appointment Count
                    request.MultipleAppointmentsRequired ? request.NumberOfAppointmentsRequired.ToString() : null
                };

                // 17. Patient Contacts
                #region BJ'S PATIENT CONTACT REPLACEMENT CODE
                parms.Add("");
                //TODO: NEED TO REMVOE THE "Patient Contacts" parms FROM THE Keys.RpcCalls.AppointmentRequestSet
                #endregion

                #region HPE ORIGINAL PATIENT CONTACT CODE (COMMENTED OUT) 
                //var ptContact = string.Empty;

                //if ( request.PatientContacts != null )
                //{
                //    foreach ( var contact in request.PatientContacts )
                //    {
                //        if ( !string.IsNullOrEmpty( ptContact ) )
                //            ptContact += "::";
                //        ptContact += contact.DateEntered + "~~" + contact.PCEnteredByUserIen + "~~" + contact.Action + "~~" + contact.PatientPhone;
                //    }
                //}
                //parms.Add( ptContact );
                #endregion

                // 18. Service Connected Priority (Optional)
                parms.Add(request.Patient.ServiceConnection);
                // 19. Service Connected % (Optional)
                parms.Add(request.Patient.ServiceConnectionPercentage);
                // 20. MRTC Calculated preferred dates, separated by pipe "|" (Optional)
                parms.Add(null);
                // 21. Clinic Stop Pointer (Optional)
                parms.Add(request.ClinicStopIen);
                // 22. Appointment Type, pointer to Appointment Type file (Optional)
                var apptTypeID = AppointmentTypes(request.Patient.IEN).FirstOrDefault(a => a.Name.ToUpper() == request.AppointmentType.ToUpper());
                parms.Add(apptTypeID.Ien);
                // 23. Patient Status (Optinal)
                parms.Add(request.EnrolleeStatus);
                // 24. (Optional - For Master MRTC Requests ONLY) Multiple pointers to Appointment File separated by "|"
                if (request.IsMRTCParent && (request.MRTCAppointmentRequestIENs.Count > 0))
                {
                    var ienList = string.Join("|", request.MRTCAppointmentRequestIENs.ToArray());
                    parms.Add(ienList);
                }
                else
                    parms.Add(null);
                // 25. (Optional - MRTC Child Request ONLY) MRTC Master Request IEN
                if (request.IsMRTCChild && !string.IsNullOrEmpty(request.MRTCParentRequestIEN))
                    parms.Add(request.MRTCParentRequestIEN);
                else
                    parms.Add(null);

                var dt = _adt.callOVIDRPC(Keys.RpcCalls.AppointmentRequestSet, parms);

                if (dt == null || dt.Rows.Count != 1)
                    return true;

                var r = dt.Rows[0];
                if (!string.IsNullOrEmpty(r["ERRORTEXT"].ToString()))
                {
                    var sError = r["ERRORTEXT"].ToString();
                    throw new Exception("Unable to Update Appointment Request.\n" + sError);
                }

                if (string.IsNullOrEmpty(requestIEN))
                {
                    requestIEN = r["ERRORID"].ToString();
                    request.ApptRequestIen = Convert.ToInt32(requestIEN);
                }

                // If we are updating a request, check the cached requests lists & update MRTC Data if necessary
                // If the passed parameter requestIEN is not empty, then we are updating
                if (string.IsNullOrEmpty(requestIEN) && string.IsNullOrEmpty(appointmentIEN))
                    return true;

                AppointmentRequest req;
                if (request.IsMRTCChild)
                    req = Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.FirstOrDefault(a => a.ApptRequestIen.ToString() == request.MRTCParentRequestIEN);
                else
                    req = Keys.AppointmentRequestWaitListTables.AllAppointmentRequests.FirstOrDefault(a => a.ApptRequestIen.ToString() == requestIEN);

                if (req != null && req.IsMRTCParent)
                {
                    if (!string.IsNullOrEmpty(appointmentIEN) && !req.MRTCAppointmentIENs.Contains(appointmentIEN))
                        req.MRTCAppointmentIENs.Add(appointmentIEN);

                    if (!string.IsNullOrEmpty(requestIEN) && !req.MRTCAppointmentRequestIENs.Contains(requestIEN))
                        req.MRTCAppointmentRequestIENs.Add(requestIEN);
                }
                return true;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "CreateApptAppointmentRequest", ex);
#if DEBUG
                Keys.AlertUserError(this, ex, "CreateApptAppointmentRequest Error");
#endif
            }
            return false;
        }

        public int GetMRTCChildPreferredDates(int requestIEN, List<string> prefDates)
        {
            var parm = new List<string> { requestIEN.ToString() };
            var appts = _adt.callOVIDRPC(Keys.RpcCalls.CheckAppointmentCountForMrtcRequest, parm);
            var numAppts = "";
            foreach (DataRow appt in appts.Rows)
            {
                if (string.IsNullOrEmpty(numAppts))
                    numAppts = appt[0].ToString();
                else
                    prefDates.Add(appt[0].ToString());
            }

            return Keys.Methods.IsNumeric(numAppts) ? Convert.ToInt32(numAppts) : -1;
        }

        public bool UpdateMRTCChildPreferredDates(int requestIEN, List<DateTime> mrtcPrefDates)
        {
            try
            {
                // Convert links to a "|" separated list
                var mrtcPrefDatesStr = new List<string>();

                foreach (var prefDate in mrtcPrefDates)
                    mrtcPrefDatesStr.Add(prefDate.ToShortDateString());

                var mrtcInput = string.Join("|", mrtcPrefDatesStr.ToArray());

                var parms = new List<string>
                {
                    //  1. Request IEN
                    requestIEN.ToString(),
                    //  2. MRTC - MRTC CALC PREF DATE^Parent Request IEN^AppointmentIEN
                    mrtcInput
                };

                var dt = _adt.callOVIDRPC(Keys.RpcCalls.UpdateDesiredDates, parms);

                if (dt != null && dt.Rows.Count == 1)
                {
                    var r = dt.Rows[0];
                    if (!string.IsNullOrEmpty(r["MESSAGE"].ToString()))
                    {
                        var sError = r["MESSAGE"].ToString();
                        throw new Exception("Unable to Update MRTC Child Request CID Dates.\n" + sError);
                    }
                }
                return true;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "UpdateMRTCChildPreferredDates", ex);
#if DEBUG
                Keys.AlertUserError(this, ex, "UpdateMRTCChildPreferredDates Error");
#endif
            }
            return false;
        }

        // Expects mrtcLinks to be a list of strings formatted as "AppointmentID^ApptRequestIEN"
        public bool UpdateMRTCChildLinks(int requestIEN, List<string> mrtcLinks)
        {
            try
            {
                // Convert links to a "|" separated list
                var mrtcInput = string.Join("|", mrtcLinks.ToArray());

                var parms = new List<string>
                {
                    //  1. Request IEN
                    requestIEN.ToString(),
                    //  2. MRTC - MRTC CALC PREF DATE^Parent Request IEN^AppointmentIEN
                    mrtcInput
                };

                var dt = _adt.callOVIDRPC(Keys.RpcCalls.SetMRTCChildLinks, parms);

                if (dt != null && dt.Rows.Count == 1)
                {
                    var sError = dt.Rows[0]["MESSAGE"].ToString();

                    if (sError.ToUpper() != "SUCCESS")
                        throw new Exception("Unable to Update MRTC Child Request Links.\n" + sError);
                }
                return true;
            }
            catch (Exception ex)
            {
                TraceLog.LogException(GetType().Name, "UpdateMRTCChildLinks", ex);
#if DEBUG
                Keys.AlertUserError(this, ex, "UpdateMRTCChildLinks Error");
#endif
            }
            return false;
        }

        public int GetMRTCParentBookedAppointmentsCount(string requestIEN)
        {
            //GetNumber of Appts Booked for this request..
            var req = GetApptRequestByRequestIEN(requestIEN);
            return req.MRTCAppointmentIENs.Count;

            // As of 01/17/2016 SDECAR ARMRTC is not working correctly, so use above
            //List<string> parms = new List<string> { requestIEN };
            //var rslt = _adt.callOVIDRPC(Keys.RpcCalls.MrtcNumApptsBooked, parms);
            //var cnt = "";

            //foreach (DataRow dr in rslt.Rows)
            //    cnt = dr["COUNT"].ToString();

            //if (Keys.Methods.IsNumeric(cnt))
            //    return Convert.ToInt32(cnt);
            //else
            //    return 0;
        }

        public IList<MaritalStatus> GetMaritalStatusList()
        {
            var list = new List<MaritalStatus>();

            try
            {
                // Remove any stale records from the Keys.Tables.LookupTablesCache
                Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);

                DataTable dt;
                if (Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.MartialStatusList))
                    dt = Keys.Tables.LookupTablesCache[Keys.RpcCalls.MartialStatusList].Table;
                else
                {
                    var parms = new List<string>();
                    dt = _adt.callOVIDRPC(Keys.RpcCalls.MartialStatusList, parms);
                    Keys.Tables.LookupTablesCache.Add(Keys.RpcCalls.MartialStatusList, new DataTableCache(dt, 60));
                }

                foreach (DataRow row in dt.Rows)
                {
                    var maritalstatus = new MaritalStatus
                    {
                        ien = row["MIEN"].ToString(),
                        name = row["MNAME"].ToString(),
                        abbreviation = row["MABB"].ToString(),
                        code = row["MCODE"].ToString()
                    };
                    list.Add(maritalstatus);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Marital Status List Error");
            }
            return list;
        }

        public IList<Religion> GetReligionList()
        {
            var list = new List<Religion>();

            try
            {
                // Remove any stale records from the Keys.Tables.LookupTablesCache
                Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.LookupTablesCache);

                DataTable dt;
                if (Keys.Tables.LookupTablesCache.ContainsKey(Keys.RpcCalls.ReligionList))
                    dt = Keys.Tables.LookupTablesCache[Keys.RpcCalls.ReligionList].Table;
                else
                {
                    var parms = new List<string>();
                    dt = _adt.callOVIDRPC(Keys.RpcCalls.ReligionList, parms);
                    Keys.Tables.LookupTablesCache.Add(Keys.RpcCalls.ReligionList, new DataTableCache(dt, 60));
                }

                foreach (DataRow row in dt.Rows)
                {
                    var religion = new Religion
                    {
                        Ien = row["RIEN"].ToString(),
                        Name = row["RNAME"].ToString(),
                        Abbreviation = row["RABB"].ToString(),
                        Code = row["RCODE"].ToString(),
                        Classification = new NameValue(row["RCLASSN"].ToString(), row["RCLASS"].ToString())
                    };
                    list.Add(religion);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Religion List Error");
            }
            return list;
        }

        private NameValue GetPatientSensistiveInfo(string patientIEN)
        {
            if (patientIEN == Keys.PatientIen_SensitiveRecordCheck)
                return new NameValue("0", string.Empty);

            Keys.Tables.RemoveStaleDataTableCacheRecords(Keys.Tables.PatientSensitiveRecordCache);

            if (!Keys.Tables.PatientSensitiveRecordCache.Keys.Contains(patientIEN))
            {
                var parms = new List<string>() { patientIEN };
                var count = 0;
                string[][] rpcReturn = null;

                do
                {
                    rpcReturn = _adt.CallVistaRpc(Keys.PatientPromptRpcs.SensitiveRecord, parms);

                    if (rpcReturn == null)
                        System.Threading.Thread.Sleep(250);

                } while ((count < 3) && (rpcReturn == null));

                if (rpcReturn == null)
                    throw new Exception("Unable to read the patient sensitive information.\nSee the Trace Log for details.");

                Keys.Tables.PatientSensitiveRecordCache.Add(patientIEN, new DataTableCache(rpcReturn, 15));
            }

            string[][] array = Keys.Tables.PatientSensitiveRecordCache[patientIEN].Array;
            string code = string.Empty;
            string message = string.Empty;

            foreach (var row in array)
                if (row != null)
                    foreach (var pc in row)
                        if (string.IsNullOrEmpty(code))
                            code = pc;
                        else
                            message += pc + Environment.NewLine;

            return new NameValue(code, message);

        }

        #endregion

    }
    public class ResourceUserIds
    {
        public string ResourceUserIen { get; set; }
        public string ClinicIen { get; set; }
    }
}
